/*
JMP Clinical Analytical Process

Name: AddStudyLocal
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   sasrdw 02Nov10 initial coding
   rizink Sep2011 Added Metadata functionality from SAS CST
Description: 
This process defines the default settings
for JMP Clinical. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath DefaultServerOutPath SnpComp SDTMDom ADaMDom
   ArchiveSASCode ResolveSASCode TransportCheck AddStudyLocalScript
   OutPath ServerOutPath MacroPath _slash_ ShowResultFlag FromCombFlag;

*ProcessBody;

%macro AddStudyLocal;

%let ProcessName=AddStudyLocal;

options nofmterr;

*options mprint mlogic symbolgen;

* create output folder under home folder;
%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;
%end;

%global IntDateFormat runCDISCParmsFlag;
%if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) or
   %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then
   %let IntDateFormat = DDMMYY10;
%else %let IntDateFormat = DATE9;

%let StudyName = %kleft(%ktrim(&StudyName));
%let runCDISCParmsFlag=1;
%if %length(&StudyName) > 32 %then %do;
   %put [MS_S_0001];
   %put ERROR: The study name must be 32 characters or fewer.;
   %let exiterror = 1;
   %goto exit;
%end;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%let OutPath = %jg_getPath(/user/clinical/output,,1,0,1,&StudyName);
%let OutName = &StudyName;

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas" / nosource;
%include "&MacroPath./CSTMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%let NewStudyName = &StudyName;
%CheckNewStudyName;
%let ChkNotesDir = %jg_getPath(/user/clinical/notes,,1,0,3,&StudyName);
%let notesDataset =;
%jg_getNotesDatasetName(notesDataset);
%if %length(&ChkNotesDir) %then %do;
	%if ^%sysfunc(fileexist(&ChkNotesDir.&notesDataset..sas7bdat)) %then %do;
	   %put NOTE: No notes detected under study name.;
	%end;
%end;
%else %do;
   %put NOTE: Notes detected under study name.;
   /*
   %put ERROR: Notes exist for a study previously defined as &StudyName.. Please choose a different study name or move the notes file to a new location.;
   %put ERROR: Notes file is located &ChkNotesDir.&notesDataset..sas7bdat;
   %let exiterror = 1;
   %*let tid = %sysfunc(close(&tid));
   %goto exit; */
%end;

%if &exiterror or &syserr > 4 %then %goto exit;

%if ^%length(&AInPath) & ^%length(&InPath) %then %do;
   %put [MS_S_0002];
   %put ERROR: Neither the SDTM folder or ADaM folder is specified.;
   %put At least one of these folder must be specified to run JMPC Clinical processes.;
   %let exiterror = 1;
%end;
%if &exiterror %then %goto exit;

%if %quote(%upcase(&AInPath))=  %quote(%upcase(&InPath)) %then %do;
   %put [MS_S_0003];
   %put ERROR: The SDTM and ADaM folder paths cannot be the same.;
   %let exiterror = 1;
%end;

%if %length(&InPath) %then %do;
   *check if the directory exists;
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,&InPath)) ; 
   %if ^%sysfunc(fexist(&fileref))  %then %do;
      %put [MS_S_0004];
      %put ERROR: SDTM folder &InPath specification is invalid.;
      %let exiterror = 1;
   %end;
   
   *check if there is any sas dataset in the folder;
   %FileFilter(&InPath,sasdatalist,_ext="sas7bdat");
   %FileFilter(&InPath,xptdatalist,_ext="xpt");
   %NObsVars(sasdatalist);
   %let nSASobs=%nObs;
   %NObsVars(xptdatalist);
	%if &nObs =0 & &nSASobs =0 %then %do;
		%put ERROR: There are no SAS data sets in the specified SDTM Folder.;
		%let exiterror = 1;
	%end;
%end;
%if %length(&AInPath) %then %do;
   *check if the directory exists;
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,&AInPath)) ; 
   %if ^%sysfunc(fexist(&fileref))  %then  %do;
      %put [MS_S_0005];
      %put ERROR: ADaM folder &AInPath specification is invalid.;
      %let exiterror = 1;
   %end;
   
   *check if there is any sas dataset in the folder;
   %FileFilter(&AInPath,sasdatalist,_ext="sas7bdat");
   %FileFilter(&AInPath,xptdatalist,_ext="xpt");
   %NObsVars(sasdatalist);
   %let nSASobs=%nObs;
   %NObsVars(xptdatalist);
	%if &nObs =0 & &nSASobs =0 %then %do;
		%put ERROR: There are no SAS data sets in the specified ADaM Folder.;
		%let exiterror = 1;
	%end;
%end;
%if &exiterror %then %goto exit;

%*filename _mainjslfile_ "&OutPath.sasmessage1.jsl";
%*filename _mainjslfile_ "&OutPath.sasmessage.jsl";

%if %length(&SDTMDom) %then %do;
	%let SDTMDom = %sysfunc(tranwrd(&SDTMDom, %str(,), %str( )));
	%let SDTMDom = %sysfunc(tranwrd(&SDTMDom, %str(|), %str( )));
%end;
%if %length(&ADaMDom) %then %do;
	%let ADaMDom = %sysfunc(tranwrd(&ADaMDom, %str(,), %str( )));
  	%let ADaMDom = %sysfunc(tranwrd(&ADaMDom, %str(|), %str( )));
%end;
* check if there is a need to convert the transport files;
%if &TransportCheck=Yes %then %do;
   %let InPathTmp = &InPath;
   %let JMPCInPath = %jg_getPath(/user/clinical/transportdata/sdtm,,1);
   %let JMPCAInPath = %jg_getPath(/user/clinical/transportdata/adam,,1);
   
   * check if SDTM folder has any .cpt or .xpt files;
   %if %length(&InPath) %then %do;
      %FileFilter(&InPath,CPT,_ext="cpt");
      %NObsVars(CPT);
      %let nCPT = &nObs;
      * Transport .xpt files from the input folder;
      %FileFilter(&InPath,XPT,_ext="xpt");
      %NObsVars(XPT);
      %let nXPT = &nObs;
      
      %if &nCPT > 0 | &nXPT > 0 %then %do;
         %ConvertTransportFile(&InPath, &InPath);
      %end;
   %end;
   
   * check if ADaM folder has any .cpt or .xpt files;
   %if %length(&AInPath) %then %do;
      %FileFilter(&AInPath,CPT,_ext="cpt");
      %NObsVars(CPT);
      %let nCPT = &nObs;
      * Transport .xpt files from the input folder;
      %FileFilter(&AInPath,XPT,_ext="xpt");
      %NObsVars(XPT);
      %let nXPT = &nObs;
      
      %if &nCPT > 0 | &nXPT > 0 %then %do;
         %ConvertTransportFile(&AInPath, &AInPath);
      %end;
   %end;
   
%end;

%let metaData = metadata_study_source;
* make sure either adsl or dm exists;
%if %length(&AInPath) & %length(&InPath) %then %do;
	%if ^%sysfunc(fileexist(&AInPath./adsl.sas7bdat)) and ^%sysfunc(fileexist(&InPath./dm.sas7bdat)) %then %do;
	   %if %sysfunc(fileexist(&InPath./adsl.sas7bdat)) | %sysfunc(fileexist(&AInPath./dm.sas7bdat)) %then %do;
	      %put [MS_S_0006];
	      %put ERROR: The adsl data set should be in the ADaM folder or the dm data set should in the SDTM folder.;
	      %put At least one of these data sets must be available to run JMP Clinical processes.;
	      %let exiterror =1;
	      %goto exit;
	   %end;
	   %else %do;
	      %put [MS_S_0007];
	      %put ERROR: Neither the adsl nor dm data set exists.;
	      %put At least one of these data sets must be available to run JMP Clinical processes.;
	      %let exiterror =1;
	      %goto exit;
	   %end;
	%end;
%end;
%else %if %length(&AInPath) & ^%length(InPath) %then %do;
	%if ^%sysfunc(fileexist(&AInPath./adsl.sas7bdat)) %then %do;
		%put ERROR: The adsl data set should be in the ADaM folder in order to add a study.; 
	   %let exiterror =1;
	   %goto exit;
	%end;
%end;
%else %if %length(&InPath) & ^%length(&AInPath) %then %do;
	%if ^%sysfunc(fileexist(&InPath./dm.sas7bdat)) %then %do;
		%put ERROR: The dm data set should in the SDTM folder in order to add a study.;
	   %let exiterror =1;
	   %goto exit;
	%end;
%end;

* if users select available domains, make sure either adsl or dm is selected;
%if %length(&SDTMDom) & %length(&ADaMDom) %then %do;
   %if ^%index (%upcase(&SDTMDom), DM) and ^%index (%upcase(&ADaMDom), ADSL) %then %do;
      %put [MS_S_0007];
      %put ERROR: Neither the adsl nor dm data set is selected.;
      %put At least one of these data sets must be available to run JMP Clinical processes.;
      %let exiterror =1;
      %goto exit;
   %end;
%end;
%else %if %length(&SDTMDom) & ^%length(&ADaMDom) %then %do;
   %if ^%index (%upcase(&SDTMDom), DM) %then %do;
      %put [MS_S_0007];
      %put ERROR: No dm data set is selected.;
      %put The dm data set must be available to run JMP Clinical processes.;
      %let exiterror =1;
      %goto exit;
   %end;
   %else %do;
   	%let p=1;
   	%let DMCheckFlag=0;
   	%do %while (%length(%scan(&SDTMDom,&p,' ')));
   		%let SDTMDomCheckDM= %scan(&SDTMDom,&p,' ');
   		%if  %upcase(&SDTMDomCheckDM) = DM %then %do;
   			%let DMCheckFlag=1;
   		%end;
   		%let p = %eval(&p+1);  
   	%end;
   	%if &DMCheckFlag=0 %then %do;
			%put [MS_S_0007];
			%put ERROR: No dm data set is selected.;
			%put The dm data set must be available to run JMP Clinical processes.;
			%let exiterror =1;
			%goto exit;
  	 	%end;
   %end;
%end;
%else %if ^%length(&SDTMDom) & %length(&ADaMDom) %then %do;
   %if ^%index (%upcase(&ADaMDom), ADSL) %then %do;
      %put [MS_S_0007];
      %put ERROR: No adsl data set is selected.;
      %put The adsl data set must be available to run JMP Clinical processes.;
      %let exiterror =1;
      %goto exit;
   %end;
%end;

%if %length(&InPath) %then %CheckEndSlash(InPath);
%if %length(&AInPath) %then %CheckEndSlash(AInPath);
%if %length(&OutPath) %then %CheckEndSlash(OutPath);

/*
%put _cstGRoot = &_cstGRoot;
*/

%GENERATEMETA(studname = &studyname);
%if &exiterror %then %goto exit;

%if &CSTERROR %then %do;
   %put [MS_S_0008];
   %put ERROR: Creation of MetaData Failed.;
   %let exiterror = 1;
   %goto exit;
%end;
* if there are too many duplicates and snpshot comparison is checked, an error message will threw;
%if %symexist(snpCompDup) %then %do;
   %if &snpCompDup > 2000 %then %do;
      * create JSL file to open SnapShotDups.pdf and then throw an error message;
      %PrepOutFile(AddStudyLocalScript,AddStudyLocalScriptName,&StudyName &ProcessName Script.jsl);
      %global _mainjslfile_ FullAPName;
      %let _mainjslfile_ = &AddStudyLocalScript;
      %let FullAPName=%str(Add Study from Folders);
      data _null_;
         file "&AddStudyLocalScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;  
         put '//!';
         put 'open("'"&ClientOutPath&StudyName SnapShotDups.pdf"'", JMP window);';
         put 'throw();';
      run;
      %PackageInsertFile(SAS message from &ProcessName, &OutPath, &AddStudyLocalScriptName, , text/plain, yes);
      %put [MS_S_0009];
      %put ERROR:&snpCompDup duplicate records have been detected based on the current domain keys (detected data set sort order or CDISC recommended keys) used to define unique records.;
      %put ERROR: A unique record will be examined for difference between two snapshots. Otherwise there is no way to know which duplicate records to match together.;
      %put ERROR:Please check "Duplicate and Keys Report" in &StudyName SnapShotDups.pdf file to fix the domain keys for domain data sets by using the SORT PROCEDURE based on the set of variables that will define unique records.;
      %put ERROR:Also, you can choose to disable snapshot comparison if you do not wish to track record level changes across study data updates.;
      %goto exit;
   %end;
%end;

* Add the study name to the metadata table;
%AddStudyName;

%if &SnpComp =Yes %then %do;
   %PackageInsertFile(Snapshot Duplicates PDF file from &ProcessName,&OutPath,&StudyName SnapShotDups.pdf,,pdf,yes,no);
%end;
%*put FromCombFlag=&FromCombFlag;
%if &FromCombFlag=Yes %then %do;
	%PackageInsertFile(Summary PDF file from Combined Studies,&OutPath,&StudyName SummaryCombineStudies.pdf,,pdf,yes,no);
%end;

%let RVDataset=required_variables;

* create JSL file;
%PrepOutFile(AddStudyLocalScript,AddStudyLocalScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &AddStudyLocalScript;
%let FullAPName=%str(Add Study from Folders);

data _null_;
   file "&AddStudyLocalScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newNamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:FullAPName = ""&FullAPName"";";
   put "objRefNS:StudyName = ""&StudyName"";";
   put "if(isEmpty(jg:FromCombFlag),jg:FromCombFlag=""No"");";
   put "jg:CVSInit;";
   put "Try((((((((((Window(locale:get(""AddStudyfromFolders_Text"")) << child) << child) << sib) << child) << sib) << sib) << child) << sib) << sib) << enable(0);";
   put "Try(((((((((((Window(locale:get(""CombineStudies_Text"")) << child) << child) << sib) << child) << sib) << sib) << child) << sib) << sib) << sib) << enable(0));";
   put "rc = GenerateStudyStarterFilter( ""&StudyName"" );";
   put "Try((((((((((Window(locale:get(""AddStudyfromFolders_Text"")) << child) << child) << sib) << child) << sib) << sib) << child) << sib) << sib) << enable(1);";
   put "Try(((((((((((Window(locale:get(""CombineStudies_Text"")) << child) << child) << sib) << child) << sib) << sib) << child) << sib) << sib) << sib) << enable(1));";
   put 'objRefNS:customPanel=';
   put '   VListBox(';
   put "      TextBox("" ""),";
   put "      TextBox(locale:fill(locale:get(""ThestudyXhasbeensuccessfullyadded_Text""), {""&StudyName""})),";
   put "      TextBox("" ""),";
   %if &SnpComp =Yes %then %do;
      put "      ButtonBox(locale:get(""OpenDuplicatesandKeysReport_Text""),";
      put "          Try(open(""&ClientOutPath&StudyName SnapShotDups.pdf""));";
      put "      ),";   
   %end;
 
   put "if (jg:FromCombFlag==""Yes"",";
      put "      ButtonBox(locale:get(""OpenSummaryReportfromCombinedStudies_Text""),";
      put "          Try(open(""&ClientOutPath&StudyName SummaryCombineStudies.pdf""));";
      put "      ),"; 
      put "      TextBox("" ""),";
   put "),";
   put "      ButtonBox(locale:get(""ViewIncludedandExcludedDomains_Text""),";
   put "          Try(open(""&ClientOutPath.&StudyName DomainList.pdf""));";
   put "      ),";  
   put "      TextBox("" ""),";
   %if %nrbquote(&StudyName) ne Nicardipine and %nrbquote(&StudyName) ne Nicardipine Early Snapshot %then %do;
      %if &SnpComp =Yes %then %do;
         put "      dupnote = TextBox(locale:get(""PleaseRevDupKeyRepMsg_Text"")), dupnote << Font Color(5),";
         put "      TextBox("" ""),";
      %end;
      %else %do;
         put "      dupnote = TextBox(locale:get(""PleaseNoteSnapshotcomparisondisabled_Text"")), dupnote << Font Color(5),";
         put "      TextBox("" ""),";
      %end;
   %end;
   put ");";
   
   /* Update Study List on Clinical Starter */
   put "if(jg:studies_exist == 0,";
   put " jg:study3list = {""&StudyName""};";
   put " jg:studies_exist = 1;";
   put " ,";  /* Else a study already exists, retrieve from function */
   put " jg:study3list = jg:getStudyList(""Clinical"");";
   put ");";
   put "jg:updateStarterStudyList(3,""&StudyName"",jg:sasserver);";
   put "wait();";
   
   put "// open journal when installing workflow package";
   put "try(";
   put "   _tmp_ = Load Text File(""&Outpath.addin.def"");";
   put "   _name_ = word(-1,_tmp_,""="");";
   put "   _name_1 = word(1,_name_);";
   put "   _study_ = trim(munger(_name_,length(_name_1)+2,-1));";
   put "   open(""&Outpath""||_study_||"".jrn"");";
   put ");";
     
   put "); // end of re-enable Close button Try";
   put "); // end of disable Close button Try";
run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &AddStudyLocalScriptName, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, 1);

%mend AddStudyLocal;

* run macro;
%AddStudyLocal;
