/*
JMP Clinical Analytical Process

Name: Advere Events Report
Authors: SASLII, SAS Institute Inc.

Description: 
   This process creates static reports of adverse events.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

************************************************************************;
*     Clinical Variable Specification Block of AdverseEventsReport Variable Usage;
************************************************************************;
/* 
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas","LifeSciences\MacroLib\DomainDistribution.sas"}$
$INCB: {{"EventsO","CrossoverPeriods","AETermValues","TreatmentEmergentEvents"},{"ADSLDistributionOptions"}}$
$INCD: {"AE",{""}}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: RSYS$ $UE: AE.AEDECOD or AE.AETERM$ $NOTES: either the AEDECOD or the AETERM variables will be used for AE Event occurrence.$
$CMP: OSYS$ $UE: DM.SEX$ $NOTES: Used  to calculate counts by their sex.$ 
$CMP: OSYS$ $UE: DM.RACE$ $NOTES: Used  to calculate counts by their race.$ 
$CMP: OOUT$ $UE: AE.AESEV or AE.AETOXGR or AE.TOXGR$ $NOTES: Used to categorize events by severity/toxicity.$ 
$CMP: OOUT$ $UE: AE.AEOUT$ $NOTES: Used  to categorize events by their outcome.$ 
$CMP: OOUT$ $UE: AE.AESER$ $NOTES: Used to calculate counts by categorizing by serious events.$
$CMP: OOUT$ $UE: AE.AEREL$ $NOTES: Used to calculate counts by causality.$ 
$CMP: OOUT$ $UE: AE.AEOUT$ $NOTES: Used  to calculate counts by their outcome.$ 
$CVS_STOP$
*/


*ProcessVariables;

%global StudyName InPath AInPath TrtVar TrtUse AESerCount MergeSuppDom
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL DesignByVars 
SubjectFilter WhereStmt AEWhereStmt AESerOnly TEAEOnly OutPath ServerOutPath 
CountMultiple OutName MacroPath Output_RTF Output_PDF Output_HTML Output_Text PropThresh
AEDistributionReport_Body AEDistributionReport_Contents AEDistributionReport_Frame
AdverseEventsReportScript AdverseEventsReportScriptName TrtOffSet IgnoreTEAEFlags
;

*ProcessBody;

%macro AdverseEventsReport;
   * options mprint;
   *initialization and argument checking;
   %let ProcessName = AdverseEventsReport; 
   
   %include "&MacroPath./UtilityMacros.sas" / nosource;
   %include "&MacroPath./CDISCParms.sas";
   
   %if &exiterror %then %goto exit;
   
   %let Output_HTML = No;
   
   /* Simple strings and skeleton (fillable) strings */
   %let trAdvEvCtBySevLev =         %nrbquote(%LocaleGet(AdverseEventCountbySeverityLevel_Text));
   %let trAEEvOccAndPropRepforX =      %nrbquote(%LocaleGet(AEEventOccAndProportionReportforX_Text));
   %let trAEXPropReport =           %nrbquote(%LocaleGet(AEXproportionreport_Text));
   %let trAllEventTypes =           %nrbquote(%LocaleGet(AllEventTypes_Text));
   %let trAllSubjects =             %nrbquote(%LocaleGet(AllSubjects_Text));
   %let trBodySystemCount =         %nrbquote(%LocaleGet(BodySystemCount_Text));
   %let trCount =                   %nrbquote(%LocaleGet(Count_Text));
   %let trEventLC =        %nrbquote(%LocaleGet(event_Text));
   %let trMedicationCategoryCount =    %nrbquote(%LocaleGet(MedicationCategoryCount_Text));
   %let trPercent =              %nrbquote(%LocaleGet(Percent_Text));
   %let trPercentOccurrence =          %nrbquote(%LocaleGet(PercentOccurrence_Text));
   %let trPercentOccurrenceofX =       %nrbquote(%LocaleGet(PercentOccurrenceofX_Text));
   %let trPreTreatmentEvents =      %nrbquote(%LocaleGet(PreTreatmentEvents_Text));
   %let trOfftreatmentEvents =      %nrbquote(%LocaleGet(Offtreatmentfollowupevents_Text));
   %let trOntreatmentEvents =       %nrbquote(%LocaleGet(Ontreatmentevents_Text));
   %let trRace =                 %nrbquote(%LocaleGet(Race_Text));
   %let trReportingAllEvents =      %nrbquote(%LocaleGet(ReportingAllEvents_Text));
   %let trReportingEvWithALOvXOcc =    %nrbquote(%LocaleGet(ReportingEventswithatleastOverallXOcc_Text));
   %let trSex =                  %nrbquote(%LocaleGet(Sex_Text));
   %let trSSSUnclassifiedSSS =      %nrbquote(%LocaleGet(SSSUnclassifiedSSS_Text));
   %let trTotal =                   %nrbquote(%LocaleGet(Total_Text));
   %let trTotalCount =           %nrbquote(%LocaleGet(TotalCount_Text));
   %let trTotalCountofX =           %nrbquote(%LocaleGet(TotalCountofX_Text));
   %let trTreatment =               %nrbquote(%LocaleGet(Treatment_Text));
   %let trTreatmentEmergentAdvEvents = %nrbquote(%LocaleGet(TreatmentEmergentAdverseEvents_Text));
   %let trXforXChosen =             %nrbquote(%LocaleGet(XforXChosen_Text));
   %let trXIncSerAEOnlyforXChosen =    %nrbquote(%LocaleGet(XIncludingSeriousAdverseEventsOnlyforXChosen_Text));
   
   /* Filled strings (only those that are possible to fill ahead of time) */
   %let trAEEvOccAndPropRepforRace =   %nrbquote(%LocaleFill(&trAEEvOccAndPropRepforX, &trRace));
   %let trAEEvOccAndPropRepforSex =    %nrbquote(%LocaleFill(&trAEEvOccAndPropRepforX, &trSex));
   %let trAEEvOccAndPropRepforTrt =    %nrbquote(%LocaleFill(&trAEEvOccAndPropRepforX, &trTreatment));
   %let trAERacePropReport =        %nrbquote(%LocaleFill(&trAEXPropReport, &trRace));
   %let trAESexPropReport =         %nrbquote(%LocaleFill(&trAEXPropReport, &trSex));
   %let trAETrtPropReport =         %nrbquote(%LocaleFill(&trAEXPropReport, &trTreatment));
   %let trPercentOccurrenceofRace =    %nrbquote(%LocaleFill(&trPercentOccurrenceofX, &trRace));
   %let trPercentOccurrenceofSex =  %nrbquote(%LocaleFill(&trPercentOccurrenceofX, &trSex));
   %let trTotalCountofRace =        %nrbquote(%LocaleFill(&trTotalCountofX, &trRace));
   %let trTotalCountofSex =         %nrbquote(%LocaleFill(&trTotalCountofX, &trSex));
   
   %if &Output_RTF = No and &Output_PDF = No and &Output_HTML = No %then %do;
      %put [MS_S_0010];
      %put ERROR: At least one output format must be selected.;
      %let exiterror = 1;
      %goto exit;
   %end;
   
   %if ^%sysfunc(findw(&domains, AE)) %then %do;
      %put [MS_S_0011];
      %put ERROR: The AE or ADAE data set does not exist for this study.;
      %let exiterror=1;
      %goto exit;
   %end;

   %include "&MacroPath./DomainDistribution.sas";
   %LOADDATA(domlist=AE);
   %global DomainWhereStmt;
   %AESerOnlyToAEWhereStmt;
   %let DomainWhereStmt = %nrbquote(&AEWhereStmt);
   %let CountMultiple=No;
   
   %include "&MacroPath./TEAEOTFU.sas";
   %if &exiterror %then %goto exit;   
   
   %let aesev =;
   %let aeid = %sysfunc(open(&in_ae));
   %let sevnum = %sysfunc(varnum(&aeid,AESEV));
   %if &sevnum=0 %then %do;
      %let toxgrnum = %sysfunc(varnum(&aeid,AETOXGR));
      %if &toxgrnum > 0 %then %let aesev = TOXGR;
   %end;
   %else %let aesev = sev;
   %let aerel =;
   %let relnum = %sysfunc(varnum(&aeid,AEREL));
   %if &relnum %then %let aerel = REL;
   %put aerel = &aerel;
   %let aeid = %sysfunc(close(&aeid));
   
   %if %sysfunc(findw(&AEVARS, AEDECOD)) > 0 %then %do;
      %let TmName = DECOD;
   %end;
   %else %if %sysfunc(findw(&AEVARS, AETERM)) > 0 %then %do;
      %let TmName = TERM;
   %end; 
   %else %do;
      %put [MS_S_0012];
      %put ERROR:  Neither the AEDECOD nor the AETERM variables were found in the AE data set.  At least one of these variables is required for this process.; 
      %let exiterror=1;
      %goto exit;
   %end;

   %let AEser = ;
   %let AEout = ;
   %if %sysfunc(findw(&AEVARS, AESER)) > 0 %then %do;
      %let AEser = SER;
   %end;

   %if %sysfunc(findw(&AEVARS, AEOUT)) > 0 %then %do;
      %let AEout = OUT;
   %end;
   
   %let domain = ae;
   %let extravars= &aeser &aerel  &aesev  &aeout bodsys;
   %let treemapvars=&TmName;
   %*DomainDistribution(ae,extravars= &aeser &aerel  &aesev  &aeout bodsys,treemapvars=&TmName);

   %if %length(&&in_&domain)=0 %then %do;
      %put [MS_S_0013];
      %put ERROR: The &domain or ad&domain data set does not exist for this study.;
      %let exiterror=1;
      %goto mexit;
   %end;
   %if %UPcase(&TrtUse) = NONE %then %do;
      proc freq data=&in_adsl noprint;
         tables &TrtName / out=fc;
         where not missing(&TrtName);
      run;
      
      %NObsVars(fc);
      %if &nobs < 2 %then %do;
         %let TrtLabel = ;
      %end;
      %let TrtName = ;
   %end;
   %let did = %sysfunc(open(&in_adsl));
   %let sexflag = %sysfunc(varnum(&did,SEX));
   %let raceflag = %sysfunc(varnum(&did,RACE));
   %let did = %sysfunc(close(&did));
   /*
   %let OutNameTmp=&OutName;
   %PrepOutDS(OutName,Distribution_OutDataName,&StudyName,_&domain.d);
   %let Distribution_OutData=&OutName;
   %if ^%length(&OutNameTmp) %then %let OutName = %substr(&OutName,%eval(%index(&OutName,.)+1));
   %else %let OutName =&OutNameTmp;   
   */
   %if ^%length(&OutName) %then %do;
      %let OutName = AdverseEventsReport;
   %end;
   %let FullOutName = ;
   %PrepOutFile(FullOutName,OutName2,&OutName);
   %let OutName = &OutName2;
   %let OutData = OutLib.AEReport_&_nThePro_;

   %let ColorLabel = ;
   %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) > 0 %then %do;
      %let ADSLVARS = &ADSLVARS COUNTRY;
   %end;
   %DMFlags;
   %if &exiterror %then %goto mexit;
   
   proc sort data=&&in_&domain %if %length(&DomainWhereStmt) %then (where=(%unquote(&DomainWhereStmt))); out=&domain;
      by USUBJID;
   run;
   %Nobsvars(&domain);
   %if &nobs=0 %then %do;
      %put [MS_S_0014];
      %put ERROR: There are no observations after filtering by the %upcase(&Domain) Filter.  Check that you have entered a valid filter.;
      %let exiterror=1;
      %goto exit;
   %end;
   %let fpool = 0;
   %let ftesn = 0;
   %let ftesc = 0;
   %let MissingTestResults = 0;
   %let MissingTests = 0;
   %let fspec = 0;
   %let fsev = 0;
   %let fbodsys = 0;
      
   /* add trtvar to Byvar list */
   %if %length(&TrtName) %then %do;
      %if ^%index(&DesignByVars,&TrtName) %then %do;
         %let DesignByVars = &TrtName &DesignByVars;
      %end;
   %end;
      
   %let textlabel = &trEventLC;
   %let aepos1_ = 0;
   %let aepos2_ = 0;
   %let _aetid_ = %sysfunc(open(&domain));
   %let fpos1_ = %sysfunc(varnum(&_aetid_,AE&TmName));
   %let fpos2_ = %sysfunc(varnum(&_aetid_,AEBODSYS));
   %let _aetid_ = %sysfunc(close(&_aetid_));
         
   data &domain; 
      set &domain;
      %if &fpos1_ > 0 %then %do;
         if AE&TmName = "" then AE&TmName = "&trSSSUnclassifiedSSS";
      %end;
         %else %if &fpos1_ = 0 %then %do;
            length AE&TmName $ 18;
            AE&TmName = "&trSSSUnclassifiedSSS";
         %end;
      %if &fpos2_ > 0 %then %do;
         if AEBODSYS = "" then AEBODSYS = "&trSSSUnclassifiedSSS";
      %end;
         %else %if &fpos2_ = 0 %then %do;
            length AEBODSYS $ 18;
            AEBODSYS = "&trSSSUnclassifiedSSS";
         %end;
   run;
   
   %let percentvar = AE&TmName;
   %let catbodvar = BODSYS;
   %let T_trtNameP_flag =0;

   %if ^%sysfunc(findw(&domains, DM)) %then %do;
      %put [MS_S_0015];
      %put ERROR: The DM or ADDM data set does not exist for this study.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   
   proc sort data=&in_dm out=dm;
      by USUBJID;
   run;

   /* To get any ADSL filters - where stmt or population */
   proc sort data=&in_adsl 
   out=adsl;
      by USUBJID;
   run;
   
   %nobsvars(adsl);
   %let nsubjects = &nobs;

   data dm_filter;
      merge adsl(in=inadsl) dm;
      by USUBJID;
      if inadsl;
   run;

   data &OutData;
      merge dm_filter(in=dmf) &domain ;
      by USUBJID;
      * want the line below? ;
      if missing(&domain.&TmName) then delete;
      if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
         %calculateStudyDays(&domain);
      end;
      if dmf;
   run;
   
   %if %length(&aeser) %then %do;
      %let AESerLabel = AESER;
      %CheckLabel(&OutData, AESERLabel); 
   %end;

   %if %length(&aeout) %then %do;
   %let AEOutLabel = AEOUT;
      %CheckLabel(&OutData, AEOutLabel);        
   %end; 
      
   %if %length(&aerel) %then %do;
      %let AERelLabel = AEREL;
       %CheckLabel(&OutData,AERelLabel);
   %end;
   
   proc sort data=&OutData;
         by AE&TmName USUBJID 
            %if %length(&aeser) %then %do; descending AESER %end; %if %length(&aesev) %then %do; descending AE&aesev %end; AESTDY;
   run;
      
   data &OutData;
      set &OutData;
      by AE&TmName USUBJID;
      if first.USUBJID then output;
   run;
   * copy the file for severity count;
   data outdataSer;
      set &in_ae;
   run;
   /* create counts for ordering */
   %let createcountvars = &treemapvars;
   %if %upcase(&domain) = CM or %upcase(&domain) = AE  %then %do;
      %if &fpos2_ >0 %then %do;
         %let createcountvars = &treemapvars &catbodvar;
      %end;
   %end;
      
   %let i = 1;
   %do %while(%length(%scan(&createcountvars,&i)));
      %let tmvar = %scan(&createcountvars,&i);
      %let percentvar = &domain.&tmvar;
      %put percentvar=&percentvar OutData=&OutData;

      proc sort data=&OutData(keep=&percentvar) out=getcounts;
         by &percentvar;
      run;

      proc freq data=getcounts noprint;
         tables &percentvar / out=countdt;
      run;

      %if %upcase(&CountMultiple)=NO and &i=1  %then %do;
         data countdt; set countdt;
            Percent = round((count/&nsubjects)*100,0.1);
            label Percent = "&trPercentOccurrence";
         run;
      %end;
         
      proc datasets;
         modify countdt;
         %if &i = 1 %then %do;
            label count = "&trTotalCount";
         %end;
         %else %do;
            %if %upcase(&domain) = AE %then %do;
               rename count = group_count;
               label group_count = "&trBodySystemCount";
            %end;
            %if %upcase(&domain) = CM %then %do;
               rename count = group_count;
               label group_count = "&trMedicationCategoryCount";
            %end;
         %end;
      quit;

      proc sql;
         create table &OutData as
         select * from &OutData as d1 left join countdt as d2 on d1.&percentvar=d2.&percentvar
         order by %if &fpool %then %do; POOLID %end; %else %do; USUBJID %end;;
      quit;
      %let i = %eval(&i+1);
   %end; 

   * Lili - create counts and calculate proportion for trt, sex and race;  
   %let AEserPercentVar = &domain.&aeser;
   %let AEDDtermVar =&domain.&treemapvars;
   %let AEtrtVar = &TrtVar;
   %let AErelVar =&domain.&aerel;
   %let AEoutVar=&domain.&aeout;
      
   * try to get sas name for TrtLabel;
   %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE %then %do;
      %let _tid_=%sysfunc(open(&OutData));
      %let nv_OutSet = %sysfunc(attrn(&_tid_,NVARS)); 
      %do _i_ = 1 %to &nv_OutSet;
         %let vlabel = %sysfunc(varlabel(&_tid_,&_i_));
         %let vname = %sysfunc(varname(&_tid_,&_i_));
         %*put vlabel=&vlabel trtLabel=&trtLabel vname=&vname;
         %if %length(&vlabel) %then %do;
            %if %str(%qupcase(&vlabel))=%str(%qupcase(&trtLabel)) %then %do;
               %let trtNameP=&vname;
            %end;
         %end;
         %else %if %str(%qupcase(&vname))=%str(%qupcase(&trtLabel)) %then %do;
            %let trtNameP=&vname;
         %end;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %*put AEserPercentVar=&AEserPercentVar AEDDtermVar=&AEDDtermVar AEtrtVar=&TrtVar &TrtUse &trtLabel trtNameP=&trtNameP;
      *get the total count for each trt;
      proc sort data=&in_adsl out=trtNameSum;
         by &trtNameP;
      run;
      proc freq data=trtNameSum noprint;
         tables &trtNameP /out=trtsum1;
      run;
      
      %let _tid_=%sysfunc(open(trtsum1));
      %let nv_trt = %sysfunc(attrn(&_tid_,NOBS)); 
      %let trtTotal=0;
      %do _i_ = 1 %to &nv_trt;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let trtNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&trtNameP))));
         %let trtNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let trtTotal = %eval(&trtTotal+&&trtNameT&_i_);
         %*put trtNameV=&&trtNameV&_i_ trtNameT&_i_=&&trtNameT&_i_ trtTotal=&trtTotal;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
      
      *calculate proportion for trt;
      proc sort data=&OutData(keep=&trtNameP &AEDDtermVar) out=getcounts1;
         by &AEDDtermVar &trtNameP;
      run;  
      proc freq data=getcounts1 noprint;
         tables &AEDDtermVar*&trtNameP / out=countdt1;
      run;
    
      %let trPercentOccofTrtLabel = %nrbquote(%LocaleFill(&trPercentOccurrenceofX, f1=%nrbquote(&trtLabel)));
      
      data countdt1; 
         set countdt1;
         %do _i_ = 1 %to &nv_trt;
            if &trtNameP="&&trtNameV&_i_" then do;
               Percent_trt = round((count/&&trtNameT&_i_)*100,0.1);
               label Percent_trt = "&trPercentOccofTrtLabel";
            end;
         %end;
      run;
   
      proc datasets;
         modify countdt1;
            %if %upcase(&domain) = AE %then %do;
               %let trTotalCountofTrtLabel = %nrbquote(%LocaleFill(&trTotalCountofX, f1=%nrbquote(&trtLabel)));
               rename count = group_count1;
               label group_count1 = "&trTotalCountofTrtLabel";
            %end;
            %if %upcase(&domain) = CM %then %do;
               rename count = group_count3;
               label group_count3 = "&trMedicationCategoryCount";
            %end;
      quit; 

      *merge Trt count and proportion with output table;
      proc sql;
         create table &OutData as
         select * from &OutData as d1 left join countdt1 as d2 on d1.&AEDDtermVar=d2.&AEDDtermVar and d1.&trtNameP=d2.&trtNameP
         order by %if &fpool %then %do; POOLID %end; %else %do; USUBJID %end;;
      quit;
      data countdt1;
         set countdt1;
         retain T_trtNameP 0;
         if Percent_trt ^= . then T_trtNameP=T_trtNameP+1;
      run;
      data _null_;
         set countdt1;
         call symput("T_trtNameP_flag",T_trtNameP);
      run;
   %end;

   %if &sexflag %then %do;
      *get the total count for each Sex;
      proc sort data=&in_adsl out=SexNameSum;
         by Sex;
      run;
      proc freq data=SexNameSum noprint;
         tables Sex /out=Sexsum;
      run;
      %let _tid_=%sysfunc(open(Sexsum));
      %let nv_sex = %sysfunc(attrn(&_tid_,NOBS)); 
      %let SexTotal=0;
      %do _i_ = 1 %to &nv_sex;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %*let sexNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Sex))));
         %let sexNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let SexTotal=%eval(&SexTotal+&&sexNameT&_i_);
         %*put sexNameV=&&sexNameV&_i_ sexNameT&_i_=&&sexNameT&_i_ SexTotal=&SexTotal;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   
      * need to check if the adsl and dm has the same sex category, e.g. some use Female and the other one use F;
      proc sort data=&OutData out=SexNameSum2;
         by Sex;
      run;
      proc freq data=SexNameSum2 noprint;
         tables Sex /out=Sexsum2;
      run;
      %let _tid_=%sysfunc(open(Sexsum2));
      %let nv_sex2 = %sysfunc(attrn(&_tid_,NOBS)); 
      %do _i_ = 1 %to &nv_sex2;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let sexNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Sex))));
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));

      *calculate proportion for Sex;
      proc sort data=&OutData(keep=Sex &AEDDtermVar) out=getcounts1;
         by &AEDDtermVar Sex;
      run;  
      proc freq data=getcounts1 noprint;
         tables &AEDDtermVar*Sex / out=countdt1;
      run;

      data countdt1; 
         set countdt1;
         %do _i_ = 1 %to &nv_sex;
            if Sex="&&sexNameV&_i_" then do;
               Percent_Sex = round((count/&&sexNameT&_i_)*100,0.1);
               label Percent_Sex = "&trPercentOccurrenceofSex";
            end;
         %end;
      run;
   
      proc datasets;
         modify countdt1;
            %if %upcase(&domain) = AE %then %do;
               rename count = group_count2;
               label group_count2 = "&trTotalCountofSex";
            %end;
            %if %upcase(&domain) = CM %then %do;
               rename count = group_count3;
               label group_count3 = "&trMedicationCategoryCount";
            %end;
      quit; 

      *merge Sex count and proportion with output table;
      proc sql;
         create table &OutData as
         select * from &OutData as d1 left join countdt1 as d2 on d1.&AEDDtermVar=d2.&AEDDtermVar and d1.Sex=d2.Sex
         order by %if &fpool %then %do; POOLID %end; %else %do; USUBJID %end;;
      quit;
   %end;
   %if &raceflag %then %do;
      *get the total count for each Race;
      proc sort data=&in_adsl out=RaceNameSum;
         by Race;
      run;
      proc freq data=RaceNameSum noprint;
         tables Race /out=racesum;
      run;
      
      %let _tid_=%sysfunc(open(racesum));
      %let nv_race = %sysfunc(attrn(&_tid_,NOBS)); 
      %let raceTotal=0;
      %do _i_ = 1 %to &nv_race;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let raceNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Race))));
         %let raceNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let raceTotal=%eval(&raceTotal+&&raceNameT&_i_);
         %*put raceNameV=&&raceNameV&_i_ raceNameT&_i_=&&raceNameT&_i_ raceTotal=&raceTotal;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
      %let nv_raceTmp=&nv_race;

      *calculate proportion for Race;
      proc sort data=&OutData(keep=Race &AEDDtermVar) out=getcounts1;
         by &AEDDtermVar Race;
      run;  
      proc freq data=getcounts1 noprint;
         tables &AEDDtermVar*Race / out=countdt1;
      run;

      data countdt1; 
         set countdt1;
         %do _i_ = 1 %to &nv_race;
            if Race="&&raceNameV&_i_" then do;
               Percent_race = round((count/&&raceNameT&_i_)*100,0.1);
               label Percent_race = "&trPercentOccurrenceofRace";
            end;
         %end;
      run;
   
      proc datasets;
         modify countdt1;
            %if %upcase(&domain) = AE %then %do;
               rename count = group_count3;
               label group_count3 = "&trTotalCountofRace";
            %end;
            %if %upcase(&domain) = CM %then %do;
               rename count = group_count3;
               label group_count3 = "&trMedicationCategoryCount";
            %end;
      quit; 

      *merge Race count and proportion with output table;
      proc sql;
         create table &OutData as
         select * from &OutData as d1 left join countdt1 as d2 on d1.&AEDDtermVar=d2.&AEDDtermVar and d1.Race=d2.Race
         order by %if &fpool %then %do; POOLID %end; %else %do; USUBJID %end;;
      quit;
   %end;

   * use proc report to get the summary report for proportion and count for trt;
   %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE %then %do;
      data stattable_sub;
         set &OutData;
         keep USUBJID &AEDDtermVar AEBODSYS &trtNameP Sex Race COUNT PERCENT group_count group_count1 Percent_trt group_count2 Percent_Sex group_count3 Percent_race;
      run;
      data stattable_subBST;
         set stattable_sub;
         if PERCENT< &PropThresh then delete;            
      run;
      %NObsVars(stattable_subBST);
      %if ^&nobs %then %do;
         %put [MS_S_0016];
         %put ERROR: There are no events that occur for at least &PropThresh percent of selected subjects.;
         %put ERROR: Please relax the Percent Occurrence Threshold on the dialog Output tab.;
         %let exiterror=1;
         %goto exit;
      %end;
            
      proc sort data=stattable_sub  nodupkey;
         by &AEDDtermVar &trtNameP ;
      run;
      %slicedata(stattable_sub,&trtNameP,trtLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
      %NObsVars(trtLvlData);
      %let nsub = &nObs;
      %if &nsub <  &nv_trt %then %let nv_trt=&nsub;
      %if %length(&SubjectFilter) %then %do;
         *get the total count for each trt;
         proc sort data=stattable_sub out=trtNameSum;
            by &trtNameP;
         run;
         proc freq data=trtNameSum noprint;
            tables &trtNameP /out=trtsum1;
         run;           
         %let _tid_=%sysfunc(open(trtsum1));
         %let nv_trt = %sysfunc(attrn(&_tid_,NOBS)); 
         %let trtTotal=0;
         %do _i_ = 1 %to &nv_trt;
            %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
            %let trtNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&trtNameP))));
            %let trtNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
            %let trtTotal=%eval(&trtTotal+&&trtNameT&_i_);
         %end;
         %let _tid_ = %sysfunc(close(&_tid_));
         %let nsubjects=&trtTotal;
      %end;
      * get the total count for each body system category for each USUBJID;
      proc sort data=stattable_subBST out = trt_BS nodupkey;
         by AEBODSYS &trtNameP USUBJID ;
      run;
      proc freq data=trt_BS noprint;
         tables &trtNameP /out=trt_BST;
         by AEBODSYS &trtNameP;
      run;
      proc sort data=stattable_subBST out = trt_BS2 ;
         by AEBODSYS &trtNameP;
      run;
      proc freq data=trt_BS2 noprint;
         tables &trtNameP /out=trt_BSNoUID;
         by AEBODSYS &trtNameP;
      run;
      data trt_BSNoUID;
         set trt_BSNoUID;
         rename COUNT=COUNT_UID;
         drop PERCENT;
      run;
      data trt_BST;
         merge trt_BST trt_BSNoUID;
         by AEBODSYS &trtNameP;
      run;
      %slicedata(trt_BST,&trtNameP,trtBslData,%sysfunc(pathname(work)),_SubDataList=sList);
      data trtBslData;
         set trtBslData;
         if &trtNameP="" then delete;
      run;
      %NObsVars(trtBslData);
      %let bsobs=&nObs;
      %do _i_ = 1 %to &bsobs;
         %let _tid_=%sysfunc(open(trt_BST&_i_));
         %let nv_bs2&_i_ = %sysfunc(attrn(&_tid_,NOBS));
         %do _j_ = 1 %to &&nv_bs2&_i_;
            %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
            %let BodySysLvlV&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
            %let BodySysCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
            %let BodySysTrt&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&trtNameP))));
            %let BodyUIDCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT_UID))));
         %end;
         %let _tid_ = %sysfunc(close(&_tid_));
      %end;
         
      %do _i_ = 1 %to &nv_trt;
         %let trtNameLabel&_i_=&&trtNameV&_i_;
         %let trtNameV&_i_=%sysfunc(translate(&&trtNameV&_i_,'_','.','_',' ','_','/','_','(','_',')','_',',','_','-','_',":"));
         %if %length(&&trtNameV&_i_) <29 %then %let trtNameShort&_i_=&&trtNameV&_i_; 
         %else %let trtNameShort&_i_=%substr(&&trtNameV&_i_,1,28);
         data stattable_sub&_i_;
            length Gt1_trtName&_i_ $16 Pt1_trtName&_i_ $8 count_trt $16 PERCENT_Ttrt $8;
            set stattable_sub&_i_;
            /*rename COUNT=COUNT_&&trtNameV&_i_ PERCENT=PERCENT_&&trtNameV&_i_ group_count=group_count_&&trtNameV&_i_ group_count1=group_count1_&&trtNameV&_i_ Percent_trt=Percent_trt_&&trtNameV&_i_ 
               group_count2=group_count2_&&trtNameV&_i_ Percent_Sex=Percent_Sex_&&trtNameV&_i_;*/
               Gt1_trtName&_i_=group_count1;
               %if %length(&SubjectFilter) %then %do;
                  Pt1_trtName&_i_=round((group_count1/&&trtNameT&_i_)*100,0.1);
               %end;
               %else %do;
                  Pt1_trtName&_i_=Percent_trt;
               %end;
               Gt1_trtName&_i_=trim(left(Gt1_trtName&_i_))||" ("||trim(left(Pt1_trtName&_i_))||")";
               count_trt=COUNT;  
               PERCENT_Ttrt=PERCENT;
               count_trt =trim(left(count_trt))||" ("||trim(left(PERCENT_Ttrt))||")";
               label Gt1_trtName&_i_ =&&trtNameLabel&_i_;
               rename group_count1=group_count1&_i_ Percent_trt=Percent_trt&_i_;
               *drop group_count1 Percent_trt COUNT PERCENT;
         run;
         proc sort data= stattable_sub&_i_;
            by &AEDDtermVar;
         run;

         %if &_i_=1 %then %do;
            data stattable_subt;
               set stattable_sub&_i_;
            run;
         %end;
         %else %do;
            data stattable_subt;
               merge stattable_subt(in=a) stattable_sub&_i_(in=b);
               by &AEDDtermVar;
               if a or b;
            run;
         %end;
      %end;

      data stattable_subt;
         retain count_trtTA 0;
         set stattable_subt;
         %do _i_ = 1 %to &nv_trt;
            %if &_i_ =1 %then %do;
               if group_count1&_i_ =. then count_trtTA=0;
               else count_trtTA=group_count1&_i_;
            %end;
            %else %do;
               if group_count1&_i_ =. then count_trtTA=count_trtTA+0;
               else count_trtTA=count_trtTA+group_count1&_i_;
            %end;
         %end;
      run;
      data stattable_subt;
         length percent_trtA $8 count_trtA $16;
         set stattable_subt;
         count_trtA =count_trtTA;
         percent_trtTA = round((count_trtTA/&nsubjects)*100,0.1);
         percent_trtA=percent_trtTA;
         count_trtA =trim(left(count_trtA))||" ("||trim(left(percent_trtA))||")";
         /*%do _i_ = 1 %to &nv_trt;
            if Pt1_&&trtNameV&_i_< &PropThresh then delete;
         %end;*/
         if percent_trtTA< &PropThresh then delete;
         drop percent_trtTA count_trtTA count_trt;
         rename count_trtA=count_trt;
      run;

      * get indent for AE events;
      data stattable_subt;
         set stattable_subt;
         AEDDtermVar_pdf= %str("00"x)|| " " || &AEDDtermVar;
         AEDDtermVar_text= %str('1E'x)|| " " || %str('1E'x)|| " " || %str('1E'x)|| &AEDDtermVar;
         AEDDtermVar_rtf =%str('03'x) || " " ||%str('03'x) || " " || &AEDDtermVar ;
      run;
      /* To get any ADSL filters - where stmt or population */
      /* data OutLib.stattable_subt;
            set stattable_subt;
         run;
         proc sort data=stattable_subt out=outlib.adsl_bs ;
            by USUBJID AEBODSYS;
         run;
   
         %nobsvars(outlib.adsl_bs);
         %let nbsubjects = &nobs;   
         %put nbsubjects=&nbsubjects;  */
         
      proc sort data =stattable_subt;
         by AEBODSYS &trtNameP;
      run;
               
      %slicedata(stattable_subt,AEBODSYS,bsLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
      %NObsVars(bsLvlData);
      %let nv_bodysystem = &nObs;
            
      *get the body system level;      
      %do _i_ = 1 %to &nv_bodysystem;
         %let _tid_=%sysfunc(open(stattable_subt&_i_));
         %let nv_trt2 = %sysfunc(attrn(&_tid_,NOBS));
         %do _j_ = 1 %to &nv_trt2;
            %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
            %let BodySysNameV = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
            %let trtSysNameV = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,&trtNameP))));
         %end;
         %let _tid_ = %sysfunc(close(&_tid_));
               
         %let bsSum=0;
         %let bsUIDSum=0;
         %do _m_ = 1 %to &bsobs;
            %let bsTotal&_m_=;
            %do _n_ = 1 %to &&nv_bs2&_m_;
               %*put BodySysNameV=&BodySysNameV trtSysNameV=&trtSysNameV;
               %*put BodySysLvlV&_m_.&_n_=&&BodySysLvlV&_m_.&_n_ BodySysCount&_m_.&_n_=&&BodySysCount&_m_.&_n_ BodySysTrt&_m_.&_n_=&&BodySysTrt&_m_.&_n_;
               %if %qupcase(&&BodySysLvlV&_m_.&_n_) =%qupcase(&BodySysNameV) /*& 
                %qupcase(&&BodySysTrt&_m_.&_n_)=%qupcase(&trtSysNameV) */%then %do;
                  %*put m=&_m_ n=&_n_ BodySysCount&_m_.&_n_=&&BodySysCount&_m_.&_n_;
                  %let bsTotal&_m_=&&BodySysCount&_m_.&_n_; 
                  %let bsUID&_m_ =%sysfunc(round(%sysevalf(&&BodySysCount&_m_.&_n_/&&trtNameT&_m_)*100,0.1));
                  %let bsSum=%eval(&bsSum+&&BodySysCount&_m_.&_n_);
                  %let bsUIDSum=%eval(&bsUIDSum+&&BodyUIDCount&_m_.&_n_);
                  %let bsTRTLvL&_m_ = &&BodySysTrt&_m_.&_n_;
                  %let _n_= &&nv_bs2&_m_;
               %end;
               %*put bsTotal&_m_=&&bsTotal&_m_;
            %end;
         %end;
            
         %let bsUIDPerc=%sysfunc(round(%sysevalf(&bsSum/&trtTotal)*100,0.1));
               
         data stattable_subtbs;
            set stattable_subt&_i_;
            if _n_=1 then do;
               &AEDDtermVar = "&BodySysNameV";
               AEDDtermVar_pdf="&BodySysNameV";
               AEDDtermVar_text="&BodySysNameV";
               AEDDtermVar_rtf="&BodySysNameV";
               AEBODSYS=" ";
               &trtNameP=" ";
               Sex =" ";
               Race = " ";
               count_trt ="&bsSum(&bsUIDPerc%)";
               PERCENT_Ttrt =" ";
               group_count =" ";
               group_count2 =" ";
               Percent_Sex =" ";
               group_count3 =" ";
               Percent_race=" ";
               %do _j_ = 1 %to &nv_trt;
                  Gt1_trtName&_j_=" ";
                  %if %symexist(bsTotal&_j_) %then %do;
                     %if %length(&&bsTotal&_j_) %then %do;
                        Gt1_trtName&_j_= "&&bsTotal&_j_(&&bsUID&_j_%)";
                     %end;
                  %end;
                  %else %do;
                     Gt1_trtName&_j_=.;
                  %end;
               /* if &&bsTotal&_j_ >0 then do;
                     Gt1_&&trtNameShort&_j_= "N=&&bsTotal&_j_";
                  end;
                  else do;
                     Gt1_&&trtNameShort&_j_= " ";
                  end;*/
                  Pt1_trtName&_j_= " ";
               %end;
            end;
            else delete;
         run;
         data stattable_subt&_i_;
            set stattable_subtbs stattable_subt&_i_;
         run;

         %if &_i_=1 %then %do;
            data stattable_subt;
               set stattable_subt&_i_;
            run;
         %end;
         %else %do;
            data stattable_subt;
               set stattable_subt stattable_subt&_i_;
            run;
         %end;
      %end;

      /* proc sort data =stattable_subt (keep=AEBODSYS &AEDDtermVar count_trt %do _i_ = 1 %to &nv_trt; Gt1_&&trtNameShort&_i_ %end;)  nodupkey;
         by AEBODSYS &AEDDtermVar ;*/
               
      data stattable_subtT;
         set stattable_subt;
            if _n_=1 then do;
               &AEDDtermVar = "&trTotal";
               AEDDtermVar_pdf="&trTotal";
               AEDDtermVar_text="&trTotal";
               AEDDtermVar_rtf="&trTotal";
               AEBODSYS=" ";
               &trtNameP=" ";
               Sex =" ";
               Race = " ";
               count_trt ="N=&trtTotal";
               PERCENT_Ttrt =" ";
               group_count =" ";
               group_count2 =" ";
               Percent_Sex =" ";
               group_count3 =" ";
               Percent_race=" ";
               %do _i_ = 1 %to &nv_trt;
                  Gt1_trtName&_i_= "N=&&trtNameT&_i_";
                  Pt1_trtName&_i_= " ";
               %end;
            end;
            else delete;
      run;
      data stattable_subt;
         set stattable_subtT stattable_subt;
      run;

   %end;

   %if &sexflag %then %do;
   data stattable_sub;
      set &OutData;
      keep USUBJID &AEDDtermVar AEBODSYS Sex Race COUNT PERCENT group_count group_count1 Percent_trt group_count2 Percent_Sex group_count3 Percent_race;
   run;

   data stattable_subBST;
      set stattable_sub;
      if PERCENT< &PropThresh then delete;            
   run;
         
   proc sort data=stattable_sub  nodupkey;
      by &AEDDtermVar Sex ;
   run;
   %slicedata(stattable_sub,Sex,trtLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
         
   %NObsVars(trtLvlData);
   %let nsub = &nObs;
   %if &nsub <  &nv_sex %then %let nv_sex=&nsub;
   %if %length(&SubjectFilter) %then %do;
      *get the total count for each Sex;
      proc sort data=stattable_sub out=SexNameSum;
         by Sex;
      run;
      proc freq data=SexNameSum noprint;
         tables Sex /out=Sexsum;
      run;
   
      %let _tid_=%sysfunc(open(Sexsum));
      %let nv_sex = %sysfunc(attrn(&_tid_,NOBS)); 
      %let SexTotal=0;
      %do _i_ = 1 %to &nv_sex;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let sexNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Sex))));
         %let sexNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let SexTotal=%eval(&SexTotal+&&sexNameT&_i_);
         %*put sexNameV=&&sexNameV&_i_ sexNameT&_i_=&&sexNameT&_i_ SexTotal=&SexTotal;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   %end;
         
   * get the total count for each body system category for each USUBJID;
   proc sort data=stattable_subBST out = sex_BS nodupkey;
      by AEBODSYS SEX USUBJID ;
   run;
   proc freq data=sex_BS noprint;
      tables SEX /out=sex_BST;
      by AEBODSYS SEX;
   run;
   proc sort data=stattable_subBST out = sex_BS2 ;
      by AEBODSYS SEX;
   run;
   proc freq data=sex_BS2 noprint;
      tables SEX /out=trt_BSNoUID;
      by AEBODSYS SEX;
   run;
   data trt_BSNoUID;
      set trt_BSNoUID;
      rename COUNT=COUNT_UID;
      drop PERCENT;
   run;
   data sex_BST;
      merge sex_BST trt_BSNoUID;
      by AEBODSYS SEX;
   run;
   %slicedata(sex_BST,SEX,trtBslData,%sysfunc(pathname(work)),_SubDataList=sList);
   %NObsVars(trtBslData);
   %let bsobs=&nObs;
   %do _i_ = 1 %to &bsobs;
      %let _tid_=%sysfunc(open(sex_BST&_i_));
      %let nv_bs2&_i_ = %sysfunc(attrn(&_tid_,NOBS));
      %do _j_ = 1 %to &&nv_bs2&_i_ ;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
         %let BodySysLvlV&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
         %let BodySysCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let BodySysSex&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,SEX))));
         %let BodyUIDCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT_UID))));
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   %end;

   %do _i_ = 1 %to &nv_sex;
      %let sexNameLabel&_i_=&&sexNameV&_i_;
      %let sexNameV&_i_=%sysfunc(translate(&&sexNameV&_i_,'_','.','_',' ','_','/','_','(','_',')','_',',','_','-'));
      data stattable_sub&_i_;
         length Gt1_sexName&_i_ $16 Pt1_sexName&_i_ $8;
         set stattable_sub&_i_;
         Gt1_sexName&_i_ = group_count2;
         %if %length(&SubjectFilter) %then %do;
            Pt1_sexName&_i_ = round((group_count2/&&sexNameT&_i_)*100,0.1);
         %end;
         %else %do;
            Pt1_sexName&_i_ = Percent_sex;
         %end;
         Gt1_sexName&_i_=trim(left(Gt1_sexName&_i_))||" ("||trim(left(Pt1_sexName&_i_))||")";
         label Gt1_sexName&_i_ =&&sexNameLabel&_i_;
         rename group_count2=group_count2&_i_ Percent_sex=Percent_sex&_i_;
      run;
      proc sort data= stattable_sub&_i_;
         by &AEDDtermVar;
      run;
      %if &_i_=1 %then %do;
         data stattable_subS;
            set stattable_sub&_i_;
         run;
      %end;
      %else %do;
         data stattable_subS;
            merge stattable_subS(in=a) stattable_sub&_i_(in=b);
            by &AEDDtermVar;
            if a or b;
         run;
      %end;
   %end;
   data stattable_subS;
      retain count_SexTA 0;
      set stattable_subS;
      %do _i_ = 1 %to &nv_sex;
         %if &_i_ =1 %then %do;
            if group_count2&_i_ =. then count_SexTA=0;
            else count_SexTA=group_count2&_i_;
         %end;
         %else %do;
            if group_count2&_i_ =. then count_SexTA=count_SexTA+0;
            else count_SexTA=count_SexTA+group_count2&_i_;
         %end;
      %end;
   run;
   data stattable_subS;
      length percent_Sex $8 count_Sex $16;
      set stattable_subS;
      count_Sex =count_SexTA; 
      percent_SexTA = round((count_SexTA/&nsubjects)*100,0.1);
      percent_Sex=percent_SexTA;          
      /*%do _i_ = 1 %to &nv_sex;
         if Pt1_&&sexNameV&_i_ < &PropThresh then delete;
      %end;*/
      count_Sex =trim(left(count_Sex))||" ("||trim(left(percent_Sex))||")";
      if percent_SexTA < &PropThresh then delete;
      drop percent_SexTA count_SexTA;
   run;

   data stattable_subS;
      set stattable_subS;
      AEDDtermVar_pdf= %str("00"x)|| " " || &AEDDtermVar;
      AEDDtermVar_text= %str('1E'x)|| " " || %str('1E'x)|| " " || %str('1E'x)|| &AEDDtermVar;
      AEDDtermVar_rtf =%str('03'x) || " " ||%str('03'x) || " " || &AEDDtermVar ;          
   run;
         
   proc sort data =stattable_subS;
      by AEBODSYS SEX;
   run;
               
   %slicedata(stattable_subS,AEBODSYS,bsLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
   %NObsVars(bsLvlData);
   %let nv_bodysystem = &nObs;
            
   *get the body system level;      
   %do _i_ = 1 %to &nv_bodysystem;
      %let _tid_=%sysfunc(open(stattable_subS&_i_));
      %let nv_trt2 = %sysfunc(attrn(&_tid_,NOBS)); 
      %do _j_ = 1 %to &nv_trt2;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
         %let BodySysNameV = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %let bsSum=0;
      %let bsUIDSum=0;
      %do _m_ = 1 %to &bsobs;
         %let bsTotal&_m_=;
         %do _n_ = 1 %to &&nv_bs2&_m_ ;
            %if %qupcase(&&BodySysLvlV&_m_.&_n_) =%qupcase(&BodySysNameV) /*& 
             %qupcase(&&BodySysTrt&_m_.&_n_)=%qupcase(&trtSysNameV) */%then %do;
               %let bsTotal&_m_=&&BodySysCount&_m_.&_n_; 
               %let bsUID&_m_ =%sysfunc(round(%sysevalf(&&BodySysCount&_m_.&_n_/&&sexNameT&_m_)*100,0.1));
               %let bsSum=%eval(&bsSum+&&BodySysCount&_m_.&_n_);
               %let bsUIDSum=%eval(&bsUIDSum+&&BodyUIDCount&_m_.&_n_);
               %let bsSexLvL&_m_ = &&BodySysSex&_m_.&_n_;
               %let _n_= &&nv_bs2&_m_ ;
               %*put bsTotal&_m_=&&bsTotal&_m_ bsUID&_m_=&&bsUID&_m_ bsSum=&bsSum bsSexLvL&_m_=&&bsSexLvL&_m_;
            %end; 
         %end;
      %end;
            
      %let bsUIDPerc=%sysfunc(round(%sysevalf(&bsSum/&SexTotal)*100,0.1));
               
      data stattable_subSbs;
         length sex $40;
         set stattable_subS&_i_;
         if _n_=1 then do;
            &AEDDtermVar = "&BodySysNameV";
            AEDDtermVar_pdf="&BodySysNameV";
            AEDDtermVar_text="&BodySysNameV";
            AEDDtermVar_rtf="&BodySysNameV";
            AEBODSYS=" ";
            Sex =" ";
            Race = " ";
            COUNT =" ";
            PERCENT =" ";
            group_count =" ";
            group_count1 =" ";
            Percent_trt =" ";
            group_count3 =" ";
            Percent_race=" ";
            %do _j_ = 1 %to &nv_sex;
               Gt1_sexName&_j_= " ";
               %if %length(&&bsTotal&_j_ ) %then %do;
                  Gt1_sexName&_j_= "&&bsTotal&_j_(&&bsUID&_j_%)";
               %end;
               Pt1_sexName&_j_= " ";
            %end;
            count_Sex = "&bsSum(&bsUIDPerc%)";
            percent_Sex=" ";
         end;
         else delete;
      run;
      data stattable_subS&_i_;
         set stattable_subSbs stattable_subS&_i_;
      run;

      %if &_i_=1 %then %do;
         data stattable_subS;
            set stattable_subS&_i_;
         run;
      %end;
      %else %do;
         data stattable_subS;
            set stattable_subS stattable_subS&_i_;
         run;
      %end;
   %end;
         
   data stattable_subST;
      set stattable_subS;
         if _n_=1 then do;
            &AEDDtermVar = "&trTotal";
            AEDDtermVar_pdf="&trTotal";
            AEDDtermVar_text="&trTotal";
            AEDDtermVar_rtf="&trTotal";
            AEBODSYS=" ";
            Sex =" ";
            Race = " ";
            COUNT =" ";
            PERCENT =" ";
            group_count =" ";
            group_count1 =" ";
            Percent_trt =" ";
            group_count3 =" ";
            Percent_race=" ";
            %do _i_ = 1 %to &nv_sex;
               Gt1_sexName&_i_= "N=&&sexNameT&_i_";
               Pt1_sexName&_i_= " ";
            %end;
            count_Sex = "N=&SexTotal";
            percent_Sex=" ";
         end;
         else delete;
   run;
   data stattable_subS;
      set stattable_subST stattable_subS;
   run;
   %end;
   %if &raceflag %then %do;
         
   data stattable_sub;
      set &OutData;
      keep USUBJID &AEDDtermVar AEBODSYS Sex Race COUNT PERCENT group_count group_count1 Percent_trt group_count2 Percent_Sex group_count3 Percent_race;
   run;

   data stattable_subBST;
      set stattable_sub;
      if PERCENT< &PropThresh then delete;            
   run;  
   data stattable_sub;
      set stattable_subBST;
   run;
   proc sort data=stattable_sub  nodupkey;
      by &AEDDtermVar Race ;
   run;     
   %slicedata(stattable_sub,Race,trtLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
   %NObsVars(trtLvlData);
   %let nsub = &nObs;
   %if &nsub <  &nv_race %then %let nv_race=&nsub;
         
   %if %length(&SubjectFilter) %then %do;
      *get the total count for each Race;
      proc sort data=stattable_sub out=RaceNameSum;
         by Race;
      run;
      proc freq data=RaceNameSum noprint;
         tables Race /out=racesum;
      run;
      %let _tid_=%sysfunc(open(racesum));
      %let nv_race = %sysfunc(attrn(&_tid_,NOBS)); 
      %let raceTotal=0;
      %do _i_ = 1 %to &nv_race;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let raceNameV&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Race))));
         %let raceNameT&_i_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let raceTotal=%eval(&raceTotal+&&raceNameT&_i_);
         %*put raceNameV=&&raceNameV&_i_ raceNameT&_i_=&&raceNameT&_i_ raceTotal=&raceTotal;
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   %end;
         
   * get the total count for each body system category for each USUBJID;
   proc sort data=stattable_subBST out = race_BS nodupkey;
      by AEBODSYS Race USUBJID ;
   run;
   proc freq data=race_BS noprint;
      tables Race /out=race_BST;
      by AEBODSYS Race;
   run;  
   proc sort data=stattable_subBST out = race_BS2 ;
      by AEBODSYS Race;
   run;
   proc freq data=race_BS2 noprint;
      tables Race /out=trt_BSNoUID;
      by AEBODSYS Race;
   run;
   data trt_BSNoUID;
      set trt_BSNoUID;
      rename COUNT=COUNT_UID;
      drop PERCENT;
   run;
   data race_BST;
      merge race_BST trt_BSNoUID;
      by AEBODSYS Race;
   run;
   %slicedata(race_BST,Race,trtBslData,%sysfunc(pathname(work)),_SubDataList=sList);
   %NObsVars(trtBslData);
   %let bsobs=&nObs;
   %do _i_ = 1 %to &bsobs;
      %let _tid_=%sysfunc(open(race_BST&_i_));
      %let nv_bs2&_i_  = %sysfunc(attrn(&_tid_,NOBS));
      %do _j_ = 1 %to &&nv_bs2&_i_ ;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
         %let BodySysLvlV&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
         %let BodySysCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT))));
         %let BodySysRace&_i_.&_j_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Race))));
         %let BodyUIDCount&_i_.&_j_ = %sysfunc(getvarn(&_tid_,%sysfunc(varnum(&_tid_,COUNT_UID))));
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));
   %end;

   %* handle cases when some races were filter out;
   %let _tid_=%sysfunc(open(trtBslData));
   %let nv_race2 = %sysfunc(attrn(&_tid_,NOBS)); 
   %if &nv_race2 <  &nv_raceTmp %then %do;
      %do _i_ = 1 %to &nv_race2;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_i_));
         %let raceNameV2&_i_ = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,Race))));
         %do _j_ = 1 %to &nv_raceTmp;
            %if (%quote(%upcase(&&raceNameV2&_i_)) = %quote(%upcase(&&raceNameV&_j_)))  %then %do;
               %let raceNameV2&_i_ = &&raceNameV&_j_;
               %let raceNameT2&_i_ = &&raceNameT&_j_;
               %*put raceNameV2&_i_=&&raceNameV2&_i_ raceNameT2&_i_ =&&raceNameT2&_i_;
            %end;
         %end;
      %end;
      %do _i_ = 1 %to &nv_race2;
         %let raceNameV&_i_ = &&raceNameV2&_i_;
         %let raceNameT&_i_ = &&raceNameT2&_i_;
      %end;
   %end;
   %let _tid_ = %sysfunc(close(&_tid_));
   
   %do _i_ = 1 %to &nv_race;
      %let raceNameLabel&_i_=&&raceNameV&_i_;
      %let raceNameV&_i_=%sysfunc(translate(&&raceNameV&_i_,'_','.','_',' ','_','/','_','(','_',')','_',',','_','-'));
      %if %length(&&raceNameV&_i_) <29 %then %let raceNameShort&_i_=&&raceNameV&_i_; 
      %else %let raceNameShort&_i_=%substr(&&raceNameV&_i_,1,28);
      %*put raceNameV&_i_=&&raceNameV&_i_ raceNameShort&_i_=&&raceNameShort&_i_;
      data stattable_sub&_i_;
         length Gt1_raceName&_i_ $16 Pt1_raceName&_i_ $8;
         set stattable_sub&_i_;
            Gt1_raceName&_i_ = group_count3;
            %if %length(&SubjectFilter) %then %do;
               Pt1_raceName&_i_ = round((group_count3/&&raceNameT&_i_)*100,0.1);;
            %end;
            %else %do;
               Pt1_raceName&_i_ = Percent_race;
            %end;
            Gt1_raceName&_i_=trim(left(Gt1_raceName&_i_))||" ("||trim(left(Pt1_raceName&_i_))||")";
            label Gt1_raceName&_i_ =&&raceNameLabel&_i_;
            rename group_count3=group_count3&_i_ Percent_race=Percent_race&_i_;
      run;
      proc sort data= stattable_sub&_i_;
         by &AEDDtermVar;
      run;
      %if &_i_=1 %then %do;
         data stattable_subR;
            set stattable_sub&_i_;
         run;
      %end;
      %else %do;
         data stattable_subR;
            merge stattable_subR(in=a) stattable_sub&_i_(in=b);
            by &AEDDtermVar;
            if a or b;
         run;
      %end;
   %end;
         
   data stattable_subR;
      retain count_RaceTA 0;
      set stattable_subR;
      %do _i_ = 1 %to &nv_race;
         %if &_i_ =1 %then %do;
            if group_count3&_i_ =. then count_RaceTA=0;
            else count_RaceTA=group_count3&_i_;
         %end;
         %else %do;
            if group_count3&_i_ =. then count_RaceTA=count_RaceTA+0;
            else count_RaceTA=count_RaceTA+group_count3&_i_;
         %end;
      %end;
   run;

   data stattable_subR;
      length percent_Race $8 count_Race $16;
      set stattable_subR;
      count_Race =count_RaceTA; 
      percent_RaceTA = round((count_RaceTA/&nsubjects)*100,0.1);
      percent_Race=percent_RaceTA;  
      count_Race =trim(left(count_Race))||" ("||trim(left(percent_Race))||")";
      /*%do _i_ = 1 %to &nv_race;
         if Pt1_&&raceNameV&_i_ < &PropThresh then delete;
      %end;*/
      if percent_RaceTA < &PropThresh then delete;
      drop percent_RaceTA count_RaceTA;
   run;

   * get indent for AE events;
   data stattable_subR;
      set stattable_subR;
      AEDDtermVar_pdf= %str("00"x)|| " " || &AEDDtermVar;
      AEDDtermVar_text= %str('1E'x)|| " " || %str('1E'x)|| " " || %str('1E'x)|| &AEDDtermVar;
      AEDDtermVar_rtf =%str('03'x) || " " ||%str('03'x) || " " || &AEDDtermVar ;       
   run;

   proc sort data =stattable_subR;
      by AEBODSYS Race;
   run;
               
   %slicedata(stattable_subR,AEBODSYS,bsLvlData,%sysfunc(pathname(work)),_SubDataList=sList);
   %NObsVars(bsLvlData);
   %let nv_bodysystem = &nObs;
         
   *get the body system level;      
   %do _i_ = 1 %to &nv_bodysystem;
   
   %let _tid_=%sysfunc(open(stattable_subR&_i_));
   %let nv_trt2 = %sysfunc(attrn(&_tid_,NOBS)); 
      %do _j_ = 1 %to &nv_trt2;
         %let _rc_=%sysfunc(fetchobs(&_tid_,&_j_));
         %let BodySysNameV = %sysfunc(getvarc(&_tid_,%sysfunc(varnum(&_tid_,AEBODSYS))));
      %end;
      %let _tid_ = %sysfunc(close(&_tid_));

      %let bsSum=0;
      %let bsUIDSum=0;
      %do _m_ = 1 %to &bsobs;
         %let bsTotal&_m_=;
         %do _n_ = 1 %to &&nv_bs2&_m_ ;
            %if %qupcase(&&BodySysLvlV&_m_.&_n_) =%qupcase(&BodySysNameV) %then %do;
               %let bsTotal&_m_=&&BodySysCount&_m_.&_n_; 
               %let bsUID&_m_ =%sysfunc(round(%sysevalf(&&BodySysCount&_m_.&_n_/&&raceNameT&_m_)*100,0.1));
               %let bsSum=%eval(&bsSum+&&BodySysCount&_m_.&_n_);
               %let bsUIDSum=%eval(&bsSum+&&BodyUIDCount&_m_.&_n_);
               %let bsRaceLvL&_m_ = &&BodySysRace&_m_.&_n_;
               %let _n_= &&nv_bs2&_m_ ;
            %end;
         %end;
      %end;
            
      %let bsUIDPerc=%sysfunc(round(%sysevalf(&bsSum/&RaceTotal)*100,0.1));
               
      data stattable_subRbs;
         set stattable_subR&_i_;
         if _n_=1 then do;
            &AEDDtermVar = "&BodySysNameV";
            AEDDtermVar_pdf="&BodySysNameV";
            AEDDtermVar_text="&BodySysNameV";
            AEDDtermVar_rtf="&BodySysNameV";
            AEBODSYS=" ";
            Sex =" ";
            Race = " ";
            COUNT =" ";
            PERCENT =" ";
            group_count =" ";
            group_count1 =" ";
            Percent_trt =" ";
            group_count2 =" ";
            Percent_sex=" ";
            %do _j_ = 1 %to &nv_race;
               Gt1_raceName&_j_=" ";
               %if  %length(&&bsTotal&_j_) %then %do;
                  Gt1_raceName&_j_= "&&bsTotal&_j_(&&bsUID&_j_%)";
               %end;
               Pt1_raceName&_j_= " ";
            %end;
            count_Race = "&bsSum(&bsUIDPerc%)";
            percent_Race=" ";
         end;
         else delete;
      run;
      data stattable_subR&_i_;
         set stattable_subRbs stattable_subR&_i_;
      run;

      %if &_i_=1 %then %do;
         data stattable_subR;
            set stattable_subR&_i_;
         run;
      %end;
      %else %do;
         data stattable_subR;
            set stattable_subR stattable_subR&_i_;
         run;
      %end;
   %end;
         
   data stattable_subRT;
      set stattable_subR;
         if _n_=1 then do;
            &AEDDtermVar = "&trTotal";
            AEDDtermVar_pdf="&trTotal";
            AEDDtermVar_text="&trTotal";
            AEDDtermVar_rtf="&trTotal";
            AEBODSYS=" ";
            Sex =" ";
            Race = " ";
            COUNT =" ";
            PERCENT =" ";
            group_count =" ";
            group_count1 =" ";
            Percent_trt =" ";
            group_count2 =" ";
            Percent_sex=" ";
            %do _i_ = 1 %to &nv_race;
               Gt1_raceName&_i_= "N=&&raceNameT&_i_";
               Pt1_raceName&_i_= " ";
            %end;
            count_Race = "N=&RaceTotal";
            percent_Race=" ";
         end;
         else delete;
   run;
   data stattable_subR;
      set stattable_subRT stattable_subR;
   run;
   %end;
   
   %let SubjectFilterTmp = &trAllSubjects;
   %if %length(&SubjectFilter) %then %do;
      %let SubjectFilterTmp = &SubjectFilter;
   %end;
   %if %index(%upcase(&TEAEOnly),EMERGENT) %then %do;
      %let TEAEOnlyTmp =&trTreatmentEmergentAdvEvents;
   %end;
   %else %if %index(%upcase(&TEAEOnly),PRE) %then %do;
      %let TEAEOnlyTmp =&trPreTreatmentEvents;
   %end;
   %else %if %index(%upcase(&TEAEOnly),OFF) %then %do;
      %let TEAEOnlyTmp=&trOfftreatmentEvents with the offset for end of dosing as &TrtOffSet;
   %end;
   %else %if %index(%upcase(&TEAEOnly),ON) %then %do;
      %let TEAEOnlyTmp=&trOntreatmentEvents with the offset for end of dosing as &TrtOffSet;
   %end;
   %else %do;
      %let TEAEOnlyTmp =&trAllEventTypes;
   %end;
   
   * calculate a count table by severity or by casualty;
   %if &AESerCount = Yes %then %do;
      proc sort data=&OutData out=outtmp;
         by USUBJID;
      run;
      proc sort data=outdataSer;
         by USUBJID;
      run;

      %let aeserflag=0;    
      %let _ind_ = %sysfunc(open(outtmp));
      %let sevnum = %sysfunc(varnum(&_ind_,aesev));
      
      %if &sevnum>0 %then %do;
         %let aeserflag=1;
      %end;
      %let _ind_ = %sysfunc(close(&_ind_));
      
      %if &aeserflag=0 %then %do;
         data outdataSer;
            merge outtmp outdataSer;
         run;
      %end;
      %else %do;
         data outdataSer;
            set outtmp;
         run;        
      %end;
      %let _ind_ = %sysfunc(open(outdataSer));
      %let sevnum = %sysfunc(varnum(&_ind_,aesev));
      %if &sevnum=0 %then %do;
         %let toxgrnum = %sysfunc(varnum(&_ind_,aetoxgrc));
         %if &toxgrnum > 0 %then %let AeSev = toxgrc;
      %end;
      %else %let AeSev = sev;
      %let _ind_ = %sysfunc(close(&_ind_));
      
      data outdataSer;
         set outdataSer;
         if AE&AESEV='' or &trtNameP='' then delete;
      run;
      proc sort data=outdataSer(keep=AE&AESEV AEBODSYS &trtNameP AEDECOD) out=getcounts1;
         by AE&AESEV &trtNameP;
      run;  
      proc freq data=getcounts1 noprint;
         tables AEBODSYS *AEDECOD* &trtNameP*AE&AESEV  / out=countdt1;
      run;

      data countServ;
         set countdt1;
      run;
      * check if there is any severity levels for some treatment missing;
      proc sort data=outdataSer;
         by AESEV;
      run;
      proc summary data=outdataSer;
         by AE&AESEV;
         output out=_p2;
      run;

      %nobsvars(_p2);
      %let ServNum=&nobs;
      %let tid = %sysfunc(open(_p2));  
      %do i=1 %to &ServNum;
         %let _rc_=%sysfunc(fetchobs(&tid,&i));
         %let servlevel&i =  %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,AE&AESEV))));
         %let servlevelunder&i=%sysfunc(translate(&&servlevel&i,%str(_),%str( )));
         %let servFlag&i=0;
         %put servlevel&i=&&servlevel&i;
      %end;
      %let tid = %sysfunc(close(&tid));
      
      * divide the main data by treatment level;
      %SliceData (countServ,&trtNameP,servlvl,%sysfunc(pathname(work)),_SubDataList=sList);  
      %nobsvars(servlvl);
      %let n1=&nobs;
      
      %let tid = %sysfunc(open(servlvl));
      %do i=1 %to &n1;
         %let _rc_=%sysfunc(fetchobs(&tid,&i));
         %let trtvar&i =  %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,&trtNameP))));
         %put trtvar&i=&&trtvar&i;
      %end;
      %let tid = %sysfunc(close(&tid));
      
      data severitylvl;
         set countServ;
      run;
      %SliceData (severitylvl,AE&AESEV,severitytable,%sysfunc(pathname(work)),_SubDataList=sList);
      %nobsvars(severitytable);
      %let n2=&nobs;
      
      %do i=1 %to &n1;
         *divide data by severity level;
         data countplacebo;
            set countserv&i;
         run;  
         %SliceData (countplacebo,AE&AESEV,servPlvl,%sysfunc(pathname(work)),_SubDataList=sList);
         %nobsvars(servPlvl);
         %let n3=&nobs;

         %if &n3<&n2 %then %do;

            data severitytable2;
               merge severitytable(in=ina) servPlvl(in=inb);
               by AE&AESEV;
               if ina and not inb;
            run;

            data countplacebo;
               set countplacebo severitytable2;
            run;

            %SliceData (countplacebo,AE&AESEV,servPlvl,%sysfunc(pathname(work)),_SubDataList=sList);

            %*put n1=&n1 n2=&n2 n3=&n3;
         %end;

         %let tid = %sysfunc(open(servPlvl));
         %do _j_=1 %to &n2;
            %let _rc_=%sysfunc(fetchobs(&tid,&_j_));
            %let sevvar&_j_ =  %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,AE&AESEV))));
            %let sevvarunder&_j_ = &&sevvar&_j_;
            %let sevvarunder&_j_=%sysfunc(translate(&&sevvar&_j_,%str(_),%str( )));
            %put sevvar&_j_=&&sevvarunder&_j_;
         %end; 
         %let tid = %sysfunc(close(&tid));
         
         *find missing severity level;
         %if &ServNum > &n2 %then %do;
            %do _sl_ = 1 %to &ServNum;
               %let servFlag&_sl_=0;
               %do _j_=1 %to &n2;
                  %if %nrbquote(&&servlevel&_sl_) = %nrbquote(&&sevvar&_j_) %then %do;
                     %let servFlag&_sl_=1;
                  %end;
               %end;
            %end;
         %end;

         %do j=1 %to &n2;
            data countplacebo&j;
               length count2 $6;
               set countplacebo&j;
               count2=count;
            run;
            data servsum1;
               set countplacebo&j;
               if _n_=1 then do;
                  count2="&&trtvar&i";
                  percent =.;
                  count =.;
               end;
               /*else if _n_=2 then do;
                  count2="&&sevvar&j";
                  percent =.;
                  count =.;
               end;*/
               else delete;
            run;
            data countplacebo&j;
               set countplacebo&j;
               rename Count2=sev&i._&j Percent=per&j;
               label Count2="sev&i.&&sevvarunder&j" Percent="per_&&sevvarunder&j";
               drop AE&AESEV  count;
            run;
            %if &j=1 & ^%sysfunc(fileexist(countServsum&i)) %then %do;
               data countServsum&i;
                  set countplacebo&j;
               run;
            %end;
            %else %do;
               proc sort data=countplacebo&j;
                  by AEBODSYS AEDECOD; 
               run;
               proc sort data=countServsum&i;
                  by AEBODSYS AEDECOD; 
               run;
      
               data countServsum&i;
                  merge countServsum&i countplacebo&j;
                  by AEBODSYS AEDECOD;
               run;
            %end;

         %end;
         
         * merge missing severity level in each treatment group;
         %if &servNum > &n2 %then %do;
            %do _sl_ = 1 %to &ServNum;
               %put servFlag=&&servFlag&_sl_;
               %if &&servFlag&_sl_=0 %then %do;
                  %let n2=%eval(&n2+1);
                  %let sevvarunder&n2 =&&servlevelunder&_sl_;
               
                  data servnum1(keep= AEBODSYS AEDECOD Count2);
                     length count2 $6;
                     set countServsum&i;
                     Count2=.;
                  run;
                  data servnum1;
                     set servnum1;
                     rename Count2=sev&i.&&servlevel&_sl_;
                  run;

                  %if ^%sysfunc(fileexist(countServsum&i)) %then %do;
                     data countServsum&i;
                        merge countServsum&i servnum1;
                        by AEBODSYS AEDECOD;
                     run;
                  %end;
                  %else %do;
                     data countServsum&i;
                        set servnum1;
                     run;
                  %end;
               %end;
            %end;
         %end;

         %if &i =1 %then %do;
            data countSerTotal;
               set countServsum&i;
            run;
         %end;
         %else %do;
            data countSerTotal;
               merge countSerTotal countServsum&i;
               by AEBODSYS AEDECOD;
            run;
         %end;
      %end;
      data countSerTotal;
         set countSerTotal;
         if AEBODSYS="" & AEDECOD="" then delete;
      run;
      
   %end;
      
   %if &Output_RTF = Yes %then %do;
      %if &Server = Local %then %do;
         ods rtf file="&ClientOutPath.\&OutName..rtf" bodytitle;
      %end;
      %else %if &Server ne Local %then %do;
         ods rtf file = "&ServerOutPath.\&OutName..rtf" bodytitle;
      %end;

      ods escapechar='^';
      options nobyline nodate missing = "" orientation = landscape;

      * calculate a count table by severity or by casualty;
      %if &AESerCount = Yes %then %do;

         PROC REPORT DATA=countserTotal %if %eval(&n1+&n2) >= 8 %then %do; style(column)=[fontsize=2 ]; %end;;
            COLUMN AEBODSYS AEDECOD ("&trtLabel" %do i=1 %to &n1; ("&&trtvar&i" (%do j=1 %to &ServNum; sev&i._&j %end;)) %end;)
               /* (%do i=1 %to &n1; ( %str("&&trtvar&i") (Mild Moderate Severe)) %end;)*/;
            %if %eval(&n1+&n2) > 5 & %eval(&n1+&n2) < 7 %then %do;
               DEFINE AEBODSYS / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
               DEFINE AEDECOD / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
            %end;
            %if %eval(&n1+&n2) > 6 %then %do;
               DEFINE AEBODSYS / display FORMAT= $80. WIDTH=10 SPACING=2 LEFT " " order=data; 
               DEFINE AEDECOD / display FORMAT= $80. WIDTH=10 SPACING=2 LEFT " " order=data; 
            %end;
            %else %do;
               DEFINE AEBODSYS / display FORMAT= $80. WIDTH=50 SPACING=2 LEFT " " order=data; 
               DEFINE AEDECOD / display FORMAT= $80. WIDTH=50 SPACING=2 LEFT " " order=data; 
            %end;
            **THE NO ZERO COMMAND IS USED SINCE ALL PRODUCT CATEGORIES WILL NOT BE
            REPRESENTED FOR EACH PRODUCT LINE IN THE TABLE;
            %do i=1 %to &n1;
               *  DEFINE %str(&&trtvar&i) / display "&&trtvar&i" width=20 ;
               *  define TRT01P /across;
               %do j=1 %to &n2;
                  %if %eval(&n1+&n2) > 5 & %eval(&n1+&n2) <= 6 %then %do;
                     DEFINE sev&i._&j  /display "&&sevvar&j" width=4 style(column)=[cellwidth=6%];
                  %end;
                  %else %if %eval(&n1+&n2) > 6 & %eval(&n1+&n2) < 8 %then %do;
                     DEFINE sev&i._&j  /display "&&sevvar&j" width=4 style(column)=[cellwidth=5%];
                  %end;
                  %else %if %eval(&n1+&n2) >= 8 %then %do;
                     DEFINE sev&i._&j  /display "&&sevvar&j" width=3 style(column)=[cellwidth=4%] ;
                  %end;
                  %else %do;
                     DEFINE sev&i._&j  /display "&&sevvar&j" width=20  ;
                  %end;
               %end;
            %end;
            title1 height=17pt "&trAdvEvCtBySevLev";
         RUN;  
      %end;
   
      * get the proportion and occurrence report for trt;
      %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE and &T_trtNameP_flag ^= 0 %then %do;
         %let trtMaxLen=0;
         %do i=1 %to &nv_trt;
            %let trtLen&i=%length(&&trtNameLabel&i);
            %if &trtMaxLen < &&trtLen&i %then %do;
               %let trtMaxLen=&&trtLen&i;
            %end;
         %end;

         proc report data = stattable_subt LS=97  PS=54  SPLIT="#" CENTER nowindows missing Contents="&trAETrtPropReport";
            column AEDDtermVar_rtf  ("&trtLabel" %do i=1 %to &nv_trt; ("%nrbquote(&&trtNameLabel&i)" GT1_trtName&i) %end;("&trTotal" Count_trt));
            define AEDDtermVar_rtf / display FORMAT= $80. WIDTH=70 SPACING=3 LEFT " " order=data; 
         
            %do i=1 %to &nv_trt;
               %if &trtMaxLen >45 & &nv_trt < 5 %then %do;
                  define GT1_trtName&i / display "&trCount (%)"  WIDTH=20  style(column)=[cellwidth=30%] ;
               %end;
               %else %do;
                  define GT1_trtName&i / display "&trCount (%)"  WIDTH=20;
               %end;
         *      define Pt1_&&trtNameV&i / display "&trPercent"  WIDTH=20  ;
            %end;
            define Count_trt / display "&trCount (%)"  WIDTH=20  ;

            title1 height=17pt "&trAEEvOccAndPropRepforTrt";
            %if &PropThresh>0 %then %do;
            %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
               title2 height=17pt "&trRepEvWithALOPTOcc";
            %end;
            %else %do;
               title2 height=17pt "&trReportingAllEvents";
            %end;
            %if %upcase(&AESerOnly)=YES %then %do;
               %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&StudyName: &trTIncSAEOforSFTC";
            %end;
            %else %do;
               %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&StudyName: &trTforSFTC";
            %end;
         run;
      %end;
      
      %if &sexflag %then %do;
         proc report data = stattable_subS LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAESexPropReport";
            column AEDDtermVar_rtf  ("&trSex" %do i=1 %to &nv_sex; ("&&sexNameLabel&i" GT1_sexName&i) %end;("&trTotal" Count_sex));
            
            define AEDDtermVar_rtf / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
            %do i=1 %to &nv_sex;
                define GT1_sexName&i / display "&trCount (%)"  WIDTH=20  ;
          *     define Pt1_&&sexNameV&i / display "&trPercent"  WIDTH=20  ;
            %end;
            define Count_sex / display "&trCount (%)"  WIDTH=20  ;   

            title1 height=17pt "&trAEEvOccAndPropRepforSex";
            %if &PropThresh>0 %then %do;
               %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
               title2 height=17pt "&trRepEvWithALOPTOcc";
            %end;
            %else %do;
               title2 height=17pt "&trReportingAllEvents";
            %end;
            %if %upcase(&AESerOnly)=YES %then %do;
               %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&trTIncSAEOforSFTC";
            %end;
            %else %do;
               %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&trTforSFTC";
            %end;
         run;
      %end;
      %if &raceflag %then %do;
         proc report data = stattable_subR LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAERacePropReport";
            column AEDDtermVar_rtf  ("&trRace" %do i=1 %to &nv_race; ("&&raceNameLabel&i" GT1_raceName&i) %end;("&trTotal" Count_Race));
                
            %if &nv_race >5 %then %do;
               define AEDDtermVar_rtf / display  FORMAT= $80. WIDTH=70 style(column)=[cellwidth=15%] SPACING=2 LEFT " " order=data; 
               %do i=1 %to &nv_race;
                   define GT1_raceName&i / display "&trCount (%)"  WIDTH=13 style(column)=[cellwidth=10%] ;
            *      define Pt1_&&raceNameV&i / display "&trPercent (%)"  WIDTH=20  ;
               %end;
               define Count_Race / display "&trCount (%)"  WIDTH=15  ;
            %end;
            %else %do;
               define AEDDtermVar_rtf / display  FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
               %do i=1 %to &nv_race;
                   define GT1_raceName&i / display "&trCount (%)"  WIDTH=20  ;
            *      define Pt1_&&raceNameV&i / display "&trPercent (%)"  WIDTH=20  ;
               %end;
               define Count_Race / display "&trCount (%)"  WIDTH=20  ;
            %end;
   
            title1 height=17pt "&trAEEvOccAndPropRepforRace";
            %if &PropThresh>0 %then %do;
               %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
               title2 height=17pt "&trRepEvWithALOPTOcc";
            %end;
            %else %do;
               title2 height=17pt "&trReportingAllEvents";
            %end;
            %if %upcase(&AESerOnly)=YES %then %do;
               %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&trTIncSAEOforSFTC";
            %end;
            %else %do;
               %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
               title3 height=12pt "&trTforSFTC";
            %end;
         run; 
      %end;

         ods rtf close;
         %if &syserr > 4 %then %goto exit;
      %end;
      %if &Output_PDF = Yes %then %do;
         %if &Server = Local %then %do;
            ods pdf file="&ClientOutPath.\&OutName..pdf" ;
         %end;
         %else %if &Server ne Local %then %do;
            ods pdf file="&ServerOutPath.\&OutName..pdf" ;
         %end;
         ods escapechar='^';
         options nobyline nodate missing = "" orientation = landscape;

         * calculate a count table by severity or by casualty;
         %if &AESerCount = Yes %then %do;
            PROC REPORT DATA=countserTotal ;
               COLUMN AEBODSYS AEDECOD ("&trtLabel" %do i=1 %to &n1; ("&&trtvar&i" (%do j=1 %to &n2; sev&i._&j %end;)) %end;)/* (%do i=1 %to &n1; ( %str("&&trtvar&i") (Mild Moderate Severe)) %end;)*/;
               %if %eval(&n1+&n2) > 5 %then %do;
                  DEFINE AEBODSYS / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
                  DEFINE AEDECOD / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
               %end;
               %else %do;              
                  DEFINE AEBODSYS / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
                  DEFINE AEDECOD / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
               %end;
               **THE NO ZERO COMMAND IS USED SINCE ALL PRODUCT CATEGORIES WILL NOT BE
               REPRESENTED FOR EACH PRODUCT LINE IN THE TABLE;
               %do i=1 %to &n1;
                  *  DEFINE %str(&&trtvar&i) / display "&&trtvar&i" width=20 ;
                  *  define TRT01P /across;
                  %do j=1 %to &n2;
                     %if %eval(&n1+&n2) > 5 %then %do;
                        DEFINE sev&i._&j  /display "&&sevvar&j" width=4 style(column)=[cellwidth=6%];
                     %end;
                     %else %do;
                        DEFINE sev&i._&j  /display "&&sevvar&j" width=20  ;
                     %end;
                  %end;
               %end;
               title1 height=17pt "&trAdvEvCtBySevLev";
            RUN;  
         %end;    
         * get the proportion and occurrence report for trt;
         %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE and &T_trtNameP_flag ^= 0 %then %do;
            proc report data = stattable_subt LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAETrtPropReport";

               column AEDDtermVar_pdf  ("&trtLabel" %do i=1 %to &nv_trt; ("%nrbquote(&&trtNameLabel&i)" GT1_trtName&i) %end;("&trTotal" Count_trt));
               define AEDDtermVar_pdf / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
            
               %do i=1 %to &nv_trt;
                define GT1_trtName&i / display "&trCount (%)"  WIDTH=20  ;
           *    define Pt1_&&trtNameV&i / display "&trPercent (%)"  WIDTH=20  ;
               %end;
               define Count_trt / display "&trCount (%)"  WIDTH=20  ;

               title1 height=17pt "&trAEEvOccAndPropRepforTrt";
               %if &PropThresh>0 %then %do;
                  %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                  title2 height=17pt "&trRepEvWithALOPTOcc";
               %end;
               %else %do;
                  title2 height=17pt "&trReportingAllEvents";
               %end;
               %if %upcase(&AESerOnly)=YES %then %do;
                  %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTIncSAEOforSFTC";
               %end;
               %else %do;
                  %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTforSFTC";
               %end;
            run;
         %end;

         %if &sexflag %then %do;
            proc report data = stattable_subS LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAESexPropReport";
               column AEDDtermVar_pdf  ("&trSex" %do i=1 %to &nv_sex; ("&&sexNameLabel&i" GT1_sexName&i) %end;("&trTotal" Count_sex ));
               define AEDDtermVar_pdf / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
               
               %do i=1 %to &nv_sex;
                define GT1_sexName&i / display "&trCount (%)"  WIDTH=20  ;
          *     define Pt1_&&sexNameV&i / display "&trPercent (%)"  WIDTH=20  ;
               %end;
               define Count_sex / display "&trCount (%)"  WIDTH=20  ;   
   
               title1 height=17pt "&trAEEvOccAndPropRepforSex";
               %if &PropThresh>0 %then %do;
                  %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                  title2 height=17pt "&trRepEvWithALOPTOcc";
               %end;
               %else %do;
                  title2 height=17pt "&trReportingAllEvents";
               %end;
               %if %upcase(&AESerOnly)=YES %then %do;
                  %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTIncSAEOforSFTC";
               %end;
               %else %do;
                  %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTforSFTC";
               %end;
            run;
         %end;
         %if &raceflag %then %do;
            proc report data = stattable_subR LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAERacePropReport";
               column AEDDtermVar_pdf  ("&trRace" %do i=1 %to &nv_race; ("&&raceNameLabel&i" GT1_raceName&i ) %end;("&trTotal" Count_Race ));
               define AEDDtermVar_pdf / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
                  
               %do i=1 %to &nv_race;
                define GT1_raceName&i / display "&trCount (%)"  WIDTH=20  ;
           *    define Pt1_&&raceNameV&i / display "&trPercent (%)"  WIDTH=20  ;
               %end;
               define Count_Race / display "&trCount (%)"  WIDTH=20  ;

               title1 height=17pt "&trAEEvOccAndPropRepforRace";
               %if &PropThresh>0 %then %do;
                  %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                  title2 height=17pt "&trRepEvWithALOPTOcc";
               %end;
               %else %do;
                  title2 height=17pt "&trReportingAllEvents";
               %end;
               %if %upcase(&AESerOnly)=YES %then %do;
                  %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTIncSAEOforSFTC";
               %end;
               %else %do;
                  %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                  title3 height=12pt "&trTforSFTC";
               %end;
            run; 
         %end;

         ods pdf close;
         %if &syserr > 4 %then %goto exit;
      %end;
         
      %if &Output_HTML = Yes %then %do;
         %PrepHTML(
            AEDistributionReport_Body, 
            AEDistributionReport_Frame, 
            AEDistributionReport_Contents, 
            &OutName._body.html, 
            &OutName._frame.html, 
            &OutName._contents.html
         );       
         ods html 
            body = "&AEDistributionReport_Body" (url="&bodyfile")
               frame = "&AEDistributionReport_Frame"  (url="&framefile")
               contents = "&AEDistributionReport_Contents" (url="&contentsfile")
            ;
            
            * calculate a count table by severity or by casualty;
            %if &AESerCount = Yes %then %do;
               PROC REPORT DATA=countserTotal ;
                  COLUMN AEBODSYS AEDECOD ("&trtLabel" %do i=1 %to &n1; ("&&trtvar&i" (%do j=1 %to &n2; sev&i._&j %end;)) %end;)/* (%do i=1 %to &n1; ( %str("&&trtvar&i") (Mild Moderate Severe)) %end;)*/;
                  %if %eval(&n1+&n2) > 5 %then %do;
                     DEFINE AEBODSYS / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
                     DEFINE AEDECOD / display FORMAT= $80. WIDTH=15 SPACING=2 LEFT " " order=data; 
                  %end;
                  %else %do;
                     DEFINE AEBODSYS / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
                     DEFINE AEDECOD / display FORMAT= $80. WIDTH=80 SPACING=2 LEFT " " order=data; 
                  %end;
                  **THE NO ZERO COMMAND IS USED SINCE ALL PRODUCT CATEGORIES WILL NOT BE
                  REPRESENTED FOR EACH PRODUCT LINE IN THE TABLE;
                  %do i=1 %to &n1;
                     *  DEFINE %str(&&trtvar&i) / display "&&trtvar&i" width=20 ;
                     *  define TRT01P /across;
                     %do j=1 %to &n2;
                        %if %eval(&n1+&n2) > 5 %then %do;
                           DEFINE sev&i._&j  /display "&&sevvar&j" width=4 style(column)=[cellwidth=6%];
                        %end;
                        %else %do;
                           DEFINE sev&i._&j  /display "&&sevvar&j" width=20  ;
                        %end;
                     %end;
                  %end;
                  title1 height=17pt "&trAdvEvCtBySevLev";
               RUN;  
            %end; 
            %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE and &T_trtNameP_flag ^= 0 %then %do;
               proc report data = stattable_subt LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAETrtPropReport";
                  column AEDDtermVar_text  ("&trtLabel" %do i=1 %to &nv_trt; ("&&trtNameLabel&i" GT1_trtName&i) %end;("&trTotal" Count_trt));
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
            
                  %do i=1 %to &nv_trt;
                   define GT1_trtName&i / display "&trCount (%)"  WIDTH=20  ;
               *      define Pt1_&&trtNameV&i / display "&trPercent"  WIDTH=20  ;
                  %end;
                  define Count_trt / display "&trCount (%)"  WIDTH=20  ;

                  title1 height=17pt "&trAEEvOccAndPropRepforTrt";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 height=17pt "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 height=17pt "&trReportingAllEvents";
                  %end;    
                  %if %upcase(&AESerOnly)=YES %then %do;
                     %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTIncSAEOforSFTC";
                  %end;
                  %else %do;
                     %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTforSFTC";
                  %end;
               run;
            %end;
            %if &sexflag %then %do;
               proc report data = stattable_subS LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAESexPropReport";
                  column AEDDtermVar_text  ("&trSex" %do i=1 %to &nv_sex; ("&&sexNameLabel&i" GT1_sexName&i) %end;("&trTotal" Count_sex));
               
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
                  %do i=1 %to &nv_sex;
                   define GT1_sexName&i / display "&trCount (%)"  WIDTH=20  ;
                *     define Pt1_&&sexNameV&i / display "&trPercent"  WIDTH=20  ;
                  %end;
                  define Count_sex / display "&trCount (%)"  WIDTH=20  ;

                  title1 height=17pt "&trAEEvOccAndPropRepforSex";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 height=17pt "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 height=17pt "&trReportingAllEvents";
                  %end;
                  %if %upcase(&AESerOnly)=YES %then %do;
                     %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTIncSAEOforSFTC";
                  %end;
                  %else %do;
                     %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTforSFTC";
                  %end;
               run;
            %end;

            %if &raceflag %then %do;
               proc report data = stattable_subR LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAERacePropReport";
                  column AEDDtermVar_text  ("&trRace" %do i=1 %to &nv_race; ("&&raceNameLabel&i" GT1_raceName&i) %end;("&trTotal" Count_Race));
                  
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
                  %do i=1 %to &nv_race;
                   define GT1_raceName&i / display "&trCount (%)"  WIDTH=20  ;
               *      define Pt1_&&raceNameV&i / display "&trPercent (%)"  WIDTH=20  ;
                  %end;
                  define Count_Race / display "&trCount (%)"  WIDTH=20  ;
   
                  title1 height=17pt "&trAEEvOccAndPropRepforRace";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 height=17pt "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 height=17pt "&trReportingAllEvents";
                  %end;
                  %if %upcase(&AESerOnly)=YES %then %do;
                     %let trTIncSAEOforSFTC = %nrbquote(%LocaleFill(&trXIncSerAEOnlyforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTIncSAEOforSFTC";
                  %end;
                  %else %do;
                     %let trTforSFTC = %nrbquote(%LocaleFill(&trXforXChosen, f1=%nrbquote(&TEAEOnlyTmp), f2=%nrbquote(&SubjectFilterTmp)));
                     title3 height=12pt "&trTforSFTC";
                  %end;
               run; 
            %end;
            ods html close;
            %if &syserr > 4 %then %goto mexit;
         %end;
         %if &Output_Text = Yes %then %do;
            %if &Server = Local %then %do;
               ods listing file="&ClientOutPath.\&OutName..lst";
            %end;
            %else %if &Server ne Local %then %do;
               ods listing file="&ServerOutPath.\&OutName..lst";
            %end;
               ods escapechar='^';
               options nobyline nodate missing = "" orientation = landscape;
         
               * get the proportion and occurrence report for trt;
            %if %length(&trtLabel) and %upcase(&TrtUse) ne NONE %then %do;
               proc report data = stattable_subt LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAETrtPropReport";
                  column AEDDtermVar_text  ("&trtLabel" %do i=1 %to &nv_trt; ("&&trtNameLabel&i" GT1_trtName&i ) %end;("&trTotal" Count_trt));
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
               
                  %do i=1 %to &nv_trt;
                   define GT1_trtName&i / display "&trCount (%)"  WIDTH=20  ;
                *     define Pt1_&&trtNameV&i / display "&trPercent (%)"  WIDTH=20  ;
                  %end;
                  define Count_trt / display "&trCount (%)"  WIDTH=20  ;
                  title1 "&trAEEvOccAndPropRepforTrt";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 "&trReportingAllEvents";
                  %end;
               run;
            %end;
            %if &sexflag %then %do;
               proc report data = stattable_subS LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAESexPropReport";
                  column AEDDtermVar_text  ("&trSex" %do i=1 %to &nv_sex; ("&&sexNameLabel&i" GT1_sexName&i) %end;("&trTotal" Count_sex));
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
                  
                  %do i=1 %to &nv_sex;
                   define GT1_sexName&i / display "&trCount (%)"  WIDTH=20  ;
            *      define Pt1_&&sexNameV&i / display "&trPercent (%)"  WIDTH=20  ;
                  %end;
                  define Count_sex / display "&trCount (%)"  WIDTH=20  ;            
                  title1 "&trAEEvOccAndPropRepforSex";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 "&trReportingAllEvents";
                  %end;
               run;
            %end;
            %if &raceflag %then %do;
               proc report data = stattable_subR LS=97  PS=54  SPLIT="/" CENTER  nowindows missing Contents="&trAERacePropReport";
                  column AEDDtermVar_text  ("&trRace" %do i=1 %to &nv_race; ("&&raceNameLabel&i" GT1_raceName&i) %end;("&trTotal" Count_Race));
                  define AEDDtermVar_text / display FORMAT= $80. WIDTH=70 SPACING=2 LEFT " " order=data; 
                  
                  %do i=1 %to &nv_race;
                   define GT1_raceName&i / display "&trCount (%)"  WIDTH=20  ;
               *      define Pt1_&&raceNameV&i / display "&trPercent (%)"  WIDTH=20  ;
                  %end;
                  define Count_Race / display "&trCount (%)"  WIDTH=20  ;  
                  title1 "&trAEEvOccAndPropRepforRace";
                  %if &PropThresh>0 %then %do;
                     %let trRepEvWithALOPTOcc = %nrbquote(%LocaleFill(&trReportingEvWithALOvXOcc, &PropThresh));
                     title2 "&trRepEvWithALOPTOcc";
                  %end;
                  %else %do;
                     title2 "&trReportingAllEvents";
                  %end;
               run; 
            %end;
            ods listing close;
         %end;
         %else %if &Output_Text = No %then %do;
            ods listing close;
         %end;
         %if &Output_Text = Yes %then %do;
            ods listing close;
         %end; 
   %if &exiterror=1 %then %goto exit;

   * create JSL file;
   %PrepOutFile(AdverseEventsReportScript,AdverseEventsReportScriptName,&StudyName &ProcessName Script.jsl);
   %global _mainjslfile_ FullAPName;
   %let _mainjslfile_ = &AdverseEventsReportScript;
   %let FullAPName = Adverse Events Report;
   %if &Server^=Local %then %do;
      %let FullOutName = &ClientOutPath.&OutName;  
      %let OutPath2 = &ClientOutPath;
   %end;
   %else %do;
      %let OutPath2 = &OutPath;
   %end;
   
   %let trAEReport = %nrbquote(%LocaleGet(AdverseEventSummaryReport_Text));
   %let trOpen = %nrbquote(%LocaleGet(Open_Text));

   filename jslfile "&_mainjslfile_";
   data _null_;
      file jslfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put '//!';
      put "Names Default to Here(1);";
      put "objRefNS = newnamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";        
      put "objRefNS:DataList = {};";
      put "objRefNS:TabButtonList = {};";
      put "objRefNS:TabCodeList = {};";
      put "objRefNS:TabAfterCodeList = {};";
      put "objRefNS:TabOpenList = {};";
      put "objRefNS:CreateReportList = {};";  
      put "objRefNS:ShowDataTables = 0;";
      put "objRefNS:ShowCreateReport = 0;";
      put "objRefNS:ShowNotes = 0;";   
      put "objRefNS:fbox=Function({},{default local},";
      put "   Vlistbox(";
      %if %length(&TEAEText) %then %do;
         put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)), Text Box(""""),";
      %end;
      %let _fCount_ = 0;
      %if &Output_RTF = Yes %then %do;
         %let _fCount_ = %eval(&_fCount_+1);
         %let _fList_ = &OutName..rtf;
         put "      objRefNS:Button1=ButtonBox(""&trOpen &trAEReport..rtf"",";
         put "          Open(""&ClientOutPath.&OutName..rtf"");";
         put "      ),";
         put "      objRefNS:Button1 << Underline Style(1),";   
      %end;
      %if &Output_PDF = Yes %then %do;
         %let _fCount_ = %eval(&_fCount_+1);
         %if &_fCount_ = 1 %then %do;
            %let _fList_ = &OutName..pdf;
         %end;
         %else %do;
            %let _fList_ = &_fList_,&OutName..pdf;
         %end;
         put "      objRefNS:Button2=ButtonBox(""&trOpen &trAEReport..pdf"",";
         put "          Open(""&ClientOutPath.&OutName..pdf"");";
         put "      ),";
         put "      objRefNS:Button2 << Underline Style(1),";   
      %end;
      %if &Output_HTML = Yes %then %do;
         %let _fCount_ = %eval(&_fCount_+1);
         %if &_fCount_ = 1 %then %do;
            %let _fList_ = "&bodyfile,&framefile,&contentsfile";
         %end;
         %else %do;
            %let _fList_ = &_fList_,&bodyfile,&framefile,&contentsfile;
         %end;
         put "      objRefNS:Button2=ButtonBox(""&trOpen &trAEReport..html"",";
         put "          Web(""&ClientOutPath.&framefile"");";
         put "      ),";
         put "      objRefNS:Button2 << Underline Style(1),";   
      %end;
      put "   )";
      put ");";   
      %if &_fCount_ > 1 %then %do;
         %let _fList_ = "&_fList_";
      %end;
      put "InsertInto(objRefNS:DataList,{{""  ""}});";
      put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""AdverseEventSummaryReport_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText}});";
      put "InsertInto(objRefNS:TabOpenList,1);";    
   run;
   
   %PackageInsertFile(jsl code from &ProcessName, &OutPath, &AdverseEventsReportScriptName,&_fList_,
      text/plain, yes);

   %exit:;
   %ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AdverseEventsReport;

* run the macro;
%AdverseEventsReport;


