/*
JMP Clinical Analytical Process

Name: SEND subject profile Report
Support: saslii, SAS Institute Inc.
Description: 
	This process compares distributions of demographic variables across treatment arms via a one-way 
	ANOVA or contingency analysis.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtVar TrtUse IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL DesignByVars SubjectFilter WhereStmt OutPath ServerOutPath WhereForReport
AnimalProfile_OutData MacroPath AnimalProfileScript AnimalProfileScriptName MergeSuppDom Output_PDF Output_RTF AnimalProfile_OutName runAsDrilldown;

*ProcessBody;
%let WhereForReport=&WhereStmt;
*options mprint mlogic symbolgen;

%macro AnimalProfile;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$INCD: {"MA","MI","BW","TS","EX","OM"}$
$CMP: RSYS.Required Analysis Variables$ $UE: BW.BWSTRESN$ 
$CMP: RSYS.Required Analysis Variables$ $UE: BW.BWTEST$ 
$CMP: RSYS.Required Analysis Variables$ $UE: BW.BWSTRESU$ 
$CMP: RSYS.Required Analysis Variables$ $UE: TS.TSPARMCD$
$CMP: RSYS.Required Analysis Variables$ $UE: TS.TSPARM$
$CMP: RSYS.Required Analysis Variables$ $UE: TS.TSVAL$
$CMP: RSYS.Required Analysis Variables$ $UE: EX.EXDOSE$
$CMP: RSYS.Required Analysis Variables$ $UE: EX.EXDOSU$
$CMP: RSYS.Required Analysis Variables$ $UE: OM.OMSTRESC$
$CMP: RSYS.Required Analysis Variables$ $UE: OM.OMSTRESU$
$CMP: RSYS.Required Analysis Variables$ $UE: OM.OMSPEC$
$CMP: RSYS.Required Analysis Variables$ $UE: MA.MASTRESC$ 
$CMP: RSYS.Required Analysis Variables$ $UE: MA.MASPEC$
$CMP: RSYS.Required Analysis Variables$ $UE: MI.MISTRESC$ 
$CMP: RSYS.Required Analysis Variables$ $UE: MI.MISPEC$
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = AnimalProfile; 

%include "&MacroPath./UtilityMacros.sas" / nosource;

%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%if &Output_PDF=No and &Output_RTF=No %then %do;
   %put [MS_S_0010];
   %put ERROR: At least one output format must be selected.;
   %let exiterror = 1;
   %goto exit;
%end;

*merge DS domain;
%LOADDATA(domlist=DS);
%LOADDATA(domlist=DM);

proc sort data=adsl;
	by USUBJID;
run;

proc sort data=ds;
	by USUBJID;
run;
data subject;
	merge adsl ds;
	by USUBJID;
run;  

* merge BW domain to get terminal body weight information which is the last clinical observation;
%LOADDATA(domlist=BW);
proc sort data=subject;
	by USUBJID;
run; 
data BW;
	length BWSTRESU $50;
	set BW;
	BWSTRESU=trim(left(BWTEST))||":    "||trim(left(BWSTRESN))||" "||BWSTRESU;
	if BWTEST="Terminal Body Weight";
run;
proc sort data=BW (keep=USUBJID BWTESTCD BWTEST BWSTRESC BWSTRESN BWSTRESU);
	by USUBJID;
run;
data subject;
	merge subject BW;
	by USUBJID;
run;

* get study type and route information from TS domain;
* initialize parameters;
%let SSTYP_Label=;
%let ROUTE_Label=;
%let TITLE_Label=;
%let SSTYP_val=;
%let ROUTE_val=;
%let TITLE_val=;
%let SPECIES_val=;
%let STRAIN_val=;

data ts;
	set &in_ts;
run;
data _null_;
	set ts;
	if TSPARMCD="SSTYP" then do;
		call symput ("SSTYP_Label",trim(left(TSPARM)));
		call symput ("SSTYP_val",trim(left(TSVAL)));
	end;
	if TSPARMCD="ROUTE" then do;
		call symput ("ROUTE_Label",trim(left(TSPARM)));
		call symput ("ROUTE_val",trim(left(TSVAL)));
	end;	
	if TSPARMCD="TITLE" then do;
		call symput ("TITLE_Label",trim(left(TSPARM)));
		call symput ("TITLE_val",trim(left(TSVAL)));
	end;
	if TSPARMCD="SPECIES" then do;
		call symput ("SPECIES_Label",trim(left(TSPARM)));
		call symput ("SPECIES_val",trim(left(TSVAL)));
	end;
	if TSPARMCD="STRAIN" or TSPARMCD="SBSTRAIN" then do;
		call symput ("STRAIN_Label",trim(left(TSPARM)));
		call symput ("STRAIN_val",trim(left(TSVAL)));
	end;
run;
%put SSTYP_Label=&SSTYP_Label SSTYP_val=&SSTYP_val ROUTE_Label=&ROUTE_Label ROUTE_val=&ROUTE_val;
data subject;
	set subject;
	SSTYP="&SSTYP_val";
	ROUTE="&ROUTE_val";
	SPECIES="&SPECIES_val";
	STRAIN="&STRAIN_val";
	label SSTYP="&SSTYP_Label" ROUTE="&ROUTE_Label" SPECIES="&SPECIES_Label" STRAIN="&STRAIN_Label";
run;

* load EX domain to get dose and unit information;
%LOADDATA(domlist=EX);
proc sort data=subject;
	by USUBJID;
run; 
proc sort data=EX(keep=USUBJID EXDOSE EXDOSU EXDOSFRQ) nodupkey;
	by USUBJID;
run;
data EX;
	set EX;
	EXDOSU=trim(left(EXDOSE))||" "||EXDOSU;
run;
data subject;
	merge subject EX;
	by USUBJID;
run;

* merge OM domain to get organ measurements information;
%LOADDATA(domlist=OM);
proc sort data=subject;
	by USUBJID;
run; 
data om_dup;
	set om;
	OMSTRESC=trim(left(OMSTRESC))||trim(left(OMSTRESU));
	if OMSPEC="" | index(upcase(OMTEST), "RATIO")  then delete;
run;
proc sort data=om_dup;
	by usubjid OMSPEC ;
run;

proc transpose data = om_dup  prefix = om_ out = om_tr;
	by  USUBJID ;
	id  OMSPEC;
	var   OMSTRESC ;               
	idlabel OMSPEC;
run;
proc sort data=om_tr(drop=_NAME_ _LABEL_);
	by USUBJID;
run;
data subject;
	merge subject om_tr;
	by USUBJID;
run;
%if &exiterror %then %goto exit;

* merge MI and MA domains;
%LOADDATA(domlist=MA);
proc sort data=subject;
	by USUBJID;
run;
data ma_dup;
	set ma;
	if MASTRESC="" then delete;
run; 
proc sort data=ma_dup;
	by usubjid MASPEC MASTRESC;
run;
data ma_nodup;
	set ma_dup;
	by usubjid MASPEC MASTRESC;
	retain MASTRESCTotal;
	
	if first.MASPEC then do;
		MASTRESCTotal=trim(left(MASTRESC))||": "||trim(left(MASEV));
		MASTRESCTotal2=MASTRESCTotal;
	end;
	else do;
		MASTRESCTotal=MASTRESCTotal;
		MASTRESCTotal2=trim(left(MASTRESCTotal))||" "||trim(left(MASEV));
	end;
	drop MASTRESCTotal;
	rename MASTRESCTotal2=MASTRESCTotal;
	if last.MASPEC;
run;

proc sort data=ma_dup;
	by usubjid  MASPEC;
run;
proc transpose data = ma_nodup  prefix=MA_ out = ma_tr;
	by  USUBJID ;
	id  MASPEC;
	var   MASTRESCTotal ;  
	idlabel MASPEC;
run;
proc sort data=ma_tr (drop=_NAME_);
	by USUBJID;
run;
data subject;
	merge subject ma_tr;
	by  USUBJID ;
run;
%if &exiterror %then %goto exit;

%LOADDATA(domlist=MI);
proc sort data=subject;
	by USUBJID;
run;
data mi_dup;
	set mi;
	if MISTRESC="" then delete;
run; 
proc sort data=mi_dup;
	by usubjid MISPEC MISTRESC;
run;
data mi_nodup;
	set mi_dup;
	by usubjid MISPEC MISTRESC;
	retain MISTRESCTotal;
	
	if first.MISPEC then do;
		MISTRESCTotal=trim(left(MISTRESC))||": "||trim(left(MISEV));
		MISTRESCTotal2=MISTRESCTotal;
	end;
	else do;
		MISTRESCTotal=MISTRESCTotal;
		MISTRESCTotal2=trim(left(MISTRESCTotal))||" "||trim(left(MISEV));
	end;
	drop MISTRESCTotal;
	rename MISTRESCTotal2=MISTRESCTotal;
	if last.MISPEC;
run;

proc sort data=mi_nodup;
	by USUBJID MISPEC;
run;
proc transpose data = mi_nodup  prefix=MI_ out = mi_tr;
	by  USUBJID ;
	id  MISPEC;
	var   MISTRESCTotal ;
	idlabel MISPEC;
run;
proc sort data=mi_tr;
	by USUBJID;
run;

data subject;
	merge subject mi_tr;
	by USUBJID;
run;
%if &exiterror %then %goto exit;

/*
proc sql noprint;
	create table subject as
	select * from mi_adsl a1 left join ma a2
	on a1.USUBJID=a2.USUBJID;
quit;
*/

%put trtvar=&trtvar TrtUse=&TrtUse;
%if %length(&trtvar) %then %do;
	%let trtvar = TrtLabel;
%end;

%if ^%length(&OutName) %then %do;
	%let OutName = AnimalProfile;
%end;
	
%PrepOutDS(AnimalProfile_OutData,OutDataName,adsl,_sbj);
%PrepOutDS(AnimalProfile_OutName,OutName,AnimalProfile,_sbj);
data outlib.&OutDataName;
	set subject;
run;

%let tid = %sysfunc(open(outlib.&OutDataName));
%let nv_OutSet = %sysfunc(attrn(&tid,NVARS)); 
%let j=1;
%let MAnumber=1;
%let MInumber=1;
%do i = 1 %to &nv_OutSet;
	%let temp = %sysfunc(VARNAME(&tid,&i));
	%if %upcase(%substr(&temp,1,3))=%nrbquote(OM_) %then %do;
		%global OMname_&j OMLabel_&j;
		%let OMname_&j = &temp;
		%let OMLabel_&j = %sysfunc(VARLABEL(&tid,&i));
		%let j=%eval(&j+1);
	%end;
	%if %upcase(%substr(&temp,1,3))=%nrbquote(MA_) %then %do;
		%global MAname_&MAnumber MALabel_&MAnumber;
		%let MAname_&MAnumber = &temp;
		%let MALabel_&MAnumber = %nrbquote(%sysfunc(VARLABEL(&tid,&i)));
		%let MAnumber=%eval(&MAnumber+1);
	%end;
	%if %upcase(%substr(&temp,1,3))=%nrbquote(MI_) %then %do;
		%global MIname_&MInumber MILabel_&MInumber;
		%let MIname_&MInumber = &temp;
		%let MILabel_&MInumber = %nrbquote(%sysfunc(VARLABEL(&tid,&i)));
		%let MInumber=%eval(&MInumber+1);
	%end;
%end;
%let tid = %sysfunc(close(&tid));
%let OMtotal=%eval(&j-1);
%let MAnumber=%eval(&MAnumber-1);
%let MInumber=%eval(&MInumber-1);
%put OMtotal=&OMtotal MAnumber=&MAnumber MInumber=&MInumber ;

/*
%let MISPECLabel=MISPEC;
%CheckLabel(outlib.&OutDataName, MISPECLabel); 
%let MISTRESCLabel=MISTRESC;
%CheckLabel(outlib.&OutDataName, MISTRESCLabel); 
%let DSDECODCLabel=DSDECOD;
%CheckLabel(outlib.&OutDataName, DSDECODCLabel); 
%let MISEVLabel=MISEV;
%CheckLabel(outlib.&OutDataName, MISEVLabel); 
%let DSTERMLabel=DSTERM;
%CheckLabel(outlib.&OutDataName, DSTERMLabel); */

options nodate nonumber papersize=letter topmargin=.25in bottommargin=.25in leftmargin=.25in rightmargin=.25in;
ods escapechar="^";
title; footnote;
/* define colors used for the background */ 
%let backBlue = cx007DC3; 
%let backGray = cxA9A9A9;

ods _ALL_ close;
ods pdf file="Animalprofile.pdf" notoc;
%if &Output_RTF = Yes %then %do;
	 ods rtf file="&ClientOutPath.\&OutName..rtf" bodytitle;
%end;
%if &Output_PDF = Yes %then %do;
	 ods pdf file="&ClientOutPath.\&OutName..pdf" notoc;
%end;

%let trAnimalProfIndAnimalData = %nrbquote(%LocaleGet(AnimalProfileIndividualAnimalData_Text));
%let trAnimal = %nrbquote(%LocaleGet(Animal_Text));
%let trGroup = %nrbquote(%LocaleGet(Group_Text));
%let trSEX = %nrbquote(%LocaleGet(SEX_Text));
%let trMaterial = %nrbquote(%LocaleGet(Material_Text));
%let trDose = %nrbquote(%LocaleGet(Dose_Text));
%let trRoute = %nrbquote(%LocaleGet(Route_Text));
%let trSpecies = %nrbquote(%LocaleGet(Species_Text));
%let trStrain = %nrbquote(%LocaleGet(Strain_Text));
%let trStudyType = %nrbquote(%LocaleGet(StudyType_Text));
%let trDeathDate = %nrbquote(%LocaleGet(DeathDate_Text));
%let trRemovalReason = %nrbquote(%LocaleGet(RemovalReason_Text));
%let trStudyDayWeekofDeath = %nrbquote(%LocaleGet(StudyDayWeekofDeath_Text));
%let trGrossSignOff = %nrbquote(%LocaleGet(GrossSignOff_Text));
%let trUnknown = %nrbquote(%LocaleGet(Unknown_Text));
%let trGrossRecorder = %nrbquote(%LocaleGet(GrossRecorder_Text));
%let trGrossStatus = %nrbquote(%LocaleGet(GrossStatus_Text));
%let trComplete = %nrbquote(%LocaleGet(Complete_Text));
%let trHistoSignOff = %nrbquote(%LocaleGet(HistoSignOff_Text));
%let trHistoRecorder = %nrbquote(%LocaleGet(HistoRecorder_Text));
%let trHistoStatus = %nrbquote(%LocaleGet(HistoStatus_Text));
%let trLastClinicalObservation = %nrbquote(%LocaleGet(LastClinicalObservation_Text));
%let trOrganWeights = %nrbquote(%LocaleGet(OrganWeights_Text));
%let trGrossPathologyObsCorr = %nrbquote(%LocaleGet(GrossPathologyObservationCorrelation_Text));
%let trGrossPathTheFollTissHadNVL = %nrbquote(%LocaleGet(GrossPathologyThefollowingTissueshadNoVisibleLesions_Text));
%let trHistoPathologyObsCorr = %nrbquote(%LocaleGet(HistoPathologyObservationCorrelation_Text));
%let trHistoPathTheFollTissWereWNL = %nrbquote(%LocaleGet(HistoPathologyThefollowingTissueswereWithinNormLim_Text));
%let trNormal = %nrbquote(%LocaleGet(Normal_Text));
%let utrNormal = %nrbquote(%kupcase(%nrbquote(&trNormal)));

data _null_;
	set outlib.&OutDataName end=done;
	if _n_=1 then do;
		dcl odsout dak();
	end;
*	dak.layout_absolute();
	dak.format_text(data: "&trAnimalProfIndAnimalData",style_attr: "font_size=12pt font_weight=bold width=100% just=c background=&backGray");
	dak.format_text(data: "");
	dak.table_start(style_attr:"frame=HSIDES rules=none width=7in"); 
	dak.row_start(); 
	dak.format_cell(data: "&trAnimal" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: USUBJID, just: "left"); 
	dak.format_cell(data: "&trGroup" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: SETCD, just: "left"); 
	dak.format_cell(data: "&trSEX" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: SEX, just: "left"); 
	dak.row_end(); 
	dak.row_start();
	dak.format_cell(data: "&trMaterial" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: USUBJID, just: "left"); 
	dak.format_cell(data: "&trDose" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: EXDOSU , just: "left"); 
	dak.format_cell(data: "&trRoute" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: ROUTE, just: "left"); 
	dak.row_end(); 
	dak.row_start();
	dak.format_cell(data: "&trSpecies" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: Species, just: "left");
	dak.format_cell(data: "&trStrain" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: STRAIN, just: "left"); 
	dak.format_cell(data: "&trStudyType" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: SSTYP, just: "left");  
	dak.row_end();
	dak.row_start();
	dak.format_cell(data: "&trDeathDate" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: DSSTDTC, just: "left"); 
	dak.format_cell(data: "&trRemovalReason" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: DSDECOD , just: "left"); 
	dak.format_cell(data: "&trStudyDayWeekofDeath" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: VISITDY, just: "left"); 
	dak.row_end(); 
	dak.row_start();
	dak.format_cell(data: "&trGrossSignOff" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "&trUnknown", just: "left"); 
	dak.format_cell(data: "&trGrossRecorder" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "PQPATHOL" , just: "left"); 
	dak.format_cell(data: "&trGrossStatus" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "&trComplete", just: "left"); 
	dak.row_end(); 
	dak.row_start();
	dak.format_cell(data: "&trHistoSignOff" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "&trUnknown", just: "left"); 
	dak.format_cell(data: "&trHistoRecorder" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "PQPATHOL" , just: "left"); 
	dak.format_cell(data: "&trHistoStatus" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "&trComplete", just: "left"); 
	dak.row_end(); 
	dak.table_end();

	*start with bodyweight table;
	dak.table_start(style_attr:"frame=void rules=none width=7in"); 
	dak.row_start(); 
	dak.format_cell(data: "&trLastClinicalObservation" || ":", just: "left", style_attr: "font_weight=bold", width:"5in");
/*	dak.format_cell(data: "", just: "left",width:"0.5in");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");*/
	dak.row_end(); 
	dak.row_start();
*	dak.format_cell(data: "Terminal BodyWeight" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: BWSTRESU, just: "left"); 
/*	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	dak.format_cell(data: " " , just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");*/
	dak.row_end(); 
	dak.table_end();
	
	*start with organ table;
	dak.table_start(style_attr:"frame=below rules=none width=7in");
	dak.row_start();
	dak.format_cell(data: "&trOrganWeights" || ":", just: "left", style_attr: "font_weight=bold");
	dak.format_cell(data: "");
	%do i= 1 %to &OMtotal;
		if &&OMname_&i ^="" then do;
			dak.format_cell(data: "&&OMLabel_&i"|| ":", just: "left"); 
			dak.format_cell(data: &&OMname_&i, just: "left");
		end;
		if mod(&i,3)=0 then do;
			dak.row_end(); 
			dak.row_start();
		*	dak.format_cell(data: &&OMname_&i, just: "left");
		end;
	%end;
	dak.row_end();
	dak.table_end();
	
	* start with gross pathology table;
	dak.table_start(style_attr:"frame=void rules=none width=7in");
	dak.row_start();
	dak.format_cell(data: "&trGrossPathologyObsCorr" || ":", just: "left", style_attr: "font_weight=bold",column_span:2);
	dak.row_end();
	
	%do j= 1 %to &MAnumber;
		if &&MAname_&j ^="" & ^index(upcase(&&MAname_&j),"NORMAL") & ^kindex(kupcase(&&MAname_&j),"&utrNormal") then do;
			dak.row_start();
			dak.format_cell(data: "&&MALabel_&j"|| ":", just: "left", width:"1.75in"); 
			dak.format_cell(data: &&MAname_&j, just: "left", width:"5in");
			dak.row_end();
		end;
	/*	if mod(&j,10)=0 then do;
			dak.row_end(); 
			dak.row_start();
		*	dak.format_cell(data: &&OMname_&j, just: "left");
		end;*/
	%end;
	dak.table_end();
	
	dak.table_start(style_attr:"frame=void rules=none width=7in");
	dak.row_start();
	dak.format_cell(data: "&trGrossPathTheFollTissHadNVL" || ":", just: "left", style_attr: "font_weight=bold", column_span:2);
	dak.format_cell(data: "");
	dak.format_cell(data: "");
	dak.format_cell(data: "");
	dak.row_end();	
	dak.row_start();
	%do jj= 1 %to &MAnumber;
		if index(upcase(&&MAname_&jj),"NORMAL") | kindex(kupcase(&&MAname_&jj),"&utrNormal") then do;
		*	dak.row_start();
			dak.format_cell(data: "&&MALabel_&jj"|| ":", just: "left"); 
		*	dak.format_cell(data: &&MAname_&jj, just: "left", width:"5in");
		*	dak.row_end();
		end;
		if mod(&jj,4)=0 then do;
			dak.row_end(); 
			dak.row_start();
		*	dak.format_cell(data: "&&MALabel_&jj"|| ":", just: "left"); 
		end;
	%end;
	dak.row_end();
	dak.table_end();
	
	*start with table Histo Pathology;
	dak.table_start(style_attr:"frame=void rules=none width=7in");
	dak.row_start();
	dak.format_cell(data: "&trHistoPathologyObsCorr" || ":", just: "left", style_attr: "font_weight=bold", column_span:2);
	dak.row_end();
	%do s= 1 %to &MInumber;
		if &&MIname_&s ^="" & ^index(upcase(&&MIname_&s),"NORMAL") & ^kindex(kupcase(&&MIname_&s),"&utrNormal") then do;
			dak.row_start();
			dak.format_cell(data: "&&MILabel_&s"|| ":", just: "left", width:"1.75in"); 
			dak.format_cell(data: &&MIname_&s, just: "left", width:"5in");
			dak.row_end();
		end;
	%end;
	dak.table_end();
	
	dak.table_start(style_attr:"frame=void rules=none width=7in");
	dak.row_start();
	dak.format_cell(data: "&trHistoPathTheFollTissWereWNL" || ":", just: "left", style_attr: "font_weight=bold", column_span:2);
	dak.format_cell(data: "");
	dak.format_cell(data: "");
	dak.format_cell(data: "");
	dak.row_end();
	dak.row_start();
	%do ss= 1 %to &MInumber;
		if index(upcase(&&MIname_&ss),"NORMAL") | kindex(kupcase(&&MIname_&ss),"&utrNormal") then do;
		*	dak.row_start();
			dak.format_cell(data: "&&MILabel_&ss"|| ":", just: "left"); 
		*	dak.format_cell(data: &&MIname_&ss, just: "left", width:"5in");
		*	dak.row_end();
		end;
		if mod(&ss,4)=0 then do;
			dak.row_end(); 
			dak.row_start();
		*	dak.format_cell(data: "&&MILabel_&ss"|| ":", just: "left"); 
		end;
	%end;
	dak.row_end();
	dak.table_end();
*	dak.layout_end();
	/* add a page break for each obs but the last */ 
	if not done then do; 
		dak.page(); 
	end; 
run;
ods _ALL_ close;

* create JSL file;
%PrepOutFile(AnimalProfileScript,AnimalProfileScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &AnimalProfileScript;
%let FullAPName=%str(Animal Profile);
%let FullOutName = &ClientOutPath.&OutName;
%let trAnimalProfile = %nrbquote(%LocaleGet(AnimalProfile_Text));
%let trOpen = %nrbquote(%LocaleGet(Open_Text));

* put JSL statements;
filename jslfile "&_mainjslfile_";
data _null_;
   file jslfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
    put "Names Default to Here(1);";
    put "objRefNS = newnamespace();";
    put "jg:currentObjRefNamespaceName = objRefNS << getName;";
    put "objRefNS:DataList = {};";
    put "objRefNS:TabButtonList = {};";
    put "objRefNS:TabCodeList = {};";
    put "objRefNS:TabAfterCodeList = {};";
    put "objRefNS:TabOpenList = {};";
    put "objRefNS:CreateReportList = {};";
    put "objRefNS:ShowDataTables = 0;";
    put "objRefNS:ShowNotes = 0;";
    %if &runAsDrillDown %then %do;
        put 'open("'"&FullOutName..pdf"'", JMP window);';
    %end;
    %else %do;
        put "objRefNS:fbox=Function({},{default local},";
        put "   Vlistbox(";
        put "      objRefNS:Button=ButtonBox(""&trOpen &trAnimalProfile"",";
            %let _fCount_ = 0;
            %if &Output_RTF = Yes %then %do;
                %let _fCount_ = %eval(&_fCount_+1);
                %let _fList_ = &OutName..rtf;
                put 'open("'"&FullOutName..rtf"'", JMP window);';
            %end;
            %if &Output_PDF = Yes %then %do;
                %let _fCount_ = %eval(&_fCount_+1);
                %if &_fCount_ = 1 %then %do;
                    %let _fList_ = &OutName..pdf;
                %end;
                %else %do;
                    %let _fList_ = &_fList_,&OutName..pdf;
                %end;
                put 'open("'"&FullOutName..pdf"'", JMP window);';
            %end;
            %if &_fCount_ > 1 %then %do;
                %let _fList_ = "&_fList_";
            %end;
        put "      ),";
        put "      objRefNS:Button << Underline Style(1);";
        put "   )";
        put ");";
        put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""AnimalProfile_Text"")));";
        put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
        put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
        put "InsertInto(objRefNS:TabOpenList,1);";
    %end;
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &AnimalProfileScriptName, &_fList_,text/plain, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend AnimalProfile;

* run the macro;
%AnimalProfile;