/*
JMP Clinical Analytical Process

Name: Comment Report
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process will generate a Comment Report.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName TrtUse TrtVar SubjectFilter WhereStmt Output_RTF Output_PDF 
   OutName OutPath ServerOutPath MacroPath IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL MergeSuppDom
   ProcessName CommentReportScript CommentReportScriptName BySubFl numcoval covalflg covalstr1 covalstr2 Codtc; 

*ProcessBody;

*options mprint mlogic symbolgen;

%macro CommentReport;

/*
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags"}}$
$CMP: RSYS$ $UE: CO.USUBJID and CO.COSEQ and CO.COVAL$
$CMP: CPV.{"Merge supplemental domain","Yes"}$ $UE: CO.STUDYID and CO.DOMAIN$ $NOTES: Additional requirements of domain depend on IDVAR values of SUPP file$
$CMP: OSYS$ $UE: CO.CODTC$ $NOTES: Printed if available$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = CommentReport; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%if ^%length(&OutName) %then %do;
   %let OutName = CommentReport;
%end;
%let FullOutName = ;
%PrepOutFile(FullOutName,OutName2,&OutName);
%let OutName = &OutName2;

%if &Output_RTF=No and &Output_PDF=No %then %do;
   %put [MS_S_0010];
   %put ERROR: At least one output format must be selected.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%sysfunc(findw(&domains, CO)) %then %do;
   %put [MS_S_0066];
   %put ERROR: The CO domain does not exist for this study.;
   %let exiterror = 1;
   %goto exit;
%end;

%let _ind_ = %sysfunc(open(&in_co));
%if %sysfunc(varnum(&_ind_, codtc)) = 0 %then %let Codtc = %str();
   %else %if %sysfunc(varnum(&_ind_,codtc)) > 0 %then %let Codtc = codtc;
%let _ind_ = %sysfunc(close(&_ind_));

%let numcoval = 1;
%let covalflg = 0;
%let covalstr1 = coval;
%let covalstr2 = upcase(coval);
%let _ind_ = %sysfunc(open(&in_co));
%do %while(&covalflg = 0);
   %if %sysfunc(varnum(&_ind_,coval&numcoval)) > 0 %then %do;
   %let covalstr1 = &covalstr1 coval&numcoval;
   %let covalstr2 = &covalstr2, upcase(coval&numcoval);
   %let numcoval = %eval(&numcoval+1);
   %end;
   %else %let covalflg = 1;
%end;
%let _ind_ = %sysfunc(close(&_ind_));

%LOADDATA(domlist = CO,mergesupp = 0,checkreview=0);
proc sort data = &in_co out = co(where = (coval ne ""));
   by usubjid coseq;
run;

data co;
  length covaltot $ %eval(200 * &numcoval);
  set co(keep = usubjid coseq &Codtc &covalstr1);
  covaltot = catx(" ", &covalstr2);
  drop &covalstr1;
run;

%let trNoComEntCRF = %nrbquote(%LocaleGet(NocommentswereenteredontheCRF_Text));

proc sort data = adsl;
   by usubjid;
run;

data co;
   merge adsl(in = ina) co(in = inb);
   by usubjid;
   if ina;
   if ina and not inb then do;
      covaltot = "&trNoComEntCRF";
      coseq = 1;
   end;
run;

ods listing close;
%if &Output_RTF = Yes %then %do;
   %if &Server = Local %then %do;
      ods rtf file="&ClientOutPath.\&OutName..rtf" style = statdoc;
   %end;
   %else %if &Server ne Local %then %do;
      ods rtf file = "&ServerOutPath.\&OutName..rtf" style = statdoc;
   %end;
%end;
%if &Output_PDF = Yes %then %do;
   %if &Server = Local %then %do;
      ods pdf file="&ClientOutPath.\&OutName..pdf" notoc style = statdoc;
   %end;
   %else %if &Server ne Local %then %do;
      ods pdf file="&ServerOutPath.\&OutName..pdf" notoc style = statdoc;
   %end;
%end;
%let USUBJIDLabel = USUBJID;
%CheckLabel(co,USUBJIDLabel);

ods escapechar='^';
options nobyline nodate;

%let trCommentReportforSN = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(CommentReportforX_Text)), f1=%nrbquote(&StudyName)));
%let trDateTimeOfComment = %nrbquote(%LocaleGet(DateSlashTimeofComment_Text));
%let trComment = %nrbquote(%LocaleGet(Comment_Text));

title1 "&trCommentReportforSN";
proc report nowindows missing nofs style = {frame = void rules=none} data = co;
   column usubjid &TrtName &Codtc covaltot;
   define usubjid / order "&USUBJIDLabel"; 
   define &TrtName / order "&TrtLabel";
   %if %length(&Codtc) %then %do; define &Codtc / order "&trDateTimeOfComment"; %end;
   define covaltot / left "&trComment";
   run;

%if &Output_RTF = Yes %then %do;
   ods rtf close;
%end;

%if &Output_PDF = Yes %then %do;
   ods pdf close;
%end;

* create JSL file;
%PrepOutFile(CommentReportScript,CommentReportScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &CommentReportScript;
%let FullAPName = Comment Report;
%if &Server^=Local %then %do;
   %let FullOutName = &ClientOutPath.&OutName;  
%end;

%let trOpen = %nrbquote(%LocaleGet(Open_Text));
%let trCommentReport = %nrbquote(%LocaleGet(CommentReport_Text));

filename jslfile "&_mainjslfile_";
data _null_;
   file jslfile %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put '//!';
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";        
   put "objRefNS:DataList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";  
   put "objRefNS:ShowDataTables = 0;";
   put "objRefNS:ShowCreateReport = 0;";
   put "objRefNS:ShowNotes = 0;";   
   put "objRefNS:fbox=Function({},{default local},";
   put "   Vlistbox(";   
   %let _fCount_ = 0;
   %if &Output_RTF = Yes %then %do;
      %let _fCount_ = %eval(&_fCount_+1);
      %let _fList_ = &OutName..rtf;
      put "      objRefNS:Button1=ButtonBox(""&trOpen &trCommentReport..rtf"",";
      put "          Open(""&ClientOutPath.&OutName..rtf"");";
      put "      ),";
      put "      objRefNS:Button1 << Underline Style(1),";   
   %end;
   %if &Output_PDF = Yes %then %do;
      %let _fCount_ = %eval(&_fCount_+1);
      %if &_fCount_ = 1 %then %do;
         %let _fList_ = &OutName..pdf;
      %end;
      %else %do;
         %let _fList_ = &_fList_,&OutName..pdf;
      %end;
      put "      objRefNS:Button2=ButtonBox(""&trOpen &trCommentReport..pdf"",";
      put "          Open(""&ClientOutPath.&OutName..pdf"");";
      put "      ),";
      put "      objRefNS:Button2 << Underline Style(1),";   
   %end;
   put "   )";
   put ");";   
   %if &_fCount_ > 1 %then %do;
      %let _fList_ = "&_fList_";
   %end;
   put "InsertInto(objRefNS:DataList,{{""  ""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""CommentReport_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";      
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &CommentReportScriptName,&_fList_,
   text/plain, yes);

/*
%if &Output_RTF = Yes %then %do;
   %PackageInsertFile(RTF file from &ProcessName, &ClientOutPath, &OutName..rtf, , rtf, yes);
%end;
   
%if &Output_PDF = Yes %then %do;
   %PackageInsertFile(PDF file from &ProcessName, &ClientOutPath, &OutName..pdf, , pdf, yes);
%end;
*/
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend CommentReport;
%CommentReport;

************************************;
************************************;
************************************;