/*
JMP Clinical Analytical Process

Name: ConvertToUTF8
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   saslii March5,2015
Description: 
This process defines the default settings
for JMP Clinical. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath Outfolder ServerOutPath ConvertingToUTF8Script
	OutPath ServerOutPath MacroPath _slash_ ShowResultFlag;

*ProcessBody;

%macro ConvertDatasets;

%let ProcessName=ConvertDatasets;

options nofmterr;

*options mprint mlogic symbolgen;

* create output folder under home folder;
%if &sysscp = WIN %then %do;
	%let _slash_ = \;
%end;
%else %do;
	%let _slash_ = /;
%end;

%include "&MacroPath./UtilityMacros.sas" / nosource;

%let exiterror=0;

%if %nrbquote(&SYSENCODING) ne %nrbquote(utf-8) %then %do;
	%put [MS_S_0262];
	%put ERROR: This process can run under Unicode SAS only. You will need to change the SAS configuration file in order to run this process.;
	%let exiterror = 1;
%end;

%if ^%length(&InPath) & ^%length(&Outfolder) %then %do;
	%put [MS_S_0263];
	%put ERROR: Neither the SDTM folder nor ADaM folder is specified.;
	%put At least one of these folders must be specified to run JMP Clinical processes.;
	%let exiterror = 1;
%end;
%if &exiterror %then %goto exit;

%if %quote(%upcase(&InPath))=  %quote(%upcase(&Outfolder)) %then %do;
	%put [MS_S_0264];
	%put ERROR: The Folder of Non-UTF-8 Files and Folder for UTF-8 Files after Converting cannot be the same.;
	%let exiterror = 1;
%end;

%if %length(&InPath) %then %do;
	*check if the directory exists;
	%local rc fileref;
	%let rc = %sysfunc(filename(fileref,&InPath)) ; 
	%if ^%sysfunc(fexist(&fileref))  %then %do;
		%put [MS_S_0265];
	 	%put ERROR: Folder of Non-UTF-8 files is invalid.;
	 	%let exiterror = 1;
	%end;
%end;
%if %length(&Outfolder) %then %do;
	*check if the directory exists;
	%local rc fileref;
	%let rc = %sysfunc(filename(fileref,&Outfolder)) ; 
	%if ^%sysfunc(fexist(&fileref))  %then  %do;
		%put [MS_S_0266];
		%put ERROR: Folder for UTF-8 Files after Converting specification is invalid.;
	 	%let exiterror = 1;
	%end;
%end;
%if &exiterror %then %goto exit;

%FileFilter(&InPath,CPT,_ext="sas7bdat");
%NObsVars(CPT);
%let nCPT = &nObs;
%if &nObs=0 %then %do;
	%put [MS_S_0267];
	%put ERROR: The folder specified does not have any SAS datasets. Please specify a folder with SAS datasets.;
	 %let exiterror = 1;
	 %goto exit;
%end;

libname inlib CVP "&InPath";
libname olib "&Outfolder";
libname outlib "&Outpath";

data cpt;
	set cpt;
	newname = substr(filename,1,index(filename,".")-1);
run;
data cpt;
   set cpt;
   _F_ = 'sasInFile'||trim(left(_n_));
    call symput(_F_,trim(newname));
run;
%do i =1 %to &nCPT;
	data olib.&&sasInFile&i;
		set inlib.&&sasInFile&i;
 	run ;
%end;
	


* create JSL file;
%PrepOutFile(ConvertingToUTF8Script,ConvertingToUTF8ScriptName,UTF &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ConvertingToUTF8Script;
%let FullAPName=%str(Converting To UTF8);

data _null_;
	file "&ConvertingToUTF8Script" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "//Here:FullAPName = ""&FullAPName"";";
	put "Here:StudyName = ""&StudyName"";";

	put 'sasmsgwindow=New Window(locale:get("Results_Text") || ": " || locale:get("ConvertDataSets_Text"),showmenu(0),showtoolbars(0),';
	put '   VListBox(';
	put "      TextBox("" ""),";
	put "      TextBox("" "" || locale:get(""ThefileshavebeenconvertedtoUTF8encoding_Text""));";
	put "   )";
	put ");";	

run;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &ConvertingToUTF8ScriptName, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, 1);

%mend ConvertDatasets;

* run macro;
%ConvertDatasets;
