/*
JMP Clinical Analytical Process

Name: DefineRiskThresholdDataSet
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

This process allows users to define a new risk threshold data set for RBM analyses from an existing matrix.
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global MacroPath OutName RiskThreshold_OutData OutPath RiskThresholdName ExData;

*ProcessBody;

%macro DefineRiskThresholdDataSet;

*initialization and argument checking;

%let ProcessName = DefineRiskThresholdDataSet; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./RBMMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;
%end;

%PrepOutDS(RiskThreshold_OutData,OutDataName, rbm_threshold);

libname RBM "&RBMPath";

%let trPerRandomizedSubject = %nrbquote(%LocaleGet(PerRandomizedSubject_Text));
%let trPerPatientWeek = %nrbquote(%LocaleGet(PerPatientWeek_Text));
%let trManuallyEntered = %nrbquote(%LocaleGet(ManuallyEntered_Text));
%let trEnrollment = %nrbquote(%LocaleGet(Enrollment_Text));
%let trDisposition = %nrbquote(%LocaleGet(Disposition_Text));
%let trAdverseEvents = %nrbquote(%LocaleGet(AdverseEvents_Text));
%let trSafety = %nrbquote(%LocaleGet(Safety_Text));
%let trSupplemental = %nrbquote(%LocaleGet(Supplemental_Text));
%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trLabel = %nrbquote(%LocaleGet(Label_Text));
%let trCRF = %nrbquote(%LocaleGet(CRF_Text));
%let trQuery = %nrbquote(%LocaleGet(Query_Text));
%let trAverageResp = %nrbquote(%LocaleGet(AverageXResponseTime_Text));
%let trAverageCRFResponseTime = %nrbquote(%LocaleFill(&trAverageResp, &trCRF));
%let trAverageQueryResponseTime = %nrbquote(%LocaleFill(&trAverageResp, &trQuery));
%let trOverdue = %nrbquote(%LocaleGet(Overdue_Text));
%let trOverall = %nrbquote(%LocaleGet(Overall_Text));
%let trOverallRiskIndicatorSafe = %nrbquote(%LocaleGet(OverallRiskIndicatorSafety_Text));
%let trOverallRiskIndicatorSupp = %nrbquote(%LocaleGet(OverallRiskIndicatorSupplemental_Text));

%let VarLenE = 0;
%let LabLenE = 0;
%let CatLenE = 0;
%let VarLen = 0;
%let LabLen = 0;

%global ExPath;
%let ExPath =;
%if %length(&ExData) %then %do;  
   %GetOrigFileName(ExData,OrigExData);
   %PathName(&ExData);
   %if &exiterror %then %do;
      %put [MS_S_0071];
      %put ERROR: Input dataset specification is invalid.;
      %goto exit;
   %end;
   %let ExPath = &TmpPath;
   libname ExLib "&ExPath";
   %let ExData = &TmpName;

   *** use translated data if japansese or chinese system encoding detected ***;
   %if %nrbquote(%upcase(&StudyName)) = NICARDIPINE and &ExData = rb_en %then %do;
      %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %let ExData = rb_zh;
      %else %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) %then %let ExData = rb_ja;   
   %end;

   %let _tid_ = %sysfunc(open(ExLib.&ExData));   
      %let un = %sysfunc(varnum(&_tid_,USUBJID));      
      %let sn = %sysfunc(varnum(&_tid_,SITEID));
      %let vn = %sysfunc(varnum(&_tid_,VARIABLE));
      %let dn = %sysfunc(varnum(&_tid_,RBDECOD));
      %let cn = %sysfunc(varnum(&_tid_,RBCAT));
      %let tn = %sysfunc(varnum(&_tid_,RBSTDTC));                  
   %let _tid_ = %sysfunc(close(&_tid_));      
   
   %if ^&un %then %do;
         %put ERROR: Unique Subject Identifier (USUBJID) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;
   %if ^&sn %then %do;
         %put ERROR: Study Site Identifier (SITEID) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;
   %if ^&vn %then %do;
         %put ERROR: Variable (VARIABLE) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;   
   %if ^&dn %then %do;
         %put ERROR: Standardized Term (RBDECOD) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;    
   %if ^&cn %then %do;
         %put ERROR: Category (RBCAT) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;
   %if ^&tn %then %do;
         %put ERROR: Start Date/Time (RBSTDTC) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto exit;    
   %end;   
   
   proc sql noprint;
      create table pairs as
         select distinct variable, rbdecod, rbcat  
         from ExLib.&ExData
         order by variable;
         %let numpairs = &sqlobs;   
      select distinct variable
         into :TempList separated by ' '
         from pairs
         order by variable;
         %let numvars = &sqlobs;
      select distinct rbdecod
         into :TempList separated by '~'
         from pairs
         order by rbdecod;
         %let numlabs = &sqlobs;
   quit;

   *** apply recode of categories ***;
   data pairs;
      set pairs;
      if rbcat = "&trAdverseEvents" then rbcat = "&trSafety";
      if rbcat = "&trManuallyEntered" then rbcat = "&trSupplemental";
   run;
   
   proc sql noprint;
      select distinct upcase(rbcat)
         into :CatList separated by '~'
         from pairs
         order by rbcat;
   quit;

   %let KnownCatList = %upcase(&trSafety)~%upcase(&trSupplemental)~%upcase(&trEnrollment)~%upcase(&trDisposition);

   %let i = 1;
   %let CatError = 0;
   %do %while(%length(%scan(&CatList,&i,~)));
      %let temp = %scan(&CatList,&i,~);
      %if ^%sysfunc(findw(~%upcase(&KnownCatList)~,&temp, ~)) %then %let CatError = 1;
      %let i = &i + 1;
   %end;

   %if &CatError %then %do;
      %put [MS_S_0072];
      %put ERROR: Unknown category detected in RBCAT. Category must be one of;
      %put ERROR: &trSafety, &trSupplemental, &trEnrollment or &trDisposition.;
      %let exiterror = 1;
      %goto exit;    
   %end;

   %if %sysevalf(&numvars<&numpairs) %then %do;
      %put [MS_S_0073];
      %put ERROR: &TmpName.sas7bdat has multiple RBDECOD associated with one or more levels of VARIABLE.;
      %let exiterror = 1;
   %end; 

   %if %sysevalf(&numvars<&numpairs) %then %do;
      %put [MS_S_0074];
      %put ERROR: &TmpName.sas7bdat has multiple VARIABLE associated with one or more levels of RBDECOD.;
      %let exiterror = 1;
   %end;     
   %if &exiterror %then %goto exit;
   
   *** site and subject-level variables and labels need to be distinct ***;
   proc sql noprint;
      create table subpairs as
         select distinct variable, rbdecod 
         from ExLib.&ExData
         where usubjid ne ""
         order by variable ;
      create table sitepairs as
         select distinct variable, rbdecod 
         from ExLib.&ExData
         where usubjid = ""
         order by variable;         
   quit;
      
   data overlap;
     merge subpairs(keep = variable in = ina) sitepairs(keep = variable in = inb);
     by variable;
     if ina and inb;
   run;
   
   %NObsVars(overlap);
   %if &nobs %then %do; 
      proc sql noprint;
         select distinct variable
            into :RBOverlap separated by ' '
            from overlap
            order by variable;
      quit;
   
      %put ERROR: Variables need to be distinct between subject-level indicators and site-level indicators.;
      %put ERROR: Overlap was detected for the following variables: &RBOverlap;
      %put ERROR: Modify site-level variable names in input SAS data set for risk-based monitoring to be distinct from subject-level variables;
      %let exiterror = 1;
      %goto gexit;          
   %end;

   proc sort data = subpairs;
      by rbdecod;
   run;

   proc sort data = sitepairs;
      by rbdecod;
   run;   
   
   data overlap;
     merge subpairs(keep = rbdecod in = ina) sitepairs(keep = rbdecod in = inb);
     by rbdecod;
     if ina and inb;
   run;
   %NObsVars(overlap);
   %if &nobs %then %do; 
      proc sql noprint;
         select distinct rbdecod
            into :RBOverlap separated by ' '
            from overlap
            order by rbdecod;
      quit;
   
      %put ERROR: RBDECOD needs to be distinct between subject-level indicators and site-level indicators.;
      %put ERROR: Overlap was detected for the following labels: &RBOverlap;
      %put ERROR: Modify site-level RBDECOD names in input SAS data set for risk-based monitoring to be distinct from subject-level RBDECOD;
      %let exiterror = 1;
      %goto gexit;          
   %end;
   
   proc datasets nolist;
      delete overlap subpairs sitepairs;
   quit;

   data pairs;
      length VARIABLE LABEL CATEGORY $ 200;
      set pairs(rename = (variable = var));
      VAR = UPCASE(VAR);
      VARIABLE = VAR; LABEL = RBDECOD; CATEGORY = RBCAT; OUTPUT;
      VARIABLE = CATS("AV",VAR); LABEL = catx(" ", RBDECOD, "&trPerRandomizedSubject"); CATEGORY = RBCAT; OUTPUT;
      VARIABLE = CATS("PW",VAR); LABEL = catx(" ", RBDECOD, "&trPerPatientWeek"); CATEGORY = RBCAT; OUTPUT;
      if VAR in ("QUERY") then do;
         VARIABLE = CATS("O",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("AVO",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD, "&trPerRandomizedSubject"); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("PWO",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD, "&trPerPatientWeek"); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("R",VAR); LABEL = "&trAverageQueryResponseTime"; CATEGORY = RBCAT; OUTPUT;
      end;
      if VAR in ("CRFPAGE") then do;
         VARIABLE = CATS("O",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("AVO",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD, "&trPerRandomizedSubject"); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("PWO",VAR); LABEL = catx(" ", "&trOverdue", RBDECOD, "&trPerPatientWeek"); CATEGORY = RBCAT; OUTPUT;
         VARIABLE = CATS("R",VAR); LABEL = "&trAverageCRFResponseTime"; CATEGORY = RBCAT; OUTPUT;
      end;
      drop var rbcat rbdecod;
   run;
   
   data pairs;
      set pairs;
      order = 7000 + _n_;
   run;
   
   %SHRINKCHARS(dsn = pairs, vars = VARIABLE LABEL CATEGORY);
   
   %let _tid_ = %sysfunc(open(pairs));   
      %let vn = %sysfunc(varnum(&_tid_,VARIABLE));
      %let VarLenE = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,LABEL));
      %let LabLenE = %sysfunc(varlength(&_tid_,&vn));
      %let vn = %sysfunc(varnum(&_tid_,CATEGORY));
      %let CatLenE = %sysfunc(varlength(&_tid_,&vn));
   %let _tid_ = %sysfunc(close(&_tid_));   
%end;

data _null_;
   set RBM.riskthresholds end = eof;
   where RiskThresholdName = "&RiskThresholdName";
   if eof then call symput("RiskThresholdDataSet", RiskThresholdDataSet);
run;
   
data risk;
   retain variable label Category yelper yelmag redper redmag overallwt centerflg centerval direction yelact redact comment;
   length Category $ 20;
   set RBM.&RiskThresholdDataSet;
   
   *** apply recode of categories ***;
   if upcase(variable) = "OVERALLA" then do;
      variable = "OVERALLS";
      label = "&trOverallRiskIndicatorSafe";      
   end;
   if upcase(variable) = "OVERALLM" then do;
      variable = "OVERALLX";
      label = "&trOverallRiskIndicatorSupp";      
   end;
   
   *** drop old variables ***;
   if variable in ("TOTALQUERIES","AVTOTQ","PWTOTQ","OVERDUEQUERIES","AVOVRQ","PWOVERQ","MISSPAGE","AVMISSPAGE","PWMISSPAGE","COMPDEVIATE","AVCOMPDEV","PWCOMPDEV","QUERYRESP","CRFENTRYRESP") then delete;
      
   if category ne "&trOverall" then order = _n_;
   else order = 9000 + _n_;
run;

%let _tid_ = %sysfunc(open(risk));   
   %let vn = %sysfunc(varnum(&_tid_,VARIABLE));
   %let VarLenI = %sysfunc(varlength(&_tid_,&vn));
   %let vn = %sysfunc(varnum(&_tid_,LABEL));
   %let LabLenI = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));   

/*
*** Add user-defined variables if any ***;
%if %sysfunc(fileexist(&RBMPath.user_vars.sas7bdat)) %then %do;
   proc sort data = risk out = risk_vars(keep = variable);
      by variable;
   run;

   proc sort data = RBM.user_vars out = user_vars(drop = variable_number);
      by variable;
   run;
   
   data user_vars;
      set user_vars;
      order = 5000 + _n_;
      variable = upcase(variable);
   run;

   data user_vars;
      merge user_vars(in = ina) risk_vars(in = inb);
      by variable;
      if ina and not inb;
   run;
   %NObsVars(user_vars);

   %let _tid_ = %sysfunc(open(user_vars));
   %let vn = %sysfunc(varnum(&_tid_,Variable));
   %let VarLen = %sysfunc(varlength(&_tid_,&vn));
   %let ln = %sysfunc(varnum(&_tid_,Label));
   %let LabLen = %sysfunc(varlength(&_tid_,&ln));
   %let _tid_ = %sysfunc(close(&_tid_));

   %if &nobs %then %do; 
      data user_vars(keep = newvar newlab order);
         length newvar $ %eval(&VarLen+2) newlab $ %eval(&LabLen+23);
         set user_vars;
         newvar = variable; newlab = label; order = order;
         output;
         newvar = cats("AV",variable); newlab = catx(" ",label,"&trPerRandomizedSubject"); order = order + 0.2; 
         output;
         newvar = cats("PW",variable); newlab = catx(" ",label,"&trPerPatientWeek"); order = order + 0.4; 
         output;
      run;
   
      data risk;
         length variable $ %sysfunc(max(&VarLen+2,&VarLenI)) label $ %sysfunc(max(&LabLen+23,&LabLenI));
         set risk user_vars(rename = (newvar = Variable newlab = Label) in = inb);
         if inb then Category = "&trSupplemental";
      run;
   %end;
         
   proc datasets nolist;
      delete risk_vars;
   quit;
%end;
*/

*** add variables from external data set ***;
%if %length(&ExData) %then %do;
   /*
   %if %sysfunc(fileexist(&RBMPath.user_vars.sas7bdat)) %then %do;
      data riskchk;
         length variable $ %sysfunc(max(&VarLen+2,&VarLenI));
         set risk(where = (variable in ("TOTALQUERIES","AVTOTQ","PWTOTQ","OVERDUEQUERIES","AVOVRQ","PWOVERQ","MISSPAGE","AVMISSPAGE","PWMISSPAGE", 
                                                  "COMPDEVIATE","AVCOMPDEV","PWCOMPDEV","QUERYRESP","CRFENTRYRESP"))) user_vars;
      run;

      proc datasets nolist;
         delete user_vars;
      quit;
   %end;
   %else %do;
      data riskchk;
         set risk(where = (variable in ("TOTALQUERIES","AVTOTQ","PWTOTQ","OVERDUEQUERIES","AVOVRQ","PWOVERQ","MISSPAGE","AVMISSPAGE","PWMISSPAGE", 
                                                  "COMPDEVIATE","AVCOMPDEV","PWCOMPDEV","QUERYRESP","CRFENTRYRESP")));
      run;
   %end;

   proc sort data = riskchk;
      by variable;
   run;
   
   proc sort data = pairs;
      by variable;
   run;   

   *** Check overlapping VARIABLES ***;
   data riskchk;
     merge riskchk(keep = variable in = ina) pairs(in = inb);
     by variable;
     if ina and inb;
   run;
   %NObsVars(riskchk);

   %if &nobs %then %do; 
      proc sql noprint;
         select distinct variable
            into :RBOverlap separated by ' '
            from riskchk
            order by variable;
      quit;
   
      %put [MS_S_0075];
      %put ERROR: External variables (supplied and derived) detected in selected Risk Threshold Data Set;
      %put ERROR: Variables are: &RBOverlap;
      %put ERROR: Modify variable name in input SAS data set for risk-based monitoring to be distinct considering AV PW prefixes and additional O and R prefixes for QUERY and CRFPAGE;
      %let exiterror = 1;
      %goto exit;          
   %end;
   
   proc datasets nolist;
      delete riskchk;
   quit;
   */

   proc sort data = risk;
     by variable;
   run;

   proc sort data = pairs;
     by variable;
   run;

   *** only add new variables based on VARIABLE ***;
   data newpairs;
     merge risk(keep = variable in = ina) pairs(in = inb);
     by variable;
     if inb and not ina;
   run;  
   %NObsVars(newpairs);

   %if &nobs %then %do; 
      proc sort data = newpairs;
         by label;
      run;
      
      proc sort data = risk;
         by label;
      run;
            
      *** Check overlapping LABELS ***;
      data labchk;
        length label $ %sysfunc(max(&LabLen+23,&LabLenI,&LabLenE));
        merge risk(keep = variable label in = ina) newpairs(keep = variable label in = inb);
        by label;
        if ina and inb;
      run;
      %NObsVars(labchk);

      %if &nobs %then %do; 
         proc sql noprint;
            select distinct label
               into :RBOverlap separated by ','
               from labchk
               order by label;
         quit;

         %put [MS_S_0076];
         %put ERROR: Duplicate labels detected. Labels are: &RBOverlap;
         %put ERROR: Modify label names in input SAS data set for risk-based monitoring to be distinct considering AV PW prefixes and additional O and R prefixes for QUERY and CRFPAGE;
         %let exiterror = 1;
         %goto exit;          
      %end;    
   
      proc datasets nolist;
         delete labchk;
      quit;
   
      data risk;
         length variable $ %sysfunc(max(&VarLen+2,&VarLenI,&VarLenE)) label $ %sysfunc(max(&LabLen+23,&LabLenI,&LabLenE));
         set risk newpairs;
      run;
   %end;
%end;

proc sort data = risk;
  by order;
run;
   
data &RiskThreshold_OutData;
   set risk(drop = order);
run;

proc datasets nolist;
   delete risk;
quit;

%exit:;
%put &syserr, &exiterror;

%mend DefineRiskThresholdDataSet;
%DefineRiskThresholdDataSet;

************************************;
************************************;
************************************;