/*
JMP Clinical Analytical Process

Name: Findings Report
Support: saslii, SAS Institute Inc.
Description: 
	This process compares distributions of demographic variables across treatment arms via a one-way 
	ANOVA or contingency analysis.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtVar TrtUse IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL DesignByVars SubjectFilter WhereStmt OutPath ServerOutPath WhereForReport
FindingsReport_OutData FindingsReport_OutData2 MacroPath FindingsReportScript MergeSuppDom SendDomain ;

*ProcessBody;
%let WhereForReport=&WhereStmt;
*options mprint mlogic symbolgen;

%macro FindingsIncidenceReport;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$INCD: {"MA","MI"}$
$CMP: RSYS.Required Analysis Variables$ $UE: MA.MASTRESC or MI.MISTRESC$ 
$CMP: OOUT$ $UE: DS.DSDECOD$ $NOTES: To determine disposition outcomes$	
$CMP: OOUT$ $UE: DS.DSTERM$ $NOTES: To determine disposition outcomes$
$CMP: OOUT$ $UE: MA.MASEV$ $NOTES: To determine severity level$
$CMP: OOUT$ $UE: MA.MASPEC$ $NOTES: To show Specimen Material Type$	
$CMP: OOUT$ $UE: MI.MISEV$ $NOTES: To determine severity level$
$CMP: OOUT$ $UE: MI.MISPEC$ $NOTES: To show Specimen Material Type$
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = FindingsIncidenceReport; 

%include "&MacroPath./UtilityMacros.sas" / nosource;

%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%if ^%sysfunc(findw(&domains, MA)) and ^%sysfunc(findw(&domains, MI)) %then %do;
	%put ERROR:  The MA or MI data set is required for Findings Incidence Report.;
	%let exiterror=1;
	%goto exit;
%end;

%if ^%length(&SendDomain) %then %do;
	%put ERROR: The MA or MI data set is required for findings intervention information.;
	%let exiterror=1;
	%goto exit;
%end;

%LOADDATA(domlist=&SendDomain);
%LOADDATA(domlist=DM);

%let trNormal = %nrbquote(%LocaleGet(Normal_Text)); /* Not used yet */
%let trSex = %nrbquote(%LocaleGet(Sex_Text));       /* Not used yet */
%let trTotal = %nrbquote(%LocaleGet(Total_Text));   /* Not used yet */
%let trNormal = %nrbquote(%LocaleGet(Normal_Text));
%let utrNormal = %nrbquote(%kupcase(%nrbquote(&trNormal)));
%let trNumberofAnimals = %nrbquote(%LocaleGet(NumberofAnimals_Text));
%let trMoribundSacrifice = %nrbquote(%LocaleGet(MoribundSacrifice_Text));
%let utrMoribundSacrifice = %nrbquote(%kupcase(%nrbquote(&trMoribundSacrifice)));
%let trFoundDead = %nrbquote(%LocaleGet(FoundDead_Text));
%let utrFoundDead = %nrbquote(%kupcase(%nrbquote(&trFoundDead)));
%let trNumberofCompletedAnimals = %nrbquote(%LocaleGet(NumberofCompletedAnimals_Text));
%let trTotalNumberofAnimals = %nrbquote(%LocaleGet(TotalNumberofAnimals_Text));
%let trPathIntgrpCompOfPathObsforX = %nrbquote(%LocaleGet(PathologyIntergroupComparisonofPathologyObsforX_Text));
%let trTotalNumberOfAnimalsbyTrt = %nrbquote(%LocaleGet(TotalNumberofAnimalsbyTreatment_Text));
%let trSpecimenTypePathologyComp = %nrbquote(%LocaleGet(SpecimenTypePathologyComparison_Text));

%if &SendDomain = MA %then %let NormalSTRESC=MASTRESC;
%else %let NormalSTRESC=MISTRESC;
data &SendDomain;
   length Normal $2.;
   set &SendDomain;
   if trim(left(&NormalSTRESC)) eq "NORMAL" | ktrim(kleft(&NormalSTRESC)) eq "&utrNormal" then do;
      Normal ="Y";
   end;
   else do;
      Normal ="N";
   end;
run;
proc sort data=adsl;
	by USUBJID;
run;
proc sort data=&SendDomain;
	by USUBJID;
run;

data ma_adsl;
	merge adsl &SendDomain;
	by USUBJID;
run;

%LOADDATA(domlist=DS);
proc sort data=ds;
	by USUBJID;
run;
data ma_adsl;
	merge ma_adsl ds;
	by USUBJID;
run; 

*check if MASEV has null value;
%let sevflag =0;
%let tid = %sysfunc(open(ma_adsl));
%let _mnVars_ = %sysfunc(attrn(&tid,NVARS)); 
%let _mnObs_ = %sysfunc(attrn(&tid,NOBS));
%do i = 1 %to &_mnVars_;
	%let temp = %sysfunc(VARNAME(&tid,&i));
	%if %upcase(&temp) =%upcase(&SendDomain.SEV) %then %do;
		 %do _j_ = 1 %to &_mnObs_;
		 	%let _rc_=%sysfunc(fetchobs(&tid,&_j_));
		 	%let temp2=%sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,&temp))));
			%if %length(&temp2) & %nrbquote(&temp2) ^= " " %then %do;
				%let sevflag =1;
			%end;
		%end;
	%end;
%end;
%let tid = %sysfunc(close(&tid));

%put sevflag =&sevflag;
%if %length(&trtvar) %then %do;
	%let trtvar = TrtLabel;
%end;
%PrepOutDS(FindingsReport_OutData,OutDataName,adsl,_frd);
%PrepOutDS(FindingsReport_OutData2,OutDataName2,adsl,_tcn);
proc sort data=ma_adsl;
	by USUBJID;
run;
data outlib.&OutDataName;
	set ma_adsl;
	by USUBJID;
	if first.USUBJID then do;
		count=1;
	end;
	else do;
		count=0;
	end;
run;
data dm_ds;
	merge adsl(in=ina) ds;
	if ina;
	by USUBJID;
run;
data totalcount1(keep=sex ARM DSDECOD Total);
	length Total $30;
	set dm_ds;
	Total="&trNumberofAnimals";
run;
data totalcount2(keep=sex ARM DSDECOD Total);
	length Total $30;
	set dm_ds;
	if DSDECOD="MORIBUND SACRIFICE" or DSDECOD="&utrMoribundSacrifice" or DSDECOD="FOUND DEAD" or DSDECOD="&utrFoundDead" then delete;
	Total="&trNumberofCompletedAnimals";
run;

data outlib.&OutDataName2;
	set totalcount1 totalcount2;
run;

%let MASPECLabel=&SendDomain.SPEC;
%CheckLabel(outlib.&OutDataName, MASPECLabel); 
%let MASTRESCLabel=&SendDomain.STRESC;
%CheckLabel(outlib.&OutDataName, MASTRESCLabel); 
%let DSDECODCLabel=DSDECOD;
%CheckLabel(outlib.&OutDataName, DSDECODCLabel); 
%let MASEVLabel=&SendDomain.SEV;
%CheckLabel(outlib.&OutDataName, MASEVLabel); 
%let DSTERMLabel=DSTERM;
%CheckLabel(outlib.&OutDataName, DSTERMLabel); 

* create JSL file;
%PrepOutFile(FindingsReportScript,FindingsReportScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &FindingsReportScript;
%let FullAPName=%str(Findings Incidence Report);

* put JSL statements;
data _null_;
   file "&FindingsReportScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "objRefNS = newnamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:DataList = {};";
	put "objRefNS:DataNameList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList = {};";
	put "objRefNS:frd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";

	put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
	put "if(Host Is(Windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME\Clinical\"", absolute, windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME/Clinical/"", absolute, POSIX)";
	put ");";	
	put "objRefNS:frtabu=Function({},{default local},";
	put "   If(IsEmpty(objRefNS:frd),";
 	put "      objRefNS:frd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
 	put "		);";
 	put "   If(IsEmpty(objRefNS:trn),";
  	put "      objRefNS:trn=open(""&ClientOutPath.&OutDataName2..sas7bdat"",invisible);";
 	put "		);";
 	put "		VListBox(";
   put "   Current Data Table(objRefNS:trn);";
   put "    Tabulate(";
   put "       Remove Column Label( Grouping Columns( :Sex , :Name(""&TrtLabel"")) ),";
   put "       Show Control Panel( 0 ),";
   put "          Add Table(";
   put "          Column Table( Grouping Columns( :Sex , :Name(""&TrtLabel"") ) ),";
   put "          Row Table( Grouping Columns( :Total) )";
   put "       ),";
   put "       SendToReport(";
   put "          Dispatch( {}, ""Tabulate"", OutlineBox, {Set Title( ""&trTotalNumberofAnimals"" )} )";
   put "       )";  
   put "    ),//end of tabulate;";  

	put "	 TextBox("" ""),";  
   put "   Current Data Table(objRefNS:frd);";
   put "       Tabulate(";
   put "          Remove Column Label( Grouping Columns( :Sex , :Name(""&TrtLabel"")) ),";
   put "          Show Control Panel( 0 ),";
   put "          Add Table(";
   put "             Column Table( Grouping Columns( :Sex,:Name(""&TrtLabel"") ) ),";
   %if &sevflag %then %do;
   put "             Row Table( Grouping Columns( :Name(""&MASPECLabel""),:Name(""&MASTRESCLabel""),:Name(""&MASEVLabel"")),";
   %end;
   %else %do;
   put "             Row Table( Grouping Columns( :Name(""&MASPECLabel""),:Name(""&MASTRESCLabel"")),";
   %end;
   put "          ),";
   put "       ),";   
   put "       SendToReport(";
   put "          Dispatch( {}, ""Tabulate"", OutlineBox, {Set Title( ""%nrbquote(%LocaleFill(%nrbquote(&trPathIntgrpCompOfPathObsforX), %nrbquote(&&StudyName)))"" )} )";
   put "       )";   
   put "    )//end of tabulate;";
	put "		);";
	put "	);";
	put "  //);";
	
	put "		objRefNS:CustomPanel = Expr(objRefNS:Filter= objRefNS:frd << Data Filter(<<Conditional(1),Conditional, Mode( Select( 1 ), Show( 1 ), Include( 1 ),Auto Clear(0) ),";
	put "			Add Filter(Columns( ";
	put "				:Name(""&DSDECODCLabel""),";
	put "				:Name(""&MASPECLabel""),";
	%if &sevflag %then %do;
		put "				:Name(""&MASEVLabel""),";
	%end;
    put "          :Name(""Normal""),";
	put "				:Name(""Sex""),";
	put "				:Name(""&DSTERMLabel""))";
	put "			)),";
	put "		);";	
	
	put "InsertInto(objRefNS:DataList,{{""objRefNS:trn"",""objRefNS:frd""}});";
	put "InsertInto(objRefNS:DataNameList,{{""&trTotalNumberOfAnimalsbyTrt"",""&trSpecimenTypePathologyComp""}});";
	put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""FindingsDetails_Text"")));";
	put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:frtabu()));";
	put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
	put "InsertInto(objRefNS:TabOpenList,1);";
	put "InsertInto(objRefNS:CreateReportList,{{objRefNS:frtabu}});";
	
    put "Include(JMPClinicalDir||""ProcessLibrary/AnimalProfiler.jsl"");";
    put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
	put "objRefNS:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";
    put "objRefNS:animalProfiledd = evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); animalProfileFunc = Include(""$CLINICAL_HOME\ProcessLibrary\AnimalProfiler.jsl"",<< New Context,<< ""Names Default To Here""); animalProfileFunc(this:getParent());));";
    put "objRefNS:ActionButtonList= {{Expr(locale:get(""ProfileAnimals_Text"")),Expr(locale:get(""ShowAnimals_Text"")), Expr(locale:get(""CreateAnimalFilter_Text""))}};";
    put "objRefNS:ActionCodeList= {{objRefNS:animalProfiledd(),objRefNS:spdd(),objRefNS:fpdd}};";
	put "objRefNS:ActionButtonDesc = {Expr(locale:get(""SelectAnimalsthenClick_Text""))};";
	put "objRefNS:ActionIconList ={{""/install/lifesciences/documentation/icons/clinical/ProfileAnimals.gif"", ""/install/lifesciences/documentation/icons/clinical/DataSetUser.gif"", ""/install/lifesciences/documentation/icons/clinical/AddFilter.gif""}};";
	put "objRefNS:ActionSelectionCountList ={{1,1,1}};";
   put "objRefNS:ActionRequirementFlagList ={{3,0,2}};";	
	put "//objRefNS:ActionDataTableList = {{{""objRefNS:frd""}}};";  

run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &FindingsReportScriptName, &OutDataName..sas7bdat, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend FindingsIncidenceReport;

* run the macro;
%FindingsIncidenceReport;