/*
JMP Genomics Analytical Process

Name: Manage Studies
Author: Li Li, SAS Institute Inc.
Support: Li Li, SAS Institute Inc.
History:
   sasrdw 10Nov10 initial coding

Description: 
This process defines the default settings
for JMP Genomics. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2008 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName Task InPath AInPath ServerOutPath  ManageStudyScript ManageStudyList 
    NewStudyName OutPath MacroPath NewSnap ExclTrt MigrateFlag TransportCheck KeepOnDelete 
    runCDISCParmsFlag SDTMDom ADaMDom 
    ExclTrt ExclSpid ExclSeq ExclDom ExclGrp ExclRef ExclLnk ExclLnkg
;

*options mprint mlogic symbolgen;

*options nomprint nomlogic nosymbolgen;

*ProcessBody;

%macro ManageStudies;
options nofmterr;
%let ProcessName=ManageStudies;

%let runCDISCParmsFlag=1;
%if %index(&Task,Remove) %then %do;
   %let outpath=%jg_getPath(/user/clinical/processresults,,0,0,3,%sysget(username));
%end;
%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%let notesDataset =;
%jg_getNotesDatasetName(notesDataset);
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas" / nosource;
%include "&MacroPath./CSTMacros.sas" / nosource;
%if &syserr > 4 %then %goto exit;

%*filename jslmsg "&OutPath.sasmessage.jsl";

%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;
%end;

%let trProcessName = %nrbquote(%LocaleGet(&ProcessName._Label));
%let trJMPClinical = %nrbquote(%LocaleGet(JMPClinical_Text));
%let trSnapshotNumber = %nrbquote(%LocaleGet(SnapshotNumber_Text));
%let trRefreshStudyMetadata = %nrbquote(%LocaleGet(RefreshStudyMetadata_Text));
%let trSnapshot = %nrbquote(%LocaleGet(Snapshot_Text));
%let trReport = %nrbquote(%LocaleGet(Report_Text));
%let trSRfJC = %nrbquote(%LocaleGet(StudyRemovedfromJMPClinical_Text));
%let trStudyName = %nrbquote(%LocaleGet(StudyName_Text));
%let trKeysValues = %nrbquote(%LocaleGet(KeysValues_Text));
%let trDateofNote = %nrbquote(%LocaleGet(DateofNote_Text));
%let trDeleted = %nrbquote(%LocaleGet(Deleted_Text));
%let trSDTMFolder = %nrbquote(%LocaleGet(SDTMorSENDFolder_Text));
%let trADaMFolder = %nrbquote(%LocaleGet(ADaMFolder_Text));
%let trSystemGenerated = %nrbquote(%LocaleGet(SystemGenerated_Text));

%global IntDateFormat;
%if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) or
   %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then
   %let IntDateFormat = DDMMYY10;
%else %let IntDateFormat = DATE9;

%if %length(&NewStudyName) > 32 %then %do;
   %put [MS_S_0136];
   %put ERROR: The new study name must be 32 characters or fewer.;
   %let exiterror = 1;
%end;

%put ManageStudyList=&ManageStudyList;
%let studyNameList =;
%let i = 1;
%do %while(%length(%scan(&ManageStudyList,&i,'|')));
   %let ManageStudyList&i = %scan(&ManageStudyList,&i,'|');
   %if &i=1 %then %do;
      %let StudyName= &&ManageStudyList&i;
   %end;
   %put ManageStudyList&i=&&ManageStudyList&i;
   %let i = %eval(&i+1);
%end;
%let totalStudies=%eval(&i-1);

%CHKSTUDYNAME;
%if &exiterror %then %goto exit;

%let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
%if &Server = Local %then %do;
   %let mpath = &USER_HOME;
   libname Userlib "%jg_getPath(/user/clinical/studies,,1)";
%end;
%else %do;
   %let mpath = &Metapath;    
   libname UserLib "&mpath";
%end;

%let metaData = metadata_study_source;

%if %index(&Task,Update) %then %do;
   %if %index(&Task,Snapshot) %then %let NewSnap = YES;
   %else %let NewSnap = NO;
%end;

* create JSL file;
%PrepOutFile(ManageStudyScript,ManageStudyScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ManageStudyScript;
%let FullAPName=%str(Manage Clinical Studies);

%if %index(&Task,Update) %then %do;
   %if ^%length(&InPath) and ^%length(&AInPath) and &NewSnap = YES %then %do;
      %put [MS_S_0137];
      %put ERROR: Cannot update data or compare snapshots.  No data directories provided.;
      %let exiterror = 1;
      %goto exit;
   %end;
   
   %if %length(&InPath) and %length(&AInPath) and %quote(%upcase(&AInPath)) = %quote(%upcase(&InPath)) %then %do;
      %put [MS_S_0003];
      %put ERROR: The SDTM and ADaM folder paths cannot be the same.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   *check if there is any sas dataset in the folder;
   %if %length(&InPath) %then %do;
      %FileFilter(&InPath,sasdatalist,_ext="sas7bdat");
      %FileFilter(&InPath,xptdatalist,_ext="xpt");
      %NObsVars(sasdatalist);
      %let nSASobs=%nObs;
      %NObsVars(xptdatalist);
      %if &nObs =0 & &nSASobs =0 %then %do;
         %put ERROR: There are no SAS data sets in the specified SDTM Folder.;
         %let exiterror = 1;
         %goto exit;
      %end;   
   %end;
   %if %length(&AInPath) %then %do;
      *check if there is any sas dataset in the folder;
      %FileFilter(&AInPath,sasdatalist,_ext="sas7bdat");
      %FileFilter(&AInPath,xptdatalist,_ext="xpt");
      %NObsVars(sasdatalist);
         %let nSASobs=%nObs;
         %NObsVars(xptdatalist)
      %if &nObs =0 & &nSASobs =0 %then %do;
         %put ERROR: There are no SAS data sets in the specified ADaM Folder.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;
   %if %length(&SDTMDom) %then %do;
      %let SDTMDom = %sysfunc(tranwrd(&SDTMDom, %str(,), %str( )));
      %let SDTMDom = %sysfunc(tranwrd(&SDTMDom, %str(|), %str( )));
      %let p=1;
      %do %while (%length(%scan(&SDTMDom,&p,' ')));
         %let SDTMDomCheckDM= %scan(&SDTMDom,&p,' ');
         %if ^%sysfunc(fileexist(&InPath./&SDTMDomCheckDM..sas7bdat)) & ^%sysfunc(fileexist(&InPath./&SDTMDomCheckDM..xpt)) %then %do;
            %put ERROR: The selected SDTM domains should be in the SDTM folder to update study folder.;
            %let exiterror =1;
            %goto exit;
         %end;
         %let p = %eval(&p+1);  
      %end;
   %end;
   
   %if %length(&ADaMDom) %then %do;
      %let ADaMDom = %sysfunc(tranwrd(&ADaMDom, %str(,), %str( )));
      %let ADaMDom = %sysfunc(tranwrd(&ADaMDom, %str(|), %str( )));
      %let p=1;
      %do %while (%length(%scan(&ADaMDom,&p,' ')));
         %let ADaMDomCheckDM= %scan(&ADaMDom,&p,' ');
         %if ^%sysfunc(fileexist(&AInPath./&ADaMDomCheckDM..sas7bdat)) & ^%sysfunc(fileexist(&AInPath./&ADaMDomCheckDM..xpt)) %then %do;
            %put ERROR: The selected ADAM domains should be in the ADaM folder to update study folder.;
            %let exiterror =1;
            %goto exit;
         %end;
         %let p = %eval(&p+1);  
      %end;
   %end;
   
   %if %length(&InPath) %then %CheckEndSlash(InPath);
   %if %length(&AInPath) %then %CheckEndSlash(AInPath);
   %if %length(&OutPath) %then %CheckEndSlash(OutPath);

   * check if the variable ReveiewEnabled of current study is set to 1;
   %let meta=%jg_getPath(/user/clinical/metadata,,0,0,3,&StudyName);
   libname metalib "&meta";
   %let snapfile=SNAPSHOT.sas7bdat;
   
   %if %length(&meta) %then %do;
      %if %sysfunc(fileexist(&meta.&snapfile)) %then %do;
         %global ReviewsEnabled;
         data _null_;
            set metalib.SNAPSHOT;
            call symput('ReviewsEnabled',trim(left(ReviewsEnabled)));
         run;
      %end;
      %else %do;
         * for previous added study: there is no ReviewsEnabled variable;
           %if ^%symexist(ReviewsEnabled) %then %do;
              %global ReviewsEnabled;
              %let ReviewsEnabled=1;
           %end;
      %end; 
   %end;
   %else %do;
      * for previous added study: there is no ReviewsEnabled variable;
      %if ^%symexist(ReviewsEnabled) %then %do;
         %global ReviewsEnabled;
         %let ReviewsEnabled=1;
      %end;
   %end;   
   
   %if &ReviewsEnabled = 0 %then %let NewSnap = NO;
   %else %do;
      %*ERROR check if SDTM/ADAM folders specified are not different from current study folds;

      %if %index(&Task,Snapshot) %then %do;
         data _null_;
            set Userlib.&metaData;
            if StudyName="&StudyName" then do;
               call symput ("SDTMtmpFolder",SDTMFolder);
               call symput ("ADaMtmpFolder",ADaMFolder);
            end;           
         run;
         %if %qupcase(&SDTMtmpFolder)=%qupcase(&InPath) & %qupcase(&ADaMtmpFolder)=%qupcase(&AInPath) %then %do;
            %put [MS_S_0138];
            %put ERROR: The SDTM and ADaM folders specified are the same as the current study folders.;
            %put ERROR: In order to Update Study Data and Metadata with New Snapshot, please specify a different SDTM or ADaM folder.;
            %put ERROR: Otherwise, select the option Update Study Data and Metadata.;
            %let exiterror = 1;
            %goto exit;
         %end;
      %end;
   %end;
   * check if there is a need to convert the transport files;
   %if &TransportCheck=Yes %then %do;
      * check if SDTM folder has any .cpt or .xpt files;
      %if %length(&InPath) %then %do;
         %FileFilter(&InPath,CPT,_ext="cpt");
         %NObsVars(CPT);
         %let nCPT = &nObs;
         * Transport .xpt files from the input folder;
         %FileFilter(&InPath,XPT,_ext="xpt");
         %NObsVars(XPT);
         %let nXPT = &nObs;
      
         %if &nCPT > 0 | &nXPT > 0 %then %do;
            %ConvertTransportFile(&InPath, &InPath);
         %end;
      %end;
   
      * check if ADaM folder has any .cpt or .xpt files;
      %if %length(&AInPath) %then %do;
         %FileFilter(&AInPath,CPT,_ext="cpt");
         %NObsVars(CPT);
         %let nCPT = &nObs;
         * Transport .xpt files from the input folder;
         %FileFilter(&AInPath,XPT,_ext="xpt");
         %NObsVars(XPT);
         %let nXPT = &nObs;
      
         %if &nCPT > 0 | &nXPT > 0 %then %do;
            %ConvertTransportFile(&AInPath, &AInPath);
         %end;
      %end;
   %end;
   
   * check if delta folder exits and if not set flag to create it;
   %global NeedDelta;
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta,,0,0,3,&StudyName))) ; 
   %if %sysfunc(fexist(&fileref)) %then %do;
      %let NeedDelta = NO;
   %end;
   %else %do;
      %let NeedDelta = YES;
      %let NewSnap = NO;
   %end;
   %put NeedDelta = &NeedDelta NewSnap=&NewSnap;

   %if (&MigrateFlag = N) or (&MigrateFlag = Y and &NeedDelta = NO) %then %do;
      *** Get Current Review Number ***;
      %let CurrentReview = 0;
      %let DeltaChk = ;
      filename cst "%jg_getPath(/user/clinical/metadata/base,CSTMacroVars.sas,0,0,3,&StudyName)" ;
      %if %sysfunc(fexist(cst)) %then %do;
         data _null_;
            infile cst linesize = 50 pad; 
            input text $ 1-50;
            if substr(left(text),2,18) = "let ReviewNumber =" then call symput("CurrentReview", trim(left(scan(text, 4, ' ;'))));
         run;
         data _null_;
            infile cst linesize = 50 pad; 
            input text $ 1-50;
            if substr(left(text),2,11) = "DeltaDir =" then call symput("DeltaChk", "YES");
         run;
      %end;
      %put CurrentReview = &CurrentReview;
   %end;
   %else %if &MigrateFlag = Y and &NeedDelta = YES %then %do;
      %let CurrentReview = 0;
      %let DeltaChk = ; 
   %end;
   
   %if &MigrateFlag = Y %then %let NewSnap = NO;
   /*
   %if %length(&DeltaChk) and &CurrentReview > 0 %then %do;
      %if &NewSnap = NO and %upcase(&DeleteStudy) = NO %then %do;
         %put ERROR: Study is in the review process. Any further study management can only be to update or delete the study.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;
   */

   * get previous data if this is a new snapshot;
   %if &NewSnap = YES %then %do;
      %GETPREVDATA;
   %end;

   %if %length(&InPath) %then %do;
      * check if the directory exists;
      %local rc fileref;
      %let rc = %sysfunc(filename(fileref,&InPath)) ; 
      %if ^%sysfunc(fexist(&fileref))  %then %do;
         %put [MS_S_0004];
         %put ERROR: SDTM folder &InPath specification is invalid.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;
   %if %length(&AInPath) %then %do;
      * check if the directory exists;
      %local rc fileref;
      %let rc = %sysfunc(filename(fileref,&AInPath)) ; 
      %if ^%sysfunc(fexist(&fileref))  %then  %do;
         %put [MS_S_0005];
         %put ERROR: ADaM folder &AInPath specification is invalid.;
         %let exiterror = 1;
         %goto exit;
      %end;
   %end;
   
   %GENERATEMETA(studname = &studyname);
   %if &CSTERROR %then %do;
      %put [MS_S_0139];
      %put ERROR: Creation of metadata failed.  Refer to the SAS log for details.;
      %let exiterror = 1;
      %goto exit;
   %end;

   *** manage lengths and order of metadata table variables ***;
   %let _tid_ = %sysfunc(open(Userlib.&metaData));
      %let vn = %sysfunc(varnum(&_tid_,USUBJIDlabel));
      %let vn2 = %sysfunc(varnum(&_tid_,SITEIDlabel));
      %let vn3 = %sysfunc(varnum(&_tid_,COUNTRYlabel));     
      
      %let Ulablength = %sysfunc(varlength(&_tid_,&vn));
      %if &vn2 %then %let Slablength = %sysfunc(varlength(&_tid_,&vn2));
      %else %let Slablength = 0;
      %if &vn3 %then %let Clablength = %sysfunc(varlength(&_tid_,&vn3));
      %else %let Clablength = 0;
   %let _tid_ = %sysfunc(close(&_tid_));
   
   %let Ulablength = %sysfunc(max(&Ulablength,%length(&USUBJIDlabel)));
   %let Slablength = %sysfunc(max(&Slablength,%length(&SITEIDlabel)));
   %let Clablength = %sysfunc(max(&Clablength,%length(&COUNTRYlabel)));

   proc contents noprint data = Userlib.metadata_study_source out = metachk(keep = varnum name);
   run;

   proc sql noprint;         
      select name
         into :metavarlist separated by ' '
         from metachk
         order by varnum;
   quit;
   
   proc datasets nolist;
      delete metachk;
   quit;
   
   %if &NewSnap = NO and &MigrateFlag = N %then %do;   
      *** Add note that study folders were changed ***;
      %let rn =;
      %if %sysfunc(exist(noteslib.&notesDataset)) %then %do;
         data _null_;
            set noteslib.&notesDataset end=last;
            if last then call symputx("rn",SnapshotNumber);
            %put drewrn = &rn;
         run;
      %end;

      data _null_;
         call symput("RDateC", put(datetime(), DATETIME19.));
      run;     

      data notes;
         length  Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
         Study_Name = "&StudyName";
         Report = "&trProcessName";
         User = "&sysuserid";
         Date_of_Note = datetime();
         SystemGenerated = "Y";
         %if %length(&rn) %then %do;
            SnapshotNumber = &rn;
         %end;
         %else %if %length(&ReviewNumber) %then %do;
            SnapshotNumber = &ReviewNumber;
         %end;
         %else %do;
            SnapshotNumber =.;
         %end;
         Type = "&trSnapshot";
         %if %length(&InPath) %then %do;
            Note = "&trSDTMFolder: &InPath";
            output;
         %end;
         %if %length(&AInPath) %then %do;
            Note = "&trADaMFolder: &AInPath";
            output;
         %end;
         format Date_of_Note DATETIME19.;
         label Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" SystemGenerated = "&trSystemGenerated";
      run;

      data notes;
         retain SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note;
         set %if %sysfunc(exist(noteslib.&notesDataset)) %then %do; NotesLib.&notesDataset %end; notes;
      run;

      proc sort data = notes out = NotesLib.&notesDataset;
         by SnapshotNumber Date_of_Note;
      run;

      proc datasets nolist;
         delete notes;
      quit;      
   %end;
   
   * change metadata table;
   data Userlib.&metaData;
      retain &metavarlist;
      length StudyName $32 SDTMFolder ADAMFolder $200 InitialDate LastUpdatedDate $25 CreatedBy $64 LastUpdatedBy $64 USUBJIDlabel $ &Ulablength SITEIDlabel $ &Slablength COUNTRYlabel $ &Clablength;
      set Userlib.&metaData;
      %let d=%sysfunc(today(),&IntDateFormat..);
      %let t=%sysfunc(time(),time16.);
      %let c=&d &t;
      if StudyName="&StudyName" then do;
         SDTMFolder="%str(&InPath)";
         ADaMFolder="%str(&AInPath)";
         LastUpdatedDate="&c";
         LastUpdatedBy = "%sysget(username)";
         %if %index(&Task,Snapshot) %then %do;
            SnapshotNumber=SnapshotNumber+1;
         %end;
         USUBJIDlength = &USUBJIDlength;
         SITEIDlength = &SITEIDlength;
         COUNTRYlength = &COUNTRYlength;
         USUBJIDlabel = "&USUBJIDlabel";
         SITEIDlabel = "&SITEIDlabel";
         COUNTRYlabel = "&COUNTRYlabel";         
      end;
   run;
   
   %CreateUserStudies;

   %if &NeedDelta = YES %then %do;
      %PackageInsertFile(Snapshot Duplicates PDF file from &ProcessName,&OutPath,&StudyName SnapShotDups.pdf,,pdf,yes,no);
   %end;
   %else %if &NewSnap = YES %then %do;
      %PackageInsertFile(Snapshot Duplicates PDF file from &ProcessName,&OutPath,&StudyName SnapShotDiffs.pdf,,pdf,yes,no);
   %end;

   * use jsl to handle local files so this works in local and client-server mode;
   data _null_;
      file "&ManageStudyScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "objRefNS:FullAPName = ""&FullAPName"";";
      put "objRefNS:StudyName = ""&StudyName"";";
      put "jg:CVSInit;";
      put "rc = GenerateStudyStarterFilter( ""&StudyName"" );";
      put "wait();";
      put "sasmsgwindow=New Window(locale:get(""Results_Text"") || "": "" || locale:get(""ManageClinicalStudies_Text""),   Show Menu( 0 ), Show Toolbars( 0 ),";
      put "   VListBox(";
      put "      TextBox(""\!N"" || locale:fill(locale:get(""ThedataandmetadataforstudyXhavebeenSupdated_Text""), {""&StudyName""})),";
      put "      TextBox("" ""),";
/*    %if %length(&suppmergesuccess) %then %do;
         put "      TextBox(locale:fill(locale:get(""SuppSuccess_Text""), {""&suppmergesuccess""}),<<Set Wrap(900),<< Font Color(5)),";
         put "      TextBox("" ""),";     
      %end;
      %if %length(&suppmergefail) %then %do;
         put "      TextBox(locale:fill(locale:get(""SuppFailure_Text""), {""&suppmergefail""}),<<Set Wrap(900),<< Font Color(19)),";
         put "      TextBox("" ""),";           
      %end;      */
      %if &ReviewsEnabled=0 & %index(&Task,Snapshot) %then %do;
         put "      TextBox(locale:get(""PleaseNoteThisStudyNotEnabledTrackingSnapshot_Text"")),";
      %end;
      %else %if ^&ReviewsEnabled=0  & %index(&Task,Snapshot) %then %do;
         put "      TextBox(locale:fill(locale:get(""ThecurrentreviewnumberisX_Text""), {""&CurrentReview""})),";
      %end;
      put "      TextBox("" ""),";
      %if &NeedDelta = YES & ^&ReviewsEnabled=0  %then %do;
         put "      ButtonBox(locale:get(""OpenDuplicatesReport_Text""),";
         put "          Try(open(""&ClientOutPath&StudyName SnapShotDups.pdf""),";
         put "          throw(""No duplicates exist in the study""));";
         put "      ),";   
         put "      TextBox("" ""),";
      %end;
      %else %if &NewSnap = YES %then %do;
         put "      ButtonBox(locale:get(""OpenDifferencesReport_Text""),";
         put "          Try(open(""&ClientOutPath&StudyName SnapShotDiffs.pdf""));";
         put "      ),";   
         put "      TextBox("" ""),";
      %end;
      put "      ButtonBox(locale:get(""ViewIncludedandExcludedDomains_Text""),";
      put "          Try(open(""&ClientOutPath.&StudyName DomainList.pdf""));";
      put "      ),";  
      put "   );";   
      put ");";
      put "sasmsgwindow << SetWindowIcon(jg:LSpath[1]||""\Documentation\Icons\CodeBuilder.ico"");";
   run;  
%end;
%else %if %index(&Task,Refresh) %then %do;
   %let p=1;
   %let RefdomList=;
   %let RefdomListtmp=;
   %let warningflag=0;

   %do %while(%length(%scan(&ManageStudyList,&p,'|')));
      %let StudyName= %scan(&ManageStudyList,&p,'|');
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %put StudyName=&StudyName p=&p ManageStudyList=&ManageStudyList;

      * check if the variable ReveiewEnabled of current study is set to 1;
      %let snapshotflg=0;
      %let oldmeta=%jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder); 
      libname metalib "&oldmeta";
      %let snapfile=SNAPSHOT.sas7bdat;
      %if %length(&oldmeta) %then %do;
         %if %sysfunc(fileexist(&oldmeta.&snapfile)) %then %do;
            %global ReviewsEnabled;
            data _null_;
               set metalib.SNAPSHOT;
               call symput('ReviewsEnabled',trim(left(ReviewsEnabled)));
            run;
            %let snapshotflg=1;
             data snapshot;
                set metalib.snapshot;
             run;
         %end;
         %else %do;
            * for previous added study: there is no ReviewsEnabled variable;
            %if ^%symexist(ReviewsEnabled) %then %do;
               %global ReviewsEnabled;
               %let ReviewsEnabled=1;
            %end;
         %end;
      %end;
      %else %do;
         * for previous added study: there is no ReviewsEnabled variable;
         %if ^%symexist(ReviewsEnabled) %then %do;
            %global ReviewsEnabled;
            %let ReviewsEnabled=1;
         %end;
      %end;
   
      %NObsVars(Userlib.&metaData);
     * change metadata table;
     %let SDTMSelectFlag&p=0;
     %let ADaMSelectFlag&p=0;
     %let SDTMDom=;
     %let ADaMDom=;
      data _null_;
        set Userlib.&metaData;
        %let d=%sysfunc(today(),&IntDateFormat..);
        %let t=%sysfunc(time(),time16.);
        %let c=&d &t;
        if StudyName="&StudyName" then do;
           call symput("InPath",trim(left(SDTMFolder))); 
           call symput("AInPath",trim(left(ADaMFolder))); 
           call symput("SDTMSelectFlag&p",trim(left(SDTMSelectFlag))); 
           call symput("ADaMSelectFlag&p",trim(left(ADaMSelectFlag))); 
        end;
      run;
 
     * check if folder exists;
      %if %length(&InPath) %then %do;
        %local rc fileref;
        %let rc = %sysfunc(filename(fileref,&InPath)) ; 
        %if ^%sysfunc(fexist(&fileref))  %then %do;
           %put [MS_S_0004];
           %put ERROR: SDTM folder &InPath for this study does not exist. Either the folder is deleted or renamed.;
           %let exiterror = 1;
        %end;
      %end;
     
       *check if the directory exists;
      %if %length(&AInPath) %then %do;
         %local rc fileref;
         %let rc = %sysfunc(filename(fileref,&AInPath)) ; 
         %if ^%sysfunc(fexist(&fileref))  %then  %do;
            %put [MS_S_0005];
            %put ERROR: ADaM folder &AInPath for this study does not exist. Either the folder is deleted or renamed.;
            %let exiterror = 1;
         %end;
      %end;
      
      * check if dm or adsl file available;
      %if %length(&AInPath) & %length(&InPath) %then %do;
         %if ^%sysfunc(fileexist(&AInPath./adsl.sas7bdat)) and ^%sysfunc(fileexist(&InPath./dm.sas7bdat)) %then %do;
            %if %sysfunc(fileexist(&InPath./adsl.sas7bdat)) | %sysfunc(fileexist(&AInPath./dm.sas7bdat)) %then %do;
               %put [MS_S_0006];
               %put ERROR: The adsl data set should be in the ADaM folder or the dm data set should in the SDTM folder.;
               %put Either the files are deleted or renamed.;
               %let exiterror =1;
               %goto exit;
            %end;
            %else %do;
               %put [MS_S_0007];
               %put ERROR: Neither the adsl nor dm data set exists.;
               %put Either the files are deleted or renamed.;
               %let exiterror =1;
               %goto exit;
            %end;
         %end;
      %end;
      %else %if %length(&AInPath) & ^%length(InPath) %then %do;
         %if ^%sysfunc(fileexist(&AInPath./adsl.sas7bdat)) %then %do;
            %put ERROR: The adsl data set should be in the ADaM folder. Either the file is deleted or renamed.; 
            %let exiterror =1;
            %goto exit;
         %end;
      %end;
      %else %if %length(&InPath) & ^%length(&AInPath) %then %do;
         %if ^%sysfunc(fileexist(&InPath./dm.sas7bdat)) %then %do;
            %put ERROR: The dm data set should in the SDTM folder. Either the file is deleted or renamed.;
            %let exiterror =1;
            %goto exit;
         %end;
      %end;      
      %if &exiterror %then %goto exit;  
      
      %if &&SDTMSelectFlag&p %then %do;
         %let SselPath=%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&studynameunder);
         %local rc fileref;
         %let rc = %sysfunc(filename(fileref,&SselPath)) ; 
         %if %sysfunc(fexist(&fileref)) %then %do;
            libname SDSelLib "&SselPath";
            proc sql noprint; 
               select table 
               into :SDTMDom separated by ' '
               from SDSelLib.source_tables;
            quit;
            %put WARNING: Because the specific domains &SDTMDom were selected when this study was added,only these domains will be refreshed, regardless of whether additional domains are available in input folders;
            %if ^%index(&RefdomListtmp,&studynameunder) %then %do;
               %if &p=1 %then %do;
                  %let RefdomList=&studyname;
               %end;
            /* %else %if &p= %then %do;
                  %let RefdomList=&RefdomList and &studyname;
               %end;*/
               %else %do;
                  %let RefdomList=&RefdomList, &studyname;
               %end;
               %let RefdomListtmp=&RefdomListtmp &studynameunder;
            %end;
            %let warningflag=1;
         %end;
      %end;
     %if &&ADaMSelectFlag&p %then %do;
         %let AselPath=%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&studynameunder);
         %local rc fileref;
         %let rc = %sysfunc(filename(fileref,&AselPath)) ; 
         %if %sysfunc(fexist(&fileref)) %then %do;
            libname ADSelLib "&AselPath";
            proc sql noprint; 
               select table 
               into :ADaMDom separated by ' '
               from ADSelLib.source_tables;
            quit;
            %put WARNING: Because the specific domains &SDTMDom were selected when this study was added,only these domains will be refreshed, regardless of whether additional domains are available in input folders;
            %if ^%index(&RefdomListtmp,&studynameunder) %then %do;
               %if &p=1 %then %do;
                  %let RefdomList=&studyname;
               %end;
            /* %else %if &p= %then %do;
                  %let RefdomList=&RefdomList and &studyname;
               %end;*/
               %else %do;
                  %let RefdomList=&RefdomList, &studyname;
               %end;
               %let RefdomList=&RefdomList &studyname;
               %let RefdomListtmp=&RefdomListtmp &studynameunder;
            %end;
            %let warningflag=1;
         %end;
      %end;
      * get current review number;
      %let CurrentReview = 0;
      filename cst "%jg_getPath(/user/clinical/metadata/base,CSTMacroVars.sas,0,0,3,&studynameunder)";
      %if %sysfunc(fexist(cst)) %then %do;
         data _null_;
            infile cst linesize = 50 pad; 
            input text $ 1-50;
            if substr(left(text),2,18) = "let ReviewNumber =" then call symput("CurrentReview", trim(left(scan(text, 4, ' ;'))));
         run;
      %end;
      
      %GENERATEMETA(studname = &studyname);
      
      *** manage lengths and order of metadata table variables ***;
      %let _tid_ = %sysfunc(open(Userlib.&metaData));
         %let vn = %sysfunc(varnum(&_tid_,USUBJIDlabel));
         %let vn2 = %sysfunc(varnum(&_tid_,SITEIDlabel));
         %let vn3 = %sysfunc(varnum(&_tid_,COUNTRYlabel));     

         %let Ulablength = %sysfunc(varlength(&_tid_,&vn));
         %if &vn2 %then %let Slablength = %sysfunc(varlength(&_tid_,&vn2));
         %else %let Slablength = 0;
         %if &vn3 %then %let Clablength = %sysfunc(varlength(&_tid_,&vn3));
         %else %let Clablength = 0;
      %let _tid_ = %sysfunc(close(&_tid_));

      %let Ulablength = %sysfunc(max(&Ulablength,%length(&USUBJIDlabel)));
      %let Slablength = %sysfunc(max(&Slablength,%length(&SITEIDlabel)));
      %let Clablength = %sysfunc(max(&Clablength,%length(&COUNTRYlabel)));

      proc contents noprint data = Userlib.metadata_study_source out = metachk(keep = varnum name);
      run;

      proc sql noprint;         
         select name
            into :metavarlist separated by ' '
            from metachk
            order by varnum;
      quit;

      proc datasets nolist;
         delete metachk;
      quit;
      
       * change metadata table;       
      data Userlib.&metaData;
        retain &metavarlist;      
        length StudyName $32 SDTMFolder ADAMFolder $200 InitialDate LastUpdatedDate $25 CreatedBy $64 LastUpdatedBy $64 USUBJIDlabel $ &Ulablength SITEIDlabel $ &Slablength COUNTRYlabel $ &Clablength;
        set Userlib.&metaData;
        %let d=%sysfunc(today(),&IntDateFormat..);
        %let t=%sysfunc(time(),time16.);
        %let c=&d &t;
        if StudyName="&StudyName" then do;
           LastUpdatedDate="&c";
           LastUpdatedBy = "%sysget(username)";
           USUBJIDlength = &USUBJIDlength;
           SITEIDlength = &SITEIDlength;
           COUNTRYlength = &COUNTRYlength;
           USUBJIDlabel = "&USUBJIDlabel";
           SITEIDlabel = "&SITEIDlabel";
           COUNTRYlabel = "&COUNTRYlabel";           
        end;
      run;
      
      *** Add note that study was refreshed ***;
      %let rn =;
      %if %sysfunc(exist(noteslib.&notesDataset)) %then %do;
         data _null_;
            set noteslib.&notesDataset end=last;
            if last then call symputx("rn",SnapshotNumber);
            %put drewrn = &rn;
         run;
      %end;

      data _null_;
         call symput("RDateC", put(datetime(), DATETIME19.));
      run; 
      
      %let ReviewNumber = 0;
      filename cst "%jg_getPath(/user/clinical/metadata/base,CSTMacroVars.sas,0,0,3,&studynameunder)";
      %if %sysfunc(fexist(cst)) %then %do;
         data _null_;
            infile cst linesize = 50 pad; 
            input text $ 1-50;
            if substr(left(text),2,18) = "let ReviewNumber =" then call symput("ReviewNumber", trim(left(scan(text, 4, ' ;'))));
         run;
      %end;      

      data notes;
         length  Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
         Study_Name = "&StudyName";
         Report = "&trProcessName";
         User = "&sysuserid";
         SystemGenerated = "Y";
         Date_of_Note = datetime();
         %if %length(&rn) %then %do;
            SnapshotNumber = &rn;
         %end;
         %else %if %length(&ReviewNumber) %then %do;
            SnapshotNumber = &ReviewNumber;
         %end;
         %else %do;
            SnapshotNumber =.;
         %end;
         Type = "&trSnapshot";
         Note = "&trRefreshStudyMetadata: &RDateC";
         output;
         format Date_of_Note DATETIME19.;
         label Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" SystemGenerated = "&trSystemGenerated";
      run;   

      data notes;
         retain SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note;
         set %if %sysfunc(exist(noteslib.&notesDataset)) %then %do; NotesLib.&notesDataset %end; notes;
      run;

      proc sort data = notes out = NotesLib.&notesDataset;
         by SnapshotNumber Date_of_Note;
      run;

      proc datasets nolist;
         delete notes;
      quit;
   
      %CreateUserStudies;
      
      %let p = %eval(&p+1);   
   %end;
   
   data _null_;
      file "&ManageStudyScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "objRefNS:FullAPName = ""&FullAPName"";";
      put "objRefNS:StudyName = ""&StudyName"";";
      put "jg:CVSInit;";
      %do p=1 %to &totalStudies;
        put "rc = GenerateStudyStarterFilter( ""&&ManageStudyList&p"" );";
        put "wait();";
      %end;
      put "sasmsgwindow=New Window(locale:get(""Results_Text"") || "": "" || locale:get(""ManageClinicalStudies_Text""),   Show Menu( 0 ), Show Toolbars( 0 ),";
      put "   VListBox(";
      %do p=1 %to &totalStudies;
         put "      TextBox(""\!N"" || locale:fill(locale:get(""ThemetadataforstudyXhavebeenSupdated_Text""), {""&&ManageStudyList&p""})),";
         put "      TextBox("" ""),";
         put "      ButtonBox(locale:get(""ViewIncludedandExcludedDomains_Text""),";
         put "          Try(open(""&ClientOutPath.&&ManageStudyList&p DomainList.pdf""));";
         put "      ),";         
      %end;
    *  put "      TextBox(""\!N"" || locale:fill(locale:get(""ThedataandmetadataforstudyXhavebeenSupdated_Text""), {""&StudyName""})),";
      put "      TextBox("" ""),";
      put "   );";   
      put ");";
      put "sasmsgwindow << SetWindowIcon(jg:LSpath[1]||""\Documentation\Icons\CodeBuilder.ico"");";
   run;  
%end;
%else %if %index(&Task,Rename) %then %do;
   %let OldStudyName = &StudyName;
   %let oldstudynameunder = %sysfunc(transtrn(&OldStudyName, %str( ), _));
   * check if the variable ReveiewEnabled of current study is set to 1;
   %let snapshotflg=0;
   %let oldmeta=%jg_getPath(/user/clinical/metadata,,0,0,3,&oldstudynameunder); 
   libname metalib "&oldmeta";
   %let snapfile=SNAPSHOT.sas7bdat;
   %if %length(&oldmeta) %then %do;
      %if %sysfunc(fileexist(&oldmeta.&snapfile)) %then %do;
         %global ReviewsEnabled;
         data _null_;
            set metalib.SNAPSHOT;
            call symput('ReviewsEnabled',trim(left(ReviewsEnabled)));
         run;
         %let snapshotflg=1;
         data snapshot;
            set metalib.snapshot;
         run;
      %end;
      %else %do;
         * for previous added study: there is no ReviewsEnabled variable;
         %if ^%symexist(ReviewsEnabled) %then %do;
            %global ReviewsEnabled;
            %let ReviewsEnabled=1;
         %end;
      %end;
   %end;
   %else %do;
      * for previous added study: there is no ReviewsEnabled variable;
      %if ^%symexist(ReviewsEnabled) %then %do;
         %global ReviewsEnabled;
         %let ReviewsEnabled=1;
      %end;
   %end;
   %let SDTMSelectFlag=0;
   %let ADaMSelectFlag=0;
   data _null_;
      set Userlib.&metaData;
      if StudyName="&StudyName" then do;
         call symput("SDTMSelectFlag",trim(left(SDTMSelectFlag))); 
         call symput("ADaMSelectFlag",trim(left(ADaMSelectFlag))); 
      end;
   run;   
   libname NotesLib "%jg_getPath(/user/clinical/notes,,0,0,3,&oldstudynameunder)";
   %if &ReviewsEnabled %then %do;
      libname DeltaLib "%jg_getPath(/user/clinical/delta,,0,0,3,&oldstudynameunder)";
   %end;
            
   * check if the new study name is already in the metadata table;
   %let tid = %sysfunc(open(Userlib.&metaData));
   %let nObs = %sysfunc(attrn(&tid,NOBS));
   %let nVars = %sysfunc(attrn(&tid,NVARS));
   %do i = 1 %to &nVars;
      %let vname = %sysfunc(varname(&tid,&i)); 
      %if %qupcase(&vname) = STUDYNAME %then %do;
         %do _j_=1 %to &nObs;
            %global _stName_&_j_ ;
            %let _rc_=%sysfunc(fetchobs(&tid,&_j_));
            %let _stName_&_j_= %sysfunc(getvarc(&tid,%sysfunc(varnum(&tid,StudyName))));        
            %put study name =&&_stName_&_j_;
            %if %qupcase(&NewStudyName)=%qupcase(&&_stName_&_j_) %then %do;
               %put [MS_S_0184];
               %put ERROR: The Study Name &NewStudyName already exists. Please choose a different name or remove this study first.;
               %let exiterror = 1;
               %let tid = %sysfunc(close(&tid));
            %end;
            %if %qupcase(%sysfunc(transtrn(&NewStudyName, %str( ), _))) = %qupcase(%sysfunc(transtrn(&&_stName_&_j_, %str( ), _))) %then %do;
               %put ERROR: Study Name &NewStudyName is too similar to &&_stName_&_j_ (spaces are equivalent to underscores). Please choose a different name or remove this study first.;
               %let exiterror = 1;
               %let tid = %sysfunc(close(&tid));
            %end;
         %end;
      %end;
   %end;
   %let tid = %sysfunc(close(&tid));
   %if &exiterror %then %goto exit;

   %if &SDTMSelectFlag %then %do;
      %let SselPath=%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&studynameunder);
      %local rc fileref;
      %let rc = %sysfunc(filename(fileref,&SselPath));
      %if %sysfunc(fexist(&fileref)) %then %do;
         libname SDSelLib "&SselPath";
         proc sql noprint; 
            select table 
            into :SDTMDom separated by ' '
            from SDSelLib.source_tables;
         quit;
      %end;
   %end;
   %if &ADaMSelectFlag %then %do;
      %let AselPath==%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&studynameunder);
      %local rc fileref;
      %let rc = %sysfunc(filename(fileref,&AselPath));
      %if %sysfunc(fexist(&fileref)) %then %do;
         libname ADSelLib "&AselPath";
         proc sql noprint; 
            select table 
            into :ADaMDom separated by ' '
            from ADSelLib.source_tables;
         quit;
      %end;
   %end;
   
   *** Since do not delete notes, check if notes file exists under rename ***;
   %let newstudynameundertmp = %sysfunc(transtrn(&NewStudyName, %str( ), _));
   %let ChkNotesDir = %jg_getPath(/user/clinical/notes,,0,0,3,&NewStudyName);
   %local rc fileref;
   %let rc = %sysfunc(filename(fileref,&ChkNotesDir));
   %if %sysfunc(fexist(&fileref)) %then %do;
      %let NotesExist = 0;
      libname NTsLibRn "&ChkNotesDir";   
      proc contents data = NTsLibRn._ALL_ nods;  
         ods exclude all;
         ods output members = members(keep = name where = (substr(upcase(name),1,1)= "N"));
      run;
      ods select all;

      %if %sysfunc(exist(members)) %then %do;
         %let _vn_ = %sysfunc(open(members));
         %let nobs = %sysfunc(attrn(&_vn_,nobs));
         %let _vn_ = %sysfunc(close(&_vn_));
      %end;
      %else %let nobs = 0;

      %if &nobs %then %let NotesExist = 1;

      proc datasets nolist;
         delete members;
      quit;

      %if ^&NotesExist %then %do;
         %put NOTE: No notes detected under new study name.;
         %let rc = %sysfunc(filename(fileref));
      %end;
      %else %do;
          %put [MS_S_0183];
          %put ERROR: Notes exist for a study previously defined as &NewStudyName.. Please choose a different study name or move the notes file to a new location.;
          %put ERROR: Notes file is located &ChkNotesDir.&notesDataset..sas7bdat;
          %let exiterror = 1;
          %let tid = %sysfunc(close(&tid));
          %goto exit;
      %end;
   %end;

   %let OldStudyName = &StudyName;
   %let StudyName = &NewStudyName;
   %CHKSTUDYNAME;
   %if &exiterror %then %goto exit;

   %let studynameunder = %sysfunc(transtrn(&StudyName, %str( ), _));
   %let NewClientOutPath = %jg_getPath(/user/clinical/output,,1,0,1,&studynameunder);

   data _null_;
       set Userlib.&metaData;
       if StudyName="&OldStudyName" then do;
          call symput("InPath",SDTMFolder); 
          call symput("AInPath",ADaMFolder); 
       end;
   run;

   %let trStudyXisrenamedtoY = %nrbquote(%LocaleGet(StudyXisrenamedtoY_Text));
   %let trStudyRenamed = %nrbquote(%LocaleFill(&trStudyXisrenamedtoY,f1=%nrbquote(&OldStudyName),f2=%nrbquote(&NewStudyName)));

   *** Add note that study was renamed ***;
   %let rn =;
   %if %sysfunc(exist(noteslib.&notesDataset)) %then %do;
      data _null_;
         set noteslib.&notesDataset end=last;
         if last then call symputx("rn",SnapshotNumber);
         %put drewrn = &rn;
      run;
   %end;

   data _null_;
      call symput("RDateC", put(datetime(), DATETIME19.));
   run;     

   data notes;
      length  Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
      Study_Name = "&StudyName";
      Report = "&trProcessName";
      User = "&sysuserid";
      SystemGenerated = "Y";
      Date_of_Note = datetime();
      %if %length(&rn) %then %do;
         SnapshotNumber = &rn;
      %end;
      %else %if %length(&ReviewNumber) %then %do;
         SnapshotNumber = &ReviewNumber;
      %end;
      %else %do;
         SnapshotNumber =.;
      %end;
      Type = "&trSnapshot";
      Note = "&trStudyRenamed";
      output;
      format Date_of_Note DATETIME19.;
      label Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" SystemGenerated = "&trSystemGenerated";
   run;   

   data notes;
      retain SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note;
      set %if %sysfunc(exist(noteslib.&notesDataset)) %then %do; NotesLib.&notesDataset %end; notes;
   run;

   proc sort data = notes out = NotesLib.&notesDataset;
      by SnapshotNumber Date_of_Note;
   run;

   proc datasets nolist;
      delete notes;
   quit;

   * copy notes directory;   
   libname NewNotes "%jg_getPath(/user/clinical/notes,,1,0,3,&studynameunder)";
   proc copy in=NotesLib out=NewNotes;
   run;
   
   * copy delta directory;
   %if &ReviewsEnabled %then %do;
      libname NewDelta "%jg_getPath(/user/clinical/delta,,1,0,3,&studynameunder)";
      proc copy in=DeltaLib out=NewDelta;
      run;

      * copy modified directory;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&OldStudyName)));
      %if %sysfunc(fexist(&fileref)) %then %do; 
         libname OldMod "%jg_getPath(/user/clinical/delta/modified,,0,0,3,&OldStudyName)";
         libname NewMod "%jg_getPath(/user/clinical/delta/modified,,1,0,3,&studynameunder)";
         proc copy in=OldMod out=NewMod;
         run;   
      %end;

      * copy dropped directory;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&OldStudyName)));
      %if %sysfunc(fexist(&fileref)) %then %do;  
         libname OldDrop "%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&OldStudyName)";
         libname NewDrop "%jg_getPath(/user/clinical/delta/dropped,,1,0,3,&studynameunder)";
         proc copy in=OldDrop out=NewDrop;
         run;   
      %end;
   %end;

   * copy filter directory;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/filters,,0,0,,&OldStudyName))); 
   %if %sysfunc(fexist(&fileref)) %then %do;
      libname OldFILT "%jg_getPath(/user/clinical/filters,,0,0,,&OldStudyName)";
      libname NewFILT "%jg_getPath(/user/clinical/filters,,1,0,,&studynameunder)";
      proc copy in=OldFILT out=NewFILT;
      run;
   %end;
 
   * get current review number;
   %let CurrentReview = 0;
   filename cst "%jg_getPath(/user/clinical/metadata/base,CSTMacroVars.sas,0,0,3,&studynameunder)";
   %if %sysfunc(fexist(cst)) %then %do;
      data _null_;
         infile cst linesize = 50 pad; 
         input text $ 1-50;
         if substr(left(text),2,18) = "let ReviewNumber =" then call symput("CurrentReview", trim(left(scan(text, 4, ' ;'))));
      run;
   %end;
   %put CurrentReview = &CurrentReview;

   * generate new metadata;
   %let NeedDelta = NO;
   %let NewSnap = NO;
   %GENERATEMETA(studname = &studyname);
   %if &CSTERROR %then %do;
      %put [MS_S_0008];
      %put ERROR: Creation of MetaData Failed.;
      %let exiterror = 1;
      %goto exit;
   %end;
   libname NewMeta "%jg_getPath(/user/clinical/metadata,,1,0,3,&studynameunder)";
   * copy metadata directory;
   %let oldstudynameunder = %sysfunc(transtrn(&OldStudyName, %str( ), _));
   %let StudyLogFile =%jg_getPath(/user/clinical/metadata,StudyLog.txt,0,0,3,&OldStudyName);
   filename inf "%jg_getPath(/user/clinical/metadata,StudyLog.txt,0,0,3,&OldStudyName)";
   filename outf "%jg_getPath(/user/clinical/metadata,StudyLog.txt,1,0,3,&studynameunder)";   
   %if %sysfunc(fileexist(&StudyLogFile)) %then %do;
      data _null_;
         infile inf;
         file outf;
         input;
         put _infile_;
      run;
   %end;
   libname MetaLib "%jg_getPath(/user/clinical/metadata,,0,0,3,&OldStudyName)";  

   * Copy _ProcessInfo the new studay metadata folder before old study metadata was removed;
   %if %sysfunc(exist(MetaLib._processinfo)) %then %do;
      data NewMeta._processinfo;
         set MetaLib._processinfo;
      run;
   %end;
   
   *copy snapshot to the new study metadata folder;
   %if &snapshotflg %then %do;
       data NewMeta.snapshot;
          set snapshot;
      run;
   %end;
   
/* proc copy in=MetaLib out=NewMeta;
   run;*/
   
   * change metadata table;
   data Userlib.&metaData;
      length StudyName $32 SDTMFolder ADAMFolder $200 InitialDate LastUpdatedDate $25 CreatedBy $64 LastUpdatedBy $64;
      set Userlib.&metaData;
      %let d=%sysfunc(today(),&IntDateFormat..);
      %let t=%sysfunc(time(),time16.);
      %let c=&d &t;
      if StudyName="&OldStudyName" then do;
         StudyName="&StudyName";
         LastUpdatedDate="&c";
         LastUpdatedBy = "%sysget(username)";
      end;
   run;

   %CreateUserStudies;
   
   * delete old folders except for metadata root (will delete later);
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&OldStudyName))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&OldStudyName));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&OldStudyName))) ;  
   %end;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&OldStudyName))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&OldStudyName));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&OldStudyName))) ;  
   %end; 
   %RemoveDir(%jg_getPath(/user/clinical/metadata/base,,0,0,3,&OldStudyName));
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/base,,0,0,3,&OldStudyName))) ;  
   /*
   %RemoveDir(%jg_getPath(/user/clinical/metadata,,0,0,3,&OldStudyName));
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata,,0,0,3,&OldStudyName))) ; 
   %if &rc %then %let maxrc = &rc; 
   */
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&OldStudyName))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&OldStudyName));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&OldStudyName))) ;  
   %end;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&OldStudyName))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
      %RemoveDir(%jg_getPath(/user/clinical/delta/modified,,0,0,3,&OldStudyName));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&OldStudyName))) ;    
   %end;
   * delete old filter folder;
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/filters,,0,0,3,&OldStudyName))) ;
   %if %sysfunc(fexist(&fileref)) %then %do;
       %RemoveDir(%jg_getPath(/user/clinical/filters,,0,0,3,&OldStudyName));
       %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/filters,,0,0,3,&OldStudyName))) ;  
   %end;
   %RemoveDir(%jg_getPath(/user/clinical/delta,,0,0,3,&OldStudyName));
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta,,0,0,3,&OldStudyName))) ; 
   %if &rc %then %let maxrc = &rc; 
   %RemoveDir(%jg_getPath(/user/clinical/notes,,0,0,3,&OldStudyName));
   %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/notes,,0,0,3,&OldStudyName))) ; 
   %if &rc %then %let maxrc = &rc; 
   %put maxrc = &rc;
   
   /*
   %if &maxrc %then %do;
      %let exiterror = 1;
      %goto exit;
   %end;
   */

   %*rename sas7bdat datasets when there are spaces in the old study name;
   %if &Server = Local & %index(&OldStudyName,%str( )) %then %do;
      %let oldStudyPath=%jg_getPath(/user/clinical/output,,0,0,3,&OldStudyName);
      %FileFilter(&oldStudyPath,studylist,_ext="sas7bdat");
      data studylist;
         set studylist;
         idx = index(upcase(filename),upcase("&oldstudynameunder"));
         if (idx > 0) then do;
            newName = tranwrd(upcase(filename),upcase("&oldstudynameunder"), "&studynameunder"); 
         end;
         else delete;
      run;

      data studylist;
         set studylist;
         _F_ = 'sasInFile'||trim(left(_n_));
         _NF_ = 'NewFileName'||trim(left(_n_));
         call symput(_F_,trim(filename));
         call symput(_NF_,trim(newName));
      run;
      %NObsVars(studylist);
      %let nStudy = &nObs;

      %*CheckEndSlash(&oldStudyPath);
      libname newlib "%jg_getPath(/user/clinical/output,,0,0,3,&OldStudyName)";
      %do j=1 %to &nStudy;
         %let oldfileName =%substr(&&sasInFile&j,1,%length(&&sasInFile&j)-9);
         %let newName=%substr(&&NewFileName&j,1,%length(&&NewFileName&j)-9);

         data _null_;
            put "file is newlib.&oldfileName";
            put "new file name is &newName";
            rc=rename("newlib.&oldfileName", "&newName");
            if (rc) then
            put 'Problems renaming file';
            else 
            put 'rename successful';
         run;
      %end;
   %end;
   
   data _null_;
      file "&ManageStudyScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "objRefNS:FullAPName = ""&FullAPName"";";
      put "objRefNS:StudyName = ""&StudyName"";";
      %if &Server ne Local %then %do;
         put "rc = jg:getPath(""metadatabase"", ""CSTMacroVars.sas"", 1, 1, 0, ""&newstudynameundertmp"");";
         put "rc = jg:SASGetFile2(jg:ServerOutpath||""userstudies.sas7bdat"",""&CLIENT_LSUSER_HOME.metadata_study_from_server.sas7bdat"");";
      %end;
      
      * copy CVS files that do not contain study name in contents;
      * actually need to save (rather than copy) so that their timestamps are updated;
      put "CVSFilesToCopy = {";
      put "    ""study_report_comps.sas7bdat"",";
      put "    ""study_report_vars.sas7bdat"",";
      put "    ""StarterFilter.jsl"",";
      put "    ""StudyCVS.jsl""";
      put "};";      
      put "for (i = 1, i <= NItems(CVSFilesToCopy), i++,";
      put "    sourcefile = jg:getPath(""/user/clinical/metadata"", CVSFilesToCopy[i], 0, 0, 1, ""&OldStudyName"");";
      put "    targetfile = jg:getPath(""/user/clinical/metadata"", CVSFilesToCopy[i], 1, 0, 1, ""&StudyName"");";
      put "    If(File Exists(sourcefile),";
      put "        Try(";
      put "            filecontents = LoadTextFile(sourcefile, BLOB);";
      put "            SaveTextFile(targetfile, filecontents);";
      put "            ,";
      put "            print(""ERROR: Although "" || sourcefile || "" exists, could not save it to "" || targetfile || ""."");";
      put "        );";
      put "    );";
      put ");";
      
      * delete old study metadata root;
      put "sourcedir = jg:getPath(""/user/clinical/metadata"", """", 0, 0, 1, ""&OldStudyName"");";      
      put "If(Directory Exists(sourcedir),";
      put "    Try(";
      put "        Delete Directory(sourcedir);";
      put "        ,";
      put "        print(""ERROR: Although "" || sourcedir || "" exists, could not delete it."");";
      put "    );";
      put ");";
      
      * replace old study name with new study name in output files;
      put "Include(""$LS_HOME\JSLFiles\RenameStudy.jsl"");";
      put "RenameStudy(""&OldStudyName"", ""&StudyName"", ""&ClientOutPath"");";
      put "MoveStudy(""&ClientOutPath"",""&NewClientOutPath"");";
      
      * update current study list on clinical starter;
      put "jg:_defaultexists_3 = 0;";
      put "jg:study3list = expr(StudyListClass:studyListSingleton):getStudyList();";
      put "jg:StudyName = ""&StudyName"";";
      put "jg:updateStarterStudyList(3,jg:StudyName,jg:sasserver);";
      
      * update current study list and folders in manage studies dialog;
      /*
      put "jg:ManageStudies_ManageStudyList << Set Items(jg:study3list);";
      put "jg:ManageStudies_ManageStudyList << Set(Loc(jg:study3list,jg:StudyName));";
      */
      put "jg:getStudyFolders;";
      /*
      put "jg:ManageStudies_InPath << SetText(sdtmfolder);";
      put "jg:ManageStudies_AInPath << SetText(adamfolder);";
      */
      put "jg:CVSInit;";
      put "rc = GenerateStudyStarterFilter( ""&StudyName"" );";
      put "wait();";
      put 'sasmsgwindow=New Window(locale:get("Results_Text") || ": " || locale:get("ManageClinicalStudies_Text"),showmenu(0),showtoolbars(0),';
      put '   VListBox(';
      put "      TextBox(""\!N"" || locale:fill(locale:get(""TheStudyXhasbeensuccessfullyrenamedtoX_Text""), {""&OldStudyName"", ""&StudyName""})),";
      put "      TextBox("" "")";
      put "   );";
      put ");";
      put "sasmsgwindow << SetWindowIcon(jg:LSpath[1]||""\Documentation\Icons\CodeBuilder.ico"");";
   run;  
%end;
%else %if %index(&Task,Clear) %then %do;
      
   * use jsl to handle local files so this works in local and client-server mode;
   data _null_;
      file "&ManageStudyScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "objRefNS:FullAPName = ""&FullAPName"";";
      put "objRefNS:StudyName = ""&StudyName"";";
      put "listOfFileNames=FilesInDirectory(""&ClientOutPath"");";
      put "savelist={""sasclean.jsl"",""sasmessage.jsl"",""ManageStudies.log"",""ManageStudies__&StudyName..sas"",""_processinfo.sas7bdat"",""SAS Log for ManageStudies.jsl""};";
      put "if (isList(listOfFileNames),";
      put "   for(i=1,i<=NItems(listOfFileNames), i++,";
      put "      if(!contains(savelist,listOfFileNames[i]),DeleteFile(""&ClientOutPath.""||listOfFileNames[i]))";
      put "   )";
      put ");";
      put "sasmsgwindow=New Window(locale:get(""Results_Text"") || "": "" || locale:get(""ManageClinicalStudies_Text""),   Show Menu( 0 ), Show Toolbars( 0 ),";
      put "   VListBox(";
      put "      TextBox(""\!N"" || locale:fill(locale:get(""TheStudyXoutputfolderhasbeensuccessfullycleared_Text""), {""&StudyName""})),";
      put "      TextBox("" "")";
      put "   );";   
      put ");";
      put "sasmsgwindow << SetWindowIcon(jg:LSpath[1]||""\Documentation\Icons\CodeBuilder.ico"");";
   run;

%end;
%else %if %index(&Task,Remove) %then %do;
   
   %let p=1;
   %do %while(%length(%scan(&ManageStudyList,&p,'|')));
      %let StudyName= %scan(&ManageStudyList,&p,'|');
      %let studynameunder = %sysfunc(transtrn(&studyname, %str( ), _));
      %put StudyName=&StudyName p=&p ManageStudyList=&ManageStudyList;
      
      %NObsVars(Userlib.&metaData);
      %if &nobs=1 %then %do;
         %put [MS_S_0140];
         %put ERROR: Study &StudyName could not be deleted because it is the only study defined.  You must have at least one study defined.;
         %let exiterror = 1;
         %goto exit;
      %end;

*     libname NotesLib "&Metapath.Notes&_slash_&studynameunder&_slash_";
      libname NotesLib "%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder)";
      * Copy _ProcessInfo and StudyLog to "Note" folder before metadata was removed;
      %if %sysfunc(exist(_LogLib_._processinfo)) %then %do;
         data NotesLib._processinfo;
            set _logLib_._processinfo;
         run;
      %end;
   
      filename inf "%sysfunc(pathname(_LogLib_))&_slash_.StudyLog.txt";
      filename outf "%sysfunc(pathname(NotesLib))&_slash_.StudyLog.txt";
      data _null_;
         infile inf;
         file outf;
         input;
         put _infile_;
      run;  
   
      * delete the study name row from the metadata table;
      data Userlib.&metaData;
         set Userlib.&metaData;
         if StudyName="&StudyName" then delete;
      run;
   
      %CreateUserStudies;

      * delete associated folders;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&studynameunder))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&studynameunder));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/sdtm,,0,0,3,&studynameunder))) ;  
      %end;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&studynameunder))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&studynameunder));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata/adam,,0,0,3,&studynameunder))) ;  
      %end; 
      %RemoveDir(%jg_getPath(/user/clinical/metadata/base,,0,0,3,&studynameunder));  
      %RemoveDir(%jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/metadata,,0,0,3,&studynameunder))) ; 
      %if &rc %then %let maxrc = &rc;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/dropped,,0,0,3,&studynameunder))) ;  
      %end;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta/modified,,0,0,3,&studynameunder))) ;    
      %end; 
      %RemoveDir(%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder));
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/delta,,0,0,3,&studynameunder))) ; 
      %if &rc %then %let maxrc = &rc; 
      %put maxrc = &rc;
      /*
      %if &maxrc %then %do;
         %let exiterror = 1;
         %goto exit;
      %end;
      */

      *** Add note that study was removed ***;
      %let rn =;
      %if %sysfunc(exist(noteslib.&notesDataset)) %then %do;
         data _null_;
            set noteslib.&notesDataset end=last;
            if last then call symputx("rn",SnapshotNumber);
            %put drewrn = &rn;
         run;
      %end;

      data _null_;
         call symput("RDateC", put(datetime(), DATETIME19.));
      run;     
   
      data notes;
         length  Study_Name $ 32 Report $ 60 Domain $ 8 Keys $ 200 Keys_Values $ 500 User $ 50 Note $ 1024 Type $ 24 SystemGenerated $ 1;
         Study_Name = "&StudyName";
         Report = "&trProcessName";
         User = "&sysuserid";
         SystemGenerated = "Y";
         Date_of_Note = datetime();
         %if %length(&rn) %then %do;
            SnapshotNumber = &rn;
         %end;
         %else %if %length(&ReviewNumber) %then %do;
            SnapshotNumber = &ReviewNumber;
         %end;        
         %else %do;
            SnapshotNumber =.;
         %end;
         Type = "&trSnapshot";
         Note = "&trSRfJC: &RDateC";
         output;
         format Date_of_Note DATETIME19.;
         label Study_Name = "&trStudyName" Keys_Values = "&trKeysValues" Report = "&trReport" Date_of_Note = "&trDateofNote" SnapshotNumber = "&trSnapshotNumber" SystemGenerated = "&trSystemGenerated";
      run;   

      data notes;
         retain SnapshotNumber Type usubjid Study_Name Keys Keys_Values Domain Report User Note Date_of_Note;
         set %if %sysfunc(exist(noteslib.&notesDataset)) %then %do; NotesLib.&notesDataset %end; notes;
      run;

      proc sort data = notes out = NotesLib.&notesDataset;
         by SnapshotNumber Date_of_Note;
      run;

      proc datasets nolist;
         delete notes;
      quit;

      *** copy notes directory when study deleted to create archive ***;
      %if %upcase(&KeepOnDelete) =NO %then %do;
         *** delete study notes folder from studies directory ***;
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder))) ;
         %if %sysfunc(fexist(&fileref)) %then %do;
            %RemoveDir(%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder));
            %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder))) ;  
         %end;
   
         *** delete study reviews folder from studies directory  ***;
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/reviews,,0,0,3,&studynameunder))) ;
         %if %sysfunc(fexist(&fileref)) %then %do;
            %RemoveDir(%jg_getPath(/user/clinical/reviews,,0,0,3,&studynameunder));
            %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/reviews,,0,0,3,&studynameunder))) ;    
         %end; 
      %end;

      *** delete study output folder with userid from studies directory ***;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/Output,,0,0,3,&studynameunder))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/Output,,0,0,3,&studynameunder));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/Output,,0,0,3,&studynameunder))) ;    
      %end;
      %let apath=%jg_getPath(/user/clinical/outputnouser,,0,0,3);
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/outputnouser,,0,0,3))) ;
      %if %sysfunc(fexist(&fileref)) %then %do;
         %RemoveDir(%jg_getPath(/user/clinical/outputnouser,,0,0,3));
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/outputnouser,,0,0,3))) ;    
      %end;
      
      *** delete study directory ***;
      *** check if there is reviews or notes folder under studies/studyname folder. ***;
      *** if there are folders, then do not delete study folder. ***;
      %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/studies,notes,0,0,3))) ;
      %if ^%sysfunc(fexist(&fileref)) %then %do;
         %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/studies,reviews,0,0,3))) ;
         %if ^%sysfunc(fexist(&fileref)) %then %do;
            %put "delete studies folder" &fileref;
            %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/study,,0,0,3,&studynameunder))) ;
            %if %sysfunc(fexist(&fileref)) %then %do;
               %RemoveDir(%jg_getPath(/user/clinical/study,,0,0,3,&studynameunder));
               %let rc = %sysfunc(filename(fileref,%jg_getPath(/user/clinical/study,,0,0,3,&studynameunder))) ;    
            %end; 
         %end;
      %end;
      
      %let p = %eval(&p+1);   
   %end;   
   libname NotesLib "&outpath";
   
   data _null_;
      file "&ManageStudyScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "objRefNS:FullAPName = ""&FullAPName"";";
      put "objRefNS:StudyName = ""&StudyName"";";
      %if &Server ne Local %then %do;
         put "rc = jg:SASGetFile2(jg:ServerOutpath||""userstudies.sas7bdat"",""&CLIENT_LSUSER_HOME.metadata_study_from_server.sas7bdat"");";
      %end;
      put "listOfFileNames=FilesInDirectory(""&ClientOutPath"");";
      put "savelist={""sasclean.jsl"",""sasmessage.jsl"",""ManageStudies.log"",""ManageStudies__&StudyName..sas"",""_processinfo.sas7bdat"",""SAS Log for ManageStudies.jsl""};";
      put "if (isList(listOfFileNames),";
      put "   for(i=1,i<=NItems(listOfFileNames), i++,";
      put "      if(!contains(savelist,listOfFileNames[i]),DeleteFile(""&ClientOutPath""||listOfFileNames[i]))";
      put "   )";
      put ");";
      * update current study list on clinical starter;
      put "jg:_defaultexists_3 = 0;";
      put "jg:study3list = expr(StudyListClass:studyListSingleton):getStudyList();";
      put "if(NItems(jg:study3list),jg:StudyName = jg:study3list[1],jg:StudyName="""");";
      put "//show(jg:study3list,jg:StudyName);";
      put "jg:updateStarterStudyList(3,jg:StudyName,jg:sasserver);";
      * update current study list and folders in manage studies dialog;
      /*
      put "jg:ManageStudies_ManageStudyList << Set Items(jg:study3list);";
      */
      put "sdtmFolderNew = "";";
      put "adamFolderNew = "";";
      put 'sasmsgwindow=New Window(locale:get("Results_Text") || ": " || locale:get("ManageClinicalStudies_Text"),  Show Menu( 0 ), Show Toolbars( 0 ),';
      put '   VListBox(';  
  /*    put '   objRefNS:vlb=VListBox(';*/
      put "  //    TextBox(""\!N"" || locale:fill(locale:get(""TheStudyXhasbeensuccessfullyremoved_Text""), {""&StudyName""})),";   
      %do p=1 %to &totalStudies;
         put "      TextBox(""\!N"" || locale:fill(locale:get(""TheStudyXhasbeensuccessfullyremoved_Text""), {""&&ManageStudyList&p""})),";
      %end;
      put "      TextBox("" "")";
      put "   );";   
      put ");";
      put "sasmsgwindow << SetWindowIcon(jg:LSpath[1]||""\Documentation\Icons\CodeBuilder.ico"");";
     run;

%end;

%PackageInsertFile(SAS message from &ProcessName, &OutPath, &ManageStudyScriptName, , text/plain, yes);

%exit:;

%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, 1);

%mend ManageStudies;
* run macro;
%ManageStudies;