/*
JMP Clinical Analytical Process

Name: Patient Recruitment
Authors: 
Xiaotong Jiang, UNC-Chapel Hill
Richard C. Zink, SAS Institute Inc.

Description: 
This programs implements the method of a Poisson-Gamma predictive model. It outputs a prediction 
graph and a table of current enrollment. If necessary, adaptive adjustment will be initiated to 
determine the number of new centers needed for the trial to be on schedule. 
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath TrtUse TrtVar DesignByVars SubjectFilter WhereStmt OutPath ServerOutPath 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL  WhereForReport OutName
PatRecruit_OutData MacroPath PatRecruitScript TargetEnroll TargetTime UseLast 
NewStart TruncDate CurrentTime NSim SeedNum Tick Delay Prob1 Prob2 MCenter 
tmaxdate tmindate nmax num1 first Prob1 pobs2 start StartNum ErrorM SiteActiveFlag;

*ProcessBody;

%macro PatRecruit;

/*
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags"}}$
$CMP: RSYS$ $UE: DM.SITEID and (DS.DSSTDTC or DM.RANDDT or DM.RFSTDTC)$ $NOTES: DSSTDTC used where DSDECOD is RANDOMIZED$ 
$CMP: OSYS$ $UE: SV.SVSTDTC$ $NOTES: Used (if available) as a surrogate for site active date when site has 0 randomized subjects$ 
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = PatientRecruitment; 
%let WhereForReports = &WhereStmt;

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%let WhereForReport=%nrbquote(&WhereStmt);
%let includeds = ;

%OUTNAMETRUNC(outdat = PatRecruit_OutData, truncnum = 18);

%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;.   
%end;

%let trPlots = %nrbquote(%LocaleGet(Plots_Text));
%let trRandomized = %nrbquote(%LocaleGet(Randomized_Text));
%let trTable = %nrbquote(%LocaleGet(Table_Text));
%let trScreenFailure = %nrbquote(%LocaleGet(ScreenFailure_Text));
%let trActualEnrollment = %nrbquote(%LocaleGet(ActualEnrollment_Text));
%let trNumberofdaysuntilnow = %nrbquote(%LocaleGet(Numberofdaysuntilnow_Text));
%let trStartingDateofEstandPred = %nrbquote(%LocaleGet(StartingDateofEstandPred_Text));
%let trstartdate = %nrbquote(%LocaleGet(startdate_Text));
%let trbeginning = %nrbquote(%LocaleGet(beginning_Text));
%let trRecruitmentRateday = %nrbquote(%LocaleGet(RecruitmentRateday_Text)); 
%let trRecruitmentRateweek = %nrbquote(%LocaleGet(RecruitmentRateweek_Text)); 
%let trGammasimulationfor = %nrbquote(%LocaleGet(Gammasimulationfor_Text)); 
%let trMaxLikeEst = %nrbquote(%LocaleGet(MaxLikeEst_Text)); 
%let trSimulationIndex = %nrbquote(%LocaleGet(SimulationIndex_Text)); 
%let trSumofgamma = %nrbquote(%LocaleGet(Sumofgamma_Text)); 
%let trNumberofsubjectsleft = %nrbquote(%LocaleGet(Numberofsubjectsleft_Text)); 
%let trGammasimulation = %nrbquote(%LocaleGet(Gammasimulation_Text)); 
%let trPredictedtimeBayes = %nrbquote(%LocaleGet(PredictedtimeBayes_Text)); 
%let trPredictedMean = %nrbquote(%LocaleGet(PredictedMean_Text)); 
%let trPredicted975 = %nrbquote(%LocaleGet(Predicted975_Text)); 
%let trPredicted25 = %nrbquote(%LocaleGet(Predicted25_Text)); 
%let trDateRecruitComp = %nrbquote(%LocaleGet(DateRecruitComp_Text));
%let trLowerPercentile = %nrbquote(%LocaleGet(LowerPercentile_Text)); 
%let trPredModPatRecr = %nrbquote(%LocaleGet(PredModPatRecr_Text)); 
%let trStartDate = %nrbquote(%LocaleGet(StartDate_Text)); 
%let trRecruitmentDate = %nrbquote(%LocaleGet(RecruitmentDate_Text)); 
%let trNumberPatientsRecr = %nrbquote(%LocaleGet(NumberPatientsRecr_Text)); 
%let trTargetDate = %nrbquote(%LocaleGet(TargetDate_Text)); 
%let trCurrentDate = %nrbquote(%LocaleGet(CurrentDate_Text)); 
%let trCurrentRecruit = %nrbquote(%LocaleGet(CurrentRecruit_Text)); 
%let trTargetRecruit = %nrbquote(%LocaleGet(TargetRecruit_Text)); 
%let trCurrentRecruitmentbySite = %nrbquote(%LocaleGet(CurrentRecruitmentbySite_Text)); 
%let trPatientRecruitment = %nrbquote(%LocaleGet(PatientRecruitment_Text)); 
%let trTotalrateaddMcenters = %nrbquote(%LocaleGet(TotalrateaddMcenters_Text)); 
%let trNumberofnewcenters = %nrbquote(%LocaleGet(Numberofnewcenters_Text)); 
%let trProbabilityofmeetingtargetdate = %nrbquote(%LocaleGet(Probabilityofmeetingtargetdate_Text)); 
%let trAveragepredicteddate = %nrbquote(%LocaleGet(Averagepredicteddate_Text)); 
%let trPercentile975preddate = %nrbquote(%LocaleGet(Percentile975preddate_Text)); 
%let trPercentile25preddate = %nrbquote(%LocaleGet(Percentile25preddate_Text)); 
%let trProbabilityMeetingDeadline = %nrbquote(%LocaleGet(ProbabilityMeetingDeadline_Text));    
%let trNewCenters = %nrbquote(%LocaleGet(NewCenters_Text));    
%let trNeed = %nrbquote(%LocaleGet(Need_Text));
%let trMinimum = %nrbquote(%LocaleGet(Minimum_Text));
%let trPredictedDateRecrComp = %nrbquote(%LocaleGet(PredictedDateRecrComp_Text));
%let trNumberNewCentersNeeded = %nrbquote(%LocaleGet(NumberNewCentersNeeded_Text));
%let trAdaptiveAdjustment = %nrbquote(%LocaleGet(AdaptiveAdjustment_Text));
%let trNumberofpatientsrecruited = %nrbquote(%LocaleGet(Numberofpatientsrecruited_Text)); 
%let trFirstrecruitmentdate = %nrbquote(%LocaleGet(Firstrecruitmentdate_Text)); 
%let trDurationofrecruitment = %nrbquote(%LocaleGet(Durationofrecruitment_Text)); 
%let trNumberofpatientsrecruitedstart = %nrbquote(%LocaleFill(&trNumberofpatientsrecruited, f1=%nrbquote(&trstartdate)));
%let trFirstrecruitmentdatestart = %nrbquote(%LocaleFill(&trFirstrecruitmentdate, f1=%nrbquote(&trstartdate)));
%let trDurationofrecruitmentstart = %nrbquote(%LocaleFill(&trDurationofrecruitment, f1=%nrbquote(&trstartdate)));
%let trNumberofpatientsrecruitedbegin = %nrbquote(%LocaleFill(&trNumberofpatientsrecruited, f1=%nrbquote(&trbeginning)));
%let trFirstrecruitmentdatebegin = %nrbquote(%LocaleFill(&trFirstrecruitmentdate, f1=%nrbquote(&trbeginning)));
%let trDurationofrecruitmentbegin = %nrbquote(%LocaleFill(&trDurationofrecruitment, f1=%nrbquote(&trbeginning)));

%macro CHKDT(date = );
   %let date = %sysfunc(strip(&date));
   %let mlist = JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC;
   %let nlist = 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31;
   %let dlist = 1 2 3 4 5 6 7 8 9 0;

   %let day = %substr(&date,1,2);
   %let mon = %upcase(%substr(&date,3,3));
   %let year = %substr(&date,6);

   %if ^%sysfunc(findw(&nlist, &day)) %then %let prdateerr = 1;
   %if ^%sysfunc(findw(&mlist, &mon)) %then %let prdateerr = 1;
   
   %if &mon = APR and &day = 31 %then %let prdateerr = 1;
   %if &mon = JUN and &day = 31 %then %let prdateerr = 1;
   %if &mon = SEP and &day = 31 %then %let prdateerr = 1;
   %if &mon = NOV and &day = 31 %then %let prdateerr = 1;
   %if &mon = FEB and &day = 30 %then %let prdateerr = 1;
   %if &mon = FEB and &day = 31 %then %let prdateerr = 1;
   
   %let leapyear = 0;   
   %if %sysfunc(mod(&year,4)) = 0 %then %do;
      %if %sysfunc(mod(&year,100)) = 0 %then %do;
         %if %sysfunc(mod(&year,400)) = 0 %then %do;
            %let leapyear = 1;
         %end;
      %end;
      %else %do;
         %let leapyear = 1;
      %end;
   %end;

   %if &mon = FEB and &day = 29 and &leapyear = 0 %then %let prdateerr = 1;
   
   %do i = 1 %to %length(&year);
      %let val = %substr(&year,&i,1);
      %if ^%sysfunc(findw(&dlist, &val)) %then %let prdateerr = 1;
   %end;
%mend CHKDT;

%macro CHKDTA(date = );
   %let date = %sysfunc(strip(&date));
   %let mlist = 01 02 03 04 05 06 07 08 09 10 11 12;
   %let nlist = 01 02 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31;
   %let dlist = 1 2 3 4 5 6 7 8 9 0;

   %let day = %substr(&date,1,2);
   %let mon = %upcase(%substr(&date,4,2));
   %let year = %substr(&date,7);

   %if ^%sysfunc(findw(&nlist, &day)) %then %let prdateerr = 1;   
   %if ^%sysfunc(findw(&mlist, &mon)) %then %let prdateerr = 1;

   %if &mon = 04 and &day = 31 %then %let prdateerr = 1;
   %if &mon = 06 and &day = 31 %then %let prdateerr = 1;
   %if &mon = 09 and &day = 31 %then %let prdateerr = 1;
   %if &mon = 11 and &day = 31 %then %let prdateerr = 1;
   %if &mon = 02 and &day = 30 %then %let prdateerr = 1;
   %if &mon = 02 and &day = 31 %then %let prdateerr = 1;
   
   %let leapyear = 0;   
   %if %sysfunc(mod(&year,4)) = 0 %then %do;
      %if %sysfunc(mod(&year,100)) = 0 %then %do;
         %if %sysfunc(mod(&year,400)) = 0 %then %do;
            %let leapyear = 1;
         %end;
      %end;
      %else %do;
         %let leapyear = 1;
      %end;
   %end;

   %if &mon = 02 and &day = 29 and &leapyear = 0 %then %let prdateerr = 1;

   %do i = 1 %to %length(&year);
      %let val = %substr(&year,&i,1);
      %if ^%sysfunc(findw(&dlist, &val)) %then %let prdateerr = 1;
   %end;
%mend CHKDTA;

%if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = en and &IntDateFormat = DATE9 %then %do;
   %let prdateerr = 0;
   %CHKDT(date = &TargetTime);
   %if &prdateerr %then %do;
      %put ERROR: The target date must be of this form: DDMMMYYYY, e.g. 09Feb1989.;
      %let exiterror = 1;   
   %end;
   %if %length(&CurrentTime) %then %do;
      %let prdateerr = 0;
      %CHKDT(date = &CurrentTime);
      %if &prdateerr %then %do;
         %put ERROR: The current date must be of this form: DDMMMYYYY, e.g. 09Feb1989.;
         %let exiterror = 1;   
      %end;
   %end;
   %if %length(&TruncDate) %then %do;
      %let prdateerr = 0;
      %CHKDT(date = &TruncDate);
      %if &prdateerr %then %do;
         %put ERROR: The truncation date must be of this form: DDMMMYYYY, e.g. 09Feb1989.;
         %let exiterror = 1;   
      %end;
   %end;
   %if &exiterror %then %goto exit;
%end;
%else %if &IntDateFormat = DDMMYY10 %then %do;
   %let prdateerr = 0;
   %CHKDTA(date = &TargetTime);
   %if &prdateerr %then %do;
      %put ERROR: The target date must be of this form: DD/MM/YYYY, e.g. 09/02/1989.;
      %let exiterror = 1;   
   %end;
   %if %length(&CurrentTime) %then %do;
      %let prdateerr = 0;
      %CHKDTA(date = &CurrentTime);
      %if &prdateerr %then %do;
         %put ERROR: The current date must be of this form: DD/MM/YYYY, e.g. 09/02/1989.;
         %let exiterror = 1;   
      %end;
   %end;
   %if %length(&TruncDate) %then %do;
      %let prdateerr = 0;
      %CHKDTA(date = &TruncDate);
      %if &prdateerr %then %do;
         %put ERROR: The truncation date must be of this form: DD/MM/YYYY, e.g. 09/02/1989.;
         %let exiterror = 1;   
      %end;
   %end;
   %if &exiterror %then %goto exit;
%end;

%let useDS = 0;
%if %sysfunc(findw(&domains, DS)) %then %do;
   %if %sysfunc(findw(%upcase(&DSVARS), DSDECOD)) and %sysfunc(findw(%upcase(&DSVARS), DSSTDTC)) %then %do;  
      %let useDS = 1;
         %LOADDATA(domlist = DS,mergesupp = 0,checkreview=0);
      
      data ds(keep = usubjid DSSTDTC);
         set &in_ds(where = ((index(upcase(DSDECOD), "RANDOMIZED") or index(upcase(DSDECOD), "RANDOMISED") or index(upcase(DSDECOD), upcase("&trRandomized"))) and DSSTDTC ne ""));
         by usubjid;
         if first.usubjid;
      run;            
      %nobsvars(ds);
      *** if no randomization information use other means ***;
      %if ^&nobs %then %let useDS = 0;    
   %end;
%end;

%if &useDS %then %do;
   data &in_adsl;
      merge &in_adsl(in = ina) ds(keep = usubjid dsstdtc);
      by usubjid;
      if ina;
   run;
%end;

%if %upcase(&SiteActiveFlag) = YES %then %do;
   %if %sysfunc(fileexist(&NotesDir.rbm_site.sas7bdat)) %then %do;
      %let _tid_ = %sysfunc(open(NotesLib.rbm_site));
      %let mind = %sysfunc(varnum(&_tid_,SiteActive));
      %let _tid_ = %sysfunc(close(&_tid_));     
      %if &mind %then %do;
         %NONNULLVARS(dataset = NotesLib.rbm_site , varlistname = TMPLIST);
         %if ^%sysfunc(findw(&TMPLIST, SITEACTIVE)) %then %let SiteActiveFlag = No;
      %end; 
   %end;
   %else %let SiteActiveFlag = No;
%end;

%if %upcase(&SiteActiveFlag) = YES %then %do;
   proc sort data = NotesLib.rbm_site out = rbm_site(keep = siteid SiteActive);
      by siteid;
   run;   
%end;

*** Get earliest site date in case site active date missing ***;
*** Say for a site with all screen failures ***;
%let useSV = 0;
%if %sysfunc(findw(&domains, SV)) %then %do;
   %CHECKPRES(domain = SV, var = SVSTDTC);
   %if %length(&SVSTDTC) %then %do;
      %let useSV = 1;
      %LOADDATA(domlist = SV,mergesupp = 0,checkreview=0);
      proc sort data = SV out = sv(keep = usubjid &SVSTDTC);
         by usubjid &SVSTDTC;
      run;
      
      data sv;
         set sv;
         by usubjid &SVSTDTC;
         if first.usubjid;
      run;
      
      data sv;
         merge &in_adsl(in = ina keep = usubjid siteid) sv;
         by usubjid;
         if ina;
      run;
      
      proc sort data = sv;
         by siteid;
      run;

      data sv(keep = siteid sitestsur);
         set sv;
         by siteid;
         %DATE9(var=SVSTDTC,return=sitestsur);        
         format sitestsur &IntDateFormat..;
         if first.siteid;
      run;      
   %end;
%end;

*determine the date variable;
%if &useDS %then %let DateVar = DSSTDTC;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), RANDDT)) %then %let DateVar = RANDDT;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), RFSTDTC)) %then %let DateVar = RFSTDTC;
%else %let DateVar = ;

%if ^%length(&DateVar) %then %do;
   %put ERROR: Randomization date is missing. Need one of DSSTDTC for;
   %put ERROR: records where DSDECOD = RANDOMIZED, or RANDDT or RFSTDTC;
   %let exiterror = 1;   
   %goto exit;
%end;

data &in_adsl;
   set &in_adsl;
   %if &datevar ^= RANDDT %then %do;
      %let datevar= %substr(&datevar,1,%sysevalf(%length(&datevar)-1));
      %DATE9(var=&datevar.c,return=&datevar);
      drop &datevar.c;
   %end;
run;

%let SiteLabel = SITEID;
%CheckLabel(&in_adsl,SiteLabel);

*exclude screen failures and missing dates;
data _nosf;  
   set &in_adsl;
   where upcase(arm) ^= "%upcase(&trScreenFailure)" and ^missing(&datevar);
   keep siteid usubjid arm &datevar;   
   format &datevar &IntDateFormat..;
run;

%if %upcase(&SiteActiveFlag) = YES %then %do;
   proc sort data=_nosf;
      by siteid;
   run;

   data _nosf;
      merge _nosf (in=ina) rbm_site;
      by siteid;
      if ina;
   run;
%end;

%let nmax=%sysevalf(%sysfunc(ceil(&TargetEnroll*1.1))); 

proc sort data=_nosf;
   by &datevar;
run;

*find overall first date for plotting purposes;
data _nosf;
   set _nosf end=eof;
   if _N_=1 then do;
      %if %upcase(&SiteActiveFlag) = YES %then %do;
         call symput ('firstdate',SiteActive);
      %end;
      %else %do;
         call symput ('firstdate',&datevar);
      %end;  
   end;
   %if ^%length(&CurrentTime) %then %do;
      if eof then call symput('CurrentTime',put(&datevar,&IntDateFormat..));
   %end;
run;

*formats;
%let firstdate= %sysfunc(strip(&firstdate));
%let first=%sysfunc(putn(&firstdate,&IntDateFormat..));

%if &IntDateFormat = DATE9 %then %do;
   %let cutoffdate=%sysfunc(putn("&CurrentTime."d,best.)); 
   %let lastdate=%sysfunc(putn("&TargetTime."d,best.)); 
%end;
%else %if &IntDateFormat = DDMMYY10 %then %do;
   %if %length(&CurrentTime) %then %let cutoffdate=%sysfunc(mdy(%scan(&CurrentTime, 2, "/"), %scan(&CurrentTime, 1, "/"), %scan(&CurrentTime, 3, "/")));
   %else %let cutoffdate=.;
   %if %length(&TargetTime) %then %let lastdate=%sysfunc(mdy(%scan(&TargetTime, 2, "/"), %scan(&TargetTime, 1, "/"), %scan(&TargetTime, 3, "/"))); 
   %else %let lastdate=.;
%end;

%let timediff = %sysevalf(&lastdate - &cutoffdate);

%let time=%eval(&lastdate-&firstdate+1); 
%let time1=%eval(&cutoffdate-&firstdate+1);
%let tmax=%sysevalf(%sysfunc(ceil(&lastdate*1.04)));
%let tmin=%sysevalf(%sysfunc(ceil(&firstdate*0.99)));
%let tmaxdate=%sysfunc(putn(&tmax,&IntDateFormat..));
%let tmindate=%sysfunc(putn(&tmin,&IntDateFormat..));

*check the value of TargetTime;
%if %sysevalf(&lastdate < &cutoffdate) %then %do;
   %put ERROR: The target date you chose is prior to the current date &CurrentTime.. Please pick a target date that is later than the current date or choose an earlier current date;
   %let exiterror = 1;
   %goto exit;
%end;

*check if the value of user-specified start date is within an reasonable range;
%if &IntDateFormat = DATE9 %then %do;
   %let start=%sysfunc(putn("&TruncDate"d,best.));
%end;
%else %if &IntDateFormat = DDMMYY10 %then %do;
   %if %length(&TruncDate) %then %let start=%sysfunc(mdy(%scan(&TruncDate, 2, "/"), %scan(&TruncDate, 1, "/"), %scan(&TruncDate, 3, "/")));
   %else %let start=.; 
%end;

%if (%upcase(&NewStart) = NO) %then %do;
   %let start=&firstdate;
   
   %if &IntDateFormat = DATE9 %then %do;
      %let truncdate=%sysfunc(putn(&start,&IntDateFormat..));
   %end;
   %else %if &IntDateFormat = DDMMYY10 %then %do;
      %let truncdate=%sysfunc(putn(&start,&IntDateFormat..));
   
      /* %if %length(&start) %then %let truncdate=%sysfunc(mdy(%scan(&start, 2, "/"), %scan(&start, 1, "/"), %scan(&start, 3, "/"))); 
      %else %let truncdate=.; */
   %end;
   %let ErrorM= ;
%end;
%else %if %sysevalf(&start < &firstdate) %then %do;
   %let start=&firstdate;
   %let trNoteThetruncationprior = %nrbquote(%LocaleGet(NoteThetruncationprior_Text)); 
   %let trNoteThetruncationpriorX = %nrbquote(%LocaleFill(&trNoteThetruncationprior, f1=%nrbquote(&first)));
   %let ErrorM= &trNoteThetruncationpriorX;
%end;
%else %if %sysevalf(&start ge &cutoffdate) %then %do;
   %let ErrorM= ;
   %put ERROR: The date you entered is later than the current date, &CurrentTime.. No data is selected. Please pick a date on or between &first and &CurrentTime..;
   %let exiterror = 1;
   %goto exit;
%end;
%else %do;
   %let ErrorM= ;
%end;

*calculate recruitment days and rates;
data _nt;
   set _nosf;
   retain number 0;
   number+1;
   time = &datevar - &firstdate + 1;
   rate=number/time;
   label number="&trActualEnrollment" time="&trNumberofdaysuntilnow";
   if &datevar = &Start then call symput('StartNum', number);
run;

*if the truncated date is not in the dateset;
%if ^%length(&StartNum) %then %do;
   proc sort data=_nt;
      by &datevar descending number;
   run;
   
   data _startnum;
      set _nt;
      where &datevar ge &start;
      if _N_=1 then do;
         call symput ('StartNum', strip(number));
         call symput ('ObsDate', strip(&datevar));
      end;
   run;
   %let obsdate=%sysfunc(putn(&ObsDate,&IntDateFormat..));
   %if %upcase(&NewStart) = YES %then %do;   
      %let trNoteThetruncation = %nrbquote(%LocaleGet(NoteThetruncation_Text));
      %let trNoteThetruncationX = %nrbquote(%LocaleFill(&trNoteThetruncation, f1=%nrbquote(&TruncDate)));
      %let trTheearliestdatewithin = %nrbquote(%LocaleGet(Theearliestdatewithin_Text));
      %let trTheearliestdatewithinX = %nrbquote(%LocaleFill(&trTheearliestdatewithin, f1=%nrbquote(&ObsDate), f2=%nrbquote(&startnum)));   
      %let ErrorM=&trNoteThetruncationX &trTheearliestdatewithinX;
   %end;
   %else %let ErrorM=;
%end;
%else %do;
   %let startnum=%sysfunc(trim(&startnum));
%end;

*take the subset, from the very beginning of recruitment up to T1;
*this is to account for current dates that may be early to see fit of prediction with later data;
*this is for table to show all enrollment up to T1;
data _sub;
   set _nt;
   where &datevar le &cutoffdate;
run;

*this is K1. total patients recruited;
proc sql noprint;
   select count(*) into :num1
   from _sub;
quit;

*K1;
%let num1=%sysfunc(strip(&num1));
*K2;
%let num2=%eval(&TargetEnroll-&num1);
*TargetEnroll is K;

%if %sysevalf(&TargetEnroll <= &num1) %then %do;
   %put ERROR: Target enrollment of &TargetEnroll has already been met. &num1 patients are currently enrolled.;
   %let exiterror = 1;
   %goto exit;
%end;

*get a list of all sites;
proc sort data=&in_adsl out= adsl_sort;
   by siteid &datevar;
run;

%if %upcase(&SiteActiveFlag) = YES %then %do;
data adsl_sort;
   merge adsl_sort(in=ina) rbm_site;
   by siteid;
   if ina;
run;
%end;

data adsl_site; 
   set adsl_sort;
   by siteid;
   retain opendate 0;
   if first.siteid then do;
      %if %upcase(&SiteActiveFlag) = YES %then %do;
         opendate=SiteActive;
      %end;
      %else %do;
         opendate=&datevar;
      %end;
      end;
   if last.siteid then output;
run;

*choose the sites that start prior to current date;
data adsl_site;
   set adsl_site;
   where opendate le &cutoffdate;
run;

*count the number of sites that are open prior to current date;
proc sql noprint;
   create table _sitelist as
   select distinct siteid, opendate
      into :SiteList separated by ' '
      from adsl_site
      order by siteid;
      %let ncenter = &sqlobs;
quit;

proc sort data=_sub;
   by siteid &datevar;
run;

*take the subset, from truncation date up to T1;
*data used for estimation and prediction;
data _subsub;   
   set _nt;
   where &start le &datevar le &cutoffdate;
   start=&start;
   label start="&trStartingDateofEstandPred";
run;

proc sort data=_subsub;
   by siteid &datevar;
run;

*find first and last date for each center;
data _first;
   set _subsub;
   by siteid;
   retain count 0 firstdate 0;
   if first.siteid then do;
    %if %upcase(&SiteActiveFlag) = YES %then %do;
      firstdate=SiteActive;
    %end;
    %else %do;
      firstdate=&datevar;
    %end;
      count=0;
   end;
   count+1;
   if last.siteid then do;
      output;
   end;
   label count="&trNumberofpatientsrecruitedstart"
         firstdate="&trFirstrecruitmentdatestart" ;
run;

*find first and last date for each center;
data _first2;
   set _sub;
   by siteid;
   retain count 0 firstdate 0;
   if first.siteid then do;
       %if %upcase(&SiteActiveFlag) = YES %then %do;
         firstdate=SiteActive;
       %end;
       %else %do;
         firstdate=&datevar;
       %end;
      count=0;
   end;
   count+1;
   if last.siteid then do;
      output;
   end;
   label count="&trNumberofpatientsrecruitedbegin"
         firstdate="&trFirstrecruitmentdatebegin" ;
run; 

*find tau, the duration of recruitment up to T1;
data _bysite;
   set _first2;
   %if %upcase(&SiteActiveFlag) = YES %then %do;
      variab = SiteActive;
   %end;
   %else %do;
      variab = firstdate;
   %end;
   truncd=input("&TruncDate",&IntDateFormat..); 
   if (variab le truncd) then tau=&cutoffdate-truncd+1;
   if (variab > truncd) then tau=&cutoffdate-variab+1;
   drop number time rate usubjid arm &datevar;
   label tau="&trDurationofrecruitmentstart";
run;

*merge count with tau;
data _countau;
   merge _bysite(keep=siteid tau) _first(in=ina);
   by siteid;
   if ina;
run;

*find tau, the duration of recruitment up to T1;
data _bysite2;
   set _first2;
   tau=&cutoffdate-firstdate+1;
   drop number time rate usubjid arm &datevar;
   label tau="&trDurationofrecruitmentbegin";
run;

%if &useSV %then %do;
   data _sitelist;
      merge _sitelist(in=ina) sv;
      by siteid;
      if ina;
   run;
%end;

*combine all sites together, by adding centers that have been initiated but not yet recruited people;
data _allsites;
   merge _sitelist(in=sf) _countau(in=nosf) ;
   by siteid;
   if (sf=1 and nosf=0) then do;
      count=0;
      %if %upcase(&SiteActiveFlag) = YES %then %do;
         variab=SiteActive;
      %end;
      %else %do;
         variab=&datevar;
         *** if site has all screen fails and site active date not available use earliest visit date within site as surrogate (sitestsur) ***;
         %if &useSV %then %do;
            if missing(variab) then variab = sitestsur;
         %end;
      %end;
      truncd=input("&TruncDate",&IntDateFormat..);
      if (variab le truncd) then tau=&cutoffdate-truncd+1;
      if (variab > truncd) then tau=&cutoffdate-variab+1;
      label tau="&trDurationofrecruitmentstart";
   end;
run;

*combine all sites together, by adding centers that have been initiated but not yet recruited people;
data _allsites2;
   merge _sitelist(in=sf) _bysite2(in=nosf);
   by siteid;
   if (sf=1 and nosf=0) then do;; 
      count=0;
      tau=&cutoffdate-opendate+1;
      label tau="&trDurationofrecruitmentbegin";
   end;
run;

*parameter estimation;
title 'Negative Binomial';
proc nlmixed data=_allsites;
   ods output AdditionalEstimates=AddEstimates;
   ods exclude specifications dimensions convergencestatus fitstatistics IterHistory ParameterEstimates parameters addestimates additionalestimates;
   parms alpha 1 m 1;
   p= alpha/(alpha+m*tau);
   beta = alpha/m;
   r=alpha;
   model count ~ negbin(r,p);
   estimate 'alpha' alpha;
   estimate 'm' m;
   estimate 'beta' beta;
run;

%if %index(%QUOTE(%UPCASE(&SYSWARNINGTEXT)), HESSIAN) %then %do;
   %put ERROR: The final Hessian matrix is not positive definite and therefore the estimated covariance matrix is not full rank and may be unreliable.;
   %put ERROR: Patient Recruitment is exiting...;
   %let exiterror = 1;
   %goto exit;
%end;

proc transpose data=addestimates out=est_am(rename=(col1=alpha col2=m col3=beta _name_=name) drop=_label_);
run;

*only pick the estimates;
data est_am;
   set est_am;
   n=&TargetEnroll;
   t=&time;
   where name='Estimate';
   drop name;
run;

*put enrollment data together with the estimates;
proc sql;
   create table _pred as
      select n.*, a.alpha as alphaest, a.m as mest, a.beta as betaest, a.n as n, a.t as t
         from _allsites n, est_am a;
quit;


*add recruitment rate;
data _predict;
   set _allsites2;
   rateday=count/tau;
   rateweek=rateday*7;
   keep siteid count tau rateday rateweek;
   label rateday="&trRecruitmentRateday" rateweek="&trRecruitmentRateweek";
   format rateday rateweek 5.3;
run;

*get rid of datasets that are no longer needed;
proc datasets nolist;
   delete _sitelist _bysite addestimates;
quit;

*bayesian simulation of the denominator;
data _bayesian;
   set _pred;
   %if &ncenter > 19 %then %do;
      call streaminit(&seednum);
      para1=alphaest+count;
      para2=1/(betaest+tau);
      do i = 1 to &nsim;
         x=para2*rand("Gamma", para1);
         output;
      end;
   %end;
   %else %if &ncenter < 20 %then %do;
      do i = 1 to &nsim;
         x=count/tau;
         output;
      end;
   %end;
run;

proc sort data=_bayesian;
   by i;
run;

*take the sum of denominators;
data _denom;
   set _bayesian;
   by i;
   retain denom 0;
   if first.i then denom=0;
   %if &ncenter > 19 %then %do;   
      denom+x;
      label x="&trGammasimulationfor";
   %end;
   %else %if &ncenter < 20 %then %do;
      denom+x;
      label x="&trMaxLikeEst";
   %end;
   if last.i then output;
   label i="&trSimulationIndex" denom="&trSumofgamma";
run;

*find the list of the number of subjects left to enroll by tick;
data _num;
   do j_k2=1 to &num2 by &tick;
      output;
   end;
   label j_k2="&trNumberofsubjectsleft";
run;

*add k2 to the list;
data _num;
   set _num end=last;
   output _num;
   if last and j_k2^=&num2 then do;
      j_k2=&num2;
      output _num;
   end;
run;

*simulate the numerator;
data _sim;
   set _num;
   call streaminit(&seednum);
   do i=1 to &nsim;
      y=rand("Gamma",j_k2);
      output;
   end;
   label y="&trGammasimulation";
run;

proc sort data=_sim;
   by i;
run;

data _numdenom;
   merge _denom _sim ;
   by i;
run;

proc sort data=_numdenom;
   by j_k2;
run;

*take the ratio to get predicted remaining time;
data _t;
   set _numdenom;
   pt=y/denom;
   label pt="&trPredictedtimeBayes";
run;

*calculate the mean and percentiles;
proc univariate data=_t cibasic noprint;
   by j_k2;
   var pt;
   output out=_pctl mean=mean pctlpts=2.5 97.5 pctlpre=P;
run;

*convert days to dates for the predicted values;
data _plot;
   set _pctl end=last;
   totmean=ceil(mean+&time1+&firstdate-1);
   totp2_5=ceil(p2_5+&time1+&firstdate-1);
   totp97_5=ceil(p97_5+&time1+&firstdate-1);
   totj_k2=j_k2+&num1;
   
   /* fill in the gap between k1 and 1+k1 */
   if last then do;
      output;
      totmean=&cutoffdate;
      j_k2=0;
      totj_k2=&num1;
      totp2_5=&cutoffdate;
      totp97_5=&cutoffdate;
   end;
   output;
   *format totmean totp2_5 totp97_5 finalmean finalp2_5 finalp97_5 &IntDateFormat..;
   format totmean totp2_5 totp97_5 &IntDateFormat..;
run;

proc sort data=_plot;
   by j_k2;
run;

*convert days to dates for the actual data;
data _actualpred;
   set _plot _nt;
   time =time+&firstdate-1;
   format time &IntDateFormat..;
run;

data _jmp;
   set _actualpred;
   if missing(totmean) then do;
      data="&trActualEnrollment";
      time=&datevar;
      output;
   end;
   if ^missing(totmean) then do;
      data="&trPredictedMean";
      time=totmean;
      number=totj_k2;
      output; 
      data="&trPredicted25";
      time=totp2_5;
      number=totj_k2;
      output;
      data="&trPredicted975";
      time=totp97_5;
      number=totj_k2;
      output;
   end;
   drop j_k2 mean P2_5 p97_5 usubjid siteid arm totmean totp2_5 totp97_5;
run;

*find whether the predicted remaining time of each simulated value will miss the deadline;
data _exceed;
   set _t(where = (j_k2 = &num2));
   *retain sumlate 0 total 0;
   tildet=ceil(pt+&time1+&firstdate-1);
   if tildet ge &lastdate then late=1;
   else late=0;
run;

*calculate the mean of LATE;
proc univariate data=_exceed cibasic noprint;
   var late;
   output out=_avglate mean=mean;
run;

data _null_;
   set _avglate;
   call symput("pobs",mean); 
run;

data _null_;
   set est_am;
   call symput("alphaest",alpha);
   call symput("mest",m);
run;

/* Output Datasets */
%PrepOutDS(PatRecruit_OutData,OutDataName,&Outname.PatRectab);
%let includeds = &includeds &OutDataName..sas7bdat,;

data &PatRecruit_OutData;
   set _predict;
run;

%let PatRecruit_OutData2 = ;
%PrepOutDS(PatRecruit_OutData2,OutDataName2,&Outname.PatRecfig);
%let includeds = &includeds &OutDataName2..sas7bdat,;

data &PatRecruit_OutData2;
   set _jmp;
run;

*run the second macro;
%AdaptAdjust

* create JSL file;
%PrepOutFile(PatRecruitScript,PatRecruitScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &PatRecruitScript;
%let FullAPName=%str(Patient Recruitment);

* put JSL statements;
data _null_;
   file "&PatRecruitScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";   
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList  = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:NoteDomain = """";";
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReports]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to Include Subjects: "" || objRefNS:WhereState);";   
      
   *** open datasets ***;
   put "objRefNS:PatRecFig=open(""&ClientOutPath.&OutDataName2..sas7bdat"",invisible);";
   put "objRefNS:PatRecTab=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   
   *** Tab 1 ***;
   *** insert figures and tables ***;
   put "objRefNS:PatRec=Function({},{default local},;";
   put " objRefNS:PatRecFig << select where ( :data == ""&trPredicted25"" );";
   put " objRefNS:sdt = objRefNS:PatRecFig << subset (";
   put "    Output Table Name( ""&trLowerPercentile"" ), Selected Rows(1), invisible";
   put " );";
   put " objRefNS:sdt_sort = objRefNS:sdt << sort (By( :&trNumberofdaysuntilnow ), Order(Ascending) );";
   put " col = Column(objRefNS:sdt_sort, ""&trNumberofdaysuntilnow"");";
   put " tl = col << GetAsMatrix;";
   put " col = Column(objRefNS:sdt_sort, ""&trActualEnrollment"");";
   put " nl = col << GetAsMatrix;";
   put " close (objRefNS:sdt, no save); ";
   put " close (objRefNS:sdt_sort, no save);";

   put " objRefNS:PatRecFig << select where ( :data == ""&trPredicted975"" );";
   put " objRefNS:sdt = objRefNS:PatRecFig << subset (";
   put "    Output Table Name( ""Upper Percentile"" ), Selected Rows(1), invisible";
   put " );";
   put " objRefNS:sdt_sort = objRefNS:sdt << sort (By( :&trNumberofdaysuntilnow), Order(Descending) );";
   put " col = Column(objRefNS:sdt_sort, ""&trNumberofdaysuntilnow"");";
   put " tu = col << GetAsMatrix;"; 
   put " col = Column(objRefNS:sdt_sort, ""&trActualEnrollment"");";
   put " nu = col << GetAsMatrix;";
   put " close (objRefNS:sdt, no save);";
   put " close (objRefNS:sdt_sort, no save);";
   put " lower = tl || nl;";
   put " upper = tu || nu;";
   put " all = lower |/ upper;";
   put " objRefNS:xall = all[0,1];";
   put " objRefNS:yall = all[0,2];";
   
   put " objRefNS:dt = objRefNS:PatRecFig << select where ((num(""&TruncDate"") <  :&trNumberofdaysuntilnow < num(""&CurrentTime""))& (:data == ""&trActualEnrollment""));";
   put " objRefNS:start = objRefNS:dt << subset (Output Table Name(""&trStartDate""), Selected Rows(1), invisible);";
   put " col = Column(objRefNS:start, ""&trNumberofdaysuntilnow"");";
   put " ts = col << GetAsMatrix;";
   put " col = Column(objRefNS:start, ""&trActualEnrollment"");";
   put " ns = col << GetAsMatrix;"; 
   put " close (objRefNS:start, no save);";
   put " alls = ts || ns;";
   put " all2 = num(""&CurrentTime"")||[0];";
   put " all1 = num(""&CurrentTime"")||[&num1];";
   put " all3 = num(""&TruncDate"")||[0];";
   put " allstart = alls |/ all1 |/ all2 |/ all3;";
   put " objRefNS:xstart = allstart[0,1];";
   put " objRefNS:ystart = allstart[0,2];";

   put " objRefNS:PatRecFig << clear select; ";   
   put " VlistBox(";
   put " objRefNS:Text1 = TextBox(locale:fill(locale:get(""Theprobabilityofexceedingthetargetdateis_Text""), {""%sysfunc(round(100*&pobs,0.01))""})||""  ""||";
   %if (&pobs2 le &Prob1) %then %do;
      put "  locale:get(""Adaptiveadjustmentwillbeperformed_Text""),<<Set Wrap(1000),<< Font Color(5)),";
   %end;
   %else %if (&pobs2 > &Prob1) %then %do;
      put "  locale:get(""Adaptiveadjustmentwillnotbeperformed_Text""),<<Set Wrap(1000),<< Font Color(5)),";
   %end;
   put " TextBox(""""),";
   put " objRefNS:PredPlot = objRefNS:PatRecFig << Graph Builder(";
   put "   Auto Stretching(0),";
   put "   Automatic Recalc(1),";   
   put "   Show Control Panel( 0 ),";      
   put " Variables(";
   put "    X( :&trNumberofdaysuntilnow ),";
   put "    Y( :&trActualEnrollment ),";
   put "    Overlay( :data )";
   put " ),";
   put " Elements( Line( X, Y, Legend( 7 ) ) ),";
   put " SendToReport(";
   put "       Dispatch(";
   put "       {},";
   put "       ""Graph Builder"",";
   put "       OutlineBox,";
   put "       {Set Title( ""&trPredModPatRecr"" )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""&trNumberofdaysuntilnow"",";
   put "       ScaleBox,";
   %if &IntDateFormat = DATE9 %then %do;
      put "       {Format( ""ddMonyyyy"", 10), Min( num(""&tmindate"") ), Max( num(""&tmaxdate"") ), Label Row( Inside Ticks( 0 ) )}";
   %end;
   %else %if &IntDateFormat = DDMMYY10 %then %do;
      put "       {Format( ""D/M/Y"", 10), Min( num(""&tmindate"") ), Max( num(""&tmaxdate"") ), Label Row( Inside Ticks( 0 ) )}";
   %end;
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""&trActualEnrollment"",";
   put "       ScaleBox,";
   put "       {Format( ""Best"", Use thousands separator( 1 ), 12 ),";
   put "       Min( -20 ), Max( &nmax ),";
   put "       Minor Ticks( 1 ), Label Row(";
   put "       {Show Major Grid( 1 ), Inside Ticks( 0 ),";
   put "          Major Grid Line Color( 32 ), Major Grid Line Style( 3 )}";
   put "       )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""400"",";
   put "       ScaleBox,";
   put "       {Legend Model(";
   put "          6,";
   put "          Properties( 1, {Line Color( 19 ), Line Style( ""DashDot"" )} ),";
   put "          Properties( 2, {Line Color( 19 ), Line Style( ""DashDotDot"" )} ),";
   put "          Properties( 3, {Line Color( 19 )} )";
   put "       )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""graph title"",";
   put "       TextEditBox,";
   put "       {Set Text( "" "" )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""400"",";
   put "       LegendBox,";
   put "       {Set Title( "" "" ), Position( {0, -1, 1, -1, 2, -1, 3, -1} )}";
   put "    ),";
   put "    Dispatch( {}, ""X title"", TextEditBox, {Set Text( ""&trRecruitmentDate"" )} ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""Y title"",";
   put "       TextEditBox,";
   put "       {Set Text( ""&trNumberPatientsRecr"" )}";
   put "    ),";
   put "    Dispatch(";
   put "       {},";
   put "       ""Graph Builder"",";
   put "       FrameBox,";
   put "       {Add Graphics Script(";
   put "          Description( """" ),";
   put "          Transparency( 1 );";
   put "          Pen Color( ""red"" );";
   put "          Fill Color( ""red"" );";
   put "          Transparency( 0.2 );";
   put "          Polygon(objRefNS:xall, objRefNS:yall);";
   put "          Fill Color(""green"");";
   put "          Transparency( 0.2 );";
   put "          Polygon(objRefNS:xstart,objRefNS:ystart);";
   put "          Pen Color( ""black"" );";
   put "          Transparency( 0.5 );";
   put "          Line Style( 2 );";
   put "          Line( {num(""&CurrentTime""), 0}, {num(""&CurrentTime""), &num1} );";
   put "          Line Style( 2 );";
   put "          Line( {0, &num1}, {num(""&CurrentTime""), &num1} );";
   put "          Line Style( 0 );";
   put "          Line( {0, &TargetEnroll}, {num(""&tmaxdate""), &TargetEnroll} );";
   put "          Line Style( 0 );";
   put "          Line( {num(""&TargetTime""), 0}, {num(""&TargetTime""), &TargetEnroll} );";
   put "          Line Style( 2 );";
   put "          Line( {num(""&TruncDate""), 0}, {num(""&TruncDate""), &StartNum} );";
   put "          Transparency( 1 );";
   put "          Text Size( 8 );";
   put "          Text Color( ""blue"" );";
   put "          Text Size( 8 );";
   put "          Text( {num(""&CurrentTime""), 0*&nmax}, ""&trCurrentDate=&CurrentTime"" );";
   put "          Text Size( 8 );";
   put "          Text( {1.0005*num(""&tmindate""), &num1}, ""&trCurrentRecruit=&num1"" );";
   put "          Text Size( 8 );";
   put "          Text( {num(""&TargetTime""), 0.03*&nmax}, ""&trTargetDate=&TargetTime"" );";
   put "          Text Size( 8 );";
   put "          Text( {1.0005*num(""&tmindate""), &TargetEnroll}, ""&trTargetRecruit=&TargetEnroll"" );";
   put "          Text Size( 8 );";
   put "          Text( {num(""&TruncDate""), 0.03*&nmax}, ""&trStartDate=&TruncDate"" );";
   put "       )}";
   put "    )";
   put " )  ";
   put ");";
   put "      , "; 
   put " Text Box("" &ErrorM "",<<Set Wrap(700),<< Font Color(5)),";
   put " Text Box(""""),";
   put " OutlineBox(locale:get(""&trCurrentRecruitmentbySite""),";
   put "    objRefNS:PredTable = Table Box(";
   put " Data Table Col Box( Column(objRefNS:PatRecTab, ""&SiteLabel"") ),";
   put " Data Table Col Box( Column(objRefNS:PatRecTab, ""&trNumberofpatientsrecruitedbegin"") ),";
   put " Data Table Col Box( Column(objRefNS:PatRecTab, ""&trDurationofrecruitmentbegin"")),";
   put "    Data Table Col Box( Column(objRefNS:PatRecTab, ""&trRecruitmentRateday"")),";
   put "    Data Table Col Box( Column(objRefNS:PatRecTab, ""&trRecruitmentRateweek"")),";
   put " << Table Style(""Heading Separator""), << Set Shade Alternate Rows( 1 )))";
   put "));";  
   put "InsertInto(objRefNS:DataList,{{""objRefNS:PatRecFig"", ""objRefNS:PatRecTab""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trPredModPatRecr"",""&trCurrentRecruitmentbySite""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""&trPatientRecruitment"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:PatRec()));";
   put "InsertInto(objRefNS:TabAfterCodeList, """");";
   put "InsertInto(objRefNS:CreateReportList,{{""objRefNS:Text1"",""objRefNS:PredPlot"", ""objRefNS:PredTable""}});"; 
   put "InsertInto(objRefNS:TabOpenList,1);";

   *** Tab 2 ***;
   %if (&pobs2 > &Prob1) %then %do;
   %end;
   %else %if (&pobs2 le &Prob1) and (&timediff > &delay) %then %do;
      put "objRefNS:PatRecSite=open(""&ClientOutPath.&OutDataName3..sas7bdat"",invisible);";
      put "objRefNS:PatRecSiteThing=open(""&ClientOutPath.&OutDataName4..sas7bdat"",invisible);";
      
      *** insert figures and tables ***;
      put "objRefNS:PatRecAdjust=Function({},{default local},"; 
      put "VlistBox(";
      put "If(NRows(objRefNS:PatRecSiteThing) > 1,";
      put " objRefNS:AdjustFig = objRefNS:PatRecSite << Graph Builder(";
      put "   Auto Stretching(0),";
      put " Automatic Recalc(1),";  
      put "   Show Control Panel( 0 ),";  
      put " Variables(";   
      put "    X( :&trNumberofnewcenters ),";
      put "    Y( :&trProbabilityofmeetingtargetdate)";
      put " ),";
      put " Elements( Line( X, Y, Legend( 25 ) ) )";
      put "   ,SendToReport(";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       OutlineBox,";
      put "       {Set Title( ""&trProbabilityMeetingDeadline"" )}";
      put "    ),";
      put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       FrameBox,";
      put "       {Add Graphics Script(";
      put "          Description( """" ),";
      put "          Pen Color( ""black"" );";
      put "          Transparency( 0.5 );";
      put "          Line Style( 2 );";
      put "          Line( {&MCenter, -0.2}, {&MCenter, 1} );";
      put "          Line Style( 2 );";
      put "          Line( {-0.2, &Prob2}, {&MCenter, &Prob2} );";
      put "             Transparency( 1 );";
      put "          Text Color( ""blue"" );";
      put "          Text Size( 8 );";
      put "          Text(Center Justified, {&Mcenter, -0.16}, ""&trNeed &MCenter"" );";
      put "          Text(Center Justified, {&Mcenter, -0.19}, ""&trNewCenters"" );";
      put "          Text Size( 8 );";
      put "          Text({0.5, &Prob2}, ""&trMinimum &trProbabilityMeetingDeadline=&Prob2"");";
      put "       )}";
      put "    )";
      put " )";
      put "    ))";
      put " , "; 
      put " Text Box(""""),";
      put "If(NRows(objRefNS:PatRecSiteThing) > 1,";
      put " objRefNS:AdjustFig2 = objRefNS:PatRecSite << Graph Builder(";
      put "   Auto Stretching(0),";
      put " Automatic Recalc(1),";  
      put "   Show Control Panel( 0 ),";  
      put " Variables(";
      put "    X( :&trDateRecruitComp ),";
      put "    Y( :&trNumberofnewcenters ),";
      put "    Overlay( :group )";
      put " ),";
      put " Elements( Line( X, Y, Legend( 6 ) ) ),";
      put " SendToReport(";
      put "   Dispatch(";
      put "    {},";
      put "    ""Graph Builder"",";
      put "    OutlineBox,";
      put "    {Set Title( ""&trPredictedDateRecrComp"" )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""400"",";
      put "       ScaleBox,";
      put "       {Legend Model(";
      put "          6,";
      put "          Properties( 0, {Line Color( 19 ), Line Style( ""DashDot"" )} ),";
      put "          Properties( 1, {Line Color( 19 ), Line Style( ""DashDotDot"" )} ),";
      put "          Properties( 2, {Line Color( 19 )} )";
      put "       )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""&trNumberofnewcenters"",";
      put "       ScaleBox,";
      put "       {Add Ref Line( &MCenter, ""Dashed"", ""Gray"", """", 1 )}";
      put "    ),";
      put "    Dispatch(";
      put "       {},";
      put "       ""graph title"",";
      put "       TextEditBox,";
      put "       {Set Text( "" "" )}";
      put "    ),";
      put "    Dispatch( {}, ""400"", LegendBox, {Set Title( """" )} ),";
      put "       Dispatch(";
      put "       {},";
      put "       ""Graph Builder"",";
      put "       FrameBox,";
      put "       {Add Graphics Script(";
      put "          Description( """" ),";
      put "          Pen Color( ""black"" );";
      put "          Transparency( 0.5 );";
      put "          Line Style( 2 );";
      put "          Line( {num(""&TargetTime""), 0}, {num(""&TargetTime""), &MCenter} );";
      put "             Transparency( 1 );";
      put "          Text Color( ""blue"" );";
      put "          Text Size( 8 );";
      put "          Text({num(""&TargetTime""), &Mcenter}, ""&trNumberNewCentersNeeded=&MCenter"" );";
      put "          Text Size( 8 );";
      put "          Text({num(""&TargetTime""), 0}, ""&trTargetDate = &TargetTime"");";
      put "       )}";
      put "    )";
      put " )";
      put " ));";
      put " , "; 
      put " Text Box(""""),";
      put " OutlineBox(locale:get(""Adaptive Adjustment""),";
      put "    objRefNS:AdjustTab = Data Table Box(objRefNS:PatRecSiteThing,";
      put " << Table Style(""Heading Separator""), << Set Shade Alternate Rows( 1 ))";
      put ")));";

      put "InsertInto(objRefNS:DataList,{{""objRefNS:PatRecSite"",""objRefNS:PatRecSiteThing""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trAdaptiveAdjustment &trPlots"",""&trAdaptiveAdjustment &trTable""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""&trAdaptiveAdjustment"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:PatRecAdjust()));";
      put "InsertInto(objRefNS:TabAfterCodeList, """");";
      put "If(NRows(objRefNS:PatRecSiteThing) > 1,";
      put "   InsertInto(objRefNS:CreateReportList,{{""objRefNS:AdjustTab"",""objRefNS:AdjustFig"",""objRefNS:AdjustFig2""}});";
      put "   , //else";
      put "   InsertInto(objRefNS:CreateReportList,{{""objRefNS:AdjustTab""}});";
      put " ); //end of if function";
      put "InsertInto(objRefNS:TabOpenList,1);";
   %end;
   %else %if (&pobs2 le &Prob1) and (&timediff <= &delay) %then %do;      
      *** insert figures and tables ***;
      put "objRefNS:PatRecAdjust=Function({},{default local},"; 
      put "VlistBox(";
      put "   objRefNS:Text1 = TextBox(locale:fill(locale:get(""ThetargetdateisinXdays_Text""), {""&timediff"", ""&delay""})||""  ""||";
      put "  locale:get(""Adaptiveadjustmentwillnotbeperformed_Text""),<<Set Wrap(1000),<< Font Color(5)),";
      put "));";
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""&trAdaptiveAdjustment"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:PatRecAdjust()));";
      put "InsertInto(objRefNS:TabAfterCodeList, """");";
      put "InsertInto(objRefNS:CreateReportList,{{""objRefNS:Text1""}});";
      put "InsertInto(objRefNS:TabOpenList,1);";
   %end;
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &PatRecruitScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend PatRecruit;

%macro AdaptAdjust;
   %let pobs2 = %sysevalf(1-&pobs);
   
   %if (&pobs2 > &Prob1) %then %do;
   %end;
   %else %if (&pobs2 le &Prob1) and (&timediff > &delay) %then %do;
      data _k3;
         call streaminit(&seednum);
         r=&ncenter.*&alphaest;
         p=&alphaest/(&alphaest+&mest*&delay);
         do i = 1 to &nsim;
            priortoadd=rand("NEGBINOMIAL", p,r);
            k3=&num2-priortoadd; 
            if k3 <0 then k3=0;
            output;
         end;
         *label priortoadd='Poisson_Lambda(d)~NegBin(p,r)';
      run;  
      
      *** simulate to get numerator of T(M) ***;
      data _simk3;
         set _k3;
         call streaminit(&seednum);
         num=rand("Gamma",k3);
         label num='Gamma(k3,1)';
      run;
      
      *** start the loop with one additional center ***;
      %let pobsm=0;
      %do %until (&pobsm ge &Prob2);
         *** simulate to get total rate of current data (Bayesian or ML) ***;
         data _simplus;
            call streaminit(&seednum);
            para1=&alphaest.*&MCenter;
            para2=&mest./&alphaest.;
            do i=1 to &nsim;
               plus=para2*rand("Gamma",para1);
               output;
            end;
            label plus="&trTotalrateaddMcenters";
         run;

         *** put all pieces together to get T(M) ***;
         data _tm;
            merge _simk3(keep=i num) _denom(keep=i denom) _simplus(keep=i plus);
            by i;
            tm=num/(denom+plus)+&delay;
            tildetm=ceil(tm+&cutoffdate);
            if (tildetm le &lastdate) then meet=1;
            else if (tildetm > &lastdate) then meet=0;
         run;

         *** output the probability of meeting the target time ***;
         proc univariate data=_tm cibasic noprint;
            var tildetm;
            output out=_avgtildetm mean=mean pctlpts=2.5 97.5 pctlpre=P;
         run;

         proc univariate data=_tm cibasic noprint;
            var meet;
            output out=_avgmeet mean=prob;
         run;

         data thing;
          retain m prob mean p2_5 p97_5;
          merge _avgmeet _avgtildetm;
          m = &MCenter;       
          target=&lastdate;
          mean=ceil(mean);
          p2_5=ceil(p2_5);
          p97_5=ceil(p97_5);
          label m="&trNumberofnewcenters" prob="&trProbabilityofmeetingtargetdate" 
                mean="&trAveragepredicteddate" 
                p2_5="&trPercentile25preddate" 
                p97_5="&trPercentile975preddate"
                target="&trTargetDate"; 
          format  prob percent9.2 mean p2_5 p97_5 target &IntDateFormat..;
        run; 

        %if &MCenter = 1 %then %do;              
         data allthing;
               set thing;
            *format &tmax &IntDateFormat..;
         run;
        %end;
        %else %do;
         proc append force base = allthing data = thing;
         run;
        %end;

        proc datasets nolist;
             delete thing;
        quit;

        data _null_;
         set _avgmeet;
         call symput("pobsm", left(trim(prob)));
        run;

        %let MCenter=%eval(&MCenter+1);
      %end;   
      
      %let MCenter=%eval(&MCenter-1);
      
      data _jmp2;
         set allthing;
         group="&trPredictedMean";
         predate=mean;
         output;
         group="&trPredicted25";
         predate=p2_5;
         output;
         group="&trPredicted975";
         predate=p97_5;
         output;
         label predate="&trDateRecruitComp";
         format predate &IntDateFormat..;
         drop mean p2_5 p97_5;
      run;
      
      *** Output Datasets ***;
      %let PatRecruit_OutData3 = ;
      %PrepOutDS(PatRecruit_OutData3,OutDataName3,&Outname.PatRecsite);
      %let includeds = &includeds &OutDataName3..sas7bdat,;

      data &PatRecruit_OutData3;
         set _jmp2;
      run;
      
      %let PatRecruit_OutData4 = ;
      %PrepOutDS(PatRecruit_OutData4,OutDataName4,&Outname.PatRecSiteThing);
      %let includeds = &includeds &OutDataName4..sas7bdat,;

      data &PatRecruit_OutData4;
         set allthing;
      run;
   %end;
%mend AdaptAdjust;

* run the macro;
%PatRecruit