/*
JMP Clinical Analytical Process

Name: Review Status Distribution
Authors: Russ Wolfinger, SAS Institute Inc.

Description: 
   This process shows the distribution of the review status of subjects in a study.
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath TrtUse TrtVar DesignByVars SubjectFilter WhereStmt OutPath ServerOutPath 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
ReviewStatusDist_OutData MacroPath ReviewStatusDistScript MergeSuppDom;

*ProcessBody;

%macro ReviewStatusDistribution;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","TreatmentUse","PopulationFlags"}}$
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = ReviewStatusDistribution; 
%let WhereForReports = &WhereStmt;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%let trrflg = %nrbquote(%LocaleGet(SubjectSnapshotFlag_Text));
%let trrst = %nrbquote(%LocaleGet(ReviewStatusFromPatientProfile_Text));
%let truna = %nrbquote(%LocaleGet(UserNotesAvailable_Text));
%let trsna = %nrbquote(%LocaleGet(SystemNotesAvailable_Text));
%let trYes = %nrbquote(%LocaleGet(Yes_Text));
%let trNo = %nrbquote(%LocaleGet(No_Text));
%let trReviewStatusDistribution = %nrbquote(%LocaleGet(ReviewStatusDistribution_Text));
%let trUser = %nrbquote(%LocaleGet(User_Text));
%let trReviewed = %nrbquote(%LocaleGet(Reviewed_Text));
%let trUserwhoset = %nrbquote(%LocaleGet(Userwhosetreviewstatustoreviewed_Text)); 
%let trUserswhocon = %nrbquote(%LocaleGet(Userswhocontributednotes_Text));
%let trNone = %nrbquote(%LocaleGet(None_Text)); 

%PrepOutDS(ReviewStatusDist_OutData,OutDataName,adsl,_rsd);

%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;
%end;

proc sort data=&in_adsl out=adsl;
   by USUBJID;
run;

%DMFlags;

proc sort data=&in_adslrs out=adslrs;
   by USUBJID;
run;

data review;
   merge adslrs adsl(in=_in);
   by USUBJID;
   if _in;
   label rstatus = &trrst;
   *if missing(STUDYID) then delete;
run;

%let NotesExist = 0;
%let NotesDataset = ;
%let NotesDir = %jg_getPath(/user/clinical/notes,,0,0,3,&studynameunder);
      
libname NotesLib "&NotesDir";

proc contents data = NotesLib._ALL_ nods;  
   ods exclude all;
   ods output members = members(keep = name where = (substr(upcase(name),1,1)= "N"));
run;
ods select all;

%if %sysfunc(exist(members)) %then %do;
   %let _vn_ = %sysfunc(open(members));
   %let nobs = %sysfunc(attrn(&_vn_,nobs));
   %let _vn_ = %sysfunc(close(&_vn_));
%end;
%else %let nobs = 0;

%if &nobs %then %do;
   data aggnotes;
      set noteslib.n:;
   run;
   %let NotesExist = 1;
   %let NotesDataset = work.aggnotes;
%end;
        
proc datasets nolist;
   delete members;
quit;

%if &NotesExist %then %do;
   %let sortvar = Unique_Subject_Identifier;
   %let _tid_ = %sysfunc(open(&notesDataset));
   %let flag = %sysfunc(varnum(&_tid_,&sortvar));
   %if ^&flag %then %let sortvar = usubjid;
   %let _tid_ = %sysfunc(close(&_tid_));
   
   data notes;
      set &notesDataset;
      %if &sortvar = Unique_Subject_Identifier %then %do;
         rename Unique_Subject_Identifier = USUBJID;
      %end;
   run;

   proc sort nodupkey data = notes(where = (upcase(SystemGenerated) = "Y")) out = sysnotes;
      by usubjid;
   run;

   proc sort nodupkey data = notes(where = (upcase(SystemGenerated) = "N")) out = notes;
      by usubjid user;
   run;
   
   data notes(drop = user);
      set notes;
      by usubjid user;
      retain nuser;
      if first.usubjid then nuser = user;
      else nuser = catx(", ",nuser, user);
      if last.usubjid;
   run;

   data review;
      merge review(in = ina) sysnotes(in = inb keep = usubjid) notes(in = inc keep = usubjid nuser);
      by usubjid;
      if ina;
      if inb then syscomment = "&trYes";
      else if not inb then syscomment = "&trNo";
      if inc then comment = "&trYes";
      else if not inc then comment = "&trNo";
      if upcase(rstatus) ne upcase("&trReviewed") then user = "&trNone";
      if missing(nuser) then nuser = "&trNone";
      label comment = "&truna" syscomment = "&trsna" reviewflag = "&trrflg" rstatus = "&trrst" nuser = "&trUserswhocon" user = "&trUserwhoset";
   run;
%end;
%else %do;
   data review;
      set review;
      comment = "&trNo";
      syscomment = "&trNo";
      nuser = "&trNone";
      if upcase(rstatus) ne upcase("&trReviewed") then user = "&trNone";
      label comment = "&truna" syscomment = "&trsna" user = "&trUserwhoset" nuser = "&trUserswhocon";
   run;
%end;

data &ReviewStatusDist_OutData;
   set review;
run;

%DMLabels(&ReviewStatusDist_OutData);

%let odata = &ReviewStatusDist_OutData;
%include "&MacroPath./DesignByVars.sas";
%DMFlags;
%if &exiterror %then %goto exit;
%DMLabels(&odata);
%CheckStudyLevels;

* create JSL file;
%PrepOutFile(ReviewStatusDistScript,ReviewStatusDistScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ReviewStatusDistScript;
%let FullAPName=%str(Review Status Distribution);

* put JSL statements;
data _null_;
   file "&ReviewStatusDistScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:WhereState=""\[&WhereForReports]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
      put "try(objRefNS:WhereFilters=""Additional Filter to Include Subjects: "" || objRefNS:WhereState);"; 
   
   
   put "objRefNS:rsd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "Column(""&SubjectLabel"") << Label(1);";
   put "objRefNS:dbv = {};";
   %do i=1 %to &ndbv;
      put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
   %end;
   put "objRefNS:ddw=Function({},{default local},";
   put "   If(IsEmpty(objRefNS:rsd),";
   put "      objRefNS:rsd=open(""&ClientOutPath.&OutDataName.sas7bdat"",invisible);";
   put "   );";
   put "   VListBox(";
   put " textbox(locale:get(""Codesforsubjectlevelreviewflags_Text""),<<Set Wrap(900),<< Font Color(5)),";
   put "    textbox(locale:get(""IntroducedSLRFDesc_Text""),<<Set Wrap(900),<< Font Color(5)),";
   %if &ReviewsEnabled %then %do;
      put "    textbox(locale:get(""NewRecordsSLRFDesc_Text""),<<Set Wrap(900),<< Font Color(5)),";
      put "    textbox(locale:get(""ModifiedRecordsSLRFDesc_Text""),<<Set Wrap(900),<< Font Color(5)),";
      put "    textbox(locale:get(""StableRecordsSLRFDesc_Text""),<<Set Wrap(900),<< Font Color(5)),";
   %end;
   %else %do;
      put "    textbox(locale:get(""PreviousSLRFDesc_Text""),<<Set Wrap(900),<< Font Color(5)),";
   %end;
   put "    textbox(""""),";
   put "      HListBox(";
   put "         VListBox(Distribution(";
   put "            Automatic Recalc(1),";

   %if %upcase(&SplitResults)=YES %then %do;
      put "            By(eval(dbv)),";
   %end;
   put "            Nominal Distribution(Column(:&trrst), ShowCounts(1)),";
   put "            Nominal Distribution(Column(:&trUserwhoset), ShowCounts(1)),";
   put "            Nominal Distribution(Column(:&trrflg), ShowCounts(1)),";
   put "            Nominal Distribution(Column(:&truna), ShowCounts(1)),";
   put "            Nominal Distribution(Column(:&trUserswhocon), ShowCounts(1)),";
   put "            Nominal Distribution(Column(:&trsna), ShowCounts(1)),";
   put "         SendToReport(";
   put "             Dispatch(";
   put "                {""Distributions"", ""&trrst""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "             Dispatch(";
   put "                {""Distributions"", ""&trUserwhoset""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "             Dispatch(";
   put "                {""Distributions"", ""&trrflg""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "             Dispatch(";
   put "                {""Distributions"", ""&truna""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "             Dispatch(";
   put "                {""Distributions"", ""&trUserswhocon""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "             Dispatch(";
   put "                {""Distributions"", ""&trsna""},";
   put "                ""Frequencies"",";
   put "                OutlineBox,";
   put "                {Close( 1 )}";
   put "             ),";
   put "         );";
   put "         ),";
   put "),";
   put "      )";
   put "   )";
   put ");";
   put "         objRefNS:CustomPanel = Expr(objRefNS:Filter = objRefNS:rsd << Data Filter(<<Conditional(1),";
   put "            Add Filter(";
   put "               Columns( ";
   	%if &doStudy %then %do;
   			put ":Name(""&StudyLabel""),";
      %end;
   	%if &doAge %then %do;
   		put " :Name(""&AgeLabel""),";
   	%end;
   	%if &doSex %then %do;
   		put " :Name(""&SexLabel""),";
   	%end;
   	%if &doRace %then %do;
   		put " :Name(""&RaceLabel""), ";
   	%end;
   	%if &doSite %then %do;
   		put "       :Name(""&SiteLabel""),";
   	%end;
    put "                ,:Name(""&TrtLabel"")";
   put "            )),";
   put "            Mode( Show(1), Include(1) )";  
   put "         ));";

   put "InsertInto(objRefNS:DataList,{{""objRefNS:rsd""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trReviewStatusDistribution""}});";
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Distribution_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddw()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   
   %SubjectDrillDowns;   
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ReviewStatusDistScriptName, &OutDataName..sas7bdat, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ReviewStatusDistribution;

* run the macro;
%ReviewStatusDistribution;