/*
JMP Clinical Analytical Process

Name: Study Health
Authors: Kelci Miclaus, SAS Institute Inc.

Description: 
   This process generates a graphical display and tabulation of study visit attendance.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2011 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*******************************************************************;
*     Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags","TreatmentUse","CalcStudyDay_SVO"}}$
$INCD: {"SV"}$
$CMP: RSYS.Visit Variable$ $UE: SV.VISIT and SV.VISITNUM$   
$CMP: OOUT$ $UE: DM.AGE$ $NOTES: Variable will be available in Data Filter.$
$CMP: OOUT$ $UE: DM.SEX$ $NOTES: Variable will be available in Data Filter.$
$CMP: OOUT$ $UE: DM.RACE$ $NOTES: Variable will be available in Data Filter.$
$CMP: OOUT$ $UE: DM.SITEID$ $NOTES: Variable will be available in Data Filter.$
$CMP: OOUT$ $UE: DS.USUBJID and DS.DSDECOD and DS.DSCAT$
   $NOTES: DSDECOD will be available in Data Filter for events where DSCAT is a "DISPOSITION EVENT".$
$CVS_STOP$
*/

*ProcessVariables;

%global StudyName TrtUse DesignByVars TrtVar RemoveUVs SubjectFilter WhereStmt  SVWhereStmt WhereForReports
        OutPath ServerOutPath MacroPath IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
        ProcessName StudyVisits_OutData StudyVisitsScript MergeSuppDom objReferenceName; 

*ProcessBody;

*options mprint mlogic symbolgen;

%macro StudyVisits;

*initialization and argument checking;
%let ProcessName = StudyVisits; 

%let WhereForReports=&WhereStmt;
%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%LOADDATA(domlist=SV);

%if %UPcase(&TrtUse) = NONE %then %let TrtVar = ;
%else %do;
   %let TrtVar = &TrtName;
%end;

%put Treatment var = &TrtName;

%PrepOutDS(StudyVisits_OutData,StudyVisits_OutDataName,&StudyName,_svh);

%let vpos_ = 0;
%let vnpos_ = 0;
%let dflag = 0;
%let sflag = 0;
%let vpos_ = %sysfunc(indexw(&SVVars, VISIT, ' '));
%let vnpos_ = %sysfunc(indexw(&SVVars, VISITNUM, ' '));
%let dflag = %sysfunc(indexw(&SVVars, SVSTDY,' '));
%let sflag = %sysfunc(indexw(&SVVars, SVSTDTC,' '));

%if ^&vpos_ %then %do;
   %put [MS_S_0176];
   %put ERROR: The variable VISIT is required for Study Visits Analysis;   
   %let exiterror=1;
%end;

%if &exiterror %then %goto exit;
************************************;
******* Data ***********************;
************************************;
proc sort data = &in_sv out = svi;
   by usubjid visitnum;
run;

%let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));
%let trStudyVisits = %nrbquote(%LocaleGet(StudyVisits_Text));
%let trDuplicateVisits = %nrbquote(%LocaleGet(DuplicateVisits_Text));

%if %length(&SVWhereStmt) or %Upcase(&RemoveUVs) = YES  %then %do;
   data svi;
      set svi;
      %if %length(&SVWhereStmt) and %Upcase(&RemoveUVs) = YES %then %do;
       where (%unquote(&SVWhereStmt)) and (upcase(VISIT) ^contains "UNSCHEDULED") and (kupcase(VISIT) ^contains "%kupcase(&trUnscheduled)");
      %end;
      %else %if %length(&SVWhereStmt) %then %do;
          where (%unquote(&SVWhereStmt));
      %end;
      %else %do;
         where upcase(VISIT) ^contains "UNSCHEDULED" and (kupcase(VISIT) ^contains "%kupcase(&trUnscheduled)");
      %end;
   run;
%end;

%nobsvars(svi);
%if &nobs = 0 %then %do;
   %put [MS_S_0175];
   %put ERROR:  No study visits records satisfy the where statements or other filtering.;
   %let exiterror = 1;
   %goto exit;
%end; 

/*check if adsl already has visit data to delete before merge*/
%let tid = %sysfunc(open(&in_adsl));
%let vflag = %sysfunc(varnum(&tid,VISIT));
%let vnumflag = %sysfunc(varnum(&tid,VISITNUM));
%let tid = %sysfunc(close(&tid));

proc sort data=&in_adsl out=adsl;
   by usubjid;
run;

data svi;
   merge svi(in = inb) adsl(in = ina %if &vflag %then drop = VISIT; %if &vnumflag %then drop = VISITNUM; ) ;
   by usubjid;
   if ina and inb;
   %let trSDSV = %nrbquote(%LocaleGet(StudyDayofStartofVisit_Text));
   drop svstdy errdtstdy endy errdtendy svdur svendtc log2_svdur svendy;
   label stdy = &trSDSV;
run;

*** check for duplicate visits ***;
proc sql noprint;
   create table &StudyVisits_OutData as
      select *, count(usubjid) as count
      from svi
      group by usubjid, visitnum
      order by usubjid, visitnum;
quit;

proc sql noprint;
   select max(count)
      into :MaxCount separated by ' '
      from &StudyVisits_OutData;
quit;

%nobsvars(&StudyVisits_OutData);
%if &nobs = 0 %then %do;
   %put [MS_S_0177];
   %put ERROR:  The merge of the subject visits and &in_adsl data sets resulted in 0 observations.  No subject identifiers match between the two data sets found in the defined Study folders.;
   %put ERROR:  Please check that the data sets in the Study folders are accurate.;
   %let exiterror = 1;
   %goto exit;
%end; 

proc sql noprint;
   select distinct visitnum 
      into :VisitNums separated by ' '
      from &StudyVisits_OutData
      order by visitnum;
   %if %length(&visit) %then %do;
      select distinct visit 
         into :Visits separated by '~'
         from &StudyVisits_OutData
         order by visitnum;
   %end;
quit;

*** Adding Completed/DC reason ***;
%if %sysfunc(findw(&domains, DS)) and %index(&DSVARS, DSCAT) and %index(&DSVARS,DSDECOD) %then %do;
   %LOADDATA(domlist=DS);
   proc sort data = &in_ds out = ds;
      by usubjid DSSTDTC;
   run;
   
   data ds;
      set ds;
      where upcase(DSCAT) = "DISPOSITION EVENT";
      by usubjid;
      if last.usubjid;
   run;
   
   %let trUnknown = %nrbquote(%LocaleGet(Unknown_Text));
   data &StudyVisits_OutData;
      merge &StudyVisits_OutData(in = ina) ds(keep = usubjid dsdecod);
      by usubjid;
      if ina;
      if dsdecod = "" then dsdecod = symget('trUnknown');
   run;

   %let DispLabel = DSDECOD;
   %CheckLabel(&StudyVisits_OutData,DispLabel); 
%end;

/* Incorporate Delta if snapshots performed */
%let DeltaLabel = ;  
* For previous added study, there is no global variable for ReviewsEnabled; 
%if ^%symexist(ReviewsEnabled) %then %do;
   %global ReviewsEnabled;
   %let ReviewsEnabled=1;
%end;
%if &ReviewsEnabled  and &ReviewNumber %then %do;
   %let DeltaLabel = SVRFlg;
   %CheckLabel(&StudyVisits_OutData,DeltaLabel);
%end;
   
%let SubjectLabel = USUBJID;
%CheckLabel(&StudyVisits_OutData,SubjectLabel);
%let VisitLabel = VISIT;
%CheckLabel(&StudyVisits_OutData,VisitLabel);
%let VNumLabel = VISITNUM;;
%CheckLabel(&StudyVisits_OutData,VNumLabel);
%let SITEIhlabel = SITEID;
%CheckLabel(&StudyVisits_OutData,SITEIhlabel);
%let SVSTDYLabel = STDY;;
%CheckLabel(&StudyVisits_OutData,SVSTDYLabel);
%if &sflag %then %do;
   %let SVSTDTCLabel = SVSTDTC;
   %CheckLabel(&StudyVisits_OutData,SVSTDTCLabel);
%end;

/* Get demographic labels */
%DMFlags;
%if &exiterror %then %goto exit;
%DMLabels(&StudyVisits_OutData);

%CheckStudyLevels;

%let odata = OutLib.&StudyVisits_OutDataName;
%include "&MacroPath./DesignByVars.sas";

* create JSL file;
%PrepOutFile(StudyVisitsScript,StudyVisitsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &StudyVisitsScript;
%let FullAPName=%str(Study Visits);

* put JSL statements;
data _null_;
   file "&StudyVisitsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newNamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:NoteDomain = ""SV"";";
   /* kemicl ask Xan about why takes only unique integers in assoc array 
   put "VisitValueOrder = Expr( ";
   put "    vnumlist = Associative Array(Column(""&VNumLabel""), Column(""&VisitLabel"") );";
   put "    show(vnumlist);";
   put "    vals =vnumlist<< Get Values;";
   put "    show(vals);";
   put "    Eval(Substitute(Expr(Column(""&VisitLabel"")<<Set Property(""Value Ordering"",_vals_)),Expr(_vals_),vals));";
   put " );";
   */
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:PopulationSet=""&PopulationSet"");";
   put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&SVWhereStmt)]\"");";
   put "try(objRefNS:includestring=""Visits "");";
   %let DomainWhereStmt=&SVWhereStmt;     
   %if %length(&WhereForReports) %then %do;
      put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
      put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
      %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!N Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;
   %end;
   %else %if %length(&DomainWhereStmt) %then %do;   
      put "try(objRefNS:WhereFilters=""Filter to include: ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
   %end; 
   
   put "   objRefNS:openDTExpr = Expr(If(IsEmpty(objRefNS:svdt),";
   put "      objRefNS:svdt=open(""&ClientOutPath.&StudyVisits_OutDataName..sas7bdat"",invisible);";
   put "      Column(""&TrtLabel"") << Set Modeling Type(""Ordinal"");";
   put "      Column(""&VNumLabel"") << Set Modeling Type(""Ordinal"");";
   put "      Column(""&SubjectLabel"") << Label(1);";
   /* put "      VisitValueOrder;"; */
   put "    ,";
   put "    Current Data Table(objRefNS:svdt);";
   put "   );";
   put "dbv = {};";
   %if %upcase(&TrtUse) ^= NONE %then %do;
      put "InsertInto(dbv, Column(""&TrtLabel""));";
   %end;
   %do i=1 %to &ndbv;
      put "InsertInto(dbv,Column(""&&dbv&i""));";
   %end; 
   put "    );";
   put " objRefNS:openDTExpr;";
   
   put "objRefNS:svfunc=Function({},{default local},";
   put " objRefNS:openDTExpr;";
   put " VListBox(HListBox("; 
   put "  objRefNS:svgb = Graph Builder(";
   put "    Size( 650, 550 ),";
   put "    Show Control Panel( 0 ),";
   %if %upcase(&TrtUse) ^= NONE %then %do;
      put "    Show Legend( 1),";
   %end;
   %else %do;
         put "    Show Legend( 0),";
   %end; 
   put "    Auto Stretching(0), ";
   put "    Variables(";
   put "       X(:Name(""&VisitLabel""),";
   put "          Order By(:Name(""&VNumLabel""),Ascending)),";
   %if %length(&DesignByVars) %then %do;
      %do i=1 %to &ndbv;
         put " Group Y(Column(""&&dbv&i"")),";
      %end; 
   %end;
   %if %upcase(&TrtUse) ^= NONE %then %do;
      put "       Overlay( Column(""&TrtLabel""))";
   %end;
   put "    ),";
   put "    Elements(";
   put "       Bar(";
   put "          X,";
   put "          Legend(1),";
   put "          Bar Style( ""Stacked"" ),";
   put "          Summary Statistic( ""N"" ),Label( ""Value"" )";
   put "       )";
   put "    ),";
   put "    SendToReport(";
   put "       Dispatch( {}, ""&VisitLabel"", ScaleBox, {Rotated Labels(""Angled"")}),";
   put "       Dispatch( {}, ""graph title"", TextEditBox, {Set Text( locale:get(""StudyHealth_Text"") )} ),";
   %if ^%length(&DesignByVars) %then %do;
      put "       Dispatch( {}, """", AxisBox( 2 ), {Add Axis Label( locale:get(""NumberofSubjectsatVisit_Text"") )} ),";
   %end;
   put "    )";
   put "  ),";
   put "  objRefNS:svtb = Tabulate( Show Control Panel( 0 ),";
   put "    Add Table( ";
   put "       Row Table( Grouping Columns(:Name(""&VNumLabel""),:Name(""&VisitLabel"" ) ) )";
   %if (%upcase(&SplitResults)=YES or %upcase(&TrtUse) ^= NONE) %then %do;
         put "          ,Column Table( Statistics( N ) )";
         put "          ,Column Table( Grouping Columns( eval(dbv) ))";
   %end;
   put "    )";   
   put "  )";
   put " ),";  
   put "));";   
   put "  objRefNS:CustomPanel = Expr(objRefNS:Filter=objRefNS:svdt << Data Filter(<<Conditional(1),Mode( Show( 1 ), Include( 1 ) ),Auto Clear(0),";
   put "         Add Filter(";
   put "            Columns( ";
   %if %length(&DeltaLabel) %then %do;
      put "                :Name(""&DeltaLabel""),";
   %end; 
	%if &doStudy %then %do;
			put ":Name(""&StudyLabel""),";
   %end;   
   %if &doAge %then %do;
      put "                :Name(""&AgeLabel""),";
   %end;
   %if &doSex %then %do;
      put "                :Name(""&SexLabel""),";
   %end;
   %if &doRace %then %do;
      put "                :Name(""&RaceLabel""), ";
   %end;
   %if &doSite %then %do;  
      put "                :Name(""&SITEIhlabel""),";
   %end;
   %if %sysfunc(findw(&domains, DS)) and %index(&DSVARS, DSCAT) %then %do;
      put                  ":Name(""&DispLabel""),";
   %end;
   put "          List Display)";
   put "         )";
   put "   );";
   put "  );";
   
   put "objRefNS:sv_a_func=Function({},{default local},";
   put " objRefNS:svgb << Lock Scales( 1 );";
   put ");";

   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   
   put "InsertInto(objRefNS:DataList,{{""objRefNS:svdt""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trStudyVisits""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""VisitCounts_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:svfunc()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:svgb,objRefNS:svtb}});";

   %if &MaxCount > 1 %then %do;
      put "objRefNS:dupfunc=Function({},{default local},";      
      put "objRefNS:svdt << Select Where(:count > 1);";
      put "objRefNS:svdtsub = Subset(objRefNS:svdt, selected rows(1),Columns(:&SubjectLabel,:&SITEIhlabel,:&TrtLabel, &VNumLabel,:&VisitLabel";
      %if &sflag %then %do;
         put ",:Name(""&SVSTDTCLabel"")";
      %end;
      put" ), invisible);";
      put "objRefNS:svdt << clear select;";
      put "VListBox(objRefNS:duptxt = TextBox(locale:get(""Allduplicatesarepresentedinsummary_Text""),<<Set Wrap(900),<< Font Color(5)),";
      put "Outline Box(locale:get(""DuplicateVisits_Text""),";
      put "objRefNS:dups = Data Table Box(objRefNS:svdtsub, << Table Style(""Heading Separator""))));";
      put" );";
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
      put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""DuplicateVisits_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:dupfunc()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:duptxt,objRefNS:dups}});";      
      put "InsertInto(objRefNS:DataList,{{""objRefNS:svdtsub""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trDuplicateVisits""}});";
   %end;

   /*Study day distribution by Visit plots only if dataset had date variables */
   %if &sflag or &dflag %then %do;
      put "objRefNS:fbox=Function({},{default local},";
      put "Column(objRefNS:svdt, ""&SVSTDYLabel"") << Set Modeling Type( Ordinal );";        
      put "objRefNS:dist=VListBox(";      
      %let j = 1;
      %do %while(%length(%scan(&VisitNums,&j,' ')));
         %let tempvisnum = %scan(%upcase(&VisitNums,&j," "));
         %let tempvisit = %scan(%upcase(&Visits,&j,"~"));
         put " objRefNS:svdt << Distribution(";
         put "    Automatic Recalc(1),";
         %if (&tempvisnum= . ) %then %do;
         	put "    Where(isMissing(:Name(""&VNumLabel""))),";
         %end;
         %else %do;
        	 put "    Where(:Name(""&VNumLabel"") == &tempvisnum),";
        	%end;
         put "    Continuous Distribution( Column( :&SVSTDYLabel ), Show Percents( 1 ), Show Counts( 1 ), Vertical(0)),";
         put "    SendToReport(";         
         put "    Dispatch( {}, ""Distributions"", OutlineBox, {Set Title( ""&tempvisit"" )} ),";
         put "    Dispatch( {}, ""Distrib Histogram"", FrameBox, {Frame Size( 375, 100 )} ),";
         put "    //Dispatch( {}, ""Distrib Outlier Box"", FrameBox, {Frame Size( 317, 401 )} ),";
         put "       Dispatch( {}, ""Quantiles"", OutlineBox, {Close( 1 )} ),";
         put "       Dispatch( {}, ""Summary Statistics"", OutlineBox, {Close( 1 )} )";
         put "    )";
         put " ),";
         %let j = %eval(&j+1);
      %end;
      put ")";          
      put ");";
      put "InsertInto(objRefNS:DataList,{{""objRefNS:svdt""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trStudyVisits""}});";
      put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""VisitDayDistributions_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
      put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:dist}});";
      %if &MaxCount > 1 %then %do;
         put "InsertInto(objRefNS:DataList,{{""objRefNS:svdtsub""}});";
      %end;      
   %end;

   %SubjectDrillDowns;
   /*
   put "objRefNS:lsvrep=Function({},{default local},";
   put "         if(host is(Windows),";
   put "             Include(ConvertFilePath(""$HOME/JMPC/ProcessLibrary/StudyVisitReport.jsl"", absolute, windows)),";
   put "             Include( ConvertFilePath(""$HOME/JMPC/ProcessLibrary/SutdyVisitReport.jsl"", absolute, POSIX))";
   put "         );";
   put "         jg:StudyVisitReport_updateDeps;"; 
   put "         jg:runProcess( 3,""StudyVisitReport"" );";
   put "         jg:closeJMPBIODlg(""StudyVisitReport"");";
   put ");";   
   put "InsertInto(objRefNS:ActionButtonDesc,""Click to Create Standard Report"");";
   put "InsertInto(objRefNS:ActionButtonList,{{""Visits Standard Report""}});";
   put "InsertInto(objRefNS:ActionCodeList,{{objRefNS:lsvrep()}});";
   */
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &StudyVisitsScriptName, &StudyVisits_OutDataName..sas7bdat, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend StudyVisits;
%StudyVisits;

************************************;
************************************;
************************************;