/*
JMP Clinical Analytical Process

Name: Treatment Emergent AE Summary
Authors: Wendy Czika

*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath TrtUse TrtVar DesignByVars SubjectFilter WhereStmt WhereForReports AEWhereStmt AESerOnly IncludeSevere Output_RTF Output_PDF Output_HTML Output_Text OutName 
IncSF PopulationSet IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL IncFAS IncSAF IncITT IncPPROT IncCOMPL IncRAND IncENRL 
   OutPath ServerOutPath MacroPath TEAESummaryScript JSLFile OutData NumDecimals
   TEAESummary_Body TEAESummary_Frame TEAESummary_Contents MergeSuppDom objReferenceName IgnoreTEAEFlags;
   
*options mprint mlogic symbolgen;
   
*ProcessBody;

%macro TEAESummary;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur","TreatmentUse","PopulationFlags"}}$
$INCD: {"AE"}$
$CMP: CPV.{"Include serious adverse events only","Yes"}$ $UE: AE.AESER$ $NOTES:  Only records with AESER = Y will be kept in analysis.$
$CMP: RSYS$ $UE: AE.AESEV and AE.AESER and AE.AEOUT and AE.AEACN$
$CMP: RSYS.Determine Treatment Emergent Event$ $UE: AE.AETRTEM or AE.TRTEMFL or (AE.AESTDTC and 
        (DM.TRTSDTM or DM.TRTSDT or DM.RFXSTDTC or EX.EXSTDTC or DM.RFSTDTC))$
   $NOTES: Value of Y or Yes for flag variables. See documentation for details of algorithm$$CMP: OSYS$ $UE: DM.SEX or DM.RACE or DM.AGE$ $NOTES: Used in data filters or output data set, if available.$
$CVS_STOP$
*/

*initialization and argument checking;
%let ProcessName = TreatmentEmergentAESummary; 
%let WhereForReports=&WhereStmt;

%let trTreatmentEmergentAESummary = %nrbquote(%LocaleGet(TreatmentEmergentAESummary_Text));
/*
%if &Output_RTF=No and &Output_PDF=No and &Output_HTML=No and &Output_Text=No %then %do;
   %put ERROR: At least one output format must be selected.;
   %let exiterror = 1;
   %goto exit;
%end;
*/

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;

%if &syserr > 4 %then %goto exit;

%let Output_HTML = No;
%let Output_Text = No;

%if ^%sysfunc(findw(&domains, AE)) %then %do;
   %put [MS_S_0011];
   %put ERROR: The AE or ADAE data set does not exist for this study.;
   %let exiterror=1;
   %goto exit;
%end;

%if ^%length(&OutName) %then %do;
   %let OutName = TEAE_Summary;
%end;

%PrepOutDS(OutData,OutDataName,TEAE,_Summary);

%let TEAESummaryScript=&JSLFile;
%if &Output_RTF = Yes %then %do;
   ods rtf file="&OutPath.\&OutName..rtf" bodytitle;
%end;

%if &Output_PDF = Yes %then %do;
   ods pdf file="&OutPath.\&OutName..pdf";
%end;

%if &Output_HTML = Yes %then %do;
   %PrepHTML(
      TEAESummary_Body, 
      TEAESummary_Frame, 
      TEAESummary_Contents, 
      &OutName._body.html, 
      &OutName._frame.html, 
      &OutName._contents.html
   );
   ods html 
      body = "&TEAESummary_Body" (url="&bodyfile")
      frame = "&TEAESummary_Frame"  (url="&framefile")
      contents = "&TEAESummary_Contents" (url="&contentsfile")
   ;
%end;

%if &Output_Text = No %then %do;
   ods listing close;
%end;

%if &syserr > 4 %then %goto exit;

%LOADDATA(domlist = AE,mergesupp = &MergeSuppDom, checkreview=1);
%AESerOnlyToAEWhereStmt;
   
%if %length(&AEWhereStmt) %then %do;
   data ae_ws;
      set &in_ae;
      where %unquote(&AEWhereStmt);
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0178];
      %put ERROR: The specified filter &AEWhereStmt is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end;    
   %NObsVars(ae_ws);
   %if ^&nobs %then %do;
      %put [MS_S_0020];
      %put ERROR: No observations satisfy &AEWhereStmt..;
      %put Please relax the filter.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   
   %let in_ae = ae_ws;
%end; 

proc sort data=&in_adsl out=pop(keep=USUBJID &DesignByVars &TrtName trts:);
   by &DesignByVars USUBJID;
run;

%let popid = %sysfunc(open(pop));
%let TrtLen=%sysfunc(varlen(&popid,%sysfunc(varnum(&popid,&TrtName))));
%let popid=%sysfunc(close(&popid));

proc summary data=pop nway;
   class &TrtName;
   %if %length(&DesignByVars) %then %do;
      by &DesignByVars;
   %end;
   output out=nsubj(drop=_type_ rename=(_freq_=nsubj));
run;

proc sort data=&in_ae out=adae;
   by USUBJID;
run;

%let TEAEOnly = Treatment emergent adverse events;
%TREATMENTEMERGENT(dataset = adae, domain = AE, date = AESTDTC, IgnoreFlag = %kupcase(&IgnoreTEAEFlags));
%if &exiterror %then %goto exit;
%TEAETEXTFLAG(TEAEstring = &TEAEOnly);

data adae;
   set adae;
   if _TRTEMRG = 1 then _TRTEMRGC = "Y";
   else if _TRTEMRG = 0 then _TRTEMRGC = "N";
run;
%let trtemval = Y;
%let trtemvar = _TRTEMRGC;

%let rschk = %sysfunc(open(adae));
%let armchk = %sysfunc(varnum(&rschk,ARM));
%let rschk = %sysfunc(close(&rschk));

%if &armchk %then %do;
   data adae;
      set adae(drop = arm);
   run;
%end;

%if %length(&DesignByVars) %then %do;
   proc sort data=pop out=pop;
      by USUBJID;
   run;
%end;    

/* for macro below */
data adslsort;
   set pop;
run;

%let desc1=%nrbquote(%LocaleGet(SubjectswithanyTEAE_Text));
%let desc2=%nrbquote(%LocaleGet(SubjectswithsevereTEAE_Text));
%let desc3=%nrbquote(%LocaleGet(SubjectswithanytreatmentemergentSAE_Text));
%let desc4=%nrbquote(%LocaleGet(SubjectswithanyTEAEleadingtodeath_Text));
%let desc5=%nrbquote(%LocaleGet(SubjectswithanyTEAEleadingtoPermTrtDisc_Text));

*select only subjects in pop;
data row1(keep=&TrtName &DesignByVars USUBJID) 
     row2(keep=&TrtName &DesignByVars USUBJID) 
     row3(keep=&TrtName &DesignByVars USUBJID) 
     row4(keep=&TrtName &DesignByVars USUBJID) 
     row5(keep=&TrtName &DesignByVars USUBJID) 
     all(keep=&TrtName USUBJID ae1 ae2 ae3 ae4 ae5) 
   ;
   merge pop(in=pop) adae(in=inae);
   by USUBJID;
   if pop;  ae1=0; ae2=0; ae3=0; ae4=0; ae5= 0;
   
   /* inae calculated in TEAE macro above */
   if inae and &trtemvar="&trtemval" then do;
   
      /*AESEVN=3 is the code for AESEV="SEVERE" - both in ADAE for S-A */
      if aesev="SEVERE" or aesev=upcase("%LocaleGet(Severe_Text)") then do;
         output row2;
         ae2 = 1;
      end;
      /* AESER follows SDTM */
      if aeser="Y" or aeser=upcase("%LocaleGet(Y_Text)") then do;
         output row3;
         ae3=1;
      end;
      /* AEOUTN=5 is the code for AEOUT="FATAL" - both in ADAE for S-A */
      if aeout="FATAL" or aeout=upcase("%LocaleGet(Fatal_Text)") then do;
         output row4;
         ae4 = 1;
      end;
      /* "Y" unique to S-A, DRUG WITHDRAWN is CDISC standard */
      if aeacn="Y" or aeacn="DRUG WITHDRAWN" or aeacn=upcase("%LocaleGet(Y_Text)") or aeacn=upcase("%LocaleGet(DrugWithdrawn_Text)") then do;
         output row5;
         ae5=1;
      end;
      output row1;
      ae1=1;
   end;
   output all;
   %do r=1 %to 5;
      label ae&r = "&&desc&r";
   %end;
run;

proc means data=all noprint;
   by USUBJID;
   var ae1-ae5;
   id &TrtName;
   output out=maxes(drop=_type_ _freq_) max=;
run;

/* Get in stacked format for Graph Builder */
proc transpose data=maxes out=stacked(drop=_name_);
   var ae1 %if %upcase(&IncludeSevere)=YES %then ae2; ae3 ae4 ae5;
   by USUBJID &TrtName;
run;

proc sort data=&in_adsl out=adsl;
   by USUBJID;
run;

data _null_;
   set nsubj end=last;
   call symputx("whichtrt"||strip(_n_),&TrtName);
   call symputx("count"||strip(_n_),nsubj);
   if last then call symputx("tottrts",_n_);
run;

%let SummChartLabel = %nrbquote(%LocaleGet(TEAEDescription_Text));
/* %if %length(&DesignByVars)=0 %then %do;
   %let SummChartLabel = %quote(&SummChartLabel.:);
   %do t=1 %to %eval(&tottrts-1);
      %let SummChartLabel = %quote(&SummChartLabel &&whichtrt&t (N=&&count&t), );
   %end;
   %let SummChartLabel = %quote(&SummChartLabel &&whichtrt&tottrts (N=&&count&tottrts));
%end;
%else %do;
*/
%if %length(&DesignByVars) %then %do;
   %let i = 1;
   %let SummChartLabel = %quote(&SummChartLabel by );
   %do %while(%length(%scan(&DesignByVars,&i)));
      %global dbv&i;
      %let dbv&i = %scan(&DesignByVars,&i);
      %CheckLabel(adsl, dbv&i);
      %let SummChartLabel = %quote(&SummChartLabel &&dbv&i);
      %let i = %eval(&i+1);
   %end;
%end;

data outlib.&OutDataName;
   merge stacked adsl;
   by USUBJID;
   label _label_= "&SummChartLabel";
   if col1=1;
   drop col1;
run;  

%let TEAEResults = 1
%nobsvars(outlib.&OutDataName);
%if &nobs = 0 %then %do;
   %let TEAEResults = 0;
   %goto noresultexit;
%end;

%let odata=outlib.&OutDataName;
%include "&MacroPath./DesignByVars.sas";

%let trtvarlen = ;
%VarAttributes(outlib.&OutDataName,&TrtName,_Len=trtvarlen);

%do r=1 %to 5;
   %Nobsvars(row&r);
   
   %if &nobs>0 %then %do;
      proc sort data=row&r nodupkey;
         by &DesignByVars USUBJID;
      run;

      proc freq data=row&r noprint;
         tables &TrtName / out=freq&r;
         %if %length(&DesignByVars) %then %do;
            by &DesignByVars;
         %end;
      run;
      
      data freq&r;
         length Description $69;
         merge freq&r nsubj;
         by &DesignByVars &TrtName;
         description="&&desc&r";
      run;
   %end;
   %else %do;
      data freq&r;
         length description $69;
         set nsubj;
         count = 0;
         description="&&desc&r";
      run;
   %end;
      
%end;

data aefinal;
   set freq1 %if %upcase(&IncludeSevere)=YES %then freq2; freq3 freq4 freq5 end=_last;
   length TreatN $%eval(&TrtLen + 10);
   retain maxnsubj 0;
   if count=. then count = 0;
   pct_col = 100 * Count / Nsubj;
   pct_col = round(pct_col,0.1**&NumDecimals);
   TreatN = cat(strip(&TrtName)," (N=",strip(nsubj),")");
   if count > maxnsubj then maxnsubj = count;
   if _last then call symputx("axismax",maxnsubj); 
run;

%if %length(&DesignByVars) %then %do;
   proc sort data=aefinal;
      by &DesignByVars;
   run;
%end;

proc means sum noprint data = aefinal;
   var count;
   output out = anyevent sum = sum;
run;

data _null_;
   set anyevent;
   call symput("anyaepresent", sum);
run;

* display table;
proc format;
   picture pct low-high='009)'( prefix='(');
run;

%let domain = %nrbquote(%LocaleGet(AdverseEvents_Text));
%let title = %nrbquote(%LocaleGet(AdverseEventSummaryReport_Text));

proc datasets nolist;
   delete all;
quit;

%let trTrEmAESumm = %nrbquote(%LocaleGet(TreatmentEmergentAESummary_Text));
%let trTEAEfoot = %nrbquote(%LocaleGet(TEAETreatmentemergentadverseevent_Text));
%let trSAEfoot = %nrbquote(%LocaleGet(SAESeriousadverseevent_Text));
%let trNPCTfoot = %nrbquote(%LocaleGet(NPCTNumAndPercentageOfSubjsWithALOTEAE_Text));

PROC REPORT DATA=WORK.aefinal LS=97  PS=54  SPLIT="/" CENTER nowindows missing contents="&title";
   break before Description/skip;
   COLUMN  Description  treatN,( COUNT PCT_COL) ;

   DEFINE  Description / group FORMAT= $69. WIDTH=69     SPACING=2   LEFT " " order=data;
   DEFINE  TREATN / across FORMAT= $%eval(&TrtLen + 10). WIDTH=%eval(&TrtLen + 10) SPACING=2 LEFT "&TrtLabel" ;
   DEFINE  COUNT / SUM FORMAT= BEST9. WIDTH=9     SPACING=2   RIGHT "N" ;
   DEFINE  PCT_COL / SUM FORMAT= 8.&NumDecimals Width=10 SPACING=2  left "%";
   %if %length(&DesignByVars) %then %do;
      by &DesignByVars;
   %end;
   title height=17pt "&trTrEmAESumm";
   %if %length(&WhereStmt) %then %do; 
      title2 " &WhereStmt ";
   %end;
   %if %length(&AEWhereStmt) %then %do; 
      title3 " &AEWhereStmt ";
   %end;
      /* title2 j=left height=15pt color=black "Safety Population"; */
   footnote j=left height=10pt color=black "&trTEAEfoot, &trSAEfoot";
   footnote2 j=left height=10pt color=black "&trNPCTfoot";  
   ods proclabel = "&domain";       
RUN;

%if &Output_RTF = Yes %then %do;
   ods rtf close;
%end;

%if &Output_PDF = Yes %then %do;
   ods pdf close;
%end;

%if &Output_HTML = Yes %then %do;
   ods html close;
%end;

%if &Output_Text = No %then %do;
   ods listing;
%end;

%if &syserr > 4 %then %goto exit;


%if &Output_RTF = Yes %then %do;
   %PackageInsertFile(RTF file from &ProcessName, &OutPath, &OutName..rtf, , rtf, yes, no);   
%end;

%if &Output_PDF = Yes %then %do;
   %PackageInsertFile(PDF file from &ProcessName, &OutPath, &OutName..pdf, , pdf, yes, no);
%end;

%if &Output_HTML = Yes %then %do;
   %PackageInsertHtml(HTML from &ProcessName, &OutPath, &bodyfile, &framefile, &contentsfile, yes, no);
%end;
%noresultexit:;

* create JSL file;
%PrepOutFile(TEAESummaryScript,TEAESummaryScriptName,&StudyName &ProcessName Script for &OutDataName..jsl);

* create JSL file;
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &TEAESummaryScript;
%let FullAPName=%str(Treatment Emergent Adverse Events Summary);
%if &TEAEResults %then %do;  
%DMFlags;
%if &exiterror %then %goto exit;
%DMLabels(outlib.&OutDataName);
%CheckStudyLevels;

   %let TrtValList = ;
   %do t=1 %to %eval(&tottrts-1);
      %let TrtValList = &TrtValList %nrbquote(""&&whichtrt&t""),;
   %end;
   %let TrtValList = &TrtValList %nrbquote(""&&whichtrt&tottrts"");

   %let TrtCountList = ;
   %do t=1 %to %eval(&tottrts-1);
      %let TrtCountList = &TrtCountList &&count&t,;
   %end;
   %let TrtCountList = &TrtCountList &&count&tottrts;

   %let trOpen = %nrbquote(%LocaleGet(Open_Text));
%end;

* put JSL statements;
data _null_;
   file "&TEAESummaryScript"  lrecl=32767 %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
    put "//!";
    put "Names Default to Here(1);";
    put "objRefNS = newNamespace();";
    put "jg:currentObjRefNamespaceName = objRefNS << getName;";
    put "objRefNS:DataList = {};";
    put "objRefNS:DataNameList = {};";
    put "objRefNS:TabButtonList = {};";
    put "objRefNS:TabCodeList = {};";
    put "objRefNS:TabAfterCodeList = {};";
    put "objRefNS:TabOpenList = {};";
    put "objRefNS:CreateReportList = {};";
    put "objRefNS:NoteDomain = ""AE"";";     
    put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
    put "try(objRefNS:PopulationSet=""&PopulationSet"");";
    put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&AEWhereStmt)]\"");";
    put "try(objRefNS:includestring=""Adverse Events: "");";
    %let DomainWhereStmt=&AEWhereStmt;
    
    %if %length(&WhereForReports) %then %do;
        put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
        put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
        %if %length(&DomainWhereStmt) %then %do;
            put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
        %end;
    %end;
    %else %if %length(&DomainWhereStmt) %then %do;
      put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
    %end;  
    %if &TEAEResults %then %do;            
       put "objRefNS:teae=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";

       put "objRefNS:teaefunc=Function({},{default local},";
       put "   If(IsEmpty(objRefNS:teae),";
       put "      objRefNS:teae=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible), ";
       put "        Current Data Table(objRefNS:teae)";
       put "   );";
       put " Column(""&TrtLabel"") << Set Modeling Type(""Ordinal"");"; 
       put "   VListBox(";
       %if %length(&TEAEText) %then %do;
          put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)), Text Box(""""),";
       %end;       
       put "   HListBox("; 
       put "    objRefNS:teaegb = Graph Builder(";
       put "      Size( 650, 500 ),";
       put "      Show Control Panel( 0 ),";
       put "    Show Legend( 0 ),";
       put "    Auto Stretching(1), ";
       put "      Variables(";
       put "         X( Column(""&TrtLabel"")),";
       put "         Group X( Column(""&SummChartLabel""), Size( 120 ) ),";
       %if %length(&DesignByVars) %then %do;
           %do i=1 %to &ndbv;
               put " Group Y(Column(""&&dbv&i"")),";
           %end;  
       %end;
       put "       Overlay( Column(""&TrtLabel""))";
       put "      ),";
       put "      Elements(";
       put "         Bar(";
       put "            X,";
       put "            Legend( 5 ),";
       put "            Bar Style( ""Side by side"" ),";
       put "            Summary Statistic( ""N"" )";
       put "         )";
       put "      ),";
       put "      SendToReport(";
       put "         Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
       put "      Dispatch( {}, """", AxisBox( 2 ), {Add Axis Label( locale:get(""Count_Text"") )} ),";
       put "      Dispatch( {}, ""X title"", TextEditBox, {Set Text( """" )} ),";
       %if &trtvarlen > 25 %then %do;
         put "       Dispatch({}, ""&TrtLabel"", ScaleBox, {LabelRow(LabelOrientation(""Vertical""))})";
       %end;
       put "      )";
       put "    ),";
       put "    TextBox("" ""),";
       puT "    VlistBox(";
       %if %length(&DesignByVars)=0 %then %do;
           put "   objRefNS:teaeb = Border Box(Left( 10 ), Right( 10 ), Top( 10 ),Bottom( 10 ), Sides( 15 ),";
           put "        objRefNS:tb = TableBox(";
           put "          String Col Box(""&TrtLabel"", {&TrtValList}),";
           put "           Number Col Box(locale:get(""NumberofSubjects_Text""), {&TrtCountLIst})";
           put "        )";
           put "   ),";
       %end;
       put " )),";
       %if &Output_RTF = Yes or &Output_PDF = Yes or &Output_HTML = Yes %then %do;
          put " Text Box(""""),";
       %end;
       %if &Output_RTF = Yes %then %do;
          put " objRefNS:Button1=ButtonBox(""&trOpen &OutName..rtf"",";
          put "    Open(""&ClientOutPath.&OutName..rtf"");";
          put "    ),";
          put " objRefNS:Button1 << Underline Style(1);";    
       %end;
       %if &Output_PDF = Yes %then %do;
          put " objRefNS:Button2=ButtonBox(""&trOpen &OutName..pdf"",";
          put "    Open(""&ClientOutPath.&OutName..pdf"");";
          put "    ),";
          put " objRefNS:Button2 << Underline Style(1);";    
       %end;    
       %if &Output_HTML = Yes %then %do;
          put " objRefNS:Button3=ButtonBox(""&trOpen &OutName..html"",";
          put "    Web(""&ClientOutPath.&framefile"");";
          put "    ),";
          put " objRefNS:Button3 << Underline Style(1);";    
       %end;    
       put "));";
       
	   put " objRefNS:CustomPanel = Expr(objRefNS:Filter= objRefNS:teae << Data Filter(<<Conditional(1),Mode( Show( 1 ), Include( 1 ) ),Auto Clear(0),";
	   put "         Add Filter(";
	   put "            Columns( ";
		%if &doStudy %then %do;
				put ":Name(""&StudyLabel""),";
	   %end;
		%if &doAge %then %do;
			put " :Name(""&AgeLabel""),";
		%end;
		%if &doSex %then %do;
			put " :Name(""&SexLabel""),";
		%end;
		%if &doRace %then %do;
			put " :Name(""&RaceLabel""), ";
		%end;
		%if &doSite %then %do;
			put "       :Name(""&SiteLabel"")";
		%end;
	   put "         ))";
	   put "   ););";
 

       put "objRefNS:teae_a_func=Function({},{default local},";
       put " objRefNS:teaegb << Lock Scales( 1 );";
       %if %length(&DesignByVars)=0 %then %do;
           put "   objRefNS:tb<<Table Style(""Heading Separator"");";   
       %end;
       put ");";

      put "InsertInto(objRefNS:DataList,{{""objRefNS:teae""}});";
      put "InsertInto(objRefNS:DataNameList,{{""&trTreatmentEmergentAESummary""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""SummaryChart_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:teaefunc()));";
      put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:teae_a_func()));";
      put "InsertInto(objRefNS:TabOpenList,1);";
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText, objRefNS:teaegb}});";
      %SubjectDrillDowns;
   %end;
   %else %do;
      put "objRefNS:teaefunc=Function({},{default local},";
      put "VListBox(";
      %if %length(&TEAEText) %then %do;
          put "objRefNS:TEAEText = TextBox(""&TEAEText"", <<Set Wrap(1000)),";
       %end;       
      put "    TextBox(locale:get(""ThereAreNoTEAEstoReport_Text""),<<Set Font Size(14), <<Set Wrap(850));";
      put ")";
      put ");";
      
      put "objRefNS:ShowDataTables = 0;";
      put "objRefNS:ShowNotes = 0;";
      
      put "InsertInto(objRefNS:DataList,{{""""}});";
      put "InsertInto(objRefNS:DataNameList,{{""""}});";
      put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""SummaryChart_Text"")));";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:teaefunc()));";
      put 'InsertInto(objRefNS:TabAfterCodeList, " ");';    
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:TEAEText}});";
      put "InsertInto(objRefNS:TabOpenList,1);";   
   %end;

   /* put "objRefNS:showReportFunc=function({},{default local},";
   put "    objRefNS:reportWB = WebBrowserBox();";
   put ");"; 
    
   put "objRefNS:postReportFunc=Function({},{default local},";
   put "    objRefNS:reportWB << navigate(""&outpath.&framefile"");";
   put "    objRefNS:reportWB << setSize(800, 600);";
   put ");"; 

   put "InsertInto(objRefNS:DataList,{{""""}});";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:showReportFunc()));";
   put "InsertInto(objRefNS:TabButtonList, expr(""Report""));";
   put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:postReportFunc()));";
   put "InsertInto(objRefNS:TabOpenList,1);"; */

   
   
run;

footnote;
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &TEAESummaryScriptName, &OutDataName..sas7bdat, yes);


%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend TEAESummary;

* run the macro;
%TEAESummary;
