/*
JMP Clinical Analytical Process

Name: Trial Design
Authors: Kelci Miclaus, SAS Institute Inc.

Description: 
   	 
*/

/*
-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

%global StudyName InPath AInPath OutPath ServerOutPath MacroPath POPULATIONSET MergeSuppDom
TrialDesign_OutDatadm TrialDesign_Outdatatx TrialDesign_Outdatata TrialDesignScript;

*ProcessBody;

%macro TrialDesign;

/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$CMP: RSYS.Required Analysis Variables$ $UE: EX.EXDOSE$
$CMP: RSYS.Required Analysis Variables$ $UE: EX.EXDOSU$
$CMP: RSYS.Required Analysis Variables$ $UE: TX.TXPARMCD$
$CMP: RSYS.Required Analysis Variables$ $UE: TX.SETCD$
$CMP: RSYS.Required Analysis Variables$ $UE: TA.ARMCD$
$CVS_STOP$
*/

* initialization and argument checking;
%let ProcessName = TrialDesign; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";

%if &exiterror %then %goto exit;

libname InLib "&InPath";

%if ^%sysfunc(exist(inlib.tx)) %then %do;
	%put ERROR:  The tx data set does not exist in the specified SDTM or ADAM folder.;
	%let exiterror=1;
%end;

%if ^%sysfunc(exist(inlib.ta)) %then %do;
	%put ERROR:  The ta data set does not exist in the specified SDTM or ADAM folder.;
	%let exiterror=1;
%end;

%if ^%sysfunc(exist(inlib.dm)) %then %do;
	%put ERROR:  The dm data set does not exist in the specified SDTM or ADAM folder.;
	%let exiterror=1;
%end;

%if &exiterror %then %goto exit;

%PrepOutDS(TrialDesign_OutDatadm,OutDataNamedm,dm,_tdm);
%PrepOutDS(TrialDesign_OutDatatx,OutDataNametx,tx,_ttx);
%PrepOutDS(TrialDesign_OutDatata,OutDataNameta,ta,_tta);

%LoadData(domlist=dm);

data tx;
set &in_TX;
run;
data ta;
set &in_Ta;
run;
data ts;
set &in_TS;
run;
* load EX domain to get dose and unit information;
%LOADDATA(domlist=EX);
proc sort data=adsl;
	by USUBJID;
run; 
proc sort data=EX(keep=USUBJID EXDOSE EXDOSU EXDOSFRQ) nodupkey;
	by USUBJID;
run;
data EX;
	set EX;
	EXDOSU=trim(left(EXDOSE))||" "||EXDOSU;
run;
data adsl_ex;
	merge adsl EX;
	by USUBJID;
run;

* subset tx domain to get group label information;
data tx_grp;
	set tx;
	if TXPARMCD="GRPLBL";
	label TXVAL="Group Label";
	rename TXVAL=GroupLabel;
run;
proc sort data=adsl_ex;
	by SETCD;
run;
proc sort data =tx_grp(keep=GroupLabel SETCD);
	by SETCD;
run;
data adsl_tx;
	merge adsl_ex tx_grp;
	by SETCD;
run;

*subset tx domain to get group number information;
data tx_spg;
	set tx;
	if TXPARMCD="SPGRPCD";
	label TXVAL="Group Number";
	rename TXVAL=GroupNumber;
run;
proc sort data =tx_spg(keep=GroupNumber SETCD);
	by SETCD;
run;
data adsl_tx;
	merge adsl_tx tx_spg;
	by SETCD;
run;

*subset tx domain to get dose level information;
data tx_trt;
	set tx;
	if TXPARMCD="TRTDOS";
	label TXVAL="Dose Level";
	rename TXVAL=DoseLevel;
run;
proc sort data =tx_trt(keep=DoseLevel SETCD);
	by SETCD;
run;
data adsl_tx;
	merge adsl_tx tx_trt;
	by SETCD;
run;

*merge adsl data with ta domain based on ARMCD;
proc sort data=adsl_tx;
	by ARMCD;
run;
proc sort data=ta out=ta_sub(keep=ARMCD EPOCH);
	by ARMCD;
run;
data &TrialDesign_OutDatadm;
	merge adsl_tx ta_sub;
	by ARMCD;
run;

/*data &TrialDesign_OutDatadm; 
	set inlib.dm; 
run;*/
data &TrialDesign_OutDatatx; 
	set tx; 
run;
data &TrialDesign_OutDatata; 
	set ta; 
run;

 %if &syserr > 4 %then %do;
	%let exiterror = 1;
	%goto exit;
 %end;
 
%let ARMLabel=ARM;
%CheckLabel(outlib.&OutDataNameta, ARMLabel); 
%let EPOCHLabel=EPOCH;
%CheckLabel(outlib.&OutDataNameta, EPOCHLabel); 
%let ElementLabel=Element;
%CheckLabel(outlib.&OutDataNameta, ElementLabel); 
%let TAETORDLabel=TAETORD;
%CheckLabel(outlib.&OutDataNameta, TAETORDLabel); 
%let SETCDLabel=SETCD;
%CheckLabel(outlib.&OutDataNamedm, SETCDLabel); 
%let GroupLabelLabel=GroupLabel;
%CheckLabel(outlib.&OutDataNamedm, GroupLabelLabel); 
%let GroupNumberLabel=GroupNumber;
%CheckLabel(outlib.&OutDataNamedm, GroupNumberLabel); 
%let DMARMLabel=ARM;
%CheckLabel(outlib.&OutDataNamedm, DMARMLabel); 

* create JSL file;
%PrepOutFile(TrialDesignScript,TrialDesignScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &TrialDesignScript;
%let FullAPName=%str(Trial Design);

* put JSL statements;
data _null_;
   file "&TrialDesignScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "objRefNS = newnamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:DataList = {};";
	put "objRefNS:DataNameList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	
	put "if(hostis(windows), _slash_ = ""\"", _slash_ = ""/"");";
	put "if(Host Is(Windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME\Clinical\"", absolute, windows),";
	put "   JMPClinicalDir = ConvertFilePath(""$JMP_HOME/Clinical/"", absolute, POSIX)";
	put ");";
	put "//fsize = 550;";
	put "sdtmFolder = ""&InPath"";";	

	put "objRefNS:tdfunc=Function({},{default local},";
	put " If(IsEmpty(objRefNS:dm),";
	put "		objRefNS:dm = open(""&ClientOutPath.&OutDataNamedm..sas7bdat"",invisible);";
	put "  ,";  /*else for isempty*/;
	put "  Current Data Table(objRefNS:dm);";
	put " );";	
	put " If(IsEmpty(objRefNS:ta),";
	put "		objRefNS:ta = open(""&ClientOutPath.&OutDataNameta..sas7bdat"",invisible);";
	put "  ,";  /*else for isempty*/;
	put "  Current Data Table(objRefNS:ta);";
	put " );";
	put "	Outline Box(locale:get(""TrialDesignSummaryTable_Text""),";
	put "	VListBox(";
	put "	//	Include(""$CLINICAL_HOME\JSLFiles\Trial Design.jsl"");";
	put "Current Data table(objRefNS:dm);";
	put "	Data Table( objRefNS:dm ) << Tabulate(";
	put "		Show Control Panel(0),";
	put "		Add Table(";
	put "		Column Table( Grouping Columns( :Name(""&EPOCHLabel"")) ),";
	put "		Row Table( Grouping Columns( :Name(""&DMARMLabel""),:Name(""&GroupNumberLabel""), :Name(""&GroupLabelLabel""), :Name(""&SETCDLabel"") ) )";
	put "		),";
	put "		SendToReport(";
	put "			Dispatch(";
	put "			{},";
	put "			""Tabulate"",";
	put "			OutlineBox,";
	put "			{Set Title( """" )}";
	put "			)";
	put "		)";
	put "	),";
	put "	Data Table( objRefNS:ta ) <<Tabulate(";
	put "		Show Control Panel( 0 ),";
	put "		//Order by count of grouping columns( 1 ),";
	put "		Add Table(";
	put "			Column Table( Grouping Columns( :Name(""&ElementLabel"") ) ),";
	put "			Row Table( Grouping Columns( :Name(""&ARMLabel"") ) )";
	put "		),";
	put "		SendToReport(";
	put "			Dispatch(";
	put "			{},";
	put "			""Tabulate"",";
	put "			OutlineBox,";
	put "			{Set Title( """" )}";
	put "			)";
	put "		)";
	put "	),	";
	put "	);"; /*end vlist */
	put "	);"; 
	put ");";  /*end function*/	
	
	put 'InsertInto(objRefNS:DataList,{{"objRefNS:dm" ,"objRefNS:ta"}});';
	put "InsertInto(objRefNS:DataNameList,{{locale:get(""TrialDesignOverview_Text""),locale:get(""TreatmentbyElement_Text"")}});";
	put "InsertInto(objRefNS:TabButtonList, locale:get(""TrialDesign_Text""));";
	put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:tdfunc()));';
	put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
	put 'InsertInto(objRefNS:TabOpenList,1);';
	
	put "objRefNS:tafunc=Function({},{default local},";
	put " If(IsEmpty(objRefNS:ta),";
	put "		objRefNS:ta = open(""&ClientOutPath.&OutDataNameta..sas7bdat"",invisible);";
	put "  ,";  /*else for isempty*/;
	put "  Current Data Table(objRefNS:ta);";
	put " );";
	put "Current Data table(objRefNS:ta);";
	put "	VListBox(";
	put "Graph Builder(";
	put "	Size( 1096, 505 ),";
	put "	Show Control Panel( 0 ),";
	put "	Variables(";
	put "		Y( :Name(""&ARMLabel"") ),";
	put "		Group X(";
	put "			:Name(""&ElementLabel""),";
	put "			Order By(";
	put "				:Name(""&TAETORDLabel""),";
	put "				Ascending,";
	put "				Order Statistic( ""Mean"" )";
	put "			)";
	put "		),";
	put "		Overlay( :Name(""&ElementLabel"") )";
	put "	),";
	put "	Elements(";
	put "		Bar(";
	put "			Y,";
	put "			Legend( 2 ),";
	put "			Bar Style( ""Side by side"" ),";
	put "			Summary Statistic( ""Mean"" )";
	put "		)";
	put "	),";
	put "	SendToReport(";
	put "		Dispatch(";
	put "			{},";
	put "			"""",";
	put "			ScaleBox,";
	put "			{Show Major Ticks( 0 ), Show Minor Ticks( 0 ), Show Labels( 0 )}";
	put "		),";
	put "		Dispatch(";
	put "			{},";
	put "			""400"",";
	put "			LegendBox,";
	put "			{Position( {-1, -1, -1, -1, -1, -1, -1, -1, -1, -1} )}";
	put "		),";
	put "	Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} )";
	put "	)";
	put "	);";
	put "	);"; /*end vlist */
	put ");";  /*end function*/
	put 'InsertInto(objRefNS:DataList,{{"objRefNS:ta" }});';
	put "InsertInto(objRefNS:TabButtonList, locale:get(""TrialARMs_Text""));";
	put 'InsertInto(objRefNS:TabCodeList,expr(objRefNS:tafunc()));';
	put 'InsertInto(objRefNS:TabAfterCodeList, " ");';
	put 'InsertInto(objRefNS:TabOpenList,1);';
run;


%PackageInsertFile(jsl code from &ProcessName, &OutPath, &TrialDesignScriptName, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend TrialDesign;

* run the macro;
%TrialDesign;


