/*
JMP Clinical Analytical Process

Name: UpdateStudyRiskData
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Opens data set for manual entry of study criteria
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath MacroPath OutName UpdateStudyRiskScript UpdateStudyRisk_OutData OutPath ServerOutPath PopulationSet;

%let PopulationSet = All Subjects;

*ProcessBody;

%macro UpdateStudyRiskData;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$INCD: {"DM"}$
$CMP: RSYS$ $UE: DM.SITEID and DM.COUNTRY$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = UpdateStudyRiskData; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./RBMMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%if &sysscp = WIN %then %do;
   %let _slash_ = \;
%end;
%else %do;
   %let _slash_ = /;
%end;

libname RBM "&metapath.RBM&_slash_";

%if %sysfunc(findw(%upcase(&DMVARS), SITEID)) and %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) %then %do;
   %if %index(%upcase(&in_adsl), ADSL) %then %do;
      %LOADDATA(domlist = DM,mergesupp = 0,checkreview=1);
   %end;
   %GENERATERBMSITE;
   %if &exiterror %then %goto exit;
%end;
%else %do;
   %if ^%sysfunc(findw(%upcase(&DMVARS), SITEID)) %then %do;
      %put [MS_S_0179];
      %put ERROR: SITEID is required for Risk Based Monitoring.;
      %put ERROR: SITEID is not in ADDM or DM in this order. Cannot update Study Risk Data Set;
   %end;
   %if ^%sysfunc(findw(%upcase(&DMVARS), COUNTRY)) %then %do;
      %put [MS_S_0180];
      %put ERROR: COUNTRY is required for Risk Based Monitoring.;
      %put ERROR: COUNTRY is not in ADDM or DM in this order. Cannot update Study Risk Data Set;
   %end;
   %let exiterror = 1;
   %goto exit;
%end;

%PrepOutDS(UpdateStudyRisk_OutData,OutDataName, rbm_site);

%let _tid_ = %sysfunc(open(NotesLib.rbm_site));   
   %let cityind = %sysfunc(varnum(&_tid_,CITY));
%let _tid_ = %sysfunc(close(&_tid_));   

data &UpdateStudyRisk_OutData;
   set NotesLib.rbm_site;
run;

%let trCity = %nrbquote(%LocaleGet(City_Text));

%let SiteLabel = SITEID;
%CheckLabel(&UpdateStudyRisk_OutData,SiteLabel);
%let _tid_ = %sysfunc(open(&UpdateStudyRisk_OutData));
%let cind = %sysfunc(varnum(&_tid_,COUNTRY));
%let _tid_ = %sysfunc(close(&_tid_));  
%if &cind %then %do;
   %let CountryLabel = COUNTRY;
   %CheckLabel(&UpdateStudyRisk_OutData,CountryLabel);
%end;

* create JSL file;
%PrepOutFile(UpdateStudyRiskScript,UpdateStudyRiskScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &UpdateStudyRiskScript;
%let FullAPName=%str(Update Study Risk Data Set);

* put JSL statements;
data _null_;
   file "&UpdateStudyRiskScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:TabsSectionFlag = 0;";
   put "objRefNS:MacroPath = ""&MacroPath"";";
   put "objRefNS:OutFile = ""&OutDataName"";";
   put "objRefNS:NotesPath = ""&NotesDir"";";
   put "objRefNS:Outpath = ""&Outpath"";";
   put "objRefNS:RBMPath = ""&metapath.RBM&_slash_"";";
   put "objRefNS:tabfunc=Function({},{default local},";
   put "   objRefNS:SiteLabVal = ""&SiteLabel"";";
   put "   objRefNS:CountryLabVal = ""&CountryLabel"";";
   put "   objRefNS:rbmsite = open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "   Column(objRefNS:rbmsite, ""&SiteLabel"") << set lock(1);";
   put "   Try(Column(objRefNS:rbmsite, ""&CountryLabel"") << set lock(1));";
   put "   objRefNS:initialRBMSiteRowCnt = nRows(objRefNS:rbmsite);";
   put "   objRefNS:GeoFlag = 1;";
   
   %if &cind %then %do;
      put "   objRefNS:rbmsite << Set Scroll Lock Columns( :&SiteLabel, :&CountryLabel);";
   %end;
   %else %do;
      put "   objRefNS:rbmsite << Set Scroll Lock Columns( :&SiteLabel);";
   %end;
   put "   //objRefNS:rbmsite << Close Side Panels;";
   put "   Column(objRefNS:rbmsite,""&SiteLabel"") << label(1);";
   %if &cityind %then %do;
      put "   Column(objRefNS:rbmsite,""&trCity"") << label(1);";
   %end;
   put "   objRefNS:rbmsite << Select All Rows;";
   put "   objRefNS:rbmsite << Markers (12);";
   put "   objRefNS:rbmsite << Clear Select;";   
   put "   objRefNS:rbmsite << Close Side Panels;";   
   put "   Include(""$CLINICAL_HOME\JSLFiles\RBMFunctions.jsl"");";
   put "   returnContent = VListBox(";
   put "      objRefNS:rbmsite << New Data Box,";
   put "      messageTextBox = TextBox(locale:get(""RBMNoteDoNotChangeTableStructure_Text""), <<setWrap(800), setFontStyle(""bold"") )";
   put "   );";
   put "   messageTextBox << setAutoStretching(1, 0);";
   put "   messageTextBox << setMaxSize(1000, 50);";
   put "   returnContent;";
   put ");";    
   
   put "InsertInto(objRefNS:DataList,{{""objRefNS:rbmsite""}});";
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{},{}};";
   put "objRefNS:ActionCodeList ={{},{},{}};";
   put "objRefNS:ActionDataTableList = {{},{},{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""DataSet_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""SaveStudyRiskDataSet_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:SaveStudyRisk()));"; 
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:rbmsite""}});";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:rbmsite""}});";
 
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""Mapping_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""GeocodeSites_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:GeoCode()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:rbmsite""}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""MapGeocodingHelp_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(local({parent=namespace(objRefNS:getParent())}, objRefNS:MapGeodcodeHelp(); expr(parent:docker):setDockerContent(locale:get(""USCities_Text""), objRefNS:uscity<<newDataBox()); expr(parent:docker):setDockerContent(locale:get(""NonUSCities_Text""), objRefNS:worldcity<<newDataBox());)));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:rbmsite""}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ShowMap_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:GeoMap()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:rbmsite""}});";

   put "objRefNS:ActionSelectionCountList = {{0},{0, 0, 0}};";
   put "objRefNS:ActionIconList = {{""&IconPath.SaveAsDataSet.gif""},";
   put "                          {""&IconPath.Globe.gif"",""&IconPath.LocaleGuessing.gif"",""&IconPath.TypeGeography.gif""}};";
   
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""StudyRiskDataSet_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:tabfunc()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "objRefNS:ShowDataTables = 0;";
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &UpdateStudyRiskScriptName, &OutDataName..sas7bdat, yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend UpdateStudyRiskData;
%UpdateStudyRiskData;

************************************;
************************************;
************************************;