/*
JMP Clinical Analytical Process

Name: Variable Report
Authors: SAS Institute Inc.

Description:
	This process launches the JMP Clinical Study Variable Report.

*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName OutPath ServerOutPath MacroPath VariableScript PDFRTF;

*ProcessBody;

%macro VariableReport;

*initialization and argument checking;
%let ProcessName = VariableReport; 

%let PDFRTF = %LOWCASE(&PDFRTF);

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%let includeds =;

* create JSL file;
%PrepOutFile(VariableScript,VariableScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &VariableScript;
%let FullAPName = %str(Variable Report);
%let trVariableReport = %nrbquote(%LocaleGet(VariableReportNS_Text));

data _null_;
   file "&VariableScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "objRefNS = newnamespace();";
	put "jg:currentObjRefNamespaceName = objRefNS << getName;";
	put "objRefNS:FullAPName = ""&FullAPName"";";
	put "objRefNS:StudyName = ""&StudyName"";";
	put "Names Default to Here(1);";
	put "objRefNS:DataList = {};";
	put "objRefNS:TabButtonList = {};";
	put "objRefNS:TabCodeList = {};";
	put "objRefNS:TabAfterCodeList = {};";
	put "objRefNS:TabOpenList = {};";
	put "objRefNS:CreateReportList = {};";
        put "objRefNS:ShowDataTables = 0;";
        put "objRefNS:ShowCreateReport = 0;";
        put "objRefNS:ShowNotes = 0;";	
	put "jg:CVSInit;";
	put "rc = GenerateStudyStarterFilter( ""&StudyName"", 0, 1, ""&PDFRTF"" );";
	put "Try(";
	put "   Copy File(jg:getPath(""/user/clinical/metadata"", ""VariableReport.&PDFRTF""), ""&ClientOutPath.&trVariableReport..&PDFRTF"");";
	put ");";
	put "Try(";
	put "   Open(""&ClientOutPath.&trVariableReport..&PDFRTF"");";
	put ");";	
	put "objRefNS:fbox=Function({},{default local},";
	put "    if (File Exists(""&ClientOutPath.&trVariableReport..&PDFRTF""),";	
	put "        Vlistbox(";
	put "            objRefNS:Button=ButtonBox(locale:fill(locale:get(""OpenX_Text""), {""&trVariableReport..&PDFRTF""}),";
	put "                Try(";
	put "                    Open(""&ClientOutPath.&trVariableReport..&PDFRTF"");";
	put "                );";
	put "            ),";
	put "            objRefNS:Button << Underline Style(1);";
	put "            if (!jg:CVSOptions[""MASTER""],";	
	put "              TextBox(locale:get(""WarningCVSDisabledSoNewRepCNBC_Text"") || "" "" ||";
	put "                  locale:get(""ThisIsPrevGenRepAccNotGuarIfSDC_Text""),<<Set Wrap(900),<< Font Color(6))";
	put "            )";
	put "        ),";
	put "        if (!jg:CVSOptions[""MASTER""],";	
	put "            TextBox(locale:get(""ErrorCVSDisabledSoNewRepCNBC_Text"") ";
	put "                ,<<Set Wrap(900),<< Font Color(3))";
	put "            ,";
	put "            TextBox(locale:get(""ErrorNewRepCNBCEnsureDiskNotFull_Text"") ";
	put "                ,<<Set Wrap(900),<< Font Color(3))";
	put "        )";
	put "    )";
	put ");";
	put "InsertInto(objRefNS:DataList,{{""  ""}});";
	put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""VariableReport_Text"")));";
	put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:fbox()));";
	put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
	put "InsertInto(objRefNS:TabOpenList,1);";        
	
run;

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &VariableScriptName,"&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend VariableReport;

* run the macro;
%VariableReport;