/*
JMP Clinical Analytical Process

Name: Weekend Holiday
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
   This process identifies dates on Weekends and Holidays.  
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName WeekendHolidayScript WeekendHoliday_OutData OutPath ServerOutPath PopulationSet 
        WhereStmt WhereForReports GetVis SiteCount AlphaValue;

*ProcessBody;

%macro WeekendHoliday;

/*    
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","PopulationFlags","CalcStudyDay_SVO","CalcStudyDay_FindingsO"}}$
$CMP: RSYS$ $UE: DM.SITEID and (((FINDINGS.USUBJID and FINDINGS.xxDTC) or (SV.USUBJID and SV.SVSTDTC)) or (DM.USUBJID and DM.DMDTC) 
          or (MH.USUBJID and MH.MHDTC))$ 
$CMP: CPV.{"Derive visits from:","SV domain"}$ $UE: SV.SVSTDTC$          
$CMP: CPV.{"Derive visits from:","findings domains"}$ $UE: FINDINGS.xxDTC$          
$CMP: OSYS$ $UE: SV.SVENDTC $ $NOTES: If available, calculates all days in between as part of the visit.$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Added to filter$
$CMP: OOUT$ $UE: xx.VISITNUM$  $NOTES: In output data set, if available.$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = WeekendHoliday;

%let WhereForReports=&WhereStmt;

%let trXVolcanoPlot = %nrbquote(%LocaleGet(XVolcanoPlot_Text));
%let trXDistribution = %nrbquote(%LocaleGet(XDistribution_Text));
%let blank =;
%let trVolcanoPlot = %nrbquote(%LocaleFill(&trXVolcanoPlot, &blank));
%let trDistribution = %nrbquote(%LocaleFill(&trXDistribution, &blank));
%let trVolcanoPlot = %sysfunc(strip(&trVolcanoPlot));
%let trDistribution = %sysfunc(strip(&trDistribution));

%let RBMPath = &MetaPath.RBM/;
libname RBMPath "&RBMPath";

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./DQFMacros.sas";
%if &exiterror %then %goto exit;

%let includeds=;

%OUTNAMETRUNC(outdat = WeekendHoliday_OutData, truncnum = 20);
%DQFTRANS;

%let trSunday = %nrbquote(%LocaleGet(Sunday_Text));
%let trMonday = %nrbquote(%LocaleGet(Monday_Text));
%let trTuesday = %nrbquote(%LocaleGet(Tuesday_Text));
%let trWednesday = %nrbquote(%LocaleGet(Wednesday_Text));
%let trThursday = %nrbquote(%LocaleGet(Thursday_Text));
%let trFriday = %nrbquote(%LocaleGet(Friday_Text));
%let trSaturday = %nrbquote(%LocaleGet(Saturday_Text));
%let trWeekday = %nrbquote(%LocaleGet(Weekday_Text));
%let trBoxingDay = %nrbquote(%LocaleGet(BoxingDay_Text));
%let trCanInd = %nrbquote(%LocaleGet(CanadianIndependence_Text));
%let trCanIndObs = %nrbquote(%LocaleGet(CanadianIndependenceObserved_Text));
%let trChristmas = %nrbquote(%LocaleGet(Christmas_Text));
%let trChristmasObs = %nrbquote(%LocaleGet(ChristmasObserved_Text));
%let trColumbusDayUSA = %nrbquote(%LocaleGet(ColumbusDayUSA_Text));
%let trEaster = %nrbquote(%LocaleGet(Easter_Text));
%let trGoodFriday = %nrbquote(%LocaleGet(GoodFriday_Text));
%let trEasterMonday = %nrbquote(%LocaleGet(EasterMonday_Text));
%let trLaborDayUSA = %nrbquote(%LocaleGet(LaborDayUSA_Text));
%let trMLKJrBD = %nrbquote(%LocaleGet(MartinLutherKingJrBirthday_Text));
%let trMemDayUSA = %nrbquote(%LocaleGet(MemorialDayUSA_Text));
%let trNewYearsDay = %nrbquote(%LocaleGet(NewYearsDay_Text));
%let trNewYearsDayObs = %nrbquote(%LocaleGet(NewYearsDayObserved_Text));
%let trThanksgivingDayUSA = %nrbquote(%LocaleGet(ThanksgivingDayUSA_Text));
%let trThanksgivingFriUSA = %nrbquote(%LocaleGet(ThanksgivingFridayUSA_Text));
%let trThanksgivingDayCAN = %nrbquote(%LocaleGet(ThanksgivingDayCAN_Text));
%let trIndependenceDayUSA = %nrbquote(%LocaleGet(IndependenceDayUSA_Text));
%let trPresDayUSA = %nrbquote(%LocaleGet(PresidentsDayUSA_Text));
%let trVetRemDay = %nrbquote(%LocaleGet(VeteranRemembranceDay_Text));
%let trVetDayObsUSA = %nrbquote(%LocaleGet(VeteransDayObservedUSA_Text));
%let trVictoriaDayCAN = %nrbquote(%LocaleGet(VictoriaDayCAN_Text));
%let trCanadaDay = %nrbquote(%LocaleGet(CanadaDay_Text));
%let trCanadaDayObs = %nrbquote(%LocaleGet(CanadaDayObserved_Text));
%let trLabourDayCAN = %nrbquote(%LocaleGet(LabourDayCAN_Text));
%let trNoHoliday = %nrbquote(%LocaleGet(NoHoliday_Text));
%let trHoliday = %nrbquote(%LocaleGet(Holiday_Text));
%let trHolidayFlag = %nrbquote(%LocaleGet(HolidayFlag_Text));
%let trWeekend = %nrbquote(%LocaleGet(Weekend_Text));
%let trMonFri = %nrbquote(%LocaleGet(MonFri_Text));

%let svnvars = 0;
%let svlvars = ;

%if %sysfunc(findw(&domains, SV)) and %sysfunc(findw(&GetVis, SV)) %then %do;
   %let domainlist = DM MH SV;
   %if %sysfunc(indexw(&SVVARS, SVSTDTC, ' ')) %then %do;
      %let svnvars = %eval(&svnvars + 1);
      %let svlvars = &svlvars SVSTDTC;
   %end;
   %if %sysfunc(indexw(&SVVARS, SVENDTC, ' ')) %then %do;
      %let svnvars = %eval(&svnvars + 1);
      %let svlvars = &svlvars SVENDTC;
   %end;
%end;
%else %if ^%sysfunc(findw(&domains, SV)) and %sysfunc(findw(&GetVis, SV)) %then %do;
   %let domainlist = DM MH;
%end;
%else %do;
   %let domainlist = DM MH &findingsdomains;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
   %put [MS_S_0053];
   %put ERROR: SITEID is not available in either DM, ADDM, or ADSL.;
   %let exiterror = 1;
   %goto exit;
%end;

%ADDCOUNTRYTODATA;
%let SiteCount = 1;
%GETSITELISTS;
%if &exiterror %then %goto exit;

%let i = 1;
%let flag = 0;
%do %while(%length(%scan(&domainlist,&i,' ')));
   %let temp = %scan(&domainlist,&i,' ');
   
   %if %sysfunc(findw(&domains, &temp)) and &temp ne PP %then %do;

      %let visitnum = ;
      %if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) %then %let visitnum = visitnum;

      %if &temp ne SV and %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %do;      
         %let _tid_ = %sysfunc(open(&&in_&temp));
         %let daten = %sysfunc(varnum(&_tid_,&temp.DTC));      
         %let DateLabel = %sysfunc(varlabel(&_tid_,&daten));
         %let _tid_ = %sysfunc(close(&_tid_));  

         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);        
         data temp;
            length domain $ 8 dtlabel $ 40;
            %if &temp = DM %then %do;
               set &&in_&temp(keep = usubjid &temp.dtc);
            %end;
            %else %do;
               set &&in_&temp(keep = usubjid &temp.dtc &visitnum);
            %end;
            %DATE9(var=&temp.DTC,return=dt);
            %if &temp = DM %then %do;
               dtlabel = symget('trDemogDate');
            %end;
            %else %if &temp = MH %then %do;
               dtlabel = symget('trMedHistDate');
            %end;
            %else %do;
               dtlabel = symget('trVisitDate');
            %end;       
            drop &temp.DTC;
            %if ^%length(&visitnum) %then %do;
               visitnum = .;
               label visitnum = "&trVisitNum";
            %end;
            rename d9partial_&temp.DTC = imputedate;
            domain = "&temp";
         run;
                  
         %if ^&flag %then %do;
            data all;
               set temp;
            run;        
            %let flag = 1;          
         %end;
         %else %do;
            proc append force base = all data = temp;
            run;
         %end;                
      %end;
      %else %if &temp = SV and &svnvars %then %do;
         %LOADDATA(domlist = SV,mergesupp = 0,checkreview=0);
         %if &svnvars = 1 %then %do;
            data temp;
               length domain $ 8 dtlabel $ 40;
               set &&in_&temp(keep = usubjid &svlvars &visitnum);
               %DATE9(var=&svlvars,return=dt);     
               %if &svlvars = SVSTDTC %then %do;
                  dtlabel = symget('trVisitStartDt');
                  rename d9partial_SVSTDTC = imputedate;
               %end;
               %else %do;
                  dtlabel = symget('trVisitEndDt');
                  rename d9partial_SVENDTC = imputedate;
               %end;
               domain = "&temp";
               drop &svlvars;
            run;
         %end;
         %else %if &svnvars = 2 %then %do;
            proc sort data = &&in_&temp out = temp;
               by usubjid &visitnum;
            run;

            data temp;
               set &&in_&temp(keep = usubjid &svlvars &visitnum);
               %DATE9(var=SVSTDTC,return=sdt);
               %DATE9(var=SVENDTC,return=edt);
               if sdt ne . and edt = . then edt = sdt;
               else if sdt = . and edt ne . then sdt = edt;
               else if sdt = . and edt = . then delete; 
               
               if d9partial_SVSTDTC or d9partial_SVENDTC then d9partial_MID = 1;
               else d9partial_MID = 0;
               
            run;
            
            proc sort data = temp;
               by usubjid &visitnum;
            run;
            
            data temp(drop = i d9partial_SVSTDTC d9partial_SVENDTC d9partial_MID);
               length domain $ 8 dtlabel $ 40;
               set temp;
               by usubjid &visitnum;               
               do i = sdt to edt;
                  dt = i;
                  if dt = sdt then do;
                     dtlabel = symget('trVisitStartDt');
                     imputedate = d9partial_SVSTDTC;
                  end;
                  else if dt = edt then do;
                     dtlabel = symget('trVisitEndDt');
                     imputedate = d9partial_SVENDTC;
                  end;
                  else do;
                     dtlabel = symget('trMidVisitDt');
                     imputedate = d9partial_MID;
                  end;                  
                  domain = "&temp";
                  output;
               end;
               format dt &IntDateFormat..;
               drop sdt edt;              
            run;           
         %end;
         
         %if ^&flag %then %do;
            data all;
               set temp;
            run;        
            %let flag = 1;          
         %end;
         %else %do;
            proc append force base = all data = temp;
            run;
         %end;

      %end;   

   %end;    
   %let i = %eval(&i+1);
%end;

proc datasets nolist;
   delete temp;
quit;

%if ^%sysfunc(exist(all)) %then %do;
   %put [MS_S_0182];
   %put ERROR: There are no available visit dates to analyze for this study.;
   %let exiterror=1;
   %goto exit;
%end;

proc sort data = all out = all(where = (dt > .));
   by usubjid dtlabel imputedate dt domain;
run;

%let nall = 0;
%if %sysfunc(exist(all)) %then %do;
   %nobsvars(all);
   %let nall = &nobs;
%end;

%if ^&nall %then %do;
   %put [MS_S_0182];
   %put ERROR: There are no available visit dates to analyze for this study.;
   %let exiterror=1;
   %goto exit;
%end;

data all(drop = domain);
   set all;
   by usubjid dtlabel imputedate dt domain;
   retain domainlist;
   if first.dt then domainlist = strip(domain);
   else if ^findw(domainlist, strip(domain)) then domainlist = catx(" ", domainlist, strip(domain));
   if last.dt;
run;

data all;
   length weekday $ 20;
   merge subject(in = ina) all(in = inb rename = (domainlist = domain));
   by usubjid;
   if ina and inb;
   if weekday(dt) = 1 then weekday = symget('trSunday');
   else if weekday(dt) = 2 then weekday = symget('trMonday');
   else if weekday(dt) = 3 then weekday = symget('trTuesday');
   else if weekday(dt) = 4 then weekday = symget('trWednesday');
   else if weekday(dt) = 5 then weekday = symget('trThursday');
   else if weekday(dt) = 6 then weekday = symget('trFriday');   
   else if weekday(dt) = 7 then weekday = symget('trSaturday');
   label weekday = "&trWeekday";
run;

data all1;
   length hlabel $ 50 hflag $ 20;
   set all;
      if dt = HOLIDAY("BOXING", year(dt)) and country in ("CA","CAN","CANADA") then hlabel = symget('trBoxingDay');
      else if dt = HOLIDAY("CANADA", year(dt)) and country in ("CA","CAN","CANADA") then hlabel = symget('trCanInd');
      else if dt = HOLIDAY("CANADAOBSERVED", year(dt)) and country in ("CA","CAN","CANADA") then hlabel = symget('trCanIndObs');
      else if dt = HOLIDAY("CHRISTMAS", year(dt)) then hlabel = symget('trChristmas');
      else if ((month(dt) = 12 and day(dt) = 26 and weekday(dt) = 2) or (month(dt) = 12 and day(dt) = 24 and weekday(dt) = 6)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trChristmasObs');
      else if dt = HOLIDAY("COLUMBUS", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trColumbusDayUSA');
      else if dt = HOLIDAY("EASTER", year(dt)) then hlabel = symget('trEaster');
      else if dt = HOLIDAY("EASTER", year(dt)) - 2 then hlabel = symget('trGoodFriday');
      else if dt = HOLIDAY("EASTER", year(dt)) + 1 then hlabel = symget('trEasterMonday');
      else if dt = HOLIDAY("LABOR", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trLaborDayUSA');
      else if dt = HOLIDAY("MLK", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trMLKJrBD');
      else if dt = HOLIDAY("MEMORIAL", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trMemDayUSA');
      else if dt = HOLIDAY("NEWYEAR", year(dt)) then hlabel = symget('trNewYearsDay');
      else if ((month(dt) = 1 and day(dt) = 2 and weekday(dt) = 2) or (month(dt) = 12 and day(dt) = 31 and weekday(dt) = 6)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trNewYearsDayObs');
      else if dt = HOLIDAY("THANKSGIVING", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trThanksgivingDayUSA');
      else if dt = HOLIDAY("THANKSGIVING", year(dt)) + 1 and country in ("US","USA","UNITED STATES") then hlabel = symget('trThanksgivingFriUSA');
      else if dt = HOLIDAY("THANKSGIVINGCANADA", year(dt)) and country in ("CA","CAN","CANADA") then hlabel = symget('trThanksgivingDayCAN');
      else if dt = HOLIDAY("USINDEPENDENCE ", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trIndependenceDayUSA');
      else if dt = HOLIDAY("USPRESIDENTS", year(dt)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trPresDayUSA');
      else if dt = HOLIDAY("VETERANS", year(dt)) and country in ("US","USA","UNITED STATES","CA","CAN","CANADA") then hlabel = symget('trVetRemDay');
      else if ((month(dt) = 11 and day(dt) = 10 and weekday(dt) = 6) or (month(dt) = 11 and day(dt) = 12 and weekday(dt) = 2)) and country in ("US","USA","UNITED STATES") then hlabel = symget('trVetDayObsUSA');
      else if dt = HOLIDAY("VICTORIA", year(dt)) and country in ("CA","CAN","CANADA") then hlabel = symget('trVictoriaDayCAN');
      else if month(dt) = 7 and day(dt) = 1 and country in ("CA","CAN","CANADA") then hlabel = symget('trCanadaDay');
      else if ((month(dt) = 7 and day(dt) = 2 and weekday(dt) = 2) or (month(dt) = 7 and day(dt) = 3 and weekday(dt) = 2)) and country in ("CA","CAN","CANADA") then hlabel = symget('trCanadaDayObs');
      else if dt = NWKDOM(1, 2, 9, year(dt))  and country in ("CA","CAN","CANADA") then hlabel = symget('trLabourDayCAN');
run;   

%let HLen1 = 50;
%if %sysfunc(fileexist(&RBMPath.holidays.sas7bdat)) %then %do;
   data holidays;
      set RBMPath.holidays;
   run;

   proc sql; 
      create table all2 as
         select * from all, holidays
         where all.country = holidays.country and startdate <= all.dt <= enddate;
   quit;
   
   %let _tid_ = %sysfunc(open(holidays));
   %let vn = %sysfunc(varnum(&_tid_,holiday));
   %let HLen1 = %sysfunc(varlength(&_tid_,&vn));
   %let _tid_ = %sysfunc(close(&_tid_));  
   
   data all2;
      set all2(drop = startdate enddate rename = (holiday = hlabel));
   run;  
%end;

data all1;
   length hlabel $ %sysfunc(max(50,&HLen1));
   set all1
   %if %sysfunc(fileexist(&RBMPath.holidays.sas7bdat)) %then %do; all2 %end;;
run;

proc datasets nolist;
   delete all;
quit;

proc sort data = all1;
   by usubjid dtlabel dt domain;
run;

data all1(drop = hlabel);
   length hlabel2 $ 1000;
   set all1;
   by usubjid dtlabel dt;
   retain hlabel2;
   if first.dt then hlabel2 = hlabel;
   else hlabel2 = catx(",",hlabel2,hlabel);
   if last.dt; 
run;

data all1;
   length imputeddate $ 3;
   set all1(rename = (hlabel2 = hlabel));
   if missing(hlabel) then hlabel = symget('trNoHoliday');
   if hlabel = symget('trNoHoliday') then hflag = symget('trNoHoliday');
   else hflag = symget('trHoliday');
   label hlabel = "&trHoliday" hflag = "&trHolidayFlag";   
   if imputedate = 1 then imputeddate = "&trYes";
   else if imputedate = 0 then imputeddate = "&trNo";
   format dt &IntDateFormat..;
   label dt = "&trDate" dtlabel = "&trDateLabel" domain = "&trDOMAIN" imputeddate = "&trImputedDate";
run;

%PrepOutDS(WeekendHoliday_OutData,OutDataName, &Outname.WeekHol);
%let includeds = &includeds &OutDataName..sas7bdat,;

data &WeekendHoliday_OutData;
   set all1;
run;

%let hlabelLabel = hlabel;
%CheckLabel(&WeekendHoliday_OutData,hlabelLabel);
%let hflagLabel = hflag;
%CheckLabel(&WeekendHoliday_OutData,hflagLabel);
%let WEEKDAYLabel = WEEKDAY;
%CheckLabel(&WeekendHoliday_OutData,WEEKDAYLabel);
%let DTLABELlabel = dtlabel;
%CheckLabel(&WeekendHoliday_OutData,DTLABELlabel);

%let DoVolc = 1;
data dt;
   length dttest $ 20;
   set all1(where = (domain not in ("MH","DM") and imputeddate = "&trNo"));
   dttest = "&trHoliday"; output;   
   dttest = "&trWeekend"; output;   
run;
%nobsvars(dt)

%if ^&nobs %then %let DoVolc = 0;
%else %do;
   proc datasets nolist;
      delete all1;
   quit;

   data dt(keep = usubjid dttest visitnum dt dtlabel siteid country domain dtstresc int);
      length dtstresc $ 20;
      set dt;
      if dttest = "&trHoliday" then do;
         dtstresc = hflag;
      end;
      else do;
         if weekday in ("&trSaturday","&trSunday") then dtstresc = "&trWeekend";
         else dtstresc = "&trMonFri";
      end;
      label dttest = "&trTest" dtstresc = "&trSTRESC";
      int = 1;
   run;

   %let byvar_dt = ;
   %let temp = dt;
   %GETFREQS(variable = dtstresc);

   proc datasets nolist;
      delete dt;
   run;

   proc sort data = dt2;
      by dttest;
   run;

   %if %eval(&NumSites > 1) %then %do;
      %let firstflag = 0;
      %let i = 1;
      %do %while(%length(%scan(&SiteList,&i,' ')));
         %let tempsite = %scan(%upcase(&SiteList),&i," ");
         %let tempsitesize = %scan(%upcase(&SiteSamp),&i," ");
         %let trSiteTS = %nrbquote(%LocaleFill(&trSiteX, f1=%nrbquote(&tempsite)));
         %let trNotSiteTS = %nrbquote(%LocaleFill(&trNotSiteX, f1=%nrbquote(&tempsite)));
         data temp;
            length Label $ 30;
            set dt2;
            if siteid = "&tempsite" then Label = "&trSiteTS";
            else Label = "&trNotSiteTS";
            label Label = "&trLabel";
         run;

         proc freq noprint data = temp;
            tables label * dtstresc / chisq sparse out = stats(drop = PERCENT PCT_COL) outpct;
            output out = rowmean(keep = dttest XP2_FISH) fisher;
            weight count;
            by dttest; 
         run;   

         data stats;
            length siteid $ &SiteidLen;
            set stats(rename = (PCT_ROW = Percent));
            SITEID = "&tempsite";
            SITESAMP = &tempsitesize;
            label percent = "&trPercentofMeasurements" siteid = "&SITEIDlabel" sitesamp = "&trSiteSampleSize";
         run;

         proc sort data = stats out = stats2;
            by dttest dtstresc label;
         run;

         data stats2;
            set stats2;
            by dttest dtstresc label;
            retain diff;
            if first.dtstresc then diff = .;
            if first.dtstresc and ^last.dtstresc and kindex(label, "&trNot") then diff = -percent;
            else if last.dtstresc and ^first.dtstresc and ^kindex(label, "&trNot") then diff = diff + percent;
            if last.dtstresc and diff > .;
            label diff = "&trDiffProps";
         run;

         data stats2;
            set stats2;
            diff2 = abs(diff);
         run;

         *** Take largest absolute difference, but in direction of single site if tied;
         proc sort data = stats2;
            by dttest diff2 diff; 
         run;

         data stats2(drop = diff2 count percent);
            set stats2;
            by dttest; 
            if last.dttest;
         run;

         %nobsvars(rowmean);
         %if &nobs %then %do;
            data stats2(drop = stresc);     
               merge stats2(in = ina) rowmean(rename = (XP2_FISH = Fisher));
               by dttest;
               if ina;
               if missing(Fisher) then Fisher = 1;
               Nlog10Fisher = -log10(Fisher);
               label Fisher = "&trFisherExact" Nlog10Fisher = "&trNegLog10(&trFisherExact)";
            run;
         %end;

         %if ^&firstflag %then %do;
            data summary_volcano;
               set stats2;
            run;
            data summary_WeekendHoliday;
               set stats;
            run;
            %let firstflag = 1;
         %end;
         %else %do;   
            proc append force base = summary_volcano data = stats2;
            run;
            proc append force base = summary_WeekendHoliday data = stats;
            run;     
         %end;

         proc datasets nolist;
            delete temp stats stats2 rowmean;
         quit;

         %let i = %eval(&i+1);
      %end;

      %FDRPVALUES(inpv = Fisher, nlog10p = Nlog10Fisher);

      %let WeekendHoliday_OutData_v =;    
      %PrepOutDS(WeekendHoliday_OutData_v,OutDataName_v, &Outname.WeekHol, _vol);
      %let includeds = &includeds &OutDataName_v..sas7bdat,;

      data &WeekendHoliday_OutData_v;
         merge summary_volcano(in = ina) country(in = inb);
         by siteid;
         if ina and inb;
         drop dtstresc;
         label dttest = "&trTest";
      run;

      %let WeekendHoliday_OutData_d = ;      
      %PrepOutDS(WeekendHoliday_OutData_d,OutDataName_d, &Outname.WeekHol, _hist);
      %let includeds = &includeds &OutDataName_d..sas7bdat,;

      data &WeekendHoliday_OutData_d;
         set summary_WeekendHoliday;
         label dttest = "&trTest" dtstresc = "&trSTRESC";
      run;  
   %end;
%end;

proc datasets nolist;
   delete summary_WeekendHoliday summary_volcano dt2;
quit;

* create JSL file;
%PrepOutFile(WeekendHolidayScript,WeekendHolidayScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &WeekendHolidayScript;
%let FullAPName=%str(Weekdays and Holidays);

* put JSL statements;
data _null_;
   file "&WeekendHolidayScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   
   put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
   put "try(objRefNS:WhereState=""\[&WhereForReports]\"");";
   put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
   put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);"; 

   %if &DoVolc %then %do;
      %if %eval(&NumSites > 1) %then %do;
         put "objRefNS:SiteLab = ""&SITEIDLabel"";";
         put "objRefNS:TestLab = ""&trTEST"";";
         put "objRefNS:STRESC = ""&trSTRESC"";";   
         put "objRefNS:hist = ""&ClientOutPath.&OutDataName_d..sas7bdat"";";
         put "objRefNS:volcano=Function({},{default local},";
         put "   objRefNS:dvolcano = open(""&ClientOutPath.&OutDataName_v..sas7bdat"",invisible);";
         put "  Column(""&SITEIDLabel"") << Label(1);";
         put "  Column(""&trTest"") << Label(1);";
         put "  Column(""&trSiteSampleSize"") << Label(1);";         
         put "   VListBox(";

         %if %sysfunc(findw(&GetVis, SV)) %then %do;
            put "   objRefNS:T0 = TextBox(locale:get(""Includesalldatesthatoccur_Text""),<<Set Wrap(1000), <<Font Color(5)),"; 
         %end;   
         put "   objRefNS:T1 = TextBox(""&trAnalysisPerformedUsing %sysfunc(lowcase(&trVisitDate))."",<<Set Wrap(1000), <<Font Color(5)),";
         put "   objRefNS:T2 = TextBox(locale:get(""Testsbaseduponnonimputeddates_Text""),<<Set Wrap(1000), <<Font Color(5)),";
         %if %length(&RawCutOff) %then %do;
            put "   objRefNS:T3 = TextBox(locale:fill(locale:get(""XLineDrawnatMinusLogpX_Text""), {locale:get(""FDR_Text""), ""%sysfunc(round(&RawCutOff,0.0001))""})||"" (alpha=&AlphaValue)""),"; 
         %end;
         %else %do;
            put "   objRefNS:T3 = TextBox(locale:fill(locale:get(""NoPvalsSigAtAlphaXAfterAppXAdj_Text""), {""&AlphaValue"", locale:get(""FDR_Text"")}),<<Set Wrap(1000)),"; 
         %end;
         put "   HListBox(";
         put "    objRefNS:Filter = objRefNS:dvolcano << Data Filter(<<Conditional(1),";
         put "       Add Filter(Columns(""&COUNTRYLabel"",""&SITEIDLabel"", ""&trTest"")),";
         put "       Mode( Show(1), Include(1) )";
         put "    ),";               
         put "   objRefNS:VolcPlot = Graph Builder(";
         put "    Auto Stretching(0),";
         put "    Automatic Recalc(1),";
         put "   Show Control Panel( 0 ),";
         put "   Variables(";
         put "    X( :&trDiffProps ),";
         put "    Y( :Name( ""&trNegLog10(&trFisherExact)"" ) ),";
         put "    Color( :&SITEIDLabel )";
         put "   ),";
         put "   Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
         put "   SendToReport(";
         put "    Dispatch(";
         put "       {},";
         put "       ""Graph Builder"",";
         put "       OutlineBox,";
         put "       {Set Title( locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""WeekdaysandHolidays_Text"")}) )}";
         put "    ),";
         put "    Dispatch(";
         put "       {},";
         put "       ""&trDiffProps"",";
         put "       ScaleBox,";
         put "       {}";
         put "    ),";
         %if %length(&RawCutOff) %then %do;
            put "    Dispatch(";
            put "       {},";
            put "       ""&trNegLog10(&trFisherExact)"",";
            put "       ScaleBox,";
            put "       {Add Ref Line(";
            put "          &RawCutOff,";
            put "          Dashed,";
            put "          ""Red"",";
            put "          """",";
            put "          2";
            put "       )}";
            put "    ),";
         %end;
         put "    Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
         put "    Dispatch( {}, ""Graph Builder"", FrameBox, {Marker Size( 6 )} )";
         put "   )";
         put "   )";
         put "));";
         put ");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:dvolcano""}});";
         put "InsertInto(objRefNS:DataNameList,{{""&trVolcanoPlot""}});";   
         put "InsertInto(objRefNS:TabButtonList, expr(locale:fill(locale:get(""XVolcanoPlot_Text""),{locale:get(""WeekdaysandHolidays_Text"")})));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:volcano()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T1,objRefNS:T2,objRefNS:T3,objRefNS:VolcPlot}});";
      %end;
   %end;
   
   put "objRefNS:ddw=Function({},{default local},";
   put "   objRefNS:dmd=open(""&ClientOutPath.&OutDataName..sas7bdat"",invisible);";
   put "   Column(""&SubjectLabel"") << Label(1);";          
   put " if (N Rows(objRefNS:dmd) != 0,";  
   put "    VListBox(";
   %if %sysfunc(findw(&GetVis, SV)) %then %do;
      put "   objRefNS:T0 = TextBox(locale:get(""Includesalldatesthatoccur_Text""),<<Set Wrap(1000), <<Font Color(5)),"; 
   %end;
   put "      HListBox(";
   put "        objRefNS:Filter = objRefNS:dmd << Data Filter(<<Conditional(1),";
   put "            Add Filter(";
   put "               Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIDlabel""), Name(""&hflaglabel""), :Name(""&trImputedDate""), :Name(""&hlabelLabel""), :Name(""&WEEKDAYLabel"")),"; 
   put "            ),";
   put "            Mode( Show(1), Include(1) )";
   put "         ),";
   put "        objRefNS:dp = Distribution(";
   put "            Automatic Recalc(1),";
   put "             Nominal Distribution(Column(:Name(""&hlabelLabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "             Nominal Distribution(Column(:Name(""&WEEKDAYLabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "             Nominal Distribution(Column(:Name(""&COUNTRYLabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "             Nominal Distribution(Column(:Name(""&SITEIDlabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "             Nominal Distribution(Column(:Name(""&trDomain"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "             Nominal Distribution(Column(:Name(""&DTLABELlabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
   put "            SendToReport(";
   put "               Dispatch( {""&hlabelLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
   put "               Dispatch( {""&WEEKDAYLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
   put "               Dispatch( {""&COUNTRYLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
   put "               Dispatch( {""&SITEIDlabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";     
   put "               Dispatch( {""&trDomain""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";     
   put "               Dispatch( {""&DTLABELlabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";    
   put "            )";
   put "        ),";
   put "try(Column(""&WEEKDAYLabel"") <<Set Property(""Value Ordering"",{""&trSunday"",""&trMonday"",""&trTuesday"",""&trWednesday"",""&trThursday"",""&trFriday"",""&trSaturday""}));";
   put "      )"; 
   put "    )";
   put "    , //else ";
   put "    VListBox(textbox(locale:get(""NoWeekendsorHolidayswereIdentified_Text""),<<Font Color(5), <<Set Wrap(400))";
   put "    )";
   put "    )";
   put ");";
   put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:DataNameList,{{""&trDistribution""}});";
   put "InsertInto(objRefNS:TabButtonList, Expr(locale:get(""DistributionDetails_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddw()));";
   put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
   put "InsertInto(objRefNS:TabOpenList,1);";
   %if %sysfunc(findw(&GetVis, SV)) %then %do;
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:T0,objRefNS:dp}});";
   %end;
   %else %do;
      put "InsertInto(objRefNS:CreateReportList,{{objRefNS:dp}});";
   %end;
   
   put "objRefNS:fpdd = evalexpr(Function({},{default local},  this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); reviewRefNS = namespace(reportRefNS:getParent()); createFilterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\FilterSubjects.jsl"",<< newContext); createFilterFunc(reviewRefNS << getName);));";  
   put "objRefNS:cpdd = evalexpr(Function({}, {default local}, this = namespace(expr(objRefNS<<getName)); clusterFunc = Include(""$CLINICAL_HOME\ProcessLibrary\ClusterSubjects.jsl"",<< New Context,<< ""Names Default To Here""); clusterFunc(this:getParent());));";
   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  
   put "objRefNS:spdd =  Function({}, {},";
   put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";  
   put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
   
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{}};";
   put "objRefNS:ActionCodeList ={{},{}};";
   put "objRefNS:ActionDataTableList = {{},{}};";

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromVolcanothenClick_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSites_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSite()));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""BarChart_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:HolBar()));";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:dvolcano""}});";
 
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectRecordsfromDomainTabsthenClick_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:spdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""ClusterSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:cpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:dmd""}});";
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""CreateSubjectFilter_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:fpdd()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{""objRefNS:dmd""}});";
   
   put "objRefNS:ActionSelectionCountList = {{1,1},{1,1,3,1}};";
   put "objRefNS:ActionIconList = {{""&IconPath.DataSetHighlightRows.gif"", ""&IconPath.BarGraph.gif""},";
   put "   {""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"",""&IconPath.HierarchicalClustering.gif"",""&IconPath.AddFilter.gif""}};";  
   put "objRefNS:ActionRequirementFlagList = {{0,0},{3, 0, 3, 2}};";
      
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &WeekendHolidayScriptName, "&includeds", text/plain);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend WeekendHoliday;
%WeekendHoliday;

************************************;
************************************;
************************************;