/*
Name: SAS Data Quality and Fraud Macros
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro ADDCOUNTRYTODATA(varlist = );
   %if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) > 0 %then %do;
      proc sort data = &in_adsl out = subject(keep = usubjid siteid &varlist);
         by usubjid;
      run;

      %LOADDATA(domlist = DM,mergesupp = 0,checkreview=0);
      proc sort data = &in_dm out = dm;
         by usubjid;
      run;

      data subject;
         merge subject(in = ina) dm(keep = usubjid country);
         by usubjid;
         if ina;
      run;
   %end;
   %else %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) %then %do;
      proc sort data = &in_adsl out = subject(keep = usubjid siteid &varlist country);
         by usubjid;
      run;
   %end;
   %else %do;
      proc sort data = &in_adsl out = subject(keep = usubjid siteid &varlist);
         by usubjid;
      run;

      data subject;
         length Country $ 24;
         set subject;
         country = symget('trNotProvided');
         label country = "&trCountry";
      run;
   %end;
   
   %global SUBJECTlabel SITEIDlabel COUNTRYLabel;
   
   %let SUBJECTlabel = USUBJID;
   %CheckLabel(subject,SUBJECTlabel);
   %let SITEIDlabel = SITEID;
   %CheckLabel(subject,SITEIDlabel);
   %let COUNTRYLabel = COUNTRY;
   %CheckLabel(subject,COUNTRYLabel);  
%mend ADDCOUNTRYTODATA;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro DQFTRANS;
   %global trNotProvided trCountry trAgeDifference trAllSubjects trAnalysisPerformedUsing
           trBetween trCovariateGroupID trDate trDateLabel trDemogDate trDiffCorr trDiffCorrTest
           trDiffCorrTestp trDiffMissingRates trDiffTrCorr trDigit trDistance trF 
           trFalseDiscoveryRatepvalue trFirstDigit trFisherExact trFrequencyofRecordswithAvail
           trFrequencyofRecordswithMissing trHeightDifference trIncludeVisitandTime trJMPDark 
           trLabel trLastDigit trLocOfMeas trM trMahDist trMaleGender trMaxPercentDifference
           trMedHistDate trMethOfTestExam trMidVisitDt trMissFlag trMissingStatus trMVV trNA
           trNegLog10 trNegLog10p trNMiss trNo trNonMissing trNOSMV trNot trNotSite trNotSiteX
           trNumberofPairs trPearson trPercentofMeasurements trPercentofTotalVisits trPlannedTime 
           trPOSMV trPosOfSubj trProblematicDate trPropSubs trRace trRawRowMeanScorePVal trREMCMHPV
           trSex trSite trSiteFlag trSiteX trSpecialVisit trSpecType trStdDev trStudyDayofStartofVisit 
           trTestName trUnscheduled trVariable trVisitDate trVisitDateProblem trVisitEndDt trVisitNum
           trVisitStartDt trWeightDifference trWithin tryear tryears trYes trDOMAIN trPValue trN trMean 
           trSTD trDiffMean trVarRatio trStdDiffMean trCategory trSubcategory trOutlier trDiffProps trSTRESN
           trLowerOutlier trUpperOutlier trSkewRatio trKurtRatio trSkewness trKurtosis trSiteActiveDate 
           trStudyDay trCumulativeCount trCount trAll trSTRESC trPGenAssoc trEnrollmentbyX trOverallEnrollment 
           trTreatmentRuns trCumulativeCountforX trCountforX trRunCountforX trRunIDforX trActualArm trPlannedArm 
           trCumulativeCountforA trCountforA trRunCountforA trRunIDforA trCumulativeCountforP trCountforP 
           trRunCountforP trRunIDforP trEnrollmentbyA trEnrollmentbyP trRandomized trNonOutlier trOutlierAbove
           trOutlierBelow trOutlierType trTest trEthnicity trBirthday trVisit trDifference trStdMeanDifference trSiteSampleSize
           trImputedDate trlog2 trORRES trFrequency
   ;

   %let UniqueSubjectIdentifier = %nrbquote(%LocaleGet(UniqueSubjectIdentifier_Text));
   %let trlog2 = %nrbquote(%LocaleGet(log2_Text));
   %let trNotProvided = %nrbquote(%LocaleGet(NotProvided_Text));
   %let trCountry = %nrbquote(%LocaleGet(Country_Text));
   %let trBirthday = %nrbquote(%LocaleGet(Birthday_Text));
   %let trMVV = %nrbquote(%LocaleGet(MatchingVariableValue_Text));
   %let trSex = %nrbquote(%LocaleGet(Sex_Text));
   %let trRace = %nrbquote(%LocaleGet(Race_Text));
   %let trAllSubjects = %nrbquote(%LocaleGet(AllSubjects_Text));
   %let trCovariateGroupID = %nrbquote(%LocaleGet(CovariateGroupID_Text));
   %let trMaleGender = %nrbquote(%LocaleGet(MaleGender_Text));
   %let trM = %nrbquote(%LocaleGet(M_Text));
   %let trF = %nrbquote(%LocaleGet(F_Text));
   %let tryear = %nrbquote(%LocaleGet(year_Text));
   %let tryears = %nrbquote(%LocaleGet(years_Text));
   %let trWithin = %nrbquote(%LocaleGet(Within_Text));
   %let trBetween = %nrbquote(%LocaleGet(Between_Text));
   %let trSiteFlag = %nrbquote(%LocaleGet(SiteFlag_Text));
   %let trAgeDifference = %nrbquote(%LocaleGet(AgeDifference_Text));
   %let trHeightDifference = %nrbquote(%LocaleGet(HeightDifference_Text));
   %let trWeightDifference = %nrbquote(%LocaleGet(WeightDifference_Text)); 
   %let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));
   %let trDemogDate = %nrbquote(%LocaleGet(DemographyDate_Text));
   %let trMedHistDate = %nrbquote(%LocaleGet(MedicalHistoryDate_Text));
   %let trVisitDate = %nrbquote(%LocaleGet(VisitDate_Text));
   %let trVisitNum = %nrbquote(%LocaleGet(VisitNumber_Text));
   %let trVisitStartDt = %nrbquote(%LocaleGet(VisitStartDate_Text));
   %let trVisitEndDt = %nrbquote(%LocaleGet(VisitEndDate_Text));
   %let trMidVisitDt = %nrbquote(%LocaleGet(MidVisitDate_Text));
   %let trVariable = %nrbquote(%LocaleGet(Variable_Text));
   %let trLabel = %nrbquote(%LocaleGet(Label_Text));
   %let trNOSMV = %nrbquote(%LocaleGet(NumberofSubjectsMissingValues_Text));
   %let trPOSMV = %nrbquote(%LocaleGet(PercentageofSubjectsMissingValues_Text));
   %let trMissFlag = %nrbquote(%LocaleGet(MissingFlag_Text));
   %let trDistance = %nrbquote(%LocaleGet(Distance_Text));
   %let trDate = %nrbquote(%LocaleGet(Date_Text));
   %let trDateLabel = %nrbquote(%LocaleGet(DateLabel_Text));
   %let trYes = %nrbquote(%LocaleGet(Yes_Text));
   %let trNo = %nrbquote(%LocaleGet(No_Text));
   %let trSpecialVisit = %nrbquote(%LocaleGet(SpecialVisit_Text));
   %let trVisitDateProblem = %nrbquote(%LocaleGet(VisitDateProblem_Text));
   %let trProblematicDate = %nrbquote(%LocaleGet(ProblematicDate_Text));
   %let trSiteX = %nrbquote(%LocaleGet(SiteX_Text));
   %let trNotSiteX = %nrbquote(%LocaleGet(NotSiteX_Text));
   %let trRawRowMeanScorePVal = %nrbquote(%LocaleGet(RawRowMeanScorePvalue_Text));
   %let trNegLog10 = %nrbquote(%LocaleGet(Neglog10_Text));
   %let trREMCMHPV = %nrbquote(%LocaleGet(RawExactMonteCarloMHPvalue_Text));
   %let trPercentofTotalVisits = %nrbquote(%LocaleGet(PercentofTotalVisits_Text));
   %let trStudyDayofStartofVisit = %nrbquote(%LocaleGet(StudyDayofStartofVisit_Text));
   %let trNot = %nrbquote(%LocaleGet(Not_Text));
   %let trMaxPercentDifference = %nrbquote(%LocaleGet(MaxPercentDifference_Text));
   %let trFalseDiscoveryRatepvalue = %nrbquote(%LocaleGet(FalseDiscoveryRatepvalue_Text));
   %let trTestName = %nrbquote(%LocaleGet(TestName_Text));
   %let trNumberofPairs = %nrbquote(%LocaleGet(NumberofPairs_Text));
   %let trPearson = %nrbquote(%LocaleGet(PearsonCorrelation_Text));
   %let trDiffCorr = %nrbquote(%LocaleGet(DifferenceinCorrelations_Text));
   %let trDiffTrCorr = %nrbquote(%LocaleGet(DifferenceinFisherTransformedCorrelations_Text));
   %let trDiffCorrTest = %nrbquote(%LocaleGet(DifferenceinCorrelationsTest_Text));
   %let trDiffCorrTestp = %nrbquote(%LocaleGet(DifferenceinCorrelationsTestp_Text));
   %let trNegLog10p = %nrbquote(%LocaleGet(Neglog10pvalue_Text));
   %let trSite = %sysfunc(transtrn(&trSiteX,%str({{^1}}),%str()));
   %let trNotSite = %sysfunc(transtrn(&trNotSiteX,%str({{^1}}),%str()));
   %let trNA = %nrbquote(%LocaleGet(NA_Text));
   %let trPlannedTime = %nrbquote(%LocaleGet(PlannedTimePointName_Text));
   %let trFisherExact = %nrbquote(%LocaleGet(FisherExactTest_Text));
   %let trDiffMissingRates = %nrbquote(%LocaleGet(DifferenceinMissingRates_Text));
   %let trMissingStatus = %nrbquote(%LocaleGet(MissingStatus_Text));
   %let trFrequencyofRecordswithAvail = %nrbquote(%LocaleGet(FrequencyofRecordswithAvailable_Text));
   %let trFrequencyofRecordswithMissing = %nrbquote(%LocaleGet(FrequencyofRecordswithMissing_Text));
   %let trPropSubs = %nrbquote(%LocaleGet(ProportionofSubjectswithData_Text));
   %let trIncludeVisitandTime = %nrbquote(%LocaleGet(IncludesvisitsandtimeswhereatleastX_Text));   
   %let trNonMissing = %nrbquote(%LocaleGet(NonMissingComponents_Text));
   %let trDigit = %nrbquote(%LocaleGet(Digit_Text));
   %let trFirstDigit = %nrbquote(%LocaleGet(FirstDigit_Text));
   %let trLastDigit = %nrbquote(%LocaleGet(LastDigit_Text));
   %let trLocOfMeas = %nrbquote(%LocaleGet(LocationofMeasurement_Text));
   %let trMethOfTestExam = %nrbquote(%LocaleGet(MethodofTestorExamination_Text));
   %let trPosOfSubj = %nrbquote(%LocaleGet(PositionofSubject_Text));
   %let trSpecType = %nrbquote(%LocaleGet(SpecimenType_Text));
   %let trPercentofMeasurements = %nrbquote(%LocaleGet(PercentofMeasurements_Text));
   %let trAnalysisPerformedUsing = %nrbquote(%LocaleGet(AnalysisPerformedUsing_Text)); 
   %let trStdDev = %nrbquote(%LocaleGet(StdDevNS_Text));
   %let trNMiss = %nrbquote(%LocaleGet(NMiss_Text));  
   %let trJMPDark = %nrbquote(%LocaleGet(JMPDark_Text));
   %let trMahDist = %nrbquote(%LocaleGet(MahalanobisDistance_Text));
   %let trDOMAIN = %nrbquote(%LocaleGet(Domain_Text));
   %let trPValue = %nrbquote(%LocaleGet(PValue_Text));
   %let trN = %nrbquote(%LocaleGet(N_Text));
   %let trMean = %nrbquote(%LocaleGet(Mean_Text));
   %let trSTD = %nrbquote(%LocaleGet(StandardDeviation_Text));
   %let trDiffMean = %nrbquote(%LocaleGet(DifferenceinMeans_Text));
   %let trVarRatio = %nrbquote(%LocaleGet(RatioofVariances_Text));
   %let trStdDiffMean = %nrbquote(%LocaleGet(StandardizedDifferenceinMeans_Text));
   %let trCategory = %nrbquote(%LocaleGet(Category_Text));
   %let trSubcategory = %nrbquote(%LocaleGet(Subcategory_Text));
   %let trOutlier = %nrbquote(%LocaleGet(Outlier_Text));
   %let trDiffProps = %nrbquote(%LocaleGet(DifferenceinProportions_Text));
   %let trSTRESN = %nrbquote(%LocaleGet(NumericResultFindinginStandardUnits_Text));
   %let trLowerOutlier = %nrbquote(%LocaleGet(LowerOutlierLimit_Text));
   %let trUpperOutlier = %nrbquote(%LocaleGet(UpperOutlierLimit_Text)); 
   %let trSkewRatio = %nrbquote(%LocaleGet(RatioofSkewness_Text));
   %let trKurtRatio = %nrbquote(%LocaleGet(RatioofKurtosis_Text));
   %let trSkewness = %nrbquote(%LocaleGet(Skewness_Text));
   %let trKurtosis = %nrbquote(%LocaleGet(Kurtosis_Text));
   %let trSiteActiveDate = %nrbquote(%LocaleGet(SiteActiveDate_Text));
   %let trStudyDay = %nrbquote(%LocaleGet(StudyDay_Text));
   %let trCount = %nrbquote(%LocaleGet(Count_Text));
   %let trCumulativeCount = %nrbquote(%LocaleGet(CumulativeCount_Text));
   %let trAll = %nrbquote(%LocaleGet(All_Text));
   %let trSTRESC = %nrbquote(%LocaleGet(CharacterResultsinStandardFormat_Text));
   %let trORRES = %nrbquote(%LocaleGet(ResultsinOriginalUnits_Text));
   %let trPGenAssoc = %nrbquote(%LocaleGet(PValueGeneralAssociation_Text));
   %let trEnrollmentbyX = %nrbquote(%LocaleGet(EnrollmentbyX_Text));
   %let trOverallEnrollment = %nrbquote(%LocaleGet(OverallEnrollment_Text));
   %let trTreatmentRuns = %nrbquote(%LocaleGet(TreatmentRuns_Text));
   %let trCumulativeCountforX = %nrbquote(%LocaleGet(CumulativeCountforX_Text));
   %let trCountforX = %nrbquote(%LocaleGet(CountforX_Text));
   %let trRunCountforX = %nrbquote(%LocaleGet(RunCountforX_Text));
   %let trRunIDforX = %nrbquote(%LocaleGet(RunIDforX_Text));
   %let trActualArm = %nrbquote(%LocaleGet(ActualArm_Text));
   %let trPlannedArm = %nrbquote(%LocaleGet(PlannedArm_Text));
   %let trCumulativeCountforA = %nrbquote(%LocaleFill(&trCumulativeCountforX, &trActualArm));   
   %let trCountforA = %nrbquote(%LocaleFill(&trCountforX, &trActualArm));
   %let trRunCountforA = %nrbquote(%LocaleFill(&trRunCountforX, &trActualArm));
   %let trRunIDforA = %nrbquote(%LocaleFill(&trRunIDforX, &trActualArm));
   %let trEnrollmentbyA = %nrbquote(%LocaleFill(&trEnrollmentbyX, &trActualArm));   
   %let trCumulativeCountforP = %nrbquote(%LocaleFill(&trCumulativeCountforX, &trPlannedArm));  
   %let trCountforP = %nrbquote(%LocaleFill(&trCountforX, &trPlannedArm));
   %let trRunCountforP = %nrbquote(%LocaleFill(&trRunCountforX, &trPlannedArm));
   %let trRunIDforP = %nrbquote(%LocaleFill(&trRunIDforX, &trPlannedArm)); 
   %let trEnrollmentbyP = %nrbquote(%LocaleFill(&trEnrollmentbyX, &trPlannedArm));
   %let trRandomized = %nrbquote(%LocaleGet(Randomized_Text));
   %let trNonOutlier = %nrbquote(%LocaleGet(NonOutlier_Text));
   %let trOutlierAbove = %nrbquote(%LocaleGet(OutlierAbove_Text));
   %let trOutlierBelow = %nrbquote(%LocaleGet(OutlierBelow_Text));
   %let trOutlierType = %nrbquote(%LocaleGet(OutlierType_Text));
   %let trTest = %nrbquote(%LocaleGet(Test_Text));
   %let trEthnicity = %nrbquote(%LocaleGet(Ethnicity_Text));
   %let trVisit = %nrbquote(%LocaleGet(Visit_Text));
   %let trDifference = %nrbquote(%LocaleGet(Difference_Text));
   %let trStdMeanDifference = %nrbquote(%LocaleGet(StandardizedMeanDifference_Text));
   %let trSiteSampleSize = %nrbquote(%LocaleGet(SiteSampleSize_Text));
   %let trImputedDate = %nrbquote(%LocaleGet(ImputedDate_Text));
   %let trFrequency = %nrbquote(%LocaleGet(Frequency_Text));
%mend DQFTRANS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro GETSITELISTS(errflg = 0, cerrflg = 0);
   %global SiteList SiteSamp numsites;

   %let SiteList = ;
   %let SiteSamp = ;

   %if &cerrflg %then %do;
      %if &SiteCount = 1 %then %do;
         %put ERROR: At least 2 subjects are required to cluster within site.;
         %let exiterror = 1;
      %end;
   %end;   

   proc sql noprint;
      create table siteids as
         select distinct siteid, count(siteid) as count 
         from subject
         group by siteid
         having calculated count >= &SiteCount
         order by siteid;
         %let numsites = &sqlobs;   
      select siteid
         into :SiteList separated by ' '
         from siteids
         order by siteid;     
      select count
         into :SiteSamp separated by ' '
         from siteids
         order by siteid;     
   quit;
   
   %global SiteidLen; 
   %let _tid_ = %sysfunc(open(siteids));
   %let vn = %sysfunc(varnum(&_tid_,siteid));
   %let SiteidLen = %sysfunc(varlength(&_tid_,&vn));
   %let _tid_ = %sysfunc(close(&_tid_));

   data siteids;
      set siteids(keep = siteid count rename = (count = SiteSamp));
      label sitesamp = "&trSiteSampleSize";
   run;
         
   %if &numsites = 0 %then %do;
      %put [MS_S_0209];
      %put ERROR: There are no sites available. Analysis cannot be performed.;
      %put ERROR: Try relaxing the requirement on the minimum number of subjects per site.;
      %let exiterror = 1;
   %end;
   
   %if &errflg %then %do;
      %if &numsites = 1 %then %do;
         %put [MS_S_0210];
         %put ERROR: There is only one site available. Analysis cannot be performed.;
         %put ERROR: Try relaxing the requirement on the minimum number of subjects per site.;
         %let exiterror = 1;
      %end;
   %end;
      
   proc sql noprint;
      create table country as
         select distinct siteid, country
         from subject
         order by siteid;
   quit;

   data country;
      set country;
      by siteid;
      if first.siteid;
   run;  
%mend GETSITELISTS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro INTEVENT(list = , predose = 0);
   %let i = 1;
   %do %while(%length(%scan(&list,&i,' ')));
      %let temp = %scan(&list,&i,' ');

      %if %sysfunc(indexw(&&&temp.VARS, &temp.DECOD, ' ')) %then %do;
         %let availabledomains = &availabledomains &temp;
         
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         proc sort data = &&in_&temp(where = (not missing (&temp.DECOD))) out = &temp;
            by usubjid &temp.DECOD;
         run;

         %if &predose %then %do;       
            %if %sysfunc(indexw(&&&temp.VARS, &temp.STDTC, ' ')) and &temp ne MH %then %let dtc = &temp.STDTC;
            %else %let dtc = ;
         %end;
         %else %let dtc = ;         
         
         *** Delete prespecified not occuring records ***;
         data &temp(keep = usubjid &temp.DECOD count &dtc);
            set &temp;
            count = 1;
            %PRESPECIFY(dom = &temp);
         run;

         %if &predose %then %do;
            *** if possible, limit measurements to those prior to dosing ***;
            %if %length(&dtc) %then %do;
               %TREATMENTEMERGENT(dataset = &temp, domain = &temp, date = &temp.STDTC);

               data &temp(drop = &temp.STDTC _TRTEMRG);
                  set &temp;
                  where _TRTEMRG = 0;
               run;
            %end;
         %end;

         proc freq noprint data = &temp;
            tables &temp.DECOD / out = outname(drop = count percent);
         run;
                  
         data outname;
            length &temp.short $ 8;
            set outname;
            &temp.short = cats("&temp",_n_);   
         run;

         proc sort data = &temp;
            by usubjid &temp.DECOD;
         run;

         ods exclude all;
         ods noresults;
         proc means noprint nway data = &temp;
            class usubjid &temp.DECOD;
            var count;
            output out = &temp(drop = _type_ _freq_) n = count;
         run;
         ods select all;
         ods results;

         proc sql noprint;      
            create table events as
               select distinct &temp.DECOD
               from &temp
               order by &temp.DECOD;      
         quit;

         proc sql;
            create table events1 as
            select aa.usubjid, bb.&temp.DECOD
            from subject as aa, events as bb
            order by aa.usubjid, bb.&temp.DECOD;
         quit;

         data &temp;
           merge &temp(in = ina) events1(in = inb);
           by usubjid &temp.DECOD;
           if inb and not ina then countflag = 1;
         run;

         proc sort data = &temp;
            by &temp.decod;
         run;
         
         proc means noprint nway data = &temp;
            class &temp.DECOD;
            var count;
            output out = droprare(drop = _type_ _freq_) n = n nmiss = nmiss;
         run;
         
         data droprare(keep = &temp.DECOD);
            set droprare;
            keeppct = round(100 * n /(n + nmiss), 0.01);
            if keeppct < &EventIntInc then delete;
         run;
         
         data &temp(drop = countflag);
            merge droprare(in = ina) &temp;
            by &temp.decod;
            if ina;
            if countflag = 1 then count = 0;
         run;
         
         proc datasets nolist;
            delete droprare;
         quit;
                    
         data &temp;
            merge &temp(in = ina) outname;
            by &temp.decod;
            if ina;
         run;

         proc datasets nolist;
            delete events events1 outname;
         quit;
         
         proc sort data = &temp;
            by usubjid &temp.decod;
         run;        
           
         proc transpose data = &temp out = &temp(drop = _name_ _label_);
            by usubjid;
            var count;
            id &temp.short; 
            idlabel &temp.DECOD;
         run;   
      %end;

      %let i = %eval(&i+1);
   %end;
%mend INTEVENT;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro GETFINDINGS(predose = 0);
   *** create by lists for each domain ***;
   %if &FindDistFl = Yes %then %do;
      %let i = 1;
      %do %while(%length(%scan(&findingsdomains,&i,' ')));
         %let temp = %scan(&findingsdomains,&i,' ');

         %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %let variable = &temp.STRESN;
         %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %let variable = &temp.ORRES;
         
         %if %upcase(&AllFind) = NO %then %do;
            %let ifcond = and %length(&&&temp.findlist);
         %end;
         %else %do;
            %let ifcond =;
         %end;           

         %if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) and %sysfunc(indexw(&&&temp.VARS, &variable, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.TESTCD, ' '))
         and %sysfunc(indexw(&&&temp.VARS, &temp.TEST, ' ')) &ifcond %then %do;
            %if %sysfunc(indexw(&&&temp.VARS, &temp.TPTNUM, ' ')) %then %let tpt = &temp.tptnum;
            %else %let tpt =;

            %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %let visit = visit;
            %else %let visit = ;

            %if &predose %then %do;
               %if %sysfunc(indexw(&&&temp.VARS, &temp.DTC, ' ')) %then %let dtc = &temp.DTC;
               %else %let dtc = ;
            %end;
            %else %let dtc = ;

            %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
            
            data &temp(keep = usubjid &temp.testcd &variable VISITNUM &tpt &visit &dtc);
               set &&in_&temp;
               %if %upcase(&AllFind) = NO %then %do;
                  where &temp.testcd in ( %unquote(&&&temp.findlist) ) and not missing (&variable);
               %end;
               %else %do;
                  where not missing (&variable);
               %end;                                
            run;            
            
            proc sort data = &temp out = &temp;
               by usubjid &temp.testcd visitnum &tpt;
            run;
            
            %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %do;                  
               %let ORRESLabel = &variable;
               %CheckLabel(&temp,ORRESLabel);

               data &temp(where = (not missing (&variable)));
                  set &temp(rename = (&variable = cval));
                  &variable = input(cval,best12.);
                  drop cval;
                  label &variable = "&ORRESLabel";
               run;     
            %end;

            *** Delete unscheduled records ***;
            %if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
               data &temp(drop = visit);
                  set &temp;
                  if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
               run;
            %end;

            %NObsVars(&temp);                  
            %if &nobs %then %let availabledomains = &availabledomains &temp;
            
            %if &predose %then %do;
               %if %length(&dtc) %then %do;
                  %TREATMENTEMERGENT(dataset = &temp, domain = &temp, date = &temp.DTC);

                  data &temp(drop = &dtc _TRTEMRG);
                     set &temp;
                     where _TRTEMRG = 0;
                  run;
               %end;          
            %end;

            ods exclude all;
            ods noresults;
            proc means noprint nway missing data = &temp;
               class usubjid &temp.testcd visitnum &tpt;
               var &variable;
               output out = &temp(drop = _type_ _freq_) mean = &variable;
            run;
            ods select all;
            ods results;

            data &temp;
               set &temp;
               %if %length(&tpt) %then %do;
                  if missing(&tpt) then level = cats(&temp.testcd,"_","V",visitnum);         
                  else level = cats(&temp.testcd,"_","V",visitnum,"_","T",&tpt);         
               %end;
               %else %do;
                  level = cats(&temp.testcd,"_","V",visitnum);
               %end;
            run;

            proc sort data = &temp out = &temp;
               by usubjid level;
            run;

            proc transpose data = &temp out = &temp(drop = _name_ _label_);
               by usubjid;
               var &variable;
               id level;
               idlabel level;
            run;              
         %end;
         %let i = %eval(&i+1);
      %end;
   %end;
%mend GETFINDINGS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro FDRPVALUES(inpv = , nlog10p = );

   proc multtest noprint inpvalues(&inpv) = summary_volcano fdr out = summary_volcano;
   run; 

   %global RawCutOff;
   %let RawCutOff = ;
   
   data summary_volcano(drop = RawCutOff);
      set summary_volcano end = eof;
      retain RawCutOff;
      if _n_ = 1 and . < fdr_p <= &AlphaValue then RawCutOff = &nlog10p;
      else if . < fdr_p <= &AlphaValue then do;
         if missing(RawCutOff) then RawCutOff = &nlog10p;
         else if &nlog10p < RawCutOff then RawCutOff = &nlog10p;
      end;
      if eof and RawCutOff > . then call symput("RawCutOff", left(trim(round(RawCutOff,0.0001))));
      label fdr_p = "&trFalseDiscoveryRatepvalue";
   run;

%mend FDRPVALUES;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro DOMBYLISTS(inclvisit = 1, incllabs = 0);

   %global suffix varsuff varsufflabel;

   %if %upcase(&DQFByVars) = YES %then %do;
      %let suffix = CAT SCAT LOC METHOD POS SPEC TPT;
   %end;
   %else %do;
      %let suffix = ;
   %end;

   *** create by lists for each domain ***;
   %let i = 1;
   %do %while(%length(%scan(&domainlist,&i,' ')));
      %let temp = %scan(&domainlist,&i,' ');
      %global subbyvar_&temp byvar_&temp var_&temp;
      %let subbyvar_&temp =;
      %let byvar_&temp = ;
      %let var_&temp = ;

      %if &inclvisit %then %do;
         %if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) %then %let byvar_&temp = &&&byvar_&temp VISITNUM;
      %end;

      %let j = 1; 
      %do %while(%length(%scan(&suffix,&j,' ')));
         %let stemp = %scan(&suffix,&j,' ');
         %if %sysfunc(indexw(&&&temp.VARS, &temp&stemp, ' ')) %then %let byvar_&temp = &&&byvar_&temp &temp&stemp;
         
         %if &incllabs %then %do;
            %global &stemp.Label&i;
            %if %sysfunc(indexw(&&&temp.VARS, &temp&stemp, ' ')) %then %do;
               %let &stemp.Label&i = &temp&stemp;
               %if &stemp = TPT and %sysfunc(findw(upcase(&bdsdomains), &temp)) %then %do;
                  %let &stemp.Label&i = ATPT;
                  %CheckLabel(&&in_&temp,&stemp.Label&i);
               %end;
               %else %do;
                  %CheckLabel(&&in_&temp,&stemp.Label&i);
               %end;
            %end;
            %else %do;
               %let &stemp.Label&i = ;
            %end;
         %end;

         %let j = %eval(&j+1);
      %end;   

      %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS and %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %let var_&temp = &&&var_&temp &temp.STRESN;
      %else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT and %sysfunc(indexw(&&&temp.VARS, &temp.STRESC, ' ')) %then %let var_&temp = &&&var_&temp &temp.STRESC;
      %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS and %sysfunc(indexw(&&&temp.VARS, &temp.ORRES, ' ')) %then %let var_&temp = &&&var_&temp &temp.ORRES;
      
      %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS and %length(&&&var_&temp) and &ProcessName ne Frequencies %then %let got1 = 1;
      %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS and %length(&&&var_&temp) and &ProcessName ne Frequencies %then %let got1 = 1;
      %else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT and %length(&&&var_&temp) and &ProcessName = Frequencies %then %let got1 = 1;
      %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS and %length(&&&var_&temp) and &ProcessName = Frequencies %then %let got1 = 1;
      
      %let i = %eval(&i+1);
      %let subbyvar_&temp = &&&byvar_&temp;        
   %end;

   %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %let varsuff = STRESN;
   %else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT %then %let varsuff = STRESC;
   %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %let varsuff = ORRES;
   
   %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %let varsufflabel = &trSTRESN;
   %else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT %then %let varsufflabel = &trSTRESC;
   %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %let varsufflabel = &trORRES;
%mend DOMBYLISTS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro REMOVEDUPS;

      *** Test for duplicates within by variables ***;
      data &temp.test;
         set &temp;
         by usubjid &&&byvar_&temp &temp.testcd;
         if first.&temp.testcd ne 1 or last.&temp.testcd ne 1;
      run;
      %NObsVars(&temp.test);

      *** if dups present, add date and time to by variables ***;
      %if &nobs and %length(&&&temp.DTC) %then %do;      
         %let byvar_&temp = &temp.dtc &&&byvar_&temp;
         proc sort data = &temp out = &temp;
            by usubjid &&&byvar_&temp &temp.testcd;
         run;     

         data &temp.test;
            set &temp;
            by usubjid &&&byvar_&temp &temp.testcd;
            if first.&temp.testcd ne 1 or last.&temp.testcd ne 1;
         run;     
         %NObsVars(&temp.test);
      %end;         

      proc datasets nolist;
         delete &temp.test;
      quit;

      *** if dups still present,remove them ***;
      %if &nobs %then %do;    
         data &temp.test;
            set &temp;
            by usubjid &&&byvar_&temp &temp.testcd;
            if not first.&temp.testcd;
         run;

         proc sort data = &temp.test(keep = usubjid &temp.seq);
            by usubjid &temp.seq;
         run;

         proc sort data = &temp;
            by usubjid &temp.seq;
         run;
         
         data &temp.chk;
            merge &temp(in = ina) &temp.test(in = inb);
            by usubjid &temp.seq;
            if ina and inb;
         run;
         %nobsvars(&temp.chk);
         
         %if &nobs %then %do;
            proc sort data = &temp.chk;
               by usubjid &&&byvar_&temp &temp.testcd;
            run;   
            title "Data in &temp domain that was removed to enable transpose";
            proc print width = min data = &temp.chk;
            run;
         %end;
         
         proc datasets nolist;
            delete &temp.chk;
         quit;
                  
         data &temp;
            merge &temp(in = ina) &temp.test(in = inb);
            by usubjid &temp.seq;
            if ina and not inb;
         run;
                  
         proc sort data = &temp out = &temp;
            by usubjid &&&byvar_&temp &temp.testcd;
         run;     
      %end;

%mend REMOVEDUPS;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;

%macro GETFREQS(variable = , variable2 = );
      proc sql noprint;
         create table &temp.1 as
            %if %length(&&&byvar_&temp) %then %do;
               select distinct siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test), %str( ), %str(,))), count(int) as Total
            %end;
            %else %do;
               select distinct siteid, %sysfunc(transtrn(%str(&temp.test), %str( ), %str(,))), count(int) as Total
            %end;          
            from &temp
            %if %length(&&&byvar_&temp) %then %do;
               group by siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test), %str( ), %str(,))); 
            %end;
            %else %do;
               group by siteid, %sysfunc(transtrn(%str(&temp.test), %str( ), %str(,))); 
            %end;
         %if %length(&variable2) %then %do;
            create table &temp.5 as
               select distinct &temp.test, &variable, &variable2 
               from &temp
               order by &temp.test, &variable;
         %end;
         %else %do;        
            create table &temp.5 as
               select distinct &temp.test, &variable
               from &temp
               order by &temp.test, &variable;
         %end;       
         create table &temp.6 as
            select distinct &temp.test, count(&temp.test) as NumVals
            from &temp.5
            group by &temp.test
            order by &temp.test;
         create table &temp.2 as
            select * 
            from &temp.1 as aa, &temp.5 as bb
            where aa.&temp.test = bb.&temp.test
            order by 
            %if %length(&&&byvar_&temp) %then %do;
               siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test &variable), %str( ), %str(,)));
            %end;
            %else %do;
               siteid, %sysfunc(transtrn(%str(&temp.test &variable), %str( ), %str(,)));
            %end;                             
         create table &temp.3 as 
            %if %length(&&&byvar_&temp) %then %do;
               select distinct siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test &variable), %str( ), %str(,))), count(&variable) as Count
            %end;
            %else %do;
               select distinct siteid, %sysfunc(transtrn(%str(&temp.test &variable), %str( ), %str(,))), count(&variable) as Count
            %end;
            from &temp
            %if %length(&&&byvar_&temp) %then %do;
               group by siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test &variable), %str( ), %str(,))); 
            %end;
            %else %do;
               group by siteid, %sysfunc(transtrn(%str(&temp.test,&variable), %str( ), %str(,))); 
            %end;
         create table &temp.4 as
            select *
            from &temp.2 natural left join &temp.3
            order by 
            %if %length(&&&byvar_&temp) %then %do;
               siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test &variable), %str( ), %str(,)));
            %end;
            %else %do;
               siteid, %sysfunc(transtrn(%str(&temp.test &variable), %str( ), %str(,)));
            %end;
         create table &temp.7 as
            select * 
            from &temp.4 natural left join &temp.6
            order by  
            %if %length(&&&byvar_&temp) %then %do;
               siteid, %sysfunc(transtrn(%str(&&&byvar_&temp &temp.test &variable), %str( ), %str(,)));
            %end;
            %else %do;
               siteid, %sysfunc(transtrn(%str(&temp.test &variable), %str( ), %str(,)));
            %end;
      quit;

      proc datasets nolist;
         delete &temp.1 &temp.2 &temp.3 &temp.5 &temp.4 &temp.6;
         change &temp.7 = &temp.2;
      quit;     
%mend GETFREQS;

%macro LABELRENAME;
   %if %substr(&tempvar,%eval(%length(&temp)+1)) = LOC %then %do;
      label &tempvar = "&trLocOfMeas";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = METHOD %then %do;
      label &tempvar = "&trMethOfTestExam";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = POS %then %do;
      label &tempvar = "&trPosOfSubj";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = SPEC %then %do;
      label &tempvar = "&trSpecType";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = TPT %then %do;                
      label &tempvar = "&trPlannedTime";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = CAT %then %do;
      label &tempvar = "&trCategory";
   %end;
   %else %if %substr(&tempvar,%eval(%length(&temp)+1)) = SCAT %then %do;
      label &tempvar = "&trSubcategory";
   %end;
   %else %if &tempvar = VISITNUM %then %do;
      label &tempvar = "&trVisitNum";
   %end;                   
   %if &tempvar ne VISITNUM %then %do;
      rename &tempvar = %substr(&tempvar,%eval(%length(&temp)+1));
   %end;    
%mend LABELRENAME;

*************************************************************************************;
*************************************************************************************;
*************************************************************************************;