
/***************************************************************
JMP Clinical Macro

Name: Domain Distribution
Support: Kelci Miclaus, SAS Institute Inc.

Description: 
      Macro for analysis processing for domain distribution reports. 

-----------------------------------------------------------
Copyright (C) 2010 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
****************************************************************/


/*
$CVS_START$
$ID: ADSLDistributionOptions$
$CMP: OOUT$ $UE: DM.AGE$ $NOTES: Variable will be available in Data Filter and Distributions.$
$CMP: OOUT$ $UE: DM.SEX$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.RACE$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.SITEID$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.COUNTRY$ $NOTES: Variable will be available in Demographic Grouping, Data Filter, and Distributions.$
$CMP: OOUT$ $UE: DM.STUDYID$ $NOTES: Use like other demography only if multiple levels deteced indicating a multi-study analysis.$
$CVS_STOP$
*/

%macro DomainDistribution(domain,extravars=,treemapvars=,findingsflag=0,aetreatcomp = 0);
   %let tclabel = %nrbquote(%LocaleGet(TotalCount_Text));
   %let polabel = %nrbquote(%LocaleGet(PercentOccurrence_Text));
   %let trNone = %nrbquote(%LocaleGet(None_Text));
   
   %if %length(&&in_&domain)=0 %then %do;
      %put [MS_S_0013];
      %put ERROR: The &domain or ad&domain data set does not exist for this study.;
      %let exiterror=1;
      %goto mexit;
   %end;
   
   %if %UPcase(&TrtUse) = NONE or &findingsflag %then %do;
      proc freq data=&in_adsl noprint;
         tables &TrtName / out=fc;
         where not missing(&TrtName);
      run;
      
      %if ^&findingsflag %then %do;
         %NObsVars(fc);
         %if &nobs < 2 %then %do;
            %let TrtLabel = ;
         %end;
      %end;
      
      %if %upcase(&TrtUse) = NONE %then %do;
         %let TrtName = ;
         %let TrtLabel = ;
      %end;
   %end;
   
   %if &findingsflag %then %do;
      %global Distribution_OutDatamr  Distribution_OutDataNamemr Distribution_Outdatamt Distribution_OutdataNamemt;
      %if %length(&Distribution_OutData) %then %do;
         %let Distribution_OutDatamr = &Distribution_OutData.mr;
         %let Distribution_Outdatamt = &Distribution_Outdata.mt;
      %end;
      %PrepOutDS(Distribution_OutDatamr,Distribution_OutDataNamemr,&StudyName,_&domain.mr);
      %PrepOutDS(Distribution_OutDatamt,Distribution_OutDataNamemt,&StudyName,_&domain.mt);
      
      %let CODateVar = &domain.DTC;
   %end;    
   %PrepOutDS(Distribution_OutData,Distribution_OutDataName,&StudyName,_&domain.d);

   /*kemicl add to attach domain to setting */
   %global OutName;
   %let OutName = &StudyName._&domain;
   %let ColorLabel = ;
   %DMFlags;
   %if &exiterror %then %goto mexit;   
   %put findingsflag = &findingsflag;
   
   %if &findingsflag = 0 %then %do;
      %if %length(&DomainWhereStmt) %then %do;
         data &&in_&domain;
            set &&in_&domain;
            where %unquote(&DomainWhereStmt);
         run;
         %if &syserr > 4 %then %do;
            %put [MS_S_0206];
            %put ERROR: The specified filter &DomainWhereStmt is invalid. Change the filter to be valid for the domain you are considering.;
            %let exiterror=1;
            %goto mexit;
         %end;
      %end;

      %Nobsvars(&domain);
      %if &nobs=0 %then %do;
         %put [MS_S_0207];
         %put ERROR: There are no observations after filtering by the %upcase(&Domain) Filter. Check that you have entered a valid filter.;
         %let exiterror=1;
         %goto mexit;
      %end;

      %let fpool = 0;
      %let ftesn = 0;
      %let ftesc = 0;
      %let MissingTestResults = 0;
      %let MissingTests = 0;
      %let fspec = 0;
      %let fsev = 0;
      %let fbodsys = 0;
      %let FCountPlots = 0;
      
      %let CODateVar = &domain.STDTC;
   %end;
   
   /* Check for TrtPeriods */
   %if (%sysfunc(findw(&&&domain.VARS, %upcase(&CODateVar)))=0 or %upcase(&TrtUse)=NONE or %upcase(&domain)=MH) 
      %then %let NTrtPeriods = 1;
   
   /* assign crossover treatments */
   %if &NTrtPeriods > 1 %then %do;
      %AssignCOTreatment(&domain, &domain, &CODateVar);
   %end;
   %else %do;
      %let TrtVarList = ;
   %end;
   
   %if &findingsflag = 0 %then %do;
      /* add trtvar to Byvar list */
      %if %length(&TrtName) %then %do;
         %if ^%index(&DesignByVars,&TrtName) %then %do;
            %let DesignByVars = &TrtName &DesignByVars;
         %end;
      %end;
   %end;
   
   %let trUnclassified = %nrbquote(%LocaleGet(SSSUnclassifiedSSS_Text));
   
   /* If Findings then data are already formed for analysis from FindingsMacros */
   %if %upcase(&domain) = CM %then %do;
      %let cmpos1_ = 0;
      %let cmpos2_ = 0;
      %let _cmtid_ = %sysfunc(open(&domain));
      %let fpos1_ = %sysfunc(varnum(&_cmtid_,CM&TmName));
      %let fpos2_ = %sysfunc(varnum(&_cmtid_,CMCAT));
      %let _cmtid_ = %sysfunc(close(&_cmtid_));
         
      data &domain; 
         set &domain;
         %if &fpos1_ > 0 %then %do;
            if CM&TmName = "" then CM&TmName = "&trUnclassified";
         %end;
            %else %if &fpos1_ = 0 %then %do;
               length CM&TmName $ 18;
               CM&TmName = "&trUnclassified";
            %end;
         %if &fpos2_ > 0 %then %do;
            if CMCAT = "" then CMCAT = "&trUnclassified";
         %end;
            %else %if &fpos2_ = 0 %then %do;
               length CMCAT $ 18;
               CMCAT = "&trUnclassified";
            %end;
      run;
      %let percentvar = CM&TmName;
      %let catbodvar = CAT;
   %end;
   %else %if %upcase(&domain) = AE %then %do;
      %let textlabel = %nrbquote(%LocaleGet(event_Text));
      %let aepos1 = 0;
      %let aepos2_ = 0;
      %let fpos2_ = 0;
      %let _aetid_ = %sysfunc(open(&domain));
      %let fpos1_ = %sysfunc(varnum(&_aetid_,AE&TmName));
      %if %length(&TmGroup) %then %let fpos2_ = %sysfunc(varnum(&_aetid_,AE&TmGroup));;
      %let _aetid_ = %sysfunc(close(&_aetid_));
         
      data &domain; 
         set &domain;
         %if &fpos1_ > 0 %then %do;
            if AE&TmName = "" then AE&TmName = "&trUnclassified";
         %end;
         %else %if &fpos1_ = 0 %then %do;
            length AE&TmName $ 18;
            AE&TmName = "&trUnclassified";
         %end;
      run;
      
      %let percentvar = AE&TmName;
      %let catbodvar = &TmGroup;
   %end;
   %else %if %symexist(TmGroup) %then %do;
      %if %length(&TmGroup) %then %do;
         /* kemicl extra check to be sure TmGroup exists */
         %let _tid_ = %sysfunc(open(&domain));
         %let tmgroupExists = %sysfunc(varnum(&_tid_,&domain.&TmGroup));
         %let _tid_ = %sysfunc(close(&_tid_));
         %if ^&tmgroupExists %then %let TMGroup = ;
      %end;
   %end;
         
   %if ^%symexist(SLDOSEVar) %then %let SLDOSEVar = 0;
   
   /* use adsl or dm to merge in subject info and compute denominators for percents */
   %let StudyVar = ;
   %if %sysfunc(findw(%upcase(&ADSLvars), STUDYID)) %then %let StudyVar = STUDYID; 
   proc sort data=&in_adsl out=adsl;
      by USUBJID;
   run;
      
   %let adata =adsl;
   %if &NtrtPeriods > 1 %then %do;
      data adsls; 
         set &adata;
         array trts{*} &TrtVarList;
         do i=1 to dim(trts);
            &TrtName = trts[i];
            output;
         end;
         drop i;
         /* Kemicl added to remove dropouts that were not in all treatment periods */
         if missing(&TrtName) then delete;
      run;
      
      %let adata = adsls;
   %end;
   
   /* Check number of subjects in adsl */
   %nobsvars(&adata);
   %let nsubjects = &nobs;
   %put nsubjects = &nsubjects;
   
   /*kemicl:S1054584 only merge in country from DM if using adsl to avoid conflicting treatment information */
   %if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) > 0 %then %do;
      proc sort data=&in_dm out = dmwc;
         by USUBJID;
      run;
      
      data dm1;
         merge &adata(in=inadsl) dmwc(keep = usubjid country);
         by USUBJID;
         if inadsl;
      run;
   %end;
   %else %do;
      data dm1; 
         set &adata;
      run;
   %end;
   
   /* Add Country to ADSLVARS and call DMFlags again now that Country merged in from DM if not exist in ADSL */
   %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) > 0 %then %do;
      %let ADSLVARS = &ADSLVARS COUNTRY;
   %end;
   %DMFlags;
   %if &exiterror %then %goto mexit;
   
   /* Check for more than one level of Study ID to see if combined studies  */
   %let StudyLevs = ;
   %let StudyList = ;
   %if %sysfunc(findw(%upcase(&ADSLvars), STUDYID)) %then %do;
      %GetLevels(dm1,STUDYID,StudyList,_LPrefix = NLGrp,_nLvls=StudyLevs,_vType= nLGType);
   %end;

   %let DMlist = ;
   %if %length(&TrtName) %then %let DMlist = &DMList &TrtName;
   %if &doSex and %upcase(SEX) ne %upcase(&TrtName) %then %let DMlist = &DMlist SEX;
   %if &doRace and %upcase(RACE) ne %upcase(&TrtName) %then %let DMlist = &DMlist RACE;
   %if &doCtry and %upcase(COUNTRY) ne %upcase(&TrtName) %then %let DMlist = &DMlist COUNTRY;
   %if &doSite and %upcase(SITEID) ne %upcase(&TrtName) %then %let DMlist = &DMlist SITEID;
   %if (&StudyLevs > 1 and %upcase(STUDYID) ne %upcase(&TrtName) and ^%index(%Upcase(&DesignByVars),STUDYID)) %then %let DMlist = &DMlist STUDYID;
   
   /* IF CROSSOVER ANALYSIS: Denominator for treatment based on sum of unique subjects per treatment period */
   %let i = 1;
   proc sql noprint;
      %do %while (%length(%scan(&DMlist,&i))>0);
         %let j = %eval(&i+1);
         create table dm&j as select *, %if &i=1 %then 1/&nsubjects as p_None, &nsubjects as tot_None,; 1/freq(%scan(&DMlist,&i)) as p_%scan(&DMlist,&i), freq(%scan(&DMlist,&i)) as tot_%scan(&DMlist,&i)
         from dm&i
         group by %scan(&DMlist,&i)
         order by usubjid;
         %let i = &j;
      %end;
   quit;
   
   proc datasets nolist;;
      change dm&i = dm_filter;
   quit; 
   
   %if &NtrtPeriods > 1 %then %do;
      proc sort data= dm_filter;
         by USUBJID TRT;
      run;
      
      proc sort data = &domain;
         by USUBJID TRT;
      run;
   %end;
   
   %put fpool = &fpool;
   
   %if &fpool %then %do;
      proc sort data=sdtm.pooldef out=pooldef;
         by USUBJID;
      run;

      data dm_filter;
         merge dm_filter pooldef;
         by USUBJID;
      run;

      * use the demographics from the first subject in each pool;
      proc sort data=dm_filter nodupkey;
         by POOLID;
      run;

      data domdata;
         merge dm_filter(in=dmf) &domain ;
         by POOLID;
         * want the line below? ;
         if missing(&domain.&tmvar) then delete;;        
         if dmf;
      run;
   %end;
   %else %do;
      %let trsdosox = %nrbquote(%LocaleGet(StudyDayofStartofX_Text));
      %let trCount = %nrbquote(%LocaleGet(Count_Text));
      
      data domdata;
         merge &domain  dm_filter(in=dmf) ;
         by USUBJID %if &NtrtPeriods > 1 %then TRT;;
         _Count_ = 1;
         label _Count_ = "&trCount";
         * want the line below? ;
         %if &findingsflag %then %do;
            if missing(&domain.&tmvar) then delete;
            %if %upcase(&domain) = LB and &ftesn %then %do;
               _NumRes_ = &domain.STRESN;
               %LabsNormalize(&domain.STRESN, &NormRef, &LogShft);
               %if %length(&LogShft) and %length(&NormRef) %then %do;
                  label &domain.STRESN = "&LogShft(Numeric Result/&NormRef)";
               %end;
               %else %if %length(&NormRef) %then %do;
                  label &domain.STRESN = "Numeric Result/&NormRef";
               %end;
               %else %if %length(&LogShft) %then %do;
                  label &domain.STRESN = "&LogShft(Numeric Result)";
               %end;
              %else %do;
                  label &domain.STRESN = "Numeric Result";
               %end;
            %end;
            /* create variable for baseline flag */
            length MeasurementType $ 18;
            label MeasurementType = "Findings Measurement Type";
             %if &blflag %then %do;
               if &BLFlagVar = 'Y' then do;
            %end;
            %else %do;
               if &domain.DY <= 1 then do;
            %end;
               MeasurementType = "Baseline";
            end;
            else do;
               MeasurementType = "Trial";
            end;
         %end;
         %else %if %upcase(&domain) = SU %then %do;
            %let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(SubstanceAbuse_Text)))); 
            if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
               label &domain.STDY = "&trlabel";
            end;
         %end;
         %else %if %upcase(&domain) = MH %then %do;
            %let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(MedicalHistoryEvent_Text))));
            label &domain.STDY = "&trlabel";
         %end;       
         %else %if %upcase(&domain) = DS %then %do;
            %let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(DispositionEvent_Text))));
            if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
               label &domain.STDY = "&trlabel";
            end;
         %end;       
         %else %if %upcase(&domain) = CE %then %do;
            %let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(ClinicalEvent_Text))));
            if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
               label &domain.STDY = "&trlabel";
            end;
         %end;
         %else %if %upcase(&domain) = DV %then %do;
            %let trlabel = %nrbquote(%LocaleFill(&trsdosox, %nrbquote(%LocaleGet(ProtocolDeviation_Text))));
            if missing(&domain.STDY) or missing(%upcase(&domain)DUR) then do;
               label &domain.STDY = "&trlabel";
            end;
         %end;                
         %else %do;
            if missing(&domain.&TmName) then delete;
         %end;
         if dmf;
      run;
   %end;
   
   %nobsvars(domdata);
   %if &nobs=0 %then %do;
      %put [MS_S_0208];
      %put ERROR: The merge of the %upcase(&domain) and &in_adsl data sets resulted in 0 observations. Please check your data for accuracy.;
      %let exiterror=1;
      %goto mexit;
   %end;
      
   %if %upcase(&domain) = AE %then %do;
      %let i = 1;
      %let AElabellist = ;
      %do %while (%length(%scan(&AEevars,&i)));
         %let CAEvar = AE%scan(&AEevars,&i);
         %if %length(&CAEvar) %then %do;
            %let &CAEvar.Label = &CAEvar;
            %CheckLabel(domdata, &CAEvar.Label);
            %let AElabellist = &AELabelList &&CAEvar.Label;
         %end;
         %put &AELabelList;
         %let i = %eval(&i+1);
      %end;
   %end;
   
   /* create a first occurrence flag in the output data set based on sort date */
   
   
   /* kemicl account for crossover to count unique events per subject per period */
   %if  %upcase(&domain) = AE  and %upcase(&CountMultiple)=NO %then %do;
      /* applying translated string for sorting */
      %if %length(&aeser) %then %do;
         data domdata;
            set domdata;
            if upcase(AESER) = upcase("%LocaleGet(N_Text)") | upcase(AESER) = "N" then Eng_AESER = "N";
            else if upcase(AESER) = upcase("%LocaleGet(Y_Text)") | upcase(AESER) = "Y" then Eng_AESER = "Y";
         run;
      %end;
      %if %length(&aesev) %then %do;
         data domdata;
            set domdata;
            if upcase(AESEV) = upcase("%LocaleGet(Mild_Text)") | upcase(AESEV) ="MILD" then Eng_AESEV = "Mild";
            else if upcase(AESEV) = upcase("%LocaleGet(Moderate_Text)") | upcase(AESEV) ="MODERATE" then Eng_AESEV = "Moderate";
            else if upcase(AESEV) = upcase("%LocaleGet(Severe_Text)") | upcase(AESEV) ="SEVERE" then Eng_AESEV = "Severe";
         run;
      %end;
      proc sort data=domdata out=domdata;
            by AE&TmName  %if &NTrtPeriods > 1  %then PERIOD; USUBJID 
               %if %length(&aeser) %then %do; descending Eng_AESER %end; %if %length(&aesev) %then %do; descending Eng_AE&aesev %end; AESTDY;
      run;
      
      data domdata; set domdata;
         by AE&TmName %if &NTrtPeriods > 1 %then PERIOD; USUBJID;
         if first.USUBJID then output;
      run;
   %end;
   %else %if %upcase(&CountMultiple)=NO %then %do;
      proc sort data=domdata out=domdata;
            by &domain.&TmName  %if &NTrtPeriods > 1  %then PERIOD; USUBJID &domain.STDY;
      run;
      
      data domdata; set domdata;
         by &domain.&TmName  %if &NTrtPeriods > 1  %then PERIOD; USUBJID;
         if first.USUBJID then output;
      run;
   %end;
   
      /*create counts for barcharts, used to be treemaps*/
   %if %length(&treemapvars) and ^&findingsflag %then %do;
      /* create counts for ordering */
      %let createcountvars = &treemapvars;
      %if %upcase(&domain) = CM or %upcase(&domain) = AE  %then %do;
         %if &fpos2_ >0 %then %do;
            %let createcountvars = &treemapvars &catbodvar;
         %end;
      %end;
      
      %let i = 1;
      %do %while(%length(%scan(&createcountvars,&i)));
         %let tmvar = %scan(&createcountvars,&i);
         %let percentvar = &domain.&tmvar;
   
         proc sort data=domdata(keep=&percentvar) out=getcounts;
            by &percentvar;
         run;

         proc freq data=getcounts noprint;
            tables &percentvar / out=countdt;
         run;

         %if %upcase(&CountMultiple)=NO and &i=1  %then %do;
            data countdt; set countdt;
               Percent = round((count/&nsubjects)*100,0.1);
               label Percent = "&polabel";
            run;
         %end;
         
         proc datasets;
            modify countdt;
            %if &i = 1 %then %do;
               label count = "&tclabel";
            %end;
            %else %do;
               %if %upcase(&domain) = AE %then %do;
                  rename count = group_count;
                  label group_count = "Body System Count";
               %end;
               %if %upcase(&domain) = CM %then %do;
                  rename count = group_count;
                  label group_count = "Medication Category Count";
               %end;
            %end;
         quit;

         proc sql;
            create table domdata as
            select * from domdata as d1 left join countdt as d2 on d1.&percentvar=d2.&percentvar
            order by %if &fpool %then %do; POOLID %end; %else %do; USUBJID %end;;
         quit;
         %let i = %eval(&i+1);
      %end; 
   %end;

   %DMLabels(domdata);

   /* Treatment Incidence Filter, for now group across crossovers*/
   %if &aetreatcomp %then %do;
      proc sort data=domdata(keep = &domain.&TmName &TrtName p_&TrtName) out=ddsort;
         by &domain.&TmName &TrtName; 
      run;
      
      proc means data=ddsort;
         by &domain.&TmName &TrtName; 
         var p_&TrtName;
         output out=ddsums sum(p_&TrtName) = TrtIncidence;
      run;
      
      proc transpose data = ddsums out=sumst(drop=_name_);
         by  &domain.&TmName;
         var TrtIncidence;
         id &TrtName;
         idlabel &TrtName;
      run;
      
      %sasname(&&tlev&plev);
      %let sascontrollev = &sasname;
      %do i = 1 %to %eval(&nlev);
         %sasname(&&tlev&i);
         %let sastrtlev&i = &sasname;
         %let rrlabel&i = %nrbquote(%LocaleFill(%nrbquote(%LocaleGet(RelativeRiskforX_Text)), f1 = %nrbquote(&&tlev&i)));
      %end;
      %let maxrrlabel = %nrbquote(%LocaleGet(MaxTreatmentRR_Text));
      %let trtocclabel = %nrbquote(%LocaleGet(TreatmentOnlyOccurrenceFlag_Text)); 
      
      data rrs(keep = &domain.&TmName RR: %if %eval(&nlev) > 2 %then MaxRRValue; TrtoccFl);
         set sumst;
         %do i = 1 %to %eval(&nlev);
            %if &i ^= &plev %then %do;
               RR_trtlev&i._over_control = round(&&sastrtlev&i/&sascontrollev,0.01);
               label RR_trtlev&i._over_control = &&rrlabel&i;
            %end;
         %end;
         %if %eval(&nlev) > 2 %then %do;
            MaxRRvalue = max(of rr:);
            label MaxRRvalue = &maxrrlabel;
         %end;
         if missing(&sascontrollev) then TrtOccFl = 1;
         else TrtOccFl = 0;
         label TrtOccFl= "Treatment Only Occurrence Flag";
      run;
      
      data domdata;
         set domdata;
         if 0 then set rrs;
         if _n_ = 1 then do;
            declare hash h(dataset:"rrs");
            h.defineKey("&domain.&TmName");
            h.defineData(ALL:"YES");
            h.defineDone();
         end;
         
         if h.find() = 0;
      run;
   %end; 

   %if &fpool %then %do;
      %let SubjectLabel = POOLID;
   %end;
   %else %do;
      %let SubjectLabel = USUBJID;
   %end;
   %CheckLabel(domdata,SubjectLabel);
   
   %if &findingsflag %then %do;
      /* kemicl changed to be tmvar so it could be either TESTCD or SPEC for SEND */
      %let DecodLabel = &domain.&tmvar;
      %let StdyLabel = &domain.DY;
      %if &ftesn %then %do;
         %let TestLabelN = &domain.STRESN;
         %CheckLabel(domdata,TestLabelN);
      %end;
      %if &ftesc %then %do;
         %let TestLabelC = &domain.STRESC;
         %CheckLabel(domdata,TestLabelC);
      %end;
      
      /*Logic for checking for visit and fhilowflag */
      %let FCountPlots = 0;
      %if &fhilowflag and (&fvisit or &fvisitn) %then %do;
         %let TimeOLabel = ;
         %let usevnum = 0;
         %let HiLowLabel = &Fdomain.NRIND;
         %CheckLabel(domdata,HiLowLabel);
         %if &fvisit %then %do;
            %let TimeLabel = VISIT;
            %CheckLabel(domdata,TimeLabel);
            %if &fvisitn %then %do;
               %let TimeOLabel = VISITNUM;
               %CheckLabel(domdata,TimeOLabel);
            %end;       
         %end;
         %else %do;
            %let TimeLabel = VISITNUM;
            %CheckLabel(domdata,TimeLabel);
            %let usevnum = 1;
         %end;
         
         /* Get Tests that have non missing xxNRIND values for plots */
         proc sort data=domdata(where=(not missing(&Fdomain.NRIND )) keep=&Fdomain.NRIND &domain.&tmvar) nodupkey out=cnttests;
            by &domain.&tmvar;
         run;

         %nobsvars(cnttests);
         %if &nobs %then %do;
            %let FCountPlots = 1;
            data _null_; set cnttests end=_e_;
               call symput("CountTests"||trim(left(_n_)),trim(left(&domain.&tmvar)));
               if _e_ then call symput("TotCountTests", _n_);
            run;
         %end;
      %end; 
      
      /* kemicl added for getting missing findings tabulate and for which tests to do anova and contingency for */
      %let NumVar = &domain.STRESN;
      %if %upcase(&domain) = LB and &ftesn %then %let NumVar = _NumRes_;
      data &Distribution_OutDatamr %if &ftesn %then notmissn(keep=&domain.&tmvar); %if &ftesc %then notmissc(keep=&domain.&tmvar &domain.STRESC); ;
         set domdata;
         %if &ftesn and &ftesc %then %do;
            if missing(&NumVar) and not missing (&domain.STRESC) then output notmissc;
            else if missing(&NumVar) and missing (&domain.STRESC) then output &Distribution_OutDatamr;
            else if not missing(&NumVar) then output notmissn;
         %end;
         %else %if &ftesn %then %do;
            if missing(&NumVar) then output &Distribution_OutDatamr;
            else output notmissn;
         %end;
         %else %if &ftesc %then %do;
            if missing(&domain.STRESC) then output &Distribution_OutDatamr;
            else output notmissc;
         %end;
      run;
      
      /* result of data step: 
         &Distribution_OutDatamr data has all tests that had missing values
         notmissc has all tests with categorical test values to use for contingency plots
         notmissn has all tests with numeric results for use in anova plots 
      */
      %let MissingTestResults = 0;
      %let MissingTests = 0;
      %let NumTestResults = 0;
      %let CatTestResults = 0;
      %let nNumTests = 0;
      %nobsvars(&Distribution_OutDatamr); 
      %if &nobs %then %let MissingTestResults = 1;
      %if &ftesn %then %do;
         %nobsvars(notmissn);
         %let nNumTests = &nobs;
         %if &nNumTests %then %do;
            %let NumTestResults = 1;
            proc sort data=notmissn(keep=&domain.&tmvar) nodupkey;
               by &domain.&tmvar;
            run;

            data notmissn; set notmissn end=_e_;
               call symput("TestsN"||trim(left(_n_)),trim(left(&domain.&tmvar)));
               if _e_ then call symput("TotTestsN", _n_);
            run;
         %end;
      %end;
      /* find if there are mixed tests that have categorical values for contingency analysis */
      %if &ftesc %then %do;
         %nobsvars(notmissc);  
         %if &nobs %then %do;
            proc sort data=notmissc(keep=&domain.&tmvar &domain.STRESC) nodupkey;
               by &domain.&tmvar &domain.STRESC;
            run;
            /*removed tests with numeric results if there was mixed cases of missing/nonmissing stresn and stresc*/
            %if &nNumTests %then %do;
               data notmissc; merge notmissc(in=a) notmissn(in=b);
                  by &domain.&tmvar;
                  if a and b then delete;
               run;
            %end;
            
            /* find and keep only test with more than 1 level of the categorical response */
            data notmissc; set notmissc;
               by &domain.&tmvar;
               retain fcat;
               if first.&domain.&tmvar then fcat = 1;
               else fcat+1;
               if fcat = 1 then delete;
            run;
            %nobsvars(notmissc);
            %if &nobs %then %do;
               %let CatTestResults = 1;
               proc sort data=notmissc nodupkey;
                  by &domain.&tmvar;
               run;

               data _null_; set notmissc end=_e_;
                  call symput("TestsC"||trim(left(_n_)),trim(left(&domain.&tmvar)));
                  if _e_ then call symput("TotTestsC", _n_);
               run;
            %end;
            %else %do;
               %let CatTestResults = 0;
            %end;
         %end;
         %else %do;
            %let CatTestResults = 0;
         %end; 
      %end;
      
      /* find subjects missing tests completely (not recorded in table) only for nonpooled data*/
      %if ^&fpool %then %do;
         proc sort data = &domain(keep = &domain.&tmvar) out = utests nodupkey;
            by &domain.&tmvar;
         run;
         
         data _null_;
            set utests end=_e_;
            call symput("AllTests"||trim(left(_n_)),&domain.&tmvar);
            if _e_ then call symput("NallTests",_n_);
         run;

         data alltests; 
            set adsl(keep = USUBJID &DesignByVars %if &NTrtPeriods = 1 %then  &TrtName;
            %if &doSex %then SEX; %if &doAge %then AGE; %if &doRace %then  RACE; %if &doSite %then SITEID;);
            %do i = 1 %to &Nalltests;
               &domain.&tmvar = "%nrbquote(&&AllTests&i)";
               output;
            %end;
            label USUBJID="&SubjectLabel";
         run;
         
         proc sort data=&domain(keep = USUBJID &domain.&tmvar %if &NTrtPeriods >1 %then  &TrtName;) out = wtids nodupkey;
            by USUBJID &domain.&tmvar;
         run;
         
         data &Distribution_OutDatamt; 
            merge alltests(in=a) wtids(in=b);
            by USUBJID &domain.&tmvar;
            if a and not b;
         run;

         %nobsvars(&Distribution_OutDatamt);
         %if &nobs %then %let MissingTests =1;
      %end;
   %end;
   %else %do;
      %let DecodLabel = &domain.&TmName;
      %let StdyLabel = &domain.STDY;
   %end;

   %CheckLabel(domdata,StdyLabel);
   %CheckLabel(domdata,DecodLabel); 

   %let dsid = %sysfunc(open(domdata));
   
   %let doEPOCH = 0;
   %let doEPOCH = %sysfunc(varnum(&dsid,EPOCH));
   %if &doEPOCH %then %do;
      %let EpochLabel = EPOCH;
      %CheckLabel(domdata,EpochLabel);
   %end;
   %if %length(&extravars) %then %do;
      %let i=1;
      %let f=1;
      %do %while(%length(%scan(&extravars,&i)));
         %let ev = %scan(&extravars,&i);
         %if &findingsflag %then %do;
            %let &ev.Label = &ev;
         %end;
         %else %do;  
            %let &ev.Label = &domain.&ev;
         %end;
         %let vnum = %sysfunc(varnum(&dsid,&&&ev.Label));
         %if &vnum > 0 %then %do;
            %CheckLabel(domdata,&ev.Label);  
            %let evl&f = &&&ev.Label;
            %let f=%eval(&f+1);
         %end;
         %let i=%eval(&i+1);
      %end;
      %let nev = %eval(&f-1);
   %end;    
   %else %let nev = 0;
   
   *** Added 2 to label so TRTLABEL from CMTRT doesnt overwrite treatment label ***;
   %if %length(&treemapvars) %then %do;
      %let i=1;
      %let f=1;
      %do %while(%length(%scan(&treemapvars,&i)));
         %let tv = %scan(&treemapvars,&i);
         %let &tv.Label2 = &domain.&tv;
         %let vnum = %sysfunc(varnum(&dsid,&&&tv.Label2));
         %if &vnum > 0 %then %do;
            %CheckLabel(domdata,&tv.Label2); 
            %let tvl&f = &&&tv.Label2;
            %let f=%eval(&f+1);
         %end;
         %let i=%eval(&i+1);
      %end;
      %let ntv = %eval(&f-1);
   %end;    
   %else %let ntv = 0;  
   
   %let dsid = %sysfunc(close(&dsid));

   %if %upcase(&domain) = CM or %upcase(&domain) = AE %then %do;        
      %if &fpos2_ >0 %then %do;
         %let ColorLabel = &domain.&catbodvar;
         %CheckLabel(domdata,ColorLabel);
         %let GrpCountLabel = group_count;
         %CheckLabel(domdata,GrpCountLabel);
      %end; 
   %end;
   %else %if %symexist(TmGroup) %then %do;
      %if %length(&TmGroup) %then %do;
         %let ColorLabel = &domain.&TmGroup;
         %CheckLabel(domdata, ColorLabel);
      %end;
   %end;
      
   %if &findingsflag and &fbodsys %then %do;
      %let ColorLabel = &domain.BODSYS;
      %CheckLabel(domdata,ColorLabel);
   %end;
   
   %let odata = domdata;
   %include "&MacroPath./DesignByVars.sas";
   
   /*Set up macros for cross drilldowns */
   %if %upcase(&domain) = CM %then %do;
      %let APddname = AEDistribution;
      %let crossdomain = AE;
   %end;
   %else %if %upcase(&domain) = AE %then %do;
      %let APddname = InterventionsDistribution;
      %let crossdomain = CM;
   %end;
   
   %if %upcase(&CountMultiple)=NO %then %let AxLab = %nrbquote(%LocaleGet(Subjects_Text));
   %else %let AxLab = %nrbquote(%LocaleGet(Occurrences_Text));
   
   * create JSL file;
   %PrepOutFile(DistributionScript,DistributionScriptName,&StudyName &ProcessName Script for &domain..jsl);
   %global _mainjslfile_ FullAPName;
   %let _mainjslfile_ = &DistributionScript;
   %if &findingsflag=1 %then %let FullAPName=%str(Findings Distribution (%upcase(&domain)));
   %else %let FullAPName=%str(%upcase(&domain) Distribution);
   
   %let DeltaLabel = ;  
   * For previous added study, there is no global variable for ReviewsEnabled; 
   %if ^%symexist(ReviewsEnabled) %then %do;
      %global ReviewsEnabled;
      %let ReviewsEnabled=1;
   %end;
   %if &ReviewsEnabled  and &ReviewNumber %then %do;
      %let DeltaLabel = &domain.RFlg;
      %CheckLabel(domdata,DeltaLabel);
   %end;
   
   %if &StudyLevs > 1 %then %do;
      %put Data contains records from &StudyLevs Studies;
      %let StudyLabel = STUDYID;
      %CheckLabel(domdata, StudyLabel);
   %end;
   
   * S1044040 - handling value ordering for levels of AESEV (TC 01/16/14);
   %CheckVarExist(AESEV,domdata,,_ShowError=0);
   %if &exiterror %then %do;
      * AESEV does not exist;
      %let exiterror = 0;
   %end;
   %else %do;
      %let AESEVLabel = AESEV;
      %CheckLabel(domdata,AESEVLabel);
      %*put AESEVLabel=&AESEVLabel;
   %end;

   /*create final output data */
   data &Distribution_OutData; set domdata; run;
   
   * put JSL statements;
   data _null_;
      file "&DistributionScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
      put "//!";
      put "Names Default to Here(1);";
      put "objRefNS = newNamespace();";
      put "jg:currentObjRefNamespaceName = objRefNS << getName;";
      put "objRefNS:DataList = {};";
      put "objRefNS:DataNameList = {};";
      put "objRefNS:TabButtonList = {};";
      put "objRefNS:TabCodeList = {};";
      put "objRefNS:TabAfterCodeList = {};";
      put "objRefNS:TabOpenList = {};";
      put "objRefNS:CreateReportList = {};";
      put "objRefNS:TMView = 0;";
      put "objRefNS:Percents = 0;";
      put "objRefNS:NoteDomain = ""%upcase(&domain)"";";
      put "try(objRefNS:SubjectFilter=""\[&SubjectFilter]\"");";
      put "try(objRefNS:PopulationSet=""\[&PopulationSet]\"");";
      
      /* Try to Delete Domain Distribution Filter */
      %if %symexist(SFname) %then %do;
         %if %upcase(&SFname) = DISTRIBUTION_DRILL_DOWN %then %do;
            put "Include(""$CLINICAL_HOME\JSLFiles\SubjectFilterUtilityFunctions.jsl"");"; 
            put "try(DeleteSubjectFilterByName(""Distribution Drill Down"", empty(), ""&studyname""));";
         %end;
      %end;
      %if %upcase(&ProcessName) = EVENTSDISTRIBUTION %then %do;
         put " objRefNS:includestring= ""Events: "";"; 
         %let DomainWhereStmt=&EventsWhereStmt;
      %end;
      %else %if %upcase(&ProcessName) = FINDINGSDISTRIBUTION %then %do;
         put "objRefNS:includestring=""Findings Tests: "";";
         %let DomainWhereStmt = &WhereTests;
      %end;
      %else %if %upcase(&ProcessName) = AEDISTRIBUTION %then %do;
         put " objRefNS:includestring=""Adverse Events: "";";
      %end;
      %else %do;
         put " objRefNS:includestring=""Interventions: "";";
      %end;
      put "try(objRefNS:DomainWhereStmt=""\[%nrbquote(&DomainWhereStmt)]\"");";
      
      %if %length(&WhereForReports) %then %do;
         put "try(objRefNS:WhereState=""\[%nrbquote(&WhereForReports)]\"");";
         put "try(objRefNS:WhereFilters=""Additional Filter to include Subjects: "" || objRefNS:WhereState);";
         %if %length(&DomainWhereStmt) %then %do;
            put "try(objRefNS:WhereFilters=objRefNS:WhereFilters||""\!NAdditional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
         %end;
      %end;
      %else %if %length(&DomainWhereStmt) %then %do;
         put "try(objRefNS:WhereFilters=""Additional Filter to include ""||objRefNS:includestring || objRefNS:DomainWhereStmt);";
      %end;    

      put "OutPath= ""&OutPath"";";
      put "objRefNS:PopulationSet= ""&PopulationSet"";";
      put "ClientOutPath=""&ClientOutPath"";";
      put "outFolder = ""&ClientOutPath"";";
      
      put "DtExpr  = Expr(";
      put "   If(IsEmpty(objRefNS:dmd),";
      put "      objRefNS:dmd=open(""&ClientOutPath.&Distribution_OutDataName..sas7bdat"",invisible);";
      put "         Column(""&SubjectLabel"") << Label(1);";
      put "         try( Column(""&AESEVLabel"") << Set Property(""Value Ordering"",{""%LocaleGet(Mild_Text)"",""%LocaleGet(Moderate_Text)"",""%LocaleGet(Severe_Text)""}) );";
      
      %if %length(&StdyLabel) %then %do;
         put "       Try(Column(""&StdyLabel"") << Set Modeling Type( Continuous));";
      %end;
      put "        New Column(""%  "",";
      %if %length(&TrtLabel) %then %do;
         put "       set Formula((:Name(""p_&TrtName"")))";
      %end;
      %else %do;
         put "       set Formula((:Name(""p_None"")))";
      %end;
      put "       );";
      
      put "        New Column(locale:get(""N_Text""),";
      %if %length(&TrtLabel) %then %do;
         put "       set Formula((:Name(""tot_&TrtName"")))";
      %end;
      %else %do;
         put "       set Formula((:Name(""tot_None"")))";
      %end;
      put "       );";      
      %if %length(&ColorLabel) %then %do;
         put "    objRefNS:dmd << Color by Column(:Name(""&ColorLabel""), Color Theme( ""JMP Light"" ));";
         put "    vcoloraa = Associative Array();";
         put "    For Each Row( vcoloraa[Column(""&ColorLabel"")[Row()]] = Color Of( Row State( Row())));";
         put "    vcolors = vcoloraa << Get Values;";
         put "    For(";
         put "       v = 1;";
         put "       vc = vcoloraa <<first;, !IsEmpty(vc), vc = vcoloraa <<next(vc);";
         put "       v++;,";
         put "       vcolors[v] = Substitute(Expr(_label_ = _color_), Expr( _label_ ), vc, Expr( _color_ ), vcoloraa[vc])";
         put "    );";
         put "    Column(""&ColorLabel"") << Set Property( ""Value Colors"", vcolors);";
      %end;
      %if &aetreatcomp %then %do;
         put "    Try(Column(""&trtocclabel"") << Set Modeling Type(Nominal));";
      %end;
      put "  ,";  /*else for isempty*/;
      put "    Current Data Table(objRefNS:dmd);";
      put "  );";
      put "objRefNS:dbv = {};";
      %do i=1 %to &ndbv;
         %if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
            put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
         %end;
      %end;
      put ");";
      /* run/open data table now before results function calls */
      put "DtExpr;"; 
      
      %if %upcase(&domain) = DV %then %let yaxis = %eval(10*&TMLen);
      %else %let yaxis = 625;
      

      /* kemicl not using treemaps any more but keep in there in case we revive */
      %macro treemap1(ii,groupx1,overx2);
         put " VListBox(";
         put "    objRefNS:gb&ii = Graph Builder(";
         %if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbv)),";
         %end;       
         put "       Show Control Panel(0), Show Legend(0), Auto Stretching(0),";
         put "       Variables(";
         %if %length(&groupx1) %then %do;
            put "    X( Column(&groupx1) ),";
         %end;
         put "          X( :Name(""&&tvl&ii"")";
         %if %length(&groupx1) %then %do;
            put "             ,Position(1)";
         %end;       
         %if ^&findingsflag %then %do;
            put "            ,Order By(:Name(""&tclabel""), Descending)";
         %end;
         put "          )";
         %do b=2 %to &ndbv;
            put "    , Group X( :Name(""&&dbv&b"") )";
         %end;
         %if %length(&ColorLabel) %then %do;
               put "    , Color(:Name(""&ColorLabel""))";
         %end;
         put "       ),";
         %if %length(&groupx1) %then %do;
            put "       Elements( Treemap( X(1),X(2)) )";
         %end;
         %else %do;
            put "       Elements( Treemap( X) )";
         %end;
         put "          ,Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:fill(locale:get(""XCountsGraph_Text""),{""%upcase(&domain)""}) )} )";       
         put "    )),";    
      %mend;      
      %macro barplot1(ii,groupx1,overx2);
         put "    VListBox(";
         put "    objRefNS:gb&ii = Graph Builder(";
         %if &findingsflag=0  %then %do;
            put "       Size(675,&yaxis),";
         %end;
         %else %do;
            put "       Size(560,420),";
         %end;
         /*%if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbv)),";
         %end;    
         */
         put "       Show Control Panel(0), Show Legend(1), Auto Stretching(0),";
         put "       Variables(";
         put "          X( :Name(""&&tvl&ii"")";
         %if %length(&groupx1) %then %do;
            put "             ,Position(1)";
         %end;       
         %if ^&findingsflag %then %do;
            put "            ,Order By(:Name(""&tclabel""), Descending)";
         %end;
         put "          )";
         %do b=2 %to &ndbv;
            put "    , Group X( :Name(""&&dbv&b"") )";
         %end;
         %if %length(&groupx1) %then %do;
            put "    ,Group Y( Column(&groupx1) )";
         %end;       
         %if %length(&overx2) %then %do;
               put "    , Overlay(Column(&overx2))";
         %end;
         put "       ),";
         %if &findingsflag %then %do;
            put "       Elements( Histogram( X, Legend( 2 )))";
         %end;
         %else %do;
            put "       Elements( Bar( X, Legend( 2 ), Bar Style( ""Stacked"" ), Summary Statistic( ""Sum"" ) ))";
         %end;
         put "          ,SendToReport(";
         put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),"; 
         put "             Dispatch( {}, ""Graph Builder"", OutlineBox, {Set Title( locale:fill(locale:get(""XCountsGraph_Text""),{""%upcase(&domain)""}) )} )";  
         put "          )";
         put "    )),";
      %mend;
 
       
 		put "objRefNS:findVarByRole= function({varlist, rolename}, {i},";
       put "	 for (i = 1, i <= n items(varlist), i++,"; 
       put "       if (varlist[i][""Role""] == rolename, Return(i));"; 
       put " );"; 
       put " 0;     // not found";
		put ");";  
		
      %if &findingsflag=0  %then %do;
         %macro tabulate1(ii,gpcols,firstgroup);
            %let trTotal = %nrbquote(%LocaleGet(Total_Text));
            %let Ntext = %nrbquote(%LocaleGet(N_Text));
            put "    objRefNS:tbhlb&ii = HListBox(";
            put "    objRefNS:tb1 = Tabulate(Show Control Panel( 0 ),";
            put "       Change Item Label( Statistics( N, ""&trTotal"" ) ";
            %if %upcase(&CountMultiple)=NO %then %do;
               put "       ,Statistics( Sum, ""  "" )";
            %end;
            put "       ),";
            %if %upcase(&CountMultiple)=NO %then %do;
               put "       Set Format( Sum(:Name(""%  "")(9,1,Percent))),";
            %end;
            %if %upcase(&SplitResults)=YES %then %do;
               put "         By(eval(objRefNS:dbv)),";
            %end;
            put "       Order by count of grouping columns( 1 ),";
            %if %length(&ColorLabel) %then %do;
               put "       Add Table( Row Table( Grouping Columns( :Name(""&ColorLabel""),:Name(""&&tvl&ii"") ),";
               put "       )";
            %end;
            %else %do;
               put "       Add Table( Row Table( Grouping Columns( :Name(""&&tvl&ii"") ),"; 
               put "       )";
            %end;          
            %if %length(&gpcols) %then %do;
               put "       ,Column Table( Grouping Columns( &gpcols ) ";
               %if %upcase(&CountMultiple)=NO %then %do;
                  put "          ,Statistics( Sum ),";
                  put "          Analysis Columns( :&trCount, :Name(""%  ""))";
               %end;
               put "       )";
            %end;
            put "       ,Column Table(Statistics(N)),";
            put "    ),";  /* close Add Table */
            %if %length(&ColorLabel) %then %do;
               put "    Display Column Width( Row Label(""&ColorLabel""), 270),";
            %end;
            put "       Display Column Width( Row Label(""&&tvl&ii""), 255)";
            put " ),";  /* close tabulate */
            put " objRefNS:Table2 = Tabulate(Show Control Panel(0), Automatic Recalc(0),";
            put "    Change Item Label( Statistics( Mean, "" "" ) ),";
            put "    Add Table(";
            put "       Column Table(Analysis Columns(:Name(""&Ntext"")), Statistics(Mean))";
            %if %length(&firstgroup) %then %do;
               put "       ,Row Table( Grouping Columns(&firstgroup))";
            %end;
            put "    ),";
            put "    SendToReport(Dispatch({},""Tabulate"",";
            put "       OutlineBox,{Set Title( locale:get(""DemographyTotalSubjectCounts_Text""))})";
            put "    )";
            PUT " ),"; /*CLOSE TABULATE */
            put " ),"; /* close Hlistbox */
         %mend;
      %end;
      
      put "objRefNS:ddw=Function({},{default local},";
      put " DtExpr;";
      put "objRefNS:CSlist = {};";
      %if &doSite %then %do;
         put "InsertInto(objRefNS:CSlist,Column(""&SiteLabel""));";
      %end;
      %if &doCtry %then %do;
         put "InsertInto(objRefNS:CSlist,Column(""&CtryLabel""));";
      %end;
      %if &doSex %then %do;
         put "InsertInto(objRefNS:CSlist,Column(""&SexLabel""));";
      %end;
      %if &doRace %then %do;
         put "InsertInto(objRefNS:CSlist,Column(""&RaceLabel""));";
      %end;
      
      %do i=1 %to &ndbv;
         %if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
            put "InsertInto(objRefNS:CSlist,Column(""&&dbv&i""));";
         %end;
      %end;
      %if (&ntv and &findingsflag=0) %then %do;
         put "       objRefNS:tmlist={";
         put "          ""&trNone"" ";       
         %if %length(&TrtLabel) %then %do;
            put "       ,""&TrtLabel"" ";
         %end;
         %if &doSex %then %do;
            put "       , ""&SexLabel"" ";
         %end;
         %if &doRace %then %do;
            put "       , ""&RaceLabel"" ";
         %end;                   
         %if &doCtry %then %do;
            put "       , ""&CtryLabel"" ";
         %end;    
         %if &doSite %then %do;
            put "       , ""&SiteLabel"" ";
         %end;
         %if ^%index(%Upcase(&DesignByVars),STUDYID) and &StudyLevs > 1 %then %do;
            put "       , ""&StudyLabel"" ";
         %end;
         put "       };";        
         
         put "       objRefNS:tm2list={";
         put "          ""&trNone"" ";
         %if &NtrtPeriods > 1 %then %do;
            put "       ,  ""Period"" ";
         %end;       
         %if %upcase(&domain) = AE %then %do;
            %let j = 1;
            %do %while (%length(%scan(&AELabelList,&j)));
               %let curravar = &%scan(&AELabelList,&j);
               put "       ,""&curravar"" ";
               %let j = %eval(&j+1);
            %end;
         %end;
         %if %upcase(&domain) = EX %then %do;
            %if &SLDOSEVar %then %do;
               put "    , ""&DoseLabel"" ";
            %end;
            %if &SLCATVar %then %do;
               put "    ,  ""&CatLabel"" ";
            %end;
         %end;
         %if &doEPOCH %then %do;
            put "       , ""&EPOCHLabel"" ";
         %end; 
         %if %length(&DeltaLabel) %then %do;
            put "          , ""&DeltaLabel"" ";
         %end;
         put "       };";
         
         put "   for(t=2,t<=NItems(objRefNS:tmlist),t++,";
         put "      cts = (CurrentDataTable() << Summary(Group(:(objRefNS:tmlist[t])), N)) << Sort(By(Column(3)),Order(Descending),Replace Table);";
         put "      valueorder = Column(objRefNS:tmlist[t])<<GetAsMatrix;";
         put "      Close(cts,nosave);";
         put "     :(objRefNS:tmlist[t]) << Set Property(""Value Ordering"",valueorder);";
         put "   );";   
         /* kemicl Create associative array for percents */
         put "objRefNS:plist = {""p_None""};";
         put "objRefNS:nlist = {""tot_None""};";
         %let k = 1;
         %do %while (%length(%scan(&DMlist,&k))>0);
            put "InsertInto(objRefNS:plist,""p_%scan(&DMlist,&k)"");";
            put "InsertInto(objRefNS:nlist,""tot_%scan(&DMlist,&k)"");";
            
            %let k = %eval(&k+1);
         %end;
         put " objRefNS:CParray = Associative Array(objRefNS:tmlist,objRefNS:plist);";
         put " objRefNS:CNarray = Associative Array(objRefNS:tmlist,objRefNS:nlist);";
         
         put "    objRefNS:GBPexpr = expr(";
         %do i = 1 %to &ntv;        
            put "          objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
            put "          objRefNS:gbr&i << Add Variable({:Name(""%  ""), Role(""Y"")});";
            put "          objRefNS:gb&i << SendToReport(";
            put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
            put "             Dispatch({},""Y title"", TextEditBox,";
            put "                {Rotate Text(""Left""), Set Text(locale:get(""PercentOccurrenceinDemographicGroup_Text""))}";
            put "          ));";
            put "          report(objRefNS:gb&i)[AxisBox(2)]<<format(""Percent"",9,0);";
         %end;
         put "    );";
         
         put "  objRefNS:GBCexpr = expr(";
         %do i = 1 %to &ntv;
            put "          objRefNS:gb&i << SendToReport(";
            put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),";
            put "             Dispatch({},""Y title"", TextEditBox,";
            %if %upcase(&CountMultiple)=YES %then %do;
               put "                {Rotate Text(""Left""), Set Text(locale:get(""Count_Text""))}";
            %end;
            %else %do;
               put "                {Rotate Text(""Left""), Set Text(locale:get(""SubjectCount_Text""))}";
            %end;
            put "          ));";
         %end;
         put "    );";
                  
      %end;
      put " VListBox(";   
      put "    Vlistbox(";
      %if %upcase(&ProcessName) ne FINDINGSDISTRIBUTION %then %do;
         %if %length(&TEAEText) %then %do;
            put "objRefNS:TEAEText = TextBox(""     "" ||""&TEAEText"", <<Set Wrap(1000)),";
         %end;                     
      %end;      
      put "    HListBox(";
      %if %upcase(&CountMultiple)=NO %then %do;
         put "   objRefNS:CPTB = TextBox(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateNumSubjExperiencingAnX_Text""),{""&textlabel""}), <<Set Wrap(650),<<Set Font Size(9)),";
      %end;
      %else %if %upcase(&CountMultiple)=YES %then %do;
         put "   objRefNS:CPTB = TextBox(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateTotalNumOccOfAnX_Text""),{""&textlabel""}), <<Set Wrap(650),<<Set Font Size(9)),";
      %end;
      %if %upcase(&CountMultiple) = NO %then %do;
         put "       TextBox(""  ""),";
         put "       objRefNS:pcbb = ButtonBox(locale:get(""ShowPercents_Text""),<<UnderlineStyle(1),";
         put "       <<Set Function(Function({this},";
         put "          objRefNS:CurrBName = this << Get Button Name();";
         put "          If(objRefNS:CurrBName == locale:get(""ShowPercents_Text""),";
         put "             objRefNS:GBPexpr;";
         put "             objRefNS:Percents = 1;";
         put "             objRefNS:CPTB<< Set Text(""     "" || locale:fill(locale:get(""DispValsIndPercSubjWithinDemGrpExpAnX_Text""),{""&textlabel""}));"; 
         put "             this << setButtonName(locale:get(""ShowCounts_Text""));";
         put "             ,";
         %do i = 1 %to &ntv;
            put "          objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
            put "          objRefNS:gb&i << SendToReport(";
            put "             //Dispatch({},""Graph Builder"",OutlineBox,{Set Title( locale:fill(locale:get(""XCountsGraph_Text""),{""%upcase(&domain)""}) )}),";          
            put "             Dispatch( {}, ""graph title"", TextEditBox, {Set Text( "" "" )} ),"; 
            put "             Dispatch({},""Y title"", TextEditBox,";
            put "                {Set Text("" "")}";
            put "          ));";
            put "          objRefNS:gbr&i << Remove Variable({:Name(""%  ""), Role(""Y"")});";
         %end; 
         put "             objRefNS:Percents = 0;";
         put "             objRefNS:GBCexpr;";
         put "             objRefNS:CPTB << Set Text(""     "" || locale:fill(locale:get(""DisplayedCountsIndicateNumSubjExperiencingAnX_Text""),{""&textlabel""}));";
         put "             this << setButtonName(locale:get(""ShowPercents_Text""))";
         put "       ))));";
      %end;
      put "    ),";
      put "      objRefNS:gbthlb = Vlistbox(";
      /* Draw distributions displays initially */
      %if &findingsflag = 0 %then %do;
         %do i=1 %to &ntv;
            %if %symexist(dbv1) %then %do;
               %if &NtrtPeriods > 1 %then %do;
                  %barplot1(&i,%quote(""&dbv1""),%quote(""Period""));
               %end;
               %else %do;
                  %barplot1(&i,%quote(""&dbv1""),%str());
               %end;    
            %end;
            %else %do;
               %barplot1(&i,%str(),%str());
            %end;
         %end;
         %if (&ntv and &findingsflag=0) %then %do;
            put " objRefNS:GBCexpr; ";
         %end;
      %end;
      %if &ntv = 0 or &findingsflag %then %do;
         put "      objRefNS:dd = Distribution(";
         put "            Arrange in Rows(3),";
         put "            Automatic Recalc( 1 ),";
         %if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbv)),";
         %end;

         %if %index(&treemapvars,&TmName)=0 and &findingsflag = 0 %then %do;
            put "         Nominal Distribution(Column(:Name(""&DecodLabel"")), ShowCounts(1),Order By( ""Count Ascending"" )),";
         %end;
         %if &findingsflag %then %do;
            put " Nominal Distribution(Column(:Name(""&tvl1"")), ShowCounts(1),Order By( ""Count Ascending"" )),";
         %end;
         %if &findingsflag and %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "          Nominal Distribution(Column(:Name(""&TrtLabel"")), ShowCounts(1)),";
            %if &NtrtPeriods > 1 %then %do;
               put "          Nominal Distribution(Column(:Name(""Period"")), ShowCounts(1)),"; 
            %end;          
         %end; 
         %do i=1 %to &nev;
            put "         Nominal Distribution(Column(:Name(""&&evl&i"")), ShowCounts(1)),";
         %end;       
         %if%length(&StdyLabel) %then %do;
            put "         Continuous Distribution(Column(:Name(""&StdyLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;          
         put "         SendToReport(";
         %if %length(&StdyLabel) %then %do;
            put " //            Dispatch( {""&StdyLabel""}, ""Distrib Histogram"", FrameBox, {Frame Size(85, 275)} ),";
         %end;
         %if &findingsflag %then %do;
            put "            Dispatch( {""&tvl1""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end; 
         %if %index(&treemapvars,&TmName)=0 and &findingsflag = 0 %then %do;
            put "            Dispatch( {""&DecodLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;
         %if &findingsflag and  %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "            Dispatch( {""&TrtLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;
         %do i=1 %to &nev;
               put "            Dispatch( {""&&evl&i""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;

         put "         )";
         put "      ),";
      %end;
      put "     ),";
      put " ));  //end listboxes"; 
      put ");";  /* End function for count plot content */
      
      /* Expression for Tabbed report to insert data filter into LHP */
      %if &findingsflag = 0 %then %do;
         put "objRefNS:CustomPanel = Expr(objRefNS:Filter= objRefNS:dmd<< Data Filter(<<Conditional(1),";
         put "       Add Filter(";
         put "          Columns( ";
         %if %length(&DeltaLabel) %then %do;
            put "                :Name(""&DeltaLabel""),";
         %end;
         %if %upcase(&CountMultiple)=NO %then %do;
            put "             :Name(""&tclabel""),";
            put "             :Name(""&polabel""),";
         %end;
         %if &aetreatcomp %then %do;
            %if %eval(&nlev) > 2 %then %do;
               put "          :Name(""&maxrrlabel""),";
            %end;
            %else %do;
               %do i = 1 %to %eval(&nlev);
                  %if &i ^= &plev %then %do;
                     put "    :Name(""&&rrlabel&i""),";
                  %end;
               %end;
            %end;
            put "             :Name(""&trtocclabel""),";
         %end;
         %do i=1 %to &nev;
            %if &i=1 %then %do;
               put "         :Name(""&&evl&i""),";
            %end;
            %else %do;
               put "         ,:Name(""&&evl&i""),";
            %end;
         %end;
         %if ^%index(%Upcase(&DesignByVars),STUDYID) and &StudyLevs > 1 %then %do;
            put "       :Name(""&StudyLabel""), ";
         %end;         
         %if &doAge %then %do;
            put " :Name(""&AgeLabel""),";
         %end;
         %if &doSex %then %do;
            put " :Name(""&SexLabel""),";
         %end;
         %if &doRace %then %do;
            put " :Name(""&RaceLabel""), ";
         %end;
         %if &doTrt & (%index(&DesignByVars, &TrtName)= 0) & %length(&TrtLabel) %then %do;
            put " :Name(""&TrtLabel""),";
         %end;

         %if &doSite %then %do;
            put "       :Name(""&SiteLabel""),";
         %end;
         put "          )";
         %if (%upcase(&CountMultiple)=NO and %upcase(&domain) = AE) %then %do;
            %if &PropThresh > 0 %then %do;
               put " , Where( :&polabel >= &PropThresh),";
            %end;
         %end;
         %if %length(&ColorLabel) %then %do;
                  put "       , Display(:Name(""&ColorLabel""), Size(200,95), List Display)";
         %end;
         %if &doSite %then %do;
                  put "       , Display(:Name(""&SiteLabel""), Size(200,95), List Display)";
         %end;
         put "       ),";
         put "       Mode( Show( 1 ), Include( 1 ) )";      
         put "    );";
         put "    objRefNS:dmd<<ClearSelect;";
         put " );"; /*end custom filter expression */
      %end;
      %else %do;
         put "objRefNS:CustomPanel = Expr(objRefNS:Filter= objRefNS:dmd << Data Filter(<<Conditional(1),";
         put "       Add Filter(";
         put "          Columns( ";
         %if &ReviewsEnabled %then %do;
            put "                :Name(""&DeltaLabel"")";
         %end;
         %if ^%index(%Upcase(&DesignByVars),STUDYID) and &StudyLevs > 1 %then %do;
            put "       ,:Name(""&StudyLabel""), ";
         %end;           
         %if &doAge %then %do;
            put " ,:Name(""&AgeLabel""),";
         %end;
         %if &doSex %then %do;
            put " ,:Name(""&SexLabel""),";
         %end;
         %if &doRace %then %do;
            put " ,:Name(""&RaceLabel""), ";
         %end;
         %if &doTrt & (%index(&DesignByVars, &TrtName)= 0) & %length(&TrtLabel) %then %do;
            put " ,:Name(""&TrtLabel""),";
         %end;
         %if &doSite %then %do;
            put "       ,:Name(""&SiteLabel"")";
         %end;
         %if %index(%upcase(&extravarlist),VISIT) %then %do;
            %let vindex =  %sysfunc(FINDW(%upcase(&extravarlist),VISIT,1," ",E));
            put "         ,:Name(""&&evl&vindex"")";
         %end;
         %if %index(%upcase(&extravarlist),%upcase(&domain)TPT) %then %do;
            %let vindex =  %sysfunc(FINDW(%upcase(&extravarlist),%upcase(&domain)TPT,1," ",E));
            put "         ,:Name(""&&evl&vindex"")";
         %end;
         put "                 )), Mode( Show( 1 ), Include( 1 ))";     
         put "       )); ";  /*end data filter and custom panel expression */
      %end;
      
      %if (&ntv and &findingsflag=0) %then %do; 
         put "objRefNS:AddLeftControlPanel = 1;";
         put "objRefNS:ddwa = Function({},{default local},";
         put "   objRefNS:LeftControlVBox << Append(";
         put "    VListBox(OutlineBox(""View"",<< outlinecloseorientation(""Vertical""),";
         put "     Outlinebox(locale:get(""DemographicGrouping_Text""),";
         put "     objRefNS:glb=ListBox(objRefNS:tmlist,";
         %if %Upcase(&TrtUse) ne NONE %then %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tmlist)),<<setselected(2,1)";
         %end;
         %else %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tmlist)),<<setselected(1,1)";
         %end;
         put "   )),";
         put "   OutlineBox(locale:fill(locale:get(""XStacking_Text""), {""%upcase(&domain)""}),";
         put "    objRefNS:glb2 = ListBox(objRefNS:tm2list,";
         %if &NTrtPeriods > 1 %then %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tm2list)),<<setselected(2,1)";
         %end;
         %else %do;
            put "       maxselected(1),nlines(nitems(objRefNS:tm2list)),<<setselected(1,1)";
         %end;
         put "   )))";
         put "  ));";      
         /* kemicl - script for column switcher box */            
         put " objRefNS:glb << Set Script(";
         put "    SelectOK = 1;";
         put "    try((objRefNS:glb<<GetSelected)[1],SelectOK = 0);";
         put "    If(SelectOK,";
         put "   SVar = (objRefNS:glb<<GetSelected)[1];";
         put "   SVar2 = (objRefNS:glb2<<GetSelected)[1];";
         put "   SPVar = objRefNS:CPArray[Svar];";
         put "   SNVar = objRefNS:CNArray[Svar];";
            /* Kemicl - update the formula for percent column to use right denominator */ 
         put "    Eval(parse(EvalInsert(""\[Column(""%  "")<<Set Formula(:Name(""^SPvar^""))]\"")));";
         put "    Eval(parse(EvalInsert(""\[Column(locale:get(""N_Text""))<<Set Formula(:Name(""^SNvar^""))]\"")));";
         
         %do i=1 %to &ntv;
         	put "   objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
         	put "	  objRefNS:gbVars = objRefNS:gbr&i<<GetVariables();";
         	put "	  objRefNS:groupVarIndex = objRefNS:findVarByRole(objRefNS:gbVars,""Group Y"");";
         	put "	  if(objRefNS:groupVarIndex, objRefNS:gbr&i<<RemoveVariable(objRefNS:groupVarIndex));";
				put "   objRefNS:tbhlb&i << Delete;";
            put "   dbv2 = {};";
            put "   dbvFirst = {};";
            put "   If(SVar == ""&trNone"",";
               put "    If(SVar2 != ""&trNone"",";
               put "       InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
               put "    );";     
            put " ,";  /*else for grouping var */
            	put "	   objRefNS:gbr&i << AddVariable({Column(Svar),Role(""Group Y"")});";
            	put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]));";
            	put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";            
            	put "    If(Svar2 != ""&trNone"",";
               put "    	InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
            	put "    );";
            put "  );";          
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";
         %end;
         /* else if nothing selected, set to NONE */
         put "    ,";
         put "    objRefNS:glb << SetSelected(1,1);";
         put "    );";  /*end SelectOK condition */
         put " );";  /* End Column Switcher script */ 
         
         /* Script for stacking columns */
         /* kemicl - script for column switcher box */            
         put " objRefNS:glb2 << Set Script(";
         put "    SelectOK = 1;";
         put "    try((objRefNS:glb2<<GetSelected)[1],SelectOK = 0);";
         put "    If(SelectOK,";
         %do i=1 %to &ntv;
            put "   objRefNS:gbr&i = report(objRefNS:gb&i)[Graph Builder Box(1)];";
         	put "	  objRefNS:gbVars = objRefNS:gbr&i<<GetVariables();";
         	put "	  objRefNS:groupVarIndex = objRefNS:findVarByRole(objRefNS:gbVars,""Overlay"");";
         	put "	  if(objRefNS:groupVarIndex, objRefNS:gbr&i<<RemoveVariable(objRefNS:groupVarIndex));";
         	put "	  objRefNS:tbhlb&i << Delete;";
            put "   SVar2 = (objRefNS:glb2<<GetSelected)[1];";
            put "   SVar = (objRefNS:glb<<GetSelected)[1];";
            put "   dbv2 = {};";
            put "   dbvFirst = {};";
            put "   If(SVar2 == ""&trNone"",";
            put "    If(SVar != ""&trNone"",";
               put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]));";
               put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";               
            put "    );";
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";        
            put " ,";  /* else for stacking var */
				put "	   objRefNS:gbr&i << AddVariable({Column(Svar2),Role(""Overlay"")});";            
            put "    InsertInto(dbv2,Column((objRefNS:glb2<<GetSelected)[1]));";
            put "    If(SVar != ""&trNone"",";
               put "    InsertInto(dbv2,Column((objRefNS:glb<<GetSelected)[1]),1);";
               put "    InsertInto(dbvFirst,Column((objRefNS:glb<<GetSelected)[1]));";      
            put "    );";
            put "    objRefNS:tbvlb << Append( ";
                        %tabulate1(&i,%nrstr(eval(dbv2)),%nrstr(eval(dbvFirst)));
            put "    );";           
            put "  );";
         %end;
         /* else if nothing selected, set to NONE */
         put "    ,";
         put "    objRefNS:glb2 << SetSelected(1,1);";         
         put "    );";  /*end SelectOK condition */
         put " );";  /* End stacking Switcher script */           
         put ");";   /* End column switcher objRefNS:dd Function */  
      %end; 
   
      %if &ntv and &findingsflag=0  %then %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""CountsGraph_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:TEAEText, objRefNS:gb1";
         %do i = 2 %to &ntv;
            put ",objRefNS:gb&i";
         %end;
         put "}});";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Distributions_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:dd}});";
      %end;
      put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
      put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
      put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddw()));";
      %if (&ntv and &findingsflag=0) %then %do;
         put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:ddwa()));";
         put "objRefNS:CloseTabOB = 1;";
      %end;
      %else %do;
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";     
      %end;
      put "InsertInto(objRefNS:TabOpenList,1);";
      
      %if &findingsflag = 0 %then %do; 
         put " objRefNS:tbfunc = Function({}, {default local},";
         put "    DtExpr;";
         put "    objRefNS:tbvlb = Vlistbox(";
         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "    objRefNS:tvcol = {Column(""&TrtLabel"")};";
            put "    objRefNS:tvfcol = objRefNS:tvcol;";
            %if &NtrtPeriods > 1 %then %do;
               put "    InsertInto(objRefNS:tvcol,{Column(""Period"")});";
            %end;
            %do i=1 %to &ntv;
               %tabulate1(&i,%nrstr(eval(objRefNS:tvcol)),%nrstr(eval(objRefNS:tvfcol))); 
            %end;
         %end;
         %else %do;
            %do i=1 %to &ntv;
               %tabulate1(&i,%nrstr({}),%nrstr({})); 
            %end;
         %end;
         put "    );";
         put " );";
      %end; 
      
      %if &ntv and &findingsflag=0  %then %do;
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""CountsTable_Text"")));";
         put "InsertInto(objRefNS:CreateReportList, {{objRefNS:tbhlb1";
         %do i = 2 %to &ntv;
            put ",objRefNS:tbhlb&i";
         %end;
         put "}});";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
         put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:tbfunc()));";
         put "InsertInto(objRefNS:TabAfterCodeList,"" "");";
         put "objRefNS:CloseTabOB = 1;";
         put "InsertInto(objRefNS:TabOpenList,1);";         
      %end;

      %if &ntv and &findingsflag=0  %then %do;
         put "objRefNS:ddwd=Function({},{default local},";
         put "   If(IsEmpty(objRefNS:dmd),";
         put "      objRefNS:dmd=open(""&ClientOutPath.&Distribution_OutDataName..sas7bdat"",invisible);";
         put "         Column(""&SubjectLabel"") << Label(1);";
         put "         try( Column(""&AESEVLabel"") << Set Property(""Value Ordering"",{""%LocaleGet(Mild_Text)"",""%LocaleGet(Moderate_Text)"",""%LocaleGet(Severe_Text)""}) );";
         put "   );";
         put "objRefNS:dbvd = {};";
         %do i=1 %to &ndbv;
            %if %quote(&&dbv&i) ne %quote(&TrtLabel) %then %do;
               put "InsertInto(objRefNS:dbvd,Column(""&&dbv&i""));";
            %end;
         %end;
         put " VListBox(";   
         put "      objRefNS:dd2 = Distribution(";
      put "               //Arrange in Rows(4),";
         put "            Automatic Recalc( 1 ),";
         %if %upcase(&SplitResults)=YES %then %do;
            put "         By(eval(objRefNS:dbvd)),";
         %end;

         put "         Nominal Distribution(Column(:Name(""&DecodLabel"")),ShowCounts(1), Order By( ""Count Ascending"" )),";
         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "          Nominal Distribution(Column(:Name(""&TrtLabel"")), ShowCounts(1)),";
            %if &NtrtPeriods > 1 %then %do;
               put "          Nominal Distribution(Column(:Name(""Period"")), ShowCounts(1)),"; 
            %end;
         %end;
         %do i=1 %to &nev;
            put "         Nominal Distribution(Column(:Name(""&&evl&i"")), ShowCounts(1)";
            %if %length(&ColorLabel)  %then %do;
               %if "&&evl&i" = "&ColorLabel" %then %do;
                  put "       ,Order By( ""Count Ascending"" )";
               %end;
            %end;
            put "),";
         %end; 
         %if &SLDOSEVar %then %do;
               put "         Continuous Distribution(Column(:Name(""&DoseLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;             
         %if %length(&StdyLabel) %then %do;
            put "         Continuous Distribution(Column(:Name(""&StdyLabel"")), ShowCounts(0), Quantiles(0),Summary Statistics(0)),";
         %end;       
         put "         SendToReport(";
         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
            put "            Dispatch( {""&TrtLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;
         put " //          Dispatch( {""&StdyLabel""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put " //          Dispatch( {""&StdyLabel""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";  
         put "            Dispatch( {""&DecodLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %do i=1 %to &nev;
               put "            Dispatch( {""&&evl&i""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
         %end;

         put "         )";
         put "      ))";
         put " );";
         put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
         put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""Distributions_Text"") || "" ""));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ddwd()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:dd2}});";
      %end;
         
      %if &findingsflag %then %do;
      	%let trAbnormal = %nrbquote(%upcase(%LocaleGet(Abnormal_Text)));
      	%let trHigh = %nrbquote(%upcase(%LocaleGet(High_Text)));
      	%let trNormal = %nrbquote(%upcase(%LocaleGet(Normal_Text)));
      	%let trLow =  %nrbquote(%upcase(%LocaleGet(Low_Text)));
         %if &FCountPlots %then %do;
            put "objRefNS:cpfunc = Function({},{default local},";
            put " DtExpr;";
            put " Column(""&HiLowLabel"")<< Set Property(""Value Ordering"",{""&trAbnormal"",""&trHigh"",""&trNormal"",""&trLow"",""A"",""H"",""N"",""L""});";
            %if &usevnum %then %do;
               put "    Column(""&TimeLabel"")<< Set Modeling Type( Ordinal);";
            %end;          
            put "objRefNS:cplb = VListBox(";
            %do c = 1 %to &TotCountTests;
               put "    Graph Builder(";
               put "       Size(715,545), Show Control Panel(0), AutoStretching(0),";
               put "       Where(:Name(""&DecodLabel"") == ""&&CountTests&c""),";
               put "       Variables(";
               put "          X(:Name(""&TimeLabel"")";
               %if %length(&TimeOLabel) %then %do;
                  put "          ,Order By( :Name(""&TimeOLabel""), Ascending)";
               %end;
               put "          ),";
               put "          Group Y(:Name(""&HiLowLabel""))";
               %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
                  put "       ,Overlay(:Name(""&TrtLabel""))";
               %end;
               put "       ),";
               put "       Elements(Bar(X,Bar Style( ""Side by side"" ),Summary Statistic( ""Sum"" ))),";
               put "       SendToReport(";
               put "          Dispatch({},""Graph Builder"", Outlinebox, {Set Title(locale:fill(locale:get(""XTest_Text""), {""\[%nrbquote(&&CountTests&c)]\""}))}),";
               put "          Dispatch({},""graph title"", TextEditBox,{Set Text(locale:fill(locale:get(""XCounts_Text""), {""&HiLowLabel""}))})";
               put "       )";
               put "    ),";
               put "    TextBox(" "),";
            %end;
            put "));";
            
            put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
            put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
            put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""CountPlots_Text"")));";
            put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:cpfunc()));";
            put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
            put "InsertInto(objRefNS:TabOpenList,1);";
            put "InsertInto(objRefNS:CreateReportList,{{objRefNS:cplb}});";
         %end;

         %if %UPcase(&TrtUse) ne NONE and %length(&TrtLabel) and (&NumTestResults or &CatTestResults) %then %do;
            put "objRefNS:ffunc=Function({},{default local},";
            put " DtExpr;";
            put "objRefNS:Tvlb = HListBox(";
            %if &NumTestResults %then %do;
               put " objRefNS:obn = OutlineBox(locale:get(""OnewayAnalysesforNumericTests_Text""),";
               put "   VListBox(";
               %do  t = 1 %to &TotTestsN;
                  put "      VlistBox(";
                  put "       ow&t = Oneway(";
                  put "            Automatic Recalc( 1 ),"; 
                  /* kemicl disable byvars
                  %if %upcase(&SplitResults)=YES %then %do;
                     put "         By(eval(objRefNS:dbv)),";
                  %end;
                  */
                  put "         Y(:Name(""&TestLabelN"")),";
                  put "         X(:Name(""&TrtLabel"")),";
                  put "         Where(:Name(""&DecodLabel"") == ""\[%nrbquote(&&TestsN&t)]\""),";
                  put "         Means( 1 ),";
                  put "         Box Plots( 1 ),";
                  put "         Mean Diamonds( 1 ),";
                  put "         SendToReport(";
                  put "          Dispatch( {}, ""Oneway Anova"", OutlineBox, Close( 1 ) )";
                  put "         )";
                  put "       );";
                  put "       try(owr&t = ow&t << report;);";
                  put "       try(owr&t[OutlineBox(1)]<<Set Title(locale:fill(locale:get(""XResults_Text""), {""\[%nrbquote(&&TestsN&t)]\""})););";
                  put "      ),";
               %end;
               put "   )),";  
            %end;
            %if &CatTestResults %then %do;
               %if &NumTestResults %then %do;
                  put " TextBox(" "),";
               %end;
               put " objRefNS:obc = OutlineBox(locale:get(""ContingencyAnalysesforCategoricalTests_Text""),";
               put "   VListBox(";
               %do  t = 1 %to &TotTestsC;
                  put "      VlistBox(";
                  put "       cp&t =Contingency(";
                  put "            Automatic Recalc( 0 ), Contingency Table(0),";   
                  /* kemicl disable byvars for now
                  %if %upcase(&SplitResults)=YES %then %do;
                     put "         By(eval(objRefNS:dbv)),";
                  %end;
                  */
                  put "         Y(:Name(""&TestLabelC"")),";
                  put "         X(:Name(""&TrtLabel"")),";
                  put "         Where(:Name(""&DecodLabel"") == ""\[%nrbquote(&&TestsC&t]\)""),";
                  put "         Means( 1 ),";
                  put "         Box Plots( 1 ),";
                  put "         Mean Diamonds( 1 ),";
                  put "         SendToReport(";
                  put "          Dispatch( {}, ""Tests"", OutlineBox, {Close( 1 )} )";
                  put "         )";
                  put "       );";
                  put "       try(cpr&t = cp&t << report;);";
                  put "       try(cpr&t[OutlineBox(1)]<<Set Title(locale:fill(locale:get(""XResults_Text""), {""\[%nrbquote(&&TestsC&t)]\""})););";
                  put "      ),";
               %end;
               put "   )),";  
            %end;
            put " ););"; /* end vlistbox and function*/

            put "InsertInto(objRefNS:DataList,{{""objRefNS:dmd""}});";
            put "InsertInto(objRefNS:DataNameList,{{locale:fill(locale:get(""XDistribution_Text""), {""%upcase(&domain)""})}});";
            put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""TestResults_Text"")));";
            put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:ffunc()));";
            put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
            put "InsertInto(objRefNS:TabOpenList,1);";
            put "InsertInto(objRefNS:CreateReportList,{{objRefNS:Tvlb}});";
         %end;
      %end;
      
      %if &MissingTestResults or &MissingTests %then %do;
         put "objRefNS:mtfunc = Function({}, {default local},";
         %if &MissingTestResults %then %do;
            put " If(IsEmpty(objRefNS:mrdt),";
            put "    objRefNS:mrdt = open(""&ClientOutpath.&Distribution_OutDataNamemr..sas7bdat"",invisible);";
            put "    Column(""&SubjectLabel"") << Label(1);";
            put " ,";
            put "    Current Data Table(objRefNS:mrdt);";
            put " );";
         %end;
         %if &MissingTests %then %do;
            put " If(IsEmpty(objRefNS:mtdt),";
            put "    objRefNS:mtdt = open(""&ClientOutpath.&Distribution_OutDataNamemt..sas7bdat"",invisible);";
            put "    Column(""&SubjectLabel"") << Label(1);";
            put " ,";
            put "    Current Data Table(objRefNS:mtdt);";
            put " );";
         %end;       
         put "objRefNS:dbv = {};";
         %do i=1 %to &ndbv;
            put "InsertInto(objRefNS:dbv,Column(""&&dbv&i""));";
         %end;
         put " objRefNS:MTvlb = VListBox(";
         %if &MissingTests %then %do;
            put "    TextBox(locale:get(""TableBelowShowsSubjCountsForTestsNotRecRep_Text""),";
            put "        <<Font Color(5), <<Set Font Size(10), <<Set Wrap( 650 )";
            put "    ),";
            put "    TextBox("" ""),";
            put "    Current Data Table(objRefNS:mtdt);";
            put "    Tabulate(Show Control Panel( 0 ),";
            put "       Add Table(Column Table( Statistics( N ) ),";
            %if %upcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
               put "       Column Table( Grouping Columns( :Name(""&TrtLabel"" ) ))";
            %end;
            %if %upcase(&SplitResults)=YES %then %do;
               put "       ,Row Table( Grouping Columns( :Name(""&DecodLabel""), eval(objRefNS:dbv) ) )";
            %end;
            %else %do;
               put "       ,Row Table( Grouping Columns( :Name(""&DecodLabel"")) )";
            %end; 
            put "       ),";
            put "       SendToReport(Dispatch( {}, ""Tabulate"", OutlineBox, {Set Title( locale:get(""MissingTests_Text"") )} ))";           
            put "    ),";
         %end;       
         %if &MissingTestResults %then %do;
            put "    TextBox(locale:get(""TableBelowShowsSCFTWMissResSTRESNorSTRESC_Text""),";
            put "        <<Font Color(5), <<Set Font Size(10), <<Set Wrap( 650 )";
            put "    ),";
            put "    TextBox("" ""),";
            put "    Current Data Table(objRefNS:mrdt);";
            put "    Tabulate(Show Control Panel( 0 ),";
            put "       Add Table(Column Table( Statistics( N ) ),";
            %if %upcase(&TrtUse) ne NONE and %length(&TrtLabel) %then %do;
               put "       Column Table( Grouping Columns( :Name(""&TrtLabel"" ) ))";
            %end;
            %if %upcase(&SplitResults)=YES %then %do;
               put "       ,Row Table( Grouping Columns( :Name(""&DecodLabel""), eval(objRefNS:dbv) ) )";
            %end;
            %else %do;
               put "       ,Row Table( Grouping Columns( :Name(""&DecodLabel"")) )";
            %end;       
            put "       ),";
            put "       SendToReport(Dispatch( {}, ""Tabulate"", OutlineBox, {Set Title( locale:get(""MissingTestResults_Text"") )} ))";
            put "    );";
         %end;
         put " ));"; /*close vlist and function*/
         /*kelci add missing tests completely */
         
         %if &MissingTests and  &MissingTestResults %then %do;
            put "InsertInto(objRefNS:DataList,{{""objRefNS:mtdt"", ""objRefNS:mrdt""}});";
            put "InsertInto(objRefNS:DataNameList,{{locale:get(""MissingTests_Text""),locale:get(""MissingTestResults_Text"")}});";
         %end;
         %else %if &MissingTests %then %do;
            put "InsertInto(objRefNS:DataList,{{""objRefNS:mtdt""}});";
            put "InsertInto(objRefNS:DataNameList,{{locale:get(""MissingTests_Text"")}});";

         %end;
         %else %do;
               put "InsertInto(objRefNS:DataList,{{""objRefNS:mrdt""}});";
               put "InsertInto(objRefNS:DataNameList,{{locale:get(""MissingTestResults_Text"")}});";

         %end;    
         put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""MissingTestDetails_Text"")));";
         put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:mtfunc()));";
         put "InsertInto(objRefNS:TabAfterCodeList, "" "");";
         put "InsertInto(objRefNS:TabOpenList,1);";
         put "InsertInto(objRefNS:CreateReportList,{{objRefNS:MTvlb}});";
      %end;
      %SubjectDrillDowns;
      
      %if &findingsflag=0  %then %do;
         put "objRefNS:ActionButtonList[2]={};";
         put "objRefNS:ActionCodeList[2] ={};";
         put "objRefNS:ActionIconList[2] = {};";
         put "objRefNS:ActionSelectionCountList[2] = {};";
         put "objRefNS:ActionRequirementFlagList[2] = {};";
   
         put "InsertInto(objRefNS:ActionButtonDesc , {Expr(locale:get(""ClicktoView_Text""))});";;
         put "Include(""$CLINICAL_HOME\JSLFiles\Distribution Drill Down Functions.jsl"");";   
         
         %if &doSite %then %do;
            put "InsertInto(objRefNS:ActionButtonList[1], {expr(locale:get(""DemographicCounts_Text""))});";
            put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:SSfctn(""&SiteLabel"",""&TrtLabel"", ""&AxLab"")));";
            put "InsertInto(objRefNS:ActionIconList[1],""&IconPath.DemographicCounts.gif"");";
            put "InsertInto(objRefNS:ActionSelectionCountList[1],1);";           
            put "InsertInto(objRefNS:ActionRequirementFlagList[1],0);";
         %end;          
         %if %upcase(&domain) = CM or %upcase(&domain) = AE %then %do;
            %if %sysfunc(findw(&domains, %upcase(&crossdomain))) > 0 %then %do;
               put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:fill(locale:get(""RelatedX_Text""),{""%upcase(&crossdomain)""}))});";
               put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""&crossdomain"",""&APddname"",locale:fill(locale:get(""RelatedX_Text""),{""%upcase(&crossdomain)""}))));";
               %if %upcase(&crossdomain) = AE %then %let RXIcon = %nrbquote(&IconPath.RelatedAE.gif);
               %else %if %upcase(&crossdomain) = CM %then %let RXIcon = %nrbquote(&IconPath.RelatedConcomitantMedications.gif);
               %else %if %upcase(&crossdomain) = EG %then %let RXIcon = %nrbquote(&IconPath.RelatedECG.gif);
               %else %if %upcase(&crossdomain) = LB %then %let RXIcon = %nrbquote(&IconPath.Labs.gif);
               %else %if %upcase(&crossdomain) = VS %then %let RXIcon = %nrbquote(&IconPath.RelatedVitals.gif);
               %else %let RXIcon = %nrbquote(&IconPath.DonutGraph3d.gif);
               put "InsertInto(objRefNS:ActionIconList[2],""&RXIcon"");";
               put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";
               put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
            %end;
         %end;                
         /* put in the Time Trends drill down */
         %if %sysfunc(findw(&domains, LB)) %then %do;
            put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedLabs_Text""))});";
            put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""LB"",""FindingsTimeTrends"",locale:get(""RelatedLabs_Text""))));";
            put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.Labs.gif"");";
            put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
            put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
         %end;
         %if %sysfunc(findw(&domains, VS))>0 %then %do;
            put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedVitals_Text""))});";
            put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""VS"",""FindingsTimeTrends"",locale:get(""RelatedVitals_Text""))));";
            put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.RelatedVitals.gif"");";
            put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
            put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
         %end;
         %if %sysfunc(findw(&domains, EG))>0 %then %do;
            put "InsertInto(objRefNS:ActionButtonList[2], {expr(locale:get(""RelatedECG_Text""))});";
            put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:AECMfctn(""EG"",""FindingsTimeTrends"",locale:get(""RelatedECG_Text""))));"; 
            put "InsertInto(objRefNS:ActionIconList[2],""&IconPath.RelatedECG.gif"");";
            put "InsertInto(objRefNS:ActionSelectionCountList[2],1);";           
            put "InsertInto(objRefNS:ActionRequirementFlagList[2],3);";
         %end;          
         %if %upcase(&domain) = AE %then %do;
            put "objRefNS:ActionButtonList[3]={expr(locale:get(""UniqueOccurrenceSubjectCounts_Text""))};";
            put "objRefNS:ActionCodeList[3] ={expr(objRefNS:AECountsfctn(""&SubjectLabel"", ""&DecodLabel""))};";
            put "objRefNS:ActionIconList[3] = {""&IconPath.SingleOccuranceSubjectCountReport.gif""};";
            put "objRefNS:ActionSelectionCountList[3] = {0};";
            put "objRefNS:ActionRequirementFlagList[3] = {0};";
         %end;

      %end;

      
   run;
   
   %if &MissingTestResults or &MissingTests %then %do;
      %PackageInsertFile(jsl code from &ProcessName, &OutPath, &DistributionScriptName, "&Distribution_OutDataName..sas7bdat, &Distribution_OutDataNamemr..sas7bdat, &Distribution_OutDataNamemt..sas7bdat", text/plain);  
   %end;
   %else %do;
      %PackageInsertFile(jsl code from &ProcessName, &OutPath, &DistributionScriptName, &Distribution_OutDataName..sas7bdat, text/plain); 
   %end; 
   
   %mexit:;

%mend;

