* Create data set of patient profile data;

libname outlib "&outpath";

* Check if variable &old in dataset &Dset has label.;
* If label exists then replace &_old (or asign &_new) by label.;
%macro CheckLabel(_Dset,_Old,_New=,_Flag=);
	%if %sysfunc(notname(&&&_old)) %then %do;
		%put Variable &&&_old is not a valid SAS variable name.;
	%end;
	%else %do;
		%let _ds_ = %sysfunc(open(&_dset));
		%if &_ds_ %then %do;
			%if %sysfunc(varnum(&_ds_,&&&_old)) %then %do;
				%let _l_ = %qsysfunc(varlabel(&_ds_,%sysfunc(varnum(&_ds_,&&&_old))));
				%if %length(&_l_) %then %do;
					%if %length(&_Flag) %then %let &_Flag = 1;
					%if %length(&_new) %then %let &_new = &_l_;
					%else %let &_old = &_l_;
				%end;
				%else %if %length(&_new) %then %let &_new = &&&_old;
			%end;
			%else %do;
				%put [MS_S_0230];
				%put ERROR: Variable &&&_old not found in dataset &_dset.;
			%end;
			%let _ds_ = %sysfunc(close(&_ds_));
		%end;
		%else %do;
			%put [MS_S_0231];
			%put ERROR: Open for data set &_dset failed when checking label status - %sysfunc(sysmsg());
		%end;
	%end;
%mend CheckLabel;

* drop variables from a sas data set that are all missing;
* code from http://support.sas.com/kb/24/622.html;
%macro DropMissing(data=);
	%if %sysfunc(exist(&data)) %then %do;
		%NObsVars(&data);
		%if &nobs %then %do;
			%let droplist = ;

			/*Create two macro variables, num_qty and char_qty, to hold*/
			/*the number of numeric and character variables, respectively.*/
			/*These will be used to define the number of elements in the ARRAYs*/
			/*in the next DATA step.*/
			data _null_;
			  set &data (obs=1);
			  array num_vars[*] _NUMERIC_;
			  array char_vars[*] _CHARACTER_;
			  call symputx('num_qty', dim(num_vars));
			  call symputx('char_qty', dim(char_vars));
			run;

			data _null_;
			  set &data end=finished;

			  /* Use the reserved word _NUMERIC_ to load all numeric variables  */
			  /* into the NUM_VARS array.  Use the reserved word _CHARACTER_ to */ 
			  /* to load all character variables into the CHAR_VARS array.      */
			  array num_vars[*] _NUMERIC_;
			  array char_vars[*] _CHARACTER_;

			  /* Create 'flag' arrays for the variables in NUM_VARS and CHAR_VARS. */
			  /* Initialize their values to 'missing'.  Values initialized on an   */
			  /* array statement are retained.                                     */
			  array num_miss [&num_qty] $ (&num_qty * 'missing');
			  array char_miss [&char_qty] $ (&char_qty * 'missing'); 

			  /* LIST will contain the list of variables to be dropped.  Ensure  */
			  /* it's length is sufficient.                                      */
			  length list $ 2048; 

			  /* Check for non-missing values.  Reassign the corresponding 'flag' */
			  /* value accordingly.                                               */
			  do i=1 to dim(num_vars);
				 if num_vars(i) ne . then num_miss(i)='non-miss';
			  end;
			  do i=1 to dim(char_vars);
				 if char_vars(i) ne '' then char_miss(i)='non-miss';
			  end;

			  /* On the last observation of the data set, if a 'flag' value is still */
			  /* 'missing', the variable needs to be dropped.  Concatenate the       */
			  /* variable's name onto LIST to build the values of a DROP statement   */
			  /* to be executed in another step.                                     */
			  if finished then do;
			  do i= 1 to dim(num_vars);
				 if num_miss(i) = 'missing' then list=trim(list)||' '||trim(vname(num_vars(i)));
			  end;
			  do i= 1 to dim(char_vars);
				 if char_miss(i) = 'missing' then list=trim(list)||' '||trim(vname(char_vars(i)));
			  end;
			  call symput('droplist',list);
			  end;
			run;

			/* Use the macro variable DROPLIST in the DROP statement.  PROC DATASETS can */
			/* be used to drop the variables instead of a DATA step.                  */
			%put droplist = &droplist;
			%if %length(%trim(&droplist)) %then %do;
				data &data;
				  set &data;
				  drop &droplist;
				run;
			%end;
		%end;
	%end;
%mend DropMissing;

%let domain = dm;

%include "&MacroPath./CDISCParms.sas" / nosource;
%include "&MacroPath./CSTMacros.sas" / nosource;

%macro FilterDomains;
	%if %length(&idomains) %then %do;
		%let domains0 = &domains;
		%let domains = ;
		%let i = 1;
		%do %while(%length(%scan(&domains0,&i,' ')));
			%let dom = %scan(&domains0,&i,' ');
			%if &behaviord = 1 %then %do;
				%if %sysfunc(findw(&idomains, &dom)) %then %do;
					%let domains = &domains &dom;
				%end;
			%end;
			%else %do;
				%if ^%sysfunc(findw(&idomains, &dom)) %then %do;
					%let domains = &domains &dom;
				%end;
			%end;
			%let i = %eval(&i+1);
		%end;
		%put included domains = &domains;
	%end;
%mend;

%FilterDomains;

dm "Log" clear;
dm "Output" clear;

* options mprint mlogic symbolgen;
* options nomprint nomlogic nosymbolgen;

%global ndom maxnsg;
%let ndom = 0;
%let maxnsg = 1;


**********************************************************;
**********************************************************;
**********************************************************;

%macro ORDERDOMAINS;

%let _j = 1;
%if %sysfunc(findw(&domains, SV)) %then %let _j = %eval(&_j+1);
%if %sysfunc(findw(&domains, EX)) %then %let _j = %eval(&_j+1);
%if %sysfunc(findw(&domains, DS)) %then %let _j = %eval(&_j+1);

domn = 0;
if domain = "SV" then domn = 1;

%if %length(&eventsdomains) %then %do;
	%let _i = 1;
	%do %while(%length(%scan(&eventsdomains,&_i,' ')));
    	%let temp = %scan(%upcase(&eventsdomains),&_i,' ');	
		%if &temp = DS %then %do;
			%let _j0 = 1;
			%if %sysfunc(findw(&domains, SV)) %then %let _j0 = %eval(&_j0+1);
			%if %sysfunc(findw(&domains, EX)) %then %let _j0 = %eval(&_j0+1);
        	if domain = "DS" then domn = &_j0;
		%end;
		%else %if &temp = MH %then %do;
			if domain = "MH" then domn = 0;
		%end;
		%else %do;
			if domain = "&temp" then domn = &_j;
			%if %sysfunc(findw(&domains, &temp)) %then %let _j = %eval(&_j+1);
		%end;
		%let _i = %eval(&_i + 1);
	%end;
%end;
%if %length(&interventionsdomains) %then %do;
	%let _i = 1;
	%do %while(%length(%scan(&interventionsdomains,&_i,' ')));
    	%let temp = %scan(%upcase(&interventionsdomains),&_i,' ');	
		%if &temp = EX %then %do;
			%let _j0 = 1;
			%if %sysfunc(findw(&domains, SV)) %then %let _j0 = %eval(&_j0+1);
        	if domain = "EX" then domn = &_j0;
		%end;
		%else %do;
			if domain = "&temp" then domn = &_j;
			%if %sysfunc(findw(&domains, &temp)) %then %let _j = %eval(&_j+1);
		%end;
		%let _i = %eval(&_i + 1);
	%end;
%end;
%if %length(&findingsdomains) %then %do;
	%let _i = 1;
	%do %while(%length(%scan(&findingsdomains,&_i,' ')));
    	%let temp = %scan(%upcase(&findingsdomains),&_i,' ');	
		if domain = "&temp" then domn = &_j;
		%if %sysfunc(findw(&domains, &temp)) %then %let _j = %eval(&_j+1);
		%let _i = %eval(&_i + 1);
	%end;
%end;
%mend ORDERDOMAINS;

%macro NObsVars(_ds); 
	%global nvars nobs nlobs;
   %let _dset_ = &_ds;
   %let _dsid_ = %sysfunc(open(&_dset_));
   %if &_dsid_ %then %do;
      %let nobs = %sysfunc(attrn(&_dsid_,NOBS));
      %let nlobs = %sysfunc(attrn(&_dsid_,NLOBS));
      %let nvars = %sysfunc(attrn(&_dsid_,NVARS));
   %end;
   %else %do;
      %let nobs = 0;
      %let nlobs = 0;
      %let nvars = 0;
      %let exiterror = 1;      
   %end;
   %let _dsid_ = %sysfunc(close(&_dsid_));
%mend NObsVars;

**********************************************************;
**********************************************************;
**********************************************************;

%macro PROFILE;

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%let notesDataset =;
%jg_getNotesDatasetName(notesDataset);

%LocaleInit();
%let trrflg = %nrbquote(%LocaleGet(ReviewFlag_Text));
%let trStable = %nrbquote(%LocaleGet(Stable_Text));
%let trNormalRange = %nrbquote(%LocaleGet(NormalRange_Text));
%let trStartDTofVisit = %nrbquote(%LocaleGet(StartDateTimeofVisit_Text));
%let trVisitDay = %nrbquote(%LocaleGet(VisitDay_Text));
%let trStudyDay = %nrbquote(%LocaleGet(StudyDay_Text));
%let trResult = %nrbquote(%LocaleGet(Result_Text));
%let trDose = %nrbquote(%LocaleGet(Dose_Text));
%let trStartDay = %nrbquote(%LocaleGet(StartDay_Text));
%let trEndDay = %nrbquote(%LocaleGet(EndDay_Text));
%let trImp = %nrbquote(%LocaleGet(ImpDot_Text));

%let estring = ERROR;
%let keepvars = USUBJID STDY ENDY XOFFSET1 XOFFSET2 YOFFSET PPLABEL1-PPLABEL4 AXISLABEL1-AXISLABEL4 DOMAIN CATEGORY;
%if ^%length(&in_adsl) %then %do;
   %put &estring.: The ADSL, DM or ADDM data set does not exist for this study.;
   %let exiterror = 1;
   %goto sexit;
%end;

%let rschk = %sysfunc(open(&in_dm));
%let rsdchk = %sysfunc(varnum(&rschk,RFSTDTC));
%let agechk = %sysfunc(varnum(&rschk,AGE));
%let brthchk = %sysfunc(varnum(&rschk,BRTHDTC));
%let rschk = %sysfunc(close(&rschk));

data subjects;
	if 0 then set profilesubjects;
	if _N_ = 1 then do;
		declare hash h(dataset:"profilesubjects");
		h.defineKey("USUBJID");
		h.defineData("USUBJID");
		h.defineDone();
	end;
	set &in_dm end=eof_dm;
	if h.find() = 0;
	keep USUBJID 
		%if &rsdchk %then RFSTDTC;
		%if &agechk %then AGE;
		%if &brthchk %then BRTHDTC;
	;
run; 

/*
data outlib.subjects;
	set subjects;
run
*/

proc sort data=subjects;
	by USUBJID;
run;

%NObsVars(subjects);
%if not &nobs %then %do;
	%put &estring.: No subjects matched the selected values of USUBJID.  The default study is likely mismatched with the table.;
	%put ;
	%put Run any AP with the study you wish to make active, then try again.;
	%put NOTE: Profiling stopped.;
	%let exiterror = 1;
	%goto sexit;
%end;

%if %length(&domains) %then %do;
	%let ndomain = %sysfunc(countw(&domains));
%end;
%else %do;
	%let ndomain = 0;
%end;


%put domains = &domains;
%put eventsdomains = &eventsdomains;
%put interventionsdomains = &interventionsdomains;
%put findingsdomains = &findingsdomains;
%put ndomain = &ndomain;
%let AESeverityVar = ;
%let AESeverityType = N;
%let AESeverity4 = 0;
%let AESeverity5 = 0;
%let fdomainspc = ;
%let fdomainspn = ;
%if ^%symexist(MergeSuppDom) %then %let MergeSuppDom = 0;
%put sysencoding = &SYSENCODING;
/*
%if %nrbquote(&SYSENCODING) eq wlatin1 %then %do;
	%let max_label_len = 32;
%end;
%else %do;
	%let max_label_len = 64;
%end;
*/
%let max_label_len = 48;
%let mllm3 = %eval(&max_label_len-3);

*******************************************************************;
*		Clinical Variable Specification Block of CDISC Variable Usage;
*******************************************************************;
/* 	
$CVS_START$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$CMP: OSYS.Severity Variable$ $UE: AE.AESEV or AE.AETOXGR$ 
$CMP: OSYS.Dose Variable$ $UE: EX.EXDOSE$ 
$CMP: OSYS.Treatment Variable$ $UE: EX.EXTRT$ 
$CMP: OSYS.Term Variable$ $UE: INTERVENTIONS.xxDECOD or INTERVENTIONS.xxTERM or INTERVENTIONS.xxTRT$ 
$CMP: OSYS$ $UE: INTERVENTIONS.xxCAT$ 
$CMP: OSYS.Term Variable$ $UE: EVENTS.xxDECOD or EVENTS.xxTERM or EVENTS.xxTRT$ 
$CMP: OSYS$ $UE: EVENTS.xxCAT$ 
$CMP: OSYS$ $UE: FINDINGS.xxSTRESN$ 
$CMP: OSYS$ $UE: FINDINGS.xxPOS$ 
$CMP: OSYS$ $UE: FINDINGS.xxLOC$ 
$CMP: OSYS$ $UE: FINDINGS.xxFAST$ 
$CMP: OSYS$ $UE: FINDINGS.xxSPEC$ 
$CMP: OSYS$ $UE: FINDINGS.xxMETHOD$ 
$CMP: OSYS$ $UE: FINDINGS.xxTPT$ 
$CMP: OSYS$ $UE: FINDINGS.xxTPTNUM$ 
$CMP: OSYS$ $UE: FINDINGS.xxBLFL$ 
$CMP: OSYS$ $UE: FINDINGS.xxSTRESU$ 
$CMP: OSYS$ $UE: FINDINGS.xxTESTCD$ 
$CVS_STOP$
*/

%let fcdoms = ;
%let fndoms = ;
%do ii=1 %to &ndomain;
	%let temp = %scan(%upcase(&domains),&ii,' ');
	%put temp=&temp &temp.VARS=&&&temp.VARS;

	options notes;
		
	%LOADDATA(domlist=&temp,mergesupp=&MergeSuppDom,filterds=subjects);
	
	/*
	libname cout "C:\temp";
	data cout.h&temp;
		set &&in_&temp;
	run;
	*/
	
	* split findings into numeric and categorical rows;
	%let fcat = 0;
	%let fnum = 0;
	%if %sysfunc(findw(&findingsdomains, &temp)) %then %do;
		data temp_num temp_cat;
			set &&in_&temp;
			if (missing(&temp.STRESN) or index(upcase(&temp.CAT),"QUALITATIVE")) and not missing(&temp.STRESC) then output temp_cat;
			else if not missing(&temp.STRESN) then output temp_num;
		run;
		%NObsVars(temp_cat);
		%let fcat = &nobs;
		%NObsVars(temp_num);
		%let fnum = &nobs;	
		%put fcat=&fcat fnum=&fnum;
	%end;

	%if %sysfunc(findw(&eventsdomains, &temp)) or %sysfunc(findw(&interventionsdomains, &temp)) or
		(%sysfunc(findw(&findingsdomains, &temp)) and &fcat)
		%then %do;
		%put categorical domain;

		* determine severity variable;
		%let SeverityVar = ;
		%if &temp = AE %then %do;
			%if %sysfunc(findw(&&&temp.VARS, &temp.TOXGR)) > 0 %then %let SeverityVar = &temp.TOXGR; 
			%else %if %sysfunc(findw(&&&temp.VARS, &temp.SEV)) > 0 %then %let SeverityVar = &temp.SEV;
			%else %do;
				%put WARNING: Either &temp.SEV or &temp.TOXGR must be in the &temp data set.;
			%end;
			%let AESeverityVar = &SeverityVar;
			/*
			%if %length(&AESeverityVar) %then %do;
				%VarAttributes(&&in_&temp,&AESeverityVar,_type=AESeverityType);
			%end;
			*/
		%end;
		
		%if &temp = EX %then %do;
			* make sure dose is numeric;
			data ex;
				set &&in_&temp;
				exdose0 = 0 + EXDOSE;
			run;
			proc sql noprint;
				create table &temp as
				select *, min(exdose0) as YP10_1, max(exdose0) as YP90_1 
				from ex
				group by EXTRT
				order by USUBJID;
			quit;
			%let in_ex = &temp;
			/*
			data outlib.ex;
				set &temp;
			run;
			*/
		%end;
		
		data &temp.tmp;
		   length domain $ 8;
			/*
			if 0 then set subjects;
			if _N_ = 1 then do;
				declare hash h(dataset:"subjects");
				h.defineKey("USUBJID");
				h.defineData(ALL:"YES");
				h.defineDone();
			end;
			*/
			%if &fcat %then %do;
				set temp_cat
			%end;
			%else %do;
			   set &&in_&temp 
			%end;
				/*(keep= USUBJID &temp.RFLG
				%if %sysfunc(findw(&&&temp.VARS, &temp.DY)) %then &temp.DY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STDY)) %then &temp.STDY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.ENDY)) %then &temp.ENDY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STDTC)) %then &temp.STDTC;
				%if %sysfunc(findw(&&&temp.VARS, &temp.ENDTC)) %then &temp.ENDTC;
            %if %sysfunc(findw(&&&temp.VARS, &temp.STRF)) %then &temp.STRF;
            %if %sysfunc(findw(&&&temp.VARS, &temp.ENRF)) %then &temp.ENRF;
            %if %sysfunc(findw(&&&temp.VARS, &temp.STRTPT)) %then &temp.STRTPT;
            %if %sysfunc(findw(&&&temp.VARS, &temp.STTPT)) %then &temp.STTPT;            
            %if %sysfunc(findw(&&&temp.VARS, &temp.ENRTPT)) %then &temp.ENRTPT;
            %if %sysfunc(findw(&&&temp.VARS, &temp.ENTPT)) %then &temp.ENTPT;   
            %if %sysfunc(findw(&&&temp.VARS, &temp.PRESP)) %then &temp.PRESP;
            %if %sysfunc(findw(&&&temp.VARS, &temp.OCCUR)) %then &temp.OCCUR;                                    
				%if %sysfunc(findw(&&&temp.VARS, &temp.DECOD)) %then &temp.DECOD;
				%if %sysfunc(findw(&&&temp.VARS, &temp.TERM)) %then &temp.TERM;
				%if %sysfunc(findw(&&&temp.VARS, &temp.BODSYS)) %then &temp.BODSYS;
				%if %sysfunc(findw(&&&temp.VARS, &temp.SER)) %then &temp.SER;
				%if %sysfunc(findw(&&&temp.VARS, &temp.ACN)) %then &temp.ACN;
				%if %sysfunc(findw(&&&temp.VARS, &temp.REL)) %then &temp.REL;
				%if %sysfunc(findw(&&&temp.VARS, &temp.OUT)) %then &temp.OUT;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STDTC)) and &temp = MH %then &temp.STDTC;
            %if %length(&SeverityVar) %then &SeverityVar; 
				%if %sysfunc(findw(&interventionsdomains, &temp)) %then &temp.TRT;
				%if %sysfunc(findw(&findingsdomains, &temp)) %then %do;
					%let fdomainspc = &fdomainspc &temp;
					%if %sysfunc(findw(&&&temp.VARS, &temp.TESTCD)) %then &temp.TESTCD;
					%if %sysfunc(findw(&&&temp.VARS, &temp.STRESC)) %then &temp.STRESC;
					%if %sysfunc(findw(&&&temp.VARS, &temp.SPEC)) %then &temp.SPEC;
				%end;
				%if &temp = EX %then EXDOSE;
				%if &temp = EX %then YP10_1;
				%if &temp = EX %then YP90_1;
				)*/ end=eof_ds2;
			/*
			if h.find() = 0;
			*/
			domain = "&temp";
			
			length AXISLABEL1-AXISLABEL4 $ &max_label_len CATEGORY $ &max_label_len LeftSymbol $ 1 RightSymbol $ 1 LineType $ 1;

			%if &ReviewsEnabled %then %do;
         	rename &temp.RFLG = RevFlag;
         %end;
         
         _impstart = "";
         _impend = "";
         *** Impute Missing Dates Based on Timing Flags ***;
         /*
			%PROFILEDATEIMPUTE;
			*/
			
			*** Delete Prespecified Records for CM,SU,MH,CE that did not occur ***;
			%PRESPECIFY(dom = &temp);
			
			%if %sysfunc(findw(&&&temp.VARS, &temp.PRESP)) %then %do;
			   drop &temp.PRESP;
			%end;
			%if %sysfunc(findw(&&&temp.VARS, &temp.OCCUR)) %then %do;
			   drop &temp.OCCUR;
			%end;
			%if %sysfunc(findw(&&&temp.VARS, &temp.SEQ)) %then %do;
				drop &temp.SEQ;
			%end;
			%if %sysfunc(findw(&&&temp.VARS, &temp.HLT)) %then %do;
				drop &temp.HLT;
			%end;
			%if %sysfunc(findw(&&&temp.VARS, &temp.HLGT)) %then %do;
				drop &temp.HLGT;
			%end;
			%if %sysfunc(findw(&&&temp.VARS, &temp.CAT)) %then %do;
				if lengthn(&temp.CAT) then do;
					CATEGORY = &temp.CAT;
					if upcase(CATEGORY)="OTHER" then CATEGORY="ZZZ";
				end;
				else CATEGORY = "ZZZ";
			%end;
			%else %do;
				CATEGORY = "";
			%end;
			/*
			%calculateStudyDays(&temp);
			*/
			%if &temp = EX %then %do;
				AXISLABEL1 = EXTRT;
				AXISLABEL2 = AXISLABEL1;
				AXISLABEL3 = AXISLABEL1;
				AXISLABEL4 = AXISLABEL1;
				PPLABEL1 = catx(" ", EXTRT,"&trDose:", EXDOSE, "[&trStudyDay:", stdy, "&trEndDay:", cats(endy,"]"));
				PPLABEL2 = PPLABEL1;
				PPLABEL3 = PPLABEL1;
				PPLABEL4 = PPLABEL1;
				OUTLIER1 = 0;
				STNRLO = .;
				STNRHI = .;
				if missing(EXDOSE) then delete;
			%end;
			%else %do;
				length &temp.DECOD1 $ 512;
				%if &fcat %then %do;
				    where not missing(&temp.STRESC);
				%end;
				OUTLIER1 = 0;
				if not missing(&temp.DECOD) then &temp.DECOD1 = trim(left(&temp.DECOD));
				else if not missing(&temp.TERM) then &temp.DECOD1 = trim(left(&temp.TERM));
				else if not missing(&temp.TRT) then &temp.DECOD1 = trim(left(&temp.TRT));
				else &temp.DECOD1 = "";
				if &temp.DECOD1 = "" then do;
					%if %sysfunc(findw(&findingsdomains, &temp)) %then %do; 
					   &temp.DECOD1 = &temp.TESTCD;
					   /*
						&temp.DECOD1 = catx(' ',&temp.TESTCD,&temp.TEST);
						*/
						/*
						%if %sysfunc(findw(&&&temp.VARS, &temp.SPEC)) %then %do;
							&temp.DECOD1 = catx(' ',&temp.TESTCD,&temp.SPEC,&temp.STRESC);
						%end;
						%else %do;
							&temp.DECOD1 = catx(' ',&temp.TESTCD,&temp.STRESC);
						%end;	
						*/
						/*
						%if &temp = LB %then %do;
						    if trim(left(upcase(&temp.STRESC))) ne "NORMAL" then OUTLIER1 = 1;
						%end;
						*/
						%if %sysfunc(findw(&&&temp.VARS, &temp.STRESC)) %then %do;
							if trim(left(upcase(&temp.STRESC))) = "ABNORMAL" then OUTLIER1 = 1;
						%end;
						%if %sysfunc(findw(&&&temp.VARS, &temp.RIND)) %then %do;
						   if not missing(&temp.RIND) and upcase(compress(&temp.RIND)) ne "NORMAL" then OUTLIER1 = 1;
						%end;
					%end;		
				end;
				if &temp.DECOD1 = "" then delete;
				AXISLABEL1 = &temp.DECOD1;
				AXISLABEL2 = AXISLABEL1;
				AXISLABEL3 = AXISLABEL1;
				AXISLABEL4 = AXISLABEL1;
				
				LeftSymbol = "";
				RightSymbol = "";
				LineType = "";
				%let termlist = "NOT RECOVERED/NOT RESOLVED", "NOT RECOVERED/RESOLVED",
					"NOT RECOVERED", "NOT RESOLVED", "RECOVERING/RESOLVING", "RECOVERING", 
					"RESOLVING", "CONTINUING", "ONGOING";
				%let termlista = "AFTER";
				%let termlistb = "BEFORE";
				%let termlistc = "DURING", "DURING/AFTER", "COINCIDENT";
					
				%if &temp = AE %then %do;
					%if %sysfunc(indexw(&&&temp.VARS, AEOUT, ' ')) %then %do;
						if (upcase(AEOUT) in (&termlist)) then do;
							if missing(ENDY) then RightSymbol = ">";
							else RightSymbol = "?";
						end;
					%end;
					if (STDY = 0 or missing(STDY)) or (missing(ENDY) and RightSymbol = "") then LineType = "D";
					%if %length(&AESeverityVar) %then %do;
					   if not missing(&AESeverityVar) and length(compress(&AESeverityVar)) > 1 then call symputx('AESeverityType',"C");
						if upcase(&AESeverityVar) = "LIFE THREATENING" or upcase(&AESeverityVar) = "LIFE-THREATENING"  or 
							 &AESeverityVar = "%LocaleGet(LifeThreatening_Text)" or &AESeverityVar = "4" then call symputx('AESeverity4',1);
						else if upcase(&AESeverityVar) = "DEATH" or upcase(&AESeverityVar) = "FATAL" or
							 &AESeverityVar = "%LocaleGet(Death_Text)" or &AESeverityVar = "5" then call symputx('AESeverity5',1);
					%end;
				%end;
				%else %if &fcat %then %do; 
					if missing(ENDY) then ENDY = STDY;
				%end;
				%else %if &temp ne DS %then %do;
					%if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)STRF, ' ')) %then %do;
						if upcase(&temp.STRF) in (&termlist) then do;
							if (STDY = 0 or missing(STDY)) then LeftSymbol = "<";
							else LeftSymbol = "?";
						end;
						else if upcase(&temp.STRF) in (&termlistb) then do;
							if (STDY = 0 or missing(STDY)) then LeftSymbol = "<";
							else if (STDY > 0) then LeftSymbol = "?";
						end;
						else if upcase(&temp.STRF) in (&termlistc) then do;
							if (STDY < 0) then LeftSymbol = "?";
						end;
					%end;
					%if %sysfunc(indexw(&&&temp.VARS, %upcase(&temp)ENRF, ' ')) %then %do;
						if upcase(&temp.ENRF) in (&termlist) then do;
							if missing(ENDY) then RightSymbol = ">";
							else RightSymbol = "?";
						end;
						else if upcase(&temp.ENRF) in (&termlista) then do;
							if missing(ENDY) then RightSymbol = ">";
							else if (ENDY <= 0) then RightSymbol = "?";
						end;
						else if upcase(&temp.ENRF) in (&termlistb) then do;
							if (ENDY > 0) then RightSymbol = "?";
						end;
						else if upcase(&temp.ENRF) in (&termlistc) then do;
							if (ENDY <= 0) then RightSymbol = "?";
						end;
					%end;
					if ((STDY = 0 or missing(STDY)) and LeftSymbol = "") or 
						(missing(ENDY) and RightSymbol = "") then LineType = "D";
				%end;
				if not missing(_impstart) and _impstart = "Y" and LeftSymbol = "" then LeftSymbol = "-";
				
				%if &temp = CM or &temp = SU or &temp = CE or (&temp = AE and %sysfunc(findw(&&&temp.VARS, &temp.REL)) = 0) %then %do;
				   if _impstart = "" and _impend = "" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "&trEndDay:", cats(endy,"]"));
				   else if _impstart = "Y" and _impend = "" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "(&trImp)", "&trEndDay:", cats(endy,"]"));
				   else if _impstart = "" and _impend = "Y" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "&trEndDay:", endy,"(&trImp)]");
				   else if _impstart = "Y" and _impend = "Y" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "(&trImp)", "&trEndDay:", endy,"(&trImp)]");
				%end;
				%else %if (&temp = AE and %sysfunc(findw(&&&temp.VARS, &temp.REL)) > 0) %then %do;
				   if _impstart = "" and _impend = "" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "&trEndDay:", cats(endy,"],",&temp.REL));
				   else if _impstart = "Y" and _impend = "" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "(&trImp)", "&trEndDay:", cats(endy,"],",&temp.REL));
				   else if _impstart = "" and _impend = "Y" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "&trEndDay:", endy,"(&trImp)],",&temp.REL);
				   else if _impstart = "Y" and _impend = "Y" then PPLABEL1 = catx(" ", &temp.DECOD1, "[&trStartDay:", stdy, "(&trImp)", "&trEndDay:", endy,"(&trImp)],",&temp.REL);
				%end;
				%else %if &temp = MH %then %do;
					PPLABEL1 = AXISLABEL2;
				%end;
				%else %if &fcat %then %do;
					PPLABEL1 = catx(" ", cats(&temp.DECOD1,":"), &temp.STRESC, "[&trStudyDay:", cats(stdy,"]"));
				%end;
				%else %do;
					PPLABEL1 = catx(" ", AXISLABEL2,"[&trStudyDay:", cats(stdy,"]"));
				%end;
				PPLABEL2 = PPLABEL1;
				PPLABEL3 = PPLABEL1;
				PPLABEL4 = PPLABEL1;
			%end;
			
			*** dropping mhcat. will delete anything without an --TERM or --DECOD;
			%if &temp = MH %then %do;
				*** if date is missing, set values to 0;
				%if %sysfunc(findw(&MHVARS, MHSTDTC)) > 0 %then %do; 
					 if MHSTDTC ne "" then do;
						EVENT_YEAR = 0 + substr(MHSTDTC, 1, 4);
						if lengthn(scan(MHSTDTC,2,"-")) then do;
							EVENT_MONTH = 0 + substr(MHSTDTC, 6, 2);
						end;
						else do;
							EVENT_MONTH = 6;
						end;
						if lengthn(scan(MHSTDTC,3,"-")) then do;
							EVENT_DAY = 0 + substr(MHSTDTC, 9, 2);
						end;
						else do;
							EVENT_DAY = 15;
						end;						
					end;
					else if MHSTDTC = "" then do;
						EVENT_YEAR = 0;
						EVENT_MONTH = 0;
						EVENT_DAY = 0;
					end;
				%end;
				%else %do;
					EVENT_YEAR = 0;
					EVENT_MONTH = 0;
					EVENT_DAY = 0;
				%end;
				
				TRIAL_YEAR = substr(RFSTDTC, 1, 4);
				TRIAL_MONTH = substr(RFSTDTC, 6, 2);				
				%if %sysfunc(findw(&DMVARS, DMDTC)) > 0 %then %do;
					if missing(TRIAL_YEAR) and missing(TRIAL_MONTH) then do;
						TRIAL_YEAR = substr(DMDTC, 1, 4);
						TRIAL_MONTH = substr(DMDTC, 6, 2);
					end;
				%end;
			%end;	
		run;
		
		/*
		data outlib.&temp.tmp;
			set &temp.tmp;
		run;
		*/
		
		%if &temp = EX %then %do;
			proc sql noprint;
				create table &temp.tmp as
				select *, min(stdy) as XMIN, max(stdy) as XMAX 
				from &temp.tmp
				group by USUBJID
				order by USUBJID;
			quit;
			%let &temp.vlist = EXTRT;
		%end;
		%else %if &temp ne EX and &temp ne MH %then %do;

			* keep only obs from first study day;
			proc sort data=&temp.tmp(keep =USUBJID &temp.DECOD1 stdy) out=nc;
				by USUBJID &temp.DECOD1 stdy;
			run;

			data nc;
				set nc;
				by USUBJID &temp.DECOD1 stdy;
				if first.&temp.DECOD1;
			run;

			* sort by first study day;
			proc sort data=nc;
				by USUBJID stdy &temp.DECOD1;
			run;

			data nc;
				set nc;
				&temp.SORT = _n_;
				drop stdy;
			run;

			proc sql;
				create table &temp.tmp(drop=&temp.DECOD1 USUBJID1 USUBJID2 &temp.DECOD11 &temp.DECOD12) as
				select coalesce(A.USUBJID1, B.USUBJID2) as USUBJID, coalesce(A.&temp.DECOD11, B.&temp.DECOD12) as &temp.DECOD1, a.*, b.*  
				from &temp.tmp(rename = (USUBJID = USUBJID1 &temp.DECOD1 = &temp.DECOD11)) as A left join nc(rename = (USUBJID = USUBJID2 &temp.DECOD1 = &temp.DECOD12)) as B 
					on A.USUBJID1 = B.USUBJID2 and A.&temp.DECOD11 = B.&temp.DECOD12;
			quit;
			%let &temp.vlist = &temp.SORT;
		%end;
		%if &fcat %then %do;
			%let fdomainspc = &fdomainspc &temp;
			data &temp.tmpc;
				set &temp.tmp;
			run;
			%let fcdoms = &fcdoms &temp.tmpc;
		%end;
	%end;
	%if %sysfunc(findw(&findingsdomains, &temp)) and &fnum %then %do;
	
		%put numeric domain;
		
		* search for test code modifier variables and build a list of them;
		%let modlist = POS LOC FAST SPEC METHOD TPT;
		%let modvlist = ;
		%let modvlistc = ;
		%let modvlist1 = ;
		%let modvlist2 = ;
		%do jj=1 %to %sysfunc(countw(&modlist));
			%let modifier = %scan(&modlist,&jj,' ');
			%if %sysfunc(findw(&&&temp.VARS, &temp.&modifier)) %then %do;
				%if &modifier = TPT %then %do;
					%if %sysfunc(findw(&&&temp.VARS, &temp.TPTNUM)) %then %do;
						%let modvlist = &modvlist &temp.TPTNUM;
						%if ^%length(&modvlistc) %then %do;
							%let modvlistc = &temp.TPTNUM;
						%end;
						%else %do;
							%let modvlistc = &modvlistc, &temp.TPTNUM;
						%end;
					%end;
					%else %do;
						%let modvlist = &modvlist &temp.&modifier;
						%if ^%length(&modvlistc) %then %do;
							%let modvlistc = &temp.&modifier;
						%end;
						%else %do;
							%let modvlistc = &modvlistc, &temp.&modifier;
						%end;
					%end;
					%if ^%length(&modvlist2) %then %do;
						%let modvlist2 = &temp.&modifier;
					%end;
					%else %do;
						%let modvlist2 = &modvlist2 &temp.&modifier;
					%end;
				%end;
				%else %do;
					%let modvlist = &modvlist &temp.&modifier;
					%if ^%length(&modvlistc) %then %do;
						%let modvlistc = &temp.&modifier;
					%end;
					%else %do;
						%let modvlistc = &modvlistc, &temp.&modifier;
					%end;
					%if ^%length(&modvlist1) %then %do;
						%let modvlist1 = &temp.&modifier;
					%end;
					%else %do;
						%let modvlist1 = &modvlist1 &temp.&modifier;
					%end;
					%if ^%length(&modvlist2) %then %do;
						%let modvlist2 = &temp.&modifier;
					%end;
					%else %do;
						%let modvlist2 = &modvlist2 &temp.&modifier;
					%end;
				%end;
			%end;
		%end;
		%put modvlist = &modvlist;
		%put modvlistc = &modvlistc;
		%put modvlist1 = &modvlist1;
		%put modvlist2 = &modvlist2;

		/*
		libname oc "C:\temp";
		data oc.&temp.tmp0;
			set &temp;
		run;
		*/
		
		%ComputeBaseline(&temp,&temp.2);
		
		data &temp.tmp;
			/*
			if 0 then set subjects;
			if _N_ = 1 then do;
				declare hash h(dataset:"subjects");
				h.defineKey("USUBJID");
				h.defineData(ALL:"YES");
				h.defineDone();
			end;
			*/
			set &temp.2 
				/*(keep=USUBJID STDY &temp.RFlg &temp.TESTCD &temp.TEST &temp.STRESN BLCHG PBLCHG PofBL &modvlist
				%if %sysfunc(findw(&&&temp.VARS, &temp.DY)) %then &temp.DY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STDY)) %then &temp.STDY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.ENDY)) %then &temp.ENDY;
				%if %sysfunc(findw(&&&temp.VARS, &temp.DTC)) %then &temp.DTC;						
				%if %sysfunc(findw(&&&temp.VARS, VISITNUM)) %then VISITNUM;
				%if %sysfunc(findw(&&&temp.VARS, &temp.CAT)) %then &temp.CAT;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STRESU)) %then &temp.STRESU;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STNRLO)) and &temp = LB %then LBSTNRLO;
				%if %sysfunc(findw(&&&temp.VARS, &temp.STNRHI)) and &temp = LB %then LBSTNRHI;
				%if %sysfunc(indexw(&&&temp.VARS, &temp.BLFL)) %then &temp.BLFL;
				)*/ end=eof_ds2;
			/*
			if h.find() = 0;
			*/
			if missing(&temp.STRESN) or index(upcase(&temp.CAT),"QUALITATIVE") then delete;
			if missing(ENDY) then ENDY = STDY;
			%if &ReviewsEnabled %then %do;
				rename &temp.RFLG = RevFlag;
			%end;
		run;

		/*
		libname oc "C:\temp";
		data oc.&temp.tmp;
			set &temp.tmp;
		run;
		*/
	
		proc sort data=&temp.tmp;
			by &temp.testcd &modvlist;
		run;
		
		* read in pre-computed quantiles for all subjects;
		data findm;
			set basemeta.&temp.quantiles;
		run;
		
		/*
		libname oc "C:\temp";
		data oc.&temp.findm;
			set findm;
		run;
		*/
		
		* verify if xxTPTNUM exists in both;

		data &temp.tmp;
			merge &temp.tmp findm;
			by &temp.testcd &modvlist1;
		run;

		data &temp.tmp;
		   length domain $ 8;
			set &temp.tmp;
			array YVAR[4] &temp.STRESN BLCHG PBLCHG PofBL;
			array YP25[4] YP25_1-YP25_4;
			array YP50[4] YP50_1-YP50_4;
			array YP75[4] YP75_1-YP75_4;
			array RESIDUAL[4] RESIDUAL1-RESIDUAL4;
			array OUTLIER[4] OUTLIER1-OUTLIER4;
			STRESN = &temp.STRESN;
			where not missing(&temp.STRESN);
			%if %sysfunc(findw(&&&temp.VARS, &temp.STRESU)) %then %do;
				STRESU = &temp.STRESU;
			%end;
			domain = "&temp";
			length AXISLABEL1-AXISLABEL4 $ &max_label_len CATEGORY $ &max_label_len;
			AXISLABEL1 = &temp.TESTCD;			
			%if %length(&modvlist1) %then %do;
				%let j = 1;
				%do %while(%length(%scan(&modvlist1,&j,' ')));
					%let modifier = %scan(&modvlist1,&j,' ');
					if not missing(&modifier) then do; 
						AXISLABEL1 = catx(' ',AXISLABEL1,&modifier);
					end;
					%let j = %eval(&j+1);
				%end;
			%end;
			AXISLABEL2 = &temp.TESTCD;			
			%if %length(&modvlist2) %then %do;
				%let j = 1;
				%do %while(%length(%scan(&modvlist2,&j,' ')));
					%let modifier = %scan(&modvlist2,&j,' ');
					if not missing(&modifier) then do; 
						AXISLABEL2 = catx(' ',AXISLABEL2,&modifier);
					end;
					%let j = %eval(&j+1);
				%end;
			%end;
			AXISLABEL3 = AXISLABEL1;
			AXISLABEL4 = AXISLABEL2;
			%if %sysfunc(findw(&&&temp.VARS, &temp.CAT)) %then %do;
				if lengthn(&temp.CAT) then do;
					CATEGORY = &temp.CAT;
					if upcase(CATEGORY)="OTHER" then CATEGORY="ZZZ";
				end;
				else CATEGORY = "ZZZ";
			%end;
			%else %do;
				CATEGORY = "";
			%end;
			PPLABEL1 = catx(' ',AXISLABEL2,"&trResult:",STRESN,"[&trStudyDay:", cats(stdy,"]"));
			PPLABEL2 = catx(' ',AXISLABEL2,"&trResult:",BLCHG,"[&trStudyDay:", cats(stdy,"]"));
			PPLABEL3 = catx(' ',AXISLABEL2,"&trResult:",PBLCHG,"[&trStudyDay:", cats(stdy,"]"));
			PPLABEL4 = catx(' ',AXISLABEL2,"&trResult:",PofBL,"[&trStudyDay:", cats(stdy,"]"));
			do i=1 to 4;
				RESIDUAL[i] = (YVAR[i] - YP50[i])/max(0.01,(YP75[i] - YP25[i]));
				if abs(RESIDUAL[i]) >= 1.5 then OUTLIER[i] = 1;
				else OUTLIER[i] = 0;
			end;
			STNRLO = &temp.STNRLO;
			STNRHI = &temp.STNRHI;
			length NorRange $ 32;
			if missing(STNRLO) or missing(STNRHI) then NorRange = "";
			else NorRange = trim(left(STNRLO)) || "-" || trim(left(STNRHI));
			%if &ReviewsEnabled %then %do;
				label RevFlag="&trrflg" NorRange="&trNormalRange";
			%end;
			%else %do;
				label NorRange="&trNormalRange";
			%end;
			/*
			keep &keepvars &temp.TESTCD &temp.TEST &temp.CAT &modvlist STRESN STRESU BLCHG PBLCHG PofBL
				%if %sysfunc(findw(&&&temp.VARS, VISITNUM)) %then VISITNUM; 
				%if %sysfunc(findw(&&&temp.VARS, &temp.TPTNUM)) %then &temp.TPTN_; 
				YP10_: YP90_: OUTLIER: STNRLO STNRHI NorRange %if &ReviewsEnabled %then RevFlag ;
			;	
			*/
			drop _BASE YP25_: YP50_: YP75_: i RESIDUAL:;
		run;
		
		/*
		data outlib.&temp.tmp;
			set &temp.tmp;
		run;
		*/

		* add grouping variables if grouping data set is present;
		%if %length(&&ing_&temp) %then %do;
			%GroupVars(&temp,&temp.tmp,&temp.tmp);
			%if %length(&modvlist) %then %do;			
				%let &temp.vlist = &temp.GRPN &temp.SGRPN &modvlist;
				%let &temp.vlistc = &temp.GRPN, &temp.SGRPN, &modvlistc;
				data &temp.tmp;
					set &temp.tmp;
					AXISLABEL3 = &temp.GRP;
					%if %length(&modvlist1) %then %do;
						%let j = 1;
						%do %while(%length(%scan(&modvlist1,&j,' ')));
							%let modifier = %scan(&modvlist1,&j,' ');
							if not missing(&modifier) then do; 
								AXISLABEL3 = catx(' ',AXISLABEL3,&modifier);
							end;
							%let j = %eval(&j+1);
						%end;
					%end;
					AXISLABEL4 = &temp.GRP;			
					%if %length(&modvlist2) %then %do;
						%let j = 1;
						%do %while(%length(%scan(&modvlist2,&j,' ')));
							%let modifier = %scan(&modvlist2,&j,' ');
							if not missing(&modifier) then do; 
								AXISLABEL4 = catx(' ',AXISLABEL4,&modifier);
							end;
							%let j = %eval(&j+1);
						%end;
					%end;
				run;
			%end;
			%else %do;
				%let &temp.vlist = &temp.GRPN &temp.SGRPN;
				%let &temp.vlistc = &temp.GRPN, &temp.SGRPN;
				data &temp.tmp;
					set &temp.tmp;
					AXISLABEL3 = &temp.GRP;
					AXISLABEL4 = &temp.GRP;
				run;			
			%end;
		%end;
		%else %do;
			%if %length(&modvlist) %then %do;
				%let &temp.vlist = &temp.TESTCD &modvlist;
				%let &temp.vlistc = &temp.TESTCD, &modvlistc;
			%end;
			%else %do;
				%let &temp.vlist = &temp.TESTCD;
				%let &temp.vlistc = &temp.TESTCD;
			%end;
		%end;

		proc sql noprint;
			create table &temp.tmp as
			select *, min(stdy) as XMIN, max(stdy) as XMAX 
			from &temp.tmp
			group by USUBJID
			order by USUBJID, &&&temp.vlistc;
		quit;
		
		/*
		libname cout "C:\temp";
		data cout.f&temp;
			set &temp.tmp;
		run;
		*/			
		
		%let fdomainspn = &fdomainspn &temp;
		%let fndoms = &fndoms &temp.tmp;
	
	%end;	
	%DropMissing(data=&temp.tmp);
	%let ndom = %eval(&ndom + 1);
	
%end;

* combine into one data set;
data all(compress=yes);
	length CATEGORY $ &max_label_len;
	set 
		%let i = 1;
		%do %while(%length(%scan(&domains,&i,' ')));
			%let temp = %scan(%upcase(&domains),&i,' ');
			%if (%sysfunc(findw(&eventsdomains, &temp)) and &temp ne MH) or %sysfunc(findw(&interventionsdomains, &temp)) 
			   %then &temp.tmp;
			%let i = %eval(&i+1);
		%end;	
		&fndoms &fcdoms 
	;
	* add sv variables here to save time;
	%if %sysfunc(findw(&domains, SV)) > 0 %then %do;
  		%if %sysfunc(findw(&SVVARS, SVSTDTC)) > 0 %then %do;
			length SVSTDTC $32;
			SVSTDTC = "";
			label SVSTDTC = "&trStartDTofVisit";
		%end;
		%else %do;
			VISITDY = .;
			label VISITDY = "&trVisitDay";
		%end;
	%end;
	if missing(STDY) then STDY = 0;
	if missing(XMIN) then XMIN = 0;
	if missing(XMAX) then XMAX = 0;
run;

/*
data outlib.all;
	set all;
run;
*/

%if %length(&fdomainspn) %then %do;
	data a;
		set all;
		_a_ = 1;
		if missing(STNRLO) and missing(STNRHI) then delete;
		keep _a_;
	run;
	%NObsVars(a);
	%let hilo = &nobs;
%end;
%else %do;
	%let hilo = 0;
%end;

*** SV;
%let stdtc_sv = 0;
%let vday_sv = 0;
%let vnum_sv = 0;
%let vname_sv = 0;
%let visit_var = ;

%if %sysfunc(findw(&domains, SV)) > 0 %then %do;
  	%if %sysfunc(findw(&SVVARS, SVSTDTC)) > 0 %then %let stdtc_sv = 1;
   %if %sysfunc(findw(&SVVARS, VISITDY)) > 0 %then %let vday_sv = 1;
   %if %sysfunc(findw(&SVVARS, VISITNUM)) > 0 %then %let vnum_sv = 1;
   %if %sysfunc(findw(&SVVARS, VISIT)) > 0 %then %do;
   	%let vname_sv = 1;
   	%let visit_var = VISIT;
   %end;
   
   %put stdtc_sv = &stdtc_sv;
   %put vday_sv = &vday_sv;
   %put vnum_sv = &vnum_sv;
   %put vname_sv = &vname_sv;
   %put visit_var = &visit_var;

	/* %LOADDATA(domain = SV, mergesupp = 0);*/
	
	data svtmp;
		if 0 then set subjects;
		if _N_ = 1 then do;
			declare hash h(dataset:"subjects");
			h.defineKey("USUBJID");
			h.defineData(ALL:"YES");
			h.defineDone();
		end;
		set &in_sv(keep = USUBJID 
			%if &stdtc_sv %then SVSTDTC;
			%if &vday_sv %then VISITDY;
			%if &vnum_sv %then VISITNUM;
			%if &vname_sv %then VISIT;
			%if %sysfunc(findw(&SVVARS, SVSTDY)) > 0 %then SVSTDY;
			%if %sysfunc(findw(&SVVARS, SVENDY)) > 0 %then SVENDY;) end=eof_ds2;
		if h.find() = 0;
		%if &stdtc_sv %then %do; 
			%calculatestudydays(sv);
		%end;
		%else %if &vday_sv %then %do;
			* drop stdy;
			stdy = visitdy;
		%end;
		%else %do;
			stdy = .;
		%end;
		keep USUBJID stdy 
			%if &stdtc_sv %then SVSTDTC;
			%if &vday_sv %then VISITDY;
			%if &vnum_sv %then VISITNUM;
			%if &vname_sv %then VISIT;
		;			
	run;

	%if ^%symexist(ReviewsEnabled) %then %do;
		%global ReviewsEnabled;
		%let ReviewsEnabled=1;
	%end;
	%if %symexist(DeltaDir) & &ReviewsEnabled %then %do;	   
 	 *** Collapsing Review Flags Over Visits ***;
		proc sort nodupkey data=deltalib.sv(where = (SVRFlg ne "&trStable")) 
			out=delsv(keep = USUBJID %if &vnum_sv %then VISITNUM; &visit_var svRFlg rename = (svRFlg = RevFlag));
		   by USUBJID %if &vnum_sv %then VISITNUM; &visit_var ; 
		run;
	
		proc sort data=svtmp;
			by USUBJID %if &vnum_sv %then VISITNUM; &visit_var ;
		run;
	%end;
	
	/*
	libname co "C:\temp";
	data co.svtmp;
		set svtmp;
	run;
	*/

%end;

* Use SV if SVSTDTC present;
%if %sysfunc(findw(&domains, SV)) and &vnum_sv and &stdtc_sv %then %do;
	%let sv = 1;
	
	proc sort data=svtmp out=sv;
		by USUBJID visitnum SVSTDTC;
		where not missing(visitnum) and not missing(stdy);
	run;
	
	/*
	data allv;
		set svtmp;
		where not missing(visitnum) and not missing(stdy);
	run;
	proc sort data=allv;
		by USUBJID stdy SVSTDTC;
	run;
	* compute min visitnum for each day in case there are multiple values;
	proc means data=allv noprint;
		by USUBJID stdy SVSTDTC;
		var visitnum;
		output out=sv min=VISITNUM;
	run;
	proc sort data=sv;
		by USUBJID visitnum SVSTDTC;
	run;
	* now compute min stdy for each vistnum in case there are multiple values;
	proc means data=sv noprint;
		by USUBJID visitnum SVSTDTC;
		var stdy;
		output out=sv min=STDY;
	run;
	*/
	
	data sv;
		set sv;
		DOMAIN = "SV";
		length AXISLABEL1-AXISLABEL4 CATEGORY $ &max_label_len;
		AXISLABEL1 = "VISIT";				
		AXISLABEL2 = AXISLABEL1;
		AXISLABEL3 = AXISLABEL1;
		AXISLABEL4 = AXISLABEL1;
		PPLABEL1 = catx(' ',AXISLABEL2,VISITNUM, "[&trStudyDay:", cats(stdy,"]"));
		PPLABEL2 = PPLABEL1;
		PPLABEL3 = PPLABEL1;
		PPLABEL4 = PPLABEL1;
		CATEGORY = "";
	run;
	
	%if %symexist(DeltaDir) & &ReviewsEnabled %then %do;	
		%let _tid_ = %sysfunc(open(sv));
		%let vn = %sysfunc(varnum(&_tid_,usubjid));
		%let ulen1 = %sysfunc(varlength(&_tid_,&vn));
		%let _tid_ = %sysfunc(close(&_tid_));
		%let _tid_ = %sysfunc(open(delsv));
		%let vn = %sysfunc(varnum(&_tid_,usubjid));
		%let ulen2 = %sysfunc(varlength(&_tid_,&vn));
		%let _tid_ = %sysfunc(close(&_tid_));
	
		data sv;
			length RevFlag $ 16 usubjid $ %sysfunc(max(&ulen1,&ulen2));
		   merge sv(in = ina) delsv;
		   by USUBJID visitnum;
		   if ina;
		   if RevFlag = "" then RevFlag = "&trStable";
		run;		
	
		proc datasets nolist;
		   delete delsv;
		quit;
	%end;	
	
	/*
	data all;
		set all;
		length SVSTDTC $32;
		SVSTDTC = "";
		label SVSTDTC="&trStartDTofVisit";
	run;
	*/
	
	/*
	proc append base=all data=sv force;
	run;
	*/
	
	data all;
		set all sv;
	run;
	
	%let svvlist = VISITNUM SVSTDTC;
%end;
%else %if %sysfunc(findw(&domains, SV)) and &vnum_sv and &vday_sv %then %do;

	proc sort data=svtmp out=sv;
		by USUBJID visitnum VISITDY;
		where not missing(visitnum) and not missing(stdy);
	run;

	/*
   * Use observed visit days filling in with SV and VISITDY;
	* compute visit days across all domains;
	%let _tid_ = %sysfunc(open(all));
	%let vnum = %sysfunc(varnum(&_tid_,VISITNUM));
	%let _tid_ = %sysfunc(close(&_tid_));
	%let sv = 1;
	%if &vnum %then %do;
		data allv;
			set all(keep = USUBJID visitnum stdy);
			where not missing(visitnum) and not missing(stdy);
		run;
		proc sort data=allv;
			by USUBJID stdy;
		run;
		* compute min visitnum for each day in case there are multiple values;
		proc means data=allv noprint;
			by USUBJID stdy;
			var visitnum;
			output out=sv min=VISITNUM;
		run;
		proc sort data=sv;
			by USUBJID visitnum;
		run;
		* now compute min stdy for each visitnum in case there are multiple values;
		proc means data=sv noprint;
			by USUBJID visitnum;
			var stdy;
			output out=sv min=STDY;
		run;
	%end;

	data allv2;
		set svtmp;
		where not missing(visitnum) and not missing(stdy);
	run;

	proc sort data=allv2;
		by USUBJID stdy VISITDY;
	run;
	* compute min visitnum for each day in case there are multiple values;
	proc means data=allv2 noprint;
		by USUBJID stdy VISITDY;
		var visitnum;
		output out=sv2 min=VISITNUM;
	run;
	proc sort data=sv2;
		by USUBJID visitnum VISITDY;
	run;
	* now compute min stdy for each visitnum in case there are multiple values;
	proc means data=sv2 noprint;
		by USUBJID visitnum VISITDY;
		var stdy;
		output out=sv2 min=STDY;
	run;

	*** Combine keeping observed date if present;
	%if &vnum %then %do;
		data sv(drop = stdy2);
			merge sv2 sv(rename = (stdy = stdy2));
			by USUBJID visitnum;
			if stdy2 > . then stdy = stdy2;
		run;
	%end;
	%else %do;
		data sv;
			set sv2;
		run;
	%end;
	*/
	
	data sv;
		set sv;
		DOMAIN = "SV";
		length AXISLABEL1-AXISLABEL4 CATEGORY $ &max_label_len;
		AXISLABEL1 = "VISIT";				
		AXISLABEL2 = AXISLABEL1;
		AXISLABEL3 = AXISLABEL1;
		AXISLABEL4 = AXISLABEL1;
		PPLABEL1 = catx(' ',AXISLABEL2,VISITNUM, "[&trStudyDay:", cats(stdy,"]"));
		PPLABEL2 = PPLABEL1;
		PPLABEL3 = PPLABEL1;
		PPLABEL4 = PPLABEL1;
		CATEGORY = "";
	run;
	%if %symexist(DeltaDir) & &ReviewsEnabled %then %do;	
		%let _tid_ = %sysfunc(open(sv));
		%let vn = %sysfunc(varnum(&_tid_,usubjid));
		%let ulen1 = %sysfunc(varlength(&_tid_,&vn));
		%let _tid_ = %sysfunc(close(&_tid_));
		%let _tid_ = %sysfunc(open(delsv));
		%let vn = %sysfunc(varnum(&_tid_,usubjid));
		%let ulen2 = %sysfunc(varlength(&_tid_,&vn));
		%let _tid_ = %sysfunc(close(&_tid_));
		
		data sv;
			length RevFlag $ 16 usubjid $ %sysfunc(max(&ulen1,&ulen2));
		   merge sv(in = ina) delsv;
		   by USUBJID visitnum;
		   if ina;
		   if RevFlag = "" then RevFlag = "&trStable";
		run;
	
		proc datasets nolist;
		   delete delsv;
		quit;	
	%end;
	
	/*
	data all;
		set all;
		VISITDY = .;
		label VISITDY = "&trVisitDay";
	run;
	*/
	
	/*
	proc append base=all data=sv force;
	run;
	*/
	
	data all;
		set all sv;
	run;

	%let svvlist = VISITNUM VISITDY;
%end;
%else %do;
   * Use observed visit days if visit date or day not present above;
	* compute visit days across all domains;
	%put computing observed visits;
	%let _tid_ = %sysfunc(open(all));
	%let vnum = %sysfunc(varnum(&_tid_,VISITNUM));
	%let vnum2 = %sysfunc(varnum(&_tid_,REVFLAG));
	%let vnum3 = %sysfunc(varnum(&_tid_,VISIT));
	%let _tid_ = %sysfunc(close(&_tid_));
	%if &vnum %then %do;
		%let sv = 1;
		%if &vnum3 %then %do;
			%let visit_var = VISIT;
		%end;
		%else %do;
			%let visit_var = ;
		%end;
		%put visit_var = &visit_var;

		%if ^%symexist(ReviewsEnabled) %then %do;
			%global ReviewsEnabled;
			%let ReviewsEnabled = 1;
		%end;
		
		data allv;
			set all(keep = USUBJID visitnum %if &ReviewsEnabled and &vnum2 %then revflag; stdy &visit_var);
			where not missing(visitnum) and not missing(stdy);
			%if &ReviewsEnabled and ^&vnum2 %then %do;
				RevFlag = "";
			%end;
		run;

		%if %symexist(DeltaDir) & &ReviewsEnabled %then %do;	
			 proc sort nodupkey data = all(where = (revflag ne "&trStable")) out = delsv(keep = USUBJID visitnum &visit_var);
			 	by USUBJID visitnum; 
			 run;
		%end;
		
		/*
		libname oc "C:\temp";
		data oc.allv;
			set allv;
		run;
		*/
		
		proc sort nodupkey data=allv out=sv(keep = USUBJID visitnum stdy &visit_var);
			by USUBJID visitnum; 
		run;
		
		/*
		proc sort data=allv;
			by USUBJID stdy;
		run;

		* compute min vistnum for each day in case there are multiple values;
		proc means data=allv noprint;
			by USUBJID stdy;
			var visitnum;
			output out=sv min=VISITNUM;
		run;
		proc sort data=sv;
			by USUBJID visitnum;
		run;
		* now compute min stdy for each vistnum in case there are multiple values;
		proc means data=sv noprint;
			by USUBJID visitnum;
			var stdy;
			output out=sv min=STDY;
		run;
		*/
		
		data sv;
			set sv;
			DOMAIN = "SV";
			length AXISLABEL1-AXISLABEL4 CATEGORY $ &max_label_len;
			AXISLABEL1 = "VISIT";				
			AXISLABEL2 = AXISLABEL1;
			AXISLABEL3 = AXISLABEL1;
			AXISLABEL4 = AXISLABEL1;
			PPLABEL1 = catx(' ',AXISLABEL2,VISITNUM, "[&trStudyDay:", cats(stdy,"]"));
			PPLABEL2 = PPLABEL1;
			PPLABEL3 = PPLABEL1;
			PPLABEL4 = PPLABEL1;
			CATEGORY = "";
			VISITDY = STDY;
			label VISITDY="&trVisitDay";
		run;
		
		%if %symexist(DeltaDir) & &ReviewsEnabled %then %do;
			data sv;
				length RevFlag $ 16;
				merge sv(in = ina) delsv;
				by USUBJID visitnum;
				if ina;
				if RevFlag = "" then RevFlag = "&trStable";
			run;

			proc datasets nolist;
				delete delsv;
			quit;	
		%end;
		
		/*
		data all;
			set all;
			VISITDY = .;
		run;
		*/

		proc append base=all data=sv force;
		run;
		%let svvlist = VISITNUM VISITDY;
		%let domains = &domains SV;
		%let ndom = %eval(&ndom + 1);
		%let ndomain = %eval(&ndomain + 1);
	%end;
	%else %do;
		%let sv = 0;
		%let svvlist = ;
	%end;
%end;

%put domains = &domains;
/*
%if ^%symexist(modvlist) %then %let modvlist = ;
*/

* shorten axis labels;
data all(compress=yes);
	length AXISLABEL1-AXISLABEL4 CATEGORY $ &max_label_len STDY ENDY 8;
   set all;
	if length(AXISLABEL1) >= &max_label_len then AXISLABEL1 = kstrcat(trim(left(ksubstr(AXISLABEL1,1,&mllm3))),"...");
	if length(AXISLABEL2) >= &max_label_len then AXISLABEL2 = kstrcat(trim(left(ksubstr(AXISLABEL2,1,&mllm3))),"...");
	if length(AXISLABEL3) >= &max_label_len then AXISLABEL3 = kstrcat(trim(left(ksubstr(AXISLABEL3,1,&mllm3))),"...");
	if length(AXISLABEL4) >= &max_label_len then AXISLABEL4 = kstrcat(trim(left(ksubstr(AXISLABEL4,1,&mllm3))),"...");
	if length(CATEGORY) >= &max_label_len then CATEGORY = kstrcat(trim(left(ksubstr(CATEGORY,1,&mllm3))),"...");
   %ORDERDOMAINS;
   /*
   * add back in any missing variables that may have been dropped;
	%if %length(&modvlist) %then %do;
		%do jj=1 %to %sysfunc(countw(&modvlist));
			%let modifier = %scan(&modvlist,&jj,' ');
			if missing(&temp.&modifier) then &temp.modifier = "";
		%end;
	%end;
	*/
run;

data a1;
	set all;
	if (0);
run;
%let aid = %sysfunc(open(a1));

* assemble full list of sorting variables and then sort the data;
%let fullvarlist = ;
%let i = 1;
%do %while(%length(%scan(&domains,&i,' ')));
    %let temp = %scan(%upcase(&domains),&i,' ');
    %if &temp ne MH and (%sysfunc(findw(&eventsdomains, &temp)) or %sysfunc(findw(&interventionsdomains, &temp))
    		or %sysfunc(findw(&findingsdomains, &temp))) %then %do;
    		* verify each variable exists in the data set since ones with all missing values may have been deleted;
			%let j = 1;
			%if %symexist(&&temp.vlist) %then %do;
				%do %while(%length(%scan(&&&temp.vlist,&j,' ')));
					%let var = %scan(&&&temp.vlist,&j,' ');
					%let svn = %sysfunc(varnum(&aid,&var));
					%put temp = &temp  var = &var  svn = &svn;
					%if &svn %then %do;
						%let fullvarlist = &fullvarlist &var;
					%end;
					%let j = %eval(&j+1);
				%end;
			%end;
	%end;
	%let i = %eval(&i+1);
%end;
%let fullvarlist = %sysfunc(transtrn(%quote(&fullvarlist), %str(,) , %sysfunc(TRIMN(%str()))));

%let aid = %sysfunc(close(&aid));

%let i = 1;
%do %while(%length(%scan(&fullvarlist,&i,' ')));
	%let i = %eval(&i+1);
%end;
%let nfv = %eval(&i-1);
%put nfv = &nfv;
%put fullvarlist = &fullvarlist;

proc sort data=all;
	by USUBJID domn &fullvarlist stdy;
run;

/*%LOADDATA(domain = ADSL, mergesupp = 0);*/

%let trtvar = ARM;
%let temp = ADSL;
%if %sysfunc(findw(&&&temp.VARS, ACTARM)) > 0 %then %let trtvar = ACTARM;
%let DesignByVars = ;
%DMFlags;

%if %length(&SDTMMETA) & %length(&adslpresent) %then %do;
	proc sql noprint;
		create table ADSL as
		select * from &in_ADSL a1 left join &in_DM a2
		on a1.USUBJID=a2.USUBJID;
	quit;
	%let in_adsl = work.adsl;
%end;
data outlib.patient_profile_adsl;
	if 0 then set profilesubjects;
	if _N_ = 1 then do;
		declare hash h(dataset:"profilesubjects");
		h.defineKey("USUBJID");
		h.defineData(ALL:"YES");
		h.defineDone();
	end;
	set &in_adsl(keep = %if %sysfunc(findw(&&&temp.VARS, STUDYID)) > 0 %then %do; STUDYID %end; 
		USUBJID &TrtVar &DMKeepVars) end=eof_ds2;
	if h.find() = 0;
run;		

proc sort data=outlib.patient_profile_adsl;
	by USUBJID;
run;

%let maxsubjlen = 1;
%let maxaxlen1 = 1;
%let maxaxlen2 = 1;
%let maxaxlen3 = 1;
%let maxaxlen4 = 1;

* compute min and max counter for y axis positions;
data all(compress=yes drop=_min _max _max1) 
	maxcounter(keep=USUBJID counter _min _max _max1);
	set all end=_last;
	by USUBJID domn &fullvarlist;
	if missing(AXISLABEL2) then AXISLABEL2 = AXISLABEL1;
	if missing(AXISLABEL3) then AXISLABEL3 = AXISLABEL1;
	if missing(AXISLABEL4) then AXISLABEL4 = AXISLABEL2;
	retain _min _max _max1 _maxsubjlen 1 _maxaxlen1 1 _maxaxlen2 1 _maxaxlen3 1 _maxaxlen4 1;
	_maxsubjlen = max(lengthn(USUBJID),_maxsubjlen); 
	_maxaxlen1 = max(lengthn(AXISLABEL1),_maxaxlen1); 
	_maxaxlen2 = max(lengthn(AXISLABEL2),_maxaxlen2); 
	_maxaxlen3 = max(lengthn(AXISLABEL3),_maxaxlen3); 
	_maxaxlen4 = max(lengthn(AXISLABEL4),_maxaxlen4); 
	Exists="False";
	dmode = 1;
	if first.USUBJID then do;
		_min = stdy;
		_max = stdy;
		_max1 = stdy;
		counter = 1;
	end;
	%if %length(&findingsdomains) or %length(&interventionsdomains) or %length(&eventsdomains) %then %do;
		else do;
			if first.domn then do;
				counter+2.5;
				if dmode=3 then do;
					counter+1;
					if domain = "EX" then counter+2;
					%let i = 1;
					%do %while(%length(%scan(&findingsdomains,&i,' ')));
						%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
						if domain = "&temp" then counter+2;
						%let i = %eval(&i + 1);
					%end;
				end;
			end;
			else if first.%scan(&fullvarlist,1) %do v=2 %to &nfv; or first.%scan(&fullvarlist,&v) %end; then do;
				counter+1;
				if dmode = 3 then do;
					if domain = "EX" then counter+2;
					%let i = 1;
					%do %while(%length(%scan(&findingsdomains,&i,' ')));
						%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
						if domain = "&temp" then counter+2;
						%let i = %eval(&i + 1);
					%end;
				end;
			end;
			else Exists="True";
		end;
	%end;

	if not missing(stdy) then do;
		if (stdy lt _min) then _min = max(-22,stdy);
	   if (stdy gt _max) then _max = stdy;
   	%let i = 1;
		%do %while(%length(%scan(&findingsdomains,&i,' ')));
	    	%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
			if domain = "&temp" and (stdy gt _max1) then _max1 = stdy;
			%let i = %eval(&i + 1);
		%end;
	end;

	drop _maxsubjlen _maxaxlen1-_maxaxlen4;
	
	output all;

	if last.USUBJID then do;
		output maxcounter;
	end;
	if _last then do;
		call symput("maxsubjlen",trim(left(_maxsubjlen)));
		call symput("maxaxlen1",trim(left(_maxaxlen1)));
		call symput("maxaxlen2",trim(left(_maxaxlen2)));
		call symput("maxaxlen3",trim(left(_maxaxlen3)));
		call symput("maxaxlen4",trim(left(_maxaxlen4)));
	end;
run;

%put maxsubjlen = &maxsubjlen;
%put maxaxlen1 = &maxaxlen1;
%put maxaxlen2 = &maxaxlen2;
%put maxaxlen3 = &maxaxlen3;
%put maxaxlen4 = &maxaxlen4;

* perform the similar computations for mh;
%if %sysfunc(findw(&domains, MH)) and %sysfunc(findw(&eventsdomains, MH)) %then %do;
   proc sort data=mhtmp;
      by USUBJID;
   run;

	%let rschk = %sysfunc(open(mhtmp));
	%let agechk = %sysfunc(varnum(&rschk,AGE));
	%let brthchk = %sysfunc(varnum(&rschk,BRTHDTC));
	%let rschk = %sysfunc(close(&rschk));
		
	data mhtmp;
		merge mhtmp outlib.patient_profile_adsl;
		by USUBJID;
		%if &agechk %then %do;
			AGEMH = max(0,AGE - (TRIAL_YEAR + TRIAL_MONTH/12 - EVENT_YEAR - EVENT_MONTH/12));
		%end;
		%else %if &brthchk %then %do;
			AGEMH = max(0, round((mdy(EVENT_MONTH, EVENT_DAY, EVENT_YEAR) - mdy(substr(BRTHDTC, 6, 2), substr(BRTHDTC, 9, 2), substr(BRTHDTC, 1, 4)) + 1) / 365.25, 0.1));
		%end;
		%else %do;
			AGEMH = 0;
		%end;		
	run;
	
	* create new sorting variable so events appear in order of appearance on the time scale;
	%let var = MHDECOD1;
	%let newvar = MHSORT;
	%let stdy = AGEMH;
	%let dst = mhtmp;
	data nc;
		set &dst;
		keep USUBJID &var &stdy;
	run;

	* keep only obs from first study day;
	proc sort data=nc;
		by USUBJID &var &stdy;
	run;

	data nc;
		set nc;
		by USUBJID &var &stdy;
		if first.&var;
	run;

	* sort by first study day;
	proc sort data=nc;
		by USUBJID &stdy &var;
	run;

	data nc;
		set nc;
		&newvar = _n_;
		drop &stdy;
	run;

	proc sql;
		create table &dst(drop = USUBJID1 USUBJID2 &var.1 &var.2) as
			select coalesce(A.USUBJID1, B.USUBJID2) as USUBJID, coalesce(A.&var.1, B.&var.2) as &var, A.*, B.*
            from &dst(rename = (USUBJID=USUBJID1 &var=&var.1))as A left join nc(rename =(USUBJID=USUBJID2 &var=&var.2)) as B on 
			A.USUBJID1=B.USUBJID2 and A.&var.1=B.&var.2
		;
	quit;
	
	%let mhvlist = &newvar;

	proc sort data=&dst out=mh;
		by USUBJID &mhvlist;
	run;
		
	data mh(drop=_min _max) 
		maxcountermh(keep=USUBJID counter _min _max);
		set mh;
		by USUBJID &mhvlist;
		retain _min _max;
		Exists="False";
		if first.USUBJID then do;
			_min = 1e10;
			_max = -1e10;
			counter = 1;
		end;
		else do;
			if first.&mhvlist then do;
				counter+1;
			end;
			else Exists="True";
		end;

		if not missing(AGEMH) then do;
			if (AGEMH lt _min) then _min = AGEMH;
			if (AGEMH gt _max) then _max = AGEMH;
		end;

		output mh;

		if last.USUBJID then do;
			output maxcountermh;
		end;
	run;
%end;

* compute Y offset values for all domains, used in expanded mode;
data all(compress=yes);
	length AXISLABEL1 $ &maxaxlen1 AXISLABEL2 $ &maxaxlen2 AXISLABEL3 $ &maxaxlen3 AXISLABEL4 $ &maxaxlen4;
	merge all maxcounter(rename=(counter=maxcounter)) outlib.patient_profile_adsl; 
	by USUBJID;
	YY = maxcounter - counter;
	YY0 = YY;
	if not missing(STNRLO) then lowval = STNRLO;
	else lowval = YP10_1;
	if not missing(STNRHI) then highval = STNRHI;
	else highval = YP90_1;

	* compute Y01 for heat maps and line plots;
	if domain="EX" then do;
		if (highval > lowval) then do;
			Y01 = max(0,min(1,(EXDOSE-lowval)/(highval-lowval)));
			YOFFSET = 2*Y01;
		end;
		else do;
			Y01 = 0;
			YOFFSET = 0;
		end;
	end;
	%let i = 1;
	%do %while(%length(%scan(&findingsdomains,&i,' ')));
    	%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
		else if domain="&temp" and not missing(STRESN) then do;
			if (highval > lowval) then do;
				Y01 = max(0,min(1,(STRESN-lowval)/(highval-lowval)));
				YOFFSET = 2*Y01;
			end;
			else do;
				Y01 = 0;
				YOFFSET = 0;
			end;
		end;
		%let i = %eval(&i + 1);
	%end;
	else do;
		Y01 = 0;
		YOFFSET = 0;
	end;
	* attrib _all_ label='';
	%if &ReviewsEnabled %then %do;
		label RevFlag="&trrflg";
	%end;
	drop counter maxcounter lowval highval;
run;

%if %sysfunc(findw(&domains, MH)) and %sysfunc(findw(&eventsdomains, MH)) %then %do;
	data mh;
		merge mh maxcountermh(rename=(counter=maxcounter)) outlib.patient_profile_adsl; 
		by USUBJID;
		YY = maxcounter - counter;
		* attrib _all_ label='';
		%if &ReviewsEnabled %then %do;
			label RevFlag="&trrflg";
		%end;
		drop counter maxcounter;
	run;
%end;

* flag all labs that have a low or high value;
/*
%if %sysfunc(findw(&findingsdomains, LB)) > 0 %then %do;
	proc sort data=all;
		by USUBJID YY0 STDY;
	run;

	proc means data=all noprint;
		by USUBJID YY0;
		var HILO;
		output out=hilosum(keep=USUBJID YY0 HILOSUM) sum=HILOSUM;
	run;
	
	data all;
		merge all hilosum;
		by USUBJID YY0;
	run;
	
	data all;
		set all;
		by USUBJID YY0;
		retain normcounter 0;
		if first.USUBJID then normcounter = 0;
		if HILOSUM = 0 then do;
			YY0HL = 0;
			YYHL = 0;
			if first.YY0 then do;
			    if dmode=3 then do;
					%do %while(%length(%scan(&findingsdomains,&i,' ')));
				    	%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
						if domain = "&temp" then normcounter+2;
						%let i = %eval(&i + 1);
					%end;
				end;
				else normcounter+1;
			end;
		end;
		else do;
			YY0HL = YY0 - normcounter;
			YYHL = YY - normcounter;
		end;
		drop HILOSUM normcounter;
	run;
%end;
*/

* compute XOFFSET1, used in compressed mode with groups;
* compute XOFFSET2, used for time points arranged horizontally;
data all(compress=yes);
	set all;
	%if %length(&findingsdomains) %then %do;
		%let i = 1;
		%do %while(%length(%scan(&findingsdomains,&i,' ')));
	    	%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
			%if &i = 1 %then %do;
				if domain="&temp" and %length(&&ing_&temp) then do;
					XOFFSET1 = (&temp.SGRPN-1)*(_max1-_min)/100;
				end;
			%end;
			%else %do;
				else if domain="&temp" and %length(&&ing_&temp) then do;
					XOFFSET1 = (&temp.SGRPN-1)*(_max1-_min)/100;
				end;
			%end;
	      %let i = %eval(&i + 1);
		%end;
		else do;
			XOFFSET1 = 0;
		end;
		%let i = 1;
		%let first = 1;
		%do %while(%length(%scan(&findingsdomains,&i,' ')));
	    	%let temp = %scan(%upcase(&findingsdomains),&i,' ');	
			%if %sysfunc(findw(&&&temp.VARS, &temp.TPTNUM)) %then %do;
				%if &first %then %do;
					if domain="&temp" and not missing(&temp.TPTN_) then do;
						XOFFSET2 = (&temp.TPTN_-1)*(_max1-_min)/100;
					end;
					%let first = 0;
				%end;
				%else %do;
					else if domain="&temp" and not missing(&temp.TPTN_) then do;
						XOFFSET2 = (&temp.TPTN_-1)*(_max1-_min)/100;
					end;
				%end;
			%end;
	      %let i = %eval(&i + 1);
		%end;
		%if &first %then %do;
			XOFFSET2 = 0;
		%end;
		%else %do;
			else do;
				XOFFSET2 = 0;
			end;
		%end;
	%end;
	%else %do;
		XOFFSET1 = 0;
		XOFFSET2 = 0;
	%end;
run;

* non-category sort;
proc sort data=all;
	by domn &fullvarlist USUBJID stdy;
run;

data all(compress=yes);
	set all;
	* create sorting variables for showing single or multiple subjects;
	SINGLESORT2 = _n_;
	if domain in ("AE","CM","DS") then MULTISORT2 = 1;
	else MULTISORT2 = _n_;
run;

* final sort;
proc sort data=all;
	by domn CATEGORY &fullvarlist USUBJID stdy;
run;

data outlib.patient_profile_data(compress=yes);
	set all;
	* create sorting variables for showing single or multiple subjects;
	SINGLESORT = _n_;
	if domain in ("AE","CM","DS") then MULTISORT = 1;
	else MULTISORT = _n_;
run;
%NObsVars(outlib.patient_profile_data);
%let nobsppd = &nobs;

* save unique y values;
proc sort data=all out=yvals(keep=DOMAIN AXISLABEL1) nodupkey;
	by DOMN AXISLABEL1;
run;

data outlib.patient_profile_yvals;
	set yvals;
	where not missing(AXISLABEL1);
run;
%NObsVars(outlib.patient_profile_yvals);
%let nobsppyv = &nobs;
	
%if %sysfunc(findw(&domains, MH)) and %sysfunc(findw(&eventsdomains, MH)) %then %do;
	data outlib.patient_profile_data_mh(compress=yes);
		set mh;
	run;
	%NObsVars(outlib.patient_profile_data_mh);
	%if ^&Nobs %then %do;
		data outlib.patient_profile_data_mh;
			AXISLABEL1 = "";
		run;
	%end;

	proc sort data=mh out=yvals_mh(keep=AXISLABEL1) nodupkey;
		by AXISLABEL1;
	run;

	data outlib.patient_profile_yvals_mh;
		set yvals_mh;
		where not missing(AXISLABEL1);
	run;
	%NObsVars(outlib.patient_profile_yvals_mh);
	%if ^&Nobs %then %do;
		data outlib.patient_profile_yvals_mh;
			AXISLABEL1 = "";
		run;
	%end;
%end;
%else %do;
	data outlib.patient_profile_data_mh;
		AXISLABEL1 = "";
	run;
	data outlib.patient_profile_yvals_mh;
		AXISLABEL1 = "";
	run;
	/*
	data outlib.patient_profile_data_mh;
		if (0);
	run;
	data outlib.patient_profile_yvals_mh;
		if (0);
	run;
	*/
%end;


* create list of domain orders;
data _null_;
	length domain $ 8;
	%do ii=1 %to &ndomain;
		domain = "%scan(%upcase(&domains),&ii,' ')";
		%ORDERDOMAINS;
		call symputx("do&ii",trim(left(domn)));
	%end;
run;

%let domorder = ;
%do ii=1 %to &ndomain;
	%let domorder = &domorder &&do&ii;
%end;
%put domorder = &domorder;
%let SVVARS = ;
%if &AESeverity5 %then %let AESeverityNLev = 5;
%else %if &AESeverity4 %then %let AESeverityNLev = 4;
%else %let AESeverityNLev = 3;

* save variables into a jsl file that will be read back in below;
data _null_;
	file "&OutPath./profiler_vars.jsl" encoding="utf-8";
	%let i = 1;
	%do %while(%length(%scan(&domains,&i,' ')));
    	%let temp = %scan(%upcase(&domains),&i,' ');
    	%if (&temp=SV or %sysfunc(findw(&eventsdomains, &temp))) or %sysfunc(findw(&interventionsdomains, &temp)) 
        	or %sysfunc(findw(&findingsdomains, &temp)) %then %do;
    		put "&temp.Exists = 1;";
    		put "&temp.Display = 1;";
			%if %sysfunc(findw(&findingsdomains, &temp)) > 0 %then %do;
				put "&temp.grpExists = %length(&&ing_&temp);";
			%end;         
		%end;
		%let i = %eval(&i+1);
	%end;
	put "exgrpExists = 0;";
	put "domains = words(""&domains"");";
	put "eventsdomains = words(""&eventsdomains"");";
	put "eventsdomainslabel = words(""&eventsdomainslabel"",""~"");";
	put "interventionsdomains = words(""&interventionsdomains"");";
	put "interventionsdomainslabel = words(""&interventionsdomainslabel"",""~"");";
	put "findingsdomains = words(""&findingsdomains"");";
	put "findingsdomainslabel = words(""&findingsdomainslabel"",""~"");";
	put "fdomainsc = words(""&fdomainspc"");";
	put "fdomainsn = words(""&fdomainspn"");";
	%if ^%length(&domorder) %then %do;
		put "domo = {};";	
	%end;
	%else %if %sysfunc(countw(&domorder)) > 1 %then %do;
		put "domo = AsList([&domorder])[1];";
	%end;
	%else %do;
		put "domo = AsList([&domorder]);";		
	%end;
	put "maxnsg = &maxnsg;";
	put "maxsubjlen = &maxsubjlen;";
	put "maxaxlen1 = &maxaxlen1;";
	put "maxaxlen2 = &maxaxlen2;";
	put "maxaxlen3 = &maxaxlen3;";
	put "maxaxlen4 = &maxaxlen4;";
	put "severityVar = ""&AESeverityVar"";";
	put "severityType = ""&AESeverityType"";";
	put "severityNLev = &AESeverityNLev;";
	put "notesdir = jg:getPath(""/user/clinical/notes"", """", 0, 0, 1, studyName);";
	%if &Server=Local %then %do;
		put "adslrs = jg:getPath(""/user/clinical/notes"", ""adslrs.sas7bdat"", 0, 0, 1, studyName);";
		put "adslrsl = jg:getPath(""/user/clinical/notes"", ""adslrsl.sas7bdat"", 0, 0, 1, studyName);";
	%end;
	%else %do;
		put "notes = ""&ClientOutPath.&notesDataset..sas7bdat"";";
		put "adslrs = ""&ClientOutPath.adslrs.sas7bdat"";";
		put "adslrsl = ""&ClientOutPath.adslrsl.sas7bdat"";";	
	%end;
	put "reviewnum = &reviewnumber;";
	put "trtvar = ""&TrtVar"";";
	put "dmkeepvars = words(""&DMKeepVars"");";
	put "hilo = &hilo;";
	put "nobsppd = &nobsppd;";
	put "nobsppyv = &nobsppyv;";
run;
%sexit:;
%mend PROFILE;

%PROFILE;
		
**********************************************************;
**********************************************************;
**********************************************************;
