%macro CHECKIT(wherestmt = );
   data checkit;
      set editchk;
      where %unquote(&wherestmt);
   run;
   %NObsVars(checkit);
   %if &nobs %then %let oneerror = 1;

   proc datasets nolist;
      delete checkit;
   quit;
%mend CHECKIT;

%macro ADDUSERVARS(dsn = rbm_site, indsn = , user = );
   proc sql noprint;      
      select variable
         into :uservars separated by '~'
         from &user
         order by variable;
      select label
         into :userlabs separated by '~'
         from &user
         order by variable;
   quit;

   data &dsn;
      set &indsn;
      %let i = 1;
      %do %while(%length(%scan(&uservars,&i,~)));
         %let tempvar = %scan(&uservars,&i,~);   
         %let templab = %scan(&userlabs,&i,~);   
            &tempvar = .;
            label &tempvar = "&templab";
         %let i = %eval(&i+1);
      %end; 
   run;
%mend ADDUSERVARS;

%macro GENERATERBMSITE;
   %let COUNTRY = ; 
   proc sql noprint;
      %if %sysfunc(findw(%upcase(&DMVARS), COUNTRY)) %then %do;
         %let COUNTRY = COUNTRY; 
         create table rbm_site as 
            select distinct siteid, country
            from &in_DM
            order by siteid;
      %end;
      %else %do;
         create table rbm_site as 
            select distinct siteid
            from &in_DM
            order by siteid;     
      %end;
   quit;

   %let SiteLabel = SITEID;
   %CheckLabel(&in_DM,SiteLabel);
   %let _tid_ = %sysfunc(open(&in_DM));
   %let cind = %sysfunc(varnum(&_tid_,COUNTRY));
   %let _tid_ = %sysfunc(close(&_tid_));  
   %if &cind %then %do;
      %let CountryLabel = COUNTRY;
      %CheckLabel(&in_DM,CountryLabel);
   %end;

   *** keep first site if error with multiple countries per site ***;   
   data rbm_site;
      set rbm_site;
      by siteid;
      if first.siteid;
      label siteid = "&SiteLabel";
      %if &cind %then %do;
         label country = "&CountryLabel";
      %end;
   run;
   
   %let tranStateorProvince = %nrbquote(%LocaleGet(StateorProvince_Text));
   %let tranFiveDigitZipCode =   %nrbquote(%LocaleGet(FiveDigitZipCode_Text));
   %let tranTotalQueries = %nrbquote(%LocaleGet(TotalQueries_Text));
   %let tranOverdueQueries = %nrbquote(%LocaleGet(OverdueQueries_Text));
   %let tranQueryResponseTime = %nrbquote(%LocaleGet(QueryResponseTime_Text));
   %let tranComputedDeviations = %nrbquote(%LocaleGet(ComputedDeviations_Text));
   %let tranMissingPages = %nrbquote(%LocaleGet(MissingPages_Text));
   %let tranLatitude = %nrbquote(%LocaleGet(Latitude_Text));
   %let tranLongitude = %nrbquote(%LocaleGet(Longitude_Text));
   %let tranCity = %nrbquote(%LocaleGet(City_Text));
   %let tranCRFEntryResponseTime = %nrbquote(%LocaleGet(CRFEntryResponseTime_Text));
   %let tranMonitor = %nrbquote(%LocaleGet(Monitor_Text));
   %let tranSiteActiveDate = %nrbquote(%LocaleGet(SiteActiveDate_Text));
   %let tranSiteCategory = %nrbquote(%LocaleGet(SiteCategory_Text));
   %let trVendor = %nrbquote(%LocaleGet(Vendor_Text));
   
   %if ^%sysfunc(fileexist(&NotesDir.rbm_site.sas7bdat)) %then %do;
      %if %nrbquote(%upcase(&StudyName)) = NICARDIPINE or %nrbquote(%upcase(&StudyName)) = NICARDIPINE EARLY SNAPSHOT %then %do;
         %if &Server = Local %then %do;   
            libname RBMPath "&metapath.RBM&_slash_";      
         %end;
         %else %do;
            libname RBMPath "&metapath.RBM/";
         %end;
         %let _lang = %lowcase(%substr(%sysfunc(getoption(locale)),1,2));
         %if &_lang = zh %then %let lang_suffix = _zh;
         %else %if &_lang = ja %then %let lang_suffix = _ja;
         %else %let lang_suffix = _en;
         data rbm_site;
            retain siteid &country City StateProvince ZipCode Latitude Longitude SiteActive Monitor SiteCategory Vendor /* TotalQueries OverdueQueries QueryResp CRFEntryResp MissPage CompDeviate */;
            length Monitor SiteCategory Vendor $ 50 ZipCode $ 5; 
            merge rbm_site(in = ina) RBMPath.NicardipineRBM&lang_suffix(drop = TotalQueries OverdueQueries QueryResp CRFEntryResp MissPage CompDeviate);
            by siteid;
            if ina;
            Latitude = .;
            Longitude = .;
            label StateProvince = "&tranStateorProvince" ZipCode = "&tranFiveDigitZipCode"  latitude = "&tranLatitude" Vendor = "&trVendor" 
                  longitude = "&tranLongitude"  city = "&tranCity" monitor = "&tranMonitor" SiteCategory = "&tranSiteCategory" SiteActive = "&tranSiteActiveDate"                  
                  /* TotalQueries = "&tranTotalQueries" OverdueQueries = "&tranOverdueQueries" CRFEntryResp = "&tranCRFEntryResponseTime"
                  QueryResp = "&tranQueryResponseTime" CompDeviate = "&tranComputedDeviations" MissPage = "&tranMissingPages" */
                  ;
            format SiteActive &IntDateFormat..;
         run;
         *** Add user-defined variables if any ***;
         /*
         *** Need to fix if added back since more than one variable crashes ***;
         %if %sysfunc(fileexist(&metapath.RBM&_slash_.user_vars.sas7bdat)) %then %do;
            %ADDUSERVARS(dsn = rbm_site, indsn = rbm_site, user = RBM.user_vars);
         %end;
         */
         data NotesLib.rbm_site;
            set rbm_site;
         run;

         proc datasets nolist;
            delete rbm_site;
         quit;
      %end;
      %else %do;           
         data rbm_site;
            retain siteid &country City StateProvince ZipCode Latitude Longitude SiteActive Monitor SiteCategory Vendor /* TotalQueries OverdueQueries QueryResp CRFEntryResp MissPage CompDeviate */;
            length City StateProvince Monitor SiteCategory Vendor $ 50 ZipCode $ 5; 
            set rbm_site;
            Latitude = .;
            Longitude = .;
            SiteActive = .;
            /* 
            TotalQueries = .;
            OverdueQueries = .;
            CompDeviate = .;
            QueryResp = .;
            CRFEntryResp = .;
            MissPage = .;
            */            
            label StateProvince = "&tranStateorProvince" ZipCode = "&tranFiveDigitZipCode"  latitude = "&tranLatitude" Vendor = "&trVendor" 
                  longitude = "&tranLongitude"  city = "&tranCity" monitor = "&tranMonitor" SiteCategory = "&tranSiteCategory" SiteActive = "&tranSiteActiveDate"                  
                  /* TotalQueries = "&tranTotalQueries" OverdueQueries = "&tranOverdueQueries" CRFEntryResp = "&tranCRFEntryResponseTime"
                  QueryResp = "&tranQueryResponseTime" CompDeviate = "&tranComputedDeviations" MissPage = "&tranMissingPages" */
                  ;
            format SiteActive &IntDateFormat..;
         run;
         *** Add user-defined variables if any ***;
         /*
         *** Need to fix if added back since more than one variable crashes ***;
         %if %sysfunc(fileexist(&metapath.RBM&_slash_.user_vars.sas7bdat)) %then %do;
            %ADDUSERVARS(dsn = rbm_site, indsn = rbm_site, user = RBM.user_vars);
         %end;       
         */
         data NotesLib.rbm_site;
            set rbm_site;
         run;

         proc datasets nolist;
            delete rbm_site;
         quit;       
      %end;
   %end;
   %else %do;
      %if %nrbquote(%upcase(&StudyName)) = NICARDIPINE or %nrbquote(%upcase(&StudyName)) = NICARDIPINE EARLY SNAPSHOT %then %do;
         proc sql noprint;
            create table rbm_merge as
            select *
            from rbm_site(drop = country) natural left join NotesLib.rbm_site
            order by siteid;
         quit;    
      %end;
      %else %do;
         proc sql noprint;
            create table rbm_merge as
            select *
            from rbm_site natural left join NotesLib.rbm_site
            order by siteid;
         quit;          
      %end;

      *** Add user-defined variables if any ***;
      /*
      *** Need to fix if added back since more than one variable crashes ***;
      %if %sysfunc(fileexist(&metapath.RBM&_slash_.user_vars.sas7bdat)) %then %do;
         proc contents data = rbm_merge noprint out = merge_vars(keep = name rename = (name = variable));
         run;
         
         proc sort data = merge_vars;
            by variable;
         run;
         
         proc sort data = RBM.user_vars out = user_vars;
            by variable;
         run;
         
         data user_vars;
            merge user_vars(in = ina) merge_vars(in = inb);
            by variable;
            if ina and not inb;
         run;
         %NObsVars(user_vars);
         
         %if &nobs %then %do;
            %ADDUSERVARS(dsn = rbm_merge, indsn = rbm_merge, user = user_vars);        
         %end;

         proc datasets nolist;
            delete user_vars merge_vars;
         quit;
      %end;
      */
      
      data NotesLib.rbm_site;
         set rbm_merge;
         label siteid = "&SiteLabel";
         %if &cind %then %do;
            label country = "&CountryLabel";
         %end;
      run;
   %end; 
   %rbmexit:;
%mend GENERATERBMSITE;

%macro GEOCODE(indsn = );
  %let nonusaobs=0;
  %let usaobs=0;
  *** Define list of non-null variables ***;
   %NONNULLVARS(dataset = &indsn, varlistname = RBMSITEVARS);

   %if ((%index(&RBMSITEVARS,CITY) and %index(&RBMSITEVARS,STATEPROVINCE)) or %index(&RBMSITEVARS,ZIPCODE) or (%index(&RBMSITEVARS,LATITUDE) and %index(&RBMSITEVARS,LONGITUDE))) 
      and %index(&RBMSITEVARS,COUNTRY) %then %do; 

      %if %sysfunc(findw(%upcase(&RBMSITEVARS), CITY)) %then %let %upcase(CITY) = %upcase(CITY);
         %else %let %upcase(CITY) = ;
      %if %sysfunc(findw(%upcase(&RBMSITEVARS), STATEPROVINCE)) %then %let %upcase(STATEPROVINCE) = %upcase(STATEPROVINCE);
         %else %let %upcase(STATEPROVINCE) = ;
      %if %sysfunc(findw(%upcase(&RBMSITEVARS), ZIPCODE)) %then %let %upcase(ZIPCODE) = %upcase(ZIPCODE);
         %else %let %upcase(ZIPCODE) = ;
      %if %sysfunc(findw(%upcase(&RBMSITEVARS), LATITUDE)) %then %let %upcase(LATITUDE) = %upcase(LATITUDE);
         %else %let %upcase(LATITUDE) = ;
      %if %sysfunc(findw(%upcase(&RBMSITEVARS), LONGITUDE)) %then %let %upcase(LONGITUDE) = %upcase(LONGITUDE);
         %else %let %upcase(LONGITUDE) = ;      

      data usa nonusa;
         set &indsn(keep = siteid country &CITY &STATEPROVINCE &ZIPCODE);
         %if %length(&LATITUDE) and %length(&LONGITUDE) %then %do;
            if upcase(country) in ("US","USA","UNITED STATES") and not (latitude > . and longitude > .) then output usa;
            else if not (latitude > . and longitude > .) then output nonusa;
         %end;
         %else %do;
            if upcase(country) in ("US","USA","UNITED STATES") then output usa;
            else output nonusa;     
         %end;
      run;

      %if %length(&ZIPCODE) %then %do;
         data usa;
            set usa;
            zip = zipcode + 0;
         run;     

         proc geocode data = usa nocity method = zip addresszipvar = zip out = cityusa attributevar = (Y, X);
         run;
         
         data cityusa;
            set cityusa(rename = (X = long Y = lat));
         run;
         
         data cityusa;
            set cityusa(keep = siteid long lat where = (long > . and lat > .));
         run;
         %Nobsvars(cityusa);
         %let usaobs = &nobs;
      %end;
      %else %if %length(&CITY) and %length(&STATEPROVINCE) %then %do;
         proc geocode data = usa method = city addresscityvar = &CITY addressstatevar = &STATEPROVINCE lookupcity = mapsgfk.uscity_all out = cityusa attributevar = (lat, long);
         run;

         data cityusa;
            set cityusa(keep = siteid long lat where = (long > . and lat > .));
         run;
         %Nobsvars(cityusa);
         %let usaobs = &nobs;       
      %end;
      %if %length(&CITY) and %length(&STATEPROVINCE) %then %do;
         proc geocode data = nonusa method = city addresscityvar = &CITY addresscountryvar = country addressstatevar = &STATEPROVINCE out = citynonusa;
         run;
         
         data citynonusa;
            set citynonusa(keep = siteid long lat where = (long > . and lat > .));
         run;        
         %Nobsvars(citynonusa);
         %let nonusaobs = &nobs;
      %end;

      %if &usaobs or &nonusaobs %then %do;
         data &indsn(drop = lat long);
            merge &indsn 
                  %if &usaobs %then %do; cityusa %end;
                  %if &nonusaobs %then %do; citynonusa %end;;
            by siteid;
            if not (latitude > . and longitude > .) then do;
               if lat > . then latitude = lat;
               if long > . then longitude = long;
            end;
         run;
      %end;
   %end;   
%mend GEOCODE;