%macro SMQData;
   %if &sysscp = WIN %then %do;
      %let _slash_ = \;
   %end;
   %else %do;
      %let _slash_ = /;
   %end;
   
   %put MinPath = &MinPath;
   
   %if %qsubstr(&MinPath, %length(&MinPath), 1) ne %quote(&_slash_) %then %let MinPath = &MinPath&_slash_;
                                 
   %let rc = %sysfunc(filename(fileref,&MinPath));
   %if ^%sysfunc(fexist(&fileref)) %then %do;
         %put [MS_S_0232];
      %put ERROR: The MedDRA Path &MinPath does not exist.;
      %if &Server ne Local %then %do;
         %put ERROR: The MedDRA files and path should exist on the server when operating in server mode.;
      %end;
      %let exiterror=1;
      %goto smqexit;       
   %end;
                                 
   %if %sysfunc(fileexist(&MinPath.smq_content.asc)) %then %do; 
      filename content "&MinPath.smq_content.asc";
   %end;
   %else %do;
      %put [MS_S_0233];
      %put ERROR: The required smq_content.asc file is not available in &MinPath.;
      %let exiterror=1;
      %goto smqexit; 
   %end;

   %if %sysfunc(fileexist(&MinPath.smq_list.asc)) %then %do; 
      filename listfile "&MinPath.smq_list.asc";
   %end;
   %else %do;
      %put [MS_S_0234];
      %put ERROR: The required smq_list.asc file is not available in &MinPath.;
      %let exiterror=1;
      %goto smqexit; 
   %end;
   
   %if %sysfunc(fileexist(&MinPath.pt.asc)) %then %do; 
      filename prefterm "&MinPath.pt.asc";
   %end;
   %else %do;
      %put [MS_S_0235];
      %put ERROR: The required pt.asc file is not available in &MinPath.;
      %let exiterror=1;
      %goto smqexit; 
   %end; 

   %if %sysfunc(fileexist(&MinPath.llt.asc)) %then %do; 
      filename lltterm "&MinPath.llt.asc";
   %end;
   %else %do;
      %put [MS_S_0236];
      %put ERROR: The required llt.asc file is not available in &MinPath.;
      %let exiterror=1;
      %goto smqexit; 
   %end;
   
   %let pterm = ;
   %let lterm = ;
   %if %sysfunc(findw(%upcase(&AEVARS), AEDECOD)) %then %let pterm = AEDECOD;
   %if %sysfunc(findw(%upcase(&AEVARS), AELLT)) %then %let lterm = AELLT;  
   %if &exiterror %then %goto smqexit;
   
   %let aelltchk = 0;
   %let _tid_ = %sysfunc(open(&in_ae));
      %let aelltchk = %sysfunc(varnum(&_tid_,AELLT));
   %let _tid_ = %sysfunc(close(&_tid_));  

   %if %sysfunc(findw(%upcase(&AEVARS), AELLT)) and ^&aelltchk %then %do;
      %put [MS_S_0237];
      %put ERROR: AELLT detected in supplemental domain. Click option to merge supplemental domain data on filter tab of the dialog.;
      %let exiterror = 1;
      %goto smqexit;       
   %end;

   %if ^%sysfunc(findw(&domains, AE)) %then %do;
      %put [MS_S_0011];
      %put ERROR: The AE or ADAE data set does not exist for this study.;
      %let exiterror=1;
      %goto smqexit;
   %end;

   %AESerOnlyToAEWhereStmt;
   %WHEREDELTA(deltastate = AEDeltaDrop, wherestate = AEWhereStmt, deltaflag = AERFlg);
         
   %include "&MacroPath./TEAEOTFU.sas";
   %if &exiterror %then %goto exit; 
   
   %if %length(&AEWhereStmt) %then %do;
      data ae_ws;
         set &in_ae;
         where %unquote(&AEWhereStmt);
      run;
      %if &syserr > 4 %then %do;
         %put [MS_S_0019];
         %put ERROR: The specified filter &AEWhereStmt is invalid. Change the filter to be valid for the domain you are considering.;
         %let exiterror = 1;
         %goto exit;
      %end; 
      %NObsVars(ae_ws);
      %if ^&nobs %then %do;
         %put [MS_S_0020];
         %put ERROR: No observations satisfy &AEWhereStmt..;
         %put Please relax the filter.;
         %let exiterror = 1;
         %goto smqexit;
      %end;
      %let in_ae = ae_ws;
   %end; 

   *** This macro identifies all child, grandchild etc nodes for hierarchical SMQs ***;
   %macro MAKECHILDREN;
      data temp0;
         set content_smq(rename = (smq_code = parent term_code = smq_code));
      run;
      
      data hierchart;
         set temp0(rename = (smq_code = child));
         keep parent child;         
      run;
      
      %Nobsvars(temp0);

      proc sql noprint;
         create table child0 as
            select distinct parent from temp0
            order by parent;
         create table child1 as
            select distinct smq_code from temp0
            order by smq_code;            
      quit;
      
      data child0;
         set child0(rename = (parent = smq_code)) child1;
         term_code = smq_code;
      run;
      
      proc sort data = child0;
         by smq_code;
      run;
      
      data child0;
         set child0;
         by smq_code;
         if first.smq_code;
      run;

      proc sort data = temp0;
         by smq_code;
      run;

      %let i = 1;
      %do %while(&nobs > 0);
         data temp&i;
           merge temp%eval(&i-1)(in = ina) content_smq(in = inb);
           by smq_code;
           if ina and inb;
           drop smq_code;
           rename term_code = smq_code;
         run;

         proc sort data = temp&i;
            by smq_code;
         run;

          %Nobsvars(temp&i);
         %let i = %eval(&i+1);
      %end;
      %let i = %eval(&i-1);

      data smq_children(rename = (smq_code = term_code parent = smq_code));
         set temp0 %do j = 1 %to &i; temp&j %end;;
      run;
      
      data smq_children;
         set child0 smq_children;
      run;

      proc sort data = smq_children;
         by smq_code term_code;
      run;
      
      data smq_children(rename = (smq_name = group_name));
         merge smq_children(in = ina) listfile(keep = smq_code smq_name smq_level);
         by smq_code;
         if ina;
      run;

      data hierchart2;
         merge listfile(keep = smq_code smq_name smq_level where = (smq_level = 1) in = ina) smq_children(in = inb /* where = (smq_code ne term_code)*/ drop = group_name);
         by smq_code;
         if ina and inb;
      run;

      proc sort data = hierchart2;
         by term_code;
      run;

      proc sort data = hierchart;
         by child;
      run;

      data hierchart;
         merge hierchart hierchart2(rename = (term_code = child));
         by child;
         drop smq_level;
         rename smq_code = group_code;
      run;
      
      data hierchart;
         merge hierchart(in = ina) listfile(keep = smq_code smq_name rename = (smq_code = child smq_name = child_name));
         by child;
         if ina;
         if parent = . then parent = child;
      run;

      proc datasets nolist;
         delete temp0 %do j = 1 %to &i; temp&j %end; content_smq child0 child1 hierchart2;
      quit;
   %mend MAKECHILDREN;

   data pt(drop = pt_soc_code);
      length pt_name $ 100;
      infile prefterm delimiter = "$" missover dsd;
      input pt_code pt_name $ pt_soc_code;
      pt_name = upcase(pt_name);
   run;

   proc sort data = pt;
      by pt_name;
   run;

   data llt(drop = pt_code llt_currency);
      length llt_name $ 100 llt_currency $ 1;
      infile lltterm delimiter = "$" missover dsd;
      input llt_code llt_name $ pt_code llt_currency $;
      *** delete non-current LLTs ***;
      if llt_currency = "N" then delete;
      llt_name = upcase(llt_name);
   run;

   proc sort data = llt;
      by llt_name;
   run;

   data listfile;
      length smq_name $ 100 smq_description smq_source smq_note smq_algorithm $ 2000 MedDRA_version $ 5 status $ 1;
      infile listfile delimiter = "$" missover dsd;
      input smq_code smq_name $ smq_level smq_description $ smq_source $ smq_note $ MedDRA_version $ status $ smq_algorithm $;
      *** remove inactive SMQs ***;
      if status = "I" then delete;
      *** Create appropriate variable name ***;
      smq_algorithm = transtrn(smq_algorithm, "Sum(Category Term Weight)", "WeightSum"); 
      smq_name = transtrn(smq_name, " (SMQ)", ""); 
   run;
   
   data nohier(keep = smq_code);
      set listfile(where = (smq_level = 1));
   run;

   proc sort data = listfile;
      by smq_code;
   run;

   data content(drop = term_addition_version term_last_ modified_version);
      length term_category term_status $ 1 term_addition_version term_last_ modified_version $ 5;
      infile content delimiter = "$" missover dsd;
      input smq_code term_code term_level term_scope term_category $ term_weight term_status term_addition_version term_last_ modified_version;
      *** remove inactive terms ***;
      if term_status = "I" then delete;
   run;

   *** term_level = 0 are child SMQs ***;
   data content_smq(drop = term_level);      
      set content(where = (term_level = 0) drop = term_category term_status term_scope term_weight);
   run;
   
   proc sort data = content_smq(keep = smq_code) out = hier;
      by smq_code;
   run;
   
   data hier;
      set hier;
      by smq_code;
      if first.smq_code;
   run;
   
   data nohier;
      merge nohier(in = ina) hier(in = inb);
      by smq_code;
      if ina and not inb;
   run;
      
   %MAKECHILDREN;
   
   *** drop SMQ parents-children from main set ***;
   data content;
      set content(where = (term_level ne 0));
   run;

   proc sort data = content;
      by term_code;
   run;

   proc sort nodupkey data = &in_ae out = ae(keep = usubjid &lterm &pterm);
      by usubjid &lterm &pterm;
   run;

   %let _tid_ = %sysfunc(open(ae));
   %if %length(&pterm) %then %do;
      %let pn = %sysfunc(varnum(&_tid_,AEDECOD));
      %let plength = %sysfunc(varlength(&_tid_,&pn));
   %end;
   %if %length(&lterm) %then %do;
      %let ln = %sysfunc(varnum(&_tid_,AELLT));
      %let llength = %sysfunc(varlength(&_tid_,&ln));
   %end;
   %let _tid_ = %sysfunc(close(&_tid_));
   %if %length(&lterm) and %length(&pterm) %then %let termlength = %sysfunc(max(&plength,&llength));
   %else %if %length(&lterm) %then %let termlength = &llength;
   %else %if %length(&pterm) %then %let termlength = &plength;

   data ae;
      length _term _terml $ 100;
      set ae;
      %if %length(&pterm) %then %do;
         _terml = &pterm; term_level = 4;
         _term = upcase(_terml);
        output;
      %end; 
      %if %length(&lterm) %then %do;
         _terml = &lterm; term_level = 5;
         _term = upcase(_terml);
        output;
      %end;
      keep usubjid _term _terml term_level;
   run;

   proc sort nodupkey data = ae;
      by term_level _term usubjid;
   run;

   data pae(drop = _term rename = (_terml = _term pt_code = term_code) where = (term_code > .));
      merge ae(where = (term_level = 4)  in = ina) pt(rename = (pt_name = _term));
      by _term;
      if ina;
   run;

   data lae(drop = _term rename = (_terml = _term llt_code = term_code) where = (term_code > .));
      merge ae(where = (term_level = 5) in = ina) llt(rename = (llt_name = _term));
      by _term;
      if ina;
   run;

   data ae;
      set pae lae;
   run;

   proc datasets nolist;
      delete pt llt pae lae hier;
   quit;   

   %global numalgos;
   *** identify SMQs with algorithms ***;
   %let numalgos = 0;
   proc sql noprint;
      select smq_code
         into :algos separated by ' '
         from listfile
           where smq_algorithm ne "N"
           order by smq_code;
       %let numalgos = &sqlobs;
      select smq_algorithm
         into :algoscode separated by '~'
         from listfile
           where smq_algorithm ne "N"
           order by smq_code;
   quit;

   *** merge in SMQ info to terms ***;
   proc sql noprint; 
      create table aecontent as
         select * from ae, content(drop = term_status)
         where ae.term_level = content.term_level and ae.term_code = content.term_code
         order by usubjid, smq_code, term_scope;
      create table termdecode as
         select distinct _term, term_level, term_code from aecontent
         order by term_level, term_code;
   quit;  
   
   data outlib.aec;
      set aecontent;
   run;

   *** define narrow and broad searches and components for algorithms ***;
   data aecontent;
      length narrowlist broadlist $ 1000 narrowclist broadclist $ 100;
      set aecontent(drop = _term);
      by usubjid smq_code;
      retain narrowlist broadlist narrowflag broadflag narrowcnt broadcnt narrowclist broadclist A B C D E F G H I WeightSum;
      if first.smq_code then do;
         narrowlist = "";
         broadlist = "";
         narrowclist = "";
         broadclist = "";
         narrowcnt = 0;
         broadcnt = 0;
         broadflag = 0;
         narrowflag = 0;
         WeightSum = 0;
         A = 0; B = 0; C = 0; D = 0; E = 0; F = 0; G = 0; H = 0; I = 0; J = 0;
         if term_scope = 2 then do;
            narrowcnt = 1;
            narrowflag = 1;
            narrowclist = trim(left(put(term_level,8.0))); 
            narrowlist = put(term_code,8.0);
         end;
         broadflag = 1;
         broadcnt = 1;
         broadlist = put(term_code,8.0);
         broadclist = trim(left(put(term_level,8.0)));
         if term_category = "A" then A = 1;
         else if term_category = "B" then do;
            B = 1;
          WeightSum = term_weight;
         end;
         else if term_category = "C" then do;
            C = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "D" then do;
            D = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "E" then do;
            E = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "F" then do;
            F = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "G" then do;
            G = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "H" then do;
            H = 1;
            WeightSum = term_weight;
         end;
         else if term_category = "I" then do;
            I = 1;
             WeightSum = term_weight;
         end;     
      end;
      else do;
         if term_scope = 2 then do;
            narrowclist = catx(" ", narrowclist,left(put(term_level,8.0)));
            narrowlist = catx(" ", narrowlist,put(term_code,8.0));
            narrowcnt = narrowcnt + 1;
         end;
         broadlist = catx(" ", broadlist,put(term_code,8.0));
         broadclist = catx(" ", broadclist,left(put(term_level,8.0)));
         broadcnt = broadcnt + 1;

         if term_category = "A" and A = 0 then A = 1;
         else if term_category = "B" and B = 0 then do;
            B = 1;
           WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "C" and C = 0 then do;
            C = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "D" and D = 0 then do;
            D = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "E" and E = 0 then do;
            E = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "F" and F = 0 then do;
            F = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "G" and G = 0 then do;
            G = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "H" and H = 0 then do;
            H = 1;
            WeightSum = term_weight + WeightSum;
         end;
         else if term_category = "I" and I = 0 then do;
            I = 1;
            WeightSum = term_weight + WeightSum;
         end;     
      end;
      if last.smq_code;
      keep usubjid smq_code narrowlist broadlist narrowflag broadflag narrowcnt broadcnt narrowclist broadclist A B C D E F G H I WeightSum;
   run;
   
   *** merge in SMQ names ***;
   data aecontent(drop = rc);
      if _n_ = 1 then do;
            declare hash h_event();
         length smq_name $ 100 smq_code 8;
         rc = h_event.DefineKey( "smq_code" );
         rc = h_event.DefineData( "smq_name");
         rc = h_event.DefineDone();
      end;
      do until ( eof_list );
         set listfile(keep = smq_code smq_name) end = eof_list;
         rc = h_event.add();
      end;
      do until ( eof_e );
         set aecontent end = eof_e;
         smq_name = "";
         rc = h_event.find();
         output;
      end;
   run;

   *** perform algorithms ***;
   data aecontent(drop = A B C D E F G H I);
      set aecontent;
      algorithm = 0;
      %do iter = 1 %to &numalgos;
          if smq_code = "%scan(&algos,&iter,' ')" then do;
            if %length(%scan(&algoscode,&iter,"~")) then algorithm = 1;
         end;
      %end;
   run;

   *** create event records ***;
   data aecontent;
      length eventlist $ 1000 eventclist $ 100 Flag $ 9;
      set aecontent;
      if algorithm = 1 then do;
         eventlist = broadlist; eventclist = broadclist; eventcount = broadcnt; Flag = "Algorithm"; output;
      end;
      if broadflag = 1 then do;
         eventlist = broadlist; eventclist = broadclist; eventcount = broadcnt; Flag = "Broad"; output;
      end; 
      if narrowflag = 1 then do;
         eventlist = narrowlist; eventclist = narrowclist; eventcount = narrowcnt; Flag = "Narrow"; output;
      end; 
      drop narrowlist narrowclist narrowcnt broadlist broadclist broadcnt narrowflag broadflag algorithm weightsum;
   run;
   %NObsVars(aecontent);
   
   %if ^&nobs %then %do;
      %put [MS_S_0238];
      %put ERROR: Based on selected subjects and events, no subjects meet crtieria for SMQs.;
      %put Please relax the filters.;
      %let exiterror = 1;
      %goto smqexit;
   %end;    

   data aecontent;
      retain USUBJID smq_name smq_code eventlist eventclist eventcount flag;
      set aecontent;
   run;

   *** create data set of all children for cumulative classes for hierarchical SMQs ***;
   proc sql noprint; 
      create table children as
         select * from smq_children(drop = group_name smq_level), aecontent(where = (Flag ne "Algorithm") rename = (smq_code = term_code) drop = smq_name)
         where smq_children.term_code = aecontent.term_code
         order by usubjid, smq_code, flag;
   quit;

   data children;
      retain USUBJID smq_name smq_code eventlist eventclist eventcount flag;
      set children(drop = term_code);
      do i = 1 to eventcount;
         term_code = scan(eventlist, i, " ") + 0;
         term_level =  scan(eventclist, i, " ") + 0;
        output;
      end;
      drop i eventlist eventclist eventcount;
   run;

   proc sort nodupkey data = children;
      by USUBJID smq_code flag term_level term_code;
   run;

   data children;
      length eventlist $ 1000 eventclist $ 100;
      set children;
      by usubjid smq_code flag;
      retain eventlist eventclist eventcount;
      if first.flag then do;
         eventlist = put(term_code,8.0);
         eventclist = left(put(term_level,8.0));
         eventcount = 1;
      end;
      else do;
         eventlist = catx(" ", eventlist,put(term_code,8.0));
         eventclist = catx(" ", eventclist,left(put(term_level,8.0)));
         eventcount = eventcount + 1;
      end;
      if last.flag;
   run;

   data children(drop = rc term_code term_level);
      if _n_ = 1 then do;
         declare hash h_event();
         length smq_name $ 100 smq_code 8;
         rc = h_event.DefineKey( "smq_code" );
         rc = h_event.DefineData( "smq_name");
         rc = h_event.DefineDone();
      end;
      do until ( eof_list );
         set listfile(keep = smq_code smq_name) end = eof_list;
         rc = h_event.add();
      end;
      do until ( eof_e );
         set children end = eof_e;
         smq_name = "";
         rc = h_event.find();
         output;
      end;
   run;

   proc sort data = aecontent;
      by smq_code;
   run;

   *** Creating cumulative records for non-hierarchical SMQs at Level 1 ***;
   data nohier;
      length Type $ 12; 
      merge nohier(in = ina) aecontent(in = inb where = (Flag ne "Algorithm"));
      by smq_code;
      if ina and inb;
      type = "Cumulative";
   run;
   
   data aecontent;
      retain USUBJID smq_name smq_code eventlist eventclist eventcount Flag Type;
      length Type $ 12; 
      set aecontent(in = ina) children(in = inb);
      if ina then type = "Individual";
      else if inb then type = "Cumulative";
   run;
   
   data aecontent;
      set aecontent nohier;
   run;
   
   proc datasets nolist;
      delete nohier;
   quit;
      
   data aecontent(drop = rc);
      if _n_ = 1 then do;
         declare hash h_event();
         length group_name $ 100 smq_code 8;
         rc = h_event.DefineKey( "smq_code" );
         rc = h_event.DefineData( "group_name");
         rc = h_event.DefineDone();
      end;
      do until ( eof_list );
         set smq_children(drop = smq_code smq_level rename = (term_code = smq_code)) end = eof_list;
         rc = h_event.add();
      end;
      do until ( eof_e );
         set aecontent end = eof_e;
         group_name = "";
         rc = h_event.find();
         output;
      end;
   run;
   
   data aecontent;
      set aecontent;
      if group_name = "" then group_name = smq_name;
   run;

   proc sort data = aecontent;
      by usubjid smq_name flag type;
   run;

   %let StudyVar = ;
   %let SiteVar = ;
   %let SexVar = ;
   %let RaceVar = ;
   %let AgeVar = ;
   %let CountryVar = ;

   %if %sysfunc(findw(%upcase(&ADSLVARS), STUDYID)) %then %do;
      %global STUDYlabel;
      %let STUDYlabel = STUDYID;
      %let STUDYVar = STUDYID;
      %CheckLabel(&in_adsl,STUDYlabel);
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), SITEID)) %then %do;
      %global SITElabel;
      %let SITElabel = SITEID;
      %let SiteVar = SITEID;
      %CheckLabel(&in_adsl,SITElabel);
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), SEX)) %then %do;
      %global SEXlabel;
      %let SEXlabel = SEX;
      %let SexVar = SEX;
      %CheckLabel(&in_adsl,SEXlabel);
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), RACE)) %then %do;
      %global RACElabel;
      %let RACElabel = RACE;
      %let RaceVar = RACE;
      %CheckLabel(&in_adsl,RACElabel);
   %end;
   %if %sysfunc(findw(%upcase(&ADSLVARS), AGE)) %then %do;
      %global AGElabel;
      %let AGElabel = AGE;
      %let AgeVar = AGE;
      %CheckLabel(&in_adsl,AGElabel);
   %end; 
   %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY)) %then %do;
      %global COUNTRYlabel;
      %let COUNTRYlabel = COUNTRY;
      %let CountryVar = COUNTRY;
      %CheckLabel(&in_adsl,COUNTRYlabel);
   %end;
   
   proc sort data = &in_adsl out = adsl;
      by usubjid;
   run;
   
   proc sql noprint;
      create table adsl2 as
         select *, 100/freq(&TrtName) as freq_trt
         from adsl
         group by &TrtName
         order by usubjid;
   quit;

   proc datasets nolist;
      delete adsl;
      change adsl2 = adsl;
   quit;   
   
   data aecontent;
      merge aecontent(in = ina) adsl(keep = usubjid &TrtName &StudyVar &SiteVar &SexVar &RaceVar &AgeVar &CountryVar freq_trt);
      by usubjid;
      if ina;
   run;
   
   proc datasets nolist;
      delete children content;
   quit;
   
   proc sql noprint;
      create table graphtype as
         select distinct type, flag from aecontent;
   quit; 
   
   data graphtype;
      set graphtype;
      if type = "Cumulative" and flag = "Broad" then order = 5;
      else if type = "Cumulative" and flag = "Narrow" then order = 4;
      else if type = "Individual" and flag = "Narrow" then order = 1;
      else if type = "Individual" and flag = "Broad" then order = 2;
      else if type = "Individual" and flag = "Algorithm" then order = 3;
   run;
   
   %global flaglist typelist hiercodelist hiernamelist;

   proc sql noprint;
      select type
         into :typelist separated by ' '
         from graphtype
         order by order;
      select flag
         into :flaglist separated by ' '
         from graphtype
         order by order;
   quit; 
      
   proc sql noprint;
      create table smqcodelist(rename = (smq_code = term_code)) as
         select distinct smq_code from aecontent
         order by smq_code;
      create table smqcodelist2 as
         select distinct smq_code, group_name from smqcodelist, smq_children(where = (smq_level = 1))
         where smqcodelist.term_code = smq_children.term_code
         order by smq_code;
      create table hierlist as
         select * from smqcodelist2(keep = group_name), hierchart(rename = (smq_name = group_name))
         where smqcodelist2.group_name = hierchart.group_name
         order by group_name, parent, child;
      select smq_code
         into :hiercodelist separated by ' '
         from smqcodelist2
         order by smq_code;         
      select group_name
         into :hiernamelist separated by '~'
         from smqcodelist2
         order by smq_code;         
   quit;
   
   
   data aecontent(drop = rc);
      if _n_ = 1 then do;
         declare hash h_event();
         length smq_code smq_level 8;
         rc = h_event.DefineKey( "smq_code" );
         rc = h_event.DefineData( "smq_level");
         rc = h_event.DefineDone();
      end;
      do until ( eof_list );
         set listfile(keep = smq_code smq_level) end = eof_list;
         rc = h_event.add();
      end;
      do until ( eof_e );
         set aecontent end = eof_e;
         smq_level = .;
         rc = h_event.find();
         output;
      end;
   run;
   %smqexit:;
%mend SMQData;

