/*
JMP Life Sciences Path Macros

Name: UtilityPathMacros
Author: SAS Institute Inc.
Support: SAS Institute Inc.
History:

	jocrom 03Feb15 added jg_initPath
	jocrom 03Feb15 added jg_getPath

Description: UtilityPathMacros contains a collection
of useful SAS path macros that can be called from
analytical processes.
*/

/*
-----------------------------------------------------------
Copyright (C) 2015 SAS Institute, Inc. All rights reserved.

Notice:
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided.
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

%macro jg_include_ipps();
	%if %symexist(_jg_getPath_ippsloc) %then %do;
		%if %sysfunc(fileexist(&_jg_getPath_ippsloc)) %then %do;
			%include "&_jg_getPath_ippsloc";
		%end;
		%else %do;
			%put %str(ERR)%str(OR: Could not find &_jg_getPath_ippsloc)%str(.); 
		%end;
	%end;
	%else %do;
		%put %str(ERR)%str(OR: Undefined location for the installation path preferences SAS file)%str(.);
	%end;
%mend jg_include_ipps;
%jg_include_ipps();


* Refer to addinLoadCommon.jsl jg:initPath() for usage;
%macro jg_initPath(pathToInit);
	%local pathToInit _slash_ finalPathChar pathContainsFile parentPathSub currPathSub currPathSubFO rc; 
	%let pathToInit = %nrbquote(&pathToInit);
	%*Handle zero-arg case;
	%if %nrbquote(&pathToInit) ^= %nrstr() %then %do;
	%*For maximum efficiency, do nothing if the whole path already exists;
	%if ^%sysfunc(fileexist(&pathToInit)) %then %do;
		%*Make educated guess of proper slash direction to when creating directories;
		%if %sysfunc(prxmatch(%nrstr(\\), &pathToInit)) %then
			%let _slash_ = %nrstr(\);
		%else 
			%let _slash_ = %nrstr(/);
		
		%*Determine if final input path piece was a file based solely on absence of trailing slash;
		%let finalPathChar = %ksubstr(&pathToInit, %klength(&pathToInit), 1);
		%if %nrbquote(&finalPathChar) ^= %nrstr(\) and %nrbquote(&finalPathChar) ^= %nrstr(/) %then
			%let pathContainsFile = 1;
		%else
			%let pathContainsFile = 0;
		
		%*Construct progressive path steps for creating directory from input path;
		%let parentPathSub = %nrstr();
		%let currPathSub = %nrstr();
		%let currPathSubFO = %nrstr();
		%do _i_ = 1 %to %klength(&pathToInit);
			%if %nrbquote(%ksubstr(&pathToInit, &_i_, 1)) = %nrstr(\) or %nrbquote(%ksubstr(&pathToInit, &_i_, 1)) = %nrstr(/) %then %do;
				%if &_i_ > 1 %then %do;
					%let parentPathSub = &currPathSub;
					%let currPathSub = %nrbquote(%ksubstr(&pathToInit, 1, &_i_));
					%let currPathSubFO = %nrbquote(%sysfunc(prxchange(s/^.*[\/\\]([^\/\\]*)[\/\\].*?$/$1/, 1, &currPathSub)));
					%if ^%sysfunc(fileexist(&currPathSub)) and %nrbquote(&parentPathSub) ^= %nrstr() %then %do;
						%let rc = %sysfunc(dcreate(&currPathSubFO, &parentPathSub));
					%end;
				%end;
			%end;
		%end;
	%end;
	%end;
%mend jg_initPath;


* Refer to addinLoadCommon.jsl jg:getPath() for usage;
%macro jg_getPath(folderref, filename, dircreateflag, copyflag, returnflag, study);
	%local rawPath rarePath regex_user regex_user_found regex_study regex_study_found modstudy
		custom_wildcards_done loop_guard regex_custom regex_custom_found regex_custom_capt regex_custom_resolved
		donepath ret;
	%let folderref = %nrbquote(&folderref);
	%let filename = %nrbquote(&filename);
	%let study = %nrbquote(&study);
	%if &dircreateflag = %str() %then %let dircreateflag = 0;
	%if &returnflag = %str() %then %let returnflag = 1;	
	
	%if ^%symexist(_jg_getPath_Keys) or ^%symexist(_jg_getPath_Vals) %then %do;
		%put %str(ERR)%str(OR: The installation path preferences file for jg_getPath has not been loaded, or is corrupt)%str(.);
		%return;
	%end;
	
	%let rawPath = %nrbquote(%sysfunc(findw(%nrbquote(&_jg_getPath_Keys),%sysfunc(kcompress(%qlowcase(&folderref))),*,E)));
	%if &rawPath > 0 %then %do;
		%let rawPath = %nrbquote(%qscan(%nrbquote(&_jg_getPath_Vals),&rawPath,*));
	%end;
	%else %do;
		%put %str(ERR)%str(OR: Invalid path key passed to jg_getPath: &folderref)%str(.);
		%return;
	%end;
	
	%let rarePath = %nrbquote(&rawPath);
	
	%* Resolve special path wildcards;
	%let regex_user = %nrstr(<[Uu][Ss][Ee][Rr]>); 
	%let regex_user_found = %nrbquote(%sysfunc(prxmatch(%nrstr(%sysfunc(prxparse(&regex_user))), &rarePath)));
	%if &regex_user_found %then %do;
		%let rarePath = %qsysfunc(prxchange(s/&regex_user/%sysget(username)/, -1, &rarePath));
	%end;
	%let regex_study = %nrstr(<[Ss][Tt][Uu][Dd][Yy]>);
	%let regex_study_found = %nrbquote(%sysfunc(prxmatch(%nrstr(%sysfunc(prxparse(&regex_study))), &rarePath)));
	%if &regex_study_found %then %do;
		%if &study = %nrstr() %then %do;
			%if %symexist(StudyName) %then
				%let study = %nrbquote(&StudyName);
		%end;
		%if &study = %nrstr() %then %do;
			%put %str(ERR)%str(OR: You requested a study-specific folder in jg_getPath(), but did not pass in a );
			%put %str(study name, and there is no current study defined. The study folder cannot be );
			%put %str(created: &folderref)%str(.);
			%return;
		%end;
		%else %do;
			%let modstudy = &study;
			%let modstudy = %qsysfunc(tranwrd(&modstudy, %nrstr( ), %nrstr(_)));
			%let modstudy = %qsysfunc(tranwrd(&modstudy, %nrstr(@), %nrstr(\@)));
			%let rarePath = %qsysfunc(prxchange(s/&regex_study/&modstudy/, -1, &rarePath));
		%end;
	%end;
	
	%let custom_wildcards_done = 0;
	%let loop_guard = 0;
	%do %while(^&custom_wildcards_done and (%eval(&loop_guard) < 10));
		%let regex_custom = %nrstr((<.*?>));
		%let regex_custom_found = %nrbquote(%qsysfunc(prxmatch(%qsysfunc(prxparse(&regex_custom)), %nrbquote(&rarePath))));
		%if &regex_custom_found %then %do;
			%let regex_custom_capt = %nrbquote(%kscan(%ksubstr(&rarePath, &regex_custom_found),1,>)%nrstr(>));
			%*put regex_custom_capt is &regex_custom_capt;
			%let regex_custom_resolved = %jg_getPath(&regex_custom_capt,,0,0,1,&study);
			%*put regex_custom_resolved is &regex_custom_resolved;
			%if %ksubstr(&regex_custom_resolved, %klength(&regex_custom_resolved), 1) = %nrstr(\) or
				 %ksubstr(&regex_custom_resolved, %klength(&regex_custom_resolved), 1) = %nrstr(/) %then %do;
				%let regex_custom_resolved = %ksubstr(&regex_custom_resolved, 1, %eval(%klength(&regex_custom_resolved) - 1));
				%*put regex_custom_resolved is &regex_custom_resolved;
			%end;
			%let rarePath = %qsysfunc(tranwrd(&rarePath, &regex_custom_capt, &regex_custom_resolved));
			%*put rarePath is &rarePath;
		%end;
		%else %do;
			%let custom_wildcards_done = 1;
		%end;
		%let loop_guard = %eval(&loop_guard + 1);
	%end;	
	

	%* Ensure the deepest directory ends with a slash;
	%if %nrbquote(%ksubstr(&rarePath, %klength(&rarePath), 1)) ^= %nrstr(\) and %nrbquote(%ksubstr(&rarePath, %klength(&rarePath), 1)) ^= %nrstr(/) %then
		%let rarePath = %nrbquote(&rarePath%nrstr(\));
	
	%* Append a filename, if requested;
	%let rarePath = %nrbquote(%nrbquote(&rarePath)%nrbquote(&filename));
	
	%* Resolve JMP $PATH variables -- Cannot do this in SAS, so preprocessed in JSL; 
	%let donePath = %nrbquote(&rarePath);
	
	%* If requested, create missing directories;
	%if &dircreateflag > 0 %then
		%jg_initPath(&donePath);
			
	%* If requested, return the path;
	%if &returnflag > 0 %then
		%let ret = &donePath;
	%else
		%let ret = %nrstr();
	&ret
%mend jg_getPath;


%macro jg_getNotesDatasetName(varRef);
    data _null_;
        call symput("&varRef", cats('n', putc(md5("%sysget(USERDOMAIN)%sysget(USERNAME)"),'$hex31.')));
    run;
%mend jg_getNotesDatasetName;