/*
JMP Clinical Analytical Process

Name: RiskBasedMonitoring
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Provides a Risk Based Monitoring Dashboard
   
*/

/*
-----------------------------------------------------------
Copyright (C) 2013 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;
*options nomprint nomlogic nosymbolgen;

%global StudyName InPath AInPath MacroPath OutName RiskBasedMonitoringScript RiskBasedMonitoring_OutData OutPath 
        ServerOutPath PopulationSet CompWhereStmt AEWhereStmtRBM DVWhereStmtRBM IEWhereStmtRBM HOWhereStmtRBM TargetEnroll YNTargetEnroll
        TimeTrends DateStart DateEnd TimeWindowSize RBMMetdataPath RiskThresholdName ExData RBList SiteList SiteCatList SelectSiteCategories 
        MonitorAnalysis SelectSites VendorAnalysis ExPath;

%let PopulationSet = All Subjects;

*ProcessBody;

%macro RiskBasedMonitoring;

/*    
$CVS_START$
$ID: RBM_DM$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$INCD: {"DM"}$
$CMP: RSYS$ $UE: DM.SITEID and DM.COUNTRY$
$CMP: OSYS$ $UE: DM.STUDYID or DM.ARM or DM.ACTARM or DM.INVID or DM.INVNAME $ $NOTES: For output data set(s).$
$CMP: OSYS$ $UE: DM.RFXSTDTC $ $NOTES: To determine treatment status of patient.$
$CMP: OSYS$ $UE: DM.RFICDTC $ $NOTES: To determine missing consent.$
$CMP: OSYS$ $UE: DM.DTHDTC or DM.DTHFL $ $NOTES: To determine death status.$
$CVS_STOP$

$CVS_START$
$ID: RBM_DS$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData","EventOccur"}}$
$INCD: {"DS"}$
$CMP: RSYS$ $UE: DS.DSDECOD$ $NOTES: To determine disposition outcomes.$
$CMP: OSYS$ $UE: DS.DSTERM$ $NOTES: To determine disposition outcomes.$
$CMP: OSYS$ $UE: DS.DSCAT$ $NOTES: Used to determine appropriate disposition records if where statement not provided.$
$CMP: OSYS$ $UE: DS.EPOCH$ $NOTES: Used to determine appropriate disposition records if where statement not provided.$
$CMP: OSYS$ $UE: DS.DSSTDTC$
$CVS_STOP$

$CVS_START$
$ID: RBM_SV$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadData"}}$
$CMP: RSYS$ $UE: (SV.USUBJID and SV.SVSTDTC) or (FINDINGS.USUBJID and FINDINGS.xxDTC)$ $NOTES: To calculate patient-weeks on study.$
$CMP: OSYS$ $UE: SV.SVENDTC $ $NOTES: To calculate patient-weeks on study, if available.$
$CMP: OSYS$ $UE: SV.VISIT $ 
$CVS_STOP$

$CVS_START$
$ID: RBM_AE$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO","EventOccur"}}$
$INCD: {"AE"}$
$CMP: OSYS$ $UE: AE.AETERM$ $NOTES: To choose events.$
$CMP: OSYS$ $UE: AE.AESER$ $NOTES: To determine serious AEs.$
$CMP: OSYS$ $UE: AE.AEREL$ $NOTES: To determine related AEs. Not currently used.$
$CMP: OSYS$ $UE: AE.AEOUT$ $NOTES: To determine AE outcomes. $
$CMP: OSYS$ $UE: AE.AESDTH$ $NOTES: To determine AE deaths. $
$CMP: OSYS$ $UE: AE.AESTDTC$ $NOTES: For time trends. $
$CVS_STOP$

$CVS_START$
$ID: RBM_HO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO","EventOccur"}}$
$INCD: {"HO"}$
$CMP: OSYS$ $UE: AE.HOTERM$ $NOTES: To choose events.$
$CMP: OSYS$ $UE: AE.HOSTDTC$ $NOTES: For time trends. $
$CVS_STOP$

$CVS_START$
$ID: RBM_DV$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO","EventOccur"}}$
$INCD: {"DV"}$
$CMP: OSYS$ $UE: DV.DVDECOD$ $NOTES: To determine protdev outcomes.$
$CMP: OSYS$ $UE: DV.DVTERM$ $NOTES: To determine protdev outcomes.$
$CMP: OSYS$ $UE: DV.EPOCH$ $NOTES: Used to determine appropriate protdev records if where statement not provided.$
$CVS_STOP$

$CVS_START$
$ID: IE$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO"}}$
$INCD: {"IE"}$
$CMP: OSYS$ $UE: IE.IETESTCD$ $NOTES: To determine inc/exc outcomes.$
$CVS_STOP$

$CVS_START$
$ID: RBM_EX$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO"}}$
$INCD: {"EX"}$
$CMP: OSYS$ $UE: EX.EXSTDTC and EX.EXTRT$ $NOTES: To determine treatment status.$
$CVS_STOP$

$CVS_START$
$ID: CO$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO"}}$
$INCD: {"CO"}$
$CMP: OSYS$ $UE: CO.COVAL$ $NOTES: COVAL and all COVALx will be used to search for death terms.$
$CVS_STOP$

$CVS_START$
$ID: DD$
$INCF: {"LifeSciences\MacroLib\CDISCParms.sas"}$
$INCB: {{"LoadDataO"}}$
$INCD: {"DD"}$
$CVS_STOP$
*/

*initialization and argument checking;

%let ProcessName = RiskBasedMonitoring; 

%include "&MacroPath./UtilityPathMacros.sas" / nosource;
%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%include "&MacroPath./RBMMacros.sas" / nosource;
%if &exiterror %then %goto exit;

%let RBMMetadataPath = %jg_getPath(/user/clinical/rbm,,1);

/*%global origlang;
%let origlang=%sysfunc(getoption(locale));
options locale=EN_US;*/

%let _lang = %lowcase(%substr(%sysfunc(getoption(locale)),1,2));
%if &_lang = zh %then %let lang_suffix = _zh;
%else %if &_lang = ja %then %let lang_suffix = _ja;
%else %let lang_suffix = _en;
%let RBList =;

/* TRANSLATED STRING DECLARATIONS AND DEFINITIONS */
/* Declare all as global to ensure full coverage throughout this file. */
/* tr = mixed-case translation, utr = upper-case translation */
%global
trSafety 
trAdverseEvent
trAdverseEvents
trAdverseEventsandDeath
trAEDTHImpSubjsHadALOAEOFD
trAEOutcomeofFatalPresent
trAEsperPatientWeek
trAverageAEsperRandSubj
trAverageCompDevsperRandSubj
trAverageDeviationsperRandSubj
trAverageInExNotMetperRandSubj
trAverageMissPagesperRandSubj
trAverageOvrdQueriesperRandSubj
trAverageQueriesperRandSubj
trAverageSAEsperRandSubj
trB
trCanada
trChina
trCity
trCompleted
trComputedDeviations
trComputedDevsperPatientWeek
trCountry
trCountryActiveDate
trMonitorActiveDate
trVendorActiveDate
trCRFEntryResponseTime
trDead
trDeath
trDeathsperPatientWeek
trDied
trDiscontinuationReason
trDiscontinued
trDiscontinuedduetoAE
trDiscontinuedduetoDeath
trDiscontinuedOther
trDisposition
trDispositionEvent
trEditChecksforSubjLevRiskInds
trEnrollment
trEnrollmentMetrics
trExpectedRandomized
trFiveDigitZipCode
trFixed
trFrance
trGermany
trGreatBritain
trItaly
trJapan
trL
trLabel
trLatitude
trLongitude
trLosttoFollowup
trSupplemental
trSupplementalMetrics
trMean
trMedian
trMissingInformedConsent
trMissingPages
trMissingPagesperPatientWeek
trMonitor
trN
trNo
trNot
trNotAssigned
trNotRelated
trNotTreated
trObservedMinusExpectedRand
trObservedMinusTargetRand
trOngoing
trOverallIndicators
trOverallRiskIndicator
trOverallRiskIndicatorSafe
trOverallRiskIndicatorDisp
trOverallRiskIndicatorEnroll
trOverallRiskIndicatorSupp
trOverallRiskIndicatorX
trOverallRiskIndicators
trOverdueQueries
trOverdueQueriesperPatientWeek
trPatientWeeksonStudy
trPatientWithdrewfromStudy
trPercentCompletedofRandSubjs
trPercentDeathsofRandSubjs
trPercentDiscontofRandSubjs
trPercentOngoingofRandSubjs
trPercentScreenFailofTotalSubjs
trRandomized
trRandomizedperWeekActive
trRecommendedAction
trQueriesperPatientWeek
trQueryResponseTime
trRow
trSAEsperPatientWeek
trScreenFailure
trScreening
trSignal
trSignedConsent
trSiteActiveDate
trSiteIdentifiers
trSpain
trStateorProvince
trSubjectDosedPerEXDomain
trSubjectWithdrawal
trSubjectWithdrewConsent
trSubjectslistedasCompAndDisc
trSubjectsmissinformedconsent
trSubjectsThatAreBRASFON
trSubjectsThatAreRandAndNotTrt
trSubjectsThatAreTrtAndNotRand
trSubjectsWithDeathInfoSBEFC
trTargetRandomized
trTotalAEsonStudy
trTotalInclusionExclusionNotMet
trTotalProtocolDeviations
trTotalQueries
trTotalSAEsonStudy
trTotalSites
trTotalSubjects
trTreated
trU
trUnitedStates
trUnlikely
trUnlikelyRelated
trUnrelated
trValue
trVariable
trWeeksActive
trWithdrawalbySubject
trWithdrewConsent
trXperPatientWeek
trXperRandomizedSubject
trY
trYes
utrSafety
utrAdverseEvent
utrAdverseEvents
utrAdverseEventsandDeath
utrAEDTHImpSubjsHadALOAEOFD
utrAEOutcomeofFatalPresent
utrAEsperPatientWeek
utrAverageAEsperRandSubj
utrAverageCompDevsperRandSubj
utrAverageDeviationsperRandSubj
utrAverageInExNotMetperRandSubj
utrAverageMissPagesperRandSubj
utrAverageOvrdQueriesperRandSubj
utrAverageQueriesperRandSubj
utrAverageSAEsperRandSubj
utrB
utrCanada
utrChina
utrCity
utrCompleted
utrComputedDeviations
utrComputedDevsperPatientWeek
utrCountry
utrCountryActiveDate
utrMonitorActiveDate
utrVendorActiveDate
utrCRFEntryResponseTime
utrDead
utrDeath
utrDeathsperPatientWeek
utrDied
utrDiscontinuationReason
utrDiscontinued
utrDiscontinuedduetoAE
utrDiscontinuedduetoDeath
utrDiscontinuedOther
utrDisposition
utrDispositionEvent
utrEditChecksforSubjLevRiskInds
utrEnrollment
utrEnrollmentMetrics
utrExpectedRandomized
utrFiveDigitZipCode
utrFixed
utrFrance
utrGermany
utrGreatBritain
utrItaly
utrJapan
utrL
utrLabel
utrLatitude
utrLongitude
utrLosttoFollowup
utrSupplemental
utrSupplementalMetrics
utrMean
utrMedian
utrMissingInformedConsent
utrMissingPages
utrMissingPagesperPatientWeek
utrMonitor
utrN
utrNo
utrNot
utrNotAssigned
utrNotRelated
utrNotTreated
utrObservedMinusExpectedRand
utrObservedMinusTargetRand
utrOngoing
utrOverallIndicators
utrOverallRiskIndicator
utrOverallRiskIndicatorSafe
utrOverallRiskIndicatorDisp
utrOverallRiskIndicatorEnroll
utrOverallRiskIndicatorSupp
utrOverallRiskIndicatorX
utrOverallRiskIndicators
utrOverdueQueries
utrOverdueQueriesperPatientWeek
utrPatientWeeksonStudy
utrPatientWithdrewfromStudy
utrPercentCompletedofRandSubjs
utrPercentDeathsofRandSubjs
utrPercentDiscontofRandSubjs
utrPercentOngoingofRandSubjs
utrPercentScreenFailofTotalSubjs
utrRandomized
utrRandomizedperWeekActive
utrRecommendedAction
utrQueriesperPatientWeek
utrQueryResponseTime
utrRow
utrSAEsperPatientWeek
utrScreenFailure
utrScreening
utrSignal
utrSignedConsent
utrSiteActiveDate
utrSiteIdentifiers
utrSpain
utrStateorProvince
utrSubjectDosedPerEXDomain
utrSubjectWithdrawal
utrSubjectWithdrewConsent
utrSubjectslistedasCompAndDisc
utrSubjectsmissinformedconsent
utrSubjectsThatAreBRASFON
utrSubjectsThatAreRandAndNotTrt
utrSubjectsThatAreTrtAndNotRand
utrSubjectsWithDeathInfoSBEFC
utrTargetRandomized
utrTotalAEsonStudy
utrTotalInclusionExclusionNotMet
utrTotalProtocolDeviations
utrTotalQueries
utrTotalSAEsonStudy
utrTotalSites
utrTotalSubjects
utrTreated
utrU
utrUnitedStates
utrUnlikely
utrUnlikelyRelated
utrUnrelated
utrValue
utrVariable
utrWeeksActive
utrWithdrawalbySubject
utrWithdrewConsent
utrXperPatientWeek
utrXperRandomizedSubject
utrY
utrYes
trNoteNoIssues
trCategory
trRiskIndicator
trRiskVariable
trOverall
trDate
trScore
trTimeInterval
trLowerDate
trRedLowerThreshold
trRedUpperThreshold
trYellowLowerThreshold
trYellowUpperThreshold
trY 
trYes
trPerRandomizedSubject
trPerPatientWeek
trCRF
trQuery
trAverageResp
trAverageCRFResponseTime
trAverageQueryResponseTime
trOverdue
trSiteCategory
trVendor
QSiteList
QSiteCatList
SiteWhere
trDeathReportedinCommentsDomain
trHealthcareEncounters
trTotalHealthcareEncounters
trHealthcareEncountersAV
trHealthcareEncountersPW
utrTotalHealthcareEncounters
utrHealthcareEncountersAV
utrHealthcareEncountersPW
trDeathDetails
trSLevelRiskInd
trCLevelRiskInd
trMLevelRiskInd
trVLevelRiskInd
trTLevelRiskInd
trPLevelRiskInd
trSupplementalDataSetforRBM
;

%let QSiteList = ;
%if %length(&SiteList) %then %do;
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&SiteList),&i,'~')));
      %let temp = %kscan(%bquote(&SiteList),&i,'~');
      %if &i = 1 %then %let QSiteList = %nrbquote("&temp");
      %else %let QSiteList = &QSiteList,%nrbquote("&temp");
      %let i = %eval(&i+1);
   %end;
%end;
%if %length(&QSiteList) %then %let QSiteList = siteid in ( &QSiteList );

%let QSiteCatList = ;
%if %length(&SiteCatList) %then %do;
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&SiteCatList),&i,'~')));
      %let temp = %kscan(%bquote(&SiteCatList),&i,'~');
      %if &i = 1 %then %let QSiteCatList = %nrbquote("&temp");
      %else %let QSiteCatList = &QSiteCatList,%nrbquote("&temp");
      %let i = %eval(&i+1);
   %end;
%end;
%if %length(&QSiteCatList) %then %let QSiteCatList = SiteCategory in ( &QSiteCatList );

%if %length(&SiteList) and %length(&QSiteCatList) %then %let SiteWhere = &QSiteList and &QSiteCatList;
%else %if %length(&SiteList) %then %let SiteWhere = &QSiteList;
%else %if %length(&QSiteCatList) %then %let SiteWhere = &QSiteCatList;
%else %let SiteWhere =;

%if %upcase(&MonitorAnalysis) = YES %then %do;
   %if %sysfunc(fileexist(&NotesDir.rbm_site.sas7bdat)) %then %do;
      %let _tid_ = %sysfunc(open(NotesLib.rbm_site));
      %let mind = %sysfunc(varnum(&_tid_,MONITOR));
      %let _tid_ = %sysfunc(close(&_tid_));     
      %if &mind %then %do;
         %NONNULLVARS(dataset = NotesLib.rbm_site , varlistname = TMPLIST);
         %if ^%sysfunc(findw(&TMPLIST, MONITOR)) %then %let MonitorAnalysis = No;
      %end; 
   %end;
   %else %let MonitorAnalysis = No;
%end;

%if %upcase(&VendorAnalysis) = YES %then %do;
   %if %sysfunc(fileexist(&NotesDir.rbm_site.sas7bdat)) %then %do;
      %let _tid_ = %sysfunc(open(NotesLib.rbm_site));
      %let vind = %sysfunc(varnum(&_tid_,VENDOR));
      %let _tid_ = %sysfunc(close(&_tid_));     
      %if &vind %then %do;
         %NONNULLVARS(dataset = NotesLib.rbm_site , varlistname = TMPLIST);
         %if ^%sysfunc(findw(&TMPLIST, VENDOR)) %then %let VendorAnalysis = No;
      %end; 
   %end;
   %else %let VendorAnalysis = No;
%end;

/* Definitions */

%let trSLevelRiskInd = %nrbquote(%LocaleGet(SiteLevelRiskIndicators_Text));
%let trCLevelRiskInd = %nrbquote(%LocaleGet(CountryLevelRiskIndicators_Text));
%let trMLevelRiskInd = %nrbquote(%LocaleGet(MonitorLevelRiskIndicators_Text));
%let trVLevelRiskInd = %nrbquote(%LocaleGet(VendorLevelRiskIndicators_Text));
%let trTLevelRiskInd = %nrbquote(%LocaleGet(SiteLevelTimeTrends_Text));
%let trPLevelRiskInd = %nrbquote(%LocaleGet(SubjectLevelRiskIndicators_Text));
%let trRedLowerThreshold = %nrbquote(%LocaleGet(RedLowerThreshold_Text));
%let trRedUpperThreshold = %nrbquote(%LocaleGet(RedUpperThreshold_Text));
%let trYellowLowerThreshold = %nrbquote(%LocaleGet(YellowLowerThreshold_Text));
%let trYellowUpperThreshold = %nrbquote(%LocaleGet(YellowUpperThreshold_Text));
%let trLowerDate = %nrbquote(%LocaleGet(LowerDate_Text)); 
%let trTimeInterval = %nrbquote(%LocaleGet(TimeInterval_Text)); 
%let trDate = %nrbquote(%LocaleGet(Date_Text)); 
%let trScore = %nrbquote(%LocaleGet(Score_Text)); 
%let trOverall = %nrbquote(%LocaleGet(Overall_Text)); 
%let trRiskVariable = %nrbquote(%LocaleGet(RiskVariable_Text)); 
%let trRiskIndicator = %nrbquote(%LocaleGet(RiskIndicator_Text)); 
%let trCategory = %nrbquote(%LocaleGet(Category_Text)); 
%let trNoteNoIssues = %nrbquote(%LocaleGet(NoteNoIssues_Text)); 
%let trSafety = %nrbquote(%LocaleGet(Safety_Text)); /* Safety */
%let trAdverseEvent = %nrbquote(%LocaleGet(AdverseEvent_Text)); /* Adverse Event */
%let trAdverseEvents = %nrbquote(%LocaleGet(AdverseEvents_Text)); /* Adverse Events */
%let trAdverseEventsandDeath = %nrbquote(%LocaleGet(AdverseEventsandDeath_Text)); /* Adverse Events and Death */
%let trAEDTHImpSubjsHadALOAEOFD = %nrbquote(%LocaleGet(AEDTHImpSubjsHadALOAEOofForDorSAEResDeath_Text)); 
/* AEDTH implies subjects had at least one AE outcome (AEOUT) of FATAL or DEATH or an SAE that resulted in DEATH (AESDTH). */
%let trAEOutcomeofFatalPresent = %nrbquote(%LocaleGet(AEOutcomeofFatalPresent_Text)); /* AE Outcome of Fatal Present */
%let trAEsperPatientWeek = %nrbquote(%LocaleGet(AEsperPatientWeek_Text)); /* AEs per PatientWeek */
%let trAverageAEsperRandSubj = %nrbquote(%LocaleGet(AverageAEsperRandomizedSubject_Text)); /* Average AEs per Randomized Subject */
%let trAverageCompDevsperRandSubj = %nrbquote(%LocaleGet(AverageComputedDeviationsperRandomizedSubject_Text)); /* Average Computed Deviations per Randomized Subject */
%let trAverageDeviationsperRandSubj = %nrbquote(%LocaleGet(AverageDeviationsperRandomizedSubject_Text)); /* Average Deviations per Randomized Subject */
%let trAverageInExNotMetperRandSubj = %nrbquote(%LocaleGet(AverageInclusionorExclusionNotMetperRandSubject_Text)); /* Average Inclusion or Exclusion Not Met per Randomized Subject */
%let trAverageMissPagesperRandSubj = %nrbquote(%LocaleGet(AverageMissingPagesperRandomizedSubject_Text)); /* Average Missing Pages per Randomized Subject */ 
%let trAverageOvrdQueriesperRandSubj = %nrbquote(%LocaleGet(AverageOverdueQueriesperRandomizedSubject_Text)); /* Average Overdue Queries per Randomized Subject */
%let trAverageQueriesperRandSubj = %nrbquote(%LocaleGet(AverageQueriesperRandomizedSubject_Text)); /* Average Queries per Randomized Subject */
%let trAverageSAEsperRandSubj = %nrbquote(%LocaleGet(AverageSAEsperRandomizedSubject_Text)); /* Average SAEs per Randomized Subject */
%let trB = %nrbquote(%LocaleGet(B_Text)); /* B */
%let trCanada = %nrbquote(%LocaleGet(Canada_Text)); /* Canada */
%let trChina = %nrbquote(%LocaleGet(China_Text)); /* China */
%let trCity = %nrbquote(%LocaleGet(City_Text)); /* City */
%let trCompleted = %nrbquote(%LocaleGet(Completed_Text)); /* Completed */
%let trComputedDeviations = %nrbquote(%LocaleGet(ComputedDeviations_Text)); /* Computed Deviations */
%let trComputedDevsperPatientWeek = %nrbquote(%LocaleGet(ComputedDeviationsperPatientWeek_Text)); /* Computed Deviations per PatientWeek */
%let trCountry = %nrbquote(%LocaleGet(Country_Text)); /* Country */
%let trCountryActiveDate = %nrbquote(%LocaleGet(CountryActiveDate_Text)); /* Country Active Date */
%let trMonitorActiveDate = %nrbquote(%LocaleGet(MonitorActiveDate_Text)); /* Monitor Active Date */
%let trVendorActiveDate = %nrbquote(%LocaleGet(VendorActiveDate_Text)); /* Vendor Active Date */
%let trCRFEntryResponseTime = %nrbquote(%LocaleGet(CRFEntryResponseTime_Text)); /* CRF Entry Response Time */
%let trDead = %nrbquote(%LocaleGet(Dead_Text)); /* Dead */
%let trDeath = %nrbquote(%LocaleGet(Death_Text)); /* Death */
%let trDeathsperPatientWeek = %nrbquote(%LocaleGet(DeathsperPatientWeek_Text)); /* Deaths per PatientWeek */
%let trDied = %nrbquote(%LocaleGet(Died_Text)); /* Died */
%let trDiscontinuationReason = %nrbquote(%LocaleGet(DiscontinuationReason_Text)); /* Discontinuation Reason */
%let trDiscontinued = %nrbquote(%LocaleGet(Discontinued_Text)); /* Discontinued */
%let trDiscontinuedduetoAE = %nrbquote(%LocaleGet(DiscontinuedduetoAE_Text)); /* Discontinued due to AE */
%let trDiscontinuedduetoDeath = %nrbquote(%LocaleGet(DiscontinuedduetoDeath_Text)); /* Discontinued due to Death */
%let trDiscontinuedOther = %nrbquote(%LocaleGet(DiscontinuedOther_Text)); /* Discontinued Other */
%let trDisposition = %nrbquote(%LocaleGet(Disposition_Text)); /* Disposition */
%let trDispositionEvent = %nrbquote(%LocaleGet(DispositionEvent_Text)); /* Disposition Event */
%let trEditChecksforSubjLevRiskInds = %nrbquote(%LocaleGet(EditChecksforSubjectLevelRiskIndicators_Text)); /* Edit Checks for Subject-Level Risk Indicators */
%let trEnrollment = %nrbquote(%LocaleGet(Enrollment_Text)); /* Enrollment */ 
%let trEnrollmentMetrics = %nrbquote(%LocaleGet(EnrollmentMetrics_Text)); /* Enrollment Metrics */
%let trExpectedRandomized = %nrbquote(%LocaleGet(ExpectedRandomized_Text)); /* Expected Randomized */
%let trFiveDigitZipCode = %nrbquote(%LocaleGet(FiveDigitZipCode_Text)); /* Five Digit Zip Code */
%let trFixed = %nrbquote(%LocaleGet(Fixed_Text)); /* Fixed */
%let trFrance = %nrbquote(%LocaleGet(France_Text)); /* France */
%let trGermany = %nrbquote(%LocaleGet(Germany_Text)); /* Germany */
%let trGreatBritain = %nrbquote(%LocaleGet(GreatBritain_Text)); /* Great Britain */
%let trItaly = %nrbquote(%LocaleGet(Italy_Text)); /* Italy */
%let trJapan = %nrbquote(%LocaleGet(Japan_Text)); /* Japan */
%let trL = %nrbquote(%LocaleGet(L_Text)); /* L */
%let trLabel = %nrbquote(%LocaleGet(Label_Text)); /* Label */
%let trLatitude = %nrbquote(%LocaleGet(Latitude_Text)); /* Latitude */
%let trLongitude = %nrbquote(%LocaleGet(Longitude_Text)); /* Longitude */
%let trLosttoFollowup = %nrbquote(%LocaleGet(LosttoFollowup_Text)); /* Lost to Followup */
%let trSupplemental = %nrbquote(%LocaleGet(Supplemental_Text)); /* Supplemental */
%let trSupplementalMetrics = %nrbquote(%LocaleGet(SupplementalMetrics_Text)); /* Supplemental Metrics */
%let trMean = %nrbquote(%LocaleGet(Mean_Text)); /* Mean */
%let trMedian = %nrbquote(%LocaleGet(Median_Text)); /* Median */
%let trMissingInformedConsent = %nrbquote(%LocaleGet(MissingInformedConsent_Text)); /* Missing Informed Consent */
%let trMissingPages = %nrbquote(%LocaleGet(MissingPages_Text)); /* Missing Pages */
%let trMissingPagesperPatientWeek = %nrbquote(%LocaleGet(MissingPagesperPatientWeek_Text)); /* Missing Pages per PatientWeek */
%let trMonitor = %nrbquote(%LocaleGet(Monitor_Text)); /* Monitor */
%let trN = %nrbquote(%LocaleGet(N_Text)); /* N */
%let trNo = %nrbquote(%LocaleGet(No_Text)); /* No */
%let trNot = %nrbquote(%LocaleGet(Not_Text)); /* Not */
%let trNotAssigned = %nrbquote(%LocaleGet(NotAssigned_Text)); /* Not Assigned */
%let trNotRelated = %nrbquote(%LocaleGet(NotRelated_Text)); /* Not Related */
%let trNotTreated = %nrbquote(%LocaleGet(NotTreated_Text)); /* Not Treated */
%let trObservedMinusExpectedRand = %nrbquote(%LocaleGet(ObservedMinusExpectedRandomized_Text)); /* Observed Minus Expected Randomized */
%let trObservedMinusTargetRand = %nrbquote(%LocaleGet(ObservedMinusTargetRandomized_Text)); /* Observed Minus Target Randomized */
%let trOngoing = %nrbquote(%LocaleGet(Ongoing_Text)); /* Ongoing */
%let trOverallIndicators = %nrbquote(%LocaleGet(OverallIndicators_Text)); /* Overall Indicators */
%let trOverallRiskIndicator = %nrbquote(%LocaleGet(OverallRiskIndicator_Text)); /* Overall Risk Indicator */
%let trOverallRiskIndicatorSafe = %nrbquote(%LocaleGet(OverallRiskIndicatorSafety_Text)); /* Overall Risk Indicator Safety */
%let trOverallRiskIndicatorDisp = %nrbquote(%LocaleGet(OverallRiskIndicatorDisposition_Text)); /* Overall Risk Indicator Disposition */
%let trOverallRiskIndicatorEnroll = %nrbquote(%LocaleGet(OverallRiskIndicatorEnrollment_Text)); /* Overall Risk Indicator Enrollment */
%let trOverallRiskIndicatorSupp = %nrbquote(%LocaleGet(OverallRiskIndicatorSupplemental_Text)); /* Overall Risk Indicator Supplemental */
%let trOverallRiskIndicatorX = %nrbquote(%LocaleGet(OverallRiskIndicatorX_Text)); /* Overall Risk Indicator {{^1}} */
%let trOverallRiskIndicators = %nrbquote(%LocaleGet(OverallRiskIndicators_Text)); /* Overall Risk Indicators */
%let trOverdueQueries = %nrbquote(%LocaleGet(OverdueQueries_Text)); /* Overdue Queries */
%let trOverdueQueriesperPatientWeek = %nrbquote(%LocaleGet(OverdueQueriesperPatientWeek_Text)); /* Overdue Queries per PatientWeek */
%let trPatientWeeksonStudy = %nrbquote(%LocaleGet(PatientWeeksonStudy_Text)); /* PatientWeeks on Study */
%let trPatientWithdrewfromStudy = %nrbquote(%LocaleGet(PatientWithdrewfromStudy_Text)); /* Patient Withdrew from Study */
%let trPercentCompletedofRandSubjs = %nrbquote(%LocaleGet(PercentCompletedofRandomizedSubjects_Text)); /* Percent Completed of Randomized Subjects */
%let trPercentDeathsofRandSubjs = %nrbquote(%LocaleGet(PercentDeathsofRandomizedSubjects_Text)); /* Percent Deaths of Randomized Subjects */
%let trPercentDiscontofRandSubjs = %nrbquote(%LocaleGet(PercentDiscontinuedofRandomizedSubjects_Text)); /* Percent Discontinued of Randomized Subjects */
%let trPercentOngoingofRandSubjs = %nrbquote(%LocaleGet(PercentOngoingofRandomizedSubjects_Text)); /* Percent Ongoing of Randomized Subjects */
%let trPercentScreenFailofTotalSubjs = %nrbquote(%LocaleGet(PercentScreenFailofTotalSubjects_Text)); /* Percent Screen Fail of Total Subjects */
%let trQueriesperPatientWeek = %nrbquote(%LocaleGet(QueriesperPatientWeek_Text)); /* Queries per PatientWeek */
%let trQueryResponseTime = %nrbquote(%LocaleGet(QueryResponseTime_Text)); /* Query Response Time */
%let trRandomized = %nrbquote(%LocaleGet(Randomized_Text)); /* Randomized */
%let trRandomizedperWeekActive = %nrbquote(%LocaleGet(RandomizedperWeekActive_Text)); /* Randomized per Week Active */
%let trRecommendedAction = %nrbquote(%LocaleGet(RecommendedAction_Text)); /* Recommended Action */
%let trRow = %nrbquote(%LocaleGet(Row_Text)); /* Row */
%let trSAEsperPatientWeek = %nrbquote(%LocaleGet(SAEsperPatientWeek_Text)); /* SAEs per PatientWeek */
%let trScreenFailure = %nrbquote(%LocaleGet(ScreenFailure_Text)); /* Screen Failure */
%let trScreening = %nrbquote(%LocaleGet(Screening_Text)); /* Screening */
%let trSignal = %nrbquote(%LocaleGet(Signal_Text)); /* Signal */
%let trSignedConsent = %nrbquote(%LocaleGet(SignedConsent_Text)); /* Signed Consent */
%let trSiteActiveDate = %nrbquote(%LocaleGet(SiteActiveDate_Text)); /* Site Active Date */
%let trSiteIdentifiers = %nrbquote(%LocaleGet(SiteIdentifiers_Text)); /* Site Identifiers */
%let trSpain = %nrbquote(%LocaleGet(Spain_Text)); /* Spain */
%let trStateorProvince = %nrbquote(%LocaleGet(StateorProvince_Text)); /* State or Province */
%let trSubjectDosedPerEXDomain = %nrbquote(%LocaleGet(SubjectDosedPerEXDomain_Text)); /* Subject Dosed Per EX Domain */
%let trSubjectWithdrawal = %nrbquote(%LocaleGet(SubjectWithdrawal_Text)); /* Subject Withdrawal */
%let trSubjectWithdrewConsent = %nrbquote(%LocaleGet(SubjectWithdrewConsent_Text)); /* Subject Withdrew Consent */
%let trSubjectslistedasCompAndDisc = %nrbquote(%LocaleGet(Subjectslistedascompletedanddiscontinued_Text)); /* Subjects listed as completed and discontinued. */
%let trSubjectsmissinformedconsent = %nrbquote(%LocaleGet(Subjectsmissinginformedconsent_Text)); /* Subjects missing informed consent. */
%let trSubjectsThatAreBRASFON = %nrbquote(%LocaleGet(SubjectsThatAreBothRandAndScrFailsOrNeither_Text)); /* Subjects that are both randomized and screen failures or neither. */
%let trSubjectsThatAreRandAndNotTrt = %nrbquote(%LocaleGet(Subjectsthatarerandomizedandnottreated_Text)); /* Subjects that are randomized and not treated. */
%let trSubjectsThatAreTrtAndNotRand = %nrbquote(%LocaleGet(Subjectsthataretreatedandnotrandomized_Text)); /* Subjects that are treated and not randomized. */
%let trSubjectsWithDeathInfoSBEFC = %nrbquote(%LocaleGet(SubjectsWithDeathInfoShouldBeExamForConsist_Text)); /* Subjects with death information. Should be examined for consistency. */
%let trTargetRandomized = %nrbquote(%LocaleGet(TargetRandomized_Text)); /* Target Randomized */
%let trTotalAEsonStudy = %nrbquote(%LocaleGet(TotalAEsonStudy_Text)); /* Total AEs on Study */
%let trTotalInclusionExclusionNotMet = %nrbquote(%LocaleGet(TotalInclusionorExclusionNotMet_Text)); /* Total Inclusion or Exclusion Not Met */
%let trTotalProtocolDeviations = %nrbquote(%LocaleGet(TotalProtocolDeviations_Text)); /* Total Protocol Deviations */
%let trTotalQueries = %nrbquote(%LocaleGet(TotalQueries_Text)); /* Total Queries */
%let trTotalSAEsonStudy = %nrbquote(%LocaleGet(TotalSAEsonStudy_Text)); /* Total SAEs on Study */
%let trTotalSites = %nrbquote(%LocaleGet(TotalSites_Text)); /* Total Sites */
%let trTotalSubjects = %nrbquote(%LocaleGet(TotalSubjects_Text)); /* Total Subjects */
%let trTreated = %nrbquote(%LocaleGet(Treated_Text)); /* Treated */
%let trU = %nrbquote(%LocaleGet(U_Text)); /* U */
%let trUnitedStates = %nrbquote(%LocaleGet(UnitedStates_Text)); /* United States */
%let trUnlikely = %nrbquote(%LocaleGet(Unlikely_Text)); /* Unlikely */
%let trUnlikelyRelated = %nrbquote(%LocaleGet(UnlikelyRelated_Text)); /* Unlikely Related */
%let trUnrelated = %nrbquote(%LocaleGet(Unrelated_Text)); /* Unrelated */
%let trValue = %nrbquote(%LocaleGet(Value_Text)); /* Value */
%let trVariable = %nrbquote(%LocaleGet(Variable_Text)); /* Variable */
%let trWeeksActive = %nrbquote(%LocaleGet(WeeksActive_Text)); /* Weeks Active */
%let trWithdrawalbySubject = %nrbquote(%LocaleGet(WithdrawalbySubject_Text)); /* Withdrawal by Subject */
%let trWithdrewConsent = %nrbquote(%LocaleGet(WithdrewConsent_Text)); /* Withdrew Consent */
%let trXperPatientWeek = %nrbquote(%LocaleGet(XperPatientWeek_Text)); /* {{^1}} per PatientWeek */
%let trXperRandomizedSubject = %nrbquote(%LocaleGet(XperRandomizedSubject_Text)); /* {{^1}} per Randomized Subject */
%let trY = %nrbquote(%LocaleGet(Y_Text)); /* Y */
%let trYes = %nrbquote(%LocaleGet(Yes_Text)); /* Yes */
%let trPerRandomizedSubject = %nrbquote(%LocaleGet(PerRandomizedSubject_Text));
%let trPerPatientWeek = %nrbquote(%LocaleGet(PerPatientWeek_Text));
%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trCRF = %nrbquote(%LocaleGet(CRF_Text));
%let trQuery = %nrbquote(%LocaleGet(Query_Text));
%let trAverageResp = %nrbquote(%LocaleGet(AverageXResponseTime_Text));
%let trAverageCRFResponseTime = %nrbquote(%LocaleFill(&trAverageResp, &trCRF));
%let trAverageQueryResponseTime = %nrbquote(%LocaleFill(&trAverageResp, &trQuery));
%let trOverdue = %nrbquote(%LocaleGet(Overdue_Text));
%let trSiteCategory = %nrbquote(%LocaleGet(SiteCategory_Text));
%let trVendor = %nrbquote(%LocaleGet(Vendor_Text));
%let trDeathReportedinCommentsDomain = %nrbquote(%LocaleGet(DeathReportedinCommentsDomain_Text));
%let trHealthcareEncounters = %nrbquote(%LocaleGet(HealthcareEncounters_Text));
%let trTotalHealthcareEncounters = %nrbquote(%LocaleGet(TotalHealthcareEncountersonStudy_Text));
%let trHealthcareEncountersAV = %nrbquote(%LocaleGet(AverageHealthcareEncountersperRandomizedSubject_Text));
%let trHealthcareEncountersPW = %nrbquote(%LocaleGet(HealthcareEncountersperPatientWeek_Text));
%let trDeathDetails = %nrbquote(%LocaleGet(DeathDetails_Text));
%let trSupplementalDataSetforRBM = %nrbquote(%LocaleGet(SupplementalSASdatasetforRBM_Text));

/* Create an uppercase version for each of the above. */
%let utrSafety = %qupcase(&trSafety);
%let utrAdverseEvent = %qupcase(&trAdverseEvent);
%let utrAdverseEvents = %qupcase(&trAdverseEvents);
%let utrAdverseEventsandDeath = %qupcase(&trAdverseEventsandDeath);
%let utrAEDTHImpSubjsHadALOAEOFD = %qupcase(&trAEDTHImpSubjsHadALOAEOFD);
%let utrAEOutcomeofFatalPresent = %qupcase(&trAEOutcomeofFatalPresent);
%let utrAEsperPatientWeek = %qupcase(&trAEsperPatientWeek);
%let utrAverageAEsperRandSubj = %qupcase(&trAverageAEsperRandSubj);
%let utrAverageCompDevsperRandSubj = %qupcase(&trAverageCompDevsperRandSubj);
%let utrAverageDeviationsperRandSubj = %qupcase(&trAverageDeviationsperRandSubj);
%let utrAverageInExNotMetperRandSubj = %qupcase(&trAverageInExNotMetperRandSubj);
%let utrAverageMissPagesperRandSubj = %qupcase(&trAverageMissPagesperRandSubj);
%let utrAverageOvrdQueriesperRandSubj = %qupcase(&trAverageOvrdQueriesperRandSubj);
%let utrAverageQueriesperRandSubj = %qupcase(&trAverageQueriesperRandSubj);
%let utrAverageSAEsperRandSubj = %qupcase(&trAverageSAEsperRandSubj);
%let utrB = %qupcase(&trB);
%let utrCanada = %qupcase(&trCanada);
%let utrChina = %qupcase(&trChina);
%let utrCity = %qupcase(&trCity);
%let utrCompleted = %qupcase(&trCompleted);
%let utrComputedDeviations = %qupcase(&trComputedDeviations);
%let utrComputedDevsperPatientWeek = %qupcase(&trComputedDevsperPatientWeek);
%let utrCountry = %qupcase(&trCountry);
%let utrCountryActiveDate = %qupcase(&trCountryActiveDate);
%let utrMonitorActiveDate = %qupcase(&trMonitorActiveDate);
%let utrVendorActiveDate = %qupcase(&trVendorActiveDate);
%let utrCRFEntryResponseTime = %qupcase(&trCRFEntryResponseTime);
%let utrDead = %qupcase(&trDead);
%let utrDeath = %qupcase(&trDeath);
%let utrDeathsperPatientWeek = %qupcase(&trDeathsperPatientWeek);
%let utrDied = %qupcase(&trDied);
%let utrDiscontinuationReason = %qupcase(&trDiscontinuationReason);
%let utrDiscontinued = %qupcase(&trDiscontinued);
%let utrDiscontinuedduetoAE = %qupcase(&trDiscontinuedduetoAE);
%let utrDiscontinuedduetoDeath = %qupcase(&trDiscontinuedduetoDeath);
%let utrDiscontinuedOther = %qupcase(&trDiscontinuedOther);
%let utrDisposition = %qupcase(&trDisposition);
%let utrDispositionEvent = %qupcase(&trDispositionEvent);
%let utrEditChecksforSubjLevRiskInds = %qupcase(&trEditChecksforSubjLevRiskInds);
%let utrEnrollment = %qupcase(&trEnrollment);
%let utrEnrollmentMetrics = %qupcase(&trEnrollmentMetrics);
%let utrExpectedRandomized = %qupcase(&trExpectedRandomized);
%let utrFiveDigitZipCode = %qupcase(&trFiveDigitZipCode);
%let utrFixed = %qupcase(&trFixed);
%let utrFrance = %qupcase(&trFrance);
%let utrGermany = %qupcase(&trGermany);
%let utrGreatBritain = %qupcase(&trGreatBritain);
%let utrItaly = %qupcase(&trItaly);
%let utrJapan = %qupcase(&trJapan);
%let utrL = %qupcase(&trL);
%let utrLabel = %qupcase(&trLabel);
%let utrLatitude = %qupcase(&trLatitude);
%let utrLongitude = %qupcase(&trLongitude);
%let utrLosttoFollowup = %qupcase(&trLosttoFollowup);
%let utrSupplemental = %qupcase(&trSupplemental);
%let utrSupplementalMetrics = %qupcase(&trSupplementalMetrics);
%let utrMean = %qupcase(&trMean);
%let utrMedian = %qupcase(&trMedian);
%let utrMissingInformedConsent = %qupcase(&trMissingInformedConsent);
%let utrMissingPages = %qupcase(&trMissingPages);
%let utrMissingPagesperPatientWeek = %qupcase(&trMissingPagesperPatientWeek);
%let utrMonitor = %qupcase(&trMonitor);
%let utrN = %qupcase(&trN);
%let utrNo = %qupcase(&trNo);
%let utrNot = %qupcase(&trNot);
%let utrNotAssigned = %qupcase(&trNotAssigned);
%let utrNotRelated = %qupcase(&trNotRelated);
%let utrNotTreated = %qupcase(&trNotTreated);
%let utrObservedMinusExpectedRand = %qupcase(&trObservedMinusExpectedRand);
%let utrObservedMinusTargetRand = %qupcase(&trObservedMinusTargetRand);
%let utrOngoing = %qupcase(&trOngoing);
%let utrOverallIndicators = %qupcase(&trOverallIndicators);
%let utrOverallRiskIndicator = %qupcase(&trOverallRiskIndicator);
%let utrOverallRiskIndicatorSafe = %qupcase(&trOverallRiskIndicatorSafe);
%let utrOverallRiskIndicatorDisp = %qupcase(&trOverallRiskIndicatorDisp);
%let utrOverallRiskIndicatorEnroll = %qupcase(&trOverallRiskIndicatorEnroll);
%let utrOverallRiskIndicatorSupp = %qupcase(&trOverallRiskIndicatorSupp);
%let utrOverallRiskIndicatorX = %qupcase(&trOverallRiskIndicatorX);
%let utrOverallRiskIndicators = %qupcase(&trOverallRiskIndicators);
%let utrOverdueQueries = %qupcase(&trOverdueQueries);
%let utrOverdueQueriesperPatientWeek = %qupcase(&trOverdueQueriesperPatientWeek);
%let utrPatientWeeksonStudy = %qupcase(&trPatientWeeksonStudy);
%let utrPatientWithdrewfromStudy = %qupcase(&trPatientWithdrewfromStudy);
%let utrPercentCompletedofRandSubjs = %qupcase(&trPercentCompletedofRandSubjs);
%let utrPercentDeathsofRandSubjs = %qupcase(&trPercentDeathsofRandSubjs);
%let utrPercentDiscontofRandSubjs = %qupcase(&trPercentDiscontofRandSubjs);
%let utrPercentOngoingofRandSubjs = %qupcase(&trPercentOngoingofRandSubjs);
%let utrPercentScreenFailofTotalSubjs = %qupcase(&trPercentScreenFailofTotalSubjs);
%let utrRandomized = %qupcase(&trRandomized);
%let utrRandomizedperWeekActive = %qupcase(&trRandomizedperWeekActive);
%let utrRecommendedAction = %qupcase(&trRecommendedAction);
%let utrQueriesperPatientWeek = %qupcase(&trQueriesperPatientWeek);
%let utrQueryResponseTime = %qupcase(&trQueryResponseTime);
%let utrRow = %qupcase(&trRow);
%let utrSAEsperPatientWeek = %qupcase(&trSAEsperPatientWeek);
%let utrScreenFailure = %qupcase(&trScreenFailure);
%let utrScreening = %qupcase(&trScreening);
%let utrSignal = %qupcase(&trSignal);
%let utrSignedConsent = %qupcase(&trSignedConsent);
%let utrSiteActiveDate = %qupcase(&trSiteActiveDate);
%let utrSiteIdentifiers = %qupcase(&trSiteIdentifiers);
%let utrSpain = %qupcase(&trSpain);
%let utrStateorProvince = %qupcase(&trStateorProvince);
%let utrSubjectDosedPerEXDomain = %qupcase(&trSubjectDosedPerEXDomain);
%let utrSubjectWithdrawal = %qupcase(&trSubjectWithdrawal);
%let utrSubjectWithdrewConsent = %qupcase(&trSubjectWithdrewConsent);
%let utrSubjectslistedasCompAndDisc = %qupcase(&trSubjectslistedasCompAndDisc);
%let utrSubjectsmissinformedconsent = %qupcase(&trSubjectsmissinformedconsent);
%let utrSubjectsThatAreBRASFON = %qupcase(&trSubjectsThatAreBRASFON);
%let utrSubjectsThatAreRandAndNotTrt = %qupcase(&trSubjectsThatAreRandAndNotTrt);
%let utrSubjectsThatAreTrtAndNotRand = %qupcase(&trSubjectsThatAreTrtAndNotRand);
%let utrSubjectsWithDeathInfoSBEFC = %qupcase(&trSubjectsWithDeathInfoSBEFC);
%let utrTargetRandomized = %qupcase(&trTargetRandomized);
%let utrTotalAEsonStudy = %qupcase(&trTotalAEsonStudy);
%let utrTotalInclusionExclusionNotMet = %qupcase(&trTotalInclusionExclusionNotMet);
%let utrTotalProtocolDeviations = %qupcase(&trTotalProtocolDeviations);
%let utrTotalQueries = %qupcase(&trTotalQueries);
%let utrTotalSAEsonStudy = %qupcase(&trTotalSAEsonStudy);
%let utrTotalSites = %qupcase(&trTotalSites);
%let utrTotalSubjects = %qupcase(&trTotalSubjects);
%let utrTreated = %qupcase(&trTreated);
%let utrU = %qupcase(&trU);
%let utrUnitedStates = %qupcase(&trUnitedStates);
%let utrUnlikely = %qupcase(&trUnlikely);
%let utrUnlikelyRelated = %qupcase(&trUnlikelyRelated);
%let utrUnrelated = %qupcase(&trUnrelated);
%let utrValue = %qupcase(&trValue);
%let utrVariable = %qupcase(&trVariable);
%let utrWeeksActive = %qupcase(&trWeeksActive);
%let utrWithdrawalbySubject = %qupcase(&trWithdrawalbySubject);
%let utrWithdrewConsent = %qupcase(&trWithdrewConsent);
%let utrXperPatientWeek = %qupcase(&trXperPatientWeek);
%let utrXperRandomizedSubject = %qupcase(&trXperRandomizedSubject);
%let utrY = %qupcase(&trY);
%let utrYes = %qupcase(&trYes);
%let utrTotalHealthcareEncounters = %qupcase(&trTotalHealthcareEncounters);
%let utrHealthcareEncountersAV = %qupcase(&trHealthcareEncountersAV);
%let utrHealthcareEncountersPW = %qupcase(&trHealthcareEncountersPW);

/* END OF TRANSLATED STRING DECLARATIONS AND DEFINITIONS */

* data error checking and set up;
%if ^%sysfunc(findw(&domains, DM)) %then %do;
   %put [MS_S_0157];
   %put ERROR: DM or ADDM is required for Risk Based Monitoring.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = DM, var = STUDYID);
%CHECKPRES(domain = DM, var = COUNTRY);
%CHECKPRES(domain = DM, var = RFXSTDTC); *first dose;
%CHECKPRES(domain = DM, var = RFICDTC); *informed consent;
%CHECKPRES(domain = DM, var = DTHDTC); 
%CHECKPRES(domain = DM, var = DTHFL);
%CHECKPRES(domain = DM, var = SITEID); 
%CHECKPRES(domain = DM, var = ARM); 
%CHECKPRES(domain = DM, var = ACTARM); 
%CHECKPRES(domain = DM, var = INVID); 
%CHECKPRES(domain = DM, var = INVNAM); 
%CHECKPRES(domain = DM, var = DMDTC);
%CHECKPRES(domain = DM, var = RFENDTC);

%if ^%length(&SITEID) %then %do;
   %put [MS_S_0158];
   %put ERROR: SITEID is required for Risk Based Monitoring.;
   %let exiterror = 1;
   %goto exit;
%end;

%if ^%length(&COUNTRY) %then %do;
   %put [MS_S_0159];
   %put ERROR: COUNTRY is required for Risk Based Monitoring.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %upcase(&TimeTrends) = YES and ^%length(&DMDTC) and ^%length(&DateStart) %then %do;
   %put ERROR: DMDTC not detected in DM or ADSL. Date Start must be specified for time trends.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %upcase(&TimeTrends) = YES and ^%length(&RFENDTC) and ^%length(&DateEnd) %then %do;
   %put ERROR: RFENDTC not detected in DM or ADSL. Date End must be specified for time trends.;
   %let exiterror = 1;
   %goto exit;
%end;

%let dmlist = usubjid &STUDYID &COUNTRY &RFXSTDTC &RFICDTC &DTHDTC &DTHFL &SITEID &ARM &ACTARM &INVID &INVNAM;

%if ^%sysfunc(findw(&domains, DS)) %then %do;
   %put [MS_S_0160];
   %put ERROR: DS or ADDS is required for Risk Based Monitoring.;
   %let exiterror = 1;
   %goto exit;
%end;

%CHECKPRES(domain = DS, var = DSTERM); 
%CHECKPRES(domain = DS, var = DSDECOD);
%CHECKPRES(domain = DS, var = DSCAT); 
%CHECKPRES(domain = DS, var = EPOCH, adddomain = 1);           
%CHECKPRES(domain = DS, var = DSSTDTC);                    

%if ^%length(&DSDECOD) %then %do;
   %put [MS_S_0161];
   %put ERROR: DSDECOD is required for Risk Based Monitoring.;
   %let exiterror = 1;
   %goto exit;
%end;

%let dslist = usubjid &DSTERM &DSDECOD &DSCAT &DSEPOCH &DSSTDTC;

%if %sysfunc(findw(&domains, AE)) %then %do;   
   %CHECKPRES(domain = AE, var = AETERM); 
   %CHECKPRES(domain = AE, var = AESTDTC); 
   %CHECKPRES(domain = AE, var = AESER);
   %CHECKPRES(domain = AE, var = AEREL); 
   %CHECKPRES(domain = AE, var = AEOUT);           
   %CHECKPRES(domain = AE, var = AESDTH);

   %let aelist = usubjid &AETERM &AESTDTC &AESER &AEREL &AEOUT &AESDTH;
%end;

%global SVSTDTC;
%let SVSTDTC =;
%if %sysfunc(findw(&domains, SV)) %then %do;
   %CHECKPRES(domain = SV, var = SVSTDTC);
   %CHECKPRES(domain = SV, var = SVENDTC);
   %CHECKPRES(domain = SV, var = VISIT);

   %let svlist = usubjid &SVSTDTC &SVENDTC &VISIT;   
%end;

%let RBMFINDLIST = ;
%if %length(&findingsdomains) %then %do;
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i,' ')));
      %let temp = %scan(&findingsdomains,&i,' ');
      %CHECKPRES(domain = &temp, var = &temp.dtc); 
      %if %length(&&&temp.dtc) %then %do;
         %let RBMFINDLIST = &RBMFINDLIST &temp;
      %end;
      %let i = %eval(&i+1);
   %end;   
%end;

%if ^(%length(&SVSTDTC) or %length(&RBMFINDLIST)) %then %do;
   %put [MS_S_0162];
   %put ERROR: No SV or Findings domains are present to calculate patient-weeks on study.;
   %let exiterror = 1;
   %goto exit;
%end;

%if %sysfunc(findw(&domains, DV)) %then %do;
   %CHECKPRES(domain = DV, var = EPOCH, adddomain = 1);           
   %CHECKPRES(domain = DV, var = DVDECOD);                    
   %CHECKPRES(domain = DV, var = DVTERM);                    

   %let dvlist = usubjid &DVEPOCH &DVDECOD &DVTERM;   
%end;

%if %sysfunc(findw(&domains, IE)) %then %do;
   %CHECKPRES(domain = IE, var = IETESTCD);           

   %let ielist = usubjid &IETESTCD;   
%end;

*** EX ***;

%if %sysfunc(findw(&domains, EX)) %then %do;
   %CHECKPRES(domain = EX, var = EXSTDTC);           
   %CHECKPRES(domain = EX, var = EXTRT);

   %let exlist = usubjid &EXSTDTC &EXTRT;   
%end; 

*** CO ***;

%if %sysfunc(findw(&domains, CO)) %then %do;
   %CHECKPRES(domain = CO, var = COVAL);
   %let covalstr1 = &COVAL;
   %if %length(&covalstr1) %then %do;
      %let numcoval = 1;
      %let covalflg = 0;
      %do %while(&covalflg = 0);
         %if %sysfunc(findw(%upcase(&COVARS), %upcase(COVAL&numcoval))) %then %do;
            %let covalstr1 = &covalstr1 COVAL&numcoval;
            %let numcoval = %eval(&numcoval+1);
         %end;
         %else %let covalflg = 1;
      %end;
   %end;
   %let colist = &covalstr1;   
%end; 

*** HO ***;

%if %sysfunc(findw(&domains, HO)) %then %do;   
   %CHECKPRES(domain = HO, var = HOTERM); 
   %CHECKPRES(domain = HO, var = HOSTDTC); 

   %let holist = usubjid &HOTERM &HOSTDTC;
%end;

libname WtPath "&RBMMetadataPath";

data _null_;
   set WtPath.riskthresholds end = eof;
   where RiskThresholdName = "&RiskThresholdName";
   if eof then call symput("RiskThresholdDataSet", RiskThresholdDataSet);
run;

*** External ***;

%if %length(&ExData) %then %do;  
   %GetOrigFileName(ExData,OrigExData);
   %PathName(&ExData);
   %if &exiterror %then %do;
      %put [MS_S_0163];
      %put ERROR: Input dataset specification is invalid.;
      %goto gexit;
   %end;
   %let ExPath = &TmpPath;
   libname ExLib "&ExPath";
   %let ExData = &TmpName;
%end;

*******************;
*** gather data ***;
*******************;

* put the main processing code in a macro so data sets can be subsetted by date;
%macro GATHER_DATA(date_lower=0000,date_upper=9999,timetrendsgather = 0);

%put Entering GATHER_DATA with date_lower=&date_lower and date_upper=&date_upper;

%global SiteLabel CountryLabel VarOrder UserVars includeds QSwitchList NoActFlg
   RiskBasedMonitoring_OutData_si RiskBasedMonitoring_OutData_co RiskBasedMonitoring_OutData_mo RiskBasedMonitoring_OutData_ss 
   RiskBasedMonitoring_OutData_ri RiskBasedMonitoring_OutData_sis RiskBasedMonitoring_OutData_mr RiskBasedMonitoring_OutData_vo RiskBasedMonitoring_OutData_vr
   SiteGroup SubsGroup SafetyGroup SuppGroup EnrlGroup OverGroup_Siteid
   CountryListMap Flag EnrlGroupCountry OverGroup_Country EnrlGroupMonitor EnrlGroupVendor OverGroup_Monitor OverGroup_Vendor
   FlagUSA FlagCAN FlagFRA FlagDEU FlagGbr FlagITA FlagESP FlagCHN FlagJPN CODTH
; 

*** DM ***;
%if %index(%upcase(&in_adsl), ADSL) %then %do;
   %LOADDATA(domlist = DM,mergesupp = 0,checkreview=1);
%end;
proc sort data = &in_dm out = rbm_dm(keep = &dmlist);
   by usubjid;
   %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) and %length(&DMDTC) %then %do;
      where DMDTC <= "&date_upper";
      * where "&date_lower" < DMDTC <= "&date_upper";
   %end;
run;
%if %length(&QSiteCatList) %then %do;
   proc sort data = NotesLib.rbm_site out = rbm_site(keep = siteid SiteCategory %if %upcase(&MonitorAnalysis) = YES %then %do; Monitor %end; %if %upcase(&VendorAnalysis) = YES %then %do; Vendor %end;);
      by siteid;
   run;

   proc sort data = rbm_dm out = rbm_dm;
      by siteid;
   run;   
   
   data rbm_dm;
      merge rbm_dm(in = ina) rbm_site;
      by siteid;
      if ina;
   run;
   
   proc sort data = rbm_dm out = rbm_dm;
      by usubjid;
   run;
   
   proc datasets nolist;
      delete rbm_site;
   quit;   
%end;
%else %if ^%length(&QSiteCatList) and (%upcase(&MonitorAnalysis) = YES or %upcase(&VendorAnalysis) = YES) %then %do;
   proc sort data = NotesLib.rbm_site out = rbm_site(keep = siteid %if %upcase(&MonitorAnalysis) = YES %then %do; Monitor %end; %if %upcase(&VendorAnalysis) = YES %then %do; Vendor %end;);
      by siteid;
   run;

   proc sort data = rbm_dm out = rbm_dm;
      by siteid;
   run;   
   
   data rbm_dm;
      merge rbm_dm(in = ina) rbm_site;
      by siteid;
      if ina;
   run;
   
   proc sort data = rbm_dm out = rbm_dm;
      by usubjid;
   run;
   
   proc datasets nolist;
      delete rbm_site;
   quit;   
%end;
%if %length(&QSiteList) or %length(&QSiteCatList) %then %do;
   data rbm_dm;
      set rbm_dm;
      where %unquote(&SiteWhere);
   run;
%end;

%let SiteLabel = SITEID;
%CheckLabel(rbm_dm,SiteLabel);
%let _tid_ = %sysfunc(open(rbm_dm));
%let cind = %sysfunc(varnum(&_tid_,COUNTRY));
%let _tid_ = %sysfunc(close(&_tid_));  
%if &cind %then %do;
   %let CountryLabel = COUNTRY;
   %CheckLabel(rbm_dm,CountryLabel);
%end;

*** Merge in Global Country example for Nicardipine ***;
%if %nrbquote(%upcase(&StudyName)) = NICARDIPINE or %nrbquote(%upcase(&StudyName)) = NICARDIPINE EARLY SNAPSHOT %then %do;
    libname RBMPath "&RBMMetadataPath";      

   proc sort data = rbm_dm out = rbm_dm;
      by siteid;
   run;
   
   data rbm_dm;
      merge rbm_dm(in = ina drop = country) RBMPath.NicardipineRBM&lang_suffix (keep = siteid country);
      by siteid;
      if ina;
   run;

   proc sort data = rbm_dm out = rbm_dm;
      by usubjid;
   run;   
%end;

%global NumCountry;
%let NumCountry = 0; 

%let CountryListMap = ;
%let MapUSA = ;
%let MapCAN = ; 
%let MapFRA = ; 
%let MapDEU = ; 
%let MapITA = ; 
%let MapJPN = ; 
%let MapCHN = ; 
%let MapESP = ; 
%let MapGBR = ;
%let FlagUSA = ;
%let FlagCAN = ; 
%let FlagFRA = ; 
%let FlagDEU = ; 
%let FlagITA = ; 
%let FlagJPN = ; 
%let FlagCHN = ; 
%let FlagESP = ; 
%let FlagGBR = ;
%if %length(&COUNTRY) %then %do;
   proc sql noprint;      
      select distinct upcase(country)
         into :CountryList separated by '~'
         from rbm_dm;
      %let NumCountry = &sqlobs; 
   quit;

   %if %sysfunc(findw(~%upcase(&CountryList)~,UNITED STATES, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,USA, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,US, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrUnitedStates, ~)) %then %let MapUSA = USA;
   %if %sysfunc(findw(~%upcase(&CountryList)~,CANADA, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,CAN, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,CA, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrCanada, ~)) %then %let MapCAN = CAN;
   %if %sysfunc(findw(~%upcase(&CountryList)~,FRANCE, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,FRA, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,FR, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrFrance, ~)) %then %let MapFRA = FRA;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,GERMANY, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,DEU, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,DE, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrGermany, ~)) %then %let MapDEU = DEU;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,ITALY, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,ITA, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,IT, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrItaly, ~)) %then %let MapITA = ITA;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,JAPAN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,JPN, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,JP, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrJapan, ~)) %then %let MapJPN = JPN;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,CHINA, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,CHN, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,CN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrChina, ~)) %then %let MapCHN = CHN;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,SPAIN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,ESP, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,ES, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrSpain, ~)) %then %let MapESP = ESP;   
   %if %sysfunc(findw(~%upcase(&CountryList)~,GREAT BRITAIN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,GBR, ~)) or 
       %sysfunc(findw(~%upcase(&CountryList)~,GB, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrGreatBritain, ~)) %then %let MapGBR = GBR;   

   %let CountryListMap = &MapUSA &MapCAN &MapFRA &MapDEU &MapGBR &MapITA &MapESP &MapCHN &MapJPN;
   
   %if %length(&CountryListMap) %then %do;
      %let CountryListMap = %sysfunc(compbl(&CountryListMap));
   %end;

   %if %sysfunc(findw(~%upcase(&CountryList)~,UNITED STATES, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrUnitedStates, ~)) %then %let FlagUSA = UNITED STATES;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,USA, ~)) %then %let FlagUSA = USA;    
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,US, ~)) %then %let FlagUSA = US;
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,CANADA, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrCanada, ~)) %then %let FlagCAN = CANADA;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,CAN, ~)) %then %let FlagCAN = CAN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,CA, ~)) %then %let FlagCAN = CA;
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,FRANCE, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrFrance, ~)) %then %let FlagFRA = FRANCE;   
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,FRA, ~)) %then %let FlagFRA = FRA;    
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,FR, ~)) %then %let FlagFRA = FR;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,GERMANY, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrGermany, ~)) %then %let FlagDEU = GERMANY;   
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,DEU, ~)) %then %let FlagDEU = DEU;   
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,DE, ~)) %then %let FlagDEU = DE;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,ITALY, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrItaly, ~)) %then %let FlagITA = ITALY;   
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,ITA, ~)) %then %let FlagITA = ITA;   
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,IT, ~)) %then %let FlagITA = IT;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,JAPAN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrJapan, ~)) %then %let FlagJPN = JAPAN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,JPN, ~)) %then %let FlagJPN = JPN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,JP, ~)) %then %let FlagJPN = JP;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,CHINA, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrChina, ~)) %then %let FlagCHN = CHINA;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,CHN, ~)) %then %let FlagCHN = CHN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,CN, ~)) %then %let FlagCHN = CN;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,SPAIN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrSpain, ~)) %then %let FlagESP = SPAIN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,ESP, ~)) %then %let FlagESP = ESP;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,ES, ~)) %then %let FlagESP = ES;   
   
   %if %sysfunc(findw(~%upcase(&CountryList)~,GREAT BRITAIN, ~)) or %sysfunc(findw(~%upcase(&CountryList)~,&utrGreatBritain, ~)) %then %let FlagGBR = GREAT BRITAIN;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,GBR, ~)) %then %let FlagGBR = GBR;
   %else %if %sysfunc(findw(~%upcase(&CountryList)~,GB, ~)) %then %let FlagGBR = GB;   
%end;

***********************************************;
*** External data using CDISC-like standard ***;
***********************************************;

%global RBVariables RBVariables_Subject RBVariables_Site RBLabels RBLabels_Subject RBLabels_Site AllRBVariables AllRBLabels AllRBVariables_Subject AllRBLabels_Subject AllRBVarOrder AllRBLabOrder;
%let RBVariables =;
%let RBVariables_Subject =;
%let RBVariables_Site =;
%let RBLabels =;
%let RBLabels_Subject =;
%let RBLabels_Site =;
%let AllRBVariables =; 
%let AllRBLabels =;
%let AllRBVariables_Subject =; 
%let AllRBLabels_Subject =;
%let AllRBVarOrder =; 
%let AllRBLabOrder =;

%if %length(&ExData) %then %do;  
   *** use translated data if japansese or chinese system encoding detected ***;
   %if %nrbquote(%upcase(&StudyName)) = NICARDIPINE and &ExData = rb_en %then %do;
      %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(zh) %then %let ExData = rb_zh;
      %else %if %nrbquote(%lowcase(%substr(%sysfunc(getoption(locale)),1,2))) = %nrbquote(ja) %then %let ExData = rb_ja;   
   %end;
   
   proc contents data = ExLib.&ExData;
      ods exclude all;
      ods output Variables = Variables(keep = variable);
   run;
   
   proc sql noprint;
      select distinct upcase(variable)
         into :RBList separated by ' '
         from variables
         order by upcase(variable);
   quit;   
   
   %if ^%sysfunc(findw(&RBList, USUBJID)) %then %do;
         %put ERROR: Unique Subject Identifier (USUBJID) is required for the supplemental SAS data set for RBM (even if missing for all values).;
         %let exiterror = 1;
         %goto gexit;    
   %end;
   %if ^%sysfunc(findw(&RBList, SITEID)) %then %do;
         %put ERROR: Study Site Identifier (SITEID) is required for the supplemental SAS data set for RBM (even if missing for all values).;
         %let exiterror = 1;
         %goto gexit;    
   %end;
   %if ^%sysfunc(findw(&RBList, VARIABLE)) %then %do;
         %put ERROR: Variable (VARIABLE) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto gexit;    
   %end;   
   %if ^%sysfunc(findw(&RBList, RBDECOD)) %then %do;
         %put ERROR: Standardized Term (RBDECOD) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto gexit;    
   %end;    
   %if ^%sysfunc(findw(&RBList, RBCAT)) %then %do;
         %put ERROR: Category (RBCAT) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto gexit;    
   %end;
   %if ^%sysfunc(findw(&RBList, RBSTDTC)) %then %do;
         %put ERROR: Start Date/Time (RBSTDTC) is required for the supplemental SAS data set for RBM.;
         %let exiterror = 1;
         %goto gexit;    
   %end;      

   proc datasets nolist;
      delete variables;
   quit;  
   
   proc sql noprint;
      create table pairs as
         select distinct variable, rbdecod, rbcat  
         from ExLib.&ExData
         order by variable;
         %let numpairs = &sqlobs;   
      select distinct variable
         into :RBVariables separated by ' '
         from pairs
         order by variable;
         %let numvars = &sqlobs;
      select distinct rbdecod
         into :RBLabels separated by '~'
         from pairs
         order by variable;
         %let numlabs = &sqlobs;
   quit;
   
   *** apply recode of categories ***;
   data pairs;
      set pairs;
      if rbcat = "&trAdverseEvents" then rbcat = "&trSafety";
      if rbcat = "&trManuallyEntered" then rbcat = "&trSupplemental";
   run;
   
   proc sql noprint;
      select distinct upcase(rbcat)
         into :CatList separated by '~'
         from pairs
         order by rbcat;
   quit;   

   %let KnownCatList = %upcase(&trSafety)~%upcase(&trSupplemental)~%upcase(&trEnrollment)~%upcase(&trDisposition);

   %let i = 1;
   %let CatError = 0;
   %do %while(%length(%scan(&CatList,&i,~)));
      %let temp = %scan(&CatList,&i,~);
      %if ^%sysfunc(findw(~%upcase(&KnownCatList)~,&temp, ~)) %then %let CatError = 1;
      %let i = &i + 1;
   %end;

   %if &CatError %then %do;
      %put [MS_S_0072];
      %put ERROR: Unknown category detected in RBCAT. Category must be one of;
      %put ERROR: &trSafety, &trSupplemental, &trEnrollment or &trDisposition.;
      %let exiterror = 1;
   %end;

   %if %sysevalf(&numvars<&numpairs) %then %do;
      %put [MS_S_0073];
      %put ERROR: &TmpName.sas7bdat has multiple RBDECOD associated with one or more levels of VARIABLE.;
      %let exiterror = 1;
   %end; 

   %if %sysevalf(&numvars<&numpairs) %then %do;
      %put [MS_S_0074];
      %put ERROR: &TmpName.sas7bdat has multiple VARIABLE associated with one or more levels of RBDECOD.;
      %let exiterror = 1;
   %end; 
   %if &exiterror %then %goto gexit;
   
   *** site and subject variables and labels need to be distinct ***;
   proc sql noprint;
      create table subpairs as
         select distinct variable, rbdecod 
         from ExLib.&ExData
         where usubjid ne ""
         order by variable ;
      select distinct variable
         into :RBVariables_Subject separated by ' '
         from subpairs
         order by variable;
      select distinct rbdecod
         into :RBLabels_Subject separated by '~'
         from subpairs
         order by variable;         
      create table sitepairs as
         select distinct variable, rbdecod 
         from ExLib.&ExData
         where usubjid = ""
         order by variable;
      select distinct variable
         into :RBVariables_Site separated by ' '
         from sitepairs
         order by variable;                  
      select distinct rbdecod
         into :RBLabels_Site separated by '~'
         from sitepairs
         order by variable;         
   quit;
      
   data overlap;
     merge subpairs(keep = variable in = ina) sitepairs(keep = variable in = inb);
     by variable;
     if ina and inb;
   run;
   %NObsVars(overlap);
   %if &nobs %then %do; 
      proc sql noprint;
         select distinct variable
            into :RBOverlap separated by ' '
            from overlap
            order by variable;
      quit;
   
      %put ERROR: Variables need to be distinct between subject-level indicators and site-level indicators.;
      %put ERROR: Overlap was detected for the following variables: &RBOverlap;
      %put ERROR: Modify site-level variable names in input SAS data set for risk-based monitoring to be distinct from subject-level variables;
      %let exiterror = 1;
      %goto gexit;          
   %end;

   proc sort data = subpairs;
      by rbdecod;
   run;

   proc sort data = sitepairs;
      by rbdecod;
   run;   
   
   data overlap;
     merge subpairs(keep = rbdecod in = ina) sitepairs(keep = rbdecod in = inb);
     by rbdecod;
     if ina and inb;
   run;
      
   %NObsVars(overlap);
   %if &nobs %then %do; 
      proc sql noprint;
         select distinct rbdecod
            into :RBOverlap separated by ' '
            from overlap
            order by rbdecod;
      quit;
   
      %put ERROR: RBDECOD needs to be distinct between subject-level indicators and site-level indicators.;
      %put ERROR: Overlap was detected for the following labels: &RBOverlap;
      %put ERROR: Modify site-level RBDECOD names in input SAS data set for risk-based monitoring to be distinct from subject-level RBDECOD;
      %let exiterror = 1;
      %goto gexit;          
   %end;
   
   proc datasets nolist;
      delete overlap subpairs sitepairs;
   quit;
     
   data rbm_rb;
      set ExLib.&ExData;      
      int = 1;
      if RBCAT = "&trAdverseEvents" then RBCAT = "&trSafety";
      if RBCAT = "&trManuallyEntered" then RBCAT = "&trSupplemental";      
      %if ^%sysfunc(findw(&RBList, RBFREQ)) %then %do;
         RBFREQ = 1;      
      %end;
      %else %do;
         if missing(RBFREQ) the RBFREQ = 1;
      %end;
      %if ^%sysfunc(findw(&RBList, RBENDTC)) %then %do;
         RBENDTC = RBSTDTC;
      %end;
      if lengthn(RBSTDTC) = 19 then 
         RBSTDT = dhms(mdy(substr(RBSTDTC, 6, 2), substr(RBSTDTC, 9, 2), substr(RBSTDTC, 1, 4)),substr(RBSTDTC,12,2),substr(RBSTDTC,15,2),substr(RBSTDTC,18,2));
      else if lengthn(RBSTDTC) = 16 then 
         RBSTDT = dhms(mdy(substr(RBSTDTC, 6, 2), substr(RBSTDTC, 9, 2), substr(RBSTDTC, 1, 4)),substr(RBSTDTC,12,2),substr(RBSTDTC,15,2),"00");
      else if lengthn(RBSTDTC) = 13 then 
         RBSTDT = dhms(mdy(substr(RBSTDTC, 6, 2), substr(RBSTDTC, 9, 2), substr(RBSTDTC, 1, 4)),substr(RBSTDTC,12,2),"00","00");
      else if lengthn(RBSTDTC) = 10 then 
         RBSTDT = dhms(mdy(substr(RBSTDTC, 6, 2), substr(RBSTDTC, 9, 2), substr(RBSTDTC, 1, 4)),"00","00","00");
      if lengthn(RBENDTC) = 19 then 
         RBENDT = dhms(mdy(substr(RBENDTC, 6, 2), substr(RBENDTC, 9, 2), substr(RBENDTC, 1, 4)),substr(RBENDTC,12,2),substr(RBENDTC,15,2),substr(RBENDTC,18,2));
      else if lengthn(RBENDTC) = 16 then 
         RBENDT = dhms(mdy(substr(RBENDTC, 6, 2), substr(RBENDTC, 9, 2), substr(RBENDTC, 1, 4)),substr(RBENDTC,12,2),substr(RBENDTC,15,2),"00");
      else if lengthn(RBENDTC) = 13 then 
         RBENDT = dhms(mdy(substr(RBENDTC, 6, 2), substr(RBENDTC, 9, 2), substr(RBENDTC, 1, 4)),substr(RBENDTC,12,2),"00","00");
      else if lengthn(RBENDTC) = 10 then 
         RBENDT = dhms(mdy(substr(RBENDTC, 6, 2), substr(RBENDTC, 9, 2), substr(RBENDTC, 1, 4)),"00","00","00");
      days = intck('dtday', RBSTDT, RBENDT);
   run;
   
   *** for time trends ***;
   %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
      data rbm_rb;
         set rbm_rb;   
         where "&date_lower" < RBSTDTC <= "&date_upper";
      run;
   %end;
   
   proc sort data = rbm_rb;
      by usubjid siteid;
   run;
   
   *** Generate totals per subject and variable ***;
   proc means nway missing noprint data = rbm_rb;
      class usubjid siteid variable rbdecod;
      var int;
      freq rbfreq;
      output out = rbm_rbt(drop = _TYPE_ _FREQ_) sum = sum;
   run;
   
   proc transpose data = rbm_rbt(where = (missing(usubjid))) out = rbm_rbt_site(drop = _name_);
      by siteid;
      var sum;
      id variable;
      idlabel rbdecod;
   run; 
         
   proc transpose data = rbm_rbt(where = (^missing(usubjid))) out = rbm_rbt(drop = _name_);
      by usubjid siteid;
      var sum;
      id variable;
      idlabel rbdecod;
   run;
    
   data rbm_rbt;
      merge rbm_dm(in = ina keep = usubjid siteid) rbm_rbt;
      by usubjid siteid;
      if ina;
      %let i = 1;
      %do %while(%length(%scan(&RBVariables_Subject,&i,' ')));      
         %let temp = %scan(&RBVariables_Subject,&i,' '); 
         if missing(&temp) then &temp = 0;         
         %let i = %eval(&i+1); 
      %end;   
   run;
   
   proc sql noprint;
      create table sites as
         select distinct siteid
         from rbm_dm
         order by siteid;
   quit;
      
   data rbm_rbt_site;
      merge sites(in = ina) rbm_rbt_site;
      by siteid;
      if ina;
      %let i = 1;
      %do %while(%length(%scan(&RBVariables_Site,&i,' ')));      
         %let temp = %scan(&RBVariables_Site,&i,' '); 
         if missing(&temp) then &temp = 0;         
         %let i = %eval(&i+1); 
      %end;   
   run;   
   
   proc datasets nolist;
      delete sites;
   quit;   
   
   *** Generate totals for overdue queries and CRF pages and response times ***;
   %if %sysfunc(findw(&RBVariables, QUERY)) or %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do;   
      proc means nway noprint data = rbm_rb;
         class usubjid variable rbdecod;
         var int;
         freq rbfreq;
         output out = rbm_rbo(drop = _TYPE_ _FREQ_) sum = sum;
         where variable in ("QUERY","CRFPAGE") and missing(RBENDTC);
      run;
      
      proc transpose data = rbm_rbo out = rbm_rbo(drop = _name_);
         by usubjid;
         var sum;
         id variable;
      run;

      %let _tid_ = %sysfunc(open(rbm_rbo));   
         %let QUERYind = %sysfunc(varnum(&_tid_,QUERY));
         %let CRFPAGEind = %sysfunc(varnum(&_tid_,CRFPAGE));
      %let _tid_ = %sysfunc(close(&_tid_));
      
      %if &QUERYind or &CRFPAGEind %then %let OVERDUEind = 1;
      %else  %let OVERDUEind = 0;

      data rbm_rbo;
         merge rbm_dm(in = ina keep = usubjid) rbm_rbo;
         by usubjid;
         if ina;
         %if %sysfunc(findw(&RBVariables, QUERY)) %then %do;
            if missing(query) then query = 0;
            rename query = OQUERY;
         %end;
         %if %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do;
            if missing(crfpage) then crfpage = 0;
            rename crfpage = OCRFPAGE;
         %end;       
      run;      
      
      proc means nway noprint data = rbm_rb;
         class usubjid variable rbdecod;
         var days;
         freq rbfreq;
         output out = rbm_rbr(drop = _TYPE_ _FREQ_) mean = mean;
         where variable in ("QUERY","CRFPAGE") and not missing(RBENDTC);
      run;
      
      proc transpose data = rbm_rbr out = rbm_rbr(drop = _name_);
         by usubjid;
         var mean;
         id variable;
      run;
      
      data rbm_rbr;
         merge rbm_dm(in = ina keep = usubjid) rbm_rbr;
         by usubjid;
         if ina;
         %if %sysfunc(findw(&RBVariables, QUERY)) %then %do;
            query = round(query, 0.01);
            rename query = RQUERY;           
         %end;
         %if %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do;
            crfpage = round(crfpage, 0.01);
            rename crfpage = RCRFPAGE;       
         %end;       
      run;
      
      data rbm_rbt;
         merge rbm_rbt rbm_rbr %if &OVERDUEind %then %do; rbm_rbo %end; ;
         by usubjid;
      run;
      
      proc datasets nolist;
         delete rbm_rbo rbm_rbr;
      quit;      
   %end;

   %let i = 1;
   %do %while(%length(%scan(&RBVariables,&i)));
      %let temp = %scan(&RBVariables,&i);
      %let templab = %scan(&RBLabels,&i,~);
      %if &temp = QUERY and &QUERYind %then %do;
         %let temp = QUERY OQUERY RQUERY;
         %let templab = &templab~&trOverdue &templab~&trAverageQueryResponseTime;
      %end;
      %else %if &temp = QUERY and ^&QUERYind %then %do;
         %let temp = QUERY RQUERY;
         %let templab = &templab~&trAverageQueryResponseTime;
      %end;
      %if &temp = CRFPAGE and &CRFPAGEind %then %do;
         %let temp = CRFPAGE OCRFPAGE RCRFPAGE;
         %let templab = &templab~&trOverdue &templab~&trAverageCRFResponseTime;
      %end;
      %else %if &temp = CRFPAGE and &CRFPAGEind %then %do;
         %let temp = CRFPAGE RCRFPAGE;
         %let templab = &templab~&trAverageCRFResponseTime;
      %end;

      %if &i = 1 %then %do;
         %let AllRBVariables = &temp; 
         %let AllRBLabels = &templab;
      %end;
      %else %do;
         %let AllRBVariables = &AllRBVariables &temp; 
         %let AllRBLabels = &AllRBLabels~&templab;    
      %end;
      %let i = &i + 1;
   %end;       

   %let i = 1;
   %do %while(%length(%scan(&RBVariables_Subject,&i)));
      %let temp = %scan(&RBVariables_Subject,&i);
      %let templab = %scan(&RBLabels_Subject,&i,~);
      %if &temp = QUERY and &QUERYind %then %do;
         %let temp = QUERY OQUERY RQUERY;
         %let templab = &templab~&trOverdue &templab~&trAverageQueryResponseTime;
      %end;
      %else %if &temp = QUERY and ^&QUERYind %then %do;
         %let temp = QUERY RQUERY;
         %let templab = &templab~&trAverageQueryResponseTime;
      %end;
      %if &temp = CRFPAGE and &CRFPAGEind %then %do;
         %let temp = CRFPAGE OCRFPAGE RCRFPAGE;
         %let templab = &templab~&trOverdue &templab~&trAverageCRFResponseTime;
      %end;
      %else %if &temp = CRFPAGE and &CRFPAGEind %then %do;
         %let temp = CRFPAGE RCRFPAGE;
         %let templab = &templab~&trAverageCRFResponseTime;
      %end;

      %if &i = 1 %then %do;
         %let AllRBVariables_Subject = &temp; 
         %let AllRBLabels_Subject = &templab;
      %end;
      %else %do;
         %let AllRBVariables_Subject = &AllRBVariables_Subject &temp; 
         %let AllRBLabels_Subject = &AllRBLabels_Subject~&templab;    
      %end;
      %let i = &i + 1;
   %end;
   
   proc datasets nolist;
      delete rbm_rb;
   quit;      
   
   data checkit;
      length VARIABLE $ 100;
      %let i = 1;
      %do %while(%length(%scan(&AllRBVariables,&i)));
         %let temp = %scan(&AllRBVariables,&i);
         variable = "&temp"; output;
         %let i = &i + 1;
      %end;   
   run;
   
   proc sort data = checkit;
      by variable;
   run;
   
   %SHRINKCHARS(dsn = checkit, vars = VARIABLE);
   
   proc sort data = WtPath.&RiskThresholdDataSet out = checkit2(keep = variable);
      by variable;
   run;
   
   data checkit2;
      merge checkit2(in = ina) checkit(in = inb);
      by variable;
      if inb and not ina;
   run;
   %nobsvars(checkit2);   
   %if &nobs %then %do;
      %put [MS_S_0165];
      %put ERROR: Variables detected in the supplemental SAS data set for RBM that are not contained in the selected Risk Threshold Data Set.;
      %put ERROR: Either update the selected Risk Threshold Data Set with variables from the selected supplemental data set using ;
      %put ERROR: Manage Risk Threshold Data Sets under the Settings Menu or choose another Risk Threshold Data Set that includes these variables;
      %let exiterror = 1;
      %goto gexit;            
   %end;
   
   proc datasets nolist;
      delete checkit checkit2;
   quit;
%end;

*** DS ***;
%LOADDATA(domlist = DS,mergesupp = 0,checkreview=0);
proc sort data = &in_ds out = rbm_ds(keep = &dslist);
   by usubjid &DSSTDTC;
   %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
      where "&date_lower" < &DSSTDTC <= "&date_upper";
   %end;
run;

*** AE ***;      

%if %sysfunc(findw(&domains, AE)) %then %do;   
   %LOADDATA(domlist = AE,mergesupp = 0,checkreview=0);
   proc sort data = &in_ae out = rbm_ae;
      by usubjid;
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         where "&date_lower" < &AESTDTC <= "&date_upper";
      %end;
   run;
   
   data rbm_ae(keep = &aelist);
      set rbm_ae;
      %if %length(%unquote(&AEWhereStmtRBM)) %then %do;
         where %unquote(&AEWhereStmtRBM);
      %end;
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0166];
      %put ERROR: The specified filter &AEWhereStmtRBM is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   
   data rbm_ae;
      set rbm_ae;
      int = 1;
   run;     
%end;

*** HO ***;      

%if %sysfunc(findw(&domains, HO)) %then %do;   
   %LOADDATA(domlist = HO,mergesupp = 0,checkreview=0);
   proc sort data = &in_ho out = rbm_ho;
      by usubjid;
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         where "&date_lower" < &HOSTDTC <= "&date_upper";
      %end;
   run;
   
   data rbm_ho(keep = &holist);
      set rbm_ho;
      %if %length(%unquote(&HOWhereStmtRBM)) %then %do;
         where %unquote(&HOWhereStmtRBM);
      %end;
   run;
   %if &syserr > 4 %then %do;
      %put [MS_S_0167];
      %put ERROR: The specified filter &HOWhereStmtRBM is invalid.  Change the filter to be valid for the domain you are considering.;
      %let exiterror = 1;
      %goto exit;
   %end; 
   
   data rbm_ho;
      set rbm_ho;
      int = 1;
   run;     
%end;

*** SV ***;

%if %sysfunc(findw(&domains, SV)) %then %do;
   %LOADDATA(domlist = SV,mergesupp = 0,checkreview=0);
   %put SVSTDTC = &SVSTDTC;
   %put SVENDTC = &SVENDTC;
   proc sort data = SV out = rbm_sv(keep = &svlist);
      by usubjid &SVSTDTC &SVENDTC;
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         %if %length(&SVSTDTC) %then %do;
            where "&date_lower" < &SVSTDTC <= "&date_upper";
         %end;
         %else %do;
            where "&date_lower" < &SVENDTC <= "&date_upper";      
         %end;
      %end;
   run;   
%end;

*** Findings ***;

%if %length(&findingsdomains) %then %do;
   %let i = 1;
   %do %while(%length(%scan(&findingsdomains,&i,' ')));
      %let temp = %scan(&findingsdomains,&i,' ');
      %CHECKPRES(domain = &temp, var = &temp.dtc); 
      %if %length(&&&temp.dtc) %then %do;
         %LOADDATA(domlist = &temp,mergesupp = 0,checkreview=0);
         proc sort data = &&in_&temp out = rbm_&temp(keep = usubjid &temp.dtc);
            by usubjid;
            %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
               where "&date_lower" < &&&temp.dtc <= "&date_upper";
            %end;
         run;

         data rbm_&temp(drop = &temp.dtc);
            set rbm_&temp;
            %DATE9(var=&temp.dtc,return=dt);
         run;
      %end;
      %let i = %eval(&i+1);
   %end;   
%end;

*** DV ***;

%if %sysfunc(findw(&domains, DV)) %then %do;
   %LOADDATA(domlist = DV,mergesupp = 0,checkreview=0);
   proc sort data = &in_dv out = rbm_dv;
      by usubjid;
      /*
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         where "&date_lower" < ?? <= "&date_upper";
      %end;
      */
   run;

   data rbm_dv(keep = &dvlist);
      set rbm_dv;
      %if %length(%unquote(&DVWhereStmtRBM)) %then %do;
         where (%unquote(&DVWhereStmtRBM)) and &DVTERM ne "";
      %end;
      %else %do;
         where &DVTERM ne "";
      %end;
   run;     

   data rbm_dv;
      set rbm_dv;
      int = 1;
   run;
%end;

*** IE ***;

%if %sysfunc(findw(&domains, IE)) %then %do;
   %LOADDATA(domlist = IE,mergesupp = 0,checkreview=0);
   proc sort data = &in_ie out = rbm_ie;
      by usubjid;
      /*
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         where "&date_lower" < ?? <= "&date_upper";
      %end;
      */
   run;

   data rbm_ie(keep = &ielist);
      set rbm_ie;
      %if %length(%unquote(&IEWhereStmtRBM)) %then %do;
         where %unquote(&IEWhereStmtRBM);
      %end;
   run;
   
   data rbm_ie;
      set rbm_ie;
      int = 1;
   run;     
%end;

*** EX ***;

%if %sysfunc(findw(&domains, EX)) %then %do;
   %LOADDATA(domlist = EX,mergesupp = 0,checkreview=0);
   proc sort data = &in_ex out = rbm_ex(keep = &exlist);
      by usubjid &EXSTDTC;
      %if %quote(&date_lower) ^= %quote(0000) or %quote(&date_upper) ^= %quote(9999) %then %do;
         where "&date_lower" < &EXSTDTC <= "&date_upper";
      %end;
   run;
%end; 

*** CO ***;

%if %sysfunc(findw(&domains, CO)) %then %do;
   %LOADDATA(domlist = CO,mergesupp = 0,checkreview=0);
   proc sort data = &in_co out = rbm_co(keep = usubjid &colist);
      by usubjid;
   run;
   
   data rbm_co;
      set rbm_co;
      covaltot = upcase(catx(" ", %sysfunc(transtrn(%str(&colist), %str( ), %str(,)))));
      drop &colist;
   run;
   
   data rbm_co(keep = usubjid codth);
      set rbm_co;
      by usubjid;
      retain codth;
      if first.usubjid and (index(covaltot, "DEATH") or index(covaltot, "DIED") or index(covaltot, "DEAD") or
         index(covaltot, "&utrDeath") or index(covaltot, "&utrDied") or index(covaltot, "&utrDead")) then codth = 1;
      else if first.usubjid then codth = 0;
      else if (index(covaltot, "DEATH") or index(covaltot, "DIED") or index(covaltot, "DEAD") or
         index(covaltot, "&utrDeath") or index(covaltot, "&utrDied") or index(covaltot, "&utrDead")) then codth = 1;
      if last.usubjid;
   run;
%end;

*** DD ***;

%if %sysfunc(findw(&domains, DD)) %then %do;
   %LOADDATA(domlist = DD,mergesupp = 0,checkreview=0);
   proc sort data = &in_dd out = rbm_dd(keep = usubjid);
      by usubjid;
   run;
   
   data rbm_dd;
      set rbm_dd;
      by usubjid;
      if first.usubjid;
   run;   
%end; 

****************************;
*** Flags and Indicators ***;
****************************;

*** Randomized ***;   
data rbm_rand(keep = usubjid);
   set rbm_ds(where = (index(upcase(&DSDECOD), "RANDOMIZED") or index(upcase(DSDECOD), "RANDOMISED") or index(upcase(&DSDECOD), "&utrRandomized")));
   by usubjid;
   if last.usubjid;
run;

*** Screen Fail ***;
%if %length(&DSEPOCH) and %length(&DSCAT) %then %do;
   data rbm_scrnfail(keep = usubjid);
      set rbm_ds(where = ((upcase(&DSEPOCH) = "SCREENING" or upcase(&DSEPOCH) = "&utrScreening") and 
                         (upcase(&DSCAT) = "DISPOSITION EVENT" or upcase(&DSCAT) = "&utrDispositionEvent") and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      by usubjid;
      if last.usubjid;
   run;
%end;
%else %if %length(&DSEPOCH) %then %do;
   data rbm_scrnfail(keep = usubjid);
      set rbm_ds(where = ((upcase(&DSEPOCH) = "SCREENING" or upcase(&DSEPOCH) = "&utrScreening") and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      by usubjid;
      if last.usubjid;
   run;
%end;
%else %do;
   data rbm_scrnfail(keep = usubjid);
      set rbm_ds(obs = 1);
      if _n_ = 1 then delete;
   run;
%end;

*** Discontinuations ***;

data rbm_disc(keep = usubjid discreas);      
   %if %length(%unquote(&CompWhereStmt)) %then %do;
      set rbm_ds;
      where (%unquote(&CompWhereStmt)) and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted");
   %end;
   %else %do;
      %if %length(&DSEPOCH) and %length(&DSCAT) %then %do;
         set rbm_ds(where = ((index(upcase(&DSEPOCH),"TREATMENT") or index(upcase(&DSEPOCH),"&utrTreatment")) and 
                   (upcase(&DSCAT) = "DISPOSITION EVENT" or upcase(&DSCAT) = "&utrDispositionEvent") and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      %end;
      %else %if %length(&DSEPOCH) %then %do;
         set rbm_ds(where = ((index(upcase(&DSEPOCH),"TREATMENT") or index(upcase(&DSEPOCH),"&utrTreatment")) and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      %end;       
      %else %if %length(&DSCAT) %then %do;
         set rbm_ds(where = ((upcase(&DSCAT) = "DISPOSITION EVENT" or upcase(&DSCAT) = "&utrDispositionEvent") and (upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      %end;
      %else %do;
         set rbm_ds(where = ((upcase(&DSDECOD) ne "COMPLETED" and upcase(&DSDECOD) ne "&utrCompleted")));
      %end;      
   %end;
   by usubjid &DSSTDTC;
   if last.usubjid;
   rename dsdecod = discreas;
   label discreas = "&trDiscontinuationReason";
run;

*** Completers ***;

data rbm_comp(keep = usubjid);      
   %if %length(%unquote(&CompWhereStmt)) %then %do;
      set rbm_ds;
      where (%unquote(&CompWhereStmt)) and (upcase(&DSDECOD) = "COMPLETED" or upcase(&DSDECOD) = "&utrCompleted");
   %end;
   %else %do;
      %if %length(&DSEPOCH) and %length(&DSCAT) %then %do;
         set rbm_ds(where = ((index(upcase(&DSEPOCH),"TREATMENT") or index(upcase(&DSEPOCH),"&utrTreatment")) and 
                   (upcase(&DSCAT) = "DISPOSITION EVENT" or upcase(&DSCAT) = "&utrDispositionEvent") and (upcase(&DSDECOD) = "COMPLETED" or upcase(&DSDECOD) = "&utrCompleted")));
      %end;
      %else %if %length(&DSEPOCH) %then %do;
         set rbm_ds(where = ((index(upcase(&DSEPOCH),"TREATMENT") or index(upcase(&DSEPOCH),"&utrTreatment")) and (upcase(&DSDECOD) = "COMPLETED" or upcase(&DSDECOD) = "&utrCompleted")));
      %end;       
      %else %if %length(&DSCAT) %then %do;
         set rbm_ds(where = ((upcase(&DSCAT) = "DISPOSITION EVENT" or upcase(&DSCAT) = "&utrDispositionEvent") and (upcase(&DSDECOD) = "COMPLETED" or upcase(&DSDECOD) = "&utrCompleted")));
      %end;
      %else %do;
         set rbm_ds(where = ((upcase(&DSDECOD) = "COMPLETED" or upcase(&DSDECOD) = "&utrCompleted")));
      %end;      
   %end;
   by usubjid &DSSTDTC;
   if last.usubjid;
run;

*** Treated ***;
%if %sysfunc(findw(&domains, EX)) %then %do;
   data rbm_ex(keep = usubjid);     
      set rbm_ex;
      by usubjid &EXSTDTC;
      if last.usubjid;
   run;
%end;

*** Patient Weeks on Trial ***;

%if %length(&SVSTDTC) %then %do;
   data rbm_patwk;
      set rbm_sv;
      date = SVSTDTC; output;
      drop SVSTDTC;
      %if %length(&SVENDTC) %then %do;
         date = SVENDTC; output;
         drop SVENDTC;
      %end;
   run;

   proc sort data = rbm_patwk out = rbm_patwk(where = (date ne ""));
      by usubjid date;
   run;

   data rbm_patwk(keep = usubjid first last);
      set rbm_patwk(drop = visit);         
      by usubjid;
      retain first last;
      %DATE9(var=date,return=dt);
      drop date;
      if first.usubjid then first = dt;
      if last.usubjid then last = dt;
      format first last &IntDateFormat..;
      if last.usubjid;
   run;      
%end;
%else %if %length(&RBMFINDLIST) %then %do;
   data rbm_patwk;
      set
      %let i = 1;
      %do %while(%length(%scan(&RBMFINDLIST,&i,' ')));
         %let temp = %scan(&RBMFINDLIST,&i,' ');
            rbm_&temp
         %let i = %eval(&i+1);
      %end;
      ;
   run;        

   proc sort data = rbm_patwk out = rbm_patwk(where = (dt > .));
      by usubjid dt;
   run;

   data rbm_patwk(keep = usubjid first last);
      set rbm_patwk;         
      by usubjid;
      retain first last;
      if first.usubjid then first = dt;
      if last.usubjid then last = dt;
      format first last &IntDateFormat..;
      if last.usubjid;
   run;            

   proc datasets nolist;
      delete
      %let i = 1;
      %do %while(%length(%scan(&RBMFINDLIST,&i,' ')));
         %let temp = %scan(&RBMFINDLIST,&i,' ');
            %if &temp ne IE %then %do;            
               rbm_&temp
            %end;
         %let i = %eval(&i+1);
      %end;
      ;
   quit;      
%end;

*** Adverse events ***;

%if %sysfunc(findw(&domains, AE)) %then %do;
   *** SAE ***;
   %if %length(&AESER) %then %do;
      data rbm_sae;
         set rbm_ae(where = (upcase(&AESER) in ("Y","YES","&utrY","&utrYes") and &AETERM ne ""));        
      run;      
   %end;
   %else %do; 
      data rbm_sae;
         set rbm_ae(obs = 1);
         if _n_ = 1 then delete;
      run;   
   %end;

   *** Related SAE ***;
   %if %length(&AEREL) %then %do;
      data rbm_rsae;
         set rbm_sae(where = (upcase(&AEREL) not in 
         ("N","NO","NOT RELATED","NOT","UNRELATED","UNLIKELY","UNLIKELY RELATED")
         and upcase(&AEREL) not in ("&utrN","&utrNo","&utrNotRelated","&utrNot","&utrUnrelated","&utrUnlikely","&utrUnlikelyRelated") and &AETERM ne ""));        
      run;      
   %end;
   %else %do; 
      data rbm_rsae;
         set rbm_sae(obs = 1);
         if _n_ = 1 then delete;
      run;   
   %end; 

   *** Related AE ***;
   %if %length(&AEREL) %then %do;
      data rbm_rae;
         set rbm_ae(where = (upcase(&AEREL) not in 
         ("N","NO","NOT RELATED","NOT","UNRELATED","UNLIKELY","UNLIKELY RELATED")
         and upcase(&AEREL) not in ("&utrN","&utrNo","&utrNotRelated","&utrNot","&utrUnrelated","&utrUnlikely","&utrUnlikelyRelated") and &AETERM ne ""));        
      run;      
   %end;
   %else %do; 
      data rbm_rae;
         set rbm_ae(obs = 1);
         if _n_ = 1 then delete;
      run;   
   %end;

   *** Fatal AE ***; 
   %if %length(&AEOUT) or %length(&AESDTH) %then %do;
      data rbm_aedie(keep = usubjid);
         %if %length(&AEOUT) and %length(&AESDTH) %then %do;
            set rbm_ae(where = (upcase(&AEOUT) = "FATAL" or index(upcase(&AEOUT), "DEATH") or kindex(upcase(&AEOUT), "&utrDeath") or upcase(&AESDTH) in ("Y","YES","&utrY","&utrYes")));
         %end;
         %else %if %length(&AEOUT) %then %do;
            set rbm_ae(where = (upcase(&AEOUT) = "FATAL" or index(upcase(&AEOUT), "DEATH") or kindex(upcase(&AEOUT), "&utrDeath")));
         %end;
         %else %if %length(&AESDTH) %then %do;
            set rbm_ae(where = (upcase(&AESDTH) in ("Y","YES","&utrY","&utrYes")));
         %end;
         by usubjid;
         if last.usubjid;
      run;
   %end;
   %else %do; 
      data rbm_aedie(keep = usubjid);
         set rbm_ae(obs = 1);          
         if _n_ = 1 then delete;        
      run;   
   %end;

   proc means nway noprint data = rbm_ae;
      class usubjid;
      var int;
      output out = rbm_ae(drop = _TYPE_ _FREQ_) sum = totae;
   run;

   proc means nway noprint data = rbm_rae;
      class usubjid;
      var int;
      output out = rbm_rae(drop = _TYPE_ _FREQ_) sum = totrae;
   run;

   proc means nway noprint data = rbm_sae;
      class usubjid;
      var int;
      output out = rbm_sae(drop = _TYPE_ _FREQ_) sum = totsae;
   run;

   proc means nway noprint data = rbm_rsae;
      class usubjid;
      var int;
      output out = rbm_rsae(drop = _TYPE_ _FREQ_) sum = totrsae;
   run;           
%end;

*** Hospitalizations ***;

%if %sysfunc(findw(&domains, HO)) %then %do;
   proc means nway noprint data = rbm_ho;
      class usubjid;
      var int;
      output out = rbm_ho(drop = _TYPE_ _FREQ_) sum = totho;
   run;
%end;

*** Deviations ***;
%if %sysfunc(findw(&domains, DV)) %then %do;
   proc means nway noprint data = rbm_dv;
      class usubjid;
      var int;
      output out = rbm_dv(drop = _TYPE_ _FREQ_) sum = totdv;
   run;     
%end;

*** Inclusion-Exclusion ***;
%if %sysfunc(findw(&domains, IE)) %then %do;
   proc means nway noprint data = rbm_ie;
      class usubjid;
      var int;
      output out = rbm_ie(drop = _TYPE_ _FREQ_) sum = totie;
   run;     
%end;

****************************;
*** Put Together Data    ***;
****************************;

 data rbm_jmpc;
    merge rbm_dm(in = ina)
       rbm_rand(in = inrand)
       rbm_scrnfail(in = insf)
       rbm_comp(in = incomp)
       rbm_disc(in = indisc)
       %if %sysfunc(findw(&domains, EX)) %then %do; rbm_ex(in = inex) %end;
       %if %length(&SVSTDTC) or %length(&RBMFINDLIST) %then %do; rbm_patwk(in = inpatwk) %end;
       %if %sysfunc(findw(&domains, AE)) %then %do;
          rbm_ae(in = inae)
          /* rbm_rae(in = inrae) */
          rbm_sae(in = insae)
          /* rbm_rsae(in = inrsae) */
          rbm_aedie(in = inaedie)
       %end;
       %if %sysfunc(findw(&domains, HO)) %then %do; rbm_ho(in = inho) %end;   
       %if %sysfunc(findw(&domains, DV)) %then %do; rbm_dv(in = indv) %end;   
       %if %sysfunc(findw(&domains, IE)) %then %do; rbm_ie(in = inie) %end;
       %if %sysfunc(findw(&domains, CO)) %then %do; rbm_co(in = inco) %end;
       %if %sysfunc(findw(&domains, DD)) %then %do; rbm_dd(in = indd) %end;
       %if %length(&ExData) %then %do; rbm_rbt(in = inrb) %end;
       ;
    by usubjid;
    if ina;
    aedth = 0;
    label aedth = "&trAEOutcomeofFatalPresent";
    %if %length(&SVSTDTC) or %length(&RBMFINDLIST) %then %do;
      patweeks = (last - first + 1) / 7;
      label patweeks = "&trPatientWeeksonStudy";
    %end;
    %if %sysfunc(findw(&domains, AE)) %then %do;
       if missing(totae) then totae = 0;
       /* if missing(totrae) then totrae = 0; */
       if missing(totsae) then totsae = 0;
       /* if missing(totrsae) then totrsae = 0; */
       if inaedie then aedth = 1;
       else aedth = 0;
       label totae = "&trTotalAEsonStudy" totsae = "&trTotalSAEsonStudy" /* totrae = "Total Related AEs on Study" totrsae = "Total Related SAEs on Study" */;
    %end;
    %if %sysfunc(findw(&domains, HO)) %then %do;
       if missing(totho) then totho = 0;
       label totho = "&trTotalHealthcareEncounters";
    %end;    
    %if %sysfunc(findw(&domains, DV)) %then %do;
       if missing(totdv) then totdv = 0;
       label totdv = "&trTotalProtocolDeviations";
    %end;
    %if %sysfunc(findw(&domains, CO)) %then %do;
       if missing(codth) then codth = 0;
       label codth = "&trDeathReportedinCommentsDomain";
       %let CODTH = CODTH;
    %end;
    %else %do;
       %let CODTH =;
    %end;
    %if %sysfunc(findw(&domains, IE)) %then %do;
       if missing(totie) then totie = 0;
       label totie = "&trTotalInclusionExclusionNotMet";
    %end;
    %if %sysfunc(findw(&domains, EX)) %then %do; 
       if inex then trtex = 1;
       else trtex = 0;
       label trtex = "&trSubjectDosedPerEXDomain";
    %end;
    if inrand then rand = 1;
    else rand = 0;

    *** Determine Screen Failures ***;
    scrnfail = 0;
    %if %length(&ARM) and %length(&ACTARM) %then %do;
       if ((upcase(&ARM) = "SCREEN FAILURE" or upcase(&ARM) = "&utrScreenFailure") or (upcase(&ACTARM) = "SCREEN FAILURE" or upcase(&ACTARM) = "&utrScreenFailure") or insf) then scrnfail = 1;
    %end;    
    %else %if %length(&ARM) %then %do;
       if ((upcase(&ARM) = "SCREEN FAILURE" or upcase(&ARM) = "&utrScreenFailure") or insf) then scrnfail = 1;
    %end;
    %else %if %length(&ACTARM) %then %do;
       if ((upcase(&ACTARM) = "SCREEN FAILURE" or upcase(&ACTARM) = "&utrScreenFailure") or insf) then scrnfail = 1;
    %end;
    %else %if %length(&DSEPOCH) %then %do;
       if insf then scrnfail = 1;
    %end;
    %else %do;
       if rand = 0 then scrnfail = 1;
    %end;
    
    *** Determine Not Treats ***;
    treated = 0;
    %if %length(&RFXSTDTC) %then %do;
       if &RFXSTDTC ne "" then treated = 1;
    %end;
    %else %if %sysfunc(findw(&domains, EX)) %then %do;    
       if trtex = 1 then treated = 1;
    %end;
    %else %if %length(&ACTARM) %then %do;
       if upcase(&ACTARM) not in ("SCREEN FAILURE","NOT TREATED","NOT ASSIGNED","&utrScreenFailure","&utrNotTreated","&utrNotAssigned") then treated = 1;   
    %end;
    %else %if %length(&ARM) %then %do;
       if upcase(&ARM) not in ("SCREEN FAILURE","NOT TREATED","NOT ASSIGNED","&utrScreenFailure","&utrNotTreated","&utrNotAssigned") then treated = 1;   
    %end;
    %else %do;
       if rand = 1 then treated = 1; 
    %end;

    *** Consent ***;
    %if %length(&RFICDTC) %then %do;
      if &RFICDTC ne "" then consent = 1;
      else consent = 0;
      if &RFICDTC ne "" then missconsent = 0;
      else missconsent = 1; 
      label missconsent = "&trMissingInformedConsent" consent = "&trSignedConsent";
    %end;
    
    %if %sysfunc(findw(&domains, DD)) %then %do;
       if indd then deathdom = 1;
       else deathdom = 0;
       label deathdom = "&trDeathDetails";
    %end;

    *** Death ***;
    %if %length(&DTHDTC) and %length(&DTHFL) and %length(&CODTH) %then %do;
       if &DTHDTC ne "" or upcase(&DTHFL) in ("YES","Y","&utrY","&utrYes") or aedth or codth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;
    %else %if %length(&DTHDTC) and %length(&DTHFL) %then %do;
       if &DTHDTC ne "" or upcase(&DTHFL) in ("YES","Y","&utrY","&utrYes") or aedth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;
    %else %if %length(&CODTH) and %length(&DTHFL) %then %do;
       if upcase(&DTHFL) in ("YES","Y","&utrY","&utrYes") or aedth or codth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;
    %else %if %length(&DTHDTC) and %length(&CODTH) %then %do;
       if &DTHDTC ne "" or aedth or codth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;    
    %else %if %length(&DTHDTC) %then %do;
       if &DTHDTC ne "" or aedth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;
    %else %if %length(&DTHFL) %then %do;
       if upcase(&DTHFL) in ("YES","Y","&utrY","&utrYes") or aedth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;
    %else %if %length(&CODTH) %then %do;
       if aedth or codth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;    
    %else %do;
       if aedth or upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then death = 1;
       else death = 0;
    %end;    

    %if %sysfunc(findw(&domains, DD)) %then %do;
       if deathdom = 1 then death = 1;            
    %end;
    
    if upcase(discreas) in ("DEATH","DIED","DEAD","&utrDeath","&utrDied","&utrDead") then discdeath = 1;
    else discdeath = 0;

    *** LTFU ***;
    if upcase(discreas) in ("LOST TO FOLLOW-UP","LOST TO FOLLOWUP","LOST TO FOLLOW UP","LTFU","&utrLosttoFollowup") then ltfu = 1;
    else ltfu = 0;
    
    *** DC to AE ***;
    if upcase(discreas) in ("ADVERSE EVENT","AE","&utrAdverseEvent") then dcae = 1;
    else dcae = 0;
       
    *** Withdrew Consent ***;
    if upcase(discreas) in ("WITHDRAWAL BY SUBJECT","SUBJECT WITHDRAWAL","WITHDREW CONSENT","SUBJECT WITHDREW CONSENT")
       or upcase(discreas) in ("&utrWithdrawalbySubject","&utrSubjectWithdrawal","&utrWithdrewConsent","&utrSubjectWithdrewConsent") then withdrew = 1;
    else withdrew = 0;

    *** DC OTHER ***;
    if not missing(discreas) and not discdeath and not ltfu and not dcae and not withdrew then dcoth = 1;
    else dcoth = 0;
       
    if indisc then discon = 1;
    else if rand then discon = 0;

    if incomp then complete = 1;
    else if rand then complete = 0;       
    
    if rand and not(discon) and not(complete) then ongoing = 1;
    else if rand then ongoing = 0;
    label rand = "&trRandomized" scrnfail = "&trScreenFailure" discon = "&trDiscontinued" complete = "&trCompleted" ongoing = "&trOngoing" 
          treated = "&trTreated" death = "&trDeath" ltfu = "&trLosttoFollowup" dcae = "&trDiscontinuedduetoAE" discdeath = "&trDiscontinuedduetoDeath" withdrew = "&trPatientWithdrewfromStudy"
          dcoth = "&trDiscontinuedOther";
    %if %length(&ExData) %then %do;
      %let i = 1;
      %do %while(%length(%scan(&AllRBVariables,&i)));
         %let temp = %scan(&AllRBVariables,&i);
         %let templab = %scan(&AllRBLabels,&i,~);
         label &temp = "&templab";
         %let i = &i + 1;
      %end;                   
    %end;                      
run;

proc datasets nolist;
   delete rbm_rand
       rbm_scrnfail
       rbm_comp
       rbm_disc
       %if %sysfunc(findw(&domains, EX)) %then %do; rbm_ex %end;
       %if %length(&SVSTDTC) or %length(&RBMFINDLIST) %then %do; rbm_patwk %end;
       %if %sysfunc(findw(&domains, AE)) %then %do;
          rbm_ae
          rbm_sae
          rbm_aedie
       %end;
       %if %sysfunc(findw(&domains, HO)) %then %do; rbm_ho %end;   
       %if %sysfunc(findw(&domains, DV)) %then %do; rbm_dv %end;   
       %if %sysfunc(findw(&domains, IE)) %then %do; rbm_ie %end;
       %if %length(&ExData) %then %do; rbm_rbt %end;
       ;
quit;

*** This date is used to calculate enrollment performance ***;
proc means noprint data = rbm_jmpc;
   var last;
   output out = maxdate max = maxlast;
run;

data maxdate;
   set maxdate;
   call symput("MaxLastDate", trim(left(maxlast)));
run;

proc datasets nolist;
   delete maxdate;
quit;

data rbm_jmpc;
   set rbm_jmpc(drop = first last);
   int = 1;
run;

%if %sysfunc(fileexist(&NotesDir.rbm_site.sas7bdat)) %then %do;
   proc sort data = NotesLib.rbm_site(keep = Siteid Country City StateProvince ZipCode Latitude Longitude SiteActive Monitor SiteCategory Vendor) out = rbm_site;
      by siteid &country;
   run;   
%end;
%else %do;
   proc sort nodupkey data = rbm_dm out = rbm_site(keep = siteid &country);
      by siteid &country;
   run;
%end;
%if %length(&QSiteList) or %length(&QSiteCatList) %then %do;
   data rbm_site;
    set rbm_site;
      where %unquote(&SiteWhere);
   run;
%end;

%if %sysfunc(exist(rbm_rbt_site)) %then %do;
   data rbm_site;
      merge rbm_site(in = ina) rbm_rbt_site;
      by siteid;
      if ina;
   run;

   proc datasets nolist;
      delete rbm_rbt_site;
   quit;
%end;
   
proc sort data = WtPath.&RiskThresholdDataSet out = RiskThreshold(drop = comment);
   by variable;
run; 

*** apply recode of categories ***;
data RiskThreshold;
   set RiskThreshold;
   if upcase(variable) = "OVERALLA" then do;
      variable = "OVERALLS";
      label = "&trOverallRiskIndicatorSafe";      
   end;
   if upcase(variable) = "OVERALLM" then do;
      variable = "OVERALLX";
      label = "&trOverallRiskIndicatorSupp";      
   end;
   
   *** drop old variables ***;
   if variable in ("TOTALQUERIES","AVTOTQ","PWTOTQ","OVERDUEQUERIES","AVOVRQ","PWOVERQ","MISSPAGE","AVMISSPAGE","PWMISSPAGE","COMPDEVIATE","AVCOMPDEV","PWCOMPDEV","QUERYRESP","CRFENTRYRESP") then delete;
run;

/* kemicl - add check for lengths of action columns */
%let yellen = ;
%let redlen = ;
%VarAttributes(RiskThreshold,yelact,_Len=yellen);
%VarAttributes(RiskThreshold,redact,_Len=redlen);
%let ActionLen =  %sysfunc(max(&yellen, &redlen));
/* kemicl - end check for lengths */
 
/* kemicl - check if there are any actions defined and create flag */
data noactcheck;
   set RiskThreshold;
   where not missing(yelact) or not missing(redact);
run;

%nobsvars(noactcheck);
%let NoActFlg = 1;
%if &nobs %then %let NoActFlg = 0;

%let AllAnalVars1 = RAND SCRNFAIL MISSCONSENT TREATED COMPLETE ONGOING DISCON DCAE LTFU WITHDREW PATWEEKS DEATH DISCDEATH TOTAE TOTSAE TOTHO TOTIE TOTDV;
%let AllAnalVars2 = SITEACTIVE &AllRBVariables;
%let AllAnalVars = &AllAnalVars1 &AllAnalVars2; 
%let SiteAnalVars = SITEACTIVE &AllRBVariables_Site; 
%let AllLocVars = COUNTRY CITY STATEPROVINCE ZIPCODE LATITUDE LONGITUDE MONITOR SITECATEGORY VENDOR;   

%macro CALCSUMS(level = SITEID);
   proc means sum nway noprint data = rbm_jmpc;
      class &level;
      var int rand scrnfail treated complete ongoing discon dcae ltfu withdrew patweeks death discdeath dcoth
         %if %sysfunc(findw(&domains, AE)) %then %do;
            totae /* totrae totrsae */
            %if %length(&AESER) %then %do; totsae %end;
         %end;
         %if %sysfunc(findw(&domains, HO)) %then %do;
            totho 
         %end;       
         %if %length(&RFICDTC) %then %do;
            missconsent
         %end;
         %if %length(&ExData) %then %do;
            &RBVariables_Subject %if &QUERYind %then %do; OQUERY %end; %if &CRFPAGEind %then %do; OCRFPAGE %end;
         %end;
      ;
      output out = sumstat(drop = _type_ _freq_) sum = total rand scrnfail treated complete ongoing discon dcae ltfu withdrew patweeks death discdeath dcoth
         %if %sysfunc(findw(&domains, AE)) %then %do;
            totae /* totrae totrsae */
            %if %length(&AESER) %then %do; totsae %end;
         %end;
         %if %sysfunc(findw(&domains, HO)) %then %do;
            totho 
         %end;       
         %if %length(&RFICDTC) %then %do;
            missconsent
         %end;
         %if %length(&ExData) %then %do;
            &RBVariables_Subject %if &QUERYind %then %do; OQUERY %end; %if &CRFPAGEind %then %do; OCRFPAGE %end;
         %end;       
      ;
   run;   
   
   *** Deviations and IE not met for Randomized Subjects ***;
   %if %sysfunc(findw(&domains, IE)) or %sysfunc(findw(&domains, DV)) %then %do;   
      proc means sum nway noprint data = rbm_jmpc;
         class &level;
         var 
            %if %sysfunc(findw(&domains, IE)) %then %do;
               totie
            %end;
            %if %sysfunc(findw(&domains, DV)) %then %do;
               totdv
            %end;
         ;
         output out = sumstat2(drop = _type_ _freq_) sum = 
            %if %sysfunc(findw(&domains, IE)) %then %do;
               totie
            %end;
            %if %sysfunc(findw(&domains, DV)) %then %do;
               totdv
            %end;
         ;
         where rand = 1;
      run; 

      data sumstat;
         merge sumstat sumstat2;
         by &level;
      run;
   
      proc datasets nolist;
         delete sumstat2;
      quit; 
   %end;

   *** Averages for Query or CRFPage times from external data set ***;  
   %if %sysfunc(findw(&RBVariables, QUERY)) or %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do;
      proc means mean nway noprint data = rbm_jmpc;
         class &level;
         var 
         %if %sysfunc(findw(&RBVariables, QUERY)) %then %do; RQUERY %end;
         %if %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do; RCRFPAGE %end;
         ;
         output out = sumstat3(drop = _type_ _freq_) 
                mean = 
         %if %sysfunc(findw(&RBVariables, QUERY)) %then %do; RQUERY %end;
         %if %sysfunc(findw(&RBVariables, CRFPAGE)) %then %do; RCRFPAGE %end;
         ;
      run;

      data sumstat;
         merge sumstat sumstat3;
         by &level;
      run;
   
      proc datasets nolist;
         delete sumstat3;
      quit;       
   %end; 
      
   %if &level = SITEID %then %do;
      %global AvailAnalVars DropVars AvailLocVars;   
  
      data sumstat;
         merge sumstat rbm_site;
         by &level;
      run;         
      %NONNULLVARS(dataset = sumstat, varlistname = RBMVARS);
      %let AvailAnalVars = ;
      %let DropVars = ;
      %let i = 1;
      %do %while(%length(%scan(&AllAnalVars,&i,' ')));
         %let temp = %scan(&AllAnalVars,&i,' ');
         %if %sysfunc(findw(&RBMVARS, %upcase(&temp))) %then %let AvailAnalVars = &AvailAnalVars &temp;
         %else %if %sysfunc(findw(&SiteAnalVars, %upcase(&temp))) %then %let DropVars = &DropVars &temp;
         %let i = %eval(&i+1);
      %end;
      %let AvailLocVars = ;
      %let i = 1;
      %do %while(%length(%scan(&AllLocVars,&i,' ')));
         %let temp = %scan(&AllLocVars,&i,' ');
         %if %sysfunc(findw(&RBMVARS, %upcase(&temp))) %then %let AvailLocVars = &AvailLocVars &temp;
         %else %let DropVars = &DropVars &temp;
         %let i = %eval(&i+1);
      %end;
   %end;
   %else %if &level = COUNTRY or &level = MONITOR or &level = VENDOR %then %do;
      data rbm_site;
         set rbm_site;
         site = 1;
      run;
   
      proc means sum nway noprint data = rbm_site;
         class &level;
         var site
         %let i = 1;
         %do %while(%length(%scan(&RBVariables_Site,&i,' ')));
            %let temp = %scan(&RBVariables_Site,&i,' ');   
            %if %sysfunc(findw(&AvailAnalVars, &temp)) %then %do; &temp %end;
            %let i = %eval(&i+1);
         %end;       
         ;
         output out = rbm_site_totals(drop = _type_ _freq_) 
                sum = TotalSites
         %let i = 1;
         %do %while(%length(%scan(&RBVariables_Site,&i,' ')));
            %let temp = %scan(&RBVariables_Site,&i,' ');   
            %if %sysfunc(findw(&AvailAnalVars, &temp)) %then %do; &temp %end;
            %let i = %eval(&i+1);
         %end;       
         ;
      run;
            
      data sumstat;
         merge sumstat rbm_site_totals;         
         by &level;
      run;   

      proc datasets nolist;
         delete rbm_site_totals;
      quit;    
   %end;
      
   data sumstat;
      set sumstat;
      pctcomp = complete / rand * 100;
      pctongoing = ongoing / rand * 100;
      pctdiscon = discon / rand * 100;
      pctdeath = death / rand * 100;
      pctscrnfail = scrnfail / total * 100;
      patweeks = patweeks;
      pwdeath = death / patweeks;
      format pctcomp pctongoing pctdiscon pctdeath pctscrnfail 8.1 patweeks pwdeath 8.2;
      label pctcomp = "&trPercentCompletedofRandSubjs" pctongoing = "&trPercentOngoingofRandSubjs" pctdiscon = "&trPercentDiscontofRandSubjs"
            pctdeath = "&trPercentDeathsofRandSubjs" pwdeath = "&trDeathsperPatientWeek" total = "&trTotalSubjects" pctscrnfail = "&trPercentScreenFailofTotalSubjs";
      %if %sysfunc(findw(&domains, DV)) %then %do;
         avgdv = totdv / rand;
         label avgdv = "&trAverageDeviationsperRandSubj";
         format avgdv 8.2; 
      %end;
      %if %sysfunc(findw(&domains, IE)) %then %do;
         avgie = totie / rand;
         label avgie = "&trAverageInExNotMetperRandSubj";
         format avgie 8.2;
      %end;
      %if %sysfunc(findw(&domains, AE)) %then %do;
         avgae = totae / rand;
         pwae = totae / patweeks;
         format avgae pwae 8.2;
         /* %if %length(&AEREL) %then %do;
            avgrae = round(totrae / rand, 0.1);
            pwrae = round(totrae / patweeks, 0.1);
         %end; */
         %if %length(&AESER) %then %do;
            avgsae = totsae / rand;
            pwsae = totsae / patweeks;
            format avgsae pwsae 8.2;
         %end;
         /* %if %length(&AESER) and %length(&AEREL) %then %do;
            avgrsae = round(totrsae / rand, 0.1);
            pwrsae = round(totrsae / patweeks, 0.1);
         %end; */
         label avgae = "&trAverageAEsperRandSubj";
         /*%if %length(&AEREL) %then %do;
            label avgrae = "Average Related AEs per Randomized Subject";
         %end; */
         %if %length(&AESER) %then %do;
            label avgsae = "&trAverageSAEsperRandSubj";
         %end;
         /* %if %length(&AESER) and %length(&AEREL) %then %do;
            label avgrsae = "Average Related SAEs per Randomized Subject";
         %end; */
         label pwae = "&trAEsperPatientWeek"; 
         /* %if %length(&AEREL) %then %do;
            label pwrae = "Related AEs per PatientWeek";
         %end; */
         %if %length(&AESER) %then %do;
            label pwsae = "&trSAEsperPatientWeek";
         %end;
         /*%if %length(&AESER) and %length(&AEREL) %then %do;
            label pwrsae = "Related SAEs per PatientWeek";
         %end;*/
      %end;
      %if %sysfunc(findw(&domains, HO)) %then %do;
         avgho = totho / rand;
         pwho = totho / patweeks;
         format avgho pwho 8.2;
         label avgho = "&trHealthcareEncountersAV";
         label pwho = "&trHealthcareEncountersPW"; 
      %end;    
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) and &level = SITEID %then %do;
         WeeksActive = (&MaxLastDate - SITEACTIVE + 1) / 7;
         label WeeksActive = "&trWeeksActive";
         format weeksactive 8.2;
      %end;    
      %if &level = COUNTRY or &level = MONITOR or &level = VENDOR %then %do;
         label TotalSites = "&trTotalSites";
      %end;
      *** External Data Source ***;     
      %if %length(&ExData) %then %do;
         %let i = 1;
         %do %while(%length(%scan(&AllRBVariables,&i)));
            %let temp = %scan(&AllRBVariables,&i);
            %let templab = %scan(&AllRBLabels,&i,~);
            %if &temp ^= RQUERY and &temp ^= RCRFPAGE %then %do;
               %let trLTPRS = %nrbquote(%LocaleFill(&trXperRandomizedSubject, f1=%nrbquote(&templab)));
               %let trLTPPW = %nrbquote(%LocaleFill(&trXperPatientWeek, f1=%nrbquote(&templab)));
               AV&temp = &temp / rand;
               PW&temp = &temp / patweeks;
               label &temp = "&templab";
               label av&temp = "&trLTPRS";
               label pw&temp = "&trLTPPW";
               format av&temp pw&temp 8.2;                                    
            %end;
            %else %if &temp = RQUERY or &temp = RCRFPAGE %then %do;
               label &temp = "&templab";           
               format &temp 8.2;
            %end;          
            %if &i = 1 %then %do;
               %if &temp ^= RQUERY and &temp ^= RCRFPAGE %then %do;
                  %let AllRBVarOrder = &temp AV&temp PW&temp;
                  %let AllRBLabOrder = &templab~&trLTPRS~&trLTPPW;
               %end;
               %else %if &temp = RQUERY or &temp = RCRFPAGE %then %do;
                  %let AllRBVarOrder = &temp;
                  %let AllRBLabOrder = &templab;
               %end;
            %end;
            %else %do;
               %if &temp ^= RQUERY and &temp ^= RCRFPAGE %then %do;
                  %let AllRBVarOrder = &AllRBVarOrder &temp AV&temp PW&temp;
                  %let AllRBLabOrder = &AllRBLabOrder~&templab~&trLTPRS~&trLTPPW;
               %end;
               %else %if &temp = RQUERY or &temp = RCRFPAGE %then %do;
                  %let AllRBVarOrder = &AllRBVarOrder &temp;
                  %let AllRBLabOrder = &AllRBLabOrder~&templab;
               %end;
            %end;
            %let i = &i + 1;
         %end;                   
      %end;          
   run;

   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      %if &level = COUNTRY or &level = MONITOR or &level = VENDOR %then %do;
         proc means sum nway noprint data = sumstatsiteid;
            class &level;
            var weeksactive;
            output out = rbm_site_wks(drop = _type_ _freq_) sum = weeksactive;
         run;

         proc means min nway noprint data = sumstatsiteid;
            class &level;
            var siteactive;
            output out = rbm_site_act(drop = _type_ _freq_) min = siteactive;
         run;

         data sumstat;
            merge sumstat rbm_site_wks(keep = &level weeksactive) rbm_site_act(keep = &level siteactive);
            by &level;
            label WeeksActive = "&trWeeksActive"; 
            %if &level = COUNTRY %then %do;
               label siteactive = "&trCountryActiveDate";
            %end;
            %else %if &level = MONITOR %then %do;
               label siteactive = "&trMonitorActiveDate";
            %end;          
            %else %if &level = VENDOR %then %do;
               label siteactive = "&trVendorActiveDate";
            %end;                      
            format weeksactive 8.2;
         run;
         
         proc datasets nolist;
            delete rbm_site_wks rbm_site_act;
         quit;    
      %end;
   
      proc means noprint data = sumstat;
         var rand WeeksActive;
         output out = enrollstat sum = randsum WeeksActivesum;
      run;

      data enrollstat;
         set enrollstat;
         ExpRate = randsum / WeeksActivesum;
         call symput("ExpRate", trim(left(ExpRate)));
      run;

      proc datasets nolist;
         delete enrollstat;
      quit;

      data sumstat;
         set sumstat;
         exprand = round(&ExpRate*WeeksActive);         
         diffrand = rand - exprand;
         siterate = rand / WeeksActive;           
         label diffrand = "&trObservedMinusExpectedRand" exprand = "&trExpectedRandomized" siterate = "&trRandomizedperWeekActive";
         format siterate 8.3;
         %if %upcase(&YNTargetEnroll) = YES %then %do;
            targetrand = round(&TargetEnroll*WeeksActive);
            targetdiffrand = rand - targetrand;
            label targetdiffrand = "&trObservedMinusTargetRand" targetrand = "&trTargetRandomized";
         %end;
      run;     
   %end;

   data sumstat;
      set sumstat;
      drop &dropvars;
   run;

   proc transpose data = sumstat out = sumstattrans(drop = _label_ rename = (_name_ = variable col1 = value));
      var _numeric_;
      by &level;
   run;

   data sumstattrans;
      set sumstattrans;
      variable = upcase(variable);     
   run;

   proc sql noprint; 
      create table RiskInds as
         select * from sumstattrans, RiskThreshold(where = (yelper > . or yelmag > . or redper > . or redmag > .))
         where upcase(sumstattrans.variable) = upcase(RiskThreshold.variable)
         order by variable, sumstattrans.&level;      
   quit;

   data RiskInds;
      set RiskInds;
      %let i = 1;
      %do %while(%length(%scan(&uservars,&i,' ')));
         %let temp = %scan(&uservars,&i,' ');
         %let trLTPRS = %nrbquote(%LocaleFill(&trXperRandomizedSubject, f1=%nrbquote(&&LAB&temp)));
         %let trLTPPW = %nrbquote(%LocaleFill(&trXperPatientWeek, f1=%nrbquote(&&LAB&temp)));         
         if upcase(variable) = "&temp" then label = "&&LAB&temp";          
         else if upcase(variable) = "AV&temp" then label = "&trLTPRS";
         else if upcase(variable) = "PW&temp" then label = "&trLTPPW";
         %let i = %eval(&i+1);
      %end;             
   run;

   proc datasets nolist;
      delete sumstattrans;
   quit;

   proc means noprint nway data = RiskInds;
      class variable;
      var value;
      output out = riskstats mean = Mean median = Median std = StdDev;
   run;

   %macro RiskThings;
      centerflg = upcase(centerflg);
      direction = upcase(direction);
      if missing(direction) then direction = "&trB";
      if missing(centerflg) then centerflg = "&utrMedian";
      redmag = abs(redmag);
      yelmag = abs(yelmag);
      redper = abs(redper);
      yelper = abs(yelper);
      if centerflg in("FIXED","&utrFixed") and missing(centerval) then centerval = 0;
      if direction not in("U","L","B","&trU","&trL","&trB") then delete;
      if centerflg not in("MEAN","MEDIAN","FIXED","&utrMean","&utrMedian","&utrFixed") then delete;
      if missing(redmag) and redper > . then redmag = max(0,yelmag);
      if missing(yelmag) and yelper > . then yelmag = max(0,redmag);
      if centerflg in("MEAN","&utrMean") then center = mean;
      else if centerflg in("MEDIAN","&utrMedian") then center = median;
      else if centerflg in("FIXED","&utrFixed") then center = centerval;
      label center="Center";
   %mend RiskThings;
   
   data RiskInds;
      merge RiskInds riskstats(drop = _type_ _freq_);
      by variable;
      overallwt = abs(overallwt);
      if missing(overallwt) then overallwt = 0;
      %RiskThings;
      if missing(value) or missing(center) or missing(StdDev) or StdDev = 0 then Score = .;
      else if (direction in("B","&trB")) then Score = abs(value - center)/StdDev;
      else if (direction in("U","&trU")) then Score = max(0,(value - center)/StdDev);
      else if (direction in("L","&trL")) then Score = max(0, -(value - center)/StdDev);
   run;

   data Overall;
      set RiskInds(where = (not missing(score) and overallwt > 0));
      statwt = score * overallwt;
   run;
      
   *********************;
   *** Start Overall ***;
   *********************;

   proc means noprint nway data = Overall;
      class &level;
      var statwt overallwt;
      output out = overallstat(drop = _TYPE_ _FREQ_) sum = statwt overallwt;
   run;

   %global Overall_&level;
   %let Overall_&level =;  
   %NObsVars(overallstat);
   %if &nobs %then %let Overall_&level = Overall;  

   proc sql noprint;
      create table Overall2 as
      select *
      from overallstat, RiskThreshold(where = (variable = "OVERALL"));
   quit;

   data Overall2(drop = overallwt statwt);
      set Overall2;
      value = statwt / overallwt;
   run;

   proc means noprint nway data = Overall2;
      class variable;
      var value;
      output out = riskstats2(drop = _TYPE_ _FREQ_) mean = Mean median = Median std = StdDev;
   run;

   data Overall2;
      merge Overall2 riskstats2;
      by variable;
   run;

   *********************;
   ***** End Overall ***;
   *********************;

   *********************;
   *** Start Others ****;
   *********************;

   proc means noprint nway data = Overall;
      class &level Category;
      var statwt overallwt;
      output out = overallstat3(drop = _TYPE_ _FREQ_) sum = statwt overallwt;
   run;

   proc sql noprint;
      create table Overall3 as
      select *
      from overallstat3, RiskThreshold(where = (variable in ("OVERALLS","OVERALLD","OVERALLE","OVERALLX")));
   quit;
   
   %let _lang = %lowcase(%substr(%sysfunc(getoption(locale)),1,2));
   %let ssp = %klength(&trOverallRiskIndicator) + 1;
   
   data Overall3(drop = cat2);
      set Overall3;
      Cat2 = kleft(kSubstr(Label, &ssp));
      /* Second condition below is necessary for difference between zh Disposition in vs. not in, a sentence. */
      if Category = Cat2 or (symget('_lang') ^= "en" and
         (Category = "&trSafety" and Label in ("&trOverallRiskIndicatorSafe","&utrOverallRiskIndicatorSafe")) or
         (Category = "&trDisposition" and Label in ("&trOverallRiskIndicatorDisp","&utrOverallRiskIndicatorDisp")) or
         (Category = "&trEnrollment" and Label in ("&trOverallRiskIndicatorEnroll","&utrOverallRiskIndicatorEnroll")) or
         (Category = "&trSupplemental" and Label in ("&trOverallRiskIndicatorSupp","&utrOverallRiskIndicatorSupp")));
   run;

   %global OverallCats_&level OverallVars_&level;
   %let OverallCats_&level=; 
   %let OverallVars_&level=;
   proc sql noprint;      
      select distinct Category
         into :OverallCats_&level separated by '~'
         from Overall3
         order by variable;
      select distinct variable
         into :OverallVars_&level separated by ' '
         from Overall3
         order by variable;
   quit;       

   data Overall3(drop = overallwt statwt);
      set Overall3;
      value = statwt / overallwt;
   run;

   proc means noprint nway data = Overall3;
      class variable Category;
      var value;
      output out = riskstats3(drop = _TYPE_ _FREQ_) mean = Mean median = Median std = StdDev;
   run;
   
   proc sort data = Overall3;
      by variable category;
   run;

   data Overall3;
      merge Overall3 riskstats3;
      by variable Category;
   run;
   
   *********************;
   ***** End Others ****;
   *********************;

   proc datasets nolist;
      delete riskstats riskstats2 riskstats3 Overall overallstat overallstat3;
      change Overall2 = Overall;       
   quit;    

   data RiskInds;
      set RiskInds Overall(in = inb) Overall3(in = inc);
      if inb or inc then do;
         %RiskThings;
      end;
      drop mean median centerval;         
   run;
   
   /* apply risk indicator logical rules to compute red and yellow thresholds and corresponding signal 1=green, 2=yellow, or 3=red */
   /* kemicl - added logic to keep recommended action based on signal 5Feb2014 */
   data RiskInds;
      set RiskInds;
      length RecAction $&ActionLen;
      label RecAction = "&trRecommendedAction";
      if missing(value) or missing(center) or (missing(redper) and missing(redmag) and missing(yelper) and missing(yelmag)) then signal = .;
      else do;       
         red_thresh_low = .;
         red_thresh_upp = .;
         yel_thresh_low = .;
         yel_thresh_upp = .;
         if center = 0 then scale = 1;
         else scale = abs(center);

         if not missing(redper) or not missing(redmag) then do;         
            if not missing(redper) and not missing(redmag) then offset = max(redper*scale/100,redmag);
            else if not missing(redper) then offset = redper*scale/100;
            else offset = redmag;
            if (direction in("B","&trB","L","&trL")) then red_thresh_low = center - offset;
            if (direction in("B","&trB","U","&trU")) then red_thresh_upp = center + offset;
         end;

         if not missing(yelper) or not missing(yelmag) then do;         
            if not missing(yelper) and not missing(yelmag) then offset = max(yelper*scale/100,yelmag);
            else if not missing(yelper) then offset = yelper*scale/100;
            else offset = yelmag;
            if (direction in("B","&trB","L","&trL")) then yel_thresh_low = center - offset;
            if (direction in("B","&trB","U","&trU")) then yel_thresh_upp = center + offset;
         end;
         
         if not missing(red_thresh_low) and value <= red_thresh_low then signal = 3;
         else if not missing(yel_thresh_low) and value <= yel_thresh_low then signal = 2;
         else if not missing(red_thresh_upp) and value >= red_thresh_upp then signal = 3;
         else if not missing(yel_thresh_upp) and value >= yel_thresh_upp then signal = 2;
         else signal = 1;
         
         /*
         if redper > . and yelper > . then do;
            if (direction in("B","&trB")) and (yelper < abs((value - center)/abs(center))*100 <= redper) and (abs(value) >= yelmag) then signal = 2;
            else if (direction in("B","&trB")) and (abs((value - center)/abs(center))*100 > redper) and (abs(value) >= redmag) then signal = 3;
            else if (direction in("U","&trU")) and (yelper < (value - center)/abs(center)*100 <= redper) and (abs(value) >= yelmag) then signal = 2;
            else if (direction in("U","&trU")) and ((value - center)/abs(center)*100 > redper) and (abs(value) >= redmag) then signal = 3;
            else if (direction in("L","&trL")) and (yelper < -(value - center)/abs(center)*100 <= redper) and (abs(value) >= yelmag) then signal = 2;
            else if (direction in("L","&trL")) and (-(value - center)/abs(center)*100 > redper) and (abs(value) >= redmag) then signal = 3;
         end;
         else if redper > . then do;
            if (direction in("B","&trB")) and (abs((value - center)/abs(center))*100 > redper) and (abs(value) >= redmag) then signal = 3;
            else if (direction in("U","&trU")) and ((value - center)/abs(center)*100 > redper) and (abs(value) >= redmag) then signal = 3;
            else if (direction in("L","&trL")) and (-(value - center)/abs(center)*100 > redper) and (abs(value) >= redmag) then signal = 3;
         end;
         else if yelper > . then do;
            if (direction in("B","&trB")) and (abs((value - center)/abs(center))*100 > yelper) and (abs(value) >= yelmag) then signal = 2;
            else if (direction in("U","&trU")) and ((value - center)/abs(center)*100 > yelper) and (abs(value) >= yelmag) then signal = 2;
            else if (direction in("L","&trL")) and (-(value - center)/abs(center)*100 > yelper) and (abs(value) >= yelmag) then signal = 2;
         end;
         else do;
            if redmag > . and yelmag > . then do;
               if (direction in("B","&trB")) and (yelmag <= abs(value - center) < redmag ) then signal = 2;
               else if (direction in("B","&trB")) and (abs(value - center) >= redmag ) then signal = 3;
               else if (direction in("U","&trU")) and (yelmag <= (value - center) < redmag ) then signal = 2;
               else if (direction in("U","&trU")) and ((value - center) >= redmag ) then signal = 3;
               else if (direction in("L","&trL")) and (yelmag <= -(value - center) < redmag ) then signal = 2;
               else if (direction in("L","&trL")) and (-(value - center) >= redmag ) then signal = 3;
            end;
            else if redmag > . then do;
               if (direction in("B","&trB")) and (abs(value - center) >= redmag ) then signal = 3;
               else if (direction in("U","&trU")) and ((value - center) >= redmag ) then signal = 3;
               else if (direction in("L","&trL")) and (-(value - center) >= redmag ) then signal = 3;                 
            end;
            else if yelmag > . then do;
               if (direction in("B","&trB")) and (abs(value - center) >= yelmag ) then signal = 2;
               else if (direction in("U","&trU")) and ((value - center) >= yelmag ) then signal = 2;
               else if (direction in("L","&trL")) and (-(value - center) >= yelmag ) then signal = 2;                 
            end;
         end;
         */
         /*
         if value > . then do;
            if (yelper > . or redper > .) and center ne 0 and missing(signal) then signal = 1;
            else if (yelper > . or redper > .) and center = 0 and missing(signal) then signal = 4;
            else if (yelmag > . or redmag > .) and missing(signal) then signal = 1;
         end;
         else do;
            signal = 4;    
         end;
         */
      end;
      if signal = 2 then RecAction = yelact;
      else if signal = 3 then RecAction = redact;
      label red_thresh_low="&trRedLowerThreshold" red_thresh_upp="&trRedUpperThreshold" 
         yel_thresh_low="&trYellowLowerThreshold" yel_thresh_upp="&trYellowUpperThreshold";
   run;
   
   proc datasets nolist;
      delete RiskStats Overall3;
   quit;

   %if %length(&&Overall_&level) %then %do;
      data sumstat(drop = variable);
         merge sumstat RiskInds(keep = &level variable value signal where = (variable = "OVERALL") rename = (value = Overall));
         by &level;
         label overall = "&trOverallRiskIndicator" signal = "&trSignal";
         format overall 8.3;
         %if &level = COUNTRY %then %do;
            drop site vendor monitor;
         %end;
         %else %if &level = MONITOR %then %do;
            drop site country vendor;
         %end;
         %else %if &level = MONITOR %then %do;
            drop site country monitor;
         %end;         
      run;
   %end;
   %else %do;
      data sumstat(drop = variable);
         set sumstat;
         signal = .;
         label signal = "&trSignal";
         %if &level = COUNTRY %then %do;
            drop site vendor monitor;
         %end;
         %else %if &level = MONITOR %then %do;
            drop site country vendor;
         %end;
         %else %if &level = MONITOR %then %do;
            drop site country monitor;
         %end;         
      run;  
   %end;
   
   data temp;
      set RiskInds(where = (variable in ("OVERALLS", "OVERALLD", "OVERALLE", "OVERALLX")) keep = &level variable value label);
      format value 8.3;
   run;
   
   proc sort data = temp;
      by &level;
   run;
   
   proc transpose data = temp out = temp(drop = _name_);
      by &level;
      var value;
      id variable;
      idlabel label;
   run;
   
   %if %length(&&OverallVars_&level) %then %do;
      data sumstat;
         merge sumstat temp;
         by &level;
      run;
   %end;

   proc datasets nolist;
      change sumstat = sumstat&level;        
      change RiskInds = RiskInds&level;         
      delete temp;
   quit;
   
   %cexit:;
%mend CALCSUMS;

%global includeds;
%let includeds =;
%CALCSUMS(level = SITEID);

%if %sysevalf(&NumCountry > 0) and ^&timetrendsgather %then %do;
   %CALCSUMS(level = COUNTRY);
%end;
%if &exiterror %then %goto gexit;

%if %upcase(&MonitorAnalysis) = YES and ^&timetrendsgather %then %do;
   %CALCSUMS(level = MONITOR);
%end;

%if %upcase(&VendorAnalysis) = YES and ^&timetrendsgather %then %do;
   %CALCSUMS(level = VENDOR);
%end;
%if &exiterror %then %goto gexit;

%if %length(&RiskBasedMonitoring_OutData)>12 %then %let RiskBasedMonitoring_OutData = %substr(&RiskBasedMonitoring_OutData,1,12);
%if %length(&RiskBasedMonitoring_OutData) %then %let Outname = &RiskBasedMonitoring_OutData._;
%else %let Outname = &RiskBasedMonitoring_OutData;
%let RiskBasedMonitoring_OutData =;
%let RiskBasedMonitoring_OutData_si = ;      
%PrepOutDS(RiskBasedMonitoring_OutData_si,OutDataName_si, &Outname.rbm_site_sum);
data &RiskBasedMonitoring_OutData_si;
   set sumstatsiteid;
   %if %length(&DropVars) %then %do;
      drop &DropVars;
   %end;
   label siteid = "&SiteLabel" country = "&CountryLabel";
run;
proc datasets nolist;
   delete sumstatsiteid;
quit;

%if %sysevalf(&NumCountry > 0) and ^&timetrendsgather %then %do;
   %let RiskBasedMonitoring_OutData_co = ;      
   %PrepOutDS(RiskBasedMonitoring_OutData_co,OutDataName_co, &Outname.rbm_country_sum);
   %let includeds = &includeds &OutDataName_co..sas7bdat,;     
   data &RiskBasedMonitoring_OutData_co;
      set sumstatcountry;
      %if %length(&DropVars) %then %do;
         drop &DropVars;
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         drop weeksactive;
      %end;
      label country = "&CountryLabel";
   run;
   proc datasets nolist;
      delete sumstatcountry;
   quit;
%end;

%if %upcase(&MonitorAnalysis) = YES and ^&timetrendsgather %then %do;
   %let RiskBasedMonitoring_OutData_mo = ;
   %PrepOutDS(RiskBasedMonitoring_OutData_mo,OutDataName_mo, &Outname.rbm_monitor_sum);
   %let includeds = &includeds &OutDataName_mo..sas7bdat,;     
   data &RiskBasedMonitoring_OutData_mo;
      set sumstatmonitor;
      %if %length(&DropVars) %then %do;
         drop &DropVars;
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         drop weeksactive;
      %end;
      label monitor = "&trMonitor";
   run;
   proc datasets nolist;
      delete sumstatmonitor;
   quit;
%end;

%if %upcase(&VendorAnalysis) = YES and ^&timetrendsgather %then %do;
   %let RiskBasedMonitoring_OutData_vo = ;
   %PrepOutDS(RiskBasedMonitoring_OutData_vo,OutDataName_vo, &Outname.rbm_vendor_sum);
   %let includeds = &includeds &OutDataName_vo..sas7bdat,;     
   data &RiskBasedMonitoring_OutData_vo;
      set sumstatvendor;
      %if %length(&DropVars) %then %do;
         drop &DropVars;
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         drop weeksactive;
      %end;
      label vendor = "&trVendor";
   run;
   proc datasets nolist;
      delete sumstatvendor;
   quit;
%end;

****************************;
***** Define Groupings *****;
****************************;

%macro GetOverallPhrase(_CatForPhrase);
   %if &_CatForPhrase = %nrbquote(&trSafety) %then
      %let OverallPhrase = %nrbquote(&trOverallRiskIndicatorSafe);
   %else %if &_CatForPhrase = %nrbquote(&trDisposition) %then
      %let OverallPhrase = %nrbquote(&trOverallRiskIndicatorDisp);
   %else %if &_CatForPhrase = %nrbquote(&trEnrollment) %then
      %let OverallPhrase = %nrbquote(&trOverallRiskIndicatorEnroll);
   %else %if &_CatForPhrase = %nrbquote(&trSupplemental) %then
      %let OverallPhrase = %nrbquote(&trOverallRiskIndicatorSupp);
   %else
      %let OverallPhrase = %nrbquote(%LocaleFill(&trOverallRiskIndicatorX, f1=%nrbquote(&temp)));
   &OverallPhrase
%mend;

%global SiteGroup VarOrder;
%let SiteGroup = ;
%let SiteGroup = {&SiteLabel;
%let VarOrder = siteid;
%if &cind %then %do;
   %let SiteGroup = &SiteGroup,&CountryLabel;
   %let VarOrder = &VarOrder &country;
%end;

%let i = 1;
%do %while(%klength(%kscan(&AvailLocVars,&i,' ')));
   %let temp = %kscan(&AvailLocVars,&i,' ');
   %if &temp = STATEPROVINCE %then %do;
      %let SiteGroup = &SiteGroup,&trStateorProvince;
      %let VarOrder = &VarOrder stateprovince;
   %end;
   %else %if &temp = ZIPCODE %then %do;
      %let SiteGroup = &SiteGroup,&trFiveDigitZipCode;
      %let VarOrder = &VarOrder zipcode;
   %end;
   %else %if &temp = LATITUDE %then %do;
      %let SiteGroup = &SiteGroup,&trLatitude;
      %let VarOrder = &VarOrder Latitude;
   %end;
   %else %if &temp = LONGITUDE %then %do;
      %let SiteGroup = &SiteGroup,&trLongitude;
      %let VarOrder = &VarOrder Longitude;
   %end;
   %else %if &temp = CITY %then %do;
      %let SiteGroup = &SiteGroup,&trCity;
      %let VarOrder = &VarOrder City;
   %end;
   %else %if &temp = MONITOR %then %do;
      %let SiteGroup = &SiteGroup,&trMonitor;
      %let VarOrder = &VarOrder Monitor;
   %end;
   %else %if &temp = SITECATEGORY %then %do;
      %let SiteGroup = &SiteGroup,&trSiteCategory;
      %let VarOrder = &VarOrder SiteCategory;
   %end; 
   %else %if &temp = VENDOR %then %do;
      %let SiteGroup = &SiteGroup,&trVendor;
      %let VarOrder = &VarOrder Vendor;
   %end;    
   %let i = %eval(&i+1);
%end;
%let SiteGroup = &SiteGroup};

%if %klength(&Overall_SITEID) %then %do;
   %let OverGroup_SITEID = {&trOverallRiskIndicator,;
   %let VarOrder = &VarOrder overall; 
   %let i = 1;
   %if %klength(&OverallCats_SITEID) %then %do;
      %do %while(%klength(%kscan(%bquote(&OverallCats_SITEID),&i,'~')));
         %let temp = %kscan(%bquote(&OverallCats_SITEID),&i,'~');
         %let tempvar = %kscan(%bquote(&OverallVars_SITEID),&i,' ');
         %let VarOrder = &VarOrder &tempvar;
         %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
         %let Overgroup_SITEID = &OverGroup_SITEID.&OverallPhrase,;
         %let i = %eval(&i+1);
      %end;
   %end;
   %let OverGroup_SITEID = %ksubstr(%quote(&OverGroup_SITEID), 1, %eval(%klength(&OverGroup_SITEID) - 1))};
%end;
%else %do;
   %let OverGroup_SITEID = {};
%end;

%if %klength(&Overall_COUNTRY) and ^&timetrendsgather %then %do;
   %let OverGroup_COUNTRY = {&trOverallRiskIndicator,;
   %let i = 1;
   %if %klength(&OverallCats_COUNTRY) %then %do;
      %do %while(%klength(%kscan(%bquote(&OverallCats_COUNTRY),&i,'~')));
         %let temp = %kscan(%bquote(&OverallCats_COUNTRY),&i,'~');
         %let tempvar = %kscan(%bquote(&OverallVars_COUNTRY),&i,' ');
         %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
         %let Overgroup_COUNTRY = &OverGroup_COUNTRY.&OverallPhrase,;
         %let i = %eval(&i+1);
      %end;
   %end;
   %let OverGroup_COUNTRY = %ksubstr(%quote(&OverGroup_COUNTRY), 1, %eval(%klength(&OverGroup_COUNTRY) - 1))};
%end;
%else %do;
   %let OverGroup_COUNTRY = {};
%end;

%if %upcase(&MonitorAnalysis) = YES and ^&timetrendsgather %then %do;
   %if %klength(&Overall_MONITOR) %then %do;
      %let OverGroup_MONITOR = {&trOverallRiskIndicator,;
      %let i = 1;
      %if %klength(&OverallCats_MONITOR) %then %do;
         %do %while(%klength(%kscan(%bquote(&OverallCats_MONITOR),&i,'~')));
            %let temp = %kscan(%bquote(&OverallCats_MONITOR),&i,'~');
            %let tempvar = %kscan(%bquote(&OverallVars_MONITOR),&i,' ');
            %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
            %let Overgroup_MONITOR = &OverGroup_MONITOR.&OverallPhrase,;
            %let i = %eval(&i+1);
         %end;
      %end;
      %let OverGroup_MONITOR = %ksubstr(%quote(&OverGroup_MONITOR), 1, %eval(%klength(&OverGroup_MONITOR) - 1))};
   %end;
   %else %do;
      %let OverGroup_MONITOR = {};
   %end;
%end;

%if %upcase(&VendorAnalysis) = YES and ^&timetrendsgather %then %do;
   %if %klength(&Overall_VENDOR) %then %do;
      %let OverGroup_VENDOR = {&trOverallRiskIndicator,;
      %let i = 1;
      %if %klength(&OverallCats_VENDOR) %then %do;
         %do %while(%klength(%kscan(%bquote(&OverallCats_VENDOR),&i,'~')));
            %let temp = %kscan(%bquote(&OverallCats_VENDOR),&i,'~');
            %let tempvar = %kscan(%bquote(&OverallVars_VENDOR),&i,' ');
            %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
            %let Overgroup_VENDOR = &OverGroup_VENDOR.&OverallPhrase,;
            %let i = %eval(&i+1);
         %end;
      %end;
      %let OverGroup_VENDOR = %ksubstr(%quote(&OverGroup_VENDOR), 1, %eval(%klength(&OverGroup_VENDOR) - 1))};
   %end;
   %else %do;
      %let OverGroup_VENDOR = {};
   %end;
%end;

%let EnrlGroup = {&utrTotalSubjects,&utrRandomized,&utrScreenFailure,&utrPercentScreenFailofTotalSubjs,&utrTreated,&utrPatientWeeksonStudy,;
%let VarOrder = &VarOrder total rand scrnfail pctscrnfail treated patweeks; 
%if %length(&RFICDTC) %then %do;
   %let EnrlGroup = &EnrlGroup.&utrMissingInformedConsent,;
   %let VarOrder = &VarOrder missconsent;
%end;
%if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
   %let EnrlGroup = &EnrlGroup.&trSiteActiveDate,&trWeeksActive,&trRandomizedperWeekActive,&trExpectedRandomized,&trObservedMinusExpectedRand,;
   %let VarOrder = &VarOrder siteactive weeksactive siterate exprand diffrand;
%end;
%if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) and %upcase(&YNTargetEnroll) = YES %then %do;
   %let EnrlGroup = &EnrlGroup.&trTargetRandomized,&trObservedMinusTargetRand,;
   %let VarOrder = &VarOrder targetrand targetdiffrand;
%end;
%let EnrlGroup = %substr(%quote(&EnrlGroup), 1, %eval(%length(&EnrlGroup) - 1))};

%let SubsGroup = {&utrCompleted,&utrPercentCompletedofRandSubjs,&utrOngoing,&utrPercentOngoingofRandSubjs,&utrDiscontinued,&utrPercentDiscontofRandSubjs,;
%let SubsGroup = &SubsGroup.&utrDiscontinuedduetoDeath,&utrDiscontinuedduetoAE,&utrLosttoFollowup,&utrPatientWithdrewfromStudy,&utrDiscontinuedOther,;
%let VarOrder = &VarOrder complete pctcomp ongoing pctongoing discon pctdiscon discdeath dcae ltfu withdrew dcoth;
%if %sysfunc(findw(&domains, DV)) %then %do;
   %let SubsGroup = &SubsGroup.&trTotalProtocolDeviations,&trAverageDeviationsperRandSubj,;
   %let VarOrder = &VarOrder totdv avgdv;
%end;
%if %sysfunc(findw(&domains, IE)) %then %do;
   %let SubsGroup = &SubsGroup.&trTotalInclusionExclusionNotMet,&trAverageInExNotMetperRandSubj,;
   %let VarOrder = &VarOrder totie avgie;
%end;
%let SubsGroup = %substr(%quote(&SubsGroup), 1, %eval(%length(&SubsGroup) - 1))};

%let SafetyGroup = {&utrDeath,&utrPercentDeathsofRandSubjs,&utrDeathsperPatientWeek,;
%let VarOrder = &VarOrder death pctdeath pwdeath;
%if %sysfunc(findw(&domains, AE)) %then %do;
   %let SafetyGroup = &SafetyGroup.&trTotalAEsonStudy,&trAverageAEsperRandSubj,&trAEsperPatientWeek,; 
   %let VarOrder = &VarOrder totae avgae pwae;
   /* %if %length(&AEREL) %then %do;
      %let SafetyGroup = &SafetyGroup.Total Related AEs on Study,Average Related AEs per Randomized Subject,Related AEs per PatientWeek,; 
      %let VarOrder = &VarOrder totrae avgrae pwrae;
   %end; */
   %if %length(&AESER) %then %do;
      %let SafetyGroup = &SafetyGroup.&trTotalSAEsonStudy,&trAverageSAEsperRandSubj,&trSAEsperPatientWeek,; 
      %let VarOrder = &VarOrder totsae avgsae pwsae;
   %end;
   /* %if %length(&AESER) and %length(&AEREL) %then %do;
      %let SafetyGroup = &SafetyGroup.Total Related SAEs on Study,Average Related SAEs per Randomized Subject,Related SAEs per PatientWeek,;
      %let VarOrder = &VarOrder totrsae avgrsae pwrsae;
   %end;*/
%end;
%if %sysfunc(findw(&domains, HO)) %then %do;
   %let SafetyGroup = &SafetyGroup.&trTotalHealthcareEncounters,&trHealthcareEncountersAV,&trHealthcareEncountersPW,; 
   %let VarOrder = &VarOrder totho avgho pwho;
%end;
%let SafetyGroup = %substr(%quote(&SafetyGroup), 1, %eval(%length(&SafetyGroup) - 1))};

%let SuppGroup = {;
%let i = 1;
%do %while(%length(%scan(&uservars,&i,' ')));
   %let temp = %scan(&uservars,&i,' ');   
   %if %sysfunc(findw(&AvailAnalVars, &temp)) %then %do;
      %let SuppGroup = &SuppGroup.&&LAB&temp,&trLTPRS,&trLTPPW,;
      %let VarOrder = &VarOrder &temp av&temp pw&temp;
   %end;          
   %let i = %eval(&i+1);
%end;
%if %length(&ExData) %then %do;
      %let SuppGroup = &SuppGroup%sysfunc(transtrn(&AllRBLabOrder, %str(~), %str(,))),;
      %let VarOrder = &VarOrder &AllRBVarOrder;
%end;
%if %index(%bquote(&SuppGroup), %bquote(,)) %then %do;
   %let SuppGroup = %substr(%quote(&SuppGroup), 1, %eval(%length(&SuppGroup) - 1))};
%end;
%else %let SuppGroup = ;


%if %sysevalf(&NumCountry > 0) and ^&timetrendsgather %then %do;
   %let EnrlGroupCountry = &EnrlGroup;
   %let EnrlGroupCountry = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupCountry), %nrbquote({&utrTotalSubjects,), %nrbquote({&utrTotalSites,&utrTotalSubjects,)));
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      %let EnrlGroupCountry = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupCountry), %nrbquote(,&trSiteActiveDate,), %nrbquote(,)));
      %if %index(%nrbquote(&EnrlGroupCountry),%nrbquote(,&trWeeksActive,)) %then %do;
         %let EnrlGroupCountry = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupCountry), %nrbquote(,&trWeeksActive,), %nrbquote(,)));
      %end;
   %end;   
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      %let EnrlGroupCountry = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupCountry), %nrbquote(&trTotalSites,), %nrbquote(&trTotalSites,&trCountryActiveDate,)));
   %end;
%end;

%if %upcase(&MonitorAnalysis) = YES and ^&timetrendsgather %then %do;
   %let EnrlGroupMonitor = &EnrlGroupCountry;
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      %let EnrlGroupMonitor = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupMonitor), %nrbquote(&trCountryActiveDate,), %nrbquote(&trMonitorActiveDate,)));
   %end;
%end;

%if %upcase(&VendorAnalysis) = YES and ^&timetrendsgather %then %do;
   %let EnrlGroupVendor = &EnrlGroupCountry;
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      %let EnrlGroupVendor = %sysfunc(TRANWRD(%nrbquote(&EnrlGroupVendor), %nrbquote(&trCountryActiveDate,), %nrbquote(&trVendorActiveDate,)));
   %end;
%end;

****************************;
***** Save Data ************;
****************************;

%let RiskBasedMonitoring_OutData_ss = ;      
%PrepOutDS(RiskBasedMonitoring_OutData_ss,OutDataName_ss, &Outname.rbm_subjectsummary);

data &RiskBasedMonitoring_OutData_ss;
   set rbm_jmpc(drop = int);
   label siteid = "&SiteLabel" country = "&CountryLabel";
run;

%let RiskBasedMonitoring_OutData_ri = ;      
%PrepOutDS(RiskBasedMonitoring_OutData_ri,OutDataName_ri, &Outname.rbm_site_riskflags);

proc sql noprint;      
   select distinct upcase(siteid)
      into :sigchk separated by '~'
      from &RiskBasedMonitoring_OutData_si;
      %let s_sigchk = &sqlobs; 
quit;

data &RiskBasedMonitoring_OutData_ri;
   set RiskIndssiteid(where = (signal > .));
   %if &s_sigchk = 1 %then %do;
      if yelper > . and redper > . then delete;
   %end;
   label siteid = "&SiteLabel" variable = "&trVariable" label = "&trLabel" signal = "&trSignal";
run;

%let NumSwitch = 0;
%let SwitchList = ;
proc sql noprint;      
   select distinct label
      into :SwitchList separated by ','
      from &RiskBasedMonitoring_OutData_ri(where = ((index(upcase(label),"OVERALL RISK INDICATOR") = 0 and kindex(kupcase(label),"&utrOverallRiskIndicator") = 0)));
   %let NumSwitch = &sqlobs;
quit;
%let SwitchListO = &trOverallRiskIndicator,;
%let i = 1;
%let NumOver = 1;
%if %klength(&OverallCats_SITEID) %then %do;
   %do %while(%klength(%kscan(%bquote(&OverallCats_SITEID),&i,'~')));
      %let temp = %kscan(%bquote(&OverallCats_SITEID),&i,'~');
      %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
      %let SwitchListO = &SwitchListO.&OverallPhrase,;
      %let i = %eval(&i+1);
      %let NumOver = %eval(&NumOver+1);
   %end;
   %let NumOver = %eval(&NumOver-1);
%end;

%let SwitchList = &SwitchListO&SwitchList;
%let NumSwitch = %eval(&NumSwitch+&NumOver);
%let QSwitchList = ;
%let i = 1;
%do %while(%klength(%kscan(%bquote(&SwitchList),&i,',')));
   %let temp = %kscan(%bquote(&SwitchList),&i,',');
   %if &i = 1 %then %let QSwitchList = %nrbquote(""&temp"");
   %else %let QSwitchList = &QSwitchList,%nrbquote(""&temp"");
   %let i = %eval(&i+1);
%end;

* create stacked data set for va;
data stacked;
   set RiskIndssiteid;
   value = round(value,0.001);
run;

proc sort data=stacked;
   by siteid;
run;

data stacked;
   merge stacked &RiskBasedMonitoring_OutData_si(keep=siteid &AvailLocVars &AvailAnalVars);
   by siteid;
run;

%let RiskBasedMonitoring_OutData_sis = ;     
%PrepOutDS(RiskBasedMonitoring_OutData_sis,OutDataName_sis, &Outname.rbm_site_stack);

data &RiskBasedMonitoring_OutData_sis;
   set stacked;
   if missing(Score) and not missing(value) then Score = value;
   if variable in ("OVERALLS","OVERALLD","OVERALLE","OVERALLX") then category = "&trOverall";
   label siteid = "&SiteLabel" value = "&trValue" variable = "&trRiskVariable" 
      label = "&trRiskIndicator" signal = "&trSignal";
   drop yelact redact scale offset;
run;

%gexit:;

%mend GATHER_DATA;

%if &TimeTrends = Yes %then %do;
   
   %let dm_start_date = dmdtc;
   %let dm_end_date = rfendtc;
   
   * get trial start and end dates;
   proc sort data=&in_dm out=d1;
      by &dm_start_date;
      where not missing(&dm_start_date);
   run;
   data _null_;
      set d1(obs=1);
      call symputx("trial_start",&dm_start_date);
   run;
   proc sort data=&in_dm out=d1;
      by descending &dm_end_date;
      where not missing(&dm_end_date);
   run;
   data _null_;
      set d1(obs=1);
      call symputx("trial_end",&dm_end_date);
   run;
   %put trial_start = &trial_start;
   %put trial_end = &trial_end;
   %if ^%length(&DateStart) %then %let DateStart = &trial_start;
   %if ^%length(&DateEnd) %then %let DateEnd = &trial_end;
   
   * error checking on the dates;
   %if %quote(&DateStart) > %quote(&DateEnd) %then %do;
      %put [MS_S_0168];
      %put ERROR: The specified start date for RBM time trends, &DateStart, is after the specified end date, &DateEnd..;
      %put Please specify an earlier start date or later end date.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %if %quote(&DateStart) > %quote(&trial_end) %then %do;
      %put [MS_S_0169];
      %put ERROR: The specified start date for RBM time trends, &DateStart, is after the last subject reference end date, &trial_end..;
      %put Please specify an earlier start date.;
      %let exiterror = 1;
      %goto exit;
   %end;
   %if %quote(&DateEnd) < %quote(&trial_start) %then %do;
      %put [MS_S_0170];
      %put ERROR: The specified end date for RBM time trends, &DateEnd, is before the first subject enrollment date, &trial_start..;
      %put Please specify a later end date.;
      %let exiterror = 1;
      %goto exit;
   %end;
   /*
   %if %quote(&DateStart) < %quote(&trial_start) %then %let DateStart = &trial_start;
   %if %quote(&DateEnd) > %quote(&trial_end) %then %let DateEnd = &trial_end;
   */

   * loop over time windows and accumulate stacked data;
   %let dl = &DateStart;
   %let interval = 1;
   %do %while(&interval);
      %put interval = &interval;
      %put dl = &dl;
      data _null_;
         dtl = input("&dl",is8601da.);
         dtu = dtl + &TimeWindowSize;
         call symputx("du",put(dtu,E8601DA.));
      run;
      %put du = &du;
      %GATHER_DATA(date_lower=&dl,date_upper=&du,timetrendsgather=1);
      %if &exiterror %then %goto gexit;
      data st;
         set &RiskBasedMonitoring_OutData_sis;
         Time_Interval = &interval;
         Date_Lower = "&dl";
         Date = "&du";
         label Time_Interval="&trTimeInterval" Date_Lower="&trLowerDate" Date="&trDate" Score = "&trScore";
      run;  
      %if &interval = 1 %then %do;
         data stacked_time;
            set st;
         run;
      %end;
      %else %do;
         proc append data=st force base=stacked_time;
         run;
      %end;
      %if %quote(&du) > %quote(&DateEnd) %then %do;
         %let interval = 0;
      %end;
      %else %do;
         %let dl = &du;
         %let interval = %eval(&interval+1);
      %end;
      proc datasets nolist;
         delete RiskIndssiteid RiskIndscountry rbm_jmpc;
      quit;
   %end;
   
   %let RiskBasedMonitoring_OutData_sist = ;    
   %PrepOutDS(RiskBasedMonitoring_OutData_sist,OutDataName_sist, &Outname.rbm_site_stack_time);
   %let includeds = &includeds &OutDataName_sist..sas7bdat,;
   %let includeds1 = &includeds;

   proc sort data=stacked_time out=&RiskBasedMonitoring_OutData_sist;
      by siteid label time_interval;
   run;  
%end;
%else %do;
   %let includeds1 = &includeds;
%end;

* run GATHER_DATA on full data set;
%GATHER_DATA;
%if &exiterror %then %goto exit;
%let includeds = &includeds1 &OutDataName_si..sas7bdat,;    
%if %sysevalf(&NumCountry > 0) %then %do;
   %let includeds = &includeds &OutDataName_co..sas7bdat,;  
%end;
%let includeds = &includeds &OutDataName_ss..sas7bdat,;     
%let includeds = &includeds &OutDataName_ri..sas7bdat,;     
%let includeds = &includeds &OutDataName_sis..sas7bdat,; 


/* kemicl - added additional output data set of Site, risk and value and recommended action */
%let RiskBasedMonitoring_OutData_ra = ;      
%PrepOutDS(RiskBasedMonitoring_OutData_ra,OutDataName_ra, &Outname.rbm_site_actions);
%let includeds = &includeds &OutDataName_ra..sas7bdat,;     
   
data &RiskBasedMonitoring_OutData_ra;
   set RiskIndssiteid;
   label siteid = "&SiteLabel" value = "&trValue" variable = "&trVariable" label = "&trLabel" signal = "&trSignal";
   where not missing(RecAction);
   value = round(value,0.001);
run;

/* Create flag of whether there are actions to report for report drill down */
%let ActFlg = 0;
%nobsvars(&RiskBasedMonitoring_OutData_ra);
%if &nobs %then %do;
   %let ActFlg = 1;
   proc sort data = &RiskBasedMonitoring_OutData_ra;
      by siteid;
   run;
%end;
/* kemicl - end action data set creation */

%if %sysevalf(&NumCountry > 0) %then %do;
   %let RiskBasedMonitoring_OutData_cr = &RiskBasedMonitoring_OutData;     
   %PrepOutDS(RiskBasedMonitoring_OutData_cr,OutDataName_cr, rbm_country_riskflags);
   %let includeds = &includeds &OutDataName_cr..sas7bdat,;     

   proc sql noprint;      
      select distinct upcase(country)
         into :sigchk separated by '~'
         from &RiskBasedMonitoring_OutData_co;
         %let c_sigchk = &sqlobs; 
   quit;

   data &RiskBasedMonitoring_OutData_cr;
      set RiskIndscountry(drop = value where = (signal > .));
      %if &c_sigchk = 1 %then %do;
         if yelper > . and redper > . then delete;
      %end;      
      label siteid = "&SiteLabel" country = "&CountryLabel" variable = "&trVariable" label = "&trLabel" signal = "&trSignal";
   run;
%end;

%if %upcase(&MonitorAnalysis) = YES %then %do;
   %let RiskBasedMonitoring_OutData_mr = &RiskBasedMonitoring_OutData;     
   %PrepOutDS(RiskBasedMonitoring_OutData_mr,OutDataName_mr, rbm_monitor_riskflags);
   %let includeds = &includeds &OutDataName_mr..sas7bdat,;     

   proc sql noprint;      
      select distinct upcase(monitor)
         into :sigchk separated by '~'
         from &RiskBasedMonitoring_OutData_mo;
         %let m_sigchk = &sqlobs; 
   quit;
   
   data &RiskBasedMonitoring_OutData_mr(where = (signal > .));
      set RiskIndsmonitor(drop = value where = (signal > .));
      %if &m_sigchk = 1 %then %do;
         if yelper > . and redper > . then delete;
      %end;      
      label monitor = "&trMonitor" variable = "&trVariable" label = "&trLabel" signal = "&trSignal";
   run;
%end;

%if %upcase(&VendorAnalysis) = YES %then %do;
   %let RiskBasedMonitoring_OutData_vr = &RiskBasedMonitoring_OutData;     
   %PrepOutDS(RiskBasedMonitoring_OutData_vr,OutDataName_vr, rbm_vendor_riskflags);
   %let includeds = &includeds &OutDataName_vr..sas7bdat,;     

   proc sql noprint;      
      select distinct upcase(vendor)
         into :sigchk separated by '~'
         from &RiskBasedMonitoring_OutData_vo;
         %let v_sigchk = &sqlobs; 
   quit;
   
   data &RiskBasedMonitoring_OutData_vr(where = (signal > .));
      set RiskIndsvendor(drop = value where = (signal > .));
      %if &v_sigchk = 1 %then %do;
         if yelper > . and redper > . then delete;
      %end;      
      label vendor = "&trVendor" variable = "&trVariable" label = "&trLabel" signal = "&trSignal";
   run;
%end;

proc datasets nolist;
   delete RiskIndssiteid RiskIndscountry %if %upcase(&MonitorAnalysis) = YES %then %do; RiskIndsmonitor %end; %if %upcase(&VendorAnalysis) = YES %then %do; RiskIndsvendor %end; rbm_jmpc;
quit;

data &RiskBasedMonitoring_OutData_si;
   retain &VarOrder;
   set &RiskBasedMonitoring_OutData_si;
run;

%if %sysevalf(&NumCountry > 0) %then %do;
   data &RiskBasedMonitoring_OutData_co;
      retain %substr(&VarOrder,8,%eval(%sysfunc(findw(&VarOrder, total))-9)) totalsites %substr(&VarOrder,%eval(%sysfunc(findw(&VarOrder, total))));
      set &RiskBasedMonitoring_OutData_co;
   run;
%end;

%if %upcase(&MonitorAnalysis) = YES %then %do;
   data &RiskBasedMonitoring_OutData_mo;
      retain %substr(&VarOrder,8,%eval(%sysfunc(findw(&VarOrder, total))-9)) totalsites %substr(&VarOrder,%eval(%sysfunc(findw(&VarOrder, total))));
      set &RiskBasedMonitoring_OutData_mo;
   run;
%end;

%if %upcase(&VendorAnalysis) = YES %then %do;
   data &RiskBasedMonitoring_OutData_vo;
      retain %substr(&VarOrder,8,%eval(%sysfunc(findw(&VarOrder, total))-9)) totalsites %substr(&VarOrder,%eval(%sysfunc(findw(&VarOrder, total))));
      set &RiskBasedMonitoring_OutData_vo;
   run;
%end;

%let ENRLDISTVARS =;
%let ENRLDISTVARS2 =;

* create JSL file;
%PrepOutFile(RiskBasedMonitoringScript,RiskBasedMonitoringName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &RiskBasedMonitoringScript;
%let FullAPName=%str(Risk Based Monitoring);

%if %klength(&Overall_SITEID) %then %do;
   %let OVERDISTVARS_SITEID = &trOverallRiskIndicator,;
   %let OVERDISTVARS2_SITEID = overall; 
   %let i = 1;
   %if %klength(&OverallCats_SITEID) %then %do;
      %do %while(%klength(%kscan(%bquote(&OverallCats_SITEID),&i,'~')));
         %let temp = %kscan(%bquote(&OverallCats_SITEID),&i,'~');
         %let tempvar = %kscan(%bquote(&OverallVars_SITEID),&i,' ');
         %let OVERDISTVARS2_SITEID = &OVERDISTVARS2_SITEID &tempvar;
         %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
         %let OVERDISTVARS_SITEID = &OVERDISTVARS_SITEID.&OverallPhrase,;
         %let i = %eval(&i+1);
      %end;
   %end;
   %let OVERDISTVARS_SITEID = %ksubstr(%quote(&OVERDISTVARS_SITEID), 1, %eval(%klength(&OVERDISTVARS_SITEID) - 1));
%end;
%else %do;
   %let OVERDISTVARS_SITEID =;
   %let OVERDISTVARS2_SITEID =; 
%end;

%if %klength(&Overall_COUNTRY) %then %do;
   %let OVERDISTVARS_COUNTRY = &trOverallRiskIndicator,;
   %let OVERDISTVARS2_COUNTRY = overall; 
   %let i = 1;
   %if %klength(&OverallCats_COUNTRY) %then %do;
      %do %while(%klength(%kscan(%bquote(&OverallCats_COUNTRY),&i,'~')));
         %let temp = %kscan(%bquote(&OverallCats_COUNTRY),&i,'~');
         %let tempvar = %kscan(%bquote(&OverallVars_COUNTRY),&i,' ');
         %let OVERDISTVARS2_COUNTRY = &OVERDISTVARS2_COUNTRY &tempvar;
         %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
         %let OVERDISTVARS_COUNTRY = &OVERDISTVARS_COUNTRY.&OverallPhrase,;
         %let i = %eval(&i+1);
      %end;
   %end;
   %let OVERDISTVARS_COUNTRY = %ksubstr(%quote(&OVERDISTVARS_COUNTRY), 1, %eval(%klength(&OVERDISTVARS_COUNTRY) - 1));
%end;
%else %do;
   %let OVERDISTVARS_COUNTRY =;
   %let OVERDISTVARS2_COUNTRY =; 
%end;

%if %upcase(&MonitorAnalysis) = YES %then %do;
   %if %klength(&Overall_MONITOR) %then %do;
      %let OVERDISTVARS_MONITOR = &trOverallRiskIndicator,;
      %let OVERDISTVARS2_MONITOR = overall; 
      %let i = 1;
      %if %klength(&OverallCats_MONITOR) %then %do;
         %do %while(%klength(%kscan(%bquote(&OverallCats_MONITOR),&i,'~')));
            %let temp = %kscan(%bquote(&OverallCats_MONITOR),&i,'~');
            %let tempvar = %kscan(%bquote(&OverallVars_MONITOR),&i,' ');
            %let OVERDISTVARS2_MONITOR = &OVERDISTVARS2_MONITOR &tempvar;
            %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
            %let OVERDISTVARS_MONITOR = &OVERDISTVARS_MONITOR.&OverallPhrase,;
            %let i = %eval(&i+1);
         %end;
      %end;
      %let OVERDISTVARS_MONITOR = %ksubstr(%quote(&OVERDISTVARS_MONITOR), 1, %eval(%klength(&OVERDISTVARS_MONITOR) - 1));
   %end;
   %else %do;
      %let OVERDISTVARS_MONITOR =;
      %let OVERDISTVARS2_MONITOR =; 
   %end;
%end;

%if %upcase(&VendorAnalysis) = YES %then %do;
   %if %klength(&Overall_VENDOR) %then %do;
      %let OVERDISTVARS_VENDOR = &trOverallRiskIndicator,;
      %let OVERDISTVARS2_VENDOR = overall; 
      %let i = 1;
      %if %klength(&OverallCats_VENDOR) %then %do;
         %do %while(%klength(%kscan(%bquote(&OverallCats_VENDOR),&i,'~')));
            %let temp = %kscan(%bquote(&OverallCats_VENDOR),&i,'~');
            %let tempvar = %kscan(%bquote(&OverallVars_VENDOR),&i,' ');
            %let OVERDISTVARS2_VENDOR = &OVERDISTVARS2_VENDOR &tempvar;
            %let OverallPhrase = %nrbquote(%GetOverallPhrase(&temp));
            %let OVERDISTVARS_VENDOR = &OVERDISTVARS_VENDOR.&OverallPhrase,;
            %let i = %eval(&i+1);
         %end;
      %end;
      %let OVERDISTVARS_VENDOR = %ksubstr(%quote(&OVERDISTVARS_VENDOR), 1, %eval(%klength(&OVERDISTVARS_VENDOR) - 1));
   %end;
   %else %do;
      %let OVERDISTVARS_VENDOR =;
      %let OVERDISTVARS2_VENDOR =; 
   %end;
%end;

%let SITEDISTVARS = &SiteLabel,&CountryLabel,&trCity,&trStateorProvince,&trMonitor,&trSiteCategory,&trVendor;
%let SITEDISTVARS2 = siteid country city stateprovince monitor sitecategory vendor;
%if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
   %let ENRLDISTVARS = &trWeeksActive,&trRandomizedperWeekActive,&trExpectedRandomized,&trObservedMinusExpectedRand;
   %let ENRLDISTVARS2 = weeksactive siterate exprand diffrand;
%end;
%if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) and %upcase(&YNTargetEnroll) = YES %then %do;
   %let ENRLDISTVARS = &ENRLDISTVARS,&trTargetRandomized,&trObservedMinusTargetRand;
   %let ENRLDISTVARS2 = &ENRLDISTVARS2 targetrand targetdiffrand;
%end;
%let DISPDISTVARS = &trPercentCompletedofRandSubjs,&trPercentOngoingofRandSubjs,&trPercentDiscontofRandSubjs,&trPercentScreenFailofTotalSubjs;
%let AEDISTVARS = &trPercentDeathsofRandSubjs,&trDeathsperPatientWeek;
%let AEDISTVARS2 = pctdeath pwdeath;
%if %sysfunc(findw(&domains, AE)) %then %do;
   %let AEDISTVARS = &AEDISTVARS,&trAverageAEsperRandSubj,&trAEsperPatientWeek;
   %let AEDISTVARS2 = &AEDISTVARS2 avgae pwae;
   %if %length(&AESER) %then %do;
      %let AEDISTVARS = &AEDISTVARS,&trAverageSAEsperRandSubj,&trSAEsperPatientWeek;
      %let AEDISTVARS2 = &AEDISTVARS2 avgsae pwsae;   
   %end;
%end;
%if %sysfunc(findw(&domains, HO)) %then %do;
   %let AEDISTVARS = &AEDISTVARS,&trHealthcareEncountersAV,&trHealthcareEncountersPW;
   %let AEDISTVARS2 = &AEDISTVARS2 avgho pwho;
%end;
%let SUPPDISTVARS = ;
%let SUPPDISTVARS2 = ;
%if %length(&ExData) %then %do;
   %let SUPPDISTVARS = &SUPPDISTVARS%sysfunc(transtrn(&AllRBLabOrder, %str(~), %str(,))),;
   %let SUPPDISTVARS2 = &SUPPDISTVARS2 &AllRBVarOrder;
%end;
/* %let SUPPDISTVARS = %substr(%quote(&SUPPDISTVARS), 1, %eval(%length(&SUPPDISTVARS) - 1))};*/

****************************;
***** Edit Checks **********;
****************************;

data editchk;
   set &RiskBasedMonitoring_OutData_ss;
run;

%let oneerror = 0;
options nodate;
ods listing close;
ods pdf file = "&OutPath.EditCheck.pdf";

title1"&trEditChecksforSubjLevRiskInds";
title2"&trSubjectsWithDeathInfoSBEFC";
title3"&trAEDTHImpSubjsHadALOAEOFD";
proc print noobs label data = editchk;
   var usubjid siteid 
   %if %length(&DTHDTC) %then &DTHDTC; 
   %if %length(&DTHFL) %then &DTHFL;
   %if %sysfunc(findw(&domains, DD)) %then %do;
      deathdom   
   %end;
   AEDTH
   DISCREAS
   %if %length(&CODTH) %then &CODTH;
   ;
   where death = 1;
run;

%CHECKIT(wherestmt = %quote(death = 1));

title2"&trSubjectsThatAreBRASFON";
proc print noobs label data = editchk;
   var usubjid siteid rand scrnfail;
   where rand = scrnfail;
run;

%CHECKIT(wherestmt = %quote(rand = scrnfail));

title2"&trSubjectsThatAreTrtAndNotRand";
proc print noobs label data = editchk;
   var usubjid siteid rand scrnfail;
   where treated = 1 and rand = 0;
run;

%CHECKIT(wherestmt = %quote(treated = 1 and rand = 0));

title2"&trSubjectsThatAreRandAndNotTrt";
proc print noobs label data = editchk;
   var usubjid siteid rand treated;
   where treated = 0 and rand = 1;
run;

%CHECKIT(wherestmt = %quote(treated = 0 and rand = 1));

title2"&trSubjectslistedasCompAndDisc";
proc print noobs label data = editchk;
   var usubjid siteid complete discon;
   where complete = 1 and discon = 1;
run;

%CHECKIT(wherestmt = %quote(complete = 1 and discon = 1));

%if %length(&RFICDTC) %then %do;
   title2"&trSubjectsmissinformedconsent";
   proc print noobs label data = editchk;
      var usubjid siteid &RFICDTC;
      where missing(&RFICDTC);
   run;
   
   %CHECKIT(wherestmt = %quote(missing(&RFICDTC)));
%end;

%if ^&oneerror %then %do;
   title2;
   ods escapechar = "^";
   ods pdf text = "^S={font=('Arial')}  ^S={FOREGROUND=blue} &trNoteNoIssues"; 
%end;

ods pdf close;
ods listing;
options date;

****************************;
***** JSL statements *******;
****************************;

data _null_;
   file "&RiskBasedMonitoringScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
   put "//!";
   put "Names Default to Here(1);";
   put "objRefNS = newnamespace();";
   put "jg:currentObjRefNamespaceName = objRefNS << getName;";
   put "objRefNS:DataList = {};";
   put "objRefNS:DataNameList = {};";
   put "objRefNS:TabButtonList = {};";
   put "objRefNS:TabCodeList = {};";
   put "objRefNS:TabAfterCodeList = {};";
   put "objRefNS:TabOpenList = {};";
   put "objRefNS:CreateReportList = {};";
   put "objRefNS:TabsSectionFlag = 0;";
   put "objRefNS:tmlist={";
   %let top = %sysfunc(countw(%bquote(&QSwitchList), '","'));
   %do i = 1 %to &top;
      %if &i = &top %then %do;
         put "%scan(%bquote(&QSwitchList),&i,',')";
      %end;
      %else %do;
         put "%scan(%bquote(&QSwitchList),&i,','),";
      %end;
   %end;
   put "};";
   put "objRefNS:Outpath = ""&Outpath"";";
   put "objRefNS:ClientOutpath = ""&ClientOutpath"";";
   put "objRefNS:serveroutpath = ""&ServerOutPath"";";
   put "objRefNS:CountryLab = ""&CountryLabel"";";
   %if %upcase(&MonitorAnalysis) = YES %then %do;
      put "objRefNS:MonitorLab = ""&trMonitor"";";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;
      put "objRefNS:VendorLab = ""&trVendor"";";
   %end;
   put "objRefNS:NoActFlg = &NoActFlg;";
   put "objRefNS:ActFlg = &ActFlg;";
   put "objRefNS:rbmsubs = open(""&ClientOutPath.&OutDataName_ss..sas7bdat"",invisible);";
   put "objRefNS:rbmsite = open(""&ClientOutPath.&OutDataName_si..sas7bdat"",invisible);";
   %if &TimeTrends = Yes %then %do;   
      put "Try(objRefNS:rbmsitetime = open(""&ClientOutPath.&OutDataName_sist..sas7bdat"",invisible);";
      put "   column(objRefNS:rbmsitetime,""&SiteLabel"") << label(1);";
      put ");";
   %end;
   %let datalist = ""objRefNS:rbmsite"";
   %if %upcase(&MonitorAnalysis) = YES %then %let datalist = &datalist,""objRefNS:rbmmonitor"";
   %if %upcase(&VendorAnalysis) = YES %then %let datalist = &datalist,""objRefNS:rbmvendor"";
   %if %sysevalf(&NumCountry > 0) %then %let datalist = &datalist,""objRefNS:rbmcountry"";   
   /* kemicl define path and data for Report Actions drill down */
   put "objRefNS:rbmactions = ""&ClientOutPath.&OUtdataName_ra..sas7bdat""; ";
   put "Try(objRefNS:rbmflag = open(""&ClientOutPath.&OutDataName_ri..sas7bdat"",invisible);";
   put "   column(objRefNS:rbmsite,""&SiteLabel"") << label(1);";
   put "   objRefNS:rbmsite << New Column( ""&trRow"",";
   put "      Formula( Row() )";
   put "   );";
   put "   if (NRows(objRefNS:rbmflag),";
   put "   objRefNS:rbmflag2 = objRefNS:rbmsite << Join(";
   put "          With( objRefNS:rbmflag ),";
   put "             Merge Same Name Columns,";
   put "             Copy formula( 0 ),";
   put "             By Matching Columns(:&SiteLabel = :&SiteLabel),";
   put "             Drop multiples( 0, 0 ),";
   put "             Name( ""Include non-matches"" )(0, 0),";
   put "             Preserve main table order( 1 ), invisible";
   put "   );";
   put "   ,";
   put "   objRefNS:rbmflag2 = objRefNS:rbmsite << Join(";
   put "          With( objRefNS:rbmsite ),";
   put "             //Merge Same Name Columns,";
   put "             Copy formula( 0 ),";
   put "             By Matching Columns(:&SiteLabel = :&SiteLabel),";
   put "             Drop multiples( 0, 0 ),";
   put "             Name( ""Include non-matches"" )(0, 0),";
   put "             Preserve main table order( 1 ), invisible";
   put "   );";
   put "   );";
   put "   Close(objRefNS:rbmflag, nosave);";            
   put "   Try(objRefNS:rbmsite << Delete Column( :Name(""&trRow"") ));";
   put "   Current Data Table(objRefNS:rbmflag2);";
   put "   For Each Row(";
   put "          If(:&trSignal[] == 1, Column(objRefNS:rbmsite, :Name(""&trLabel"")) << Color Cells (RGB Color({187,216,189}),:Name(""&trRow"")),";
   put "             :&trSignal[] == 2, Column(objRefNS:rbmsite, :Name(""&trLabel"")) << Color Cells (RGB Color({238,238,160}),:Name(""&trRow"")),";
   put "             :&trSignal[] == 3, Column(objRefNS:rbmsite, :Name(""&trLabel"")) << Color Cells (RGB Color({219,146,132}),:Name(""&trRow"")),";
   put "          );";
   put "          Current Data Table(objRefNS:rbmflag2);";
   put "   );";
   put "   Close(objRefNS:rbmflag2, nosave);"; 
   put ");";
   %if ^%sysfunc(findw(&DropVars, LATITUDE)) %then %do;
      put "   Column(objRefNS:rbmsite,""&trLatitude"") << Set Hidden(1);"; 
   %end;
   %if ^%sysfunc(findw(&DropVars, LONGITUDE)) %then %do;
      put "   Column(objRefNS:rbmsite,""&trLongitude"") << Set Hidden(1);"; 
   %end;
   %if ^%sysfunc(findw(&DropVars, ZIPCODE)) %then %do;
      put "   Column(objRefNS:rbmsite,""&trFiveDigitZipCode"") << Set Hidden(1);"; 
   %end;
   put "   Column(objRefNS:rbmsite,""&trSignal"") << Set Hidden(1);"; 
   put "   objRefNS:rbmsite << Group Columns(""&trSiteIdentifiers"", ";
   %let top = %sysfunc(countw(%bquote(&SiteGroup), ","));
   %do i = 1 %to &top;
      %if &i = &top %then %do;
         put "%scan(%bquote(&SiteGroup),&i,',')";
      %end;
      %else %do;
         put "%scan(%bquote(&SiteGroup),&i,','),";
      %end;
   %end;
   put ");";   
   put "   objRefNS:rbmsite << Group Columns(""&trDisposition"", ";
   %let top = %sysfunc(countw(%bquote(&SubsGroup), ","));
   %do i = 1 %to &top;
      %if &i = &top %then %do;
         put "%scan(%bquote(&SubsGroup),&i,',')";
      %end;
      %else %do;
         put "%scan(%bquote(&SubsGroup),&i,','),";
      %end;
   %end;
   put ");";
   %if %sysfunc(findw(&domains, AE)) %then %do;
      put "   objRefNS:rbmsite << Group Columns(""&trSafety"", ";
      %let top = %sysfunc(countw(%bquote(&SafetyGroup), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&SafetyGroup),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&SafetyGroup),&i,','),";
         %end;
      %end;
      put ");";
   %end;
   %if %length(&SuppGroup) %then %do;
      put "   objRefNS:rbmsite << Group Columns(""&trSupplemental"", ";
      %let top = %sysfunc(countw(%bquote(&SuppGroup), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&SuppGroup),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&SuppGroup),&i,','),";
         %end;
      %end;      
      put ");";
   %end;
   put "   objRefNS:rbmsite << Group Columns(""&trEnrollment"", ";
   %let top = %sysfunc(countw(%bquote(&EnrlGroup), ","));
   %do i = 1 %to &top;
      %if &i = &top %then %do;
         put "%scan(%bquote(&EnrlGroup),&i,',')";
      %end;
      %else %do;
         put "%scan(%bquote(&EnrlGroup),&i,','),";
      %end;
   %end;
   put ");";
   %if %klength(&Overall_SITEID) %then %do; 
      put "   objRefNS:rbmsite << Group Columns(""&trOverallIndicators"",";
      %let top = %sysfunc(countw(%bquote(&OverGroup_SITEID), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&OverGroup_SITEID),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&OverGroup_SITEID),&i,','),";
         %end;
      %end;    
      put ");";
   %end;
   put "   objRefNS:rbmsite << Set Scroll Lock Columns( :&SiteLabel);";
   put "   objRefNS:rbmsite << Select all Rows;";
   put "   objRefNS:rbmsite << Markers( 12);";
   put "   objRefNS:rbmsite << Clear Select;";
   put "   Current Data Table(objRefNS:rbmsite);";
   put "   For Each Row(";
   put "      If(:&trSignal == 2,color of(rowstate())=RGB Color({219,179,0}),";
   put "         :&trSignal == 3,color of(rowstate())=RGB Color({167,34,32}),";
   put "         :&trSignal == 1,color of(rowstate())=RGB Color({70,145,80}),";
   put "         :&trSignal == 4,color of(rowstate())=RGB Color({183,183,183})";
   put "      );";
   put "   );";
   %if %sysevalf(&NumCountry > 0) %then %do;   
      put "objRefNS:rbmcountry = open(""&ClientOutPath.&OutDataName_co..sas7bdat"",invisible);";
      put "Try(objRefNS:rbmcountryflag = open(""&ClientOutPath.&OutDataName_cr..sas7bdat"",invisible);";
      put "   column(objRefNS:rbmcountry,""&CountryLabel"") << label(1);";
      put "   objRefNS:rbmcountry << New Column( ""&trRow"",";
      put "      Formula( Row() )";
      put "   );";
      put "   if (NRows(objRefNS:rbmcountryflag),";      
      put "   objRefNS:rbmcountryflag2 = objRefNS:rbmcountry << Join(";
      put "          With( objRefNS:rbmcountryflag ),";
      put "             Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&CountryLabel = :&CountryLabel),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";
      put "   ,";
      put "   objRefNS:rbmcountryflag2 = objRefNS:rbmcountry << Join(";
      put "          With( objRefNS:rbmcountry ),";
      put "             //Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&CountryLabel = :&CountryLabel),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";
      put "   );";
      put "   Close(objRefNS:rbmcountryflag, nosave);";           
      put "   Try(objRefNS:rbmcountry << Delete Column( :Name(""&trRow"") ));";
      put "   Current Data Table(objRefNS:rbmcountryflag2);";
      put "   For Each Row(";
      put "          If(:&trSignal[] == 1, Column(objRefNS:rbmcountry, :Name(""&trLabel"")) << Color Cells (RGB Color({187,216,189}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 2, Column(objRefNS:rbmcountry, :Name(""&trLabel"")) << Color Cells (RGB Color({238,238,160}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 3, Column(objRefNS:rbmcountry, :Name(""&trLabel"")) << Color Cells (RGB Color({219,146,132}),:Name(""&trRow"")),";
      put "          );";
      put "          Current Data Table(objRefNS:rbmcountryflag2);";
      put "   );";
      put "   Close(objRefNS:rbmcountryflag2, nosave);"; 
      put ");";
      put "   Column(objRefNS:rbmcountry,""&trSignal"") << Set Hidden(1);"; 
      put "   objRefNS:rbmcountry << Group Columns(""&trDisposition"", ";
      %let top = %sysfunc(countw(%bquote(&SubsGroup), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&SubsGroup),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&SubsGroup),&i,','),";
         %end;
      %end;   
      put      ");";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   objRefNS:rbmcountry << Group Columns(""&trSafety"", ";
         %let top = %sysfunc(countw(%bquote(&SafetyGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SafetyGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SafetyGroup),&i,','),";
            %end;
         %end;         
         put ");";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   objRefNS:rbmcountry << Group Columns(""&trSupplemental"", ";
         %let top = %sysfunc(countw(%bquote(&SuppGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SuppGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SuppGroup),&i,','),";
            %end;
         %end;                     
         put ");";
      %end;
      put "   objRefNS:rbmcountry << Group Columns(""&trEnrollment"", ";
      %let top = %sysfunc(countw(%bquote(&EnrlGroupCountry), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&EnrlGroupCountry),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&EnrlGroupCountry),&i,','),";
         %end;
      %end;       
      put ");";
      %if %klength(&Overall_COUNTRY) %then %do;
         put "   objRefNS:rbmcountry << Group Columns(""&trOverallIndicators"", ";
         %let top = %sysfunc(countw(%bquote(&OverGroup_Country), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&OverGroup_Country),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&OverGroup_Country),&i,','),";
            %end;
         %end;             
         put ");";
      %end;
      put "   objRefNS:rbmcountry << Set Scroll Lock Columns( :&CountryLabel);";
      put "   objRefNS:rbmcountry<< Select all Rows;";
      put "   objRefNS:rbmcountry << Markers( 12);";
      put "   objRefNS:rbmcountry << Clear Select;";
      put "   For Each Row(";
      put "      If(:&trSignal == 2,color of(rowstate())=RGB Color({219,179,0}),";
      put "         :&trSignal == 3,color of(rowstate())=RGB Color({167,34,32}),";
      put "         :&trSignal == 1,color of(rowstate())=RGB Color({70,145,80}),";
      put "         :&trSignal == 4,color of(rowstate())=RGB Color({183,183,183})";
      put "      );";
      put "   );";
   %end;     

   %if %upcase(&MonitorAnalysis) = YES %then %do;
      put "objRefNS:rbmmonitor = open(""&ClientOutPath.&OutDataName_mo..sas7bdat"",invisible);";
      put "Try(objRefNS:rbmmonitorflag = open(""&ClientOutPath.&OutDataName_mr..sas7bdat"",invisible);";
      put "   column(objRefNS:rbmmonitor,""&trMonitor"") << label(1);";
      put "   objRefNS:rbmmonitor << New Column( ""&trRow"",";
      put "      Formula( Row() )";
      put "   );";
      put "   if (NRows(objRefNS:rbmmonitorflag),";
      put "   objRefNS:rbmmonitorflag2 = objRefNS:rbmmonitor << Join(";
      put "          With( objRefNS:rbmmonitorflag ),";
      put "             Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&trMonitor = :&trMonitor),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";
      put "   ,";
      put "   objRefNS:rbmmonitorflag2 = objRefNS:rbmmonitor << Join(";
      put "          With( objRefNS:rbmmonitor ),";
      put "             //Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&trMonitor = :&trMonitor),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";
      put "   );";
      put "   Close(objRefNS:rbmmonitorflag, nosave);";           
      put "   Try(objRefNS:rbmmonitor << Delete Column( :Name(""&trRow"") ));";
      put "   Current Data Table(objRefNS:rbmmonitorflag2);";
      put "   For Each Row(";
      put "          If(:&trSignal[] == 1, Column(objRefNS:rbmmonitor, :Name(""&trLabel"")) << Color Cells (RGB Color({187,216,189}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 2, Column(objRefNS:rbmmonitor, :Name(""&trLabel"")) << Color Cells (RGB Color({238,238,160}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 3, Column(objRefNS:rbmmonitor, :Name(""&trLabel"")) << Color Cells (RGB Color({219,146,132}),:Name(""&trRow"")),";
      put "          );";
      put "          Current Data Table(objRefNS:rbmmonitorflag2);";
      put "   );";
      put "   Close(objRefNS:rbmmonitorflag2, nosave);"; 
      put ");";
      put "   Column(objRefNS:rbmmonitor,""&trSignal"") << Set Hidden(1);"; 
      put "   objRefNS:rbmmonitor << Group Columns(""&trDisposition"", ";
      %let top = %sysfunc(countw(%bquote(&SubsGroup), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&SubsGroup),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&SubsGroup),&i,','),";
         %end;
      %end;   
      put      ");";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   objRefNS:rbmmonitor << Group Columns(""&trSafety"", ";
         %let top = %sysfunc(countw(%bquote(&SafetyGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SafetyGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SafetyGroup),&i,','),";
            %end;
         %end;         
         put ");";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   objRefNS:rbmmonitor << Group Columns(""&trSupplemental"", ";
         %let top = %sysfunc(countw(%bquote(&SuppGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SuppGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SuppGroup),&i,','),";
            %end;
         %end;                     
         put ");";
      %end;
      put "   objRefNS:rbmmonitor << Group Columns(""&trEnrollment"", ";
      %let top = %sysfunc(countw(%bquote(&EnrlGroupMonitor), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&EnrlGroupMonitor),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&EnrlGroupMonitor),&i,','),";
         %end;
      %end;       
      put ");";
      %if %klength(&Overall_MONITOR) %then %do;
         put "   objRefNS:rbmmonitor << Group Columns(""&trOverallIndicators"", ";
         %let top = %sysfunc(countw(%bquote(&OverGroup_Monitor), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&OverGroup_Monitor),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&OverGroup_Monitor),&i,','),";
            %end;
         %end;             
         put ");";
      %end;
      put "   objRefNS:rbmmonitor << Set Scroll Lock Columns( :&trMonitor);";
      put "   objRefNS:rbmmonitor<< Select all Rows;";
      put "   objRefNS:rbmmonitor << Markers( 12);";
      put "   objRefNS:rbmmonitor << Clear Select;";
      put "   For Each Row(";
      put "      If(:&trSignal == 2,color of(rowstate())=RGB Color({219,179,0}),";
      put "         :&trSignal == 3,color of(rowstate())=RGB Color({167,34,32}),";
      put "         :&trSignal == 1,color of(rowstate())=RGB Color({70,145,80}),";
      put "         :&trSignal == 4,color of(rowstate())=RGB Color({183,183,183})";
      put "      );";
      put "   );";
   %end;

   %if %upcase(&VendorAnalysis) = YES %then %do;
      put "objRefNS:rbmvendor = open(""&ClientOutPath.&OutDataName_vo..sas7bdat"",invisible);";
      put "Try(objRefNS:rbmvendorflag = open(""&ClientOutPath.&OutDataName_vr..sas7bdat"",invisible);";
      put "   column(objRefNS:rbmvendor,""&trVendor"") << label(1);";
      put "   objRefNS:rbmvendor << New Column( ""&trRow"",";
      put "      Formula( Row() )";
      put "   );";
      put "   if (NRows(objRefNS:rbmvendorflag),";
      put "   objRefNS:rbmvendorflag2 = objRefNS:rbmvendor << Join(";
      put "          With( objRefNS:rbmvendorflag ),";
      put "             Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&trVendor = :&trVendor),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";
      put "   ,";
      put "   objRefNS:rbmvendorflag2 = objRefNS:rbmvendor << Join(";
      put "          With( objRefNS:rbmvendor ),";
      put "             //Merge Same Name Columns,";
      put "             Copy formula( 0 ),";
      put "             By Matching Columns(:&trVendor = :&trVendor),";
      put "             Drop multiples( 0, 0 ),";
      put "             Name( ""Include non-matches"" )(0, 0),";
      put "             Preserve main table order( 1 ), invisible";
      put "   );";      
      put "   );";
      put "   Close(objRefNS:rbmvendorflag, nosave);";           
      put "   Try(objRefNS:rbmvendor << Delete Column( :Name(""&trRow"") ));";
      put "   Current Data Table(objRefNS:rbmvendorflag2);";
      put "   For Each Row(";
      put "          If(:&trSignal[] == 1, Column(objRefNS:rbmvendor, :Name(""&trLabel"")) << Color Cells (RGB Color({187,216,189}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 2, Column(objRefNS:rbmvendor, :Name(""&trLabel"")) << Color Cells (RGB Color({238,238,160}),:Name(""&trRow"")),";
      put "             :&trSignal[] == 3, Column(objRefNS:rbmvendor, :Name(""&trLabel"")) << Color Cells (RGB Color({219,146,132}),:Name(""&trRow"")),";
      put "          );";
      put "          Current Data Table(objRefNS:rbmvendorflag2);";
      put "   );";
      put "   Close(objRefNS:rbmvendorflag2, nosave);"; 
      put ");";
      put "   Column(objRefNS:rbmvendor,""&trSignal"") << Set Hidden(1);"; 
      put "   objRefNS:rbmvendor << Group Columns(""&trDisposition"", ";
      %let top = %sysfunc(countw(%bquote(&SubsGroup), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&SubsGroup),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&SubsGroup),&i,','),";
         %end;
      %end;   
      put      ");";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   objRefNS:rbmvendor << Group Columns(""&trSafety"", ";
         %let top = %sysfunc(countw(%bquote(&SafetyGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SafetyGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SafetyGroup),&i,','),";
            %end;
         %end;         
         put ");";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   objRefNS:rbmvendor << Group Columns(""&trSupplemental"", ";
         %let top = %sysfunc(countw(%bquote(&SuppGroup), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&SuppGroup),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&SuppGroup),&i,','),";
            %end;
         %end;                     
         put ");";
      %end;
      put "   objRefNS:rbmvendor << Group Columns(""&trEnrollment"", ";
      %let top = %sysfunc(countw(%bquote(&EnrlGroupVendor), ","));
      %do i = 1 %to &top;
         %if &i = &top %then %do;
            put "%scan(%bquote(&EnrlGroupVendor),&i,',')";
         %end;
         %else %do;
            put "%scan(%bquote(&EnrlGroupVendor),&i,','),";
         %end;
      %end;       
      put ");";
      %if %klength(&Overall_VENDOR) %then %do;
         put "   objRefNS:rbmvendor << Group Columns(""&trOverallIndicators"", ";
         %let top = %sysfunc(countw(%bquote(&OverGroup_Vendor), ","));
         %do i = 1 %to &top;
            %if &i = &top %then %do;
               put "%scan(%bquote(&OverGroup_Vendor),&i,',')";
            %end;
            %else %do;
               put "%scan(%bquote(&OverGroup_Vendor),&i,','),";
            %end;
         %end;             
         put ");";
      %end;
      put "   objRefNS:rbmvendor << Set Scroll Lock Columns( :&trVendor);";
      put "   objRefNS:rbmvendor<< Select all Rows;";
      put "   objRefNS:rbmvendor << Markers( 12);";
      put "   objRefNS:rbmvendor << Clear Select;";
      put "   For Each Row(";
      put "      If(:&trSignal == 2,color of(rowstate())=RGB Color({219,179,0}),";
      put "         :&trSignal == 3,color of(rowstate())=RGB Color({167,34,32}),";
      put "         :&trSignal == 1,color of(rowstate())=RGB Color({70,145,80}),";
      put "         :&trSignal == 4,color of(rowstate())=RGB Color({183,183,183})";
      put "      );";
      put "   );";
   %end;

   put "objRefNS:AddLeftControlPanel = 1;";
   put "objRefNS:ddwa = Function({},{default local},";
   put "   objRefNS:LeftControlVBox << Append(Outlinebox(locale:get(""RiskIndicators_Text""),";
   put "     objRefNS:glb=ListBox(objRefNS:tmlist,";
   put "       maxselected(1),nlines(min(10,nitems(objRefNS:tmlist))),<<setselected(1,1),";
   put "   )));";    
   put "    objRefNS:glb << Set Size(250,150);";
   put "    objRefNS:glb << Set Script(";
   put "       local({report = namespace(objRefNS:getParent())},";
   put "           objRefNS:currentlySelectedDockerTab = expr(report:docker):getSelectedDockerContentLabel();";
   put "       );";
   put "       SelectOK = 1;";
   put "       try((objRefNS:riskname = objRefNS:glb<<GetSelected)[1],SelectOK = 0);";
   put "       Try(If(SelectOK,";
   put "          objRefNS:rbmflag = open(""&ClientOutPath.&OutDataName_ri..sas7bdat"",invisible);";
   put "          If(nrows(objRefNS:rbmflag) > 0,";
   put "             objRefNS:rbmflag << Select Where( uppercase(:&trLabel) != uppercase(objRefNS:riskname[1]) );";
   put "             nsel = 0;";
   put "             for each row(";
   put "                if(selected(rowstate())==1,";
   put "                  nsel++;";
   put "                );";
   put "             );";      
   put "             If(nsel > 0, objRefNS:rbmflag << Delete Rows;);";
   put "             objRefNS:rbmsite << Update(";
   put "                With( objRefNS:rbmflag ),";
   put "                Match Columns( :&SiteLabel = :&SiteLabel ),";
   put "                Add Columns from Update table( :&trSignal )";
   put "             );";
   put "             Current Data Table(objRefNS:rbmsite);";
   put "             For Each Row(";
   put "                If(:&trSignal == 2,color of(rowstate())=RGB Color({219,179,0}),";
   put "                   :&trSignal == 3,color of(rowstate())=RGB Color({167,34,32}),";
   put "                   :&trSignal == 1,color of(rowstate())=RGB Color({70,145,80}),";
   put "                   :&trSignal == 4,color of(rowstate())=RGB Color({183,183,183})";
   put "                );";
   put "             );";
   put "          );";
   put "          Close( objRefNS:rbmflag, nosave );";   
   put "       ));";          
   put "       Try(If(SelectOK,";
   put "          objRefNS:rbmcountryflag = open(""&ClientOutPath.&OutDataName_cr..sas7bdat"",invisible);";
   put "          If(nrows(objRefNS:rbmcountryflag) > 0,";
   put "             objRefNS:rbmcountryflag << Select Where( uppercase(:&trLabel) != uppercase(objRefNS:riskname[1]) );";
   put "             nsel = 0;";
   put "             for each row(";
   put "                if(selected(rowstate())==1,";
   put "                  nsel++;";
   put "                );";
   put "             );";      
   put "             If(nsel > 0, objRefNS:rbmcountryflag << Delete Rows;);";
   put "             If(N Rows(objRefNS:rbmcountryflag) > 0,";
   put "                objRefNS:rbmcountry << Update(";
   put "                   With( objRefNS:rbmcountryflag ),";
   put "                   Match Columns( :&CountryLabel = :&CountryLabel ),";
   put "                   Add Columns from Update table( :&trSignal )";
   put "                );";
   put "                , //else";
   put "                Current Data Table(objRefNS:rbmcountry);";
   put "                For Each Row(:&trSignal = . );";
   put "             );";
   put "             Current Data Table(objRefNS:rbmcountry);";
   put "             For Each Row(";
   put "                If( :&trSignal == 2,Color Of( Row State() ) = RGB Color( {219, 179, 0} ), //else";
   put "                    :&trSignal == 3,Color Of( Row State() ) = RGB Color( {167, 34, 32} ), //else";
   put "                    :&trSignal == 1,Color Of( Row State() ) = RGB Color( {70, 145, 80} ), //else";
   put "                    :&trSignal == 4,Color of(rowstate())=RGB Color({183,183,183})";
   put "                )";
   put "             );";
   put "          );";
   put "          Close( objRefNS:rbmcountryflag, nosave ); ";     
   put "       ));";   
   %if %upcase(&MonitorAnalysis) = YES %then %do;  
      put "       Try(If(SelectOK,";
      put "          objRefNS:rbmmonitorflag = open(""&ClientOutPath.&OutDataName_mr..sas7bdat"",invisible);";
      put "          If(nrows(objRefNS:rbmmonitorflag) > 0,";
      put "             objRefNS:rbmmonitorflag << Select Where( uppercase(:&trLabel) != uppercase(objRefNS:riskname[1]) );";
      put "             nsel = 0;";
      put "             for each row(";
      put "                if(selected(rowstate())==1,";
      put "                  nsel++;";
      put "                );";
      put "             );";      
      put "             If(nsel > 0, objRefNS:rbmmonitorflag << Delete Rows;);";
      put "             If(N Rows(objRefNS:rbmmonitorflag) > 0,";
      put "                objRefNS:rbmmonitor << Update(";
      put "                With( objRefNS:rbmmonitorflag ),";
      put "                   Match Columns( :&trMonitor = :&trMonitor ),";
      put "                   Add Columns from Update table( :&trSignal )";
      put "                );";
      put "                , //else";
      put "                Current Data Table(objRefNS:rbmmonitor);";
      put "                For Each Row(:&trSignal = . );";
      put "             );";
      put "             Current Data Table(objRefNS:rbmmonitor);";
      put "             For Each Row(";
      put "                If( :&trSignal == 2,Color Of( Row State() ) = RGB Color( {219, 179, 0} ), //else";
      put "                    :&trSignal == 3,Color Of( Row State() ) = RGB Color( {167, 34, 32} ), //else";
      put "                    :&trSignal == 1,Color Of( Row State() ) = RGB Color( {70, 145, 80} ), //else";
      put "                    :&trSignal == 4,Color of(rowstate())=RGB Color({183,183,183})";
      put "                )";
      put "             );";
      put "          );";
      put "          Close( objRefNS:rbmmonitorflag, nosave );";  
      put "       ));";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;  
      put "       Try(If(SelectOK,";
      put "          objRefNS:rbmvendorflag = open(""&ClientOutPath.&OutDataName_vr..sas7bdat"",invisible);";
      put "          If(nrows(objRefNS:rbmvendorflag) > 0,";
      put "             objRefNS:rbmvendorflag << Select Where( uppercase(:&trLabel) != uppercase(objRefNS:riskname[1]) );";
      put "             nsel = 0;";
      put "             for each row(";
      put "                if(selected(rowstate())==1,";
      put "                  nsel++;";
      put "                );";
      put "             );";      
      put "             If(nsel > 0, objRefNS:rbmvendorflag << Delete Rows;);";
      put "             If(N Rows(objRefNS:rbmvendorflag) > 0,";
      put "                objRefNS:rbmvendor << Update(";
      put "                   With( objRefNS:rbmvendorflag ),";
      put "                   Match Columns( :&trVendor = :&trVendor ),";
      put "                   Add Columns from Update table( :&trSignal )";
      put "                );";
      put "                , //else";
      put "                Current Data Table(objRefNS:rbmvendor);";
      put "                For Each Row(:&trSignal = . );";
      put "             );";
      put "             Current Data Table(objRefNS:rbmvendor);";
      put "             For Each Row(";
      put "                If( :&trSignal == 2,Color Of( Row State() ) = RGB Color( {219, 179, 0} ), //else";
      put "                    :&trSignal == 3,Color Of( Row State() ) = RGB Color( {167, 34, 32} ), //else";
      put "                    :&trSignal == 1,Color Of( Row State() ) = RGB Color( {70, 145, 80} ), //else";
      put "                    :&trSignal == 4,Color of(rowstate())=RGB Color({183,183,183})";
      put "                )";
      put "             );";
      put "          );";
      put "          Close( objRefNS:rbmvendorflag, nosave );";  
      put "       ));";
   %end;
   put "       local({report = namespace(objRefNS:getParent())},";
   put "           expr(report:docker):selectDockerContent(objRefNS:currentlySelectedDockerTab);";
   put "       );";
   put "    );";
   put " );";  
   put "objRefNS:RBMStuff=Function({},{default local},";
   put "   VListBox(";
   put "      objRefNS:RiskThreshold = Text Box(locale:fill(locale:get(""ApplyingrisksdefinedinX_Text""), {""&RiskThresholdName""}),<<Set Wrap(900),<< Font Color(5)),"; 
   %if %length(&ExData) %then %do;
      put "      objRefNS:RiskExData = Text Box(locale:get(""&trSupplementalDataSetforRBM"")||"" &ExPath&ExData..sas7bdat"",<<Set Wrap(900),<< Font Color(5)),";    
   %end;
   put "      Text Box(""""),";
   %if %sysfunc(findw(&AvailLocVars, LATITUDE)) and %sysfunc(findw(&AvailLocVars, LONGITUDE)) %then %do;
      put "OutLineBox(locale:get(""SiteLevelDistributions_Text""),<<Close(1),";
   %end;
   %else %do;
      put "OutLineBox(locale:get(""SiteLevelDistributions_Text""),";
   %end;
   put "   Data Filter Context Box(VListBox(HListBox(objRefNS:rbmsite << Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
   put "   OutLineBox(locale:get(""SiteLevelDistributions_Text""),";    
   put "Text Box(""""),";
   put "        Platform(objRefNS:rbmsite,";
   put "           objRefNS:sitedist = Distribution(Automatic Recalc(1),";
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&SITEDISTVARS2),&i,' ')));
      %let temp = %kscan(%bquote(&SITEDISTVARS),&i,',');
      %let temp2 = %kscan(%bquote(&SITEDISTVARS2),&i,' ');
      %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
         put "  Nominal Distribution(Column(:Name(""&temp"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
      %end;
      %let i = %eval(&i+1);
   %end;
   put "            SendToReport(";
   put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 1 ), Set Title( ""&trSiteIdentifiers"" )}),"; 
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&SITEDISTVARS2),&i,' ')));
      %let temp = %kscan(%bquote(&SITEDISTVARS),&i,',');
      %let temp2 = %kscan(%bquote(&SITEDISTVARS2),&i,' ');
      %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
         put "               Dispatch( {""&temp""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
      %end;
      %let i = %eval(&i+1);
   %end; 
   put "        ),"; 
   put "   )),";
   %if %klength(&Overall_SITEID) %then %do;
      put "Text Box(""""),";
      put "        Platform(objRefNS:rbmsite,";
      put "           objRefNS:overdist = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_SITEID),&i,' ')));
         %let temp = %kscan(%bquote(&OVERDISTVARS_SITEID),&i,',');
         %let temp2 = %kscan(%bquote(&OVERDISTVARS2_SITEID),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trOverallRiskIndicators"" )}),"; 
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_SITEID),&i,' ')));
         %let temp = %kscan(%bquote(&OVERDISTVARS_SITEID),&i,',');
         %let temp2 = %kscan(%bquote(&OVERDISTVARS2_SITEID),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   )),";
   %end;
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      put "Text Box(""""),";
      put "        Platform(objRefNS:rbmsite,";    
      put "           objRefNS:enrldist = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trEnrollment"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   )),";
   %end;
   put "   TextBox(""""),";
   put "        Platform(objRefNS:rbmsite,";    
   put "           objRefNS:dispdist = Distribution(Automatic Recalc(1),";
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
      %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
      put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
      %let i = %eval(&i+1);
   %end;
   put "            SendToReport(";
   put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trDisposition"" )}),";
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
      %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
      put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
      put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
      %let i = %eval(&i+1);
   %end; 
   put "        ),"; 
   put "   )),TextBox(""""),";
   put "        Platform(objRefNS:rbmsite,";    
   put "           objRefNS:aedist = Distribution(Automatic Recalc(1),";
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
      %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
      %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
      %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
         put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
      %end;
      %let i = %eval(&i+1);
   %end;
   put "            SendToReport(";
   put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSafety"" )}),";
   %let i = 1;
   %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
      %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
      %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
      %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
         put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
      %end;
      %let i = %eval(&i+1);
   %end; 
   put "        ),"; 
   put "   )),TextBox(""""),";
   %if %length(&SuppGroup) %then %do;
      put "        Platform(objRefNS:rbmsite,";    
      put "           objRefNS:suppdist = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSupplemental"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))";
   %end; 
   put "))))),";
   %if %sysfunc(findw(&AvailLocVars, LATITUDE)) and %sysfunc(findw(&AvailLocVars, LONGITUDE)) %then %do;
      put "OutLineBox(locale:get(""SiteLevelMaps_Text""),";
      put "   Data Filter Context Box(VlistBox(HListBox(objRefNS:rbmsite << Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
      put "   Platform(objRefNS:rbmsite, objRefNS:siteglobe = Graph Builder(";
      put "      Size( 900, 500 ),";
      put "      Show Control Panel( 0 ),";
      put "      Auto Stretching( 0 ),";
      put "      Automatic Recalc(1),";
      put "      Variables( X( :&trLongitude ), Y( :&trLatitude ) ),";
      put "      Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
      put "      SendToReport(";
      put "         Dispatch(";
      put "            {},";
      put "            ""Graph Builder"",";
      put "            OutlineBox,";
      put "            {Set Title( locale:get(""GlobalMap_Text"") )}";
      put "         ),";
      put "         Dispatch(";
      put "            {},";
      put "            ""&trLongitude"",";
      put "            ScaleBox,";
      put "            {Scale( ""Geodesic"" ), Min( -150 ), Max( 150 ), Inc( 25 ), Minor Ticks( 0 )}";
      put "         ),";
      put "         Dispatch(";
      put "            {},";
      put "            ""&trLatitude"",";
      put "            ScaleBox,";
      put "            {Scale( ""Geodesic"" ), Min( -60 ), Max( 100 ), Inc( 25 ), Minor Ticks( 0 )}";
      put "         ),";
      put "         Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),"; 
      put "         Dispatch(";
      put "               {},";
      put "            ""Graph Builder"",";
      put "            FrameBox,";
      put "            {Background Map( Boundaries( ""World"" ) ), Marker Size( 4 ),";
      put "            Grid Line Order( 2 ), Reference Line Order( 3 ),";
      put "            DispatchSeg( Shape Seg( 1 ), {Line Width( 2 )} )}"; 
      put "         )";
      put "      )";
      put "   ))),";
      put "Text Box(""""),";
      %if %klength(&CountryListMap) %then %do;
         put "Outline Box(locale:get(""CountryLevelMaps_Text""), <<close,";
         put "   VListBox(";
         %let i = 1;
         %do %while(%klength(%kscan(&CountryListMap,&i,' ')));
            %let temp = %kscan(&CountryListMap,&i,' ');
            put "   Platform(objRefNS:rbmsite,objRefNS:Map&temp = Graph Builder(";
            put "      Show Control Panel( 0 ),";
            put "      Auto Stretching( 0 ),";
            put "      Automatic Recalc(1),";
            put "      Where( uppercase(:&CountryLabel) == ""&&Flag&temp""),";
            put "      Variables( X( :&trLongitude ), Y( :&trLatitude ) ),";
            put "      Elements( Points( X, Y, Legend( 1 ), Jitter( 1 ) ) ),";
            put "      SendToReport(";
            put "         Dispatch(";
            put "            {},";
            put "            ""Graph Builder"",";
            put "            OutlineBox,"; 
            %if &temp = USA %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""theUnitedStates_Text"")}) )}";
            %end;
            %else %if &temp = CAN %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""Canada_Text"")}) )}";
            %end;
            %else %if &temp = FRA %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""France_Text"")}) )}";
            %end;
            %else %if &temp = DEU %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""Germany_Text"")}) )}";
            %end;
            %else %if &temp = GBR %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""GreatBritain_Text"")}) )}";
            %end;
            %else %if &temp = ITA %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""Italy_Text"")}) )}";
            %end;
            %else %if &temp = ESP %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""Spain_Text"")}) )}";
            %end;
            %else %if &temp = CHN %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""China_Text"")}) )}";
            %end;
            %else %if &temp = JPN %then %do;
               put "            {Close( 1 ), Set Title( locale:fill(locale:get(""SiteLevelRiskIndicatorsinX_Text""), {locale:get(""Japan_Text"")}) )}";
            %end;
            put "         ),";
            put "         Dispatch(";
            put "            {},";
            put "            ""&trLongitude"",";
            put "            ScaleBox,";
            %if &temp = USA %then %do;
               put "            {Scale( ""Geodesic"" ), Inc( 10 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = CAN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( -126), Max( -66 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = FRA %then %do;
               put "            {Scale( ""Geodesic"" ), Min( -2), Max( 10 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = DEU %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 5), Max( 17 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = GBR %then %do;
               put "            {Scale( ""Geodesic"" ), Min( -8), Max( 3 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = ITA %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 5), Max( 20),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = ESP %then %do;
               put "            {Scale( ""Geodesic"" ), Min( -9), Max( 4 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = CHN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 79), Max( 129 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = JPN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 127), Max( 147 ),  Minor Ticks( 0 )}";
            %end;
            put "         ),";
            put "         Dispatch(";
            put "            {},";
            put "            ""&trLatitude"",";
            put "            ScaleBox,";
            %if &temp = USA %then %do;
               put "            {Scale( ""Geodesic"" ), Inc( 10 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = CAN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 36), Max( 86 ),  Inc( 10), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = FRA %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 40 ), Max( 52 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = DEU %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 46 ), Max( 56 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = GBR %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 49 ), Max( 59 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = ITA %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 36 ), Max( 48 ),  Inc( 2 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = ESP %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 35), Max( 45 ),  Inc( 2 ), Minor Ticks( 0 )}";
            %end;
            %else %if &temp = CHN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 15), Max( 57 ),  Minor Ticks( 0 )}";
            %end;
            %else %if &temp = JPN %then %do;
               put "            {Scale( ""Geodesic"" ), Min( 30), Max( 46 ),  Inc( 5 ), Minor Ticks( 0 )}";
            %end;       
            put "         ),";
            put "         Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),"; 
            put "         Dispatch(";
            put "               {},";
            put "            ""Graph Builder"",";
            put "            FrameBox,";
            %if &temp = USA %then %do;
               put "            {Background Map( Boundaries( ""US States"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = CAN %then %do;
               put "            {Background Map( Boundaries( ""Canadian Provinces"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = FRA %then %do;
               put "            {Background Map( Boundaries( ""French Regions"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = DEU %then %do;
               put "            {Background Map( Boundaries( ""German States"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = GBR %then %do;
               put "            {Background Map( Boundaries( ""UK Counties"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = ITA %then %do;
               put "            {Background Map( Boundaries( ""Italian Regions"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = ESP %then %do;
               put "            {Background Map( Boundaries( ""Spanish Autonomous Communities"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = CHN %then %do;
               put "            {Background Map( Boundaries( ""Mainland China Provinces"" ) ), Marker Size( 4 ),";
            %end;
            %else %if &temp = JPN %then %do;
               put "            {Background Map( Boundaries( ""Japanese Prefectures"" ) ), Marker Size( 4 ),";
            %end;
            put "            Grid Line Order( 2 ), Reference Line Order( 3 ),";
            put "            DispatchSeg( Shape Seg( 1 ), {Line Width( 2 )} )}"; 
            put "         )";
            put "      )";
            put "   )),"; 
            %let i = %eval(&i+1);
         %end; 
         put "   )";
         put "),";
      %end;
      put "))),";
   %end;
   %if &TimeTrends = Yes %then %do;
      put "OutLineBox(locale:get(""SiteLevelTimeTrends_Text""),<<Close(1),";
      put "   Data Filter Context Box(VlistBox(HListBox(";
      put "      objRefNS:rbmsitetime << Data Filter(Close Outline( 1 ),<<Conditional(1),local,";
      put "         Mode( Select( 0 ), Show( 1 ), Include( 1 ) ),";
      put "         Add Filter(";
      put "          Columns(";
      put "             :&SiteLabel,";
      put "             :&trRiskIndicator,";
      put "             :&trCategory,";
      put "             :&trCountry,";
      put "             :&trCity";
      put "          ),";
      put "          Where( :&trRiskIndicator == ""&trOverallRiskIndicator"" ),";
      put "       )";
      put "      ),";
      put "      Platform(objRefNS:rbmsitetime, objRefNS:sitetimetrends = Graph Builder(";
      put "         Size( 900, 500 ),";
      put "         Show Control Panel( 0 ),";
      put "         Auto Stretching( 0 ),";
      put "         Automatic Recalc(1),";
      put "         Variables( X( :&trDate ), Y( :&trScore ), Wrap( :&trRiskIndicator), Overlay( :&SiteLabel ) ),";
      put "         Elements(";
      put "            Points( X, Y, Legend( 3 ), Jitter( 0 ) ),";
      put "            Line( X, Y, Legend( 5 ) ),";
      put "         ),";
      put "   ))),";
      put "   Text Box(""""),";
      put "))),";
   %end;   
   %if %sysevalf(&NumCountry > 0) %then %do;
      put "TextBox(""""),";
      put "OutLineBox(locale:get(""CountryLevelDistributions_Text""),<<Close(1),";
      put "Data Filter Context Box(VListBox(HListBox(objRefNS:rbmcountry << Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
      put "OutLineBox(locale:get(""CountryLevelDistributions_Text""),";
      %if %klength(&Overall_COUNTRY) %then %do;
         put "Text Box(""""),";
         put "        Platform(objRefNS:rbmcountry,";
         put "           objRefNS:overdistc = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_COUNTRY),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_COUNTRY),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_COUNTRY),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trOverallRiskIndicators"" )}),"; 
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_COUNTRY),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_COUNTRY),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_COUNTRY),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )),";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "Text Box(""""),";
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmcountry,objRefNS:enrldistc = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trEnrollment"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   ))),";
      %end;
      put "Text Box(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmcountry,objRefNS:dispdistc = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trDisposition"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmcountry,objRefNS:aedistc = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSafety"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      %if %klength(&SuppGroup) %then %do;
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmcountry,objRefNS:suppdistc = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSupplemental"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )))";
      %end; 
      put "))))),";
      put "OutLineBox(locale:get(""CountryLevelMap_Text""),<<Close(1),";
      put "   Data Filter Context Box(HListBox(objRefNS:rbmcountry <<Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
      put "   Platform(objRefNS:rbmcountry, objRefNS: CountryMap = Graph Builder(";
      put "      Size( 1000, 650),";
      put "      Auto Stretching( 0 ),";
      put "      Automatic Recalc(1),";
      put "    Show Control Panel( 0 ),";
      put "    Variables(Shape( :&CountryLabel ) ),";
      put "    Elements(";
      put "       Map Shapes(";
      put "          Legend( 2 ),";
      put "          Summary Statistic( ""Mean"" ),";
      put "          Show Missing Shapes( 1 )";
      put "       )";
      put "    ),";
      put "    SendToReport(";
      put "       Dispatch(";
      put "          {},";
      put "          ""Graph Builder"",";
      put "          OutlineBox,";
      put "          {Set Title( locale:get(""GlobalMap_Text"") )}";
      put "       ),";
      put "       Dispatch(";
      put "          {},";
      put "          """",";
      put "          ScaleBox,";
      put "          {Min( -146), Max( 188), Inc( 50 ),";
      put "          Minor Ticks( 0 )}";
      put "       ),";
      put "       Dispatch(";
      put "          {},";
      put "          """",";
      put "          ScaleBox( 2 ),";
      put "          {Min( -112), Max( 118), Inc( 50 ),";
      put "          Minor Ticks( 0 )}";
      put "       ),";
      put "       Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} )";
      put "    )";
      put "   );";
      put ")))),";            
   %end;   
   %if %upcase(&MonitorAnalysis) = YES %then %do;
      put "TextBox(""""),";
      put "OutLineBox(locale:get(""MonitorLevelDistributions_Text""),<<Close(1),";
      put "Data Filter Context Box(VListBox(HListBox(objRefNS:rbmmonitor << Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
      put "OutLineBox(locale:get(""MonitorLevelDistributions_Text""),";
      %if %klength(&Overall_MONITOR) %then %do;
         put "Text Box(""""),";
         put "        Platform(objRefNS:rbmmonitor,";
         put "           objRefNS:overdistm = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_MONITOR),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_MONITOR),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_MONITOR),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trOverallRiskIndicators"" )}),"; 
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_MONITOR),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_MONITOR),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_MONITOR),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )),";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "Text Box(""""),";
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmmonitor,objRefNS:enrldistm = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trEnrollment"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   ))),";
      %end;
      put "Text Box(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmmonitor,objRefNS:dispdistm = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trDisposition"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmmonitor,objRefNS:aedistm = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSafety"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      %if %klength(&SuppGroup) %then %do;
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmmonitor,objRefNS:suppdistm = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSupplemental"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )))";
      %end; 
      put "))))),";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;
      put "TextBox(""""),";
      put "OutLineBox(locale:get(""VendorLevelDistributions_Text""),<<Close(1),";
      put "Data Filter Context Box(VListBox(HListBox(objRefNS:rbmvendor << Data Filter(Close Outline( 1 ),<<Conditional(1),local),";
      put "OutLineBox(locale:get(""VendorLevelDistributions_Text""),";
      %if %klength(&Overall_VENDOR) %then %do;
         put "Text Box(""""),";
         put "        Platform(objRefNS:rbmvendor,";
         put "           objRefNS:overdistv = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_VENDOR),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_VENDOR),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_VENDOR),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trOverallRiskIndicators"" )}),"; 
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&OVERDISTVARS2_VENDOR),&i,' ')));
            %let temp = %kscan(%bquote(&OVERDISTVARS_VENDOR),&i,',');
            %let temp2 = %kscan(%bquote(&OVERDISTVARS2_VENDOR),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )),";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "Text Box(""""),";
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmvendor,objRefNS:enrldistv = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trEnrollment"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&ENRLDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&ENRLDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&ENRLDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) and &temp2 ne weeksactive and &temp2 ne siterate %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   ))),";
      %end;
      put "Text Box(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmvendor,objRefNS:dispdistv = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trDisposition"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&DISPDISTVARS),&i,',')));
         %let temp = %kscan(%bquote(&DISPDISTVARS),&i,',');
         put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
         put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      put "        HListBox(";   
      put "           Platform(objRefNS:rbmvendor,objRefNS:aedistv = Distribution(Automatic Recalc(1),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
         %end;
         %let i = %eval(&i+1);
      %end;
      put "            SendToReport(";
      put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSafety"" )}),";
      %let i = 1;
      %do %while(%klength(%kscan(%bquote(&AEDISTVARS2),&i,' ')));
         %let temp = %kscan(%bquote(&AEDISTVARS),&i,',');
         %let temp2 = %kscan(%bquote(&AEDISTVARS2),&i,' ');
         %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
            put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
            put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
         %end;
         %let i = %eval(&i+1);
      %end; 
      put "        ),"; 
      put "   ))),TextBox(""""),";
      %if %klength(&SuppGroup) %then %do;
         put "        HListBox(";   
         put "           Platform(objRefNS:rbmvendor,objRefNS:suppdistv = Distribution(Automatic Recalc(1),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "  Continuous Distribution(Column(:Name(""&temp"")), ShowCounts(1), CountAxis(1)),";
            %end;
            %let i = %eval(&i+1);
         %end;
         put "            SendToReport(";
         put "   Dispatch( {}, ""Distributions"", OutlineBox, {Close( 0 ), Set Title( ""&trSupplemental"" )}),";
         %let i = 1;
         %do %while(%klength(%kscan(%bquote(&SUPPDISTVARS2),&i,' ')));
            %let temp = %kscan(%bquote(&SUPPDISTVARS),&i,',');
            %let temp2 = %kscan(%bquote(&SUPPDISTVARS2),&i,' ');
            %if %sysfunc(findw(%upcase(&VarOrder), %upcase(&temp2))) %then %do;
               put "               Dispatch( {""&temp""}, ""Quantiles"", OutlineBox, Close( 1 ) ),";
               put "               Dispatch( {""&temp""}, ""Summary Statistics"", OutlineBox, Close( 1 ) ),";
            %end;
            %let i = %eval(&i+1);
         %end; 
         put "        ),"; 
         put "   )))";
      %end; 
      put "))))),";
   %end;
   /*** Output Here ***/      
   put "   );";  
   put ");";
   
   put "Current Data Table(objRefNS:rbmsite);";   
   put "Include(""$CLINICAL_HOME\JSLFiles\RBMFunctions.jsl"");";   
   put "objRefNS:ActionButtonDesc = {};";
   put "objRefNS:ActionButtonList={{},{}};";
   put "objRefNS:ActionCodeList ={{},{}};";
   put "objRefNS:ActionDataTableList = {{},{}};";
   put "objRefNS:ActionButtonsperRow = {1,1};";

   put "objRefNS:ppdd = evalexpr(Function({},{default local}, this = namespace(expr(objRefNS<<getName)); reportRefNS = namespace(this:getParent()); jg:SubjectProfiler_ReviewRef = reportRefNS:getParent(); Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here"")););";  

   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""SelectSitesthenClick_Text""))});";
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ProfileSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ppdd()));";    
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:rbmsubs""}});";
   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ShowSubjects_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ShowSubjectSite(""&SiteLabel"")));";      
   put "InsertInto(objRefNS:ActionDataTableList[1], {{""objRefNS:rbmsite""}});";
   
   put "InsertInto(objRefNS:ActionButtonList[1],{expr(locale:get(""ReportActions_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[1],Expr(objRefNS:ReportSiteActions(""&SiteLabel"")));";
   put "InsertInto(objRefNS:ActionDataTableList[1], {{&datalist}});";
 
   put "InsertInto(objRefNS:ActionButtonDesc,{expr(locale:get(""Click_Text""))});";   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""OpenEditChecks_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:OpenEditCheck()));";    
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&datalist, ""objRefNS:rbmsubs""}});";
   
   put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""SelectRowsUsingRiskIndicator_Text""))});";
   put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:SelSiteRisk()));";       
   put "InsertInto(objRefNS:ActionDataTableList[2], {{&datalist, ""objRefNS:rbmsubs""}});";
   
   %let actionsellist = 0,0;
   %let actioniconlist = ""&IconPath.Report.gif"",""&IconPath.DataSetHighlightRows.gif"";
   %let actionreqlist = 0,0;

   %if %sysevalf(&NumCountry > 0) or %upcase(&MonitorAnalysis) = YES or %upcase(&VendorAnalysis) = YES %then %do;
      put "InsertInto(objRefNS:ActionButtonList[2],{expr(locale:get(""SelectSitesUsingSubgroupSelection_Text""))});";
      put "InsertInto(objRefNS:ActionCodeList[2],Expr(objRefNS:SelSiteSubgroup()));";       
      put "InsertInto(objRefNS:ActionDataTableList[2], {{%substr(%nrbquote(&datalist), %sysevalf(%index(%nrbquote(&datalist),%str(,))+1))}});";
      %let actionsellist = &actionsellist,1;
      %let actioniconlist = &actioniconlist,""&IconPath.DataSetHighlightRowsGU.gif"";
      %let actionreqlist = &actionreqlist,0;
   %end;
   put "objRefNS:ActionSelectionCountList = {{1, 1, 1},{&actionsellist}};";
   put "objRefNS:ActionIconList = {{""&IconPath.ProfileSubjects.gif"",""&IconPath.DataSetUser.gif"", ""&IconPath.DistributionAnalysis.gif""},{&actioniconlist}};";
   put "objRefNS:ActionRequirementFlagList = {{3, 0, 0},{&actionreqlist}};";
        
   put "InsertInto(objRefNS:TabButtonList, expr(locale:get(""RiskBasedMonitoring_Text"")));";
   put "InsertInto(objRefNS:TabCodeList,expr(objRefNS:RBMStuff()));";
   put "InsertInto(objRefNS:TabAfterCodeList, expr(objRefNS:ddwa()));";
   put "InsertInto(objRefNS:TabOpenList,1);";
   put "objRefNS:ShowDataTables = 2;";                  
   /*** Report List Here ***/      
   put "InsertInto(objRefNS:CreateReportList,{{objRefNS:RiskThreshold";
   %if %length(&ExData) %then %do;
      put "    ,objRefNS:RiskExData";
   %end;
      put "   ,objRefNS:sitedist,objRefNS:dispdist";
   %if %sysfunc(findw(&domains, AE)) %then %do;
      put "   ,objRefNS:aedist";
   %end;
   %if %klength(&Overall_SITEID) %then %do;
      put "   ,objRefNS:overdist";
   %end;    
   %if %klength(&SuppGroup) %then %do;
      put "   ,objRefNS:suppdist";   
   %end;
   %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
      put "   ,objRefNS:enrldist";  
   %end;   
   %if %sysfunc(findw(&AvailLocVars, LATITUDE)) and %sysfunc(findw(&AvailLocVars, LONGITUDE)) %then %do;   
      put "   ,objRefNS:siteglobe";
      %let i = 1;
      %do %while(%klength(%kscan(&CountryListMap,&i,' ')));
         %let temp = %kscan(&CountryListMap,&i,' ');
         put ",objRefNS:Map&temp";
         %let i = %eval(&i+1);
      %end;   
   %end;
   %if &TimeTrends = Yes %then %do;   
      put "   ,objRefNS:sitetimetrends";  
   %end;
   put "   ,objRefNS:rbmsite";     
   %if %sysevalf(&NumCountry > 0) %then %do;   
      put "   ,objRefNS:dispdistc";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   ,objRefNS:aedistc";
      %end;
      %if %klength(&Overall_COUNTRY) %then %do;
         put "   ,objRefNS:overdistc";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   ,objRefNS:suppdistc";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "   ,objRefNS:enrldistc"; 
      %end;
      put "   ,objRefNS:CountryMap";
      put ",objRefNS:rbmcountry";
   %end;      
   %if %upcase(&MonitorAnalysis) = YES %then %do;   
      put "   ,objRefNS:dispdistm";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   ,objRefNS:aedistm";
      %end;
      %if %klength(&Overall_MONITOR) %then %do;
         put "   ,objRefNS:overdistm";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   ,objRefNS:suppdistm";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "   ,objRefNS:enrldistm"; 
      %end;   
      put ",objRefNS:rbmmonitor";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;   
      put "   ,objRefNS:dispdistv";
      %if %sysfunc(findw(&domains, AE)) %then %do;
         put "   ,objRefNS:aedistv";
      %end;
      %if %klength(&Overall_VENDOR) %then %do;
         put "   ,objRefNS:overdistv";
      %end;
      %if %klength(&SuppGroup) %then %do;
         put "   ,objRefNS:suppdistv";  
      %end;
      %if %sysfunc(findw(&AvailAnalVars, SITEACTIVE)) %then %do;
         put "   ,objRefNS:enrldistv"; 
      %end;
      put ",objRefNS:rbmvendor";
   %end;   
   put "}});";   
   /*** Data Tables Here ***/   
   put "InsertInto(objRefNS:DataList,{{""objRefNS:rbmsite""";
   %if %sysevalf(&NumCountry > 0) %then %do;   
      put ",""objRefNS:rbmcountry""";
   %end;   
   %if %upcase(&MonitorAnalysis) = YES %then %do;   
      put ",""objRefNS:rbmmonitor""";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;
      put ",""objRefNS:rbmvendor""";
   %end;
   %if &TimeTrends = Yes %then %do;   
      put ",""objRefNS:rbmsitetime""";
   %end;   
   put ",""objRefNS:rbmsubs""";
   put"}});";   
   /*** Data Table Names Here ***/   
   put "InsertInto(objRefNS:DataNameList,{{""&trSLevelRiskInd"""; 
   %if %sysevalf(&NumCountry > 0) %then %do;   
      put ",""&trCLevelRiskInd""";
   %end;   
   %if %upcase(&MonitorAnalysis) = YES %then %do;   
      put ",""&trMLevelRiskInd""";
   %end;
   %if %upcase(&VendorAnalysis) = YES %then %do;
      put ",""&trVLevelRiskInd""";
   %end;
   %if &TimeTrends = Yes %then %do;   
      put ",""&trTLevelRiskInd""";
   %end;   
   put ",""&trPLevelRiskInd""";   
   put"}});";   

%let includeds = &includeds EditCheck.pdf,;
%let includeds = %ksubstr(%quote(&includeds), 1, %eval(%klength(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &RiskBasedMonitoringName, "&includeds", yes);
%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);
%mend RiskBasedMonitoring;
%RiskBasedMonitoring;

************************************;
************************************;
************************************;